<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:req="http://purl.org/dc/elements/1.1/"
    xmlns:ns2="http://jazz.net/xmlns/alm/qm/v0.1/" xmlns:ns3="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:ns4="http://schema.ibm.com/vega/2008/"
    xmlns:ns5="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
    xmlns:ns6="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:ns7="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
    xmlns:ns8="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/"
    exclude-result-prefixes="ns8 ns7 ns6 ns5 ns4 ns3 ns2 xsl">
    
    <!-- included stylesheets -->
    <xsl:include href="xhtml-to-xslfo.xsl"/>
    <xsl:include href="wrapping-rules.xsl"/>
    <xsl:include href="attribute-sets.xsl"/>
    
    <!-- variables -->
    <xsl:variable name="lang" select="locale"/>
    <xsl:variable name="strings" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]"/>
    
    <xsl:template match="transformArgs">
        <xsl:choose>
            <xsl:when test="(componentType='STAND_ALONE') or (componentType='STAND_ALONE_SUMMARY')">
                <fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
                    initial-page-number="1" force-page-count="no-force"
                    master-reference="PageMaster">
                    <fo:static-content flow-name="xsl-region-before">
                        <fo:block font-size="8pt" text-align="right">
                            <fo:page-number/> / <fo:page-number-citation ref-id="{reqNumber}"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:static-content flow-name="xsl-region-after">
                    	<fo:block-container height="0.1in" padding-after="0.07in">
							<fo:block text-align="center">
								<fo:leader leader-pattern='rule' rule-thickness='1pt' leader-length='100%'/>
							</fo:block>
						</fo:block-container>
                        <fo:block text-align="center">
                            <fo:retrieve-marker retrieve-class-name="artifactUri"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:flow flow-name="xsl-region-body">
                        <fo:marker marker-class-name="artifactUri">
                            <fo:block font-size="6pt">
                                <fo:basic-link external-destination="{originUrl}" color="blue"
                                    text-decoration="underline">
                                    <xsl:value-of select="originUrl"/>
                                </fo:basic-link>
                            </fo:block>
                        </fo:marker>
                        <xsl:apply-templates select="descendant::ns2:requirement"/>
                        <fo:block id="{reqNumber}"/>
                    </fo:flow>
                </fo:page-sequence>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="descendant::ns2:requirement"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ns2:requirement">
        <xsl:choose>
            <xsl:when
                test="(../../componentType='SUMMARY') or (../../componentType='STAND_ALONE_SUMMARY') or (../../componentType='COMPOSITE_SUMMARY')">
                <xsl:choose>
                    <xsl:when
                        test="../../componentType='STAND_ALONE_SUMMARY' or (../../componentType='COMPOSITE_SUMMARY')">
                        <xsl:if test="../../componentType='STAND_ALONE_SUMMARY'">
                            <fo:block font-size="14pt" space-before="6mm" space-after="2mm">
                                <xsl:value-of
                                    select="$strings/labelSummary"
                                />: <xsl:value-of
                                    select="$strings/labelRequirements"
                                /></fo:block>
                        </xsl:if>
                        <fo:table xsl:use-attribute-sets="artifact-table">
                            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                                <!-- id -->
                                <fo:table-cell xsl:use-attribute-sets="border id-header">
                                    <fo:block><xsl:value-of select="$strings/labelId"/></fo:block>
                                </fo:table-cell>
                                <!-- status -->
                                <fo:table-cell xsl:use-attribute-sets="border" width="50">
                                    <fo:block><xsl:apply-templates select="$strings/reqLabel_status"/></fo:block>
                                </fo:table-cell>
                                <!-- name -->
                                <fo:table-cell xsl:use-attribute-sets="border" width="75">
                                    <fo:block><xsl:apply-templates select="$strings/labelName"/></fo:block>
                                </fo:table-cell>
                                <!-- description -->
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <fo:block><xsl:apply-templates select="$strings/labeldescription"/></fo:block>
                                </fo:table-cell>
                                <!-- owner -->
                                <fo:table-cell xsl:use-attribute-sets="border user-header">
                                    <fo:block><xsl:apply-templates select="$strings/labelOwner"/></fo:block>
                                </fo:table-cell>
                            </fo:table-header>
                            <fo:table-body>
                                <fo:table-row>
                                    <!-- id -->
                                    <fo:table-cell xsl:use-attribute-sets="border id-cell">
                                        <fo:block><xsl:value-of select="//ns2:generatedId"/></fo:block>
                                    </fo:table-cell>
                                    <!-- status -->
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block>
                                        	<xsl:choose>
                                            	<xsl:when test="ns3:suspectStatus='updated'"> 
													<xsl:value-of select="$strings/reqStatusUpdated"/>
												</xsl:when>
                                            	<xsl:when test="ns3:suspectStatus='removed'">
													<xsl:value-of select="$strings/reqStatusRemoved"/>
												</xsl:when>
                                            	<xsl:when test="ns3:suspectStatus='normal'">
													<xsl:value-of select="$strings/reqStatusNormal"/>
												</xsl:when>
                                            	<xsl:when test="ns3:suspectStatus='new'">
													<xsl:value-of select="$strings/reqStatusNormal"/>
												</xsl:when>
												<xsl:when test="not(ns3:suspectStatus)">
													<xsl:value-of select="$strings/reqStatusNormal"/>
												</xsl:when>
                                            	<xsl:otherwise>
													<xsl:value-of select="ns3:suspectStatus"/>
												</xsl:otherwise>
                                        	</xsl:choose>
                                        </fo:block>
                                    </fo:table-cell>
                                    <!-- name -->
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block><xsl:apply-templates select="req:title"/></fo:block>
                                    </fo:table-cell>
                                    <!-- description -->
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block><xsl:value-of select="req:description"/></fo:block>
                                    </fo:table-cell>
                                    <!-- owner -->
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block><xsl:value-of select="ns3:owner"/></fo:block>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-body>
                        </fo:table>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:table-row>
                            <!-- id -->
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:value-of select="//ns2:generatedId"/></fo:block>
                            </fo:table-cell>
                            <!-- status -->
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block>
                                    <xsl:choose>
                                        <xsl:when test="ns3:suspectStatus='updated'"> 
											<xsl:value-of select="$strings/reqStatusUpdated"/>
										</xsl:when>
										<xsl:when test="ns3:suspectStatus='removed'">
											<xsl:value-of select="$strings/reqStatusRemoved"/>
										</xsl:when>
										<xsl:when test="ns3:suspectStatus='normal'">
											<xsl:value-of select="$strings/reqStatusNormal"/>
										</xsl:when>
										<xsl:when test="ns3:suspectStatus='new'">
											<xsl:value-of select="$strings/reqStatusNormal"/>
										</xsl:when>
										<xsl:when test="not(ns3:suspectStatus)">
											<xsl:value-of select="$strings/reqStatusNormal"/>
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="ns3:suspectStatus"/>
										</xsl:otherwise>
                                    </xsl:choose>
                                </fo:block>
                            </fo:table-cell>
                            <!-- name -->
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="req:title"/></fo:block>
                            </fo:table-cell>
                            <!-- description -->
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="req:description"/></fo:block>
                            </fo:table-cell>
                            <!-- owner -->
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="ns3:owner"/></fo:block>
                            </fo:table-cell>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="(../../componentType='COMPOSITE') and (../../recursive)">
                <fo:wrapper>
                    <fo:table-row>
                        <!-- id -->
                        <fo:table-cell xsl:use-attribute-sets="border id-cell">
                            <fo:block>
                                <xsl:value-of select="//ns2:generatedId"/>
                            </fo:block>
                        </fo:table-cell>
                        <!-- status -->
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block>
                                <xsl:choose>
                                   <xsl:when test="ns3:suspectStatus='updated'"> 
										<xsl:value-of select="$strings/reqStatusUpdated"/>
									</xsl:when>
									<xsl:when test="ns3:suspectStatus='removed'">
										<xsl:value-of select="$strings/reqStatusRemoved"/>
									</xsl:when>
									<xsl:when test="ns3:suspectStatus='normal'">
										<xsl:value-of select="$strings/reqStatusNormal"/>
									</xsl:when>
									<xsl:when test="ns3:suspectStatus='new'">
										<xsl:value-of select="$strings/reqStatusNormal"/>
									</xsl:when>
									<xsl:when test="not(ns3:suspectStatus)">
										<xsl:value-of select="$strings/reqStatusNormal"/>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="ns3:suspectStatus"/>
									</xsl:otherwise>
                                </xsl:choose>
                            </fo:block>
                        </fo:table-cell>
                        <!-- name -->
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block><xsl:apply-templates select="req:title"/></fo:block>
                        </fo:table-cell>
                        <!-- description -->
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block><xsl:apply-templates select="req:description"/></fo:block>
                        </fo:table-cell>
                        <!-- owner -->
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block><xsl:apply-templates select="ns3:owner"/></fo:block>
                        </fo:table-cell>
                    </fo:table-row>
                <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format" font-size="10pt"> <!-- font-size="8pt" ? -->
                    <xsl:if test="(../../componentType='COMPOSITE')">
                        <xsl:attribute name="id">
                        	<xsl:value-of select="//ns2:generatedId"/>
                        </xsl:attribute>
                    </xsl:if>
                    <fo:block font-size="14pt" space-after="4mm" border-after-style="solid"
                        border-after-width="thin">
                        <xsl:value-of select="$strings/labelRequirement"/><xsl:text> </xsl:text>
                        <xsl:value-of select="//ns2:generatedId"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/reqLabel_status"/>
						:
						<xsl:choose>
                            <xsl:when test="ns3:state='authoring.s3'">
								<xsl:value-of select="$strings/ResolvedBy"/><xsl:text> </xsl:text>
								<xsl:value-of select="ns2:resolver"/>
                            </xsl:when>
                            <xsl:when test="ns3:state='authoring.s1'">
								<xsl:value-of select="$strings/reqStateNew"/>
							</xsl:when>
                            <xsl:when test="ns3:state='authoring.s2'">
								<xsl:value-of select="$strings/InProgress"/>
							</xsl:when>
                            <xsl:when test="ns3:state='authoring.s6'">
								<xsl:value-of select="$strings/Reopened"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="ns3:state"/>
							</xsl:otherwise>
                        </xsl:choose>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/reqLabel_created_on"/>
						:
						<xsl:value-of select="ns2:creationtime"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/reqLabel_created_by"/>
						:
						<xsl:value-of select="req:creator"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelOwnedBy"/>
						:
						<xsl:value-of select="ns3:owner"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelModifiedDate"/>
						:
						<xsl:value-of select="ns3:updated"/>
                    </fo:block>
                    <fo:block space-before="4mm">
                        <xsl:value-of select="$strings/labelSeverity"/>
						:
						<xsl:apply-templates select="ns2:severity"/>
                    </fo:block>
                    <fo:block>
                    	<xsl:value-of select="$strings/reqLabel_priority"/>
						:
						<xsl:value-of 
							select="ns2:customAttributes/ns2:customAttribute[ns2:name='Requirement Priority']/ns2:value" />
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelReviewStatus"/> 
						:
						<xsl:value-of select="ns2:reviewStatus"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelTags"/>
						:
						<xsl:for-each select="ns2:tags/tag">
							<xsl:value-of select="text" />
                        	 	<xsl:if test="position()!=last()">
        							<xsl:text>, </xsl:text>
      							</xsl:if>                        	 	
                        </xsl:for-each>
                    </fo:block>
					<fo:block>
						<xsl:value-of select="$strings/labelPackageInfo"/>
						: 
						<xsl:value-of 
							select="ns2:customAttributes/ns2:customAttribute[ns2:name='Package Information']/ns2:value"/>
					</fo:block>
                    <fo:block space-before="4mm" space-after="4mm">
                        <xsl:value-of select="$strings/labeldescription" />
						:
						<xsl:value-of select="req:description"/>
                    </fo:block>
                    <xsl:apply-templates select="ns2:comments"/>
                </fo:block>
                </fo:wrapper>
            </xsl:when>
            <xsl:otherwise>
                <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format" font-size="10pt"> <!-- font-size="8pt" ? -->
                    <xsl:if test="(../../componentType='COMPOSITE')">
                        <xsl:attribute name="id">
                            <xsl:value-of select="req:identifier"/>
                        </xsl:attribute>
                    </xsl:if>
                    <fo:block font-size="14pt" space-after="4mm" border-after-style="solid"
                        border-after-width="thin">
                        <xsl:value-of
                            select="$strings/labelRequirement"/>
                        <xsl:value-of select="//ns2:generatedId"/> :
                        <xsl:value-of select="req:title"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/reqLabel_status"/>
						:
						<xsl:choose>
                            <xsl:when test="ns3:state='authoring.s3'">
								<xsl:value-of select="$strings/ResolvedBy"/>
								<xsl:value-of select="ns2:resolver"/>
                            </xsl:when>
                            <xsl:when test="ns3:state='authoring.s1'">
								<xsl:value-of select="$strings/reqStateNew"/>
							</xsl:when>
                            <xsl:when test="ns3:state='authoring.s2'">
								<xsl:value-of select="$strings/InProgress"/>
							</xsl:when>
                            <xsl:when test="ns3:state='authoring.s6'">
								<xsl:value-of select="$strings/Reopened"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="ns3:state"/>
							</xsl:otherwise>
                        </xsl:choose>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/reqLabel_created_on"/>
						:
						<xsl:value-of select="ns2:creationtime"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/reqLabel_created_by"/>
						:
						<xsl:value-of select="req:creator"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelOwnedBy"/>
						:
						<xsl:value-of select="ns3:owner"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelModifiedDate"/>
						:
						<xsl:value-of select="ns3:updated"/>
                    </fo:block>
                    <fo:block space-before="4mm">
                        <xsl:value-of select="$strings/labelSeverity"/>
						:
						<xsl:apply-templates select="ns2:severity"/>
                    </fo:block>
                    <fo:block>
                    	<xsl:value-of select="$strings/reqLabel_priority"/>
						:
						<xsl:value-of select="ns2:customAttributes/ns2:customAttribute[ns2:name='Requirement Priority']/ns2:value" />
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelReviewStatus"/>
						:
						<xsl:value-of select="ns2:reviewStatus"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelTags"/>
						:
						<xsl:for-each select="ns2:tags/tag">
            				<xsl:value-of select="text" />
							<xsl:if test="position()!=last()">
								<xsl:text>, </xsl:text>
							</xsl:if>
                        </xsl:for-each>
                    </fo:block>
					<fo:block>
						<xsl:value-of select="$strings/labelPackageInfo"/>
						: 
						<xsl:value-of select="ns2:customAttributes/ns2:customAttribute[ns2:name='Package Information']/ns2:value"/>
					</fo:block>
                    <fo:block space-before="4mm" space-after="4mm">
                        <xsl:value-of select="$strings/labeldescription"/>
						:
						<xsl:value-of select="req:description"/>
                    </fo:block>
                    <xsl:apply-templates select="ns2:comments"/>
                </fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ns2:comments">
        <xsl:if test="comment">
            <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                <fo:table font-size="8pt">
                    <fo:table-column column-width="10mm"/>
                    <fo:table-column/>
                    <fo:table-column/>
                    <fo:table-body>
                        <fo:table-row background-color="#E5EFF5">
                            <fo:table-cell padding="3pt" number-columns-spanned="3" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block space-after="4mm">
                                        <xsl:value-of
                                            select="$strings/reqLabel_discussion"
                                        />: ( <xsl:value-of select="count(comment)"/>
                                        <xsl:value-of
                                            select="$strings/reqLabel_comments"
                                        /> ) </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                        </fo:table-row>
                        <xsl:for-each select="comment">
                            <fo:table-row keep-with-next="always" background-color="#F3F3F3" font-size="8pt">
                                <fo:table-cell padding="3pt" border-left="0.5pt solid black">
                                    <fo:block-container overflow="hidden">
                                        <fo:block>
                                            <xsl:number/> . </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                                <fo:table-cell padding="3pt">
                                    <fo:block-container overflow="hidden">
                                        <fo:block text-align="left">
                                            <xsl:value-of select="creator"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                                <fo:table-cell padding="3pt" border-right="0.5pt solid black">
                                    <fo:block-container overflow="hidden">
                                        <fo:block>
                                            <xsl:value-of select="creationDate"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                            </fo:table-row>
                            <fo:table-row>
                                <fo:table-cell padding="3pt" border="0.5pt solid black" number-columns-spanned="3">
                                    <fo:block-container overflow="hidden">
                                        <fo:block text-align="left">
                                            <xsl:apply-templates select="content"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                            </fo:table-row>
                        </xsl:for-each>
                    </fo:table-body>
                </fo:table>
            </fo:block>
        </xsl:if>
    </xsl:template>
	
	<xsl:template match="ns2:severity">
		<xsl:choose>
			<xsl:when test="current()='Minor'">
				<xsl:value-of select="$strings/sevMinor"/>
			</xsl:when>
			<xsl:when test="current()='Normal'">
				<xsl:value-of select="$strings/sevNormal"/>
			</xsl:when>
			<xsl:when test="current()='Major'">
				<xsl:value-of select="$strings/sevMajor"/>
			</xsl:when>
			<xsl:when test="current()='Critical'">
				<xsl:value-of select="$strings/sevCritical"/>
			</xsl:when>
			<xsl:when test="current()='Blocker'">
				<xsl:value-of select="$strings/sevBlocker"/>
			</xsl:when>
			<xsl:when test="current()='Unclassified'">
				<xsl:value-of select="$strings/sevUnclassified"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="ns2:severity"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
    
    <xsl:template match="content">
        <xsl:apply-templates select="*|text()"/>
    </xsl:template>
    
</xsl:stylesheet>
