<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:fox="http://xml.apache.org/fop/extensions"
    xmlns:atom="http://www.w3.org/2005/Atom" 
	xmlns:ns2="http://schema.ibm.com/vega/2008/"
	xmlns:ns3="http://purl.org/dc/elements/1.1/" 
	xmlns:ns4="http://jazz.net/xmlns/alm/v0.1/"
	xmlns:ns5="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
	xmlns:ns6="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
	xmlns:ns7="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
	xmlns:ns8="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/"
	xmlns:ts="http://jazz.net/xmlns/alm/qm/v0.1/"
	xmlns:xhtml="http://www.w3.org/1999/xhtml"	
	exclude-result-prefixes="ns8 ns7 ns6 ns5 ns4 ns3 ns2 ts xsl">
	<!--
		========================= HTML FORMATTING ===========================
	-->
	<!--
		============================================ This stylesheet
		transforms most of the common HTML elements into XSL formatting
		objects. This version written 1 November 2002 by Doug Tidwell,
		dtidwell@us.ibm.com. Brought to you by your friends at developerWorks:
		ibm.com/developerWorks.
		===============================================
	-->
	<!--
		============================================ This variable sets the
		size of the page. If it's 'ltr', we an 8.5 x 11 inch page; otherwise,
		we use an A4-sized page. 'ltr' is the default. To change the value,
		you can make the following line read select="'A4'", or you can check
		the documentation for your XSLT processor to see how to set variables
		externally to the style sheet. (For Xalan, add "-PARAM page-size a4"
		to the command.) ===============================================
	-->
	<xsl:variable name="imageScale">
		.40
	</xsl:variable>
	<xsl:variable name="page-size" select="'ltr'" />
	<xsl:template match="atom:body">
		<!--
			============================================ Because character
			entities aren't built into the XSL-FO vocabulary, they're included
			here. ===============================================
		-->
		<fo:block>
			<xsl:apply-templates select="*|text()" />
		</fo:block>
	</xsl:template>
	<!--
		============================================ Templates for individual
		HTML elements begin here.
		===============================================
	-->
	<!--
		============================================ Processing for the anchor
		tag is complex. First of all, if this is a named anchor, we write an
		empty <fo:block> with the appropriate id. (In the special case that
		the next element is an <h1>, we ignore the element altogether and put
		the id on the <h1>.) Next, if this is a regular anchor and the href
		starts with a hash mark (#), we create a link with an
		internal-destination. Otherwise, we create a link with an external
		destination. ===============================================
	-->
	<xsl:template match="xhtml:a">
		<xsl:choose>
			<xsl:when test="@name">
				<xsl:if test="not(name(following-sibling::*[1]) = 'h1')">
					<!-- WI 20685 copy/paste from MS Word introduces a TOC
					     that falls into this case.  The text needs to be
					     included in the output so that the section headings
					     are visible. -->
					<xsl:apply-templates select="*|text()" />
				</xsl:if>
			</xsl:when>
			<xsl:when test="@href">
				<fo:basic-link color="blue">
					<xsl:choose>
						<xsl:when test="starts-with(@href, '#')">
							<xsl:attribute name="internal-destination">
                				<xsl:value-of select="substring(@href, 2)" />
             				</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="external-destination">
            	    			<xsl:value-of select="@href" />
              				</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:apply-templates select="*|text()" />
				</fo:basic-link>
				<xsl:if test="starts-with(@href, '#')">
					<xsl:text>
 						on page 
					</xsl:text>
					<fo:page-number-citation ref-id="{substring(@href, 2)}" />
				</xsl:if>
			</xsl:when>
		</xsl:choose>
	</xsl:template>
	<!--
		============================================ We render an address
		element in italics. ===============================================
	-->
	<xsl:template match="xhtml:address">
		<fo:block font-style="italic" space-after="12pt">
			<xsl:apply-templates select="*|text()" />
		</fo:block>
	</xsl:template>
	<!--
		============================================ For bold elements, we
		just change the font-weight.
		===============================================
	-->
	<xsl:template match="xhtml:b">
		<fo:inline font-weight="bold">
			<xsl:apply-templates select="*|text()" />
		</fo:inline>
	</xsl:template>
	<!--
		============================================ The big element is
		handled with a relative font size. That means a <big> element inside
		another <big> element will be even bigger, just as it is in HTML.
		===============================================
	-->
	<xsl:template match="xhtml:big">
		<fo:inline font-size="120%">
			<xsl:apply-templates select="*|text()" />
		</fo:inline>
	</xsl:template>
	<!--
		============================================ A blockquote is indented
		on both sides. ===============================================
	-->
	<xsl:template match="xhtml:blockquote">
		<fo:block start-indent="1.5cm" end-indent="1.5cm"
			space-after="12pt">
			<xsl:apply-templates select="*|text()" />
		</fo:block>
	</xsl:template>
	<!--
		============================================ The HTML <body> element
		contains everything in the main part of the document. This is
		analogous to the <fo:flow flow-name="xsl-region-body"> element, so we
		put the main document processing here.
		===============================================
	-->
	<!--
		============================================ We handle a break element
		by inserting an empty <fo:block>.
		===============================================
	-->
	<xsl:template match="xhtml:br">
	  <fo:block><fo:leader/></fo:block>
	</xsl:template>
	<!--
		============================================ We're handling <center>
		as a block element; if you use it, it creates a new paragraph that's
		centered on the page. ===============================================
	-->
	<xsl:template match="xhtml:center">
		<fo:block text-align="center">
			<xsl:apply-templates select="*|text()" />
		</fo:block>
	</xsl:template>
	<!--
		============================================ The <cite> element is
		rendered in italics, unless it's inside an italic (<i>) element. We
		use the parent axis to check this.
		===============================================
	-->
	<xsl:template match="xhtml:cite">
		<xsl:choose>
			<xsl:when test="parent::i">
				<fo:inline font-style="normal">
					<xsl:apply-templates select="*|text()" />
				</fo:inline>
			</xsl:when>
			<xsl:otherwise>
				<fo:inline font-style="italic">
					<xsl:apply-templates select="*|text()" />
				</fo:inline>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--
		============================================ We render <code> inline
		in a monospaced font. ===============================================
	-->
	<xsl:template match="xhtml:code">
		<fo:inline font-family="monospace">
			<xsl:apply-templates select="*|text()" />
		</fo:inline>
	</xsl:template>
	<!--
		============================================ We don't do anything with
		the <dl> element, we just handle the elements it contains. Notice that
		we're ignoring any text that appears in the <dl> itself; I'm not sure
		if that's the right call.
		===============================================
	-->
	<xsl:template match="xhtml:dl">
		<xsl:apply-templates select="*" />
	</xsl:template>
	<!--
		============================================ A definition term is
		rendered in bold. We specify keep-with-next here, although it doesn't
		always work with FOP. ===============================================
	-->
	<xsl:template match="xhtml:dt">
		<fo:block font-weight="bold" space-after="2pt"
			keep-with-next="always">
			<xsl:apply-templates select="*|text()" />
		</fo:block>
	</xsl:template>
	<!--
		============================================ We handle each <dd>
		element as an indented block beneath the defined term. If the
		following element is another <dd>, that means it's another definition
		for the same term. In that case, we don't put as much vertical space
		after the block. ===============================================
	-->
	<xsl:template match="xhtml:dd">
		<fo:block start-indent="1cm">
			<xsl:attribute name="space-after">
        <xsl:choose>
          <xsl:when test="name(following::*[1]) = 'dd'">
            <xsl:text>
3pt
</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>
12pt
</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
			<xsl:apply-templates select="*|text()" />
		</fo:block>
	</xsl:template>
	<!--
		============================================ The HTML <em> element is
		typically rendered in italics.
		===============================================
	-->
	<xsl:template match="xhtml:em">
		<fo:inline font-style="italic">
			<xsl:apply-templates select="*|text()" />
		</fo:inline>
	</xsl:template>
	<!--
		============================================ For the <font> element,
		we handle the color, face, and size attributes. Color will work if
		it's one of the twelve colors supported by XSL-FO or it's a hex value
		like x0033ff. (In other words, if you tell FOP to set the color to
		PapayaWhip, you're out of luck.) The face attribute will work if FOP
		supports it. (There are ways to add fonts to FOP, see the FOP
		documentation for more info.) Size is supported for values like
		size="14pt", size="3", and size="+3".
		===============================================
	-->
	<xsl:attribute-set name="spanAttributes" />

	<xsl:template match="xhtml:style" />


	<xsl:template match="xhtml:span">
		<xsl:element name="inline" namespace="http://www.w3.org/1999/XSL/Format">
			<xsl:apply-templates select="@style" />
			<xsl:apply-templates select="*|text()" />
		</xsl:element>
	</xsl:template>

	<xsl:template match="xhtml:div">
		<xsl:element name="block" namespace="http://www.w3.org/1999/XSL/Format">
			<xsl:apply-templates select="@style" />
			<xsl:apply-templates select="*|text()" />
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="xhtml:font">
		<xsl:element name="inline" namespace="http://www.w3.org/1999/XSL/Format">
			<xsl:apply-templates select="@style" />
			<xsl:apply-templates select="@size" />
			<xsl:apply-templates select="@face" />
			<xsl:apply-templates select="@color" />
			<xsl:apply-templates select="*|text()" />
		</xsl:element>
	</xsl:template>

	<xsl:template match="@color">
		<xsl:attribute name="color">
          <xsl:value-of select="." />
        </xsl:attribute>
	</xsl:template>

	<xsl:template match="@size">
		<xsl:variable name="size">
			<xsl:choose>
				<xsl:when test=".">
					<xsl:choose>
						<xsl:when test="contains(., 'pt')">
							<xsl:value-of select="." />
						</xsl:when>
						<xsl:when test=". = '+1'">
							<xsl:text>
110%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '+2'">
              <xsl:text>
120%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '+3'">
              <xsl:text>
130%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '+4'">
              <xsl:text>
140%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '+5'">
              <xsl:text>
150%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '+6'">
              <xsl:text>
175%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '+7'">
              <xsl:text>
200%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '-1'">
              <xsl:text>
90%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '-2'">
              <xsl:text>
80%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '-3'">
              <xsl:text>
70%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '-4'">
              <xsl:text>
60%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '-5'">
              <xsl:text>
50%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '-6'">
              <xsl:text>
40%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '-7'">
              <xsl:text>
30%
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '1'">
              <xsl:text>
8pt
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '2'">
              <xsl:text>
10pt
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '3'">
              <xsl:text>
12pt
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '4'">
              <xsl:text>
14pt
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '5'">
              <xsl:text>
18pt
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '6'">
              <xsl:text>
24pt
</xsl:text>
            </xsl:when>
            <xsl:when test=". = '7'">
              <xsl:text>
36pt
</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>
10pt
</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
          <xsl:if test="string-length($size) &gt; 0">
        <xsl:attribute name="font-size">
          <xsl:value-of select="$size" />
        </xsl:attribute>
      </xsl:if>
  </xsl:template>
  
  <xsl:template match="@face">
        <xsl:variable name="face">
      <xsl:choose>
        <xsl:when test=".">
          <xsl:value-of select="." />
        </xsl:when>
        <xsl:otherwise>
          <xsl:text />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
          <xsl:if test="string-length($face) &gt; 0">
        <xsl:attribute name="font-family">
          <xsl:value-of select="$face" />
        </xsl:attribute>
      </xsl:if>
  </xsl:template>
  
  <xsl:template match="@style">
    <xsl:variable name="textDecoration">
      <xsl:value-of select="substring-before(substring-after(.,'text-decoration: '),';')" />
    </xsl:variable>
	<xsl:variable name="width">
		<xsl:choose>
			<xsl:when
				test="contains(substring-before(substring-after(.,'width: '),';'),'px')">
				<xsl:value-of select="substring-before(substring-after(.,'width: '),'px;')" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="substring-before(substring-after(.,'width: '),';')" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:variable name="height">
		<xsl:choose>
			<xsl:when
				test="contains(substring-before(substring-after(.,'height: '),';'),'px')">
				<xsl:value-of select="substring-before(substring-after(.,'height: '),'px;')" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="substring-before(substring-after(.,'height: '),';')" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
    <xsl:variable name="textAlign">
      <xsl:value-of select="substring-before(substring-after(.,'text-align: '),';')" />
    </xsl:variable>
        <xsl:variable name="textIndent">
      <xsl:value-of select="substring-before(substring-after(.,'text-indent: '),';')" />
    </xsl:variable>
    <xsl:variable name="fontStyle">
      <xsl:value-of select="substring-before(substring-after(.,'font-style: '),';')" />
    </xsl:variable>
    <xsl:variable name="fontWeight">
      <xsl:value-of select="substring-before(substring-after(.,'font-weight: '),';')" />
    </xsl:variable>
    <xsl:variable name="fontFamily">
      <xsl:value-of select="substring-before(substring-after(.,'font-family: '),';')" />
    </xsl:variable>
    <xsl:variable name="size">
      <xsl:value-of select="substring-before(substring-after(.,'font-size: '),';')" />
    </xsl:variable>
    <xsl:variable name="backgroundColor">
    	<xsl:choose>
    		<xsl:when test="contains(.,'background-color:')">
    			<xsl:value-of select="substring-before(substring-after(.,'background-color:'),';')" />
    		</xsl:when>
    		<xsl:when test="contains(.,'background: ')">
    			<xsl:choose>
    				<xsl:when test="contains(.,'rgb')">
						<xsl:value-of select="substring-before(substring-after(.,'background: '),')')" />)
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="substring-before(substring-after(.,'background: '),' ')" />
					</xsl:otherwise>
				</xsl:choose>
		    </xsl:when>
			<xsl:when test="contains(.,'background:')">
				<xsl:choose>
				<!-- Microsoft Office 2007 Hack -->
					<xsl:when test="contains(.,'mso')">
						<xsl:value-of select="substring-before(substring-after(.,'background:'),';')" />
					</xsl:when>
    				<xsl:when test="contains(.,'rgb')">
						<xsl:value-of select="substring-before(substring-after(.,'background:'),')')" />)
					</xsl:when>
					<xsl:when test="contains(.,' ')">
						<xsl:value-of select="substring-before(substring-after(.,'background:'),' ')" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="substring-before(substring-after(.,'background:'),';')" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
    	</xsl:choose>
    </xsl:variable>
    <xsl:variable name="color">
      <xsl:choose>
        <xsl:when test="contains(.,'background-color: ')">
          <xsl:variable name="newStyle" select="concat(substring-before(.,'background-color: '),substring-after(.,'background-color: '))" />
          <xsl:value-of select="substring-before(substring-after($newStyle,'color: '),';')" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="substring-before(substring-after(.,'color: '),';')" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    
      <xsl:if test="string-length($textDecoration) &gt; 0">
        <xsl:attribute name="text-decoration">
          <xsl:value-of select="$textDecoration" />
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="string-length($fontStyle) &gt; 0">
        <xsl:attribute name="font-style">
          <xsl:value-of select="$fontStyle" />
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="string-length($textAlign) &gt; 0">
        <xsl:attribute name="text-align">
          <xsl:value-of select="$textAlign" />
        </xsl:attribute>
      </xsl:if>
<!--      <xsl:if test="string-length($textIndent) &gt; 0">-->
<!--        <xsl:attribute name="text-indent">-->
<!--          <xsl:value-of select="$textIndent" />-->
<!--        </xsl:attribute>-->
<!--      </xsl:if>-->
      <xsl:if test="string-length($fontWeight) &gt; 0">
        <xsl:attribute name="font-weight">
          <xsl:value-of select="$fontWeight" />
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="string-length($fontFamily) &gt; 0">
        <xsl:attribute name="font-family">
          <xsl:value-of select="$fontFamily" />
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="string-length($color) &gt; 0">
        <xsl:attribute name="color">
          <xsl:value-of select="$color" />
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="string-length($backgroundColor) &gt; 0">
        <xsl:attribute name="background-color">
          <xsl:value-of select="$backgroundColor" />
        </xsl:attribute>
      </xsl:if>
      
                <xsl:if test="string-length($size) &gt; 0">
        <xsl:attribute name="font-size">
          <xsl:value-of select="$size" />
        </xsl:attribute>
      </xsl:if>
      
      <xsl:if test="string-length($width) &gt; 0">
        <xsl:attribute name="content-width">
          <xsl:value-of select="$width * $imageScale" />
        </xsl:attribute>
      </xsl:if>
       <xsl:if test="string-length($height) &gt; 0">
        <xsl:attribute name="content-height">
          <xsl:value-of select="$height * $imageScale" />
        </xsl:attribute>
      </xsl:if>
      
  </xsl:template>
  <!-- ============================================
    We render the <h1> by putting a horizontal rule
    and a page break before it.  We also process 
    the id attribute; if the <h1> tag has one, we 
    use it.  If not, we see if the preceding element
    is a named anchor (<a name="x"/>).  If there is
    a named anchor before the <h1>, we use the name
    of the anchor point as the id.
    =============================================== -->
  <xsl:template match="xhtml:h1">
    <fo:block break-before="page">
      <fo:leader leader-pattern="rule" />
    </fo:block>
    <fo:block font-size="28pt" line-height="32pt" keep-with-next="always" space-after="22pt" font-family="serif">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id" />
          </xsl:when>
          <xsl:when test="name(preceding-sibling::*[1]) = 'a' and preceding-sibling::*[1][@name]">
            <xsl:value-of select="preceding-sibling::*[1]/@name" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    <h2> is in a slightly smaller font than an <h1>,
    and it doesn't have a page break or a line.
    =============================================== -->
  <xsl:template match="xhtml:h2">
    <fo:block font-size="24pt" line-height="28pt" keep-with-next="always" space-after="18pt" font-family="serif">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    <h3> is slightly smaller than <h2>.
    =============================================== -->
  <xsl:template match="xhtml:h3">
    <fo:block font-size="21pt" line-height="24pt" keep-with-next="always" space-after="14pt" font-family="serif">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    <h4> is smaller than <h3>.  For the bookmarks
    and table of contents, <h4> is the lowest level
    we include.
    =============================================== -->
  <xsl:template match="xhtml:h4">
    <fo:block font-size="18pt" line-height="21pt" keep-with-next="always" space-after="12pt" font-family="serif">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    <h5> is pretty small, and is underlined to 
    help it stand out. 
    =============================================== -->
  <xsl:template match="xhtml:h5">
    <fo:block font-size="16pt" line-height="19pt" keep-with-next="always" space-after="12pt" font-family="serif" text-decoration="underline">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    <h6> is the smallest heading of all, and is
    underlined and italicized.  
    =============================================== -->
  <xsl:template match="xhtml:h6">
    <fo:block font-size="14pt" line-height="17pt" keep-with-next="always" space-after="12pt" font-family="serif" font-style="italic" text-decoration="underline">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    We render an <hr> with a leader.  Because <hr>
    is empty, we don't have to process any child
    elements. 
    =============================================== -->
  <xsl:template match="xhtml:hr">
    <fo:block>
      <fo:leader leader-pattern="rule" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    Italics.  You can't get much simpler than that.
    =============================================== -->
  <xsl:template match="xhtml:i">
    <fo:inline font-style="italic">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    For the <img> element, we use the src attribute
    as it comes from HTML.  We also check for any 
    width and height attributes.  If those attributes
    are there, we try to use them; height="300px" is
    used as-is, while height="300" is converted to 
    the value "300px".
    =============================================== -->
  <xsl:template match="xhtml:img">
    <fo:block space-after="12pt">
      <fo:external-graphic src="image://{@src}" scaling="non-uniform">
        <xsl:if test="@width">
          <xsl:attribute name="width">
            <xsl:choose>
              <xsl:when test="contains(@width, 'px')">
                <xsl:value-of select="concat(substring-before(@width,'px') * $imageScale,'px')" />
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="concat(@width * $imageScale, 'px')" />
              </xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
        </xsl:if>
        <xsl:if test="@height">
          <xsl:attribute name="height">
            <xsl:choose>
              <xsl:when test="contains(@height, 'px')">
                <xsl:value-of select="concat(substring-before(@height,'px') * $imageScale,'px')" />
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="concat(@height * $imageScale, 'px')" />
              </xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
        </xsl:if>
        <xsl:apply-templates select="@style" />
      </fo:external-graphic>
    </fo:block>
  </xsl:template>
  
  <!-- ============================================
    The <kbd> element is in a slightly larger 
    monospaced text.
    =============================================== -->
  <xsl:template match="xhtml:kbd">
    <fo:inline font-family="monospace" font-size="110%">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    We handle the <li> elements under the <ol> and 
    <ul> elements, so there's no <li> template here.
    =============================================== -->
  <!-- ============================================
    For the <nobr> element, we simply turn off the
    wrap-option. 
    =============================================== -->
  <xsl:template match="xhtml:nobr">
    <fo:inline wrap-option="no-wrap">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    We handle an ordered list with two complications:
    If the list appears inside another list (either 
    an <ol> or <ul>), we don't put any vertical space
    after it.  The other issue is that we indent the
    list according to how deeply nested the list is. 
    =============================================== -->
  <xsl:template match="xhtml:ol">
    <xsl:if test="xhtml:li">
      <xsl:choose>
        <xsl:when test="ancestor::xhtml:ul or ancestor::xhtml:ol">
          <xsl:choose>
            <xsl:when test="parent::xhtml:li">
              <fo:list-block provisional-distance-between-starts="1cm" provisional-label-separation="0.5cm">
                <xsl:attribute name="space-after">
                  <xsl:text>0pt</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="start-indent">
                  <xsl:variable name="ancestors">
                    <xsl:value-of select="1 + (count(ancestor::xhtml:ol) + count(ancestor::xhtml:ul)) * 1.25" />
                  </xsl:variable>
                  <xsl:value-of select="concat($ancestors, 'cm')" />
                </xsl:attribute>
                <xsl:apply-templates select="*" />
              </fo:list-block>
            </xsl:when>
            <xsl:otherwise>
              <fo:list-item>
                <fo:list-item-label>
                  <fo:block />
                </fo:list-item-label>
                <fo:list-item-body>
                  <fo:block>
                    <xsl:apply-templates select="@style" />
                    <fo:list-block provisional-distance-between-starts="1cm" provisional-label-separation="0.5cm">
                      <xsl:attribute name="space-after">
                        <xsl:text>0pt</xsl:text>
                      </xsl:attribute>
                      <xsl:attribute name="start-indent">
                        <xsl:variable name="ancestors">
                          <xsl:value-of select="1 + (count(ancestor::xhtml:ol) + count(ancestor::xhtml:ul)) * 1.25" />
                        </xsl:variable>
                        <xsl:value-of select="concat($ancestors, 'cm')" />
                      </xsl:attribute>
                      <xsl:apply-templates select="*" />
                    </fo:list-block>
                  </fo:block>
                </fo:list-item-body>
              </fo:list-item>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <fo:list-block provisional-distance-between-starts="1cm" provisional-label-separation="0.5cm">
            <xsl:attribute name="start-indent">
              <xsl:text>1cm</xsl:text>
            </xsl:attribute>
            <xsl:apply-templates select="*" />
          </fo:list-block>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:template>
  <!-- ============================================
    When we handle items in an ordered list, we need
    to check if the list has a start attribute.  If
    it does, we change the starting number accordingly.
    Once we've figured out where to start counting,
    we check the type attribute to see what format
    the numbers should use.  
    =============================================== -->
  <xsl:template match="xhtml:ol/xhtml:li">
    <fo:list-item>
      <fo:list-item-label end-indent="label-end()">
        <fo:block>
              <xsl:apply-templates select="@style" />
          <xsl:variable name="value-attr">
            <xsl:choose>
              <xsl:when test="../@start">
                <xsl:number value="count(preceding-sibling::xhtml:li) + ../@start" />
              </xsl:when>
              <xsl:otherwise>
                <xsl:number value="count(preceding-sibling::xhtml:li) + 1" />
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:choose>
            <xsl:when test="../@type='i'">
              <xsl:number value="$value-attr" format="i." />
            </xsl:when>
            <xsl:when test="../@type='I'">
              <xsl:number value="$value-attr" format="I." />
            </xsl:when>
            <xsl:when test="../@type='a'">
              <xsl:number value="$value-attr" format="a." />
            </xsl:when>
            <xsl:when test="../@type='A'">
              <xsl:number value="$value-attr" format="A." />
            </xsl:when>
            <xsl:otherwise>
              <xsl:number value="$value-attr" format="1." />
            </xsl:otherwise>
          </xsl:choose>
        </fo:block>
      </fo:list-item-label>
      <fo:list-item-body start-indent="body-start()">
        <fo:block>
              <xsl:apply-templates select="@style" />
          <xsl:apply-templates select="*|text()" />
        </fo:block>
      </fo:list-item-body>
    </fo:list-item>
  </xsl:template>
  <!-- ============================================
    Your basic paragraph.
    =============================================== -->
  <xsl:template match="xhtml:p">
    <fo:block font-size="12pt" line-height="15pt" space-after="12pt">
       			 <xsl:apply-templates select="@style" />
      <xsl:apply-templates select="*|text()" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    Preformatted text is rendered in a monospaced
    font.  We also have to set the wrap-option
    and white-space-collapse properties.  
    =============================================== -->
  <xsl:template match="xhtml:pre">
    <fo:block font-family="monospace" white-space-collapse="false" wrap-option="no-wrap">
      <xsl:apply-templates select="*|text()" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    Sample text is rendered in a slightly larger
    monospaced font. 
    =============================================== -->
  <xsl:template match="xhtml:samp">
    <fo:inline font-family="monospace" font-size="110%">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    The <small> element is rendered with a relative
    font size.  That means putting one <small>
    element inside another creates really small 
    text.  
    =============================================== -->
  <xsl:template match="xhtml:small">
    <fo:inline font-size="80%">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    For strikethrough text, we use the text-decoration
    property.  
    =============================================== -->
  <xsl:template match="xhtml:strike">
    <fo:inline text-decoration="line-through">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    Strongly emphasized text is simply rendered
    in bold. 
    =============================================== -->
  <xsl:template match="xhtml:strong">
    <fo:inline font-weight="bold">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    For subscript text, we use the vertical-align
    property and decrease the font size.  
    =============================================== -->
  <xsl:template match="xhtml:sub">
    <fo:inline vertical-align="sub" font-size="75%">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    Superscript text changes the vertical-align
    property also, and uses a smaller font.
    =============================================== -->
  <xsl:template match="xhtml:sup">
    <fo:inline vertical-align="super" font-size="75%">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    Tables are a hassle.  The main problem we have
    is converting the cols attribute into some 
    number of <fo:table-column> elements.  We do 
    this with a named template called build-columns.
    Once we've processed the cols attribute, we 
    invoke all of the templates for the children 
    of this element. 
    =============================================== -->
  <xsl:template match="xhtml:table">
    <fo:table table-layout="fixed">
      <xsl:apply-templates select="@style" />
      <!-- 
        We are going to just ignore this attribute because its use is not consistent.  Sometimes it is a single 
        element and is meant to define how many columns there will be, other times it is a series of numbers which 
        define the size of each column.  This attribute has been deprecated anyway so it is best to just ignore it.
        
        <xsl:choose>
        <xsl:when test="@cols">
          <xsl:call-template name="build-columns">
            <xsl:with-param name="xhtml:cols" select="concat(@cols, ' ')" />
          </xsl:call-template>
        </xsl:when>
        <!-\-<xsl:otherwise>
          <fo:table-column column-width="200pt" />
        </xsl:otherwise>-\->
      </xsl:choose>
      -->
      <fo:table-body>
        <xsl:apply-templates select="*" />
      </fo:table-body>
    </fo:table>
  </xsl:template>
  
  <!--
  <xsl:template match="xhtml:tbody">
	<xsl:apply-templates select="*" />
  </xsl:template>
  -->
  
  <!-- 
    ============================================
    <td>
    ============================================
    For a table cell, we put everything inside a <fo:table-cell> element.  We set the padding property correctly, then 
    we set the border style.  For the border style, we look to see if any of the ancestor elements we care about specified 
    a solid border.  Next, we check for the rowspan, colspan, and align attributes.  Notice that for align, we check this 
    element, then go up the ancestor chain until we find the <table> element or we find something that specifies the 
    alignment. 
    ============================================ 
    -->
  <xsl:template match="xhtml:td">
    <fo:table-cell keep-together.within-column="always" padding-start="3pt" padding-end="3pt" padding-before="3pt" padding-after="3pt">
      <xsl:apply-templates select="@style" />
      
      <xsl:if test="@bgcolor">
        <xsl:attribute name="background-color">
          <xsl:value-of select="@bgcolor" />
        </xsl:attribute>
      </xsl:if>
      
      <xsl:if test="@colspan">
        <xsl:attribute name="number-columns-spanned">
          <xsl:value-of select="@colspan" />
        </xsl:attribute>
      </xsl:if>
      
      <xsl:if test="@rowspan">
        <xsl:attribute name="number-rows-spanned">
          <xsl:value-of select="@rowspan" />
        </xsl:attribute>
      </xsl:if>
      
      <xsl:if test="@border='1' or ancestor::xhtml:tr[@border] or ancestor::xhtml:thead[@border] or ancestor::xhtml:table[@border]">
        <xsl:attribute name="border-style">
          <xsl:text>solid</xsl:text>
        </xsl:attribute>
        <xsl:attribute name="border-color">
          <xsl:text>black</xsl:text>
        </xsl:attribute>
        <xsl:attribute name="border-width">
          <xsl:text>thin</xsl:text>
        </xsl:attribute>
      </xsl:if>
      
      <xsl:variable name="align">
        <xsl:choose>
          <xsl:when test="@align">
            <xsl:choose>
              <xsl:when test="@align='center'">
                <xsl:text>center</xsl:text>
              </xsl:when>
              <xsl:when test="@align='right'">
                <xsl:text>end</xsl:text>
              </xsl:when>
              <xsl:when test="@align='justify'">
                <xsl:text>justify</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>start</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:when test="ancestor::xhtml:tr[@align]">
            <xsl:choose>
              <xsl:when test="ancestor::xhtml:tr/@align='center'">
                <xsl:text>center</xsl:text>
              </xsl:when>
              <xsl:when test="ancestor::xhtml:tr/@align='right'">
                <xsl:text>end</xsl:text>
              </xsl:when>
              <xsl:when test="ancestor::xhtml:tr/@align='justify'">
                <xsl:text>justify</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>start</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:when test="ancestor::xhtml:thead">
            <xsl:text>center</xsl:text>
          </xsl:when>
          <xsl:when test="ancestor::xhtml:table[@align]">
            <xsl:choose>
              <xsl:when test="ancestor::xhtml:table/@align='center'">
                <xsl:text>center</xsl:text>
              </xsl:when>
              <xsl:when test="ancestor::xhtml:table/@align='right'">
                <xsl:text>end</xsl:text>
              </xsl:when>
              <xsl:when test="ancestor::xhtml:table/@align='justify'">
                <xsl:text>justify</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>start</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>start</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      
      <fo:block text-align="{$align}">
        <xsl:apply-templates select="*|text()"/>
      </fo:block>
    </fo:table-cell>
  </xsl:template>
  
  <!-- ============================================
    The rarely-used <tfoot> element contains some
    number of <tr> elements; we just invoke the 
    template for <tr> here. 
    =============================================== -->
  <xsl:template match="xhtml:tfoot">
    <xsl:apply-templates select="tr" />
  </xsl:template>
  <!-- ============================================
    If there's a <th> element, we process it just 
    like a normal <td>, except the font-weight is 
    always bold and the text-align is always center. 
    =============================================== -->
  <xsl:template match="xhtml:th">
    <fo:table-cell keep-together.within-column="always" padding-start="3pt" padding-end="3pt" padding-before="3pt" padding-after="3pt">
      <xsl:if test="@border='1' or ancestor::xhtml:tr[@border='1'] or ancestor::xhtml:table[@border='1']">
        <xsl:attribute name="border-style">
          <xsl:text>
solid
</xsl:text>
        </xsl:attribute>
        <xsl:attribute name="border-color">
          <xsl:text>
black
</xsl:text>
        </xsl:attribute>
        <xsl:attribute name="border-width">
          <xsl:text>
1pt
</xsl:text>
        </xsl:attribute>
      </xsl:if>
      <fo:block font-weight="bold" text-align="center">
        <xsl:apply-templates select="*|text()" />
      </fo:block>
    </fo:table-cell>
  </xsl:template>
  <!-- ============================================
    Just like <tfoot>, the rarely-used <thead> element
    contains some number of table rows.  We just 
    invoke the template for <tr> here. 
    =============================================== -->
  <xsl:template match="xhtml:thead">
    <xsl:apply-templates select="xhtml:tr" />
  </xsl:template>
  <!-- ============================================
    The title of the document is rendered in a large
    bold font, centered on the page.  This is the 
    <title> element in the <head> in <html>.
    =============================================== -->
  <xsl:template match="xhtml:title">
    <fo:block space-after="18pt" line-height="27pt" font-size="24pt" font-weight="bold" text-align="center">
      <xsl:apply-templates select="*|text()" />
    </fo:block>
  </xsl:template>
  <!-- ============================================
    For an HTML table row, we create an XSL-FO table
    row, then invoke the templates for everything 
    inside it. 
    =============================================== -->
  <xsl:template match="xhtml:tr">
    <fo:table-row>
    <xsl:choose>
    
      <xsl:when test="count(child::xhtml:td)=0">
      	<fo:table-cell keep-together.within-column="always"><fo:block/></fo:table-cell>
      </xsl:when>
      <xsl:otherwise>
      <xsl:apply-templates select="@style" />
      <xsl:apply-templates select="*|text()" />
      </xsl:otherwise>
      </xsl:choose>
    </fo:table-row>
  </xsl:template>
  <!-- ============================================
    Teletype text is rendered in a monospaced font.
    =============================================== -->
  <xsl:template match="xhtml:tt">
    <fo:inline font-family="monospace">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    For underlined text, we use the text-decoration
    property.
    =============================================== -->
  <xsl:template match="xhtml:u">
    <fo:inline text-decoration="underline">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    The unordered list is pretty straightforward; 
    the only complication is calculating the space-
    after and start-indent properties.  If this 
    list is inside another list, we don't put any 
    space after this one, and we calculate the 
    indentation based on the nesting level of this 
    list. 
    =============================================== -->
  <xsl:template match="xhtml:ul">
    <xsl:if test="xhtml:li">
      <xsl:choose>
        <xsl:when test="ancestor::xhtml:ul or ancestor::xhtml:ol">
          <xsl:choose>
            <xsl:when test="parent::xhtml:li">
              <fo:list-block provisional-distance-between-starts="1cm" provisional-label-separation="0.5cm">
                <xsl:attribute name="space-after">
                  <xsl:text>0pt</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="start-indent">
                  <xsl:variable name="ancestors">
                    <xsl:value-of select="1 + (count(ancestor::xhtml:ol) + count(ancestor::xhtml:ul)) * 1.25" />
                  </xsl:variable>
                  <xsl:value-of select="concat($ancestors, 'cm')" />
                </xsl:attribute>
                <xsl:apply-templates select="*" />
              </fo:list-block>
            </xsl:when>
            <xsl:otherwise>
              <fo:list-item>
                <fo:list-item-label>
                  <fo:block />
                </fo:list-item-label>
                <fo:list-item-body>
                  <fo:block>
                    <xsl:apply-templates select="@style" />
                    <fo:list-block provisional-distance-between-starts="1cm" provisional-label-separation="0.5cm">
                      <xsl:attribute name="space-after">
                        <xsl:text>0pt</xsl:text>
                      </xsl:attribute>
                      <xsl:attribute name="start-indent">
                        <xsl:variable name="ancestors">
                          <xsl:value-of select="1 + (count(ancestor::xhtml:ol) + count(ancestor::xhtml:ul)) * 1.25" />
                        </xsl:variable>
                        <xsl:value-of select="concat($ancestors, 'cm')" />
                      </xsl:attribute>
                      <xsl:apply-templates select="*" />
                    </fo:list-block>
                  </fo:block>
                </fo:list-item-body>
              </fo:list-item>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <fo:list-block provisional-distance-between-starts="1cm" provisional-label-separation="0.5cm">
            <xsl:attribute name="space-after">
              <xsl:text>12pt</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="start-indent">
              <xsl:text>1cm</xsl:text>
            </xsl:attribute>
            <xsl:apply-templates select="*" />
          </fo:list-block>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:template>
  <!-- ============================================
    List items inside unordered lists are easy; we
    just have to use the correct Unicode character
    for the bullet.  
    =============================================== -->
  <xsl:template match="xhtml:ul/xhtml:li">
    <fo:list-item>
      <fo:list-item-label end-indent="label-end()">
        <fo:block>
   			 <xsl:apply-templates select="@style" />
   			 <xsl:choose>
   			 <xsl:when test="(count(ancestor::xhtml:ol) + count(ancestor::xhtml:ul))=1">&#x2022;</xsl:when>
   			 <xsl:when test="(count(ancestor::xhtml:ol) + count(ancestor::xhtml:ul))=2">&#x2022;</xsl:when>
   			 <xsl:otherwise>&#x2022;</xsl:otherwise>
   			 </xsl:choose>
   			 </fo:block>
      </fo:list-item-label>
      <fo:list-item-body start-indent="body-start()">
        <fo:block>
   			 <xsl:apply-templates select="@style" />
          <xsl:apply-templates select="*|text()" />
        </fo:block>
      </fo:list-item-body>
    </fo:list-item>
  </xsl:template>
  <!-- ============================================
    The <var> element is rendered in italics. 
    =============================================== -->
  <xsl:template match="xhtml:var">
    <fo:inline font-style="italic">
      <xsl:apply-templates select="*|text()" />
    </fo:inline>
  </xsl:template>
  <!-- ============================================
    This template generates an <fo:table-column>
    element for each token in the cols attribute of
    the HTML <table> tag.  The template processes
    the first token, then invokes itself with the 
    rest of the string. 
    =============================================== -->
  <xsl:template name="build-columns">
    <xsl:param name="cols" />
    <xsl:if test="string-length(normalize-space($cols))">
      <xsl:variable name="next-col">
        <xsl:value-of select="substring-before($cols, ' ')" />
      </xsl:variable>
      <xsl:variable name="remaining-cols">
        <xsl:value-of select="substring-after($cols, ' ')" />
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="contains($next-col, 'pt')">
          <fo:table-column column-width="{$next-col}" />
        </xsl:when>
        <xsl:when test="number($next-col) &gt; 0">
          <fo:table-column column-width="{concat($next-col, 'pt')}" />
        </xsl:when>
        <xsl:otherwise>
          <fo:table-column column-width="50pt" />
        </xsl:otherwise>
      </xsl:choose>
      <xsl:call-template name="build-columns">
        <xsl:with-param name="cols" select="concat($remaining-cols, ' ')" />
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="xhtml:button">
    <fo:inline border-style="solid" border-color="rgb(60,60,75)" padding="0.05cm" color="rgb(80,75,143)" 
      background-color="rgb(203,221,206)">
      <xsl:value-of select="."/></fo:inline>
  </xsl:template>
</xsl:stylesheet>
