/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.client.OSLCCmProvider");

dojo.require("jazz.client.xhr");
dojo.require("com.ibm.team.repository.web.foundation.Status");

(function(){

    var Status = com.ibm.team.repository.web.foundation.Status;
    
    dojo.declare("com.ibm.rqm.defects.web.client.OSLCCmProvider", null, {
    
        constructor: function(serviceProvider){
            this._serviceProvider = serviceProvider;
        },
        
        /***
         * Create a new link in the CM service provider. linkDescription: {id, url, linkComment, linkType, rqmUrl}
         * @param {Object} linkDescription
         * @param {function} responseCallback
         */
        postLink: function(linkDescription, responseCallback){
            if (!this._serviceProvider) {
                responseCallback(new Status(Status.OK, ""), "");
            }
            else {
                var itemURL = linkDescription.url + '?oslc_cm.properties=' + escape(linkDescription.linkType);
                
                var deferredGet = jazz.client.xhrGet({
                    url: itemURL,
                    handleAs: "json",
                    headers: {
                        "Accept": "application/json"
                    }
                });
                deferredGet.addBoth(function(result){
					//FIXME - this should be handled by the caller
                    if (result instanceof Error) {
                        var status = new Status(Status.OK);
                        responseCallback(status, result);
                    }
                    else {
                        var etag = deferredGet.ioArgs.xhr.getResponseHeader('Etag');
                        
                        var newLink = {};
                        newLink["oslc_cm:label"] = linkDescription.linkComment;
                        newLink["rdf:resource"] = linkDescription.rqmUrl;
                        
                        var newLinkCollection = result[linkDescription.linkType] || []; // keep existing links
                        newLinkCollection.push(newLink);
                        
                        var data = {};
                        data[linkDescription.linkType] = newLinkCollection;
                        
                        var deferredPut = jazz.client.xhrPut({
                            url: linkDescription.url,
                            postData: dojo.toJson(data),
                            headers: {
                                "Accept": "application/json",
                                "If-Match": etag
                            },
                            contentType: "application/x-oslc-cm-change-request+json"
                        });
                        deferredPut.addBoth(dojo.hitch(this, function(result){
                            var status = new Status(Status.OK);
                            responseCallback(status, result);
                        }));
                    }
                });
            }
        },
        
        /***
         * Delete an existing link in the CM service provider
         * @param {Object} linkDescription
         * @param {Object} responseCallback
         */
        deleteLink: function(linkDescription, responseCallback){
            if (!this._serviceProvider) {
                responseCallback(new Status(Status.OK, ""), "");
            }
            else {
                var itemURL = linkDescription.url + '?oslc_cm.properties=' + escape(linkDescription.linkType);
                
                var deferredGet = jazz.client.xhrGet({
                    url: itemURL,
                    handleAs: "json",
                    headers: {
                        "Accept": "application/json"
                    }
                });
                deferredGet.addBoth(function(result){
                    if (result instanceof Error) {
                        var status = new Status(Status.OK);
                        responseCallback(status, result);
                    }
                    else {
                        var etag = deferredGet.ioArgs.xhr.getResponseHeader('Etag');
                        
                        var newLink = {};
                        newLink["oslc_cm:label"] = linkDescription.linkComment;
                        newLink["rdf:resource"] = linkDescription.rqmUrl;
                        
                        var existLinkCollection = result[linkDescription.linkType] || [];
                        // Remove the link from the existing collection
						var newLinkCollection = [];
                        dojo.forEach(existLinkCollection, function(link){
                            if (link && link["rdf:resource"] != linkDescription.rqmUrl) {
                                newLinkCollection.push(link);
                            }
                        });
                        
                        var data = {};
                        data[linkDescription.linkType] = newLinkCollection;
                        
                        var deferredPut = jazz.client.xhrPut({
                            url: linkDescription.url,
                            postData: dojo.toJson(data),
                            headers: {
                                "Accept": "application/json",
                                "If-Match": etag
                            },
                            contentType: "application/x-oslc-cm-change-request+json"
                        });
                        deferredPut.addBoth(dojo.hitch(this, function(result){
                            var status = new Status(Status.OK);
                            responseCallback(status, result);
                        }));
                    }
                });
            }
        },
        
        __sentinel: null // terminates this class definition
    });
})();
