/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.client.internal.PreferenceClient");

dojo.require("com.ibm.asq.common.web.client.internal.option");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() { // module container 

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

com.ibm.rqm.planning.web.client.internal.PreferenceClient = {
    PREFERENCE_SERVICE_URI: "com.ibm.rqm.planning.common.service.rest.IPreferenceRestService",
    REQUIREMENT_SERVICE_URI: "com.ibm.rqm.requirement.common.service.rest.IRequirementConnectionRestService",
	
    _invokeService: function(serviceURI, responseHandler, method, args){
        var serviceRequest = new ServiceRequest(serviceURI, method, args);
        TeamServerClient.invokeService(serviceRequest, responseHandler);
    },
	
	_invokePreferenceService: function(responseHandler, method, args){
        this._invokeService(this.PREFERENCE_SERVICE_URI, responseHandler, method, args);
    },
	
	_invokeRequirementService: function(responseHandler, method, args){
        this._invokeService(this.REQUIREMENT_SERVICE_URI, responseHandler, method, args);
    },
	
    getPreferenceEntry: function(serviceResponseHandler, projectArea, key){
        var args = {
            projectArea: projectArea,
            key: key
        }
        this._invokePreferenceService(serviceResponseHandler, "getPreferenceEntry", args);
    },

    getPreferenceEntries: function(serviceResponseHandler, projectAreaItemId, keyPrefix, mapToObject){
        var args = {
            projectArea: projectAreaItemId,
            keyPrefix: keyPrefix
        }
		var handler = serviceResponseHandler;
		if (mapToObject && mapToObject == true) {
			handler = this._getPreferencesObjectMapper(serviceResponseHandler, keyPrefix);
		}
        this._invokePreferenceService(handler, "getPreferenceEntries", args);
    },
			    
    savePreferenceEntries: function(serviceResponseHandler, projectArea, keys, values){
        var args = {
            projectArea: projectArea,
            keys: keys,
            values: values
        }
        this._invokePreferenceService(serviceResponseHandler, "postPreferenceEntries", args);
    },
	
	getRequirementConnections: function(serviceResponseHandler, id){
		var args = {
			id: id,
			processArea: getCurrentProjectArea().itemId
		}
		this._invokeRequirementService(serviceResponseHandler, "getRequirementConnections", args);
	},
	
	postRequirementConnection: function(serviceResponseHandler, name, connectionType, hostname, port, ignoreCheck){
		var args = {
			name: name,
			connectionType: connectionType,
			hostname: hostname,
			port: port,
			ignoreCheck: ignoreCheck,
			processArea: getCurrentProjectArea().itemId
		}
		this._invokeRequirementService(serviceResponseHandler, "postRequirementConnection", args);
	},
	
	getDeleteRequirmentConnection: function(serviceResponseHandler, id){
		var args = {
			id: id,
			processArea: getCurrentProjectArea().itemId
		}
		this._invokeRequirementService(serviceResponseHandler, "getDeleteRequirmentConnection", args);
	},
	
	getIsValidConnection: function(serviceResponseHandler, hostname, port){
		var args = {
			hostname: hostname,
			port: port,
			processArea: getCurrentProjectArea().itemId
		}
		this._invokeRequirementService(serviceResponseHandler, "getIsValidConnection", args);
	},
	
	postStopSynchronize: function(serviceResponseHandler, id, syncStopped){
		var args = {
			id: id,
			syncStopped: syncStopped,
			processArea: getCurrentProjectArea().itemId
		}
		this._invokeRequirementService(serviceResponseHandler, "postStopSynchronize", args);
	},
	
	postStartSynchronize: function(serviceResponseHandler, id) {
		var args = {
			id: id,
			processArea: getCurrentProjectArea().itemId
		}
		this._invokeRequirementService(serviceResponseHandler, "postSynchronizeNow", args);
	},
	
	postResetSynchronizeTimeStamp: function(serviceResponseHandler, id) {
		var args = {
			id: id,
			processArea: getCurrentProjectArea().itemId
		}
		this._invokeRequirementService(serviceResponseHandler, "postResetSynchronizeTimeStamp", args);
	},
	
	getSyncStatus: function(serviceResponseHandler, id) {
		var args = {
			id: id,
			processArea: getCurrentProjectArea().itemId
		}
		this._invokeRequirementService(serviceResponseHandler, "getSyncStatus", args);
	},
	
    // Summary:
    //  Obtains the current user's stored preferences.
    //    This method calls the common preference client to obtain the current user's preferences.
    //    The common user preferences are prefetched at application start up time once. The caller
    //    can either just access the stored prefernces, or pass in the refresh flag to update the 
    //    cached values. If the refresh flag is passed, this method calls the callback asynchronously.
    //    Otherwise, the callback is called immediately.
    //
	getCurrentUserPreferences: function(/*MUST Context Object of the call back function*/contextObj, /*MUST Call back function*/callback, /*OPTIONAL, Preference category*/ category, /*OPTIONAL refresh*/refresh){
        this.getCurrentUserPreferencesWithNullPref(contextObj, callback, category, refresh);
    },

    // getCurrentUserPreferences() with the option to invoke the callback with null if the specified preference is not found
    getCurrentUserPreferencesWithNullPref: function(contextObj, callback, category, refresh, callBackNullPref){
		var returnPreferences = function(userPreference){
            if (category) {
                var prefFound = false;
                for (var i = 0; i < userPreference.length; i++) {
                    if (userPreference[i].key == category) {
                        prefFound = true;
                        callback.apply(contextObj, [userPreference[i]]);
                        break;
                    }
                }
                if (callBackNullPref && !prefFound) {
                    callback.apply(contextObj, null);
                }
            }
            else {
                callback.apply(contextObj, [userPreference]);
            }
        }
        if (refresh) {
            var handler = {
                success: function(preferences){
                    returnPreferences(preferences);
                },
                error: function(e){
                    console.debug(e);
                }
            };
            var responseHandler = new ServiceResponseHandler(handler, "success", "error");
            com.ibm.asq.common.web.client.internal.CommonPreferenceClient.fetchAndStoreUserPreferences(responseHandler);
            return;
        }    
        var prefs = com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getStoredUserPreferences();
        returnPreferences(prefs);
	},
	
	saveCurrentUserPreferences: function(userPreference, handler){
		com.ibm.asq.common.web.client.internal.CommonPreferenceClient.mergeAndSaveUserPreferences(userPreference, handler);
	},
	
    _getPreferencesObjectMapper: function(responseHandler, keyPrefix) {
        var internalHandler = {
            success: function(preferences) {
				var prefsObj = null;
                if (dojo.isArray(preferences) && preferences.length > 0) {
					//map each preference to an object based on its key (minus the keyPrefix)
					prefsObj = {};
					for (var i = 0; i < preferences.length; i++) {
						var propName = preferences[i].key.substr(keyPrefix.length + 1);
						propName = propName.charAt(0) != "." ? propName : propName.substr(1); //strip leading dot
						var strValue = preferences[i].value;
						//try some basic type mapping - number, boolean, string
						if (!isNaN(new Number(strValue))) {
							prefsObj[propName] = new Number(strValue);
						} else if (strValue == "true" || strValue == "false") {
							prefsObj[propName] = (strValue == "true");
						} else {
							prefsObj[propName] = strValue;
						}
					}
				}
                responseHandler.doCallback(prefsObj);
            },
            failure: function(error){
                responseHandler.doErrorCallback(error);
            }
        };
        var handler = new ServiceResponseHandler(internalHandler, "success", "failure");
        return handler;
    }	
};

})();
