//collections
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

(function(){
dojo.provide("com.ibm.rational.test.lm.web.client.internal.collections");
    
dojo.require("com.ibm.rational.test.lm.web.client.internal.rawContainer");

dojo.declare("com.ibm.rational.test.lm.web.client.internal.collections",
             com.ibm.rational.test.lm.web.client.internal.rawContainer,
{
    constructor: function() {
        this.setServiceInterface('com.ibm.rational.test.lm.common.IResourceCollectionRestService');
    },

    _getProjectAreaArgs: function() {
        return { projectAreaItemId: com.ibm.team.process.web.projectArea.itemId };      
	},
	
	_getProjectAreaMineOnlyArgs: function(optMineOnly) {
	   return { 
	       projectAreaItemId: com.ibm.team.process.web.projectArea.itemId,
	       mineOnly: (optMineOnly == null) ? false : optMineOnly 
	   };       
	},
	
    getVirtualImageCollections: function(responseHandler) {
        return this._invokeService(responseHandler, "getManagedVirtualImageCollections", this._getProjectAreaArgs());
    },
    
    getVirtualMachineCollections: function(responseHandler) {
        return this._invokeService(responseHandler, "getManagedVirtualMachineCollections", this._getProjectAreaArgs());
    },
    
    // Associate a configuration to the collection.  Specify null for the configuration id to
    // remove the association.
    //
    setConfiguration: function(responseHandler, collId, configId) {
        var arg = { colUuid: collId };
        if (configId) {
            arg.configUuid = configId;
        }
        return this._invokeService(responseHandler, "postConfigurationToCollection", arg);
    },

    // Returns an array of summaries     
    getAllTestCells: function(responseHandler) {
        return this._invokeService(responseHandler, "getAll", this._getProjectAreaArgs());
    },

    // Returns an array of summaries
    getDeletedTestCells: function(responseHandler) {
        return this._invokeService(responseHandler, "getDeleted", this._getProjectAreaArgs());    
    },
    
    // Returns a complete DTO
    getTestCell: function(responseHandler, id) {
        return this._invokeService(responseHandler, "getTestCell", { id: id });
    },
   
    // Returns the item ID to use for null values in cell topologies and other array'd items
    // for which it is bad to use JavaScript null values.
    getNullId: function() { return "_Yc3RM2MeEd2swpeAmIeDTg"; },
    
    //
    // Requires permission to perform the "create" action on the "com.ibm.rqm.testcell.save" operation.
    // 
    // - instanceType is the integer portion of the DTO returned by getAllTestCells()/getTestCell().
    // - topologyResourceIds, topologyNotes and topologyDescriptionIds must be arrays of the same length - each slice is treated as a single item (use null for optional LRD ID and note)
    //
    // Returns a complete representation of the new test cell.
    //  

    createTestCell: function(responseHandler, name, description, testEnvironmentId, instanceType, owner, longDescription, topologyResourceIds, topologyResourceDescriptionIds, topologyNotes) {
        var arg = {
            projectAreaItemId: com.ibm.team.process.web.projectArea.itemId,
			name: name,  //required
			instanceType: instanceType  //required
        };
		
		//Build up the args, discarding those not provided
		if (description)
            arg.description = description;
		if (testEnvironmentId) 
            arg.testEnvironmentId = testEnvironmentId;
		if (owner)
            arg.owner = owner;
		if (longDescription)
            arg.longDescription = longDescription;
		if (topologyResourceIds && topologyResourceIds.length > 0) 
		{
			arg.topologyResourceIds = topologyResourceIds;
			arg.topologyResourceDescriptionIds = topologyResourceDescriptionIds;
			arg.topologyNotes = topologyNotes;
		}             		
        
        return this._invokeService(responseHandler, "postCreateTestCell", arg); 
    },
    
    //
    // Requires permission to perform the "edit" action on the "com.ibm.rqm.testcell.save" operation. 
    // 
    // Argument semantics are identical to those found in createTestCell().  The
    // test cell's item and state IDs are required.
    //
    // Returns a complete representation of the updated test cell.
    // 
    updateTestCell: function(responseHandler, itemId, stateId, name, description, testEnvironmentId, owner, longDescription, topologyResourceIds, topologyResourceDescriptionIds, topologyNotes) {

        var arg = {
			name: name,  //required
			itemId: itemId,
			stateId: stateId
        };
		
		//Build up the args, discarding those not provided
		if (description)
            arg.description = description;
		if (testEnvironmentId) 
            arg.testEnvironmentId = testEnvironmentId;
		if (owner)
            arg.owner = owner;
		if (longDescription)
            arg.longDescription = longDescription;
		if (topologyResourceIds && topologyResourceIds.length > 0) //if resource ids are there then there MUST also be arrays for Lab Resource Description ids and notes.
		{
			arg.topologyResourceIds = topologyResourceIds;
			arg.topologyResourceDescriptionIds = topologyResourceDescriptionIds;
			arg.topologyNotes = topologyNotes;
		}             		
        
        return this._invokeService(responseHandler, "postUpdateTestCell", arg); 
    },
    
    // 
    // Requires permission to perform the "archive" action on the "com.ibm.rqm.testcell.save" operation.
    //
    // Soft-deletes the referenced item.  Returns nothing.
    //
    deleteTestCell: function(responseHandler, itemId) {
        return this._invokeService(responseHandler, "postDeleteTestCell", { uuid: itemId });
    },
    
    // 
    // Requires permission to perform the "archive" action on the "com.ibm.rqm.testcell.save" operation.
    //
    // Restores the referenced item from soft-delete.  Returns nothing.
    //
    restoreTestCell: function(responseHandler, itemId) {
        return this._invokeService(responseHandler, "postRestoreTestCell", { uuid: itemId });    
    },
    
    // Does a case-insensitive partial match on the name field.  If optMineOnly
    // is specified, limits results to those owned by the current user.
    getByName: function(responseHandler, name, optMineOnly) {
        var args = this._getProjectAreaMineOnlyArgs(optMineOnly);
        
        args['name'] = name;
        return this._invokeService(responseHandler, "getByName", args);    
    },

    // Searches for those cells that reference the given test environment.  If
    // optMineOnly is specified, limits results to those owned by the current
    // user.
    getByReferencedTestEnv: function(responseHandler, testEnvId, optMineOnly) {
        var args = this._getProjectAreaArgs();
        
        args['mineOnly'] = (optMineOnly == null) ? false : optMineOnly;
        return this._invokeService(responseHandler, "getByReferencedTestEnvironment", args);
    },
    
    // Searches for all cells in the current project area.  If optMine is 
    // specified, limits results to those owned by the current user.
    getByProjectArea: function(responseHandler, optMineOnly) {
        return this._invokeService(responseHandler, "getByProjectArea", this._getProjectAreaMineOnlyArgs(optMineOnly));
    },

    // Searches for all cells based on the availability of their affiliated 
    // lab resources within a given window.  The mineOnly == true then results
    // will be limited to those test cells owned by the current user.  If 
    // optTestEnv is specified, results will be further limited to those
    // cells affiliated with the given test environment. 
    getByAvailability: function(responseHandler, from, to, mineOnly, optTestEnvId) {
        var args = this._getProjectAreaMineOnlyArgs(mineOnly);
        
        if (optTestEnvId != null) {
            args['testEnvId'] = optTestEnvId;
        }
        args['from'] = from;
        args['to'] = to;
        
        return this._invokeService(responseHandler, "getByAvailability", args);
    },
    
    // Attempts to reserve all of the resources associated with the test cell
    // within the given [from, to) window.  Returns a DTO with fields:
    //
    //   - itemIds: item IDs whose type depends on the value of idsAreReservations
    //   - idsAreReservations: boolean flag
    //       - true: itemIds are the reservations that were generated by this call
    //       - false: itemIds are the lab resources for which reservations were unobtainable
    //
    // Note that we include the test cell's state ID to ensure that we are 
    // operating on the current lab resource list only.
    //
    reserveResources: function(responseHandler, itemId, stateId, from, to) {
        return this._invokeService(responseHandler, "postReserveResources", { 'itemId': itemId, 'stateId': stateId, 'from': from, 'to': to });
    }
    
});

})();
