/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
// Author: Paul Beaulieu
//
// Summary:
//

(function(){
	
dojo.require("com.ibm.asq.common.web.client.internal.container");
dojo.provide("com.ibm.rational.test.lm.web.client.internal.config");

/*
 * Non-API object used for interacting with the server's import REST service. 
 */
dojo.declare("com.ibm.rational.test.lm.web.client.internal.config", 
			  com.ibm.asq.common.web.client.internal.container, 
    {
		constructor: function()
		{
			this.setServiceInterface('com.ibm.rational.test.lm.catalog.common.ISavedConfigurationRestService');
		},
		
		_getProjectAreaArgs: function()
		{
		  	return { projectAreaItemId: com.ibm.team.process.web.projectArea.itemId };
		},
		
        // Error string "property constraint: name" in the errorMessage field of the response indicates a name constraint violation.
		createConfiguration: function( responseHandler, config, configName, configDesc, configSummary )
		{
            var jsonConfig = config.toJson();
    		var args = this._getProjectAreaArgs();
    		
            // normalize any null/undefined args to empty strings.
    		args["configuration"] = jsonConfig;
            args["name"] = configName;
    		args["description"] = configDesc || "";
    		args["summary"] = configSummary || ""; 
    
    		return( this._invokeService( responseHandler, 'postCreateConfiguration', args ) );	
		},

        // Get the ID to use in the configPartIds input to updateConfiguration() to indicate a
        // new config part in the config input field.
        getNewConfigPartId: function() {
            // This must be kept up-to-date with the static field ISavedConfigurationRestService.NEW_LRD_ID. 
            // It must never be translated (although good luck :-)).
            return "_ljcWED5NEd6i_ZciPMaWDQ"; 
        },
        
        // Error string "property constraint: name" in the errorMessage field of the response indicates a name constraint violation.		
		updateConfiguration: function( responseHandler, id, stateId, config, configPartIds, configName, configDesc, configSummary )
		{
			var jsonConfig = config.toJson();
            
            // normalize any null/undefined args to empty strings.
    		var args = {
                'id': id, 
                'stateId': stateId, 
                'configuration': jsonConfig, 
                'resourceDescriptionIds': configPartIds, 
                'name': configName, 
                'description': configDesc || "", 
                'summary': configSummary || ""
            };
			return( this._invokeService( responseHandler, 'postUpdateConfiguration', args ) );				
		},
		
		deleteConfigurations: function( responseHandler, uuid )
		{
			var arrUUID = [];
			
			if( !dojo.isArray( uuid ) )
			{
				arrUUID[ 0 ] = uuid;
			}
			else
			{
				arrUUID = uuid;
			}
			
			var args = {'uuid': arrUUID};
			return( this._invokeService( responseHandler, 'postDeleteConfigurations', args ) );				
		},
		
		retrieveConfiguration: function( responseHandler, uuid )
		{
			return( this._retrieveConfiguration( responseHandler, 'getRetrieveConfiguration', uuid ) ); 
		},

		retrieveConfigurationWithDetails: function( responseHandler, id )
		{
			var args = this._getProjectAreaArgs();
			args["id"] = id;

			return( this._invokeService(responseHandler, 'getRetrieveConfigurationWithDetails', args));
		},
		
		_retrieveConfiguration: function( responseHandler, method, id )
		{
			var args = {'id': id };
			return( this._invokeService( responseHandler, method, args ) );			
		},
		
        getConfigurationsWithDetails: function( responseHandler, uuids )
        {
            var args = {'uuid': uuids};
            return( this._invokeService( responseHandler, 'postConfigurationsWithDetails', args ) ); 
        },
        
		getMatchingConfiguration: function( responseHandler, config )
		{
			var jsonConfig = config.toJson();
			var args = this._getProjectAreaArgs();
			
			args["configuration"] = jsonConfig;
			
			return( this._invokeService( responseHandler, 'getMatchingConfiguration', args ) );			
		},
		
		getAllWithDetails: function( responseHandler )
		{
			return( this._invokeService( responseHandler, 'getAllWithDetails', this._getProjectAreaArgs() ) );						
		},
		
		getAllMinimal: function( responseHandler )
		{
			return( this._invokeService( responseHandler, 'getAllMinimal', this._getProjectAreaArgs() ) );						
		},

		getAll: function( responseHandler)
		{
			return( this._invokeService( responseHandler, 'getAll', this._getProjectAreaArgs()) );						
		},
		searchConfigs:function(responseHandler,args){
			return( this._invokeService( responseHandler, 'getSearchTestEnvironment', args ) );
		}

	}
);

})();
