/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

//
// Author: Paul Beaulieu, Rob Rassmann (with a little butchery by Matt Dawson)
//
// Summary:
//

(function(){
dojo.provide("com.ibm.rational.test.lm.web.client.internal.group");
	
dojo.require("com.ibm.asq.common.web.client.internal.container");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var _catalog = new com.ibm.asq.common.web.client.internal.Catalog();

var groupApi = com.ibm.rational.test.lm.web.client.internal.group;
var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;


dojo.declare("com.ibm.rational.test.lm.web.client.internal.group.groupProcessor", null,
    {
        constructor: function()
        {
        },

        // Convert the group data array to associative array to allow referencing the group
        // data using attribute names.
        //
        groupsArrayToArgs: function(groupsArray) 
		{
            var groups = [];
			if (groupsArray != null) {
                var groups_length = groupsArray.length;
				for (var groupCount = 0; groupCount < groups_length; groupCount++) {
					var group = this.convertGroup(groupsArray[groupCount]);
					groups.push(group);
				}
			}
            return groups;
        },
        
        convertGroup: function(unconvertedGroup)
		{
            var convertedGroup = {};
            var groupAttr = [
                "itemId",
                "expirable",
                "name",
                "description",
                "expirationDate",
                "startDate",
                // "teamArea", this needs to be special cased
				"stateId",
                "deleted"
            ];
            var groupAttr_length = groupAttr.length;
            for (var attrCount = 0; attrCount < groupAttr_length; attrCount++) {
				var attName = groupAttr[attrCount];
                    convertedGroup[attName] = unconvertedGroup[attName];
            }
                
            // date is returned in string
            if (convertedGroup.startDate != null) {
                var date = RtlmUiUtil.convertTimeStampToDate(convertedGroup.startDate);
				// Note we set dates to 0 for null, need to convert them back
				if(date.getTime() <= 86400001){  // Check for day 0 to day 1
					date = null;
				}
                convertedGroup.startDate = date;
            } else {
                 convertedGroup.startDate = null;
            }

            if (convertedGroup.expirationDate != null) {
                var date = RtlmUiUtil.convertTimeStampToDate(convertedGroup.expirationDate);
                if(date.getTime() <= 86400001){  // Check for day 0 to day 1
                    date = null;
                }
                convertedGroup.expirationDate = date;
            } else {
                convertedGroup.expirationDate = null;
            }
			
			// teamArea is returned as an object.  Need to convert it to an Id.
			if (unconvertedGroup.teamArea != null){
				convertedGroup.teamArea = unconvertedGroup.teamArea.itemId;
			}
			return convertedGroup;
        }
});

		
// Summary:
//    Internal object to normalize the group response data for arrays.
//
dojo.declare("com.ibm.rational.test.lm.web.client.internal.group.arrayCallbackObject", null,
{
    constructor: function(handler)
    {
        this.responseHandler = handler;
    },
    successFunc: function(response) 
	{
        if (this.responseHandler) 
		{
			var processor = new groupApi.groupProcessor();
			this.responseHandler._contextObject[this.responseHandler._callbackName](processor.groupsArrayToArgs(response));
		}
    },
    errorFunc: function(errorType, errorMessage) 
	{
        if(this.responseHandler)
            this.responseHandler._contextObject[this.responseHandler._errorCallbackName](errorType, errorMessage);
    }
});
		
// Summary:
//    Internal object to normalize the group response data.
//
dojo.declare("com.ibm.rational.test.lm.web.client.internal.group.callbackObject", null,
{
    constructor: function(handler)
    {
        this.responseHandler = handler;
    },
    successFunc: function(response) 
	{
        if(this.responseHandler) 
		{
            var processor = new groupApi.groupProcessor();
			return this.responseHandler._contextObject[this.responseHandler._callbackName](processor.convertGroup(response));
		}
    },
    errorFunc: function(errorType, errorMessage) 
	{
        if(this.responseHandler)
            this.responseHandler._contextObject[this.responseHandler._errorCallbackName](errorType, errorMessage);
    }
});
 
// Summary:
//    Internal object for deleting a group.
//
dojo.declare("com.ibm.rational.test.lm.web.client.internal.group.deleteCallbackObject", null,
{
    constructor: function(handler, groupClass)
    {
        this.responseHandler = handler;
		this.groupClass = groupClass;
    },
    successFunc: function(response) 
    {
        var callbackObj = new groupApi.callbackObject(this.responseHandler);
        var startDate = null;
		if(response.startDate != null){
			startDate = response.startDate.getTime();
		}             
        var expirationDate = null;
        if(response.expirationDate != null){
            expirationDate = response.expirationDate.getTime();
        }             
        var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");
        return this.groupClass.updateGroup(myHandler,
                                response.teamArea,
                                response.itemId,
                                response.expirable, 
                                response.name, 
                                response.description, 
                                expirationDate, 
                                startDate, 
                                response.stateId,
                                true);
    },
    errorFunc: function(errorType, errorMessage) 
    {
        if(this.responseHandler)
            this.responseHandler._contextObject[this.responseHandler._errorCallbackName](errorType, errorMessage);
    }
});
 
        
// Summary:
//    Internal object to normalize the group response data for arrays.
//
dojo.declare("com.ibm.rational.test.lm.web.client.internal.group.searchCombineCallbackObject", null,
{
    constructor: function(handler)
    {
        this.responseHandler = handler;
    },
    successFunc: function(response) 
    {
        groupApi._searchCounter++;
		// Concat this result to existing results
		if(groupApi._searchResults == null){
			groupApi._searchResults = response;
		}
		else {
			// concat doesn't seem to work here so we'll iterate through the data.
            var data = response.Data;
            var data_length = data.length;
			for(var i=0; i< data_length; i++){
				groupApi._searchResults.Data.push(data[i]);
			}
		}
		
        if (groupApi._searchCounter >= groupApi._numSearches) {
            if(groupApi._searchError != null){
                // There was an error in the other search
                if (this.responseHandler) 
                    this.responseHandler._contextObject[this.responseHandler._errorCallbackName](groupApi._searchError.type,
                                                                                                 groupApi._searchError.message);
            }
            else {
                if (this.responseHandler) {
                    var processor = new groupApi.groupProcessor();
                    this.responseHandler._contextObject[this.responseHandler._callbackName](groupApi._searchResults);
                }
            }
        }
    },
    errorFunc: function(errorType, errorMessage) 
    {
        groupApi._searchCounter++;
        if (groupApi._searchCounter < groupApi._numSearches) {
            groupApi._searchError = {};
            groupApi._searchError.type = errorType;
            groupApi._searchError.message = errorMessage;
        }
        else {
            if (this.responseHandler) 
                this.responseHandler._contextObject[this.responseHandler._errorCallbackName](errorType, errorMessage);
        }
    }
});
        
        
        
dojo.declare("com.ibm.rational.test.lm.web.client.internal.group", com.ibm.asq.common.web.client.internal.container,
    {

        constructor: function()
        {
            this.setServiceInterface('com.ibm.rational.test.lm.common.IAssetGroupRestService');
            console.debug("initialized serviceInterface: "+this.serviceInterface);
        },
        
        _getProjectAreaArgs : function()
		{
            var args = { "projectAreaItemId" : com.ibm.team.process.web.projectArea.itemId };
            
            return args;
        },
        
		// Note that this method will only return assets of the type represented 
		// by the configuration (ie Machine or Virual Image), not both.             
		searchForAssetsInGroup: function(responseHandler, groupId, columns, config )
		{	
		    // Construct a search configuration with the group set.
			var root = config.getRootPart(0);
			var groupPointer = _catalog.createPointer(null, groupId);
			root.setAttributeValue("group", groupPointer);

            var searchParams = {};
			searchParams.configuration = config;
			searchParams.columns = columns;
			
            return _catalog.search(responseHandler, searchParams);
		},
		
		searchForAllAssetsInGroup: function( responseHandler, groupId, columns )
		{
			// We need to do 2 searches, one for machines and one for virtual images
			// Initialise the static variables
            groupApi._numSearches = 2;
			groupApi._searchCounter = 0;
            groupApi._searchError = null;
            groupApi._searchResults = null;

            var callbackObj = new groupApi.searchCombineCallbackObject(responseHandler);
            var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");
			
            // Construct a machine search configuration with the group set.
            var config = _catalog.createConfiguration("Machine");
			this.searchForAssetsInGroup(myHandler, groupId, columns, config);

            var callbackObj2 = new groupApi.searchCombineCallbackObject(responseHandler);
            var myHandler2 = new ServiceResponseHandler(callbackObj2, "successFunc", "errorFunc");

            // Construct a machine search configuration with the group set.
            var config = _catalog.createConfiguration("VirtualImage");
            return this.searchForAssetsInGroup(myHandler2, groupId, columns, config);
		},
		
		getGroupsInTeamArea: function( responseHandler, teamArea )
		{
            var callbackObj = new groupApi.arrayCallbackObject(responseHandler);
            var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");
            var attributes = {
                "projectAreaItemId": com.ibm.team.process.web.projectArea.itemId,
                "teamArea": teamArea
            };
            return this._invokeService(myHandler, "getByAttributes", attributes);
		},
		
		getAllGroups: function( responseHandler )
		{
            var callbackObj = new groupApi.arrayCallbackObject(responseHandler);
            var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");
			return this._invokeService(myHandler, "getAll", this._getProjectAreaArgs());
		},
		
        getGroupById: function( responseHandler, groupId )
        {
            var callbackObj = new groupApi.callbackObject(responseHandler);
            var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");

            return this._invokeService(myHandler, "getById", { "id" : groupId });
        },
        
		addAssetToGroup: function( responseHandler, groupId, assetid, assetType )
		{
            var asset_id = [ assetid ];
			return this._moveMachines(responseHandler, asset_id, groupId);
		},
				
        addAssetsToGroup: function( responseHandler, groupId, assetids )
        {
            return this._moveMachines(responseHandler, assetids, groupId);
        },
                
		deleteAssetFromGroup: function( responseHandler, assetid )
		{
            var asset_id = [ assetid ];
            return this._moveMachines(responseHandler, asset_id);
		},
		
        deleteAssetsFromGroup: function( responseHandler, assetids )
        {
            return this._moveMachines(responseHandler, assetids);
        },
        
        _moveMachines: function( responseHandler, assetIds, optGroupId )
        {
			var args = { 'resourceItemIds' : assetIds };
			
			if (optGroupId) {
				args['optionalTargetGroupItemId'] = optGroupId;
			}
			
            return this._invokeService(responseHandler, "postMoveResourcesToGroup", args);
        },
        
		createGroup:function( responseHandler,
								teamArea, 
								expirable, 
								name, 
								description, 
								expirationDate, 
								startDate, 
								deleted,
                                assets )
		{
			if(expirationDate == null){
				expirationDate = 1;
			}
            if(startDate == null){
                startDate = 1;
            }
            if(!assets) {
                assets = [];
            }
            var attributes = {
				"projectAreaItemId": com.ibm.team.process.web.projectArea.itemId,
				"teamArea": teamArea,
				"expirable": expirable,
				"name": name,
				"description": description,
				"expirationDate": expirationDate,
				"startDate": startDate,
				"deleted": deleted,
                "addAssets": assets,
                "removeAssets": []
			};
            var callbackObj = new groupApi.callbackObject(responseHandler);
            var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");

            return this._invokeService(myHandler, "postCreateAssetGroup", attributes);
		},
		
		//
		updateGroup: function(	responseHandler,
								teamArea,
								groupId,
								expirable, 
								name, 
								description, 
								expirationDate, 
								startDate, 
                                stateId,
                                deleted,
                                addAssets,
                                removeAssets )
		{
            if(expirationDate == null){
                expirationDate = 1;
            }
            if(startDate == null){
                startDate = 1;
            }
            if(!addAssets) {
                addAssets = [];
            }
            if(!removeAssets) {
                removeAssets = [];
            }
            var attributes = {
				"projectAreaItemId": com.ibm.team.process.web.projectArea.itemId,
				"teamArea": teamArea,
				"uuid": groupId,
				"expirable": expirable,
				"name": name,
				"description": description,
                "expirationDate": expirationDate,
                "startDate": startDate,
				"stateId": stateId,
				"deleted": deleted,
                "addAssets": addAssets,
                "removeAssets": removeAssets
			};
            var callbackObj = new groupApi.callbackObject(responseHandler);
            var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");
				
            return this._invokeService(myHandler, "postUpdateAssetGroup", attributes);
		},
		
		deleteGroup: function(	responseHandler, groupId )
		{
            var callbackObj = new groupApi.deleteCallbackObject(responseHandler, this);
            var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");
            return this.getGroupById(myHandler, groupId);
		}
	}
);

})();
