/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.keyword.web.client.internal.keyword.KeywordClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.process.web.client.projectarea");

(function() { //module container

var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var getCurrentPA            = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

com.ibm.rqm.keyword.web.client.internal.keyword.KeywordClient= {

	KEYWORD_SERVICE_URI: "com.ibm.rqm.keyword.common.service.rest.IKeywordRestService",
	
	_invokeService: function(responseHandler, method, appArgs) {
		var actualArgs = {};
		if (appArgs) {
			var jsonArgs= dojo.toJson(appArgs);
			actualArgs.jsonString= jsonArgs;
			if(appArgs.id)	
				actualArgs.id = appArgs.id;
			if(appArgs.externalId)	
				actualArgs.externalId = appArgs.externalId;
			if(appArgs.name)	
				actualArgs.name = appArgs.name;
			if(appArgs.newKeyword)	
				actualArgs.newKeyword = appArgs.newKeyword;
			if(appArgs.manualScript)	
				actualArgs.manualScript = appArgs.manualScript;
			if(appArgs.scripts)	
				actualArgs.scripts = appArgs.scripts;
			if(appArgs.tags)	
				actualArgs.tags = appArgs.tags;
			if(appArgs.defaultScript)	
				actualArgs.defaultScript = appArgs.defaultScript;
			if(appArgs.archive)
				actualArgs.archive = appArgs.archive;
			if(appArgs.includeArchived)
				actualArgs.includeArchived = appArgs.includeArchived;
			if(appArgs.stateId)
				actualArgs.stateId = appArgs.stateId;
	        if(appArgs.scriptName)
	            actualArgs.scriptName = appArgs.scriptName;
	        if(appArgs.scriptTypeIds)
	            actualArgs.scriptTypeIds = appArgs.scriptTypeIds;
	        if(appArgs.stateIds)
	            actualArgs.stateIds = appArgs.stateIds;
	        if(appArgs.policy)
	            actualArgs.policy = appArgs.policy;
	        if(appArgs.modifiedFrom)
	            actualArgs.modifiedFrom = appArgs.modifiedFrom;
	        if(appArgs.modifiedTo)
	            actualArgs.modifiedTo = appArgs.modifiedTo;
	        if(appArgs.queryToken) {
	            actualArgs.queryToken = appArgs.queryToken; 
	        }       
	        if(appArgs.page) {
	            actualArgs.page = appArgs.page; 
	        }       
	        if(appArgs.pageSize) {
	            actualArgs.pageSize = appArgs.pageSize; 
	        }       
	        if(appArgs.sortColumns) {
	            actualArgs.sortColumns = appArgs.sortColumns; 
	        }                               
            if(appArgs.textSearch) {
                actualArgs.textSearch = appArgs.textSearch; 
            }                               
	        if(appArgs.kindOfHandles) {
	            actualArgs.kindOfHandles = appArgs.kindOfHandles; 
	        }
			if (appArgs.summaryDTO) {
				actualArgs.summaryDTO = appArgs.summaryDTO;
			}       
		}else{
			appArgs = {};
		}
		if(!appArgs.projectAreaItemId){
			actualArgs.projectAreaItemId = getCurrentPA().itemId;	
		}
        if(!appArgs.projectArea){
            actualArgs.projectArea = actualArgs.projectAreaItemId;   
        }
		var serviceRequest= new ServiceRequest(this.KEYWORD_SERVICE_URI, method, actualArgs);
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},

	getKeywords: function(serviceResponseHandler, params){
		this._invokeService(serviceResponseHandler, "getKeywords", params);
	},
	
	getKeywordDTO: function(serviceResponseHandler, params){
		this._invokeService(serviceResponseHandler, "getKeywordDTO", params);
	},
	
	postKeyword: function(serviceResponseHandler, params){
		this._invokeService(serviceResponseHandler, "postKeyword", params);
	},
	postKeywordDTO:function(serviceResponseHandler, params){
		this._invokeService(serviceResponseHandler, "postKeywordDTO", params);
	},
	postArchiveKeyword: function(serviceResponseHandler, params){
		this._invokeService(serviceResponseHandler, "postArchiveKeyword", params);
	},

    getAllSearchResultItemIds: function(serviceResponseHandler, args)
    {
        /*
                VALID PARAMETERS
                --------------------------------
                String queryToken;
                String idType;
         */         
        var typeMap = {
            'id': 'Id',
            'itemId': 'ItemHandle',
            'versionableItemId': 'VersionedItemHandle'
        };
        var type = args.kindOfHandles;
        args.kindOfHandles = typeMap[type];
        return this._invokeService(serviceResponseHandler, "getAllItemHandles", args);
    },          
    
    searchKeywords: function(serviceResponseHandler, params){
        /*
         valid parameters:
            public String projectArea;
            public String name;
            public String tags;
            public String scriptName;
            public String[] scriptTypeIds;
            public String[] stateIds;
            public Long modifiedFrom;
            public Long modifiedTo;
            public boolean includeArchived = false;
            public String policy;
            String queryToken;
            int page;
            int pageSize;
            String[] sortColumns;
            String textSearch;
        */

        this._invokeService(serviceResponseHandler, "getPagedSearchResult", params);
    },
	getKeywordTags:function(serviceResponseHandler,params){
		return this._invokeService(serviceResponseHandler, "getKeywordTags", params);
	}
    
};

})();
