/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.WorkItemUtils");

(function()
{
	var getCurrentPA            = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
	var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
	var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
	var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var WorkItemUtils           = com.ibm.rqm.planning.web.client.internal.WorkItemUtils;
	
	com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient = 
	{
		TEST_PLAN_REST_SERVICE_URI: "com.ibm.rqm.planning.common.service.rest.ITestPlanRestService",
	
		_invokeService: function(responseHandler, method, appArgs) 
		{
			if (!appArgs) {
				appArgs = {};
			}
			if (!appArgs.processArea) {
				appArgs.processArea = getCurrentPA().itemId;
			}
			
			var serviceRequest = new ServiceRequest(this.TEST_PLAN_REST_SERVICE_URI, method, appArgs);
			TeamServerClient.invokeService(serviceRequest, responseHandler);
			return serviceRequest; //return the service request - enable clients to abort request
		},

		_invokeServiceJson: function(responseHandler, method, appArgs) 
		{
			var actualArgs= {};
			if (appArgs)
			{
				var jsonArgs= dojo.toJson(appArgs);
				actualArgs.jsonString = jsonArgs;
				if(appArgs.category)
				{
					actualArgs.category = appArgs.category;
				}
			}
			
			var serviceRequest = new ServiceRequest(this.TEST_PLAN_REST_SERVICE_URI, method, actualArgs);
			TeamServerClient.invokeService(serviceRequest, responseHandler);
			return serviceRequest; //return the service request - enable clients to abort request
		},
	
		getTestPlans: function(serviceResponseHandler, args)
		{
			//TODO add the parameters and functionality
			if ( !args ) args = { };
			return this._invokeService(serviceResponseHandler, "getTestPlansDTO", args);
		},
		
		getTestPlansSummary: function(serviceResponseHandler, args)
		{
			if ( !args ) args = { };
			return this._invokeService(serviceResponseHandler, "getTestPlansSummaryDTO", args);
		},
		
        searchTestPlans: function(serviceResponseHandler, args)
	    {
			/*
			        VALID PARAMETERS
			        --------------------------------
			        String processArea;	         //itemId    
			        String name;			        
			        String[] originatorUUID;	 //itemId		        
			        String[] stateIds;		     //eg. 'com.ibm.rqm.state.new'	        
			        String[] categories;         //itemId		        
			        boolean myItemsOnly = false; 			        
			        boolean includeArchived = false;
			        Boolean potentialChildren;
			        String policy;              //'enforce', 'truncate', 'ignore'  
                    String queryToken;
                    int page;
                    int pageSize;
                    String[] sortColumns;
                    String parentVTestPlan;    //versionableItemId - exclude test plans referenced by the identified parent test plan
                    String associatedReq;      //include only test plans that are associated with specified Requirement 
                    String textSearch;
			 */			
            if (!args) {
                args = {};
            }
            return this._invokeService(serviceResponseHandler, "getPagedSearchResult", args);
        },	

        getAllSearchResultItemIds: function(serviceResponseHandler, args)
        {
            /*
                    VALID PARAMETERS
                    --------------------------------
                    String queryToken;
                    String idType;
             */         
            var typeMap = {
                'id': 'Id',
                'itemId': 'ItemHandle',
                'versionableItemId': 'VersionedItemHandle'
            };
            var type = args.kindOfHandles;
            args.kindOfHandles = typeMap[type];
            return this._invokeService(serviceResponseHandler, "getAllItemHandles", args);
        },          
		
		searchChildPlans: function(serviceResponseHandler, args) {
			args.potentialChildren = true;
			return this.searchTestPlans(serviceResponseHandler, args);
		},
		
		getTestPlansWithWorkItems: function(serviceResponseHandler, args)
		{
			//TODO add the parameters and functionality
			if ( !args ) args = { };
			args.includeWorkItems = true;
			args.includeArchived = true;
			
			var actualHandler = serviceResponseHandler;
			if (args.id || args.uid) {
				actualHandler = this.getWorkItemMappingHandler(serviceResponseHandler);
			}
			return this._invokeService(actualHandler, "getTestPlansDTO", args);
		},		
	
		postBulkChangeTestPlan: function(serviceResponseHandler, args)
		{
			var actualHandler = this.getWorkItemMappingHandler(serviceResponseHandler);			
			return this._invokeService(actualHandler, "postBulkChangeTestPlanSummaryDTO", args);
		},
		
		postTestPlan: function(serviceResponseHandler, args)
		{
			//TODO add the parameters and functionality
			var actualHandler = this.getWorkItemMappingHandler(serviceResponseHandler);			
			return this._invokeService(actualHandler, "postTestPlanDTO", args);
		},
		
		postDeleteTestPlan: function(serviceResponseHandler, args)
		{
			//args must include either TestPlan's 'id' or 'uuid'			
			return this._invokeService(serviceResponseHandler, "postDeletePlan", args);
		},
		
        postArchiveTestPlan: function(serviceResponseHandler, args)
        {
            return this._invokeService(serviceResponseHandler, "postArchiveTestPlan", args);
        },
		
        postLockTestPlan: function(serviceResponseHandler, args)
        {
			var actualHandler = this.getWorkItemMappingHandler(serviceResponseHandler);
            return this._invokeService(actualHandler, "postLockTestPlan", args);
        },		
        
		getSnapshots: function(serviceResponseHandler, args)
		{
			if ( !args ) args = { };
			return this._invokeService(serviceResponseHandler, "getSnapshots", args);
		},
		
		getWorkItemMappingHandler: function(responseHandler, method){
			var start = new Date();
			var internalHandler = {
		  		success: function(testPlanDTOs){
					//fixup each workitem for each plan
				    testPlanDTOs = WorkItemUtils.fixupArtifactsWorkItems(testPlanDTOs);					
					responseHandler.doCallback(testPlanDTOs);
				},
				failure: function(error){
					responseHandler.doErrorCallback(error);
				}
			};
			var handler = new ServiceResponseHandler(internalHandler, "success", "failure");
			return handler;
      },
	  
	  /**
	 	* @deprecated
	 	*/
	  getConfigurations: function(serviceResponseHandler, itemId){
			var args = {testsPlanUUID: itemId}
			return this._invokeService(serviceResponseHandler, "getConfigurations", args);
	  },

	    /**
	     *  Temporary UI hack to propagate snapshots since server
	     *  is currently lacking this capability.
	     *  
	     *  THIS METHOD WILL BE REMOVED.
	     *  DO NOT USE THIS METHOD OR COPY THIS CODE ELSEWHERE
	     *  
	     *  TODO:  remove this workaround when snapshot propagation
	     *  is properly implemented
	     *  
	     *  @deprecated     DO NOT USE THIS METHOD.
	 	 */
		HACK_DO_NOT_USE__postFollowupSnapshot: function(serviceResponseHandler, params) {
			var EXECUTION_WORKITEM_SERVICE_URI = "com.ibm.rqm.execution.common.service.rest.IExecutionWorkItemRestService";
			var serviceRequest = new ServiceRequest(EXECUTION_WORKITEM_SERVICE_URI, "postFollowupSnapshot", params);
			TeamServerClient.invokeService(serviceRequest, serviceResponseHandler);
		},
	  
	  /**
	 	* @deprecated
	 	*/
	  postConfigurations: function(serviceResponseHandler, testsPlanUUID, addedConfigurations, deletedConfigurations){
			var args = {testsPlanUUID: testsPlanUUID}
			if(addedConfigurations){
				args.addedConfigurations = addedConfigurations;
			}
			if(deletedConfigurations){
				args.deletedConfigurations = deletedConfigurations;
			}
			return this._invokeService(serviceResponseHandler, "postConfigurations", args);
	  },
	  
	  getRelatedTestPlanByCase : function(serviceResponseHandler, args){
			return this._invokeService(serviceResponseHandler, "getRelatedTestPlanByCaseDTO", args);
	  },

      getRelatedTestPlanByCaseSummary : function(serviceResponseHandler, args){
            return this._invokeService(serviceResponseHandler, "getRelatedTestPlanByCaseSummaryDTO", args);
      },

      getRelatedTestPlanByCaseMinimalSummary : function(serviceResponseHandler, args){
            return this._invokeService(serviceResponseHandler, "getRelatedTestPlanByCaseMinimalSummaryDTO", args);
      },

	  getTestPlanSection : function(serviceResponseHandler, args){
			return this._invokeService(serviceResponseHandler, "getTestPlanSection", args);
	  },
	  
	  getTestPlanChildren : function(serviceResponseHandler, args){
			var wrappedHandler = {
		  		success: function(dtos){
					//we know the results are children; mark them as such
					dtos = dtos || [];
					var dtoLen = dtos.length;
					for (var i = 0; i < dtoLen; i++) {
						dtos[i].isChild = true;
					}				
					serviceResponseHandler.doCallback(dtos);
				},
				failure: function(error){
					serviceResponseHandler.doErrorCallback(error);
				}
			};
			var handler = new ServiceResponseHandler(wrappedHandler, "success", "failure");	  	
			return this._invokeService(handler, "getTestPlanChildren", args);
	  },
	  
	  getTestPlanParent : function(serviceResponseHandler, args){
			return this._invokeService(serviceResponseHandler, "getTestPlanParent", args);
	  }	  
	  
	}	
})();
