/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.rqm.buildintegration.web.provider.internal.TeamConcertBuildProvider");

dojo.require("com.ibm.rqm.buildintegration.web.client.internal.RemoteTeamConcertClient");
dojo.require("com.ibm.rqm.buildintegration.web.client.internal.TeamConcertConnectionClient");
dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.config.TeamConcertBuildWizard");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.foundation.Status");
dojo.require("com.ibm.rqm.buildintegration.web.client.internal.ExternalProjectAreaDescription");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil");

(function() {

var RemoteTeamConcertClient			= com.ibm.rqm.buildintegration.web.client.internal.RemoteTeamConcertClient;
var TeamConcertConnectionClient		= com.ibm.rqm.buildintegration.web.client.internal.TeamConcertConnectionClient;
var TeamConcertBuildWizard			= com.ibm.rqm.buildintegration.web.ui.internal.config.TeamConcertBuildWizard;
var ServiceResponseHandler			= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Status							= com.ibm.team.repository.web.foundation.Status;
var ExternalProjectAreaDescription	= com.ibm.rqm.buildintegration.web.client.internal.ExternalProjectAreaDescription;
var getCurrentProjectArea			= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var BuildUiUtil 				= com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil;

dojo.declare("com.ibm.rqm.buildintegration.web.provider.internal.TeamConcertBuildProvider",null,{

	_teamServerConnection: null,
	_externalProjectArea: null,	
	_providerId: null,
	_providerName: null,
	_providerTypeId:null,

	constructor: function(id, name,typeId) {
		this._providerId= id;
		this._providerName= name;
		this._providerTypeId = typeId;
	},
	
	getProviderId: function() {
		return this._providerId;
	},
	
	getProviderName: function() {
		return this._providerName;
	},
	getProviderTypeId:function(){
		return this._providerTypeId;
	},
	isConfigured: function() {
		return true;
	},
	
	getConnectionName: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.name : null;
	},

	getConnectionDetails: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.repositoryUrl : null;
	},

	getRepositoryUrl: function() {
		return this.getConnectionDetails();
	},

	getRepositoryDescription: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.description : null;
	},

	getProjectAreaName: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.projectAreaName : null;
	},
	
	clearConnectionDetails: function(runAfterClear) {
		TeamConcertConnectionClient.postDeleteDefaultTeamConcertConnection({providerTypeId:this._providerTypeId}, dojo.hitch(this, function(status, response) {
			var resultStatus= status;
			try {
				if (status.isOK()) {
					runAfterClear();
				} else {
					console.debug(status._message);
					resultStatus= new Status(Status.ERROR, BuildUiUtil.getMessage("labelConfigurationNotCleared"));
				}
			} finally {
				runAfterClear(resultStatus);
			}
		}));
	},

	checkProviderStatus: function(resultCallback, cancelIfNotNotLoggedIn) {
		if (this._externalProjectArea == null) {
			resultCallback(new Status(Status.ERROR, BuildUiUtil.getMessage("labelNotConfigured")));
		} else if(!this.isConfigured()){
			var serviceCallback= function(status, response) {
				var resultStatus= status;
				try {
					if (status.isOK()) {
						if(!response.projectArea.initialized){
							resultStatus= new Status(Status.ERROR, BuildUiUtil.getMessage("labelProjectAreaNotInitialized"));	
						}
					}
				} finally {
					resultCallback(resultStatus);
				}
			};

			RemoteTeamConcertClient.getProjectArea(this._externalProjectArea.getProjectAreaItemId(), this._externalProjectArea, serviceCallback, { requestLabel: BuildUiUtil.getMessage("labelCheckBuildProviderConfiguration"), cancelIfNotLoggedIn: cancelIfNotNotLoggedIn });
		}else{
			resultCallback(Status.OK_STATUS);
		}
	},

	needsConfiguration: function() {
		return true;
	},

	isConfigured: function() {
		return this._externalProjectArea != null;
	},

	getConfigurationWizard: function() {
		return new TeamConcertBuildWizard(this);
	},

	initialize: function(runAfterInitialization) {
		TeamConcertConnectionClient.getDefaultTeamConcertConnection(dojo.hitch(this, function(status, results) {
			try {
				this._teamServerConnection= null;
				this._externalProjectArea= null;
				if (status.isOK()) {	
					if (results != null) {
						this._teamServerConnection= results;
						this._externalProjectArea= new ExternalProjectAreaDescription(results.name, results.description, results.repositoryUrl, results.projectAreaItemId, results.projectAreaName);
					}
				} else {
					console.group("Failed to initialize TeamConcertDefectsProvider");
					status.log();
					console.groupEnd();
				}
			} finally {
				runAfterInitialization();
			}
		}),{providerTypeId:this._providerTypeId});
	},

	setup: function(runAfterSetup, monitor) {
		monitor.beginTask("Initialize Build Provider...");
		// initialize again, to reload the just saved configuration including the Project Area UUID
		this.initialize(dojo.hitch(this, function() {
			runAfterSetup(Status.OK_STATUS);
		}));
	
	},
	__sentinel: null // terminates this class definition
});

})();
 