/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.internal.RmLinksSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.defects.web.helper.RemoteItemPicker");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.defects.web.ui.internal.LinksTableView");
dojo.require("com.ibm.rqm.defects.web.client.internal.ResourceLinkService");
dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.rqm.defects.web.client.RequirementsManager2");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.rqm.defects.web.util.ItemPickerUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function(){ // module container
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var RemoteItemPicker = com.ibm.rqm.defects.web.helper.RemoteItemPicker;
    var LinksTableView = com.ibm.rqm.defects.web.ui.internal.LinksTableView;
    var ResourceLinkService = com.ibm.rqm.defects.web.client.internal.ResourceLinkService;
    var RequirementsManager2 = com.ibm.rqm.defects.web.client.RequirementsManager2;
    var DefectsUIUtil = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
    var ProgressDialog = com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
    var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    var ItemPickerUtil = com.ibm.rqm.defects.web.util.ItemPickerUtil;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var TestCaseClient = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
    var TestPlanClient = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    
    dojo.declare("com.ibm.rqm.defects.web.ui.internal.RmLinksSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: '<div class="com-ibm-rqm-rtcworkitem-section">' +
        '   <div dojoAttachPoint="_tableContainer"></div>' +
        '</div>',
        
        constructor: function(args){
            this._items = [];
            this._newAddedItems = [];
            this._removedItems = [];
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
        },
        
        //@Overriden from AbstractArtifactEditorSection
        initialize: function(isEditable){
            this._onInitialized();
        },
        
        _onInitialized: function(){
            if (this.getArtifact()) {
                ResourceLinkService.getValidatesLinks(this.getResourceType(), this._getResourceUri(this.getArtifact()), dojo.hitch(this, "_initalizeTableViewer"));
            }
            else {
                this._initalizeTableViewer([]);
            }
        },
        
        _initalizeTableViewer: function(items){
            if (!this._linksTableViewer) {
                dojox.data.dom.removeChildren(this._tableContainer)
                this._linksTableViewer = new LinksTableView({
                    editable: this.isEditable(),
                    printable: this.isPrintView(),
                    enableRowSelection: true,
                    actions: this.getTableActions()
                });
                this._tableContainer.appendChild(this._linksTableViewer.domNode);
            }
            if (items) {
                this._items = items;
            }
            this._linksTableViewer.refresh(this._items);
        },
        
        _getResourceUri: function(artifact){
            var projectAreaId = getCurrentProjectArea().itemId;
            var resourceUri = location.protocol + '//' + location.host + net.jazz.ajax._contextRoot + "/oslc_qm/contexts/" + projectAreaId + this.getResourcePrefix() + artifact.versionableItemId;
            return resourceUri;
        },
        
        _getResourceLinkDesc: function(artifact){
            var testLinkUrl = this._getResourceUri(artifact);
            var testLinkComment = this.getResourceLabel() + " " + artifact.id + " - " + artifact.name;
            var linkDescription = {};
            linkDescription.linkComment = testLinkComment;
            linkDescription.rqmUri = testLinkUrl;
            return linkDescription;
        },
        getResourceType: function(){
            return ResourceLinkService.RESOURCE_TYPE_REQ;
        },
        
        getResourcePrefix: function(){
            return '/resources/com.ibm.rqm.planning.VersionedTestCase/';
        },
        
        getResourceLabel: function(){
            return Constants.getMessage("labelTestCase");
        },
        
        getResourceSelectDialog: function(provider){
            return provider.services.reqSelectDialog;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            if (this._newAddedItems.length == 0 && this._removedItems.length == 0) {
                console.debug("No links to be saved")
                return;
            }
            var self = this;
            ProgressDialog.doRunWithProgress(DefectsUIUtil.getMessage("msgUpdatingRmLinks"), dojo.hitch(self, function(monitor){
                monitor.beginTask(DefectsUIUtil.getMessage("msgUpdateBackLinks"));
                // Adding back links
                for (var index in self._newAddedItems) {
                    var linkDescription = this._getResourceLinkDesc(this.getArtifact());
                    ResourceLinkService.addValidatedByLink(this.getResourceType(), self._newAddedItems[index].uri, linkDescription);
                }
                // deleting back links
                for (var index in self._removedItems) {
                    var linkDescription = this._getResourceLinkDesc(this.getArtifact());
                    ResourceLinkService.removeValidatedByLink(this.getResourceType(), self._removedItems[index].uri, linkDescription);
                }
                // update local links
                monitor.beginTask(DefectsUIUtil.getMessage("msgUpdateLocalLinks"));
                ResourceLinkService.updateValidatesLinks(this.getResourceType(), this._getResourceUri(this.getArtifact()), self._newAddedItems, self._removedItems, function(){
                    self.refreshArtifact();// refresh editor w/ latest state as the state has changed post linking
                });
                self._newAddedItems = [];
                self._removedItems = [];
                monitor.done();
            }));
        },
        
        //actions to be displayed in the table header
        getTableActions: function(){
            var actions = [];
            actions.push({
                label: DefectsUIUtil.getMessage("labelRefreshRmLinks"),
                actionHandler: dojo.hitch(this, "_handleRefreshRmLinksClicked"),
                iconClass: "refresh-icon-image",
                disabledOnReadOnly: true
            });
            actions.push({
                label: DefectsUIUtil.getMessage("labelAddRmLinks"),
                actionHandler: dojo.hitch(this, "_handleAddNewRmLinksClicked"),
                iconClass: "add-icon-image",
                disabledOnReadOnly: true
            });
            actions.push({
                label: DefectsUIUtil.getMessage("labelRemoveRmLinks"),
                actionHandler: dojo.hitch(this, "_handleRemoveRmLinksClicked"),
                iconClass: "remove-reqirement-image",
                disabledOnReadOnly: true
            });
            return actions;
        },
        
        _handleRefreshRmLinksClicked: function(){
            this.initialize(true);
        },
        
        _handleAddNewRmLinksClicked: function(){
            var self = this;
            var serviceProviderManager = com.ibm.rqm.defects.web.client.RequirementsManager2.getInstance();
            
            // discover dialog uri
            serviceProviderManager.getCurrentServiceProvider(dojo.hitch(this, function(currentServiceProvider){
                if (currentServiceProvider) {
                    var reqSelectionDialog = this.getResourceSelectDialog(currentServiceProvider);
                    var selectionDialogUri = reqSelectionDialog.uri;
                    // select an item from the selection dialog
                    var params = {
                        dialogTitle: DefectsUIUtil.getMessage("labelReqSelectionPicker"),
                        secondaryTitle: DefectsUIUtil.getMessage("labelReqSelectionPickerTitle"),
                        pickerURL: selectionDialogUri,
                        height: reqSelectionDialog.height,
                        width: reqSelectionDialog.width,
                        parentElement: this.domNode,
                        onOk: function(selectedItems){
                            var reqs = ItemPickerUtil.parseRmSelectResult(selectedItems);
                            dojo.forEach(reqs, function(req){
                                if (self._containsUri(self._newAddedItems, req.uri) || self._containsUri(self._items, req.uri)) {
                                    // do nothing
                                }
                                else 
                                    if (self._containsUri(self._removeItems, req.uri)) {
                                        self._removeItemFromArray(self._removeItems, req.uri);
                                        self._items.push(req);
                                    }
                                    else {
                                        self._newAddedItems.push(req);
                                        self._items.push(req);
                                    }
                            });
                            if (self._newAddedItems && self._newAddedItems.length > 0) {
                                self._linksTableViewer.refresh(self._items);
                                self.setDirty();
                            }
                        },
                        onClose: function(){
                        }
                    };
                    new RemoteItemPicker(params);
                }
                else {
                    // NO Rm service provider is configured.
                    var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog();
                    alertDialog.setMessage(DefectsUIUtil.getMessage("msgNoneRMProvidersConfigured"));
                    alertDialog.open();
                }
            }));
        },
        
        _handleRemoveRmLinksClicked: function(){
            var checkedRows = this._linksTableViewer.getCheckedRows();
            var self = this;
            dojo.forEach(checkedRows, function(item){
                self._removeItem(item);
            })
            if (this._removedItems && this._removedItems.length > 0) {
                this.setDirty();
            }
            this._linksTableViewer.refresh(this._items);
        },
        
        refreshArtifact: function(){
            var self = this;
            var responseHandler = new ServiceResponseHandler({
                reset: function(tests){
                    self.getParentEditor().setArtifact(tests[0]);
                },
                error: function(error){
                }
            }, "reset", "error");
            if (this.getResourceType() == ResourceLinkService.RESOURCE_TYPE_REQ) {
                TestCaseClient.getTestCase(responseHandler, this.getArtifact().id);
            }
            else {
                TestPlanClient.getTestPlansWithWorkItems(responseHandler, {
                    id: this.getArtifact().id
                });
            }
        },
        
        _removeItem: function(item){
            if (this._containsUri(this._items, item.uri)) {
                this._items = this._removeItemFromArray(this._items, item.uri);
            }
            var newAdded = false;
            if (this._containsUri(this._newAddedItems, item.uri)) {
                this._newAddedItems = this._removeItemFromArray(this._newAddedItems, item.uri);
                newAdded = true;
            }
            if (!newAdded) {
                this._removedItems.push(item);
            }
        },
        
        _containsUri: function(items, uri){
            var result = false;
            dojo.forEach(items, function(item){
                if (item.uri == uri) {
                    result = true;
                }
            });
            return result;
        },
        
        _removeItemFromArray: function(array, itemUri){
            var updatedArray = [];
            for (var i = 0; i < array.length; i++) {
                if (array[i].uri != itemUri) {
                    updatedArray.push(array[i]);
                }
            }
            return updatedArray;
        }
    });
    
})();
