/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 
(function() {
dojo.provide("com.ibm.asq.common.web.ui.internal.app.Application");

dojo.require("net.jazz.ajax.ui._Application");
dojo.require("net.jazz.ajax.ui.PageContainer");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.ProgressIndicator");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.ui.internal.app.Footer");
dojo.require("com.ibm.asq.common.web.ui.internal.app.Layout");

var PageContainer           = net.jazz.ajax.ui.PageContainer;
var PlatformUI              = net.jazz.ajax.ui.PlatformUI;

var ServiceResponseHandler  = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var InitializationData      = com.ibm.team.repository.web.client.init.InitializationData;
var RepositoryClient        = com.ibm.team.repository.web.client.internal.RepositoryClient;

var Footer                  = com.ibm.asq.common.web.ui.internal.app.Footer;
var Layout                  = com.ibm.asq.common.web.ui.internal.app.Layout;

function workbenchRootNode() {
    return net.jazz.ajax.ui.PlatformUI.getWorkbench().rootNode();
}

dojo.declare("com.ibm.asq.common.web.ui.internal.app.Application", net.jazz.ajax.ui._Application, 
{
    
    title: "",
    titleImagePath: null,      // Title image of the application if it is the only specific graphics
                               // that need to be replaced.  Can be customized through CSS as well.

    // supplying the following attributes will setup the Info Center link and CS help.
    //
    CSHelpBaseURL: null,       // base URL for context sensitive help
    InfoCenterHelpLabel: null, // label to display for Info Center at the far right of the tab bar
    InfoCenterLinkLabel: null, // label to diaplay for the Info Center link inthe CS help panel
    InfoCenterURL: null,       // the URL to the Info Center
    
    // Method to invoke during initialization.  Subclass should implement this if there
    // is any initialization code.
    //
    initApp: function() {
    },
    
    // Subclass MUST call this method last to start the application.
    //
    start: function() {
        com.ibm.asq.common.web.ui.internal.app.Application.superclass.start.call(this);
    },
    
    // The UI should be initialized after all data is initialized.  This superclass method
    // MUST be called even no additional UI initialization is required.
    initUI: function() {
        // the UI should only be initialized once
        if (!this._initUIcompleted) {
            // for min-width
            dojo.addClass(dojo.body(), "com-ibm-rational-asq");
            dojo.addClass(workbenchRootNode(), "com-ibm-team-repository-web-RootNode");
        
            // create the main body elements
            this._layout = new Layout();
            workbenchRootNode().appendChild(this._layout.domNode);
        
            // page content
            var pageArea = new PageContainer();
            // note: progress indicator styles defined in resources/ui/styles.css
            var progressIndicator = new net.jazz.ajax.ui.ProgressIndicator();
            pageArea.domNode.appendChild(progressIndicator.domNode);
            progressIndicator.domNode.className = "com-ibm-team-repository-web-ProgressIndicator";

            this._layout.contentContainer.appendChild(pageArea.domNode);

            // append footer directly to body to make it stick to the bottom
            this._footer = new Footer();
            dojo.body().appendChild(this._footer.domNode);      

            if (this.title != "") {
                this.setName(this.title);
            }
            // use app title image if it is specified
            if (this.titleImagePath) {
                this._layout.titleImageNode.style.backgroundImage = "url(" + this.titleImagePath + ")";
            }
            // setup info center and CS help if required attributes are specified
            if (this.InfoCenterHelpLabel && this.InfoCenterURL) {
                this._layout.initHelpCenter(this.InfoCenterHelpLabel, this.InfoCenterURL);
            }
            if (this.CSHelpBaseURL && this.InfoCenterLinkLabel && this.InfoCenterURL) {
                CSHelpEngine.init(this.CSHelpBaseURL, this.InfoCenterLinkLabel, this.InfoCenterURL);
            }
        
            if (dojo.isIE) {
                // CSS for the gutter height does not work on IE so programmatically set the
                // gutter height.  Note that IE trigger window resize on document resize which
                // is not an expected behavior but it works for our benefit here.
                dojo.connect(window, "onresize", this, "_setGutterHeight");
                this._setGutterHeight();
                
            this._initUIcompleted = true;
            }
        }
    },
    
    init: function() {
       // get the Jazz env to initialize the InitializationData
       var responseHandler = new ServiceResponseHandler(this, "_initSuccess", "_initFailure");
       RepositoryClient.getInitializationData(responseHandler);
    },
    
    _initSuccess: function(response) {
        console.debug("RepositoryClient get initialization data successfully!");
        InitializationData._load(response);
        
        // set the global var for the logged-in contributor
        dojo.setObject(
            "com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR",
            InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer").currentContributor);

		// set the global far for the current project area based on the last
		// segment of the document's URI and the set of project areas in the
		// repository
		var pathname = window.location.pathname;
		var projectName = pathname.substring(pathname.lastIndexOf('/')+1);
		var projectAreas = InitializationData.get("com.ibm.team.process.service.projectAreaWebUIInitializionData").projectAreas;
		var projectArea = dojo.setObject("com.ibm.team.process.web.projectArea", {});
		for (var i = 0; i < projectAreas.length; i++) {
			if (projectAreas[i].name == unescape(projectName)) {
				projectArea.itemId = projectAreas[i].itemId;
				projectArea.name = projectAreas[i].name;
				break;
			}	
		}

        // invoke subclass specific init
        this.initApp();
    },
    
    _initFailure: function(errorObject)
    {
        console.debug("initialization failed!");
    },
    
    _layout: null,
    _footer: null,
    _docHeight: 0,
    _initUIcompleted: false,

    _setGutterHeight: function() {
        // get the body height
        var docHeight = document.body.clientHeight;
        
        // resize can be change in width, we only care about height
        if (docHeight != this._docHeight) {
            if (docHeight < this._docHeight) {
                // set the height to zero to allow the content resize without the gutters
                this._layout._setGutterHeight(0);
            }
            this._docHeight = docHeight;
        
            // the gutter height is the root node height without the footer height
            var rootNodeHeight = workbenchRootNode().clientHeight;
            var footerHeight = this._footer.domNode.clientHeight;
            var gutterHeight = rootNodeHeight - footerHeight;
            // set the gutter height to fill the vertical height
            this._layout._setGutterHeight(gutterHeight);
        }
    }

});

})();
