/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    
dojo.provide("com.ibm.asq.common.web.ui.internal.catalog.CatalogExplorer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTree");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.ActionRegistry");
dojo.require("com.ibm.asq.common.web.ui.internal.catalog.CatalogModel");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function() {

var HorizontalTree			= com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTree;
var ServiceResponseHandler 	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

dojo.declare("com.ibm.asq.common.web.ui.internal.catalog.CatalogExplorer",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-catalog-explorer">' +
                    '  <button dojoType="dijit.form.CheckBox" dojoAttachPoint="showHiddenToggle" dojoAttachEvent="onClick:_handleShowHiddenToggled">' +
                    '  </button>' +
                    '  <label tabIndex="-1" dojoAttachPoint="showHiddenLabel">${labelShowHidden}</label>' +
                    '  <div dojoAttachPoint="explorerArea">' +
                    '  </div>' +
                    '</div>',
                    
    widgetsInTemplate: true,
    height: "400px",
    width: "100%",
    numVisiblePanels: 4,
    actionRegistry: null,
    canShowHidden: false,
    partIsAlwaysContainer: false,
	showWhitchTag: false,
    labelShowHidden: MessageUtil.getMessage(MessageBundle.COMMON, "labelShowHiddenValues"),
    allowAutoResize: true,
    filterNonWhitchTagParts: false,
                                
    postCreate: function()
    {
        this.inherited('postCreate', arguments);
        this.showHiddenLabel.setAttribute("for", this.showHiddenToggle.id);
        
        this.model = new com.ibm.asq.common.web.ui.internal.catalog.CatalogModel({
            partIsAlwaysContainer: this.partIsAlwaysContainer,
			showWhitchTag: this.showWhitchTag,
            showHidden: false,
            filterNonWhitchTagParts: this.filterNonWhitchTagParts
        });  
        this._hasInitCatalog = false;
        this._initUI();
    },
    
    _initUI: function()
    {
        this.htree = new HorizontalTree({
            allowAutoResize: this.allowAutoResize,
            model: this.model,
            getIconClass: dojo.hitch(this, this.getIconClass),
            getLabelClass: dojo.hitch(this, this.getLabelClass),
            nVisiblePanelsHint: this.numVisiblePanels,
            actionRegistry: this.actionRegistry,
            width: this.width,
            height: this.height
        });
        this.explorerArea.appendChild(this.htree.domNode);
        this.htree.domNode.setAttribute("tabIndex", -1);
        
        if(!this.canShowHidden) {
            dojo.style(this.showHiddenToggle.domNode, "display", "none");
            dojo.style(this.showHiddenLabel, "display", "none");
        } 
    },

    _handleShowHiddenToggled: function(event)
    {
        var state = this.showHiddenToggle.attr('value');
        this.model.showHidden = (state == "on") || false;
        this.refresh();
        if(event){
            dojo.stopEvent(event);
        }
    },
    
    installActionRegistry: function(actionRegistry) 
    {
        this.actionRegistry = actionRegistry;
        if(this.htree)
            this.htree.actionRegistry = actionRegistry;
    },
	
    hasValidSize: function()
    {
        return this.htree.hasValidSize();    
    },
    
    hasLoadedModel: function()
    {
        return this._hasInitCatalog;
    },
    
    loadModelContents: function()
    {
		if(this._hasInitCatalog) return;

		var handlerObj = {
			_self: this,
			success: function(){
				this._self._hasInitCatalog = true;
				this._self.htree.load();
			},
			failure: function(e){
			}
		};
		var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
		this.model.initCatalogSandbox(responseHandler);
    },
    
    reloadModel: function()
    {
        this._hasInitCatalog = false;
        this.loadModelContents();
    },
    
    refresh: function()
    {
        if(this.htree)
            this.htree.load(); 
    },
    
    onChange: function()
    {
        
    },
    
    getIconClass: function(item, opened)
    {
        
    },

    getLabelClass: function(item)
    {
        
    },
    
    dummy:null

});

})();
