/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.declare("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant", null, {
	
	// for type checking.  don't override of modify
	isASaveParticipant: true,
	
    _participantId: null,
	_itemId: null,
	_stateId: null,
	_versionableItemId: null,
	_itemType: "",

	constructor: function(args)
	{
		this._processing = false;
		this._dirty = true;
		this._transactionId = null;
	},

	// @override
	// return true if the item's data is valid, false otherwise
	validate: function()
	{
		return true;
	},
	
	// @override
	// return JSON containing the attributes of the item, for example:
	// { name: "My Test Plan",  description: "Test plan for x" }
	getAttributes: function()
	{
		return new Object();
	},
	
	// @override
	// return the array of links for the item.
	// each element of the array has the form:
	// { TargetItemId: <string>,  LinkType: <string> }
	getLinks: function()
	{
		return null;
	},
	
	// @override
	// return the type of the item, having the form:
	//  "com.ibm.rqm.test.TestPlan"
	getItemType: function()
	{
		return this._itemType;
	},
	
	// @override
	// return the ID of the item or null if the item is new.
	getItemId: function()
	{
		return this._itemId;
	},
	
    // @override
    // return the ID of the state of the item or null if the item is new.
    getStateId: function()
    {
        return this._stateId;
    },	
    
    // @override
    // return the participant ID of the item or null if the item is new.
    getParticipantId: function()
    {
        return this._participantId;
    },    

	// @override
	// return the versionable ID of the item or null if the item is new.
	getVersionableItemId: function() {
			return this._versionableItemId;
	},
   		
	// @override if necessary
	// set the participant to dirty if its attributes have changed
	setDirty: function(dirty)
	{
		this._dirty = dirty;
	},
	
	// @override if necessary
	// sets the item id for the participant
	setItemId: function(itemId)
	{
		this._itemId = itemId;
	},

    // @override if necessary
    // sets the state id for the participant
    setStateId: function(stateId)
    {
        this._stateId = stateId;
    },
	    
    // @override
    // sets the unique ID for this participant 
    setParticipantId: function(id)
    {
        this._participantId = id;
    },    
	
	// @override if necessary
	// sets the item type for the participant
	setItemType: function(itemType)
	{
		this._itemType = itemType;
	},
	
	// @override if necessary
	// sets the versionable item id for the participant
	setVersionableItemId: function(versionableItemId)
	{
		this._versionableItemId = versionableItemId;
	},
	
	// @override if necessary
	// check to see if the participant's attributes have changed
	isDirty: function()
	{
		return this._dirty;
	},
	
	// @override
	// process the item returned by the server after it has
	// processed the save request
	saveFinished: function(savedItem)
	{
	},
	
	// @override
	// process the status returned by the server after it has
	// encountered a save failure
	saveFailed: function(status)
	{
		//default behavior, implementors encouraged to override
		if (status != null && status.message) {
            console.error(this.declaredClass + ": Save Failed; Error: " + status.message);
		}
		
	},
	
	// boolean indicating whether this participant is being
	// processed. this method should not be overridden.
	_isProcessing: function()
	{
		return this._processing;
	},
	
	// boolean indicating whether this participant is being
	// processed. this method should not be overridden.
	_setProcessing: function(processing)
	{
		this._processing = processing;
	},

	// a transaction Id for this participant while saving.
	// this method should not be overridden.
	_setTransactionId: function(id)
	{
		this._transactionId = id;
	},
	
	// a transaction Id for this participant while saving.
	// this method should not be overridden.
	_getTransactionId: function()
	{
		return this._transactionId;
	}

});
