/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation  2009, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

 //
 // Summary:
 //    Central repository for delete action
 //


dojo.provide("com.ibm.rqm.planning.web.ui.internal.model.action.DeleteAction");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.action.AbstractAction");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.action.ActionEntityType");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressStatusMonitor");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressStatusMonitor");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");

dojo.require("dijit.Menu");			  

// This is only used by the Test Plan Tree View to delete Test Plans
// TODO use for all other delete cases

(function()
{
	var ActionEntityType = com.ibm.rqm.planning.web.ui.internal.model.action.ActionEntityType;		
	var AbstractAction = com.ibm.rqm.planning.web.ui.internal.model.action.AbstractAction;
	var PlanningClient = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
	var TestCaseClient = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
	var OptionsDialog = com.ibm.asq.common.web.ui.internal.widgets.OptionsDialog;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var ProgressUtility = com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressUtility;
	var GlobalEvents   = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
	var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
		
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.model.action.DeleteAction", AbstractAction, 
	{
		//Attributes the caller can set
		//artifactName			String identifying the artifact to be deleted
		//							   This will be used ONLY if confirmationMessage is null and the number of items to be deleted is 1
		//							   It will be used in the confirmation message.
		//artifactType			String identifying the type of artifact to be deleted
		//							   must be one of:  "testplan", "testcase"
		//confirmationMessage 	String containing the message to display confirming the delete
		//confirmationTitle		String used as the title of the confirmation dialog
		
		artifactType: null,
		artifactName: null,
		confirmationMessage: null,
		confirmationTitle: null,
		
        constructor: function(args){},
				
		confirm: function(ids)
		{
			//Set up the prompt based on either message supplied by the user or one we construct
			//based on the number of items to be deleted and whether a name was supplied by the user
		    var prompt = "";
			if (this.confirmationMessage)
				prompt = this.confirmationMessage;
			else if (ids.length == 1 && this.artifactName != null)
				prompt = Constants.getMessage("msgRemoveItem", this.getTypeLabel(this.artifactType), this.artifactName);
			else
       			prompt = Constants.getMessage("msgRemoveItem2", ids.length, this.getTypeLabel(this.artifactType));
				
        	var args = {
	            'message': prompt,
            	'okCallback': 
	                function(view) {
						view._onOK();   //Call our onOK method when the user clicks OK
                	},
            	'okCallbackData': this,
            	'cancelCallback': null,
            	'cancelCallbackData': null
        	};
			
			//Show the confirmation dialog
        	this.confirmDialog = new OptionsDialog(args);
        	this.confirmDialog.setTitle(Constants.getMessage("labelConfirmation"));
        	this.confirmDialog.open();							
		},
		
		//Callback the client when done successfully
		_deleteSuccess: function()
		{
			ProgressUtility.endTaskMonitor(this.monitorId);
			dojo.publish(GlobalEvents.ArtifactsDeleted, [this.getTypeForDojoPublish(this.artifactType), this.buildEventIDArray(this.numericIDArray)]);			
			if (this.completeCallback)
				this.completeCallback()
		},
		
		//Callback the client when there was a failure
		_deleteFailure: function(msg)
		{
			ProgressUtility.endTaskMonitor(this.monitorId);
			if (this.errorCallback)
				this.errorCallback(msg);	
		},
		
		_onOK: function()
		{
			//Call the delete method for the artifact type being deleted
			//The backend calls are structured differently for different artifact type.
			//For instance, the Test Plan delete works on a single artifact at a time
			//while the Test Case delete works on multiple artifacts at a time.
			if (this.artifactType == "testplan")
				this._doTestPlanDelete();
		},
		
		//Execute the delete operation
		execute: function(idArrays, completeCallback, errorCallback)
		{
			//Save the callback methods and the idArray
			this.completeCallback = completeCallback;
			this.errorCallback = errorCallback;
			this.idArray = idArrays.versionableIDs;
			this.numericIDArray = idArrays.numericIDs;
			
			//Confirm before deleting
			this.confirm(this.idArray);
		},
		
		//Get the artifact type label based on the artifact type
		getTypeLabel: function(type)
		{
			if (type == "testplan")
				return Constants.getMessage("labelTestPlan");
			if (type == "testcase")
				return Constants.getMessage("labelTestCase");
			if (type == "testscript")
				return Constants.getMessage("labelTestScript");
			
			return("");
		},
		
		//Get the caption to use in the progress dialog based on the artifact type
		getProgressCaption: function(type)
		{
			if (type == "testplan")
				return Constants.getMessage("msgDeletingTestPlans");
			if (type == "testcase")
				return Constants.getMessage("msgDeletingTestCases");
			if (type == "testscript")
				return Constants.getMessage("msgDeletingTestScripts");
			
			return("");
			
		},
		
		getTypeForDojoPublish: function(type)
		{
			if (type == "testplan")
				return Constants.TYPE_PLAN;
			if (type == "testcase")
				return Constants.TYPE_CASE;
			if (type == "testscript")
				return Constants.TYPE_SCRIPT;
			
			return("");
			
		},

		_doTestPlanDelete: function()
		{			
			this.selectedPlanCount = this.idArray.length;
			this.monitorId = ProgressUtility.beginTaskMonitor(this.getProgressCaption(this.artifactType), this.selectedPlanCount, null, this);

			if (this.selectedPlanCount > 0) 
			{
				this.errorPlanCount = 0;
				this.deletedPlanCount = 0;
				for (var i = 0; i < this.selectedPlanCount; i++) 
				{
					var responseHandler = new ServiceResponseHandler(this, "_deletePlanSuccess", "_deletePlanFailure");
					PlanningClient.postArchiveTestPlan(responseHandler,
						{
							uuid_versionable: this.idArray[i],
							archive: true
						});
				}
			}
		},
    
    _deletePlanSuccess: function(response) 
	{
		this.deletedPlanCount++;
		this._setDeleteStatus();
	},
    
	_deletePlanFailure: function(err) 
	{
		this.errorPlanCount++;
		this._setDeleteStatus();
	},
    
    _setDeleteStatus: function() 
	{
		ProgressUtility.updateTask(this.monitorId, this.deletedPlanCount + this.errorPlanCount)

        if ((this.deletedPlanCount + this.errorPlanCount) == this.selectedPlanCount) 
		{
			//We're all done
			this._deleteSuccess();   //What to do if there were any failures.
        }
    },
	
	//Construct an array of objects that are OK to send in the deleted artifacts event
	buildEventIDArray: function(numericIDArray)
	{
		//The delete event wants {id:<id>}
		var retVal = [];
		var length = numericIDArray.length;
		for (var i = 0; i < length; i++)
		{
			retVal.push({id:numericIDArray[i]})
		}
		return retVal;
	}		
	});
})();
