/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.module.AbstractPickerWidget");//$NON-NLS-1$
// declare dependencies on other Dojo modules
dojo.require("jazz.ui.RemoteChooserResponder");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");


(function(){
    var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    var PROCESS_SERVICE_URI = "com.ibm.team.process.internal.common.service.IProcessRestService";//$NON-NLS-1$
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    
    // declare and implement the widget "class"
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.module.AbstractPickerWidget", [dijit._Widget, dijit._Templated], {
        // Abstract functions
        // 1. sub class should override this to fill the content area
        getContentArea: function(){
            return null;
        },
        // 2. sub class should call this to send the result to the client
        send: function(resultArray, action){
            this._sendResponse(resultArray, action);
        },
        
        // sub class should override this to construct the resource URI
        getItemUri: function(itemId){
            return "";
        },
        
        getDialogArgs: function(){
            return this._dialogArguments;
        },
		
		getItemType: function() {
			return "";
		},
        
        // -- Implementation --		
        currentProjectArea: null,
        
        templateString: '<div dojoAttachPoint="_itemPickerNode"><div>' + Constants.getMessage("labelLoading") + '</div></div>',
        
        constructor: function(params){
            var searchString = location.search.replace(/^\?/, ''); // remove leading '?'
            this._dialogArguments = dojo.queryToObject(searchString);
        },
        
        postCreate: function(){
            // override the styles from the UI module (minWith big, blue background)
            var body = dojo.body();
            dojo.style(body, "minWidth", "200px");
            dojo.style(body, "background", "none");
            dojo.style(body, "overflow", "hidden");
            
            this.createDialog();
        },
        
        createDialog: function(){
            var srh = new ServiceResponseHandler(this, "_findProjectArea", "_noProjectArea");
            var serviceRequest = new ServiceRequest(PROCESS_SERVICE_URI, "getAllProjectAreas", {
                hideArchivedProjects: true
            });
            TeamServerClient.invokeService(serviceRequest, srh);
        },
        
        _findProjectArea: function(projectAreas){
            var name = this._dialogArguments.projectAreaName;
            
            for (var i = 0; i < projectAreas.length; i++) {
                if (name == projectAreas[i].name) {
                    this.currentProjectArea = projectAreas[i];
                    com.ibm.team.process.web.projectArea = this.currentProjectArea;
                }
            }
            if (this.currentProjectArea) {
                this._openDialog();
            }
            else {
                this._noProjectArea();
            }
        },
        
        _noProjectArea: function(){
            alert("No project area '" + this._dialogArguments.projectAreaName + "' found.");
            this._sendResponse(null);
        },
        
        _openDialog: function(){
            this._setContentWidget(this.getContentArea());
        },
        
        _setContentWidget: function(content){
            if (content) {
                this._itemPickerNode.removeChild(this._itemPickerNode.childNodes[0]);
                this._itemPickerNode.appendChild(content.domNode);
            }
        },
        
        _sendResponse: function(resultArray, action){
            var newItems = [];
            
            if (resultArray) { // expected [ { id, name } ]
                for (var i = 0; i < resultArray.length; i++) {
                    var item = resultArray[i];
                    
                    var name = item.id + ": " + item.name;
                    var itemUrl;
                    if (item.versionableItemId) {
	                    itemUrl = this.getItemUri(item.versionableItemId);
                    } else {
    	                itemUrl = this.getItemUri(item.itemId);
                    }
                    var newItem = {
                        "rdf:resource": itemUrl,
                        "oslc_qm:label": name
                    };
                    
                    newItems.push(newItem);
                }
            }
            
            var response = {
                "oslc_qm:message": action,
                "oslc_qm:results": newItems
            };
            console.debug(dojo.toJson(response));
            var remoteFrame = new jazz.ui.RemoteChooserResponder();
            remoteFrame.sendResponse(dojo.toJson(response));
        }
    });
    
    com.ibm.rqm.planning.web.ui.internal.module.AbstractPickerWidget.RETURN_MSG_SELECT = "oslc_qm.select";
    com.ibm.rqm.planning.web.ui.internal.module.AbstractPickerWidget.RETURN_MSG_CREATE = "oslc_qm.create";
})();
