/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.pages.GroupUtil");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.client.internal.admin");
dojo.require("com.ibm.rational.test.lm.web.client.internal.group");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.pages._GroupUtil", null,
{
    _adminService: new com.ibm.asq.common.web.client.internal.admin(),
    _groupService: new com.ibm.rational.test.lm.web.client.internal.group(),
    
    _allTeamArea: [],  // all team area is used in a dialog prompt by the lab manager to set up
                       // a default assets group in the case that there isn't one already set up
                       
    _myTeamArea: [],   // for the lab manager, it is all team area
    
    _allGroups: [],
    _myGroups: [],     // for the lab manager, this is the same as all groups
    
    _allGroupNames: [],
    _myGroupNames: [],     // for the lab manager, this is the same as all groups
    
    _initTaskCount: 3, // 3 async tasks for init - all team area, all groups, and role based my team area with my groups
    
    _invalidateGroupsList: {},
    _groupsListValid: true,
    
    callerResponseHandler: null,
    responseCount: 0,
    
    // Refresh the team area and groups - run all async init task including refresh all team area.
    //
    refresh: function(callerResponseHandler) {
        GroupUtil._initTaskCount = 3;
        GroupUtil._allTeamArea = [];
        GroupUtil.init(callerResponseHandler);
    },

    // Initialize the team area and groups 
    //
    init: function(callerResponseHandler) {
        
        var myResponseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper({
            _successHandler: function (successResponse) {
                // one more async task done
                GroupUtil._initTaskCount--;
                if (!GroupUtil.initFailure && (GroupUtil._initTaskCount == 0)) {
                    // invoke caller's response handler when all async tasks are done and no failure
                    callerResponseHandler._contextObject[callerResponseHandler._callbackName](successResponse);
                }
            },
            _failureHandler: function (failureResponse) {
                GroupUtil.initFailure = true;
                callerResponseHandler._contextObject[callerResponseHandler._errorCallbackName](failureResponse);
            }
        },
        "_successHandler",
        "_failureHandler"
        );

        if (GroupUtil._hasCreatePermission()) {
            // for lab manager, my groups and my team area are the same as all groups and all team area
            GroupUtil._initTaskCount--;
        } else {
            // For user, we need to get groups for each team area of the user.  The
            // response handler of getting the user's team area will post the request
            // to get the groups for each team area.
            console.debug("init for user");
            GroupUtil._initUserTeamArea(myResponseHandler);
        }
        
        // need to get all team area even for non-lab manager in order to map a team area
        // id to a name for groups that do not belong to the users but can be displayed
        // through the asset detail page
        //
        if (GroupUtil._allTeamArea.length == 0) {
            // there must be at least one lab manager team area for default group so
            // initialize it if zero length
            GroupUtil._initAllTeamArea(myResponseHandler);
        }
        
        // non-lab manager users have the option to see all groups as well
        GroupUtil._initAllGroups(myResponseHandler);
    },

    // Initialize all the team area within the current project area.
    //
    _initAllTeamArea: function(callerResponseHandler) {
        var myResponseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper( {
            _successHandler: function (successResponse) {
                // populate the _allTeamArea array
                GroupUtil._allTeamArea = [];
                for (var i = 0; i < successResponse.length; i++) {
                    var aTeamArea = { 'name': successResponse[i].name, 'uuid': successResponse[i].itemId };
                    GroupUtil._allTeamArea.push(aTeamArea);
                }
                if (GroupUtil._hasCreatePermission()) {
                    // for lab manager my team area are the same as all team area
                    GroupUtil._myTeamArea = GroupUtil._allTeamArea;
                }
                callerResponseHandler._contextObject[callerResponseHandler._callbackName](successResponse);
            },
            _failureHandler: function (failureResponse) {
                callerResponseHandler._contextObject[callerResponseHandler._errorCallbackName](failureResponse);
            }
        },
        "_successHandler",
        "_failureHandler"
        );
        GroupUtil._adminService.getTeamAreasFromProject(myResponseHandler, com.ibm.team.process.web.projectArea.itemId);
    },

    // Initialize all the groups.
    //
    _initAllGroups: function(callerResponseHandler) {
        var myResponseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper( {
            _successHandler: function (successResponse) {
                GroupUtil._allGroups = successResponse;
				// Initialise the names
				GroupUtil._allGroupNames = [];
				for(var i=0; i<successResponse.length; i++){
					GroupUtil._allGroupNames[i] = successResponse[i].name;
				}
				
                if (GroupUtil._hasCreatePermission()) {
                    // for lab manager my groups are the same as all groups
                    GroupUtil._myGroups = GroupUtil._allGroups;
					GroupUtil._myGroupNames = GroupUtil._allGroupNames;
                }
                callerResponseHandler._contextObject[callerResponseHandler._callbackName](successResponse);
            },
            _failureHandler: function (failureResponse) {
                console.debug("failure - " + failureResponse);
                callerResponseHandler._contextObject[callerResponseHandler._errorCallbackName](failureResponse);
            }
        },
        "_successHandler",
        "_failureHandler"
        );
        GroupUtil._groupService.getAllGroups(myResponseHandler);
    },
    
    _initUserTeamArea: function(callerResponseHandler) {
        // save the response handler
        GroupUtil.callerResponseHandler = callerResponseHandler;
        
        var myResponseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper( {
            _successHandler: function (response) {
                GroupUtil._myTeamArea = [];
                GroupUtil._myGroups = [];
                if (response) {
                    // keep track of the number of expected responses so we can do the next step when it is done.
                    // note that it would be good to have better structure for all these cascading messages!
                    GroupUtil.responseCount = response.length;
                    for (var i = 0; i < response.length; i++) {
                        var aTeamArea = { 'name': response[i].name, 'uuid': response[i].itemId };
                        GroupUtil._myTeamArea.push(aTeamArea);
                        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(GroupUtil, "_initMyGroupsSuccess", "_initGroupsFailure");
                        GroupUtil._groupService.getGroupsInTeamArea(responseHandler, response[i].itemId);
                    }
                }
                if (response && !response.length){
                    // we are not going to call _getMyGroupsSuccess at all, so respond here.
                    if (GroupUtil.callerResponseHandler){
                        GroupUtil.callerResponseHandler._contextObject[GroupUtil.callerResponseHandler._callbackName]();
                    }
                }
            },
            _failureHandler: function (failureResponse) {
                callerResponseHandler._contextObject[callerResponseHandler._errorCallbackName](failureResponse);
            }
        },
        "_successHandler",
        "_failureHandler"
        );

        GroupUtil._adminService.getTeamAreasForUser(myResponseHandler,
                        com.ibm.team.process.web.projectArea.itemId,
                        com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR.userId);
    },

    _initMyGroupsSuccess: function(response) {
        if (response) {
			GroupUtil._myGroups = response;
            // Initialise the names
            GroupUtil._myGroupNames = [];
            for(var i=0; i<response.length; i++){
                GroupUtil._myGroupNames[i] = response[i].name;
            }
        }
		if (--GroupUtil.responseCount == 0){
			// respond to our caller after the last message response
			if (GroupUtil.callerResponseHandler){
				GroupUtil.callerResponseHandler._contextObject[GroupUtil.callerResponseHandler._callbackName]();
			}
		}
    },
    
    _initGroupsFailure: function(response) {
        if (GroupUtil.callerResponseHandler){
            GroupUtil.callerResponseHandler._contextObject[GroupUtil.callerResponseHandler._callbackName]();
        }
    },

    getGroups: function() {
        return this._myGroups;
    },
    
    getAllGroups: function() {
        return this._allGroups;
    },
    
    getGroupNames: function() {
        return this._myGroupNames;
    },
    
    getAllGroupNames: function() {
        return this._allGroupNames;
    },
    
    getTeamArea: function() {
        return this._myTeamArea;
    },
    
    getAllTeamArea: function() {
        return this._allTeamArea;
    },
    
    // Return the name of a given team area with specified uuid.
    //
    getTeamAreaName: function(teamAreaId) {
        for (var i = 0; i < this._allTeamArea.length; i++) {
            if (this._allTeamArea[i].uuid == teamAreaId)
                return this._allTeamArea[i].name;
        }
        return "Unknown - id: " + teamAreaId;
    },
    
    // Methods to manage if a group is valid or not. The "Group Details"
    // page can refresh the group when needed before bringing up the page.
    //
    addGroupToInvalidate: function(groupId) {
        this._invalidateGroupsList[groupId] = true;
    },
    
    removeGroupFromInvalidate: function(groupId) {
        if (this._invalidateGroupsList[groupId]) {
            delete this._invalidateGroupsList[groupId];
        }
    },
    
    isGroupValid: function(groupId) {
        return !this._invalidateGroupsList[groupId];
    },
    
    // Methods to manage if the list of groups is valid or not. The "Manage Groups"
    // page can refresh the list when needed before bringing up the page.
    //
    isGroupsListValid: function() {
        return this._groupsListValid;
    },
    
    setGroupsListValid: function(valid) {
        this._groupsListValid = valid;
    },
	
    _hasCreatePermission: function()
    {
		var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
        var perm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_RESOURCE_GROUP);  
        return (!perm || perm[PermissionsClient.ACTION_CREATE]);           
    }

});

// Create a singleton Util.
GroupUtil = new com.ibm.rational.test.lm.web.ui.internal.pages._GroupUtil();
