/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.test.TestView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var TableContentFilterProvider = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;
var minutes = 1000*60;
var hours = minutes*60;
var days = hours*24;
var years = days*365;

dojo.declare("TestView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    parentView: null,
    
    constructor: function(testView)
    {
        this.parentView = testView;
    },
    
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        alert("Row check state changed: " + rowObject.col1);        
    },
    
    // Summary:
    //    A group check box is shown when the tables rows are checkable and the in multi-select mode.
    //    This method is called when the group's check box is checked/unchecked. The affected rows and 
    //    their states are passed.
    //
    groupRowsCheckStateChanged: function(tableViewer, rowObjects, state) 
    {
        alert("Group Rows check state changed, number of row state changed: " + rowObjects.length);                
    },
    
    // Summary:
    //    If the client registered content selectors with the table viewer, this message will be fired
    //    when the user changed the content selection.
    //
    tableContentSelectionChanged: function(tableViewer, contentSelectionValue)
    {
        var howMany = parseInt(contentSelectionValue);
        var content = this.parentView.createContent(howMany);
        tableViewer.setContent(content);
        tableViewer.show();
    },
    
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {
        alert("Table action invoked " + actionId);   
        
        if(callback) {
            callback(callbackData);
        }                     
    },
    
    // Summary:
    //    If the client have specified a header image and specified that it's an action (see TableContentProvider 
    //    for details), when the column's image is clicked, this message is invoked.
    //
    tableHeaderColumnActionInvoked: function(tableViewer, columnIndex, /*optional*/actionId)
    {
        alert("Table Column action invoked, actionId: " + actionId + ", At column: " + columnIndex);           
    },
    
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentFilterActionInvoked: function(tableViewer, actionId, filters, callback, callbackData)
    {
        alert("Filter action invoked " + actionId);   
        
        if(callback) {
            callback(actionId, filters);
        }                     
    }    
});

dojo.declare("TestView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    tableViewer: null,
    
    constructor: function(viewer) 
    {
        this.tableViewer = viewer;    
    },
    
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
        alert("cell value changed on object: " + rowObject.col1 + ", new Value: " + newValue);
        
        // store the new values in our internal data model
        if(columnIndex == 7) {
            if(dojo.isString(newValue)) {
                rowObject.selection = newValue;
            }
        } else if (columnIndex == 2) {
            rowObject.col3 = newValue;
        }
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
        var del = confirm("delete the item?" + rowObject.col1);
        if(del) {
            this.tableViewer.deleteItem(rowObject);
        }
    },
    
    // Summary:
    //    When a cell renderer type is the DROPDOWN_MENU, this methods is invoked.
    //
    cellMenuActionInvoked: function(actionId, rowObject, columnIndex)
    {
        alert(actionId + " menu action invoked on item: " + rowObject.col1);
    }
    
});

dojo.declare("TestView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
        
        var moreContents = document.createElement("div");
        var toggle = document.createElement("button");
        toggle.innerHTML = "Added Custom Control";
        moreContents.appendChild(toggle);
        listWidget.contentNode.appendChild(moreContents);
    }
});

dojo.declare("TestView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    // Summary:
    //
    getNumColumns: function(content)
    {
        return 8;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        var headings = [
            "Item",
            "Operating System",
            "Machine Manufacturer",
            "Memory",
            "Owner",
            "Time Stamp",
            "Last Used",            
            "Location"
        ];
        return headings[index];
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
        if(atColumnIndex == 0) {
            return ImageCache.TEST_PLAN;
        }
        if(atColumnIndex == 1) {
            return ImageCache.TEST_CASE;
        }
        if(atColumnIndex == 2) {
            return {
                actionId: "action1",
                tooltip: "click me!",
                src: ImageCache.EWI
            };
        }
        return null;
    },
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        if(atColumnIndex == 1 || atColumnIndex == 2 || atColumnIndex == 4 || atColumnIndex == 5)
            return true;
        return false;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return (atColumnIndex < 7);     
    },
    
    getIsCellEditable: function(rowData, atColumnIndex)
    {
        // only Peter, Paul, Mary, and Bill can edit
        return (rowData.col5 == "Peter") || (rowData.col5 == "Paul") || (rowData.col5 == "Mary") || (rowData.col5 == "Bill");
    },

    getColumnWidth: function(content, atColumnIndex)
    {
        var width = 16;
        switch(atColumnIndex) {
            case 0: 
                width = 8;
                break;
            case 1: 
                width = 18;
                break;
            case 2: 
                width = 18;
                break;
            case 3: 
                width = 11;
                break;
            case 4: 
                width = 11;
                break;
            case 5: 
                width = 12;
                break;
            case 6: 
                width = 11;
                break;
            case 7: 
                width = 11;
                break;
        }
        return width;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return rowData.col1 + "";
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        var type = TableConstants.RENDER_TYPE_TEXT;

        switch(atColumnIndex) {
            case 0: 
                type = TableConstants.RENDER_TYPE_LINK;
                break;
            case 1: 
                type = TableConstants.RENDER_TYPE_TEXT;
                break;
            case 2: 
                type = TableConstants.RENDER_TYPE_EDITABLE_TEXT;
                break;
            case 3: 
                type = TableConstants.RENDER_TYPE_TEXT;
                break;
            case 4: 
                type = TableConstants.RENDER_TYPE_TEXT;
                break;
            case 5: 
                type = TableConstants.RENDER_TYPE_TIMESTAMP;
                break;
            case 6: 
                type = TableConstants.RENDER_TYPE_DATE;
                break;
            case 7: 
                if(rowData.col5 == "Peter") {
                    type = TableConstants.RENDER_TYPE_DROPDOWN_MENU;
                }
                else if(rowData.col5 == "Jane") {
                    type = TableConstants.RENDER_TYPE_IMAGE;
                }
                else if(rowData.col5 == "Luke") {
                    type = TableConstants.RENDER_TYPE_ACTION;
                }
                else if(rowData.col5 == "Bill") {
                    type = TableConstants.RENDER_TYPE_ENUM;
                }
                else {
                    type = TableConstants.RENDER_TYPE_ACTION;
                }
                break;
        }
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        if(atColumnIndex == 7 &&(rowData.col5 == "Jane" || rowData.col5 == "Luke" || rowData.col5 == "Peter"))
            return TableConstants.ALIGN_CENTER;
            
        return TableConstants.ALIGN_LEFT;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "data";
        switch(atColumnIndex) {
            case 0: 
                val = rowData.col1;
                break;
            case 1: 
                val = rowData.col2;
                break;
            case 2: 
                val = rowData.col3;
                break;
            case 3: 
                val = rowData.col4;
                break;
            case 4: 
                val = rowData.col5;
                break;
            case 5: 
                val = rowData.col6;
                break;
            case 6: 
                val = rowData.col7;
                break;
            case 7: 
                val = rowData.col8;
                break;
        }
        return val;
    },

    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = {};
        
        val['tooltip'] = "some helpful hints as to what this is.";
        
        if(atColumnIndex == 0) {
            val['href'] = "#";
        }
        if(atColumnIndex == 7 &&(rowData.col5 == "Jane" || rowData.col5 == "Luke")) {
            val['mouseOverImage'] = ImageCache.TEST_PLAN;
            val['mouseOutImage'] = ImageCache.TEST_SCRIPT;
        }
        if(atColumnIndex == 7 &&(rowData.col5 == "Peter")) {
            val['actions'] = [
                {'id': 'delete', 'label': 'Delete', 'iconClass': "test-image-5"},  
                {'id': 'run', 'label': 'Run', 'iconClass': "test-image-3"}
            ];
        }
        if(atColumnIndex == 7 && rowData.col5 == "Bill") {
            val['select'] = (typeof rowData.selection != "undefined") ? rowData.selection : null;
            val['choices'] = [
                {'text':'room 123', 'value':'123'}, 
                {'text':'room 223', 'value':'223'}, 
                {'text':'room 323', 'value':'323'}, 
                {'text':'room 423', 'value':'423'}];
        }
        return val;
    }
    
    
});

dojo.declare("TestView.CustomField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.labelField = document.createElement("div");
        this.fieldHolderNode.appendChild(this.labelField);
        this.formField = this.labelField;
        this.formField.className = "field";
    },
          
    setValue: function(value)
    {
        this.labelField.innerHTML = value;
    },

    // Summary:
    //    Returns the value from the field. Subclass must override. A null is returned if
    //    no value is specified.
    //
    getValue: function()
    {
        return this.labelField.innerHTML;
    },
   
    // Summary:
    //    This method is called when the field's value has been changed. Consumer of the widget 
    //    overrides this method to receive notification of the value changed event.
    //
    onValueChanged: function(oldValue, newValue)
    {
    }
});

dojo.declare("TestView.TableFilterProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider,
{
    constructor: function()
    {
        this.filters = [
            {name: "available", title: "Only Available Status with a really looooonnngg line", promptTitle: "Only Available Status", type: TableContentFilterProvider.FIELD_CHECKBOX, defaultValue: true, params: null},
            {name: "isMine", title: "Only Belongs to me", promptTitle: "Only Belongs to me", type: TableContentFilterProvider.FIELD_CHECKBOX, defaultValue: false, params: null},
            {name: "owner", title: "Owner:", promptTitle: "Owner", type: TableContentFilterProvider.FIELD_TEXT, defaultValue: "me", params: null},
            {name: "type", title: "Operating System:", promptTitle: "Operating System", type: TableContentFilterProvider.FIELD_ENUM, defaultValue: "Solaris 10", 
                params: ["Windows XP", "Windows Vista", "Solaris 10", "AIX 5"]
            },
            {name: "manufacturer", title: "Manufacturer:", promptTitle: "Manufacturer", type: TableContentFilterProvider.FIELD_ENUM, defaultValue: "IBM",
                // enumeration type supports name, value structures 
                params: [
                    {name: "International Business Machine", value:"ibm"},
                    {name: "Hewlett-Packard", value:"hp"},
                    {name: "Sun Microsystems", value:"sun"},
                    {name: "Dell Computers", value:"dell"}
                ]
            },
            {name: "labgroup", title: "In Lab Group:", promptTitle: "In Lab Group", type: TableContentFilterProvider.FIELD_TEXT, defaultValue: "CTG", params: null},
            {name: "memory", title: "Memory:", promptTitle: "Memory", type: TableContentFilterProvider.FIELD_NUMERIC, defaultValue: 1024, params: null},
            {name: "custom", title: "Custom Field:", promptTitle: "Custom Field", type: TableContentFilterProvider.FIELD_CUSTOM, defaultValue: null, 
                params: null
            },
            {name: "toggle", title: "Toggle OS:", promptTitle: "Toggle OS", type: TableContentFilterProvider.FIELD_CUSTOM,
                defaultValue: null, params: null
            }
        ];
    },
    
    // Summary:
    //    The filter section has the same size width as the width of the table. The filter input
    //    fields are layout in a single column inside the panel. Adjust the width of the fields
    //    as a percentage of the width of the panel. If 50 is specified, the fields right edge will
    //    be on the mid-point of the panel, etc.
    //
    getFilterFieldAreaWidth: function()
    {
        return 90;
    },

    // Summary:
    //    The input field column consists of label and the input field. Specifies the width of the field
    //    labels as a percentage of the input field column. Please note that this is the percentage of the
    //    field area width specified above.
    //
    getFilterFieldLabelWidth: function()
    {
        return 28;
    },

    getTitle: function()
    {
        return "Filterable Attribute Builder";    
    },

    getSummaryText: function()
    {
        return "We have more attributes for you to select and filter from:";
    },
    
    getInitialVisibleFilterCount: function()
    {
        return 5;    
    },
    
    getFilterCount: function()
    {
        return this.filters.length;
    },
    
    getFilterName: function(index)
    {
        return this.filters[index].name;
    },
    
    getFilterTitle: function(index)
    {
        return this.filters[index].title;
    },
    
    getFilterPromptTitle: function(index)
    {
        return this.filters[index].promptTitle;        
    },
    
    getFilterType: function(index)
    {
        return this.filters[index].type;        
    },
    
    getFilterDefaultValue: function(index)
    {
        return this.filters[index].defaultValue;        
    },
    
    getFilterParams: function(index)
    {        
        return this.filters[index].params;
    },
    
    getCustomField: function(index)
    {
        // create our own field type.
        var customField = null; 
        if (this.filters[index].name == "custom") {
            customField = new TestView.CustomField();
            customField.setValue("My own Field Type..."); 
        }
        else if (this.filters[index].name == "toggle") {
            // create our own field type.
            customField = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField({
                popupTitle: "OS",
                popupPrompt: "Select Operating System:",
                options: [
                    {name:"Windows XP", value:"XP", checked: false},
                    {name:"Windows Vista", value:"VISTA", checked: true},
                    {name:"Solaris 10", value:"SOL10", checked: false},
                    {name:"AIX 5", value:"AIX5", checked: false}
                ]
            });
        }
        return customField;
    },
    
    getActionsInfo: function()
    {
        var actions = [
            {id:'cancel', title:'Cancel', callback:dojo.hitch(this, "filterActionInvoked"), callbackData:null, isDefault:false},
            {id:'run', title:'Run', callback:dojo.hitch(this, "filterActionInvoked"), callbackData:null, isDefault:true}
        ];
        return actions;
    },
    
    filterActionInvoked: function(actionName, filters)
    {
        alert("Filter action: " + actionName + " invoked.");
        console.dir(filters);
    }
    
});

dojo.declare("com.ibm.asq.common.web.ui.internal.test.TestView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="domNode" tabindex="-1">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" dojoAttachPoint="viewHeaderNode">' +
                    '    <div><label for="num-items">Load this many items into the table:   </label>' +
                    '      <input name="num-items" id="num-items" type="text" value="100" dojoAttachPoint="numItemsNode">' +
                    '      </input>' +
                    '      <button dojoAttachPoint="loadItemsNode" dojoAttachEvent="onclick:loadItems">Load it' +
                    '      </button>' +
                    '    </div>' +
//                    '    <div>Set the table\'s page size to:   ' +
//                    '      <input type="text" value="20" dojoAttachPoint="pageSizeNode">' +
//                    '      </input>' +
//                    '      <button dojoAttachEvent="onclick:changePageSize">Change it' +
//                    '      </button>' +
//                    '    </div>' +
                    '    <div>Change the Table display Mode:   ' +
                    '      <label for="grid-toggle"><input name="grid-toggle" id="grid-toggle" type="radio" value="grid" checked="true" dojoAttachPoint="gridToggleNode" dojoAttachEvent="onclick:changeMode">  Grid Mode' +
                    '      </input></label>' +
                    '      <label for="list-toggle"><input name="list-toggle" id="list-toggle" type="radio" value="list" dojoAttachPoint="listToggleNode" dojoAttachEvent="onclick:changeMode">  List Mode' +
                    '      </input></label>' +
                    '    </div>' +
                    '    <div>Show/hide table Controls:   ' +
                    '      <br></br>' +
                    '      <input type="checkbox" checked="true" dojoAttachPoint="showPageCtrl" dojoAttachEvent="onclick:changeControlDisplay">  Show Paging Control' +
                    '      </input>' +
                    '      <br></br>' +
                    '      <input type="checkbox" checked="true" dojoAttachPoint="showFilterCtrl" dojoAttachEvent="onclick:changeControlDisplay">  Show Filter Control' +
                    '      </input>' +
                    '      <br></br>' +
                    '      <input type="checkbox" checked="true" dojoAttachPoint="showTableHeading" dojoAttachEvent="onclick:changeControlDisplay">  Show Table Heading in Grid mode' +
                    '      </input>' +
                    '    </div>' +
                    '    <div>' +
                    '      <input type="checkbox" dojoAttachPoint="rowCheckBoxNode" dojoAttachEvent="click:changeRowCheckbox">  Make each row checkable (selectable)' +
                    '      </input>' +
                    '      <input type="checkbox" disabled="true" dojoAttachPoint="singleSelectNode" dojoAttachEvent="click:changeRowSelectionNode"> Single Selection Only' +
                    '      </input>' +
                    '      <button disabled="true" dojoAttachPoint="getCheckRowNode" dojoAttachEvent="click:getCheckedRows">Get Checked Rows' +
                    '      </button>' +
                    '    </div>' +
                    '    <div>' +
                    '      <input type="checkbox" dojoAttachPoint="editableCheckboxNode" dojoAttachEvent="click:changeViewerEditable" checked="true">  Only Peter, Paul, Mary, and Bill can edit or no one can edit' +
                    '      </input>' +
                    '    </div>' +
                    '    <div dojoAttachPoint="resultsNode" style="border:1px">' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',

    widgetsInTemplate: true,

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        var filterProvider = new TestView.TableFilterProvider();
        var tableArgs = {allowAdjustSettings: true, filterAttrProvider:filterProvider};
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer(tableArgs);
        
        this.viewHeaderNode.getContentNode().appendChild(tableViewer.domNode);
        this.tableViewer = tableViewer;
        
        var content = this.createContent(100);
        var contentProvider = new TestView.TableProvider();
        
        var views = [
            {'name': 'show all', 'value': 100},
            {'name': 'show 20 items', 'value': 20},
            {'name': 'show 40 items', 'value': 40},
            {'name': 'show 60 items', 'value': 60}
        ];
        tableViewer.setTableContentSelectors(views);
        
        var cb_func = function() {
            alert("another callback...");
        }
        tableViewer.addAction("action1", "action1", null, "test-image-1", cb_func);
        tableViewer.addAction("action2", "action2", null, "test-image-2", cb_func);
        tableViewer.addAction("action3", "action3", null, "test-image-3", cb_func);
        tableViewer.addAction("action4", "action4", null, "test-image-4", cb_func);
        
        tableViewer.setContent(content);
        tableViewer.setContentProvider(contentProvider);
        tableViewer.setTableListener(new TestView.TableListener(this));
        tableViewer.setCellModifier(new TestView.CellModifier(tableViewer));
        tableViewer.show();
    },
    
    loadItems: function()
    {
        var n = parseInt(this.numItemsNode.value);
        var content = this.createContent(n);
        this.tableViewer.setContent(content);
        this.tableViewer.show();
    },
    
    changePageSize: function()
    {
        var n = parseInt(this.pageSizeNode.value);
        this.tableViewer.setContentPageSize(n);
        this.tableViewer.show();        
    },
    
    createContent: function(nRows)
    {
        var rows = [];
        var os = [
            "Windows XP",
            "Windows 2000",
            "Linux",
            "RedHat 7 Enterprise Edition",
            "RedHat 8 Desktop Edition",
            "Vista Profressional Edition",
            "Windows XP Professional Edition",
            "Solaris 10",
            "Solaris 9 x86 edition",
            "Windows 2000 DataCenter Edition"
        ];
        var manufacturer = [
            "IBM",
            "HP",
            "Sun",
            "Dell",
            "Digital",
            "Cisco",
            "IBM",
            "IBM",
            "HP",
            "Appollo"
        ];
        var owner = [
            "TestJazzAdmin1",
            "Andrew",
            "TestJazzAdmin1",
            "Paul",
            "Peter",
            "Mary",
            "Luke",
            "TestJazzAdmin1",
            "ADMIN",
            "Laura"
        ];
        var dates = ["01","02","03","04","05","06","07","08","09","10"];
        
        for(var i = 0; i < nRows; i++) {
            var date = new Date();
            var time = date.getTime();
            time -= (i * hours);
            date.setTime(time);
            
            var obj = {
                col1: i,
                col2: os[i % 10],
                col3: manufacturer[i % 10],
                col4: (i * 256),
                col5: owner[i % 10],
                col6: "2008-03-" + dates[i % 10] + "T12:59:59.999-0500",
                col7: date,
                col8: "room 123"
            };
            rows.push(obj);
        }
        return rows;
    },
    
    changeRowCheckbox: function(event) 
    {
        var checkable = this.rowCheckBoxNode.checked;
        var single = this.singleSelectNode.checked;
        
        this.getCheckRowNode.disabled = !checkable;
        this.singleSelectNode.disabled = !checkable;
        this.resultsNode.innerHTML = "";
        
        this.tableViewer.setTableRowsCheckable(checkable, single);
        this.tableViewer.show();
    },
    
    changeRowSelectionNode: function(event)
    {
        this.changeRowCheckbox(event);
    },
    
    getCheckedRows: function(event)
    {
        var rows = this.tableViewer.getCheckedRows();
        var text = "";
        
        if(rows.length == 0) {
            text = "<b>No rows selected</b>";
        }
        else {
            text = "<b>Rows selected:</b><br></br>";
            for(var i = 0; i < rows.length; i++) {
                var rowObj = rows[i];
                text = text + (rowObj.col1 + " <br></br>");
            }
        }
        this.resultsNode.innerHTML = text;
    },
    
    changeMode: function(event)
    {
        var isList;
        if(event.currentTarget.value == "grid") {
            this.gridToggleNode.checked = true;
            this.listToggleNode.checked = false;
            isList = false;
        }
        else {
            this.gridToggleNode.checked = false;
            this.listToggleNode.checked = true;            
            isList = true;
        }
        var mode = (isList) ? TableConstants.MAJOR_MODE_LIST : TableConstants.MAJOR_MODE_GRID;
        
        this.tableViewer.setTableMajorMode(mode);
        
        // create our own custom row renderer to slip in more contents into
        // default list renderer's collapsible area widget. 
        if(isList) {
            this.tableViewer.setCustomRowRenderer(new TestView.ListRowRenderer());
        }
        this.tableViewer.show();
    },
    
    changeViewerEditable: function(event)
    {
        this.tableViewer.setEditable(this.editableCheckboxNode.checked);
        this.tableViewer.show();
    },

    changeControlDisplay: function(event)
    {
        this.tableViewer.setShowingFilterControl(this.showFilterCtrl.checked);
        this.tableViewer.setShowingPagingControl(this.showPageCtrl.checked);
        this.tableViewer.setShowingTableHeading(this.showTableHeading.checked);
        
        this.tableViewer.show();
    }

});

})();
