/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 
MessageBundle = dojo.provide("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

// predefined message bundle constant id
MessageBundle.COMMON = "COMMON"; 
MessageBundle.LM_CATALOG = "LM_CATALOG"; 

dojo.require("dojo.date.locale"); 
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.asq.common.web.ui.internal", "messages");
dojo.requireLocalization("com.ibm.asq.common.web.client.internal", "messages");

dojo.declare("com.ibm.asq.common.web.ui.internal.util._MessageUtil", null,
{
    // array of message bundles
    _msgBundle: {},

    constructor: function(){
        // initializes the common message bundle
        this.registerRequireBundle(MessageBundle.COMMON, "com.ibm.asq.common.web.ui.internal", "messages");
        this.registerRequireBundle(MessageBundle.LM_CATALOG, "com.ibm.asq.common.web.client.internal", "messages");
        console.debug("Registered COMMON bundle: \"" + this.getMessage(MessageBundle.COMMON, "bundle_description") + "\"" );
    },
    
    // Return the message bundle if it is registered.
    //
    getBundle: function(bundleId) {
        return this._msgBundle[bundleId];
    },
    
    // Register a plugin specific message bundle.  Other plugin can consume a
    // message bundle with the specific bundle id once it is registered.
    //
	registerBundle: function(bundleId, packageName, bundleName) {
        if (this.getBundle(bundleId)) {
            // ignore if the bundle is already registered
            return;
        }

        this._msgBundle[bundleId] = dojo.i18n.getLocalization(packageName, bundleName);
        console.debug("Registered bundle: id=\"" + bundleId + "\"" );
        console.debug("Registered bundle: name=\"" + bundleName + "\"" );
    },
    
    // To be backward compatible, leave registerBundle() as is so only the plugins that
    // are moved to the new translation using Eclipse NLS is using this method.
    //
	registerRequireBundle: function(bundleId, packageName, bundleName) {
		// For some reason the Jazz JavaScript optimizer complains of reference error with
		// dojo.requireLocalization() which SHOULD be ignored.
		var _dj = dojo;
//		_dj.requireLocalization(packageName, bundleName);
		this.registerBundle(bundleId, packageName, bundleName);
    },
    

    // Return the message from the specific message bundle with the specified keyword.
    // Optional token arguments are for replacing place holders in the message
    // definition.  The place holders have the syntax "%number" with the number
    // starting from 1.  For example, if the message "myStr" is defined as
    // "This is %1 of %2 messages".  getMessage("myStr", "one", "my many") will return
    // "This is one of my many messages".
    //
    getMessage: function(bundleId, keyword, tokens) {
        var message = this._msgBundle[bundleId][keyword];
        if (message == null) {
            // console.debug("Failed to get message from keyword \"" + keyword + "\"");
            return keyword;
        }
        if (arguments.length > 2) {
            for ( var i = 1; i < (arguments.length - 1); i++) {
                message = message.replace(new RegExp("%" + i, "g"), arguments[i + 1]);
            }
        }
        return message;
    }
});

// create a singleton utility
MessageUtil = new com.ibm.asq.common.web.ui.internal.util._MessageUtil();

