/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
var Utility = dojo.provide("com.ibm.rqm.planning.web.ui.internal.util.Utility");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.StringUtil");

(function() 
{
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var Constants  = com.ibm.rqm.planning.web.client.internal.Constants;
    var StringUtil = com.ibm.rqm.planning.web.ui.internal.util.StringUtil;
	
    var ACTION_PREFIX = "#action=com.ibm.rqm.planning.home.actionDispatcher";
    var VIEW_WI_ACTION_PREFIX = "#action=com.ibm.team.workitem.viewWorkItem";
    var VIEW_WI_TAB_PARM = "tab";
    
    com.ibm.rqm.planning.web.ui.internal.util.Utility = 
    {
        PLANNING_PERSPECTIVE: 1,
        CONSTRUCTION_PERSPECTIVE: 2,
        EXECUTION_PERSPECTIVE: 3,

        _planningPrefix: ACTION_PREFIX + "&subAction=",
        _constructionPrefix: ACTION_PREFIX + "&subAction=",
        _executionPrefix: ACTION_PREFIX + "&subAction=",

        /******************************************************************************************
         * Generic URI functions
         * 
         *****************************************************************************************/
        _getPlanningActionUri: function(action, kwArgs)
        {
            var str = this._planningPrefix + action;
            return(this._getActionUri(str, kwArgs));
        },
        
        _getConstructionActionUri: function(action, kwArgs)
        {
            var str = this._constructionPrefix + action;
            return(this._getActionUri(str, kwArgs));
        },
        
        _getExecutionActionUri: function(action, kwArgs)
        {
            var str = this._executionPrefix + action;
            return(this._getActionUri(str, kwArgs));
        },
        
        _getActionUri: function(str, kwArgs) 
        {
            for (var key in kwArgs) 
            {
                str += "&" + key + "=" + kwArgs[key];
            }
            return str;
        },

        _buildUri: function(method, args, perspective)
        {
            var result;
            
            switch(perspective)
            {
                case this.PLANNING_PERSPECTIVE:
                    result = this._getPlanningActionUri(method, args);
                    break;
                case this.CONSTRUCTION_PERSPECTIVE:
                    result = this._getConstructionActionUri(method, args);
                    break;
                case this.EXECUTION_PERSPECTIVE:
                    result = this._getExecutionActionUri(method, args);
                    break;
				default:
					result = this._getPlanningActionUri(method, args);
					break;
            }
            
            return(result);
        },

        /******************************************************************************************
         * Determine which perspective is currently displayed.
         * 
         * @param {Object} domNode a node in the current DOM hierarchy
         *****************************************************************************************/
        getCurrentPerspective: function (domNode) {
            while (domNode!=null) {
                if (domNode.className!=null && domNode.className.indexOf("construction-page") > -1) {
                    return this.CONSTRUCTION_PERSPECTIVE;
                }
                domNode = domNode.parentNode;
            }
            return this.PLANNING_PERSPECTIVE;
        },
        
        getCurrentPerspectiveByURI: function () {
            var location = window.location.toString();
            var qmLocation = location.substr(location.indexOf('#action'));
            if (StringUtil.startsWith(qmLocation, this._planningPrefix)) {
                return this.PLANNING_PERSPECTIVE;
            } else if (StringUtil.startsWith(qmLocation, this._constructionPrefix)) {
                return this.CONSTRUCTION_PERSPECTIVE;
            } else if (StringUtil.startsWith(qmLocation, this._executionPrefix)) {
                return this.EXECUTION_PERSPECTIVE;
            }            
            return this.PLANNING_PERSPECTIVE;
        },      
        
        /******************************************************************************************
         * Action URI methods
         * 
         * @param {Object} id
         *****************************************************************************************/
        getGenericSearchUri: function(str)
        {
            return(this._getPlanningActionUri("genericSearch", {searchStr: str}));
        },

        getNewQueryUri: function()
        {
            return(this._getPlanningActionUri("newQuery"));
        },

        getViewWorkItemURI: function(id, tab)
        {
            var uriArgs = {};
            uriArgs.id = id
            if (tab != null) {
                uriArgs.tab = tab;
            }
            return this._getActionUri(VIEW_WI_ACTION_PREFIX, uriArgs);
        },
        
        //TEST CASE URI UTILS           
        getNewTestCaseURI: function(name) 
        {
            return this._getConstructionActionUri("newTestCase", {name: name});
        },
            
        getTestCaseURI: function(/*required*/testId, /*optional*/ revision, /*optional*/perspective, /*optional*/ sectionId)
        {
            var args = {};
            if (testId != null) {               
                if (revision != null) {
                    args.uid = testId;
                    args.revision = revision;
                } else {
                    args.id = testId;
                }
            }
			if (sectionId != null) {
				args.sectionId = sectionId;
			}
            return this._buildUri("viewTestCase", args, perspective ? perspective : this.CONSTRUCTION_PERSPECTIVE);
        },
        /*              
        getTestCaseRevisionURI: function(testId, aRevision, perspective)
        {
            var args = {};
            if (testId != null && aRevision!=null)
            {
                args = {uid: testId, revision: aRevision};
            }
            return this._getConstructionActionUri("viewTestCase", args);
        },
        */      
        getCopyTestCaseURI: function(testId, aRevision, perspective)
        {
            var args = {};
            if (testId!=null && aRevision!=null)
            {
                args = {uid: testId, revision: aRevision};
            }
            return(this._buildUri("copyTestCase", args, perspective));
        },
        
        getTestCasesURI: function(queryName)
        {
            var args = {};
            if (queryName != null)
            {
                args = {name: queryName};
            }
            return this._getConstructionActionUri("viewTestCases", args);
        },

        getMyTestCasesURI: function(queryName)
        {
            var args = {};
            if (queryName != null)
            {
                args = {name: queryName};
            }
            return this._getConstructionActionUri("viewMyTestCases", args);
        },

        getTestSuitesURI: function(queryName)
        {
            var args = {};
            if (queryName != null)
            {
                args = {name: queryName};
            }
            return this._getConstructionActionUri("viewTestSuites", args);
        },

        getMyTestSuitesURI: function(queryName)
        {
            var args = {};
            if (queryName != null)
            {
                args = {name: queryName};
            }
            return this._getConstructionActionUri("viewMyTestSuites", args);
        },
        
		getTestSuiteURI: function(suiteId, perspective, /*optional*/ revision, /*optional*/ snapshotName)
        {
            var args = {};
        
            if (suiteId != null)
            {
                if (revision != null) {
                    args.uid = suiteId;
                    args.revision = revision;
                    if (snapshotName!=null) {
                        args.snapshotName = escape(snapshotName);
                    }
                } else {
                    args.id = suiteId;
                }
            }
        
            if (perspective == null)
            {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

            return(this._buildUri("viewTestSuite", args, perspective));
        },

		getCopyTestSuiteURI: function(suiteId, /*optional*/ perspective, aRevision, /*optional*/ snapshotName)
        {
            var args = {};
            if (suiteId!=null && aRevision!=null)
            {
                args = {uid: suiteId, revision: aRevision};
				if (snapshotName != null) {
					args.snapshotName = escape(snapshotName);
				}
            }
			if (perspective == null) {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

            return(this._buildUri("copyTestSuite", args, perspective));
        },

        getNewTestScriptURI: function(name, perspective) 
        {
            if (perspective == null)
            {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

            return(this._buildUri("newTestScript", {name: name}, perspective));
        },
        
        getImportScriptURI: function(name, perspective)
        {
            if (perspective == null)
            {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

            return(this._buildUri("importScripts", {name: name}, perspective));
        },
            
        getTestScriptURI: function(scriptId, perspective, /*optional*/ revision, /*optional*/ snapshotName)
        {
            var args = {};
        
            if (scriptId != null)
            {
                if (revision != null) {
                    args.uid = scriptId;
                    args.revision = revision;
                    if (snapshotName!=null) {
                        args.snapshotName = escape(snapshotName);
                    }
                } else {
                    args.id = scriptId;
                }
            }
        
            if (perspective == null)
            {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

            return(this._buildUri("viewTestScript", args, perspective));
        },
                
        getTestScriptsURI: function(queryName, perspective) 
        {
            var args = {};

            if (queryName != null) 
            {
                args = {name: queryName};
            }

            if (perspective == null)
            {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

            return(this._buildUri("viewTestScripts", args, perspective));
        },
        
		getCopyTestScriptURI: function(scriptId, /*optional*/ perspective, aRevision, /*optional*/ snapshotName)
        {
            var args = {};
            if (scriptId!=null && aRevision!=null)
            {
                args = {uid: scriptId, revision: aRevision};
				if (snapshotName != null) {
					args.snapshotName = escape(snapshotName);
				}
            }
			if (perspective == null) {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

            return(this._buildUri("copyTestScript", args, perspective));
        },

        getNewTestPlanURI: function(name) 
        {
            return this._getPlanningActionUri("newTestPlan", {name: name});
        },
            
        getTestPlanURI: function(kwargs, perspective) 
        {
            var args = {};
            if ( kwargs.itemId != null )
            {
                args = {itemId: kwargs.itemId};
            }
            else if ( kwargs.uid != null && kwargs.revision != null)
            {
                args = {uid: kwargs.uid, revision: kwargs.revision};
            }
            else if (kwargs.id != null)
            {
                args = {id: kwargs.id};
            }
            else if (kwargs.name != null)
            {
                args = {name: kwargs.name};
            }

			if (kwargs.sectionId != null) {
				args.sectionId = kwargs.sectionId;
			}
			
            if (perspective == null)
            {
                perspective = this.PLANNING_PERSPECTIVE;
            }

            return(this._buildUri("viewTestPlan", args, perspective));
//          return this._getPlanningActionUri("viewTestPlan", args);
        },
                
        getCopyTestPlanURI: function(planId, aRevision, perspective) 
        {
            var args = {};
            if (planId!=null && aRevision!=null)
            {
                args = {uid: planId, revision: aRevision};
            }
            return(this._buildUri("copyTestPlan", args, perspective));
        },
                
        getTestPlansURI: function(queryName, perspective) 
        {
            var args = {};
            if (queryName != null) 
            {
                args = {name: queryName};
            }
            if (perspective == null)
            {
                perspective = this.PLANNING_PERSPECTIVE;
            }
            
            return(this._buildUri("viewTestPlans", args, perspective));
//          return this._getPlanningActionUri("viewTestPlans", args);
        },
        
        getNewEwiURI: function(name, testCaseId) 
        {
            var testId = testCaseId != null ? testCaseId : null; 
            return this._getConstructionActionUri("newEwi", {name: name, testCaseId: testId});
        },
            
        getEwiURI: function(id, perspective) 
        {
            var args = {};
            if (id != null) {
                args = {id: id};
            }
            if (perspective == null){
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }
            return(this._buildUri("viewEwi", args, perspective));
        },
                
        getEwisURI: function(queryName, perspective) 
        {
            var args = {};
            if (queryName != null) {
                args.name = queryName;
            }

            if (perspective == null)
            {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

            return(this._buildUri("viewEwis", args, perspective));
//          return this._getPlanningActionUri("viewEwis", args);
        },
        
        // DATAPOOL URLs
        getViewDatapoolURI: function(itemId)
        {
            return this._getActionUri(this._planningPrefix + "viewDatapool", {itemId: itemId});
        },
        
        startExecution: function(ewi, scriptId, resultItemId, requestId, ergId)
        {
            var args = {};
            
            if (ewi != null)
            {
                args = {
                    ewi: ewi,
                    runScript: scriptId
                };
                if(resultItemId != null)
                    args._resultId = resultItemId;
            }
            if(requestId)
                args.requestId = requestId;
            if(ergId)
                args.ergId = ergId; 
            return this._getExecutionActionUri("startExecution", args);
        },
        
        runSuite: function(kwArgs){
            var args = {};
            if (kwArgs.groupId != null)
                args.groupId = kwArgs.groupId
            if (kwArgs.itemId != null)
                args.itemId = kwArgs.itemId
            return this._getExecutionActionUri("runSuite", args);
        },

    
        executedItemsURI: function (queryName)
        {
            var args = {};          
            if (queryName != null) {
                args.name = queryName;
            }
            return this._getExecutionActionUri("viewExecutedItems", args);
        },
    
        getEWIResultsURI: function(ewiItemId)
        {
            var args = {};          
            if (ewiItemId != null) {
                args.ewiItemId = ewiItemId;
            }           
            return this._getExecutionActionUri("viewEWIResults", args);
        },
        
        showResultsURI: function(queryName)
        {
            var args = {};          
            if (queryName != null) {
                args.name = queryName;
            }           
            return this._getExecutionActionUri("viewResults", args);
        },
        
        getResultURI: function(args)
        {
            return this._getExecutionActionUri("viewResult", args );
        },
        
        showSuiteResult: function(kwArgs)
        {
            var args = {};          
            if (kwArgs.id != null) {
                args.id = kwArgs.id;
            }           
            return this._getExecutionActionUri("showSuiteResult", args);
        },
		
		getBuildRecordURI: function(id) {
			return this._buildUri("editBuild", {id: id});
		},
		
		getBuildDefinitionURI: function(id) {
			return this._buildUri("editBuildDef", {id: id});
		},
		
		getKeywordsURI: function() {
			return this._buildUri("keywords");
		},
        
        getRequestURI: function(id) {
            return this._buildUri("viewRequest", {id: id});
        },
        
        getTestCellURI: function(id) {
            return this._buildUri("viewTestCell", {cellID: id});
        },
        
        getTestEnvironmentURI: function(id) {
            return this._buildUri("viewConfiguration", {id: id});
        },
        
        getResourceGroupURI: function(id) {
            return this._buildUri("viewGroup", {id: id});
        },

        getExecutionScheduleURI: function(id) {
            return this._buildUri("viewSequence", {sequenceId: id});
        },

        getLabResourceURI: function(details) {
            var tokens = details.split(":");
            return this._buildUri("viewAsset", {id: tokens[0], type: tokens[1]});
        },
        
        /******************************************************************************************
         * Generate Unique URL
         * 
         * @param {Object} url
         *****************************************************************************************/
        reuniquifyUrl: function(url)
        {
            return(url.replace(/&tq=([0-9]+)/, "") + "&tq=" + this.getUniqueId());
        },
        
        getUniqueId: function()
        {
            return Date.parse(new Date().toString());
        },

        /******************************************************************************************
         * Set the window location to an action with keywords
         *****************************************************************************************/
        setLocationToActionWithKeywords: function(action, kwArgs)
        {
            //TODO: fix this for all perspectives
            this._setLocationHash(this._getPlanningActionUri(action, kwArgs));
        },
    
        /******************************************************************************************
         * Set the window location hash.
         * This is pushed to a method so all changes to the URI can be traced if needed 
         *****************************************************************************************/
        _setLocationHash: function(locationHash)
        {
            window.location.hash = locationHash;
        },

        /******************************************************************************************
         * QSort functions
         * 
         * @param {Object} list
         * @param {Object} a
         * @param {Object} b
         *****************************************************************************************/
        swap: function(list, a, b)
        {
            var tmp = list[a];
            list[a] = list[b];
            list[b] = tmp;
        },
        
        partition: function(array, begin, end, pivot)
        {
            var piv = array[pivot];
            this.swap(array, pivot, end-1);
            
            var store=begin;
    
            for(var ix = begin; ix < end-1; ++ix) 
            {
                if(array[ix] <= piv)
                {
                    this.swap(array, store, ix);
                    ++store;
                }
            }
            
            this.swap(array, end-1, store);
        
            return store;
        },
        
        qsort: function(list, begin, end)
        {
            if(end-1>begin)
            {
                var pivot = begin+Math.floor(Math.random()*(end-begin));
    
                pivot = this.partition(list, begin, end, pivot);
    
                this.qsort(list, begin, pivot);
                this.qsort(list, pivot+1, end);
            }       
        },
        
        partition_obj: function(array, index, begin, end, pivot)
        {
            var piv = array[pivot].elements[index].data;
            if (typeof piv == "string")
            {
                piv = piv.toLowerCase();
            }
    
            this.swap(array, pivot, end-1);
            
            var store=begin;
    
            for(var ix = begin; ix < end-1; ++ix) 
            {
                var data = array[ix].elements[index].data;
                if (typeof data == "string")
                {
                    data = data.toLowerCase();
                }
                
                if(data <= piv)
                {
                    this.swap(array, store, ix);
                    ++store;
                }
            }
            
            this.swap(array, end-1, store);
        
            return store;
        },
        
        qsort_obj: function(list, index, begin, end)
        {
            if(end-1>begin)
            {
                var pivot = begin+Math.floor(Math.random()*(end-begin));
    
                pivot = this.partition_obj(list, index, begin, end, pivot);
    
                this.qsort_obj(list, index, begin, pivot);
                this.qsort_obj(list, index, pivot+1, end);
            }       
        },
        
        /******************************************************************************************
         * Get Icon/Image functions
         *  
         * @param {Object} path
         *****************************************************************************************/
        getArtifactIconURL: function(artifactType) {
            //handles TYPE* and MODEL_TYPE* constants
            switch(artifactType) {
                case Constants.TYPE_PLAN:
                case Constants.MODEL_TYPE_PLAN:
                    return ImageCache.TEST_PLAN;
                case Constants.TYPE_GROUP:
                case Constants.MODEL_TYPE_GROUP:
                    return ImageCache.TEST_GROUP;
                case Constants.TYPE_CASE:
                case Constants.MODEL_TYPE_CASE:                 
                    return ImageCache.TEST_CASE;                    
                case Constants.TYPE_SCRIPT:
                case Constants.MODEL_TYPE_SCRIPT:
                case Constants.MODEL_TYPE_REMOTE_SCRIPT:
                    return ImageCache.TEST_SCRIPT;
                case Constants.TYPE_PLAN_SIZING:
                    return ImageCache.TEST_PLAN;
                case Constants.TYPE_GROUP_SIZING:
                    return ImageCache.TEST_GROUP;
                case Constants.TYPE_CASE_SIZING:
                    return ImageCache.TEST_CASE;
                case Constants.TYPE_SCRIPT_SIZING:
                    return ImageCache.TEST_SCRIPT;
                case Constants.TYPE_EWILIST:
				case Constants.MODEL_TYPE_EWI:
                    return ImageCache.EWI;  
                case Constants.MODEL_TYPE_RESULT:
                case Constants.TYPE_RESULT:
                    return ImageCache.EWI_LOG; 
                case Constants.MODEL_TYPE_SUITE:
                case Constants.TYPE_SUITE:
                    return ImageCache.TEST_SUITE;
                case Constants.MODEL_TYPE_WORKITEM:
                    return ImageCache.WORK_ITEM;     
				case Constants.MODEL_TYPE_REQUIREMENT:
				case Constants.TYPE_REQUIREMENT:
                    return ImageCache.REQUIREMENT;
				case Constants.MODEL_TYPE_CONFIGURATION:
				case Constants.TYPE_CONFIGURATION:
					return ImageCache.CONFIGURATION_ICON;
				case Constants.MODEL_TYPE_BUILD_DEFINITION:
					return ImageCache.BUILD_DEF; 
				case Constants.MODEL_TYPE_BUILD_RECORD:
					return ImageCache.BUILD_RECORD; 
				case Constants.MODEL_TYPE_DATAPOOL: 
					return ImageCache.DATAPOOL;
				case Constants.MODEL_TYPE_KEYWORD:
					return ImageCache.KEYWORD;
				case Constants.CALM_LINK_TYPE_PLAN_ITEM:
				case Constants.CALM_LINK_TYPE_REQUIREMENT:
				case Constants.CALM_LINK_TYPE_REQUIREMENT_SET:
					return ImageCache.REQUIREMENT; 
                case Constants.MODEL_TYPE_RESOURCE_GROUP:
                    return ImageCache.MACHINEGROUP;
                case Constants.MODEL_TYPE_EXECUTION_SEQUENCE:
                    return ImageCache.EXECUTION_SEQ;
                case Constants.MODEL_TYPE_LAB_RESOURCE:
                    return ImageCache.MACHINE; // TODO: split into Machine/Virtual Image?
                case Constants.MODEL_TYPE_LAB_REQUEST:
                    return ImageCache.REQUEST;
                case Constants.MODEL_TYPE_TEST_CELL:
                    return ImageCache.TEST_CELL;
                case Constants.MODEL_TYPE_TEST_ENVIRONMENT:
                    return ImageCache.TEST_ENVIRONMENT;
                default: 
                    return "";
            }
        },
        
      /******************************************************************************************
       * Get URI to view the Artifact
       *  
       * @param {Object} path
       *****************************************************************************************/
      getArtifactURI: function(artifact, perspective, useRevision) {
        var type = this.getArtifactType(artifact);          
        var currentPerspective = perspective ? perspective : this.getCurrentPerspectiveByURI();
            
        switch(type) {
            case Constants.MODEL_TYPE_PLAN:
			case Constants.TYPE_PLAN:
                var args = {};
                args.id = artifact.id;
                if (artifact.snapshotInfo && useRevision) {
                    args.revision = artifact.snapshotInfo.revision;                                       
                }          
                return this.getTestPlanURI(args, currentPerspective);
            
            case Constants.MODEL_TYPE_CASE:			
            case Constants.TYPE_CASE:
                if (artifact.snapshotInfo && useRevision) {
                    return this.getTestCaseURI(artifact.itemId, artifact.snapshotInfo.revision, currentPerspective);                    
                }               
                return this.getTestCaseURI(artifact.id);
                
            case Constants.MODEL_TYPE_SCRIPT:
            case Constants.MODEL_TYPE_REMOTE_SCRIPT:            
			case Constants.TYPE_SCRIPT:			
               return this.getTestScriptURI(artifact.id, currentPerspective); 

            case Constants.MODEL_TYPE_SUITE:   
			case Constants.TYPE_SUITE:
               return this.getTestSuiteURI(artifact.id); 		   
            
            default: 
               return "";
               
         }  
      },        
        
        /******************************************************************************************
         * Returns the Constant representing the artifactType
         *  
         * @param {Object} artifact - the QM artifact DTO object instance 
         *****************************************************************************************/     
        getArtifactType: function(artifact) {
            var fullType = artifact._eQualifiedClassName;
            var type = fullType.substr(fullType.lastIndexOf(':') + 1);
            
            switch(type) {
                case Constants.CLASSNAME_PLAN:
                    return Constants.TYPE_PLAN;
                case Constants.CLASSNAME_GROUP:
                    return Constants.TYPE_GROUP;
                case Constants.CLASSNAME_CASE:
                    return Constants.TYPE_CASE;
                case Constants.CLASSNAME_SCRIPT:
                    return Constants.TYPE_SCRIPT;
                case Constants.CLASSNAME_EWI:
                    return Constants.TYPE_EWI;  
                case Constants.CLASSNAME_PLAN_SIZING:
                    return Constants.TYPE_PLAN_SIZING;
                case Constants.CLASSNAME_GROUP_SIZING:
                    return Constants.TYPE_GROUP_SIZING;
                case Constants.CLASSNAME_CASE_SIZING:
                    return Constants.TYPE_CASE_SIZING;
                case Constants.CLASSNAME_SCRIPT_SIZING:
                    return Constants.TYPE_SCRIPT_SIZING;
                default:
                    return null;                                            
            }           
        },      
        
        /******************************************************************************************
         * Returns the Constant representing the item's model type
         *  
         * @param {Object} item - the QM artifact DTO object instance 
         *****************************************************************************************/     
        getModelType: function(item) {
            var fullType = item._eQualifiedClassName;
            var type = fullType.substr(fullType.lastIndexOf(':') + 1);
            
            switch (type) {
                case Constants.CLASSNAME_PLAN:
                    return Constants.MODEL_TYPE_PLAN;
                case Constants.CLASSNAME_GROUP:
                    return Constants.MODEL_TYPE_GROUP;
                case Constants.CLASSNAME_CASE:
                    return Constants.MODEL_TYPE_CASE;
                case Constants.CLASSNAME_CASESUMMARY:
                    return Constants.MODEL_TYPE_CASE;   
                case Constants.CLASSNAME_SCRIPT:
                    return Constants.MODEL_TYPE_SCRIPT;
                default:
                    return Constants.MODEL_TYPE_UNKNOWN;
            }                                           
        },
        
      /******************************************************************************************
       * Returns the Constant representing the item's model type
       *  
       * @param {Object} item - the QM artifact DTO object instance 
       *****************************************************************************************/     
      getModelTypeByItemType: function(itemType) {
         switch (itemType) {
            case Constants.TYPE_PLAN:
               return Constants.MODEL_TYPE_PLAN;
            case Constants.TYPE_GROUP:
               return Constants.MODEL_TYPE_GROUP;
            case Constants.TYPE_CASE:
               return Constants.MODEL_TYPE_CASE;
            case Constants.TYPE_SCRIPT:
               return Constants.MODEL_TYPE_SCRIPT;
            case Constants.TYPE_EWI:
               return Constants.MODEL_TYPE_EWI;
            default:
               return Constants.MODEL_TYPE_UNKNOWN;
         }             
      },
      
      /******************************************************************************************
       * Returns the Constant representing the item's model type
       *  
       * @param {Object} item - the QM artifact DTO object instance 
       *****************************************************************************************/     
      getDisplayNameForType: function(itemType) {
         switch (itemType) {
            //Test Plan
            case Constants.TYPE_PLAN:
            case Constants.MODEL_TYPE_PLAN:
            case Constants.CLASSNAME_PLAN:   
               return Constants.TYPE_PLAN;
            //Test Case
            case Constants.TYPE_CASE:
            case Constants.MODEL_TYPE_CASE:
            case Constants.CLASSNAME_CASE:   
               return Constants.TYPE_CASE;
            //Test Script
            case Constants.TYPE_SCRIPT:
            case Constants.MODEL_TYPE_SCRIPT:
            case Constants.MODEL_TYPE_REMOTE_SCRIPT:
            case Constants.CLASSNAME_SCRIPT:
               return Constants.TYPE_SCRIPT;        
            case Constants.MODEL_TYPE_WORKITEM:
            case Constants.CLASSNAME_WORKITEM:
               return Constants.TYPE_WORKITEM;                       
            case Constants.MODEL_TYPE_EWI:
               return Constants.TYPE_EWI;
            case Constants.MODEL_TYPE_RESULT:
               return Constants.TYPE_RESULT;               
            case Constants.MODEL_TYPE_SUITE:
               return Constants.TYPE_SUITE;
			case Constants.MODEL_TYPE_REQUIREMENT:
               return Constants.TYPE_REQUIREMENT;
			case Constants.MODEL_TYPE_CONFIGURATION:
				return Constants.TYPE_CONFIGURATION;
			case Constants.MODEL_TYPE_BUILD_DEFINITION: 
			case Constants.MODEL_TYPE_BUILD_RECORD:
				return Constants.getMessage("labelBuild");
	        case Constants.MODEL_TYPE_LAB_REQUEST:
	            return Constants.getMessage("labelRequest");
	        case Constants.MODEL_TYPE_TEST_CELL:
	            return Constants.getMessage("labelTestCell");
	        case Constants.MODEL_TYPE_TEST_ENVIRONMENT:
	            return Constants.getMessage("labelTestEnvironment"); 
			case Constants.MODEL_TYPE_DATAPOOL:
				return Constants.getMessage("labelDatapool"); 
			case Constants.MODEL_TYPE_KEYWORD: 
				return Constants.getMessage("labelKeyword");
		    case Constants.MODEL_TYPE_RESOURCE_GROUP:
		        return Constants.getMessage("labelResGrp");
		    case Constants.MODEL_TYPE_EXECUTION_SEQUENCE:
		        return Constants.getMessage("labelExecutionSequence");
		    case Constants.MODEL_TYPE_LAB_RESOURCE:
		        return Constants.getMessage("labelLabRes");
			case Constants.CALM_LINK_TYPE_PLAN_ITEM:
			case Constants.CALM_LINK_TYPE_REQUIREMENT:
			case Constants.CALM_LINK_TYPE_REQUIREMENT_SET:
				return Constants.getMessage("labelCalmLinks");
            default:
               return Constants.MODEL_TYPE_UNKNOWN;
         }                
      },
        
        /******************************************************************************************
         * Get time as a string
         *****************************************************************************************/
        getTimeString: function(time) {
            var now = new Date();
            var option = {selector: "time", formatLength: "medium"}
            if(time)
                now.setTime(time);
            return dojo.date.locale.format(now, option);        
        },
        
        /******************************************************************************************
         * Get Date object from ISO string.  Need to wrap call to dojo.date.stamp.fromISOString.
         *****************************************************************************************/     
        dateFromISOString: function(timestamp) {
            //Example:  2003-10-06T06:46:00.000-0400
            // dojo.date.stamp.fromISOString wants:
            //          2005-06-30T08:05:00-07:00
            if(timestamp.length == 28) {
                timestamp = timestamp.substring(0,19) + "-" + timestamp.substring(24, 26) + ":" + timestamp.substring(26,28);
            }
            return dojo.date.stamp.fromISOString(timestamp);
        
        },      
    
        /******************************************************************************************
         * Function: formatDate
         * @param {Object} modified
         * @param {Object} isIso
         *****************************************************************************************/
        formatDate: function(modified, isIso)
        {
            var MONTH = [Constants.getMessage("labelJanuary"),
                         Constants.getMessage("labelFebruary"),
                         Constants.getMessage("labelMarch"),
                         Constants.getMessage("labelApril"),
                         Constants.getMessage("labelMay"),
                         Constants.getMessage("labelJune"),
                         Constants.getMessage("labelJuly"),
                         Constants.getMessage("labelAugust"),
                         Constants.getMessage("labelSeptember"),
                         Constants.getMessage("labelOctober"),
                         Constants.getMessage("labelNovember"),
                         Constants.getMessage("labelDecember")];

            isIso == null ? false : isIso;
            var now= new Date();
            if(isIso) {
                var modDate= dojo.date.fromIso8601(modified);
            } else {
                var modDate = modified;
            }
            var showRelative = com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getIsDateRelativeStyle();
            
            if (showRelative) {
                var deltaMinutes = dojo.date.difference(modDate, now, "minute");
                
                // if less than an hour ago, display in minutes
                if (deltaMinutes < 60) {
                    if (deltaMinutes === 0) {
                        deltaMinutes = 1;
                    }
                    return (deltaMinutes === 1) ? Constants.getCommonMessage("labelOneMinuteAgo") : Constants.getCommonMessage("labelMinutesAgo", deltaMinutes);
                }
                
                //          var curDayOfYear= dojo.date.getDayOfYear(now);
                //          var modDateDayOfYear= dojo.date.getDayOfYear(modDate);
                
                var yearOffset = dojo.date.difference(modDate, now, "year");
                
                //          var yearOffset= now.getYear() - modDate.getYear();
                //          var dayOffset= curDayOfYear - modDateDayOfYear;
                
                //          var totalDayOffset= dayOffset > 0 ?
                //                  yearOffset * 365 + dayOffset :
                //                  (yearOffset - 1) * 365 + curDayOfYear + 365 - modDateDayOfYear;
                
                var totalDayOffset = dojo.date.difference(modDate, now, "day");
                
                // if same day or less than 12 hours ago, display in hours
                if (totalDayOffset === 0 || deltaMinutes < 720) {
                    var hours = Math.round(deltaMinutes / 60);
                    return (hours === 1) ? Constants.getCommonMessage("labelOneHourAgo") : Constants.getCommonMessage("labelHoursAgo", hours);
                }
                
                // if previous calendar date, display "Yesterday"
                if (totalDayOffset === 1) {
                    var option = {
                        selector: "time",
                        formatLength: "medium"
                    }
                    var time = dojo.date.locale.format(modDate, option);
                    return Constants.getCommonMessage("labelYesterdayAt", time);
                }
            }
            var dateStyle = com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getDateFormatStyle();            
            return dojo.date.locale.format(modDate, dateStyle);
        },
        
      /******************************************************************************************
       * Function: formatDateAsLong
       * @param {Object} modifiedAsLong
       * @param {Object} isIso
       *****************************************************************************************/
      formatDateFromLong: function(modifiedAsLong, isIso){
         var date = new Date()
         date.setTime(modifiedAsLong);
            return this.formatDate(date, isIso);
      }         
    };
}
)();
