/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
var Utility = dojo.provide("com.ibm.asq.common.web.ui.internal.util.Utility");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dijit._Widget");

dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.team.process.web.client.projectarea");

Utility.PLANNING_PERSPECTIVE     = 1;
Utility.CONSTRUCTION_PERSPECTIVE = 2;
Utility.EXECUTION_PERSPECTIVE    = 3;

//dojo.require("dojo.date.common");
//dojo.require("dojo.date.serialize");
//dojo.require("dojo.date.format");

var MONTH = ["January",
			 "February",
			 "March",
			 "April",
			 "May",
			 "June",
			 "July",
			 "August",
			 "September",
			 "October",
			 "November",
			 "December"];

// added any new supported languages to this list
//
var SUPPORTED_LANG = [
    "en",
    "de",
    "es",
    "fr",
    "it",
    "ja",
    "ko",
    "pt",
    "zh",
    "ru"
];

// add any new supported languages of specific countries to this list
var SUPPORTED_COUNTRIES = [
    "en-us",
    "pt-br",
    "zh-hk",
    "zh-tw",
    "en-zz"
];

(function() 
{
	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var Constants  = com.ibm.asq.common.web.client.internal.Constants;
    var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
    var ACTION_PREFIX = "#action=com.ibm.rqm.planning.home.actionDispatcher";
    var _Widget = dijit._Widget.prototype;


    // Unassigned is a special user like ADMIN and GUEST.  The item id is fixed according to the file
    // com.ibm.team.workitem.common.internal.model.DefaultModel.java from where QMContributorRestService
    // gets the Unassigned user to send to the UI:
    //
    // public final static String NULL_CONTRIBUTOR_NAME= Messages.getString("DefaultModel.UNASSIGNED_CONTRIBUTOR_NAME"); //$NON-NLS-1$
    // public final static UUID NULL_CONTRIBUTOR_ITEM_ID= UUID.valueOf("_YNh4MOlsEdq4xpiOKg5hvA"); //$NON-NLS-1$
    var UNASSIGNED_ITEMID = "_YNh4MOlsEdq4xpiOKg5hvA";
	
	com.ibm.asq.common.web.ui.internal.util.Utility = 
	{
        _planningPrefix: ACTION_PREFIX + "&subAction=",
        _constructionPrefix: ACTION_PREFIX + "&subAction=",
        _executionPrefix: ACTION_PREFIX + "&subAction=",

		/******************************************************************************************
		 * Generic URI functions
		 * 
		 *****************************************************************************************/
		_getPlanningActionUri: function(action, kwArgs)
		{
			var str = this._planningPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getConstructionActionUri: function(action, kwArgs)
		{
			var str = this._constructionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getExecutionActionUri: function(action, kwArgs)
		{
			var str = this._executionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getActionUri: function(str, kwArgs) 
		{
		    for (var key in kwArgs) 
			{
	        	str += "&" + key + "=" + kwArgs[key];
	        }
			return str;
		},

		_buildUri: function(method, args, perspective)
		{
			var result;
			
			switch(perspective)
			{
				case Utility.PLANNING_PERSPECTIVE:
					result = this._getPlanningActionUri(method, args);
					break;
				case Utility.CONSTRUCTION_PERSPECTIVE:
					result = this._getConstructionActionUri(method, args);
					break;
				case Utility.EXECUTION_PERSPECTIVE:
					result = this._getExecutionActionUri(method, args);
					break;
			}
			
			return(result);
		},
		
		/******************************************************************************************
		 * Action URI methods
		 * 
		 * @param {Object} id
		 *****************************************************************************************/
		getGenericSearchUri: function(str)
		{
			return(this._getPlanningActionUri("genericSearch", {searchStr: str}));
		},

		getNewQueryUri: function()
		{
			return(this._getPlanningActionUri("newQuery"));
		},

		getViewWorkItemURI: function(id)
		{
			return this._getPlanningActionUri("viewWorkItem", {id: id});
		},
		     // DATAPOOL URLs
        getViewDatapoolURI: function(itemId)
        {
            return this._getPlanningActionUri("viewDatapool", {itemId: itemId});
        },
        
		getNewTestGroupURI: function(name) 
		{
			return this._getPlanningActionUri("newTestGroup", {name: name});
		},
	
		getTestGroupURI: function(groupId)
		{
			var args = {};
			if (groupId != null)
			{
				args = {id: groupId};
			}
			return this._getPlanningActionUri("viewTestGroup", args);
		},		
			
		getTestGroupsURI: function(queryName)
		{
			var args = {};
			if (queryName != null)
			{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestGroups", args);
		},
		
		getNewTestCaseURI: function(name) 
		{
			return this._getPlanningActionUri("newTestCase", {name: name});
		},
			
		getTestCaseURI: function(testId)
		{
			var args = {};
			if (testId != null)
			{
				args = {id: testId};
			}
			return this._getPlanningActionUri("viewTestCase", args);
		},
				
		getTestCasesURI: function(queryName)
		{
			var args = {};
			if (queryName != null)
			{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestCases", args);
		},
		
		getTestSuiteURI: function(suiteId, perspective){
			var args = {};
			args.id = suiteId;
			return this._buildUri("viewTestSuite", args, perspective ? perspective : Utility.CONSTRUCTION_PERSPECTIVE);
		},
		
		getNewTestScriptURI: function(name, perspective) 
		{
			if (perspective == null)
			{
				perspective = Utility.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("newTestScript", {name: name}, perspective));
		},
			
		getTestScriptURI: function(scriptId, perspective)
		{
			var args = {};
		
			if (scriptId != null)
			{
				args = {id: scriptId};
			}
		
			if (perspective == null)
			{
				perspective = Utility.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("viewTestScript", args, perspective));
		},
				
		getTestScriptsURI: function(queryName, perspective) 
		{
			var args = {};

			if (queryName != null) 
			{
				args = {name: queryName};
			}

			if (perspective == null)
			{
				perspective = Utility.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("viewTestScripts", args, perspective));
		},
		
		getNewTestPlanURI: function(name) 
		{
			return this._getPlanningActionUri("newTestPlan", {name: name});
		},
			
		getTestPlanURI: function(kwargs) 
		{
			var args = {};
			if ( kwargs.itemId != null )
			{
				args = {itemId: kwargs.itemId};
			}
			else if (kwargs.id != null)
			{
				args = {id: kwargs.id};
			}
			else if (kwargs.name != null)
			{
				args = {name: kwargs.name};
			}
			return this._getPlanningActionUri("viewTestPlan", args);
		},
				
		getTestPlansURI: function(queryName) 
		{
			var args = {};
			if (queryName != null) 
			{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestPlans", args);
		},
		
		getNewEwiURI: function(name, testCaseId) 
		{
			var testId = testCaseId != null ? testCaseId : null; 
			return this._getPlanningActionUri("newEwi", {name: name, testCaseId: testId});
		},
			
		getEwiURI: function(planName, perspective) 
		{
			var args = {};
			
			if (planName != null) 
			{
				args = {name: planName};
			}

			if (perspective == null)
			{
				perspective = Utility.PLANNING_PERSPECTIVE;
			}
			
			return(this._buildUri("viewEwi", args, perspective));
//			return this._getPlanningActionUri("viewEwi", args);
		},
				
		getEwisURI: function(queryName, perspective) 
		{
			var args = {};
			if (queryName != null) {
				args.name = queryName;
			}

			if (perspective == null)
			{
				perspective = Utility.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("viewEwis", args, perspective));
//			return this._getPlanningActionUri("viewEwis", args);
		},
		
		getSizingRecordURI: function(sizingRecordId) {
			var args = {};
			if (sizingRecordId != null)
			{
				args = {uid: sizingRecordId};
			}
			return this._getPlanningActionUri("viewTestSizing", args);
		},
		
		startExecution: function(ewi, scriptId)
		{
			var args = {};
			
			if (ewi != null)
			{
				args = {
				ewi: ewi,
				runScript: scriptId
				};
			}
			return this._getExecutionActionUri("startExecution", args);
		},
	
		executedItemsURI: function (queryName)
		{
			var args = {};			
			if (queryName != null) {
				args.name = queryName;
			}
			return this._getExecutionActionUri("viewExecutedItems", args);
		},
	
		getEWIResultsURI: function(ewiItemId)
		{
			var args = {};			
			if (ewiItemId != null) {
				args.ewiItemId = ewiItemId;
			}			
			return this._getExecutionActionUri("viewEWIResults", args);
		},
		
		showResultsURI: function(queryName)
		{
			var args = {};			
			if (queryName != null) {
				args.name = queryName;
			}			
			return this._getExecutionActionUri("viewResults", args);
		},
		
		getResultURI: function(args)
		{
			return this._getExecutionActionUri("viewResult", args );
		},
		
		getCurrentProjectAreaAlias: function()
		{
			var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			var userProjectAreas = dojo.getObject(Constants.USER_PROJECT_AREAS_OBJECT_ID);
			if (userProjectAreas!=null && userProjectAreas.length>0) {
				for (var i=0; i<userProjectAreas.length; i++) {
					var userProjectArea = userProjectAreas[i];
					if (userProjectArea.itemId == currentProjectArea.itemId) {
						return userProjectArea.alias;
					}
				}
			}
			console.error("failed to get the project area alias for current project area " + currentProjectArea.name);
			return null;
		},
		
		/******************************************************************************************
		 * Generate Unique URL
		 * 
		 * @param {Object} url
		 *****************************************************************************************/
		reuniquifyUrl: function(url)
		{
			return(url.replace(/&tq=([0-9]+)/, "") + "&tq=" + this.getUniqueId());
		},
		
		getUniqueId: function()
		{
			return Date.parse(new Date().toString());
		},

		/******************************************************************************************
		 * Set the window location to an action with keywords
	 	 *****************************************************************************************/
		setLocationToActionWithKeywords: function(action, kwArgs)
		{
			//TODO: fix this for all perspectives
			this._setLocationHash(this._getPlanningActionUri(action, kwArgs));
		},
	
		/******************************************************************************************
		 * Set the window location hash.
		 * This is pushed to a method so all changes to the URI can be traced if needed 
		 *****************************************************************************************/
		_setLocationHash: function(locationHash)
		{
			window.location.hash = locationHash;
		},

		/******************************************************************************************
		 * QSort functions
		 * 
		 * @param {Object} list
		 * @param {Object} a
		 * @param {Object} b
		 *****************************************************************************************/
		swap: function(list, a, b)
		{
			var tmp = list[a];
			list[a] = list[b];
			list[b] = tmp;
		},
		
		partition: function(array, begin, end, pivot)
		{
			var piv = array[pivot];
			this.swap(array, pivot, end-1);
			
			var store=begin;
	
			for(var ix = begin; ix < end-1; ++ix) 
			{
				if(array[ix] <= piv)
				{
					this.swap(array, store, ix);
					++store;
				}
			}
			
			this.swap(array, end-1, store);
		
			return store;
		},
		
		qsort: function(list, begin, end)
		{
			if(end-1>begin)
			{
				var pivot = begin+Math.floor(Math.random()*(end-begin));
	
				pivot = this.partition(list, begin, end, pivot);
	
				this.qsort(list, begin, pivot);
				this.qsort(list, pivot+1, end);
			}		
		},
		
		partition_obj: function(array, index, begin, end, pivot)
		{
			var piv = array[pivot].elements[index].data;
			if (typeof piv == "string")
			{
				piv = piv.toLowerCase();
			}
	
			this.swap(array, pivot, end-1);
			
			var store=begin;
	
			for(var ix = begin; ix < end-1; ++ix) 
			{
				data = array[ix].elements[index].data;
				if (typeof data == "string")
				{
					data = data.toLowerCase();
				}
				
				if(data <= piv)
				{
					this.swap(array, store, ix);
					++store;
				}
			}
			
			this.swap(array, end-1, store);
		
			return store;
		},
		
		qsort_obj: function(list, index, begin, end)
		{
			if(end-1>begin)
			{
				var pivot = begin+Math.floor(Math.random()*(end-begin));
	
				pivot = this.partition_obj(list, index, begin, end, pivot);
	
				this.qsort_obj(list, index, begin, pivot);
				this.qsort_obj(list, index, pivot+1, end);
			}		
		},
		
		/******************************************************************************************
		 * Get Icon/Image functions
		 *  
		 * @param {Object} path
		 *****************************************************************************************/
		getArtifactIconURL: function(artifactType)
		{
			switch(artifactType) {
				case Constants.TYPE_PLAN:
					return ImageCache.TEST_PLAN;
				case Constants.TYPE_GROUP:
					return ImageCache.TEST_GROUP;
				case Constants.TYPE_CASE:
					return ImageCache.TEST_CASE;
				case Constants.TYPE_SCRIPT:
					return ImageCache.TEST_SCRIPT;
				case Constants.TYPE_PLAN_SIZING:
					return ImageCache.TEST_PLAN;
				case Constants.TYPE_GROUP_SIZING:
					return ImageCache.TEST_GROUP;
				case Constants.TYPE_CASE_SIZING:
					return ImageCache.TEST_CASE;
				case Constants.TYPE_SCRIPT_SIZING:
					return ImageCache.TEST_SCRIPT;
				case Constants.TYPE_EWILIST:
					return ImageCache.EWI;	
				default: 
					return "";
					
			};		
		},
		
		/******************************************************************************************
		 * Returns the Constant representing the artifactType
		 *  
		 * @param {Object} artifact - the QM artifact DTO object instance 
		 *****************************************************************************************/		
		getArtifactType: function(artifact) {
			var fullType = artifact._eQualifiedClassName;
			var type = fullType.substr(fullType.lastIndexOf(':') + 1);
			
			switch(type) {
				case Constants.CLASSNAME_PLAN:
					return Constants.TYPE_PLAN;
				case Constants.CLASSNAME_GROUP:
					return Constants.TYPE_GROUP;
				case Constants.CLASSNAME_CASE:
					return Constants.TYPE_CASE;
				case Constants.CLASSNAME_SCRIPT:
					return Constants.TYPE_SCRIPT;
				case Constants.CLASSNAME_EWI:
					return Constants.TYPE_EWI;	
				case Constants.CLASSNAME_PLAN_SIZING:
					return Constants.TYPE_PLAN_SIZING;
				case Constants.CLASSNAME_GROUP_SIZING:
					return Constants.TYPE_GROUP_SIZING;
				case Constants.CLASSNAME_CASE_SIZING:
					return Constants.TYPE_CASE_SIZING;
				case Constants.CLASSNAME_SCRIPT_SIZING:
					return Constants.TYPE_SCRIPT_SIZING;
				default:
					return null;											
			}			
		},		
		
		/******************************************************************************************
		 * Returns the Constant representing the item's model type
		 *  
		 * @param {Object} item - the QM artifact DTO object instance 
		 *****************************************************************************************/		
		getModelType: function(item) {
			var fullType = item._eQualifiedClassName;
			var type = fullType.substr(fullType.lastIndexOf(':') + 1);
			
			switch (type) {
				case Constants.CLASSNAME_PLAN:
					return Constants.MODEL_TYPE_PLAN;
				case Constants.CLASSNAME_GROUP:
					return Constants.MODEL_TYPE_GROUP;
				case Constants.CLASSNAME_CASE:
					return Constants.MODEL_TYPE_CASE;
				case Constants.CLASSNAME_SCRIPT:
					return Constants.MODEL_TYPE_SCRIPT;
				default:
					return Constants.MODEL_TYPE_UNKNOWN;
			}											
		},
		
		
		/******************************************************************************************
		 * Get time as a string
		 *****************************************************************************************/
		getTimeString: function(time) {
			var now = new Date();
			var option = {selector: "time"};
			if(time)
				now.setTime(time);
			return dojo.date.locale.format(now, option);
		},
		
		/******************************************************************************************
		 * Get Date object from ISO string.  Need to wrap call to dojo.date.stamp.fromISOString.
		 *****************************************************************************************/		
		dateFromISOString: function(timestamp) {
			//Example:  2003-10-06T06:46:00.000-0400
			// dojo.date.stamp.fromISOString wants:
			//          2005-06-30T08:05:00-07:00
			if(timestamp.length == 28) {
				timestamp = timestamp.substring(0,19) + "-" + timestamp.substring(24, 26) + ":" + timestamp.substring(26,28);
			}
			return dojo.date.stamp.fromISOString(timestamp);
		
		},		
	
        /******************************************************************************************
         * Function: formatDate
         * @param {Object} modified
         * @param {Object} isIso
         *****************************************************************************************/
        formatDateFromLong: function(modified) {
            var d = new Date();
            d.setTime(modified);
            return this.formatDate(d, false);
        },
            
		/******************************************************************************************
		 * Function: formatDate
		 * @param {Object} modified
		 * @param {Object} isIso
		 *****************************************************************************************/
		formatDate: function(modified, isIso)
		{
			var now= new Date();
			if(isIso) {
				var modDate= dojo.date.fromIso8601(modified);
			} else {
				var modDate = modified;
			}
            var showRelative = com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getIsDateRelativeStyle();
            
            if (showRelative) {
                var deltaMinutes = dojo.date.difference(modDate, now, "minute");
                
                // if less than an hour ago, display in minutes
                if (deltaMinutes >= 0 && deltaMinutes < 60) {
                    if (deltaMinutes === 0) {
                        deltaMinutes = 1;
                    }
                    return (deltaMinutes === 1) ? Constants.getMessage("labelOneMinuteAgo") : Constants.getMessage("labelMinutesAgo", deltaMinutes);
                }
                
                //			var curDayOfYear= dojo.date.getDayOfYear(now);
                //			var modDateDayOfYear= dojo.date.getDayOfYear(modDate);
                
                var yearOffset = dojo.date.difference(modDate, now, "year");
                
                //			var yearOffset= now.getYear() - modDate.getYear();
                //			var dayOffset= curDayOfYear - modDateDayOfYear;
                
                //			var totalDayOffset= dayOffset > 0 ?
                //					yearOffset * 365 + dayOffset :
                //					(yearOffset - 1) * 365 + curDayOfYear + 365 - modDateDayOfYear;
                
                var totalDayOffset = dojo.date.difference(modDate, now, "day");
                
                // if same day or less than 12 hours ago, display in hours
                if (((totalDayOffset === 0) && (deltaMinutes >= 0)) || (deltaMinutes >= 0 && deltaMinutes < 720)) {
                    var hours = Math.round(deltaMinutes / 60);
                    return (hours === 1) ? Constants.getMessage("labelOneHourAgo") : Constants.getMessage("labelHoursAgo", hours);
                }
                
                // if previous calendar date, display "Yesterday"
                if (totalDayOffset === 1) {
					var option = {
						selector: "time",
						formatLength: "medium"
					}
					var time = dojo.date.locale.format(modDate, option);
                    return Constants.getMessage("labelYesterdayAt", time);
                }
            }
            var dateStyle = com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getDateFormatStyle();            
			return dojo.date.locale.format(modDate, dateStyle);
		},

        // all strings are in HTML format
        handleArtifactLoadError: function(type, reason) {
            var message = MessageUtil.getMessage(MessageBundle.COMMON, "msgLoadArtifactFailAndClose", type);
            if (reason) {
                message += "<br><br>" + MessageUtil.getMessage(MessageBundle.COMMON, "labelDetailLabel") + "<br>" + reason;
            }
            var args = {
                okCallback: function() {
                    dojo.publish(GlobalEvents.CloseTab, [window.location.hash]);
                },
                width:"40%",
                primaryTitle: MessageUtil.getMessage(MessageBundle.COMMON, "labelAlert"),
                message: message
            }
            var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(args);
            alertDialog.open(true);
        },
        
        // Summery:
        //    Returns true if the userId is the unassigned user.
        isUnassignedUserId: function(userId) {
            return userId == UNASSIGNED_ITEMID;
        },
		getUnassignedUserId:function(){
			return UNASSIGNED_ITEMID;
		},
        isCJKLocale: function() {
            return (dojo.locale.indexOf("zh") == 0 || dojo.locale.indexOf("ja") == 0 || dojo.locale.indexOf("ko") == 0);
        },
        
        // Summary:
        //    Return the client locale with default to English if the current locale
        //    is not one of the supported languages.
        //
        getSupportedLocale: function() {
            var languagesCount = SUPPORTED_LANG.length;
            var currentLocale = dojo.locale;
            // iterate through the list of supported language
            for (var i = 0; i < languagesCount; i++) {
                if (currentLocale.indexOf(SUPPORTED_LANG[i]) == 0) {
                    // it is a supported language, check if it is one of
                    // the country specific language
                    var countriesCount = SUPPORTED_COUNTRIES.length;
                    for (var j = 0; j < countriesCount; j++) {
                        if (currentLocale.indexOf(SUPPORTED_COUNTRIES[j]) == 0) {
                            // found exact match, return the locale
                            return currentLocale;
                        }
                    }
                    // it is a supported language but not country specific, return language code only
                    // except if it is English from other countries we default to en-us
                    // or Portuguese we only support from Brazil
                    var language = currentLocale.substring(0, 2);
                    if (language == "en") {
                        language = "en-us";
                    } else if (language == "pt") {
                        language = "pt-br";
                    }
                    return language;
                }
            }
            // it is not a supported language, default to English
            return "en-us";
        },
        
        // Summary:
        //    Given a dom node, find all dijit widgets that are currently attached to
        //    it and its children nodes, and destroy all found widgets. This is useful
        //    when an UI is updating its display by recreating certain part of the dom
        //    tree and ensures that no widget's leaked due to dynamic removal.
        // 
        destroyWidgetsFromNode: function(node) {
            var list = dojo.query('[widgetId]', node);
            var removeWidgets = list.map(dijit.byNode);
            this.destroyWidgetList(removeWidgets);
        },
        
        // Summary:
        //    Given an array of widgets, call destroy on the each of the widgets.
        //    This method handles Jazz widgets as well, if the widget has a 'destroyWidget'
        //    defined, it will call that instead of the plain destroy method. it also
        //    handles the case where certain Jazz widgets doesn't remove itself from 
        //    the dijit registry when it is destroyed.
        //
        //    NOTE: When Jazz clean up there widget destroy code, we might not need to do
        //    some of the extraneous handling to cleanup Jazz widgets.
        //
        destroyWidgetList: function(removeWidgets) {
            var i = removeWidgets.length;
            var registry = dijit.registry;
            
            while (i--) {
                var widget = removeWidgets[i];
                if (widget) {
                    var destroyed = false;
                    // handle Jazz widget destroy method. Call this if
                    // it exists.
                    if(widget.destroyWidget) {
                        widget.destroyWidget();
                        destroyed = true;
                        //console.debug("Utility.destroyWidgetList - destroyWidget widget: " + widget.id);
                    }
                    else if (widget.destroy) {
                        widget.destroy();
                        destroyed = true;
                        //console.debug("Utility.destroyWidgetList - destroying widget: " + widget.id);
                    }
                    // Jazz widgets may override the destroy method and never
                    // call the superclass's impl. This would leak in the dijit
                    // registry. Manually call the base _Widget class's destroy to
                    // properly dispose of the widget.
                    var wid = widget.id;
                    if(destroyed && registry.byId(wid)) {
                        //console.debug("Utility.destroyWidgetList - widget destroy NOT removed from dijit.registry! - " + wid);
                        _Widget.destroy.apply(widget, []);
                    }
                }
            }            
        },
        preUninitialize: function(cleanupWidgets) {
        	//var start = new Date();
            var i = cleanupWidgets.length;
            while (i--) {
                var widget = cleanupWidgets[i];
                if (widget && widget._magicPreUninitialize) {
                	widget._magicPreUninitialize();
                }
			}        
			//console.debug(">>>>>>>>>>>>>>preUninitialize time = "+((new Date())-start)/1000);
        },
        
        printException: function(className, methodName, e) {
            className = className || "undefined class";
            methodName = methodName || "undefined method";
            
            console.debug("Exception caught in " + className + "#" + methodName);
            if(e) {
                if (dojo.isObject(e)) {
                    for (var i in e) {
                        console.debug(i + ": " + e[i]);
                    }
                }
                else if(dojo.isString(e)) {
                    console.debug(e);
                }
            }
        }        
        
	};
}
)();
