/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.RelatedLinksContentArea");

dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.client.internal.option");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("jazz.util.html");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.RelatedLinksTableView");

(function(){
	
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var RelatedLinksTableView = com.ibm.rqm.planning.web.ui.internal.view.administration.RelatedLinksTableView;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var getCurrentProjectArea  = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
	var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.RelatedLinksContentArea", com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea, {
        
		constructor: function(){
			this.relatedLinks = [];			
			this.csHelpId = "rqm_admin_related_sites.html";
			
			//internal
			this._preference = null;
			this._optionsClient = new com.ibm.asq.common.web.client.internal.option();
		},
		
		postCreate: function(){
            this.inherited("postCreate", arguments);
			this.csHelp.setButtonLabel(Constants.getMessage("labelRelatedSitesHelp"));
        },
        
        setName: function(){
            this.name = Constants.getMessage("labelRelatedSites");
            dojox.data.dom.removeChildren(this._name);
            var name = document.createTextNode(this.name);
            this._name.appendChild(name);
        },
        
        setText: function(){
            this.text = Constants.getMessage("labelAdminRelatedSites");
            dojox.data.dom.removeChildren(this._text);
            var text = document.createTextNode(this.text);
            this._text.appendChild(text);
        },
        
        setContent: function(){
			this._showLinksTable();
			this._loadLinks();
        },
		
		_showLinksTable: function(){
			var args = {
				relatedLinks: [],
				onChangeListener: dojo.hitch(this, "_onLinkChange"),
				actions: this._getTableActions(),
                editable: this.isEditable(),
                enableRowSelection: this.isEditable()		
			}
			this._linksTable = new RelatedLinksTableView(args);			
			this._content.appendChild(this._linksTable.domNode);
		},
		
		_loadLinks: function() {
			var args = {
				projectAreaItemId: getCurrentProjectArea().itemId,
				key: Constants.RELATED_LINKS_PREFERENCE_KEY
			}
			var handler = new ServiceResponseHandler(this, "_setPreference", "_error");
			this._optionsClient.getProjectPreferences(handler, args.projectAreaItemId, args.key);
		},
		
		_setPreference: function(preference) {
            if (preference != null) {
                this._preference = preference;
                this.relatedLinks = dojo.fromJson(preference.value);
            } else {
                this.relatedLinks = Constants.RELATED_LINKS_PREFERENCE_DEFAULTS.concat([]);                     
            }
            this._sanitizeRelatedLinks();
            this.setDirty(false);
            this._refreshTable();
			dojo.publish(GlobalEvents.RelatedSitesUpdated, []);			
		},
		
		_refreshTable: function() {
			this._linksTable.refresh(this.relatedLinks);
		},
		
        //actions to be displayed in the table header
        _getTableActions: function(){
            var actions = [];            
            actions.push({
				id: "add",
                label: Constants.getMessage("labelAddRelatedSites"),
                actionHandler: dojo.hitch(this, "_handleAdd"),
                iconClass: "add-icon-image",
				requiresSelection: false
            });

            actions.push({
				id: "remove",
                label: Constants.getMessage("labelRemoveRelatedSites"),
                actionHandler: dojo.hitch(this, "_handleRemove"),
                iconClass: "remove-enabled-icon-image",
                requiresSelection: true
            });
			
            actions.push({
				id: "moveup",
                label: Constants.getMessage("labelMoveUp"),
                actionHandler: dojo.hitch(this, "_handleMoveUp"),
                iconClass: "move-up-image",
                requiresSelection: true,
				requiresSingleSelection: true
            });
			
            actions.push({
				id: "movedown",
                label: Constants.getMessage("labelMoveDown"),
                actionHandler: dojo.hitch(this, "_handleMoveDown"),
                iconClass: "move-down-image",
                requiresSelection: true,
                requiresSingleSelection: true
            });
            return actions;
        },
		
		_handleAdd: function() {
			var nextId = this.relatedLinks.length + 1; 
			var newLink = {
				id: nextId,
				name: "",
				url: ""
			}
			this.relatedLinks.push(newLink);
			this._refreshTable();
			this.setDirty(true);
		},
		
		_handleRemove: function() {
			var updated = [];
			var selected = this._getSelected();
			if (!selected || selected.length == 0) {
				return;
			}				
			for (var i = 0; i < this.relatedLinks.length; i++) {
				var link = this.relatedLinks[i];
				if (!this._contains(selected, link)) {
					link.id = i + 1;
					updated.push(link);
				}
			}
			this.relatedLinks = updated;
			this._updateLinkIds();
			this._refreshTable();
			this.setDirty(true);
		},
		
        
        _handleMoveUp: function() {    
            this._move(-1);
			this.setDirty(true);
        },
        
        _handleMoveDown: function() {
            this._move(+1);
			this.setDirty(true);
        },		
		
		_move: function(moveBy) {
            var selected = this._getSelected();
            if (!selected || selected.length == 0) {
                return;
            }
			
            var origIndex = selected[0].id - 1;            
            var newIndex = origIndex + moveBy;
			if (newIndex >= 0 && newIndex < this.relatedLinks.length) {
				this.relatedLinks.splice(origIndex, 1);
                this.relatedLinks.splice(newIndex, 0, selected[0]);  
				this._updateLinkIds();
				this._refreshTable();
				this._linksTable.setRowChecked(selected[0], true);
			}            		
		},
		
		_contains: function(links, link) {
            for (var i = 0; i < (links ? links.length : 0); i++) {
                if (links[i].id == link.id) {
					return true;
				}
            }			
			return false;
		},
		
		_updateLinkIds: function() {
            for (var i = 0; i < this.relatedLinks.length; i++) {
                this.relatedLinks[i].id = i + 1;
            }			
		},
		
		_getSelected: function() {
            var selected = this._linksTable ? this._linksTable.getSelectedItems() : [];
            if (!selected || selected.length == 0) {
                this._displayAlertDialog("No Selection", "Please select a link to remove.");
                return false;
            }	
			return selected;		
		},	
		
		_onLinkChange: function(rowData) {
			this.setDirty(true);
			for (var i = 0; i < this.relatedLinks.length; i++) {
				if (this.relatedLinks[i].id == (rowData.id)) {
					this.relatedLinks[i] = rowData;
				} 
			}
		},
		
        _sortLinks: function(link1, link2) {
            if (link1.id < link2.id) {
                return -1;
            } else if (link1.id > link2.id) {
                return 1;
            }
            return 0;
        },	
        
        _sanitizeRelatedLinks: function() {
           var relatedLinks =  this.relatedLinks;
           
           for (var i = 0; i < this.relatedLinks.length; i++) {
                if(relatedLinks[i].name){
                    relatedLinks[i].name = jazz.util.html.removeScript(relatedLinks[i].name);
                }
            }
           
           this.relatedLinks = relatedLinks;
        },	
		
		_displayAlertDialog: function(title, message){
            var alertDialog = new AlertDialog({
                'message': message
            });
            alertDialog.setSecondaryTitle(title);
            alertDialog.open();
        },
        
        getValues: function(){
            return [];
        },
			
		onValidate: function() {
            var result = {
				message: "",     
				pass: false
			}
            for (var i = 0; i < this.relatedLinks.length; i++) {
                var link = this.relatedLinks[i];
                if (link.name == null || dojo.trim(link.name) == 0) {                    
                    if(result.message != ""){
                      result.message = result.message + "<br>";  
                    }
                    result.message = result.message + Constants.getMessage("msgRequiredField", Constants.getMessage("labelName"), Constants.getMessage("labelRelatedSite") + " #" + link.id);
                } if (link.url == null || dojo.trim(link.url) == 0) {
                    if(result.message != ""){
                      result.message = result.message + "<br>";  
                    }
                    result.message = result.message + Constants.getMessage("msgRequiredField", Constants.getMessage("labelURL"), Constants.getMessage("labelRelatedSite") + " #" + link.id);
                }
            }
            if (result.message != "") {                
                result.pass = false;
            } else {
                result.pass = true;
            }            	
			return result;		
		},
		
		onSave: function(){
            if (this.isDirty()) {
				this._doSave();          
            }
		},
		
        _doSave: function(){
			this.parent.addSaveParticipantNum();
			
            this.relatedLinks.sort(this._sortLinks);
            this._sanitizeRelatedLinks();
            var key = Constants.RELATED_LINKS_PREFERENCE_KEY;
            var value = dojo.toJson(this.relatedLinks, true, "   ");			

            var areaItemId = getCurrentProjectArea().itemId;
			var self = this;
			var handler = {
				success: function(results){
				    self.setDirty(false); 
					self.parent.saveFinished();
					self._setPreference(results);   
				},
				error: function(e){
				    self._saveError(e);
					self.parent.saveFinished();   							
				}
			}
		
            var responsehandler = new ServiceResponseHandler(handler, "success", "error");
            if (this._preference != null) {
                this._optionsClient.updateProjectPreferences(responsehandler, areaItemId, this._preference.itemId, value);
            } else {
                this._optionsClient.createProjectPreferences(responsehandler, areaItemId, key, value);
            }	           
        }
		
    });
    
})();
