/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.ReportServerTableViewer");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");


(function() {

var TableConstants 		= com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var Utility        		= com.ibm.rqm.planning.web.ui.internal.util.Utility;
var OptionsDialog    	= com.ibm.asq.common.web.ui.internal.widgets.OptionsDialog;
var Constants      		= com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("ReportServerTableViewer.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
	// Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        var selected = tableViewer.getCheckedRows();
		
        //enable/disable actions based on state
        var actionButtons = tableViewer.getActionButtons();
        for (var i = 0; i < actionButtons.length; i++) {
            var actionObj = actionButtons[i].actionObject;
            if (actionObj) {
				if (actionObj.requiresSelection == true) {
					actionButtons[i].setDisabled(!state && !(selected && selected.length > 0));
				} else if (actionObj.requiresSingleSelection == true) {
				    actionButtons[i].setDisabled(!state && !(selected && selected.length != 1));
				}
            }
        }       
    },
    
    // Summary:
    //    A group check box is shown when the tables rows are checkable and the in multi-select mode.
    //    This method is called when the group's check box is checked/unchecked. The affected rows and 
    //    their states are passed.
    //
    groupRowsCheckStateChanged: function(tableViewer, rowObjects, state) 
    {
        this.rowCheckStateChanged(tableViewer, rowObjects, state);
    },
	
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {                
        if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null) {
                callback(callbackData);
            } else {
                callback();
            }
        } 
    }     
});

dojo.declare("ReportServerTableViewer.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    constructor: function(args)   {	
        dojo.mixin(this, args);    
		//expected params
		//this.onChangeListener   
		//this.provider 
    },
			
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
        //update the row object
		var profile = this.provider.getProfile();
		if (profile) {
			var colName = profile[columnIndex].name;
			switch (colName) {
				case this.provider.COLUMNS.ADDRESS.name:
					rowObject.address = newValue;
					break;
				case this.provider.COLUMNS.USER.name:
					rowObject.user = newValue;
					break;
				case this.provider.COLUMNS.PASS.name:
					rowObject.pass = newValue;
					break;
				case this.provider.COLUMNS.NAMESPACE.name:
					rowObject.namespace = newValue;
					break;
				default:
					break;
			}
		}
		
		//invoke listener, if present
		if (dojo.isFunction(this.onChangeListener)) {
			this.onChangeListener(rowObject);
		}
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		//no-op
    }
});

dojo.declare("ReportServerTableViewer.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
        
        var moreContents = document.createElement("div");
        var toggle = document.createElement("button");
        toggle.innerHTML = "Added Custom Control";
        moreContents.appendChild(toggle);
        listWidget.contentNode.appendChild(moreContents);
    }
});

dojo.declare("ReportServerTableViewer.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    constructor: function(args) {
        dojo.mixin(this, args);		
        this.COLUMNS = {
            ADDRESS: this.toColumn(Constants.getMessage("labelURL"), 40, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_EDITABLE_TEXT),
            USER: this.toColumn(Constants.getMessage("labelReportServerUsername"), 20, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_EDITABLE_TEXT),
            PASS: this.toColumn(Constants.getMessage("labelReportServerPassword"), 20, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_PASSWORD),
            NAMESPACE: this.toColumn(Constants.getMessage("labelReportServerNamespace"), 20, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_EDITABLE_TEXT)
        }
		
        this.COLUMNS.ALL = [this.COLUMNS.ADDRESS, this.COLUMNS.USER, this.COLUMNS.PASS, this.COLUMNS.NAMESPACE];
		
        this.columnProfile = null;
        this.columnProfileName = null;
        this.setProfile(this.profile ? this.profile : "ALL"); //default to 'ALL' (full) column profile 	
    },
    
    toColumn: function(name, width, groupable, alignment, type) {
        return {
            name: name,
            width: width,
            groupable: groupable,
            alignment: alignment,
            type: type
        }
    },  
	
    setProfile: function(/* string */ profile) {
        if (this.COLUMNS[profile]) {
            this.columnProfileName = profile;
            this.columnProfile = this.COLUMNS[profile];
        }
    },	
	
    getProfile: function() {
        return this.columnProfile;
    },	

    getColumnIndex: function(name) {
        for (var i = 0; i < this.columnProfile.length; i++) {
            if (this.columnProfile[i].name == name) {
                return i;
            }
        }
        return -1;
    },	
			
    // Summary:
    //
    getNumColumns: function(content)
    {
        return this.columnProfile.length;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        return this.columnProfile[index].name;
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {  
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].alignment;
    },   
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return false;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return false;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        var column = this.columnProfile[atColumnIndex];
        return column.width;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_relatedLink_" + rowData.address;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].alignment;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "";
        var colName = this.columnProfile[atColumnIndex].name;
        switch(colName) {
            case this.COLUMNS.ADDRESS.name: 
                val = rowData.address;
                break;
            case this.COLUMNS.USER.name: 
                val = rowData.user;
                break;  
            case this.COLUMNS.PASS.name: 
                val = rowData.pass;
                break; 
            case this.COLUMNS.NAMESPACE.name: 
                val = rowData.namespace;
                break; 
            default:
                val = "";
                break;            
        }
        return val;		
    },

    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = {};
        var colName = this.columnProfile[atColumnIndex].name;
        switch(colName) {
            case this.COLUMNS.ADDRESS.name:
				break
            case this.COLUMNS.USER.name: 
                break;
            case this.COLUMNS.PASS.name: 
                break;
            default:
                break;			        
        }
        return val; 		
    }
    
    
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.ReportServerTableViewer", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
                    '  <div>' +
                    '    <div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>' +
                    '  </div>' +
				    '</div>',

    editable: true,

    constructor: function() {
	 	this.relatedLinks = [];
		this.onChangeListener = null;
        this.showInitLoadProgress = false;
		this.actionButtons = [];
	 },

    postCreate: function() {
        this.inherited("postCreate", arguments);		

        var content = this.relatedLinks ? this.relatedLinks : [];
				
        //create table viewer       
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();		      
        this._tableViewerNode.appendChild(tableViewer.domNode);
        tableViewer.setContentPageSize(10);
        tableViewer.setContent(content);
		tableViewer.setTableRowsCheckable(false);
        this.provider = new ReportServerTableViewer.TableProvider({
            profile: Constants.TABLE_PROFILE_ALL     
        }); 		
	
        tableViewer.setContentProvider(this.provider);
        tableViewer.setTableListener(new ReportServerTableViewer.TableListener(this));
        tableViewer.setCellModifier(new ReportServerTableViewer.CellModifier({
			provider: this.provider,
			onChangeListener: this.onChangeListener
		}));       
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelRelatedSites"))); 
        
		if (this.enableRowSelection) {
            tableViewer.setTableRowsCheckable(this.editable && !this.printable, /* isSingleSelect */ false);
        }        

        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            var action = this.actions[i];
            var button = tableViewer.addAction(action.id, action.label, null, action.iconClass, action.actionHandler);
			this.actionButtons.push(button);
            if (action.requiresSelection && this.enableRowSelection) {
                button.setDisabled(true);
                tableViewer.setTableRowsCheckable(true, /* isSingleSelect */ true);
            }
            button.actionObject = action;
        }
       		  
		this.tableViewer = tableViewer;
        if(this.showInitLoadProgress) {
            tableViewer.showLoadingProgress();
        }
        this.tableViewer.show();
        this.tableViewer.setEditable(this.editable);		  
    },
	
	// API
	getSelectedItems: function() {
        return this.tableViewer.getCheckedRows();
	},
	 
    setRowChecked: function(row, state) {
        this.tableViewer.rowCheckedHandler.setRowCheckState(row, state);
    },	 
     
    refresh: function(content, skipClearStatus) {
        
	    this.relatedLinks = content ? content : [];
         var cancelProgress = this.showInitLoadProgress;
        
         // we don't create the table until we fetched the categories.
         // The table may not have existed at this point. The real data
         // has arrived. Always reset the init load progress flag. 
         if(this.showInitLoadProgress) {
             this.showInitLoadProgress = false;
         }
         if (this.tableViewer) {
            if(cancelProgress) {
                this.tableViewer.endLoadingProgress();
            }
		 	this.tableViewer.setContent(content);
		 	this.tableViewer.refresh();
		 }
         if (this.viewHeaderNode && !skipClearStatus) {
             this.viewHeaderNode.clearStatus();
         }
		 
		 // refresh the action buttons status
		 this._refreshActionStatus();
     },
	 
	 _refreshActionStatus: function(){
	 	for(var i in this.actionButtons){
			if (this.actions[i].requiresSelection && this.enableRowSelection) 
					this.actionButtons[i].setDisabled(true);
		}
	 }     
});

})();
