/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.ReqproHostnameContentArea");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.RequirementConnectionTableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function(){
	
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var RequirementConnectionTableViewer = com.ibm.rqm.planning.web.ui.internal.view.administration.RequirementConnectionTableViewer;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.ReqproHostnameContentArea", com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea, {
//        templateString: '<div>' +
//        '<table>' +
//        '<tr><td class="content-name"><span><span dojoAttachPoint="_name"></span></td></tr>' +
//        '<tr><td class="content-text"><span><span dojoAttachPoint="_text"></span><span dojoAttachPoint="_cshelp"></span></td></tr>' +
//        '<tr><td><div dojoAttachPoint="_content">' +
//        '</div></td></tr>' +
//        '</table>' +
//        '</div>',
        
		constructor: function(){
			this.csHelpId = "rqm_admin_reqpro.html";
		},
		
		postCreate: function(){
            this.inherited("postCreate", arguments);
			this.csHelp.setButtonLabel(Constants.getMessage("labelRequisiteProHostPropertiesHelp"));
        },
        
        setName: function(){
            this.name = Constants.getMessage("labelRequisiteProHostProperties");
            dojox.data.dom.removeChildren(this._name);
            var name = document.createTextNode(this.name);
            this._name.appendChild(name);
        },
        
        setText: function(){
            this.text = Constants.getMessage("labelSetRequisiteProHostProperties");
            dojox.data.dom.removeChildren(this._text);
            var text = document.createTextNode(this.text);
            this._text.appendChild(text);
        },
        
        setContent: function(){
			this._initConnectionTable();
			this.loadConnections();
        },
		
		_initConnectionTable: function(){
			var args = {
				content: [],
				parent: this
			}
			this.requirementConnectionTableViewer = new RequirementConnectionTableViewer(args);
			var requirementConnectionTableViewer = this.requirementConnectionTableViewer;
			requirementConnectionTableViewer.initialize(args)
			this._content.appendChild(requirementConnectionTableViewer.domNode);
		},
		
		loadConnections: function(){
			var self = this;
			var handler= {
				success: function(connections){
					if(connections){
						console.debug(connections.length);
						self._fillConnectionTable(connections)
					}
					self.setDirty(false);
				},
				error: function(e){
					console.error(e);
				}
			}
			var responseHandler = new ServiceResponseHandler(handler, "success", "error");
            com.ibm.rqm.planning.web.client.internal.PreferenceClient.getRequirementConnections(responseHandler, "");
		},
		
		_fillConnectionTable: function(connections){
			this.requirementConnectionTableViewer.refresh(connections);
		},
		
		deleteConnection: function(connections){
			var self = this;
			this.deletedConnectionAmount = 0;
			this.connectionsAmount = connections.length
			for(var i = 0; i < connections.length; i++){
				self.deletedConnectionAmount++;
				if(!(connections[i].itemId < 0)){
					var handler = {
						success: function(arg){
							if(self.deletedConnectionAmount == self.connectionsAmount){
								self.requirementConnectionTableViewer.removeSelectedConnections();
							}
							self.setDirty(false);
						},
						error: function(e){
							console.error(e);
						}
					}
					var responseHandler = new ServiceResponseHandler(handler, "success", "error");
		            com.ibm.rqm.planning.web.client.internal.PreferenceClient.getDeleteRequirmentConnection(responseHandler, connections[i].id);
				}else if((self.deletedConnectionAmount == self.connectionsAmount) && connections[i].itemId < 0){
					this.requirementConnectionTableViewer.removeSelectedConnections();
					self.setDirty(false);
				}
			}
		},
		
		testConnection: function(connection){
			var self = this;
			var handler = {
				success: function(arg){
					if(arg && arg == "true"){
						self._displayAlertDialog("", Constants.getMessage("msgReqProConnAvailable"));
					}else{
						self._displayAlertDialog("", Constants.getMessage("msgReqProConnUnavailable"));
					}
					
				},
				error: function(e){
					alert(e.message);
				}
			}
			var responseHandler = new ServiceResponseHandler(handler, "success", "error");
            com.ibm.rqm.planning.web.client.internal.PreferenceClient.getIsValidConnection(responseHandler, connection.hostname, connection.port);
		},
		
		saveConnection: function(connections){
			var self = this;
			this.successStatus = true;
			this.savedConnectionAmount = 0;
			this.connectionsAmount = connections.length;
			if(connections.length == 0){
				return;
			}
			
			this.parent.addSaveParticipantNum();
			for(var i = 0; i < connections.length; i++){
				var handler = {
					success: function(connect){
						if(connect){
							self.successStatus = self.successStatus && true;
							self.savedConnectionAmount++;
						}
						if(self.savedConnectionAmount == self.connectionsAmount && self.successStatus == true){
							self.loadConnections();
							self.setDirty(false);
							self.parent.saveFinished();
						}else if (self.savedConnectionAmount == self.connectionsAmount && self.successStatus == false){
							self.parent.saveFinished();
							self.setSaveErrorMessage(Constants.getMessage("msgReqProFailSaveConn"));  
						}
					},
					error: function(e){
						self.successStatus = false;
						self.savedConnectionAmount++;
						if(self.savedConnectionAmount == self.connectionsAmount){
							self.setSaveErrorMessage(Constants.getMessage("msgReqProFailSaveConn"));   
							self.parent.saveFinished();
						}
					}
				}
				var responseHandler = new ServiceResponseHandler(handler, "success", "error");
	            com.ibm.rqm.planning.web.client.internal.PreferenceClient.postRequirementConnection(responseHandler, connections[i].hostname, "ReqPro", connections[i].hostname, connections[i].port, true);
			}
		},
		
		stopSynchronization: function(connection){
			var self = this;
			var handler = {
				success: function(a){
					self.loadConnections();
				},
				error: function(e){
					self._displayAlertDialog("", Constants.getMessage("msgReqProFailSyn", connection.name));
				}
			}
			var responseHandler = new ServiceResponseHandler(handler, "success", "error");
        	com.ibm.rqm.planning.web.client.internal.PreferenceClient.postStopSynchronize(responseHandler, connection.id, !connection.syncStopped);
		},
		
		startSynchronization: function(connection) {
			var self = this;
			var handler = {
				success: function(ret) {
					if(ret && ret=="true")
						self._displayAlertDialog("", Constants.getMessage("msgReqProSyncSuccess", connection.name));
					else 
						self._displayAlertDialog("", Constants.getMessage("msgReqProSyncFailed", connection.name));
				},
				error: function(e) {
					self._displayAlertDialog("", e.message);
				}
			}
			var responseHandler = new ServiceResponseHandler(handler, "success", "error");
			com.ibm.rqm.planning.web.client.internal.PreferenceClient.postStartSynchronize(responseHandler, connection.id);
		},
		
		checkSyncStatus: function(connection){
			var self = this;
			var handler = {
				success: function(status) {
					if (status) {
						if (status == "true") 
							self._displayAlertDialog("", Constants.getMessage("msgReqProSyncStatusSuccess", connection.name));
						else 
							if (status == "false") 
								self._displayAlertDialog("", Constants.getMessage("msgReqProSyncStatusFailed", connection.name));
							else 
								if (status == "InProgress") 
									self._displayAlertDialog("", Constants.getMessage("msgReqProSyncStatusInProgress", connection.name));
								else 
									self._displayAlertDialog("", Constants.getMessage("msgReqProSyncStatusNotDone", connection.name));
					}
				},
				error: function(e) {
				}
			}
			var responseHandler = new ServiceResponseHandler(handler, "success", "error");
			com.ibm.rqm.planning.web.client.internal.PreferenceClient.getSyncStatus(responseHandler, connection.id);
		},
		
		startResetSynchronizationTimeStamp: function(connection){
			var self = this;
			var handler = {
				success: function(ret) {
					if(ret && ret=="true")
						self._displayAlertDialog("", Constants.getMessage("msgReqProAllSyncSuccess", connection.name));
					else 
						self._displayAlertDialog("", Constants.getMessage("msgReqProSyncFailed", connection.name));
				},
				error: function(e) {
					self._displayAlertDialog("", e.message);
				}
			}
			var responseHandler = new ServiceResponseHandler(handler, "success", "error");
			com.ibm.rqm.planning.web.client.internal.PreferenceClient.postResetSynchronizeTimeStamp(responseHandler, connection.id);
		},
		
		_displayAlertDialog: function(title, message){
            var alertDialog = new AlertDialog({
                'message': message
            });
            alertDialog.setSecondaryTitle(title);
            alertDialog.open();
        },
        
        getValues: function(){
            var properties = [];
			return properties;
        },
		
		onValidate: function(){
			var result ={
				pass: true,
				message: ""
			}
			var message = this.requirementConnectionTableViewer.validateNewConnections();
			if(message.length > 0){
				result.pass = false;
				result.message = message;
			}
			return result;
		},
		
		onSave: function(){
			this.requirementConnectionTableViewer._handleSaveClick();
		}
    });
    
})();
