/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.ResourceEditorContentArea");

dojo.require("com.ibm.asq.common.web.client.internal.option");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea");
dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.PreferenceResourceTableView");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() { // module container 

var ServiceResponseHandler			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var PreferenceClient				= com.ibm.rqm.planning.web.client.internal.PreferenceClient;
var getCurrentProjectArea			= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var ImageCache 						= com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
var PreferenceResourceTableViewer 	= com.ibm.rqm.planning.web.ui.internal.view.administration.PreferenceResourceTableView;
var Constants 						= com.ibm.rqm.planning.web.client.internal.Constants;
var KEY 							= Constants.SHARED_RESOURCES_LOCATION_CATEGORY;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.ResourceEditorContentArea", com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea, {
	
//	templateString: '<div>' +
//						'<div dojoAttachPoint="_content">' +
//				        '</div>'+
//						'<span dojoAttachPoint="_cshelp"></span>'+
//			        '</div>',
	
	rowCounter: 0,
	constructor: function(){
			this.csHelpId = "rqm_admin_shared_res_loc.html";
			this.resourceNodes = [];
			this._itemType = Constants.getMessage("labelResources");
			this.locations = [];
	},
	
	postCreate: function(){
            this.inherited("postCreate", arguments);
			this.csHelp.setButtonLabel(Constants.getMessage("labelSharedResourceLocationsHelp"));
        },
	
	setName: function(){
		this.name = Constants.getMessage("labelResources");
        dojox.data.dom.removeChildren(this._name);
        var name = document.createTextNode(this.name);
        this._name.appendChild(name);
    },
    
    setText: function(){
		this.text = Constants.getMessage("labelSetResourcesProperties");
        dojox.data.dom.removeChildren(this._text);
        var text = document.createTextNode(this.text);
        this._text.appendChild(text);
    },
    
    setContent: function(){
		this._getSharedLocations();
    },
	
	_getSharedLocations: function(){
		var option = new com.ibm.asq.common.web.client.internal.option();
		var responsehandler = new ServiceResponseHandler(this, "_fillSharedLocations", "_error")
		option.getProjectPreferences(responsehandler, getCurrentProjectArea().itemId, KEY);
	},
    
	_fillSharedLocations: function(prefEntry) {
		this.prefEntry = prefEntry;			
		var locations = prefEntry ? dojo.fromJson(prefEntry.value) : [];
		this.locations = locations;
		this._initializeTable();
		this.setDirty(false);
	},
	
	_initializeTable: function(){
		var locs = []
		var locations = this.locations;
		if(locations){
			var length = locations.length;
			if(length > 0){
				for(var i=0;i<length;i++){
					locs.push({id: i, location: locations[i]});
				}
			}
		}	
		if(! this._resourceTable){
			var args ={
				parent: this,
				locations: locs,
//				queryName: Constants.getMessage("labelSharedLocations"),
				cshelpId: this.csHelpId,
				actions: this.getTableActions()
			}
			this._resourceTable = new PreferenceResourceTableViewer(args)
			this._content.appendChild(this._resourceTable.domNode);	
		}else{
			this._resourceTable.refresh(locs);
		}
	},
	
	//actions to be displayed in the table header
    getTableActions: function(){
        var actions = [];
		
        actions.push({
            label: Constants.getMessage("labelAddResourceLocation"),
            actionHandler: dojo.hitch(this, "_handleAddClick"),
            iconClass: "add-icon-image",
            disabledOnReadOnly: true                 
        });
        
        actions.push({
            label: Constants.getMessage("labelRemoveResourceLocation"),
            actionHandler: dojo.hitch(this, "_handleRemoveClick"),
            iconClass: "remove-enabled-icon-image",
            disabledOnReadOnly: true                 
        });
		
//		actions.push({
//            label: Constants.getMessage("labelSaveResourceLocation"),
//            actionHandler: dojo.hitch(this, "_handleSaveClick"),
//            iconClass: "save-icon-image",
//            disabledOnReadOnly: true                 
//        });
		
		actions.push({
            label: Constants.getMessage("labelRefresh"),
            actionHandler: dojo.hitch(this, "_getSharedLocations"),
            iconClass: "refresh-icon-image",
            disabledOnReadOnly: true                 
        });
                         
        return actions;
    },
	
	_handleSaveClick: function(){
		var option = new com.ibm.asq.common.web.client.internal.option();
		
		this.parent.addSaveParticipantNum();
		
		var self = this;
		var handler = {
			success: function(results){
			    self.setDirty(false);
				self.parent.saveFinished();
				self._fillSharedLocations(results);
			},
			error: function(e){
			    self._saveError(e); 
				self.parent.saveFinished();
			}
		}
				
		var responsehandler = new ServiceResponseHandler(handler, "success", "error");
		var res = [];
		var locations = this._resourceTable.getContent();
		var length = locations.length;
		for(var i=0;i<length ;i++){
			var location = locations[i].location;
 			if(location != "")
				res.push(location);
		}
		if(this.prefEntry){
			option.updateProjectPreferences(responsehandler, getCurrentProjectArea().itemId, this.prefEntry.itemId, dojo.toJson(res));
		}else{
			option.createProjectPreferences(responsehandler, getCurrentProjectArea().itemId, KEY, dojo.toJson(res));
		}
	},
	
	_handleAddClick: function(){
		var obj = {id: this.locations.length, location: ""};
		this.locations.push("");
		this._resourceTable.appendRow(obj);
		this._resourceTable.refreshActionStatus();	
	},
	
	_handleRemoveClick: function() {
		var updatedLocs =[]
		var locs = this._resourceTable.getContent();
		var length = locs.length;
			for(var i=0;i<length ;i++){
				updatedLocs.push(locs[i].location);
		}
		this.locations = updatedLocs;
		this._removeItems(this._resourceTable.getSelectedItems());
    },      

	_removeItems: function(items) {            
        if ( !items || items.length == 0) {
            alert(Constants.getMessage("msgRemoveResource", this._itemType));
            return;
        }
        var confirmMsg = "";
        if (items.length > 1) {
            confirmMsg = Constants.getMessage("msgRemoveResource1", items.length, this._itemType);
        } else {
            confirmMsg = Constants.getMessage("msgRemoveResource2", this._itemType, items[0].location);
        }
		var self = this;
		var args = {
			message: confirmMsg,
			yesCallback: function(){
				var length = items ? items.length : 0;
				for (var i = 0; i < length; i++) {
            		self._removeItem(items[i]);    
        		}
        		self._initializeTable();
				self.setDirty(true);
				return true;          
			},
			noCallback: function() {
				return;
			}
		}
		var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
        dialog.setSecondaryTitle(Constants.getMessage("labelConfirmation"));
		dialog.open();
    },
	
	_removeItem: function(item) {       
        var updatedItems = [];        
        var items = this.locations; 
		var length = items.length; 
        for (var i = 0; i < length; i++) {
			if(items[i] == item.location){
				this.locations.splice(i, 1);
				break;
			}	
        }
    },
	
    getValues: function(){
		var res = [];
		var locations = this._resourceTable.getContent();
		var length = locations.length;
		for(var i=0;i<length ;i++){
			var location = locations[i].location;
 			if(location != "")
				res.push(location);
		}
    	return[{key: KEY, value: res}];
    },
	
	onSave: function(){
		this._handleSaveClick();
	}
	
	
});

})();