/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskFactorsDetailDialog");
dojo.provide("com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskFactorsDetailDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.data.ItemFileReadStore");

// Require the common widgets needed
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.OptionChooser");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");

// Require local widgets
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.risk.web.client.internal.mapper.RiskTypeMapper");
dojo.require("com.ibm.rqm.risk.web.client.internal.risk.RiskClient");
dojo.require("com.ibm.rqm.risk.web.ui.internal.util.RiskMessageUtil");

(function(){ // module container
    /* Jazz objects */
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    /* QM objects */
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var OptionChooser  = com.ibm.asq.common.web.ui.internal.widgets.OptionChooser;
	var SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
    var RiskClient = com.ibm.rqm.risk.web.client.internal.risk.RiskClient;
	var RiskTypeMapper = com.ibm.rqm.risk.web.client.internal.mapper.RiskTypeMapper;
	var RiskMessageUtil = com.ibm.rqm.risk.web.ui.internal.util.RiskMessageUtil;
	var getMessage=Constants.getMessage;

    dojo.declare("com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskFactorsDetailDialog", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.risk.web", "ui/internal/view/templates/assessment/RiskFactorsDetailDialog.html"),
        
        width: "500px",
        
        labelProject: getMessage("labelRiskType"),
		labelFactor: getMessage("labelRiskWithColon"),
        labelLikelyhood: getMessage("labelLikelihood"),
		labelCost:  getMessage("labelImpact"),
        labelVeryHigh: getMessage("labelVeryHighWithColon"),
        labelHigh: getMessage("labelHighWithColon"),
        labelNeutral: getMessage("labelNeutralWithColon"),
        labelLow: getMessage("labelLowWithColon"),
        labelVeryLow: getMessage("labelVeryLowWithColon"),
		labelMitigation: getMessage("labelMitigationWithColon"),
		RISK_TYPE_DYNAMIC: "Dynamic",//$NON-NLS-N$
		
        // object initializer
        constructor: function(args){
			this.title = args.title ? args.title : getMessage("labelRiskProfileDetail"); 
			this.riskTypes = [];
			this.okCallback = args.okCallback;
			this.okCallbackData = args.okCallbackData;
			this.selectedFactors = args.selectedFactors;
			this.detailTable = args.detailTable;
			this.isInEdit = args.isInEdit ? args.isInEdit : false;
			this.isInSection = args.isInSection ? args.isInSection : false;
        },
        
		postCreate: function() {
			if(!this.artifactType) {
				throw new Error("Parameter 'artifactType' is REQUIRED!");
			}
			if (this.isInSection) {
				this._commentNode.style.display = "";
			}
			if (this.isInEdit) {
				this._initializeWithFactor();
			}
			else {
				var responseHandler = new RiskTypeMapper(this, "_initialize", "_error");
				RiskClient.getAllRiskTypeDTOs(responseHandler, {artifactType: this.artifactType});
			}
		},	
		
        _initialize: function(riskTypes){
			var filteredRiskTypes = this._filterExistFactors(riskTypes);
			this.riskTypes = filteredRiskTypes ? filteredRiskTypes : [];
            dojox.data.dom.removeChildren(this._riskTypeTD);
			var select = document.createElement("select");
            for (var j = 0; j < (this.riskTypes ? this.riskTypes.length : 0); j++) {
				var option = new Option(this.riskTypes[j].name, this.riskTypes[j].id);
				SelectUtility.add( select, option );
            };
			dojo.addClass(select, "inputWidth");
			this.connect(select, "onchange", "_onTypeChange");
			this.riskTypeCombo = select;
			this._riskTypeTD.appendChild(this.riskTypeCombo);
			
			this._initCalcutations();
			
            this._updateFactorCombo(this.riskTypes[0] ? this.riskTypes[0].riskFactors : []);
			
            this._showAsDialog(dojo.hitch(this, "_onAdd"));
        },
		
		_initializeWithFactor: function() {
			var selectedFactor = this.selectedFactors[0];
			dojox.data.dom.removeChildren(this._riskTypeTD);
			this._riskTypeTD.appendChild(document.createTextNode(selectedFactor.assessFactor.riskType));
			dojox.data.dom.removeChildren(this._riskFactorTD);
			this._riskFactorTD.appendChild(document.createTextNode(selectedFactor.assessFactor.name));
			
			if (this.isInSection) {
				this._mitigatoinAction.value = selectedFactor.mitigationAction;
			}
			
			if(selectedFactor.assessFactor.name == RiskMessageUtil.getMessage("testCaseWeight")) {
				this._veryHigh.value = selectedFactor.veryHigh;
				this._high.value = selectedFactor.high;
				this._neutral.value = selectedFactor.medium;
				this._low.value = selectedFactor.low;
				this._veryLow.value = selectedFactor.veryLow;
			}
			else
				this._initCalcutations()
			this._toggleValuePane(selectedFactor.assessFactor.riskType);
			
			this._showAsDialog(dojo.hitch(this, "_onEdit"));
		},
		
		_initCalcutations: function() {
			//init likelyhood and cost combobox
			var option = new Option(getMessage("labelShortHigh"), "3");
            SelectUtility.add(this._likelyhood, option);
			var option = new Option(getMessage("labelShortMid"), "2");
            SelectUtility.add(this._likelyhood, option);
			var option = new Option(getMessage("labelShortLow"), "1");
            SelectUtility.add(this._likelyhood, option);
			
			var option = new Option(getMessage("labelShortHigh"), "3");
            SelectUtility.add(this._cost, option);
			var option = new Option(getMessage("labelShortMid"), "2");
            SelectUtility.add(this._cost, option);
			var option = new Option(getMessage("labelShortLow"), "1");
            SelectUtility.add(this._cost, option);
			
			if(this.isInEdit) {
				SelectUtility.select(this._likelyhood, this.selectedFactors[0].likelihood);
				SelectUtility.select(this._cost, this.selectedFactors[0].cost);
			}
		},
		
        _showAsDialog: function(okCallback){
            if (!this._dialog) {
                var args = {
                    primaryTitle: this.title, //getMessage("labelRiskProfileDetail"),
                    isClosable: true,
                    width: this.width,
					okCallback: okCallback,
                    contentNode: this.domNode
                }
                this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog(args);
                this._dialog.setTitle(this.title);//getMessage("labelRiskProfileDetail"));
				if(dojo.isIE) {
					this._dialog._yesAction = function() {
						 this._yesButton.disabled = true;
        				 if (dojo.isFunction(this.yesCallback)) {
							this.yesCallback(this.yesCallbackData);
						 }
						 this.close();
					}
				}
            }
            this._dialog.open(false);
        },
        
        _onTypeChange: function(){
			var selected = this.riskTypeCombo.value;
			var selectedName = "";
            for (var i = 0; i < this.riskTypes.length; i++) {
                if (this.riskTypes[i].id == selected) {
					selectedName = this.riskTypes[i].name;
					var factors =  this.riskTypes[i].riskFactors;
					this._updateFactorCombo(factors || []);
                    break;
                }
            }
			this._toggleValuePane(selectedName);
        },
		
		_updateFactorCombo: function(elements) {
			dojox.data.dom.removeChildren(this._riskFactorTD);
			var select = document.createElement("select");
			var nElements = (elements ? elements.length : 0);
            for (var j = 0; j < nElements; j++) {
            	var element = elements[j];
				var option = new Option(element.name, element.id);
				SelectUtility.add( select, option );
            }
			dojo.addClass(select, "inputWidth");
			this.connect(select, "onchange", "_onFactorChange");
			
            this.riskFactorCombo = select;//this._createComboBox(elements, dojo.hitch(this, "_onFactorChange"));.
            this._riskFactorTD.appendChild(this.riskFactorCombo);
		},
		
		_onFactorChange: function() {
		},
		
		_toggleValuePane: function(typeName) {
			if(typeName == this.RISK_TYPE_DYNAMIC) { 
				this._veryHighNode.style.display = "";
				this._highNode.style.display = "";
				this._neutralNode.style.display = "";
				this._veryLowNode.style.display = "";
				this._lowNode.style.display = "";
				this._likelyhoodNode.style.display = "none";
				this._costNode.style.display = "none";
			}else {
				this._veryHighNode.style.display = "none";
				this._highNode.style.display = "none";
				this._neutralNode.style.display = "none";
				this._veryLowNode.style.display = "none";
				this._lowNode.style.display = "none";
				this._likelyhoodNode.style.display = "";
				this._costNode.style.display = "";
			}
		},
		
        _onAdd: function() {
			var riskType = this.riskTypeCombo.value;
			var factor = this.riskFactorCombo.value;
			var selectedType = null;
			var selectedFactor = null;
			for (var i = 0; i < this.riskTypes.length; i++) {
                if (this.riskTypes[i].id == riskType) {
					selectedType =  this.riskTypes[i];
                    break;
                }
            }
			if(selectedType && selectedType.riskFactors) {
                for (var i = 0; i < selectedType.riskFactors.length; i++) {
                    if (selectedType.riskFactors[i].id == factor) {
                        selectedFactor = selectedType.riskFactors[i];
                        break;
                    }
                }
			}
			
			if(selectedType && selectedFactor) {
                var result = {
                    assessFactor: {
						id: selectedFactor.id,
                    	riskType: selectedType.name,
                    	name: selectedFactor.name
					},
					weight: "3",
					currentImpact: "-1",
                    isNew: true
                };
				if (this.isInSection) {
					result.mitigationAction = this._mitigatoinAction.value;
				}
				if(selectedType.name != RiskMessageUtil.getMessage("labelDynamic")) {
					result.assessment = getMessage("msgStaticFactorValue", this._getSelectedName(this._likelyhood), this._getSelectedName(this._cost));
					result.likelihood =  this._likelyhood.value;
					result.cost = this._cost.value;
				}
				else {
					result.assessment = this._getTCWeightValue(result);
				}
				if(this.detailTable) {
					this.detailTable.addNewFactors(new Array(result));
				}
			}
		},
		
		_onEdit: function() {
			if(this.selectedFactors[0].assessFactor.name == RiskMessageUtil.getMessage("testCaseWeight")) {
				this.selectedFactors[0].assessment = this._getTCWeightValue(this.selectedFactors[0]);
			}
			else {
				this.selectedFactors[0].assessment = getMessage("msgStaticFactorValue", this._getSelectedName(this._likelyhood), this._getSelectedName(this._cost));
				this.selectedFactors[0].likelihood =  this._likelyhood.value;
				this.selectedFactors[0].cost = this._cost.value;
			}
			if (this.isInSection) {
				this.selectedFactors[0].mitigationAction = this._mitigatoinAction.value;
			}
			if(this.detailTable) {
				this.detailTable.updateFactors(this.selectedFactors);
			}
		},
		
		_filterExistFactors: function(riskTypes) {
			var results = [];
			for(var i = 0; i < (riskTypes ? riskTypes.length:0); i++) {
				var factors = riskTypes[i].riskFactors;
				var newFactors = [];
				for(var k = 0; k < (factors ? factors.length:0); k++) {
					var isExist = false;
					for(var j = 0; j < (this.selectedFactors ? this.selectedFactors.length : 0); j++) {
						if(factors[k].id == this.selectedFactors[j].assessFactor.id) {
							isExist = true
						}
					}
					if(!isExist)
						newFactors.push(factors[k]);
				}
				if(newFactors.length > 0) {
					riskTypes[i].riskFactors = newFactors;
					results.push(riskTypes[i]);
				}
			}
			return results;
		},

		_hasTCWeightFactor: function() {
			for(var i = 0; i < (this.selectedFactors ? this.selectedFactors.length:0);i++) {
				if(this.selectedFactors[i].assessFactor.name == RiskMessageUtil.getMessage("testCaseWeight")) 
					return true;
			}
			return false;
		},
		
		_getSelectedName: function(select) {
			var options = select.options || [];
			for(var i = 0; i < options.length; i++) {
				if(options[i].selected)
					return options[i].text;
			}
			return "";
		},
		
		_getTCWeightValue: function(result) {
			//each value should be a Int
			var veryHigh = this._getIntValueFromString(this._veryHigh.value);
			if(!veryHigh || veryHigh == 0)
				return null;
			var high = this._getIntValueFromString(this._high.value);
			if(!high || veryHigh < high)
				return null;
			var neutral = this._getIntValueFromString(this._neutral.value);
			if(!neutral || high < neutral)
				return null;
			var low = this._getIntValueFromString(this._low.value);
			if(!low || neutral < low)
				return null;
			var veryLow = this._getIntValueFromString(this._veryLow.value);
			if(!veryLow || low < veryLow)
				return null;
			result.veryHigh = veryHigh;
			result.high = high;
			result.medium = neutral;
			result.low = low;
			result.veryLow = veryLow;
			return getMessage("msgCaseWeightValue", veryHigh, high, neutral, low, veryLow);
		},
		
		_getIntValueFromString: function(sValue) {
			if (sValue || dojo.trim(sValue).length > 0 || !isNaN(dojo.number.parse(sValue))) {
				return dojo.number.parse(sValue);
			}
			return null;	
		},
		
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ RiskFactorsDetailDialog#_error");
        }
    });
    
})();
