/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.buildintegration.web.ui.internal.view.build.BuildTableContentFilterProvider");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil");
dojo.require("com.ibm.rqm.buildintegration.web.client.internal.build.BuildStatesClient");
dojo.require("com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager");

(function() {

var BuildUiUtil = com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil;
var BuildStatesClient = com.ibm.rqm.buildintegration.web.client.internal.build.BuildStatesClient;
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
var TableContentFilterProvider = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;
var ManualProviderType = "com.ibm.rqm.buildintegration.common.manualProvider";
var ServiceResponseHandler  = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

dojo.declare("com.ibm.rqm.buildintegration.web.ui.internal.view.build.BuildTableContentFilterProvider",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider,
{
    constructor: function(tableView, openSection, summaryText, initialVisibleFilterCount, includeBVT) {
        this.tableView = tableView;
        this.initOpenFilter = openSection;
        this.summaryText = summaryText;
        this.initialVisibleFilterCount = initialVisibleFilterCount;

        // define the static filters
        this.filters = [];
        this.filters.push({
            name: "name",  // must match key in the arg list of the API
            title: BuildUiUtil.getMessage("labelBuild"),
            promptTitle: BuildUiUtil.getMessage("labelBuild"),
            type: TableContentFilterProvider.FIELD_TEXT,
            defaultValue: null, 
            params: null
        });

        if (includeBVT) {
            var bvtStatus = [];
            bvtStatus.push({
                name: BuildUiUtil.getRTLMSequencerMessage("labelNotStarted"),
                value: "NOTSTARTED",
                checked: false
            });
            bvtStatus.push({
                name: BuildUiUtil.getRTLMSequencerMessage("labelInProgress"),
                value: "IN_PROGRESS",
                checked: false
            });
            bvtStatus.push({
                name: BuildUiUtil.getRTLMSequencerMessage("labelCancelledSequence"),
                value: "CANCELLED",
                checked: false
            });
            bvtStatus.push({
                name: BuildUiUtil.getRTLMSequencerMessage("labelComplete"),
                value: "COMPLETE",
                checked: false
            });
            bvtStatus.push({
                name: BuildUiUtil.getRQMMessage("labelPass"),
                value: "PASS",
                checked: false
            });
            bvtStatus.push({
                name: BuildUiUtil.getRQMMessage("labelFail"),
                value: "FAIL",
                checked: false
            });
            bvtStatus.push({
                name: BuildUiUtil.getRQMMessage("labelInconclusive"),
                value: "INCONCLUSIVE",
                checked: false
            });
            bvtStatus.push({
                name: BuildUiUtil.getRQMMessage("labelBlocked"),
                value: "BLOCKED",
                checked: false
            });
            bvtStatus.push({
                name: BuildUiUtil.getRQMMessage("labelError"),
                value: "ERROR",
                checked: false
            });
            bvtStatus.push({
                name: BuildUiUtil.getRQMMessage("labelUnknown"),
                value: "UNKNOWN",
                checked: false
            });

            this.filters.push({
                name: "bvtStatus",  // must match key in the arg list of the API
                title: BuildUiUtil.getMessage("labelBvtStatus") + Constants.getMessage("labelColen"),
                promptTitle: BuildUiUtil.getMessage("labelBvtStatus"),
                type: TableContentFilterProvider.FIELD_TOGGLELIST,
                defaultValue: null,
                params: {options: bvtStatus}
            });
        }
    },

    getTitle: function() {
        return Constants.getMessage("labelFilterTitle");    
    },
    
    getIsSectionOpenInitially: function() {
       return this.initOpenFilter; 
    },
    
    getSummaryText: function() {
        return this.summaryText;
    },
    
    getInitialVisibleFilterCount: function() {
        return this.initialVisibleFilterCount;
    },
    
    getFilters: function() {
        return this.filters;
    },  
    
    getFilterCount: function() {
        return this.filters.length;
    },
    
    getFilterName: function(index) {
        return this.filters[index].name;
    },
    
    getFilterTitle: function(index) {
        return this.filters[index].title;
    },
    
    getFilterPromptTitle: function(index) {
        return this.filters[index].promptTitle;        
    },
    
    getFilterType: function(index) {
        return this.filters[index].type;
    },
    
    getFilterDefaultValue: function(index) {
        return this.filters[index].defaultValue;
    },
    
    getFilterParams: function(index) {        
        return this.filters[index].params;
    },
    
    getCustomField: function(index) {
        return null;
    },
    
    getActionsInfo: function() {
        var actions = [
            {
                id: "cancel",
                title: Constants.getMessage("labelReset"),
                callback: dojo.hitch(this, "filterResetActionInvoked"),
                callbackData: null,
                isDefault: false
            }, {
                id: "run",
                title: Constants.getMessage("labelRun"),
                callback: dojo.hitch(this, "filterRunActionInvoked"),
                callbackData: null,
                isDefault: true
            }
        ];
        return actions;
    },
    
    filterResetActionInvoked: function(actionName, filters) {
        // reset the filters.
        this.tableView.tableViewer.clearContentFilters();
    },

    filterRunActionInvoked: function(actionName, filters){
        var args = {};
        
        for (var nameKey in filters) {
            if (filters[nameKey] != null) {
                // BVT result for "PAUSED" and "CANCELLED" both use the display string of "Canceled".
                // Add "PAUSED" if "CANCELLED" is specified.
                if (dojo.isArray(filters[nameKey]) && nameKey == "bvtStatus") {
                    var bvtStatusFiltersCount = filters[nameKey].length;
                    for (var i = 0; i < bvtStatusFiltersCount; i++) {
                        if (filters[nameKey][i] == "CANCELLED") {
                            filters[nameKey].push("PAUSED");
                            break;
                        }
                    }
                }
                args[nameKey] = filters[nameKey] || "";
                if (args[nameKey] == 'Any@@') {
                    args[nameKey] = '';
                }
            }
        }
        this.tableView.runQuery(args);
    },
    
    // Summary:
    //     Get the states and status of the current build provider.  If "addState" is not null,
    //     the state filter will be added to the "filtersCollection".  If "addStatus" is not
    //     null, the status filter will be added to the "filtersCollection".
    //
    _initBuildStateStatus: function(addState, addStatus, filtersCollection, callback) {
        var BuildsManager = com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager.getInstance();
        var providerId = BuildsManager.getCurrentBuildProviderId();
        if (!providerId) {
            providerId = ManualProviderType;
        }
        var self = this;
        var serviceObj = {
            success: function(providers) {
                self._doInitBuildStateStatus(providers, addState, addStatus, filtersCollection, callback);
            },
            
            failure: function(e) {
                self._error(e);
                callback();
            }
        };

        var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
        BuildStatesClient.getBuildStates(responseHandler, {providerTypeId: providerId});
    },
    
    _doInitBuildStateStatus: function(providers, addState, addStatus, filtersCollection, callback) {
        this.buildStateStatusInitialized = true;
        var provider = providers ? providers[0] : [];

        var statusCount = provider.buildRecordStatuses.length;
        this.buildStatus = [];
        for (var i = 0; i < statusCount; i++){
            var status = provider.buildRecordStatuses[i];
            this.buildStatus.push({
                name: BuildUiUtil.getDisplayName(status.stateId, status.displayName),
                value: status.stateId,
                checked: false
            });
        }
        if (addStatus) {
            this._doAddBuildStatusFilter(this.buildStatus, filtersCollection);
        }

        var statesCount = provider.buildRecordStates.length;
        this.buildStates = [];
        for (var i = 0; i < statesCount; i++){
            var state = provider.buildRecordStates[i];
            this.buildStates.push({
                name: BuildUiUtil.getDisplayName(state.stateId, state.displayName),
                value: state.stateId,
                checked: false
            });
        }
        if (addState) {
            this._doAddBuildStateFilter(this.buildStates, filtersCollection);
        }
        
        callback();     
    },
    
    _doAddBuildStatusFilter: function(buildStatus, filtersCollection, callback) {
        var buildStatusFilter = {
            name: "statusIds",  // must match key in the arg list of the API
            title: Constants.getMessage("labelStatusWithColon"),
            promptTitle: Constants.getMessage("labelstatus"),
            type: TableContentFilterProvider.FIELD_TOGGLELIST,
            defaultValue: null,
            params: {options: buildStatus}
        };
        filtersCollection.push(buildStatusFilter);
        if (callback) {
            callback();
        } 
    },

    _doAddBuildStateFilter: function(buildStates, filtersCollection, callback) {
        var buildStateFilter = {
            name: "stateIds",  // must match key in the arg list of the API
            title: Constants.getMessage("labelStateLabel"),
            promptTitle: Constants.getMessage("labelState"),
            type: TableContentFilterProvider.FIELD_TOGGLELIST,
            defaultValue: null,
            params: {options: buildStates}
        };
        filtersCollection.push(buildStateFilter);
        if (callback) {
            callback();
        } 
    },

    addStatusFilter: function(filtersCollection, callback) {
        if (this.buildStateStatusInitialized) {
            if (this.buildStatus.length > 0) {
                this._doAddBuildStatusFilter(this.buildStatus, filtersCollection);
            }
            callback();
            return;
        }
        this._initBuildStateStatus(false, true, filtersCollection, callback);
    },
    
    addStateFilter: function(filtersCollection, callback) {
        if (this.buildStateStatusInitialized) {
            if (this.buildStates.length > 0) {
                this._doAddBuildStateFilter(this.buildStates, filtersCollection);
            }
            callback();
            return;
        }
        this._initBuildStateStatus(true, false, filtersCollection, callback);
    },
    
    addStatusAndStateFilter: function(filtersCollection, callback) {
        if (this.buildStateStatusInitialized) {
            if (this.buildStatus.length > 0) {
                this._doAddBuildStatusFilter(this.buildStatus, filtersCollection);
            }
            if (this.buildStates.length > 0) {
                this._doAddBuildStateFilter(this.buildStates, filtersCollection);
            }
            callback();
            return;
        }
        this._initBuildStateStatus(true, true, filtersCollection, callback);
    },

    _error: function(errorObj){
        console.error("Error in BuildTableContentFilterProvider: " + errorObj.message);
    }   
});

})();
