/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.common.ActionBarEntry");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.ActionBarEntry");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("dojox.data.dom");

dojo.require("jazz.core.loader");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility")

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.MenuPopup");

(function() 
{
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

    // Wrapper class of menu item in the common nav bar to mix in the methods to add
    // action entries as the vertical action bar fly-out menu.
    //
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.NavBarEntryItem", null, {
        
        constructor: function(args) {
            this.title = args.title;
            this.menuEntries = [];
        },
        
        addEntry: function(name, callback, isBold, callbackData) {
            var args = {
                label: name,
                isBold: isBold,
                callbackMethod: callback,
                callbackData: callbackData,
                onClick: function() {
                    this.callbackMethod(this.callbackData);
                }
            };
            this.menuEntries.push(args);
        },
           
        addSeparator: function() {
            this.menuEntries.push(null);
        },
        
        realize: function() {
            var navbar = dojo.getObject("com.ibm.rqm.web.globalNavBar");
            if (!navbar) {
                throw new Error("global navbar is not set");
            }

            navbar._createItem({
                label: this.title,
                align: "left",
                menuProvider: dojo.hitch(this, this._createDropdownMenu)
            });
        },
        
        _createDropdownMenu: function() {
            var menu = new dijit.Menu();
            dojo.addClass(menu.domNode, "com-ibm-rqm-planning-web-nav-bar-override");

            for (var i = 0; i < this.menuEntries.length; i++) {
                if (this.menuEntries[i]) {
                    var menuItem = new dijit.MenuItem(this.menuEntries[i]);
                    dojo.addClass(menuItem.domNode, "com-ibm-rqm-planning-web-nav-bar-override");
                    menu.addChild(menuItem);
                } else {
                    var separator = new dijit.MenuSeparator();
                    // TODO: remove class after 2.0 to adopt style from Jazz Foundation
                    dojo.addClass(separator.domNode, "com-ibm-rqm-planning-web-menu-bar");
                    menu.addChild(separator);
                }
            }
            return menu;
        }
    });

    dojo.declare(
        "com.ibm.rqm.planning.web.ui.internal.view.common.ActionBarEntry",
        [dijit._Widget, dijit._Templated],
        {
            templateString: '<div class="hover-left" dojoAttachPoint="_hoverArea">' +
                              '<div class="hover-right" dojoAttachPoint="_hoverAreaRight">' +
                                '<div class="hover-center" dojoAttachPoint="_hoverAreaCenter">' +
                                  '<div class="action-bar-icon" dojoAttachPoint="_iconArea">' +
                                    '<img dojoAttachPoint="_iconImg"></img>' +
                                    '<div class="action-bar-entry-icon-text" dojoAttachPoint="_iconText"></div>' +
                                  '</div>' +
                                '</div>' +
                              '</div>' +
                            '</div>',
    
            constructor: function()
            {
                this.alreadyLoaded = [];
            },
    
            postCreate: function(args)
            {
            },
            
            initialize: function(args)
            {
				var actionMenuDTO = args.actionMenuDTO;
				
                this._iconURL = dojo.moduleUrl(actionMenuDTO.component, actionMenuDTO.icon);
				this._iconHoverURL = dojo.moduleUrl(actionMenuDTO.component, actionMenuDTO.iconHover);
                this._imageText = Constants.getMessage(actionMenuDTO.name);
                this._contentArea = args.contentArea;
                this._popupMenuArea = args.popupMenuArea;
                this._parent = args.parent;
                
                if(args.actionDispatcherName) {
                    this.actionDispatcherName = args.actionDispatcherName;
                }
                
                // if this is CJK locale add in our cjk locale specific CSS
                if(com.ibm.asq.common.web.ui.internal.util.Utility.isCJKLocale()) {
                    dojo.addClass(this._iconText, "action-bar-entry-cjk");
                }

                // A action dispatcher name is provided. Invocations of menu entries
                // will be dispatched thru the handler.
                if(this.actionDispatcherName) {
                    var actionRegistry = PlatformUI.getWorkbench().getActionRegistry();
                    var action = actionRegistry.findAction(this.actionDispatcherName);
                
                    if(action) {
                        this.actionDispatcher = action._contextObject;
                    }
                }

                // If asMenuBarEntry is true the action bar will be horizontal in the global
                // task bar area as a menu bar.  The action bar entry will be click to drop
                // down the action menu.  Otherwise the action bar entry will have flo-out
                // menu on hover.
                //
                this._asMenuBarEntry = args.asMenuBarEntry;

                if (this._asMenuBarEntry) {
                    // create a drop down menu for the action entry in the global nav bar of the common banner
                    this._menuPopup = new com.ibm.rqm.planning.web.ui.internal.view.common.NavBarEntryItem({title: this._imageText});
                    this.topDomElement = this._menuPopup;
                } else {
                    this._isMenuShowing = false;
                    
                    this._iconImg.src = this._iconURL;
                    var iconText = com.ibm.asq.common.web.ui.internal.util.wordWrapping.wrapText(
                        this._imageText, 
                        "action-bar-entry-icon-text", 
                        68, // hard code the max width for now. The dom node has not been added to the tree. 
                        false,
                        "<br/>");
                    this._iconText.innerHTML = iconText;
                    
                    this._hoverArea.onmouseover = dojo.hitch(this, "_overIcon");
                    this._hoverArea.onmouseout = dojo.hitch(this, "_offIcon");
                    this._hoverArea.onclick = dojo.hitch(this, "_overIcon");
                    this._hoverArea.onfocus = dojo.hitch(this, "_focusIcon");
                    this._hoverArea.onblur = dojo.hitch(this, "_blurIcon");
                    this._hoverArea.onkeydown = dojo.hitch(this, "_doKeyPress");
                    
                    this._menuPopup = new com.ibm.rqm.planning.web.ui.internal.view.common.MenuPopup();
                    this._menuPopup.initialize(this, this._popupMenuArea, this._iconHoverURL);
                    this.topDomElement = this.domNode;
                }
            },
            uninitialize: function() {
            	//debugger;
            	//console.debug( "uninitialize actionbarentry "+ this._imageText);
            	var hoverArea = this._hoverArea;
            	if (hoverArea) {
            		hoverArea.onmouseover = null;
            		hoverArea.onmouseout = null;
            		hoverArea.onclick = null;
            		hoverArea.onfocus = null;
            		hoverArea.onblur = null;
            		hoverArea.onkeydown = null;
            	}
            	var menuPopup = this._menuPopup;
            	if (menuPopup) {
            		if (menuPopup.destroyRecursive){ // not available if horizontal menus in use
            			menuPopup.destroyRecursive();
            		}
            		delete this._menuPopup;
            	}
            	this.topDomElement = null;
            }, 
            _loadClassAsync: function(className, callback)
            {
                var cb = null;
                
                if(callback)
                    cb = dojo.hitch(this, callback);
                
                if (!dojo.getObject(className))
                {
                    var self = this;
                    
                    console.info(className + "- needs loading");
                    // set the class loaded flag when the async calls 
                    // us back.
                    var func = function() {
                        if(cb) cb();
                    }
                    try {
                        jazz.core.loader.load_async(className, func);
                    }
                    catch(e) {
                        com.ibm.asq.common.web.ui.internal.util.Utility.printException("ActionBarEntry", "_loadClassAsync", e);
                    }
                }
                else
                {
                    console.info(className + "- Already loaded");
                    if(cb)
                        setTimeout(function(){ cb(); }, 0);
                }
            },

            _routeAction: function(actionArgs)
            {
                var uri = "#action=" + this.actionDispatcherName;
                
                for(var argName in actionArgs) {
                    var argVal = actionArgs[argName];
                    
                    if(argName == "tq")
                        argVal = Date.parse(new Date().toString());
                        
                    uri += ("&" + argName + "=" + argVal); 
                }
                window.location.hash = uri;
            },
            
            _addMenuEntry: function(itemName, method, isBold, actionArgs)
            {
                // route the handling of the menu invocation to the action handler
                // if one is defined.
                if(this.actionDispatcher) {
                    var bindName = (actionArgs && actionArgs.subAction) ? actionArgs.subAction : itemName;
                    
                    // bind the method and associated actionName/itemName in the 
                    // action dispatcher.
                    this.actionDispatcher.registerDispatchAction(bindName, method);
                    this._menuPopup.addEntry(itemName, dojo.hitch(this, "_routeAction"), isBold, actionArgs);                    
                }
                else {
                    this._menuPopup.addEntry(itemName, method, isBold);
                }
            },
            
            _addNonDispatchableMenuEntry: function(itemName, method, isBold, actionArgs)
            {
                // Add menu entry without going through the action handler.  This is used
                // for action that redirect the page to a new URL and should not have the
                // dispatcher create a URL for a page to do the redirecting.
                //
                this._menuPopup.addEntry(itemName, method, isBold, actionArgs);
            },
            
            _addMenuSeparator: function()
            {
                this._menuPopup.addSeparator();
            },
            
            _addPage: function(args)
            {
                if (!this._contentArea) {
                    // For horizontal menu bar, the menu tries are created by the application in
                    // the common banner navBar which the HomePageView may not be loaded yet to
                    // the contentArea set.  The HomePageView set this upon loading.
                    this._contentArea = dojo.getObject("com-ibm-rqm-planning-web-page-content-area");
                }
                return this._contentArea.addPage(args);
            },
            
            _setHoverImage: function(isOver)
            {
                if (isOver)
                {
                    dojo.addClass(this._hoverArea, "hovered-left");
                    dojo.addClass(this._hoverAreaRight, "hovered-right");
                    dojo.addClass(this._hoverAreaCenter, "hovered-center");
                }
                else
                {
                    dojo.removeClass(this._hoverArea, "hovered-left");
                    dojo.removeClass(this._hoverAreaRight, "hovered-right");
                    dojo.removeClass(this._hoverAreaCenter, "hovered-center");
                }
            },

            // on focus will only highlight the icon
            _focusIcon: function(e) {
                this._setHoverImage(true);
                this.movingToPopup = false;
                this._parent.setCurrentActionEntry(this);
            },
            
            // on blur will remove the icon highlight except it is blur to
            // the popup menu which is explicitly on right arrow key pressed
            _blurIcon: function(e) {
                if (!e) {
                    e = window.event;
                }
                if (e.type == "blur") {
                    if (this.movingToPopup) {
                        // ignore this event as we are just changing focus to the popup
                        this.movingToPopup = false;
                        return;
                    } else if (this._isMenuShowing) {
                        if (dojo.isIE) {
                            if (this.unconditionalHideMenuOnBlur) {
                                this._menuPopup.hideMenu();
                                this.unconditionalHideMenuOnBlur = false;
                            } else {
                                // Blur can be from a false mouse event when the mouse is still
                                // within the popup menu.  Pass the event along for the popup
                                // menu to determine.
                                this._menuPopup.hideMenu(e);
                                if (this._menuPopup.isDisplayed) {
                                    // IE false mouse event was caught when trying to hide the popup menu.
                                    // Check if the popup is still displayed to ignore this blur.
                                    return;
                                }
                            }
                        } else {
                            this._menuPopup.hideMenu();
                        }
                        this._isMenuShowing = false;
                    }
                    this._setHoverImage(false);
                    this._parent.clearCurrentActionEntry();
                }
            },
            
            // invoked by the key down event handler of the enter key or the right arrow key
            _activateIcon: function(e) {
                // display the popup and set it to focus
                if (this._isMenuShowing) {
                    // popup is already showing (from mouse over), move the focus to
                    // the popup
                    this._menuPopup.setActionFocus();
                } else {
                    this._popupMenuArea.appendChild(this._menuPopup.domNode);
                    this._menuPopup.showMenuWithFocus(this._iconImg, e);
                    this._isMenuShowing = true;
                }
            },
            
            _overIcon: function(e) {
                if (!e) {
                    e = window.event;
                }
                if (!this._isMenuShowing) {
                    // implicitly set action to focus to seamlessly work with keyboard traversal
                    this._hoverArea.focus();
                    this._popupMenuArea.appendChild(this._menuPopup.domNode);
                    this._menuPopup.showMenu(this._iconImg, e);
                    this._isMenuShowing = true;
                }
            },
            
            _offIcon: function(e) {
                if (!e) {
                    e = window.event;
                }
                var toNode = dojo.isIE ? e.toElement : e.relatedTarget;

                if (this._isMenuShowing) {
                    // The to node will be null if the mouse is moving out of the
                    // document which we should hide the menu.
                    if (toNode) {                
                        if (dojo.isDescendant(toNode, this._hoverArea)) {
                            // false event; moved to the child node; ignore.
                            return;
                        }
                        
                        if (dojo.isDescendant(toNode, this._menuPopup.domNode)) {
                            // false event; moved to the menu popup; ignore.
                            return;
                        }
                    }
                }
                this._hoverArea.blur();

                // Mouse events are sent to the browser even when the browser window
                // is not in focus.  Focus invoked from the mouse event handler is
                // honered but not blur.  We need to do the UI clean up that blur
                // would do here otherwise there will be opened popup all over.
                this._menuPopup.hideMenu();
                this._isMenuShowing = false;
                this._setHoverImage(false);
            },
			
			_doKeyPress: function(e) {
                if (!e) {
                    e = window.event;
                }
                if ((e.type == "keydown") && e.keyCode) {
                    if ((e.keyCode == dojo.keys.ENTER) || (e.keyCode == dojo.keys.RIGHT_ARROW)) {
                        // Enter and right arrow key will bring up the popup
                        this.movingToPopup = true;
                        this._activateIcon();
				    } else if (e.keyCode == dojo.keys.UP_ARROW) {
                        // The popup is not up when this action entry is in focus to generate key
                        // event so we can safely tell the parent, the ActionBar, to select previous
                        // action entry.
                        this._parent.selectPrevActionEntry(this);
                        if (dojo.isIE) {
                            this.unconditionalHideMenuOnBlur = true;
                        }
                        dojo.stopEvent(e);
                    } else if (e.keyCode == dojo.keys.DOWN_ARROW) {
                        // Tell the parent to select the next action entry.
                        this._parent.selectNextActionEntry(this);
                        if (dojo.isIE) {
                            this.unconditionalHideMenuOnBlur = true;
                        }
                        dojo.stopEvent(e);
                    }
			    }
            }
        });
    }
)();