/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.CollapseExpandArea");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.Section");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function() { // module container

var	ImageCache			= com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
var	TooltipBuilder		= com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder;
var CSHelp				= com.ibm.asq.common.web.ui.internal.util.CSHelp;

dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.common.CollapseExpandArea",
	[dijit._Widget, dijit._Templated], 
	{
		
	templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/CollapseExpandArea.html"),
	//templateCssPath: dojo.uri.dojoUri("../com.ibm.rqm.planning.web/ui/internal/view/templates/common/CollapseExpandArea.css"),
	
	constructor: function() {
		///////////////////////////////////////////////////////////////////////////////////
		// Initialization parameters	
		//////////////////////////////////////////////////////////////////////////////////	
		this.label = null;
		this.labelTooltip = null;
		//this.id = null;
		this.sectionWidget = null;
		this.open = false;		
		this.showTwistie = true;
		this.showBorder = false;
		this.flagStatus = 0;
		
		this.editorOpen = false;
		this.editorAvailable = true;
      this.editor = null;
		this.showActionsForEditor = true;
		
		this.cshelpID = null;

		//////////////////////////////////////////////////////////////////////////////////
		// Actions
		//////////////////////////////////////////////////////////////////////////////////
		this.actions = [];	//action objects should include: 'text' && ('href' || ('obj' && 'function'))
		
		//////////////////////////////////////////////////////////////////////////////////
		// States
		//////////////////////////////////////////////////////////////////////////////////
		this._isExpanded = false;
		this._isEditorOpen = false;
				
		//////////////////////////////////////////////////////////////////////////////////
		// Expand/Edit options
		//////////////////////////////////////////////////////////////////////////////////
		this.isExpandable = true;
		this.isEditable = true;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Called on creation
	///////////////////////////////////////////////////////////////////////////////////
	postCreate: function() { },
	
	initialize: function(args) {
		//allow mixin of args on initialize()
		for (var element in args) {
			this[element] = args[element];
		}

		//if editable (editor available) connect proper events
		if (this.editorAvailable && this.editorAvailable == true) {
			//this._editImg.src = ImageCache.EDIT;
			this.connect(this.labelSpan, "onclick", this._toggleEditor);
			//this.connect(this._editImg, "onclick", this._toggleEditor);
			dojo.addClass(this.labelSpan, "cursor-pointer");

			if (this.sectionWidget && this.sectionWidget["setNameListenerNode"]) {
            this.sectionWidget.setNameListenerNode(this.labelSpan);
			}
			
		} else {
			this.editorOpen = false;
		}
		this._initializeWidget();
	},

	_initializeWidget: function() {
		//set initial collapsed/expanded state
		if (this.open && this.open == true) {
         if (this.sectionWidget._hierarchy) {
            if (dojo.hasClass("hidden") ) {
               dojo.removeClass("hidden");
            }            
         }
			this.expand();
		
      } else {
			this.collapse();
		}
		//set initial editor state
		if (this.editorOpen) {
			this._cshelp.className = "cshelp";
			if (this.showBorder) {
				this._titleRow.className = "title-row-expanded";
				this._sectionGroupBorderTop.className = "border-top";
				this._sectionGroupBorderBottom.className = "border-bottom hidden";		
				this._sectionGroupContainer.className = "section-group-collapsed";
			} else {
				var editor = this.sectionWidget._editor;
			   if (dojo.hasClass(editor, "shaded") && dojo.hasClass(editor, "hidden") ) {
               //child.className = "editor shaded bottom-border";
               dojo.addClass(editor, "bottom-border");
               dojo.addClass(this.labelSpan, "shaded");
               dojo.addClass(this.labelSpan, "cursor-pointer");
               //this.labelSpan.className = "label cursor-pointer shaded";
				}
			}
		}
		
		this._updateTwistie();
		this._updateFlag();
		
		if (this.icon) {
			this.iconImg.src = this.icon;
		}

		if (this.cshelpId != null) {
			var cshelp = new CSHelp();
			cshelp.contextID = this.cshelpId;
			this._cshelp.appendChild(cshelp.domNode);
		}

		this.setLabel(this.label);
		if ( this.labelTooltip != null ) {
			this.setLabelTooltip(this.labelTooltip);	
		}
		
		if (this.sectionWidget) {
         this.sectionsDiv.appendChild(this.sectionWidget.domNode);
		}
		this._showActions();		
		
	},
			
	///////////////////////////////////////////////////////////////////////////////////
	// API
	///////////////////////////////////////////////////////////////////////////////////
	collapse: function() {
		this._titleRow.className = "title-row-collapsed";
		if (!this.editorOpen || !this.showActionsForEditor) {
			this._actionsBar.className = "hidden";	
		}		
		if (!this.editorOpen) {
			this._sectionGroupContainer.className = this.showBorder ? "section-group-collapsed top-border-line" : "section-group-collapsed" ;
			this._cshelp.className = "hidden";
		}
		this._sectionGroupBorderTop.className = "";
		this._sectionGroupBorderBottom.className = "";		
	},
	
	expand: function() {
		if (this.showBorder) {
			this._titleRow.className = "title-row-expanded";
			this._sectionGroupBorderTop.className = "border-top";
			this._sectionGroupBorderBottom.className = "border-bottom hidden";		
		}
		this._cshelp.className = "cshelp";
		this._actionsBar.className = "actions-bar";
		this._sectionGroupContainer.className = "section-group-expanded";
	},
	
	isExpanded: function() {
		return this._isExpanded;
	},
	
	addAction: function(action) {
		this.actions.push(action);
		this._showActions();
	},
	
	registerAction: function(text, href, obj, method) {
		var action = {
			text: text, href: href, obj: obj, method: method
		}
		this.addAction(action);
	},
	
	setLabel: function(labelText)
	{
		this.label = labelText;
		this.labelSpan.innerHTML = this.label;		
	},

	setLabelTooltip: function(tooltip)
	{
		TooltipBuilder.addLabelTooltip(tooltip, this.labelSpan, this);
	},
	
	setWorkItemUrl: function(wi, name, url)
	{
		dojox.data.dom.removeChildren(this._workItemUrl);
		
		var anchor = document.createElement("A");
		anchor.id = this.label;
		anchor.className = "";
		anchor.href = url;
		var tn = document.createTextNode(name);
		anchor.appendChild(tn);
		this._workItemUrl.appendChild(anchor);
		
		TooltipBuilder.addWorkItemTip(wi, anchor);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Expand/Edit option capability setters
	///////////////////////////////////////////////////////////////////////////////////
	
	//canEditor: function(value) {
	setEditable: function(value) {
		//can be edited
		if(value == true) {
			this.isEditable = true;
			this._editImg.src = ImageCache.EDIT;
		}
		//can't be edited
		if(!value) {
			this.isEditable = false;
			this._editImg.src = "";			
		}
	},
	
	//canExpand: function(value) {
	setExpandable: function(value) {
		//can be expanded
		if(value && value == true) {
			this.isExpandable = true;
			this.twistieImg.src = this._isVisible() ? ImageCache.TWISTIE_OPEN : ImageCache.TWISTIE_CLOSED;
		} else {
			this.isExpandable = false;
			this.twistieImg.src = "";
		}
	},
	
	closeEditor: function() {
      var editor = this.sectionWidget._editor;
      if (dojo.hasClass(editor, "bottom-border")) {
         dojo.removeClass(editor, "bottom-border");
      }
      this.labelSpan.className = "label";

	},
	
	needTitleRow: function(show) {
		if (!show) {
         this._titleRow.className = "hidden";
      } else {
         this._titleRow.className = "title-row-collapsed";
      }
	},	
	
	///////////////////////////////////////////////////////////////////////////////////
	// Helpers
	///////////////////////////////////////////////////////////////////////////////////
	_updateTwistie: function() {
		if (this.showTwistie) {
			this.twistieImg.src = this.isExpanded() ? ImageCache.TWISTIE_OPEN : ImageCache.TWISTIE_CLOSED;
		} else {
			this.twistieImg.src = "";
		}
	},
	
	_updateFlag: function() {
		var img;
		switch(this.flagStatus) {
			case 0: img = ImageCache.FLAG_STATUS_INCOMPLETE;
				break;
			case 1: img = ImageCache.FLAG_STATUS_REQ_NOT_COMP;
				break;
			case 2: img = ImageCache.FLAG_STATUS_DRAFT_COMP;
				break;
			case 3: img = ImageCache.FLAG_STATUS_COMPLETED;
				break;
			default:
				//default to NO flag
				img = "";
		}		
		//this.flagImg.src = img;
	},
	
	_showActions: function() {
		dojox.data.dom.removeChildren(this._actionsBar);
		if ( this.actions != null && this.actions.length > 0 ) {
			for ( var i = 0; i < this.actions.length; i++ ) {
				var action = this.actions[i];
				var actionLink = document.createElement("a");
				actionLink.innerHTML = action.text;
				if ( action.href ) {
					actionLink.href = action.href;
				} else if ( action.obj && action.method ) {
					this.connect(actionLink, "onclick", dojo.hitch(action.obj, action.method));	
				}
				this._actionsBar.appendChild(actionLink);
			}
		}
	},
	
	_setDisplay: function(show) {
		//dojo.html.setDisplay(this.sectionsDiv, show);
		this._updateTwistie();
	},
	  
	///////////////////////////////////////////////////////////////////////////////////
	// Event handlers
	///////////////////////////////////////////////////////////////////////////////////
	_toggleExpand: function(e) {
		if(this.isExpandable) {
			//check each section
			//e.target.parentNode.parentNode.parentNode.childNodes[3].childNodes[0];
			/*
			for (var i = 0; i < this.sectionWidgets.length; i++) {
				//for each section, check its children
				var sectionHier = this.sectionWidgets[i].domNode;
				for (var j = 0; j < sectionNode.childNodes.length; j++) {
					var child = sectionNode.childNodes[j];
					if ((child.className != null) && (child.className != "")) {
						if (dojo.hasClass(child, "hierarchy") && dojo.hasClass(child, "hidden")) {
                     this._isExpanded = true;
                     dojo.removeClass(child, "hidden"); //child.className = "hierarchy";
                     this.expand();
							
						} else if (dojo.hasClass(child, "hierarchy")) {
						   this._isExpanded = false;
							dojo.addClass(child, "hidden"); //child.className = "hierarchy hidden";
							this.collapse();
						}
					}
			   }
			}
			*/
         
         var hierarchy = this.sectionWidget._hierarchy;
         if (dojo.hasClass(hierarchy, "hidden")) {
            this._isExpanded = true;
            dojo.removeClass(hierarchy, "hidden");
            this.expand();            
         } else {
            this._isExpanded = false;
            dojo.addClass(hierarchy, "hidden");
            this.collapse();            
         }        
         
			this._updateTwistie();
		}
	},
			
	_toggleEditor: function(e)
	{
		if (this.isEditable) {
         e.preventDefault();
         
         var editor = this.sectionWidget._editor;         
         if (!dojo.hasClass(editor, "hidden")) {
            //hide the editor
            dojo.addClass(editor, "hidden");
            this.labelSpan.className = "label cursor-pointer";
            if (dojo.hasClass(editor, "shaded") && dojo.hasClass(editor, "bottom-border")) {
               if (!this._isExpanded) {
                  this._sectionGroupContainer.className = this.showBorder ? "section-group-collapsed top-border-line" : "section-group-collapsed";
                  this._actionsBar.className = "hidden";
               }
               this.editorOpen = false;
            } else {
               this._titleRow.className = "title-row-collapsed";
               if (!this._isExpanded) {
                  this._actionsBar.className = "hidden";
                  this._cshelp.className = "hidden";
               }
               this._sectionGroupContainer.className = this.showBorder ? "section-group-collapsed top-border-line" : "section-group-collapsed";
               this._sectionGroupBorderTop.className = "";
               this._sectionGroupBorderBottom.className = "";
               this.editorOpen = false;
            }
         } else {
            //un-hide the editor
            dojo.removeClass(editor, "hidden");            
            if (dojo.hasClass(editor, "shaded")) {
               //show the editor
               dojo.addClass(editor, "bottom-border");
               this.labelSpan.className = "label cursor-pointer shaded";
               this._sectionGroupContainer.className = "section-group-expanded";
               if (this.showActionsForEditor) {
                  this._actionsBar.className = "actions-bar";  
               }
               this.editorOpen = true;
            } else {
               //show the editor
               dojo.removeClass(editor, "bottom-border");   
               if (this.showBorder) {
                  this._titleRow.className = "title-row-expanded";
                  if (this.showActionsForEditor) {
                     this._actionsBar.className = "actions-bar";  
                  }
                  this._sectionGroupBorderTop.className = "border-top";
                  this._sectionGroupBorderBottom.className = "border-bottom hidden";      
                  this._sectionGroupContainer.className = "section-group-collapsed";
               }
               //this.labelSpan.className = "label";
               this._cshelp.className = "hidden";
               this.editorOpen = true;             
            }       
         }
      }

      
	}
});

})();
