/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 /*
 * Author: P. Chu
 *
 * Summary:
 *   A panel with a header and content sections that can be opened or collapsed
 *   as the following.  There are different CSS style for opened and collapsed
 *   state and the content section is hidden when collapsed.
 *   
 *     /-----------+---------------+---------------------------------\    \
 *     | titleNode | titleExtNode  |headerRightNode headerCollapsNode|    |
 *     +-----------+---------------+---------------------------------+    |
 *     | infoNode                                                    |     >  header
 *     +-------------------------------------------------------------+    |
 *     | summaryNode                                                 |    |
 *     +-------------------------------------------------------------+    /
 *     | containerHeaderNode                                         |    \
 *     +-------------------------------------------------------------+    |
 *     | containerNode                                               |     > content
 *     |                                                             |    |
 *     \-------------------------------------------------------------/    /
 *
 *   Open and collapse is controlled by clicking the titleNode or press the return
 *   key when the titleNode is in focus.
 */
dojo.provide("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection");

dojo.require("dijit.TitlePane");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.InlineEditor");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function() {
var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var _textContent=dojo.isIE?"innerText":"textContent"; 

dojo.declare("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection",
[dijit.TitlePane, com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping],
{
    templateString: '<div id="${id}" dojoAttachPoint="domNode" tabIndex="0" class="com-ibm-asq-common-web-ui-collapsible-section">' +
                    '  <div dojoAttachPoint="headerContainer">' +
                    '    <div dojoAttachPoint="headerLeftBorder" class="com-ibm-asq-common-web-ui-collapsible-section-header-left">' +
                    '      <div dojoAttachPoint="headerRightBorder" class="com-ibm-asq-common-web-ui-collapsible-section-header-right">' +
                    '        <div dojoAttachPoint="headerMidBorder" class="com-ibm-asq-common-web-ui-collapsible-section-header-mid"></div>' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
                    '  <div dojoAttachPoint="headerBackContainer">' +
                    '    <div dojoAttachPoint="headerBack">' +
                    '      <div dojoAttachPoint="focusNode" dojoAttachEvent="onclick:_onWhitespaceClick">' +
                    '        <div dojoAttachPoint="titleContainerNode" class="com-ibm-asq-common-web-ui-collapsible-section-title-container">' +
                    '          <span dojoAttachPoint="headerCollapsNode" class="com-ibm-asq-common-web-ui-collapsible-section-title-header-right">' +
                    '            <img tabindex="0" dojoAttachPoint="headerCollapsImg" class="" src=""' +
                    '                 dojoAttachEvent="onmouseover:_onMouseOverImg, onmouseout:_onMouseOutImg"/>' +
                    '          </span>' +
                    '          <span dojoAttachPoint="headerRightNode" class="com-ibm-asq-common-web-ui-collapsible-section-title-header-right"></span>' +
                    '          <span dojoAttachPoint="titleNode" name="${titleName}" class="com-ibm-asq-common-web-ui-collapsible-section-title"' +
                    '                dojoAttachEvent="onclick:_onTitleClick,onkeypress:_onTitleKey" tabindex="0" waiRole="button">' +
                    '          </span>' +
                    '          <span dojoAttachPoint="viewEditableTitle" dojoType="com.ibm.asq.common.web.ui.internal.widgets.InlineEditBox" value="${title}"' +
                    '                class="com-ibm-asq-common-web-ui-collapsible-section-title"></span>' +
                    '          <span dojoAttachPoint="titleExtNode" name="${titleExtName}" class="com-ibm-asq-common-web-ui-collapsible-section-title-ext"' +
                    '                dojoAttachEvent="onkeypress:_onTitleKey" tabindex="0" waiRole="button">' +
                    '          </span>' +
                    '          <span dojoAttachPoint="titleHelpNode"></span>' +                    
                    '        </div>' +
                    '        <div dojoAttachPoint="infoNode" class="com-ibm-asq-common-web-ui-collapsible-section-info-container"></div>' +
                    '        <div dojoAttachPoint="summaryNode" class="com-ibm-asq-common-web-ui-collapsible-section-summary-container"></div>' +
                    '        <div dojoAttachPoint="summaryEditableNode" class="com-ibm-asq-common-web-ui-collapsible-section-summary-container" style="display: none;">'+
                    '           <span dojoAttachPoint="summaryNodeEdit" dojoType="com.ibm.asq.common.web.ui.internal.widgets._InlineEditArea" >' +
                    '           </span>' +                              
                    '        </div>' +
                    '      </div>' +
                    '      <div class="com-ibm-asq-common-web-ui-collapsible-section-content-outer-container" dojoAttachPoint="hideNode">' +
                    '        <div class="com-ibm-asq-common-web-ui-collapsible-section-content-inner-container" dojoAttachPoint="wipeNode">' +
                    '          <div dojoAttachPoint="containerHeaderNode" class="com-ibm-asq-common-web-ui-collapsible-section-content-area-header" waiRole="region" tabindex="-1"></div>' +
                    '          <div dojoAttachPoint="containerNode,contentNode" class="com-ibm-asq-common-web-ui-collapsible-section-content-area" waiRole="region" tabindex="-1"></div>' +
                    '        </div>' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
                    '  <div class="com-ibm-asq-common-web-ui-collapsible-section-content-bottom-container" dojoAttachPoint="bottomContainer">' +
                    '    <div dojoAttachPoint="bottomLeftBorder" class="com-ibm-asq-common-web-ui-collapsible-section-content-bottom-left">' +
                    '      <div dojoAttachPoint="bottomRightBorder" class="com-ibm-asq-common-web-ui-collapsible-section-content-bottom-right">' +
                    '        <div dojoAttachPoint="bottomMidBorder" class="com-ibm-asq-common-web-ui-collapsible-section-content-bottom-mid"></div>' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
					//The span below is used to programatically find the pixel length of a string.  Since it is off screen it is not seen nor does it interfere
					//with the page that it is on.  By having a visible but unseen span, the text can be assigned to the innerHTML and then the offsetWidth of
					//the text can be retrieved from the span.  This allows text to be "hyphenated" so that it will wrap appropriately within its confines
					//without having to have some predefined length for wrapping set up.  
					'  <span class="com-ibm-asq-common-web-ui-collapsible-section-title" dojoAttachPoint="rulerNode" style="position: absolute; left: -3000px;"></span>' +
                    '  <div class="com-ibm-asq-common-web-ui-collapsible-section-bottom-border" dojoAttachPoint="bottomNode"></div>' +
                    '</div>',

    collapsible: true,  // set to true if section is not collapsible
    titleEditable: false, //optionally make the title editable
    titleValue: "", // the value of the title that is not necessary the same as the display title
    titleExt: "", // optional extension to the title
    titleSize: null, // optional to override the font-size of the title
    titleWeight: null, // optional to override the font-weight of the title
    open: false,  // default section to be collapsed
    contextHelpId: null,
	cshelp: null,
    csHelpEngine: null,
    csHelpProviderId: null,
    bgShading: true,
    widgetsInTemplate: true,
    headerActionsAlign: "left",
	
	titleExtName: MessageUtil.getMessage(MessageBundle.COMMON, "labelTitleExt"),
	titleName: MessageUtil.getMessage(MessageBundle.COMMON, "labelTitle"),

    postCreate: function() {
        this.arrowNodeInner = document.createElement("div");
        if (!this.collapsible) {
            // section must be open if not collapsible
            this.open = true;
			this.headerCollapsNode.style.display = "none"; // This is needed for IE, otherwise a red X shows uyp where the image would be
        }
        this.inherited('postCreate', arguments);
        this.setDisplayTitle(this.title);
        this.setTitle(this.titleValue);		
        this.setTitleExt(this.titleExt);
        
        this.setTitleMode(this.titleEditable);
        if(this.titleEditable) {
            // make the edit box of the title area to match our width 
            setTimeout(dojo.hitch(this, function() {
                this.viewEditableTitle.minWidthHint = Math.floor((4 * this.focusNode.offsetWidth) / 5);
            }), 30);
            
        }
        if (this.titleSize || this.titleWeight) {
            this.setTitleSize(this.titleSize, this.titleWeight);
        }
        
        this.setCSHelpId(this.contextHelpId, this.csHelpProviderId, this.csHelpEngine);
        
        if(this.headerActionsAlign == "right") {
            dojo.addClass(this.containerHeaderNode, "com-ibm-asq-common-web-ui-collapsible-section-content-area-header-right");
            dojo.addClass(this.containerNode, "com-ibm-asq-common-web-ui-collapsible-section-content-area-action-align-right");
        }
        this._onMouseOutImg();
        this._onMouseOutHandler();

		this.connect(this.domNode, "onkeypress", "_onKeyPress");
		this.connect(this.headerCollapsImg, "onkeypress", "_onKeyImagePress");

        // invoke the first time expanding handler
        if (this.open) {
            this.onFirstExpand(this);
            this.isFirstExpanding = false;
        } else {
            this.isFirstExpanding = true;
            // only need to change styele on hover when collapsed not expanded
            this.mouseOverHandler = this.connect(this.focusNode, "onmouseover", "_onMouseOver");
            this.mouseOutHandler = this.connect(this.focusNode, "onmouseout", "_onMouseOut");
        }
    },
    
    uninitialize: function()
    {
        this.headerCollapsImg.src = null;
    },
    
    // track mouse over event to render proper style when toggle is triggered
    // from key press instead of mouse click
    _mouseOver: false,
    
    // On hover section will highlight with solid fill only when closed
    _onMouseOver: function(e){
        if (!e) {
            e = window.event;
        }
        var fromNode = dojo.isIE ? e.fromElement : e.relatedTarget;

        // check if mouse in event is from moving from a child node which
        // is already in the header not from outside of the header i.e. the
        // style is already set properly
        if (dojo.isDescendant(fromNode, this.focusNode)) {
            return;
        }
        this._onMouseOverHandler();
    },
    
    _onMouseOverHandler: function(){
        this._mouseOver = true;
        if (this.collapsible)
            this.headerCollapsNode.style.display = "";
        this.headerRightNode.style.display = "";
        if (!this.open) {
            this.focusNode.className = "com-ibm-asq-common-web-ui-collapsible-section-hover-on";
        } else {
            this.focusNode.className = "";
        }
    },
            
    _onMouseOut: function(e) {
        if (!e) {
            e = window.event;
        }
        var toNode = dojo.isIE ? e.toElement : e.relatedTarget;

        // check if mouse out event is from moving to a child node which
        // is still in the header not really out of the header
        if (dojo.isDescendant(toNode, this.focusNode)) {
            return;
        }
        this._onMouseOutHandler();
    },
    
    _onMouseOutHandler: function(){
        this._mouseOver = false;
        if (!this.open) {
            if (this.collapsible)
                this.headerCollapsNode.style.display = "none";
            this.headerRightNode.style.display = "none";
            this.focusNode.className = "";
        }
        else {
            if (this.collapsible)
                this.headerCollapsNode.style.display = "";
            this.headerRightNode.style.display = "";
        }
    },
    
    _onWhitespaceClick: function(e) {
        if (this.collapsible) {
            if (e.target && e.target == this.headerCollapsImg) {
            	dojo.stopEvent(e);
                this._onTitleClick();
            }
            if (!this.open && e.target && e.target.tagName == "DIV") {
                this._onTitleClick();
            }
            if (this.open) 
                this.focusNode.className = "";
            else 
                this.focusNode.className = "com-ibm-asq-common-web-ui-collapsible-section-hover-on";
        }
    },
    
	//Expande/Collapse the section when press Enter. The focus node is domNode
	_onKeyPress: function(e) {
		var charCode = (e.which) ? e.which : e.keyCode
		if (charCode == 13 && e.target == this.domNode) 	
			 this._onTitleClick();
	},
	
	//Expande/Collapse the section when press Enter. The focus node is image
	_onKeyImagePress: function(e) {
		var charCode = (e.which) ? e.which : e.keyCode
		if (charCode == 13 && e.target == this.headerCollapsImg) 	
			 this._onTitleClick();
	},
		
    // track mouse over event of the img to render proper style when toggle
    //  is triggered from key press instead of mouse click
    _mouseOverImg: false,
    
    //On hover section icons will appear
    _onMouseOverImg: function(e){
        if (!this.collapsible) {
            return;
        }
        this._mouseOverImg = true;
        this.headerCollapsImg.className = "com-ibm-asq-common-web-ui-collapsible-section-image-hover";
        if (this.open) {
            this.headerCollapsImg.src = ImageCache.COLLAPSE_HOV_ICON;
            this.headerCollapsImg.title = MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_Collapse");
        }
        else {
            this.headerCollapsImg.src = ImageCache.EXPAND_HOV_ICON;
            this.headerCollapsImg.title = MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_Expand");
        }
         if(e){
            dojo.stopEvent(e);
        }
    },
            
    _onMouseOutImg: function(e) {
        if (!this.collapsible) {
            return;
        }
        this._mouseOverImg = false;
        this.headerCollapsImg.className = "";
        if(this.open)
            this.headerCollapsImg.src = ImageCache.COLLAPSE_ICON;
        else
            this.headerCollapsImg.src = ImageCache.EXPAND_ICON;
    },
    
    toggle: function() {
        this.inherited('toggle', arguments);

        // Invoke the first time expanding handler on toggle.  Note that there is no need to
        // check the state because initially opened widget will invoke the handler explicitly
        if (this.isFirstExpanding) {
            this.onFirstExpand(this.firstExpandData);
            this.isFirstExpanding = false;
        }

        this.onToggle(this.toggleData);

        if (this.open) {
            // just expanded, no need to handle hover anymore
            if (this.mouseOverHandler) {
                this.disconnect(this.mouseOverHandler);
                this.mouseOverHandler = null;
            }
            if (this.mouseOutHandler) {
                this.disconnect(this.mouseOutHandler);
                this.mouseOutHandler = null;
            }
        } else {
            // just collapsed, start to handle hover
            this.mouseOverHandler = this.connect(this.focusNode, "onmouseover", "_onMouseOver");
            this.mouseOutHandler = this.connect(this.focusNode, "onmouseout", "_onMouseOut");
        }

    },

    // handler for toggle - consumer can override
    onToggle: function(toggleData) {
        this.toggleData = toggleData;
    },

    // handler to invike on expand for the first time - consumer can override
    onFirstExpand: function(firstExpandData) {
        this.firstExpandData = firstExpandData;
    },

    // hiding unused area to remove extra space in padding
    setAttributeAreaHidden: function(/* boolean */ hide) {
        this.infoNode.style.display = hide ? "none" : "block";
    },
    
    setSummaryAreaHidden: function(/* boolean */ hide) {
        this.summaryNode.style.display = hide ? "none" : "block";
    },
    
    setContentHeaderHidden: function(/* boolean */ hide) {
        this.containerHeaderNode.style.display = hide ? "none" : "block";
    },
    
    showTitleOnly: function() {
        this.setAttributeAreaHidden(true);
        this.setSummaryAreaHidden(true);
        this.setContentHeaderHidden(true);
    },

    // change handler for editable title
    editableTitleChange: function(value) {
        this.titleValue = value;
        if (this.title == "") {
            this.title = this.titleValue;
        }
        if (this.onTitleEdited && dojo.isFunction(this.onTitleEdited)) {
            this.onTitleEdited(value);
        }
    },  
    
    setTitleMode: function(editable) {
        this.titleEditable = editable;
        if (this.titleEditable) {
            this.viewEditableTitle.onChange = dojo.hitch(this, "editableTitleChange");
            
            // make the edit box of the title area to match our width 
            setTimeout(dojo.hitch(this, function() {
                this.viewEditableTitle.minWidthHint = Math.floor((4 * this.focusNode.offsetWidth) / 5);
            }), 30);
        }
        this.updateTitleMode();
    },  
    
    updateTitleMode: function() {
        this.viewEditableTitle.domNode.style.display = (this.titleEditable) ? "inline" : "none";
        this.titleNode.style.display = (this.titleEditable) ? "none" : "";         
    },
    
    // the title is not necessary the same as the displayed title
    getTitle: function() {
        return this.titleValue ? this.titleValue : this.title;
    },
    
    setTitle: function(value) {
        this.titleValue = value;
        // set display title default to the title value unless it is already set otherwise
        if (this.title == "") {
            this.setDisplayTitle(value);
        }
    },
    
    getDisplayTitle: function() {
        return this.title;
    },     
    
    setDisplayTitle: function(title) {
        this.title = title;
		var wordwrapped = this.title;
		
		if( this.title )
			wordwrapped = this.getWordWrappedString( ( ( this.titleEditable ) ? this.viewEditableTitle.domNode : this.titleNode ), this.rulerNode, this.title );
			
        if (this.titleNode) {
	        this.titleNode[_textContent] = this.title ? wordwrapped : "";
        }
        if (this.viewEditableTitle)
            this.viewEditableTitle._setDisplayValue(this.title ? wordwrapped : "");
    },
    
    setTitleSize: function(size, weight) {
        this.titleSize = size;
        this.titleWeight = weight;
        if (this.titleNode) {
            if (size)
                this.titleNode.style.fontSize = size;
            if (weight)
                this.titleNode.style.fontWeight = weight;
        }
        if (this.titleExtNode) {
            if (size)
                this.titleExtNode.style.fontSize = size;
            if (weight)
                this.titleExtNode.style.fontWeight = weight;
        }
        if (this.viewEditableTitle) {
            if (size)
                this.viewEditableTitle.displayNode.style.fontSize = size;
            if (weight)
                this.viewEditableTitle.displayNode.style.fontWeight = weight;
        }
    },
    
    setTitleExt: function(titleExt) {
		if(!titleExt || dojo.trim(titleExt).length == 0) {
			this.titleExtNode.style.display = "none";
		}
		else {
			this.titleExtNode.style.display = "";
		}
        this.titleExtNode.innerHTML = titleExt;
    },
    
    _onTitleKey: function(e){
        if (this.collapsible) {
            this.inherited('_onTitleKey', arguments);
            dojo.publish("com-ibm-asq.common-CollapsibleSection-sectionSizeChanged", null);
        }
        if(e){
            dojo.stopEvent(e);
        }
    },
    
    _onTitleClick: function(e) {
        if (this.collapsible) {
            this.toggle();
            this.updateTitleMode();
            dojo.publish("com-ibm-asq.common-CollapsibleSection-sectionSizeChanged", null);
            this._onMouseOutImg();
        }
        if(e){
            dojo.stopEvent(e);
        }
    },
    
    setCSHelpId: function(helpId, csHelpProviderId, csHelpEngine){
        this.contextHelpId = helpId;
        this.csHelpProviderId = csHelpProviderId;
        this.csHelpEngine = csHelpEngine;
        if (this.titleHelpNode) {
            dojox.data.dom.removeChildren(this.titleHelpNode);
            if (this.contextHelpId) {
                var args = {
                    'contextID': helpId,
                    'csHelpEngine': this.csHelpEngine,
                    'csHelpProviderId': this.csHelpProviderId
                };
                this.cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp(args);
                this.titleHelpNode.appendChild(this.cshelp.domNode);
            }
        }
    },
    
	setCSHelpButtonLabel: function(/* REQUIRED */ cshelpLabel) {
		if(this.cshelp)
			this.cshelp.setButtonLabel(cshelpLabel);
	},
	
	//Focus on the Section's domNode. 
	setFocus: function() {
		var node = this.domNode;
		// I have seen an error occasionally here, so we guard against UI not yet ready - see WI 33896
		if (node.clientHeight > 0){
			node.focus();
		}
	},
	
    _setCss: function() {
        this.inherited('_setCss', arguments);
        
        if (this.collapsible) {
            // Handle mouse out style if toggle is from key press instead of mouse click.
            //
            if (this._mouseOver) {
                this._onMouseOverHandler();
            } else {
                this._onMouseOutHandler();
            }
            
            this.headerCollapsImg.className = this._mouseOverImg ? "com-ibm-asq-common-web-ui-collapsible-section-image-hover" : "";
            if (this.open) {
                this.headerCollapsImg.src = this._mouseOverImg ? ImageCache.COLLAPSE_HOV_ICON : ImageCache.COLLAPSE_ICON;
            } else {
                this.headerCollapsImg.src = this._mouseOverImg ? ImageCache.EXPAND_HOV_ICON : ImageCache.EXPAND_ICON;
            }
        }
        
        // The header cursor is set to pointer on hover to allow toggle on empty are so
        // all children without specified style will inherit the pointer cursor.  Set
        // the cursor of the child if it is not explicitly set to have default cursor.
        //
        if (this.infoNode.style.display == "block") {
            dojo.forEach(this.infoNode.childNodes, function(child){
                // don't change the style if the node is a text type. On
                // IE, it doesn't support style.
                if(dojo.isIE && child.nodeName && child.nodeName.indexOf("text") > -1)
                    return;
                if (!child.style || !child.style.cursor) {
                    dojo.addClass(child, "com-ibm-asq-common-web-ui-collapsible-section-hover-off-cursor");
                }
            });
        }
        if (this.summaryNode.style.display == "block") {
            dojo.forEach(this.summaryNode.childNodes, function(child) {
                // don't change the style if the node is a text type. On
                // IE, it doesn't support style.
                if(dojo.isIE && child.nodeName && child.nodeName.indexOf("text") > -1)
                    return;
                if (!child.style || !child.style.cursor) {
                    dojo.addClass(child, "com-ibm-asq-common-web-ui-collapsible-section-hover-off-cursor");
                }
            });
        }
        // If the section shouldn't show any background shadings, change all border 
        // nodes to no background border css.
        if(!this.bgShading) {
            var borders = ["headerLeftBorder", "headerMidBorder", "headerRightBorder",
                           "bottomLeftBorder", "bottomMidBorder", "bottomRightBorder"];
            
            dojo.forEach(borders, dojo.hitch(this, function(borderNode){
                var className = this[borderNode].className + "-no-bg";
                this[borderNode].className = className;
            }));
        }
        // Need to toggle the background style to clear when collapsed.
        //
        var headerStyleExt = !this.bgShading ? "-none" : (this.open ? "-open" : "-closed");
        var styleExt = this.open ? "-open" : "-closed";
        this.headerBack.className = "com-ibm-asq-common-web-ui-collapsible-section-backing" + headerStyleExt;
        this.headerBackContainer.className = "com-ibm-asq-common-web-ui-collapsible-section-backing-container" + styleExt;
        this.titleHelpNode.className = "com-ibm-asq-common-web-ui-collapsible-section-title-help" + styleExt;       
        this.headerContainer.style.display = this.open ? "block" : "none";
        this.bottomContainer.style.display = this.open ? "block" : "none";
        this.bottomNode.style.display = this.open ? "none" : "block";
    }
});

})();