/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

EditorSection = dojo.provide("com.ibm.asq.common.web.ui.internal.view.common.EditorSection");
EditorSection.ACTION_ICON_ONLY = 0;
EditorSection.ACTION_LABEL_ONLY = 1;
EditorSection.ACTION_LABEL_AND_ICON = 2;

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

(function() { // module container

var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
var Constants = com.ibm.asq.common.web.client.internal.Constants;
var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;

dojo.declare("com.ibm.asq.common.web.ui.internal.view.common.EditorSection",
com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection,
{    
    constructor: function() {
        this.editable = false;  
		this.hideToolbarOnReadOnly = false;
		this.actionDisplay = EditorSection.ACTION_LABEL_AND_ICON;
        this.title = null;
        this.summary = null;
        this.summaryValue = null;
		this._actions = [];
				
		//AbstractArtifactEditorSection
		this.sectionProvider = null;
    },    

    postCreate: function() {
		this.inherited('postCreate', arguments);		
		if (!this.sectionProvider) {
			throw new Error("EditorSection: parameter 'sectionProvider' is REQUIRED.");
		}
		if(this.summaryNodeEdit && this.summaryValue)
		  this.summaryNodeEdit.setValue(this.summaryValue);
		this.renderContent(true);		
    },
	
	renderContent: function(updateContent) {  
        //set up the content based on AbtsractSection
        this.setDisplayTitle(this.sectionProvider.getDisplayTitle());
        this.setTitle(this.sectionProvider.getTitle());
		this.setTitleExt(null);
        //this.setTitleExt(this.sectionProvider.getTitleExt());
        var summary = this.sectionProvider.getSummaryContent(this.editable);
        if (summary) {
            this._addSummaryNodes(summary ? summary : "");
            this.setSummary(summary);
            this.setDisplaySummary(this.sectionProvider.getDisplaySummaryContent());
        }
		//info - accepts text or domNode; hide node if not provided 
        var info = this.sectionProvider.getInfoContent(this.editable);
        if (info) {
            this.setInfo(info);
			this.infoNode.style.display = "block";
        } else {
			this.infoNode.style.display = "none";
		}
		//info - accepts text or domNode
        var header = this.sectionProvider.getHeaderContent(this.editable);
        if (header) {
            this.setHeader(header);
        }          
		//info - accepts action array; hide node if not provided              
        var actions = this.sectionProvider.getActions();
        if (actions) {
            this.setActions(actions);
			this.containerHeaderNode.style.display = "block";
        } else {
			this.containerHeaderNode.style.display = "none";
		}  
        var helpID = this.sectionProvider.getHelpContextID();
        if (helpID) {
            this.setCSHelp(helpID);
        }  
        this.setEditable(this.editable);        

        if (this.open && updateContent) {
            this.setMainContent(this.sectionProvider.getMainContent(this.editable));
        }
	},
	
	
	 _onSummaryKey: function(args)
     {
      this._summaryOnChange("",args);
      this.sectionProvider.setDirty();  //mark dirty                  
     },
	
	toggleEditable: function() {
		this.editable = !this.editable;
		this.renderContent(true);
	},
	
    // the title is not necessary the same as the displayed title
    getSummary: function() {
        return this.summaryValue ? this.summaryValue : this.summary;
    },
    
    setSummary: function(value) {
        this.summaryValue = value;
    },
    
    getDisplaySummary: function() {
        return this.summary;
    },
    
    setDisplaySummary: function(summary) {
        this.summary = summary;
        if (this._summaryTextNode) {
            this._summaryTextNode.innerHTML = summary;
        }
        if (this.summaryEditable && this.summaryNodeEdit) {
            this.summaryNodeEdit.value = this.summary;
        }
    },	
	
    //@Override from CollapsibleSection
    editableTitleChange: function(value) {
        if (this.title != value) {
            this.setDisplayTitle(value);
            this.setTitle(value);
            if (this.onTitleEdited && dojo.isFunction(this.onTitleEdited)) {
                this.onTitleEdited(value);
            }
        }
    },  	
	
    _onTitleClick: function() {
		//lazily load main content
		if (!this.initialized) {
			this.setMainContent(this.sectionProvider.getMainContent(this.editable));
			this.initialized = true;
		}
		//toggle open/closed state
		this.toggle();        		
		this.updateTitleMode();
        
		//nofity sectionProvider's of expand/collapse (if provided) 
		var handler = this.open ? this.sectionProvider.getExpandHandler() : this.sectionProvider.getCollapseHandler();
		if (handler) {
			handler();
		}   
        dojo.publish("com-ibm-asq.common-CollapsibleSection-sectionSizeChanged", null);
    },	
    
	_addSummaryNodes: function(summary) {
        //clear the summary node        
        dojox.data.dom.removeChildren(this.summaryNode);
		
		// create read-only the summary text node
		this._summaryTextNode = document.createElement("span");		
        dojo.addClass(this._summaryTextNode, "com-ibm-asq-common-web-ui-editor-container-summary-text");
        this.summaryNode.appendChild(this._summaryTextNode);

        // create the editable summary text node if it can be editable
        this.summaryEditable = this.sectionProvider.isSummaryEditable();
        if (this.summaryEditable) {
            this.summaryNodeEdit.valueChanged = dojo.hitch(this,"_onSummaryKey");  
        }		
        else
        {
            //If not editable, remove the editable node (just to be on the safe line)
            dojox.data.dom.removeChildren(this.summaryEditableNode);
        }

	},

    _summaryOnChange: function(oldValue, newValue) {
        this.setSummary(newValue);
        this.sectionProvider._summaryChanged(oldValue,newValue);        
    },	
    
	setActions: function(actions) {
		//action: label, actionHandler, callbackData, iconClass, disabledOnReadOnly		
		this._actions = [];
		dojox.data.dom.removeChildren(this.containerHeaderNode);
		for (var i = 0; i < (actions ? actions.length : 0); i++) {
			this._addAction(actions[i]);
		}
	},
	
    _addAction: function(action) {
		//action: label, actionHandler, callbackData, iconClass, disabledOnReadOnly
        var arg = {
            label: action.label,
            showLabel: !action.iconClass ||       // must show label if no icon is specified
                       this.actionDisplay, // label only or both icon and label
            onClick: function( e ) {
				this.clientCallback(this.callbackData, e);
            },
            'class': "com-ibm-asq-common-web-ui-editor-container-action"
        };
        
        if (action.iconClass) {
            arg["iconClass"] = action.iconClass;
        }
        var button = new dijit.form.Button(arg);
        
        // store the client callback and data into the button
        //button.parent = this;
		button.disabledOnReadOnly = action.disabledOnReadOnly;
        button.clientCallback = action.actionHandler;
        button.callbackData = action.callbackData;
        if (button.disabledOnReadOnly) {
            button.attr('disabled',!this.editable);
        }
		this._actions.push(button);
		this.containerHeaderNode.appendChild(button.domNode);
        return button;
    },

    setEditable: function(editable) {
        this.editable = editable;
        
        // set appropriate summary node to visible
        if (this.summaryEditable) {
            this._summaryTextNode.style.display = editable ? "none" : "block";
            //show the editable summary node to visible
            if(editable)
            {
                this.summaryEditableNode.style.cssText = "";
                this.summaryEditableNode.style.display = "block";
            }                        
        }
        
        // set the enablement of actions that is aware of editable mode
        for (var i = 0; i < this._actions.length; i++) {
			if (this._actions[i].disabledOnReadOnly) {
				this._actions[i].attr('disabled',!editable);
			}
        }

        // set the tool bar display only if it care to be hidden when read-only
        if (this.hideToolbarOnReadOnly) {
			this.containerHeaderNode.style.display = editable ? "block" : "none";
		}
						
    },
	
    //////////////////////////////////////////////////////////////////////////////////////
    //     Addt'l API for Editor section support
    //////////////////////////////////////////////////////////////////////////////////////        
    isEditable: function() {
        return this.editable;
    },
	
	getProvider: function() {
		return this.sectionProvider;
	},
	
    setTitleExt: function(/* String || DOM Node */ titleExt) {
        //this.titleExtNode.innerHTML = titleExt;
        dojox.data.dom.removeChildren(this.titleExtNode);
		if(!titleExt || dojo.trim(titleExt).length == 0) {
			this.titleExtNode.style.display = "none";
		}
		else {
			this.titleExtNode.style.display = "";
			this.titleExtNode.appendChild(dojo.isString(titleExt) ? document.createTextNode(titleExt) : titleExt);
		}
    },	
    
    setCSHelp: function(helpID) {
        Utility.destroyWidgetsFromNode(this.titleHelpNode);
        dojox.data.dom.removeChildren(this.titleHelpNode);

        var help = new CSHelp({contextID: helpID});
		help.setButtonLabel(this.sectionProvider.getDisplayTitle() + Constants.getMessage("labelHelpButton"));
		
        this.titleHelpNode.appendChild(help.domNode);        
    },
	
    setHeader: function(/* String || DOM Node */ header) {
        dojox.data.dom.removeChildren(this.headerRightNode);
        this.headerRightNode.appendChild(dojo.isString(header) ? document.createTextNode(header) : header);		
    },
        
    setInfo: function(/* String || DOM Node */ info) {
        dojox.data.dom.removeChildren(this.infoNode);
		this.infoNode.appendChild(dojo.isString(info) ? document.createTextNode(info) : info);		
    },
        
    setMainContent: function(domNode) {
        dojox.data.dom.removeChildren(this.containerNode);
        this.containerNode.appendChild(domNode);
    }
});

})();
