/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.common.MenuPopup");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.MenuPopup");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function() 
{
    var PAGE_FOOTER_HEIGHT = 30;
    var POPUP_TOP_BOT_TRIM = 30;
	
	var wordWrapping=com.ibm.asq.common.web.ui.internal.util.wordWrapping;
	dojo.declare(
		"com.ibm.rqm.planning.web.ui.internal.view.common.MenuPopup",
		[dijit._Widget, dijit._Templated],
		{
			duration: 200,
			templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/MenuPopup.html"),		

            // Flag to keep track if the popup menu is already displayed so the show command
            // won't play the open animation.
            //
            isDisplayed: false,
            
            // IE mouse out event can be out to the action bar (not the action entry) that is
            // underneath the pop up menu.  This event is ignored so that moving of the mouse
            // from the pop up menu to the action entry will not hide the pop up menu.
            // However, in the case that the mouse is moved from the pop up menu to an action
            // entry that is not the corresponding entry of the pop up menu, a new menu is
            // displayed and we need to hide the used-to-be currently displayed menu.
            // Do not instantiate this attribute in the constructor to use the default
            // object as a class static attribute.
            //
            currentDisplayed: { menu: null },
			
			entryNum: 0,
            
            // do animation when showing the popup menu, default to false.
            animatePopup: false,
			
			constructor: function()
			{
                // keep track of all entries to support select with arrow keys traversal
                this.allActions = [];
                this.currentAction = null;
			},
	
			postCreate: function(args)
			{
				this.bgIframe = new dijit.BackgroundIframe(this.domNode);

				// Setup fade-in and fade-out functions.
                if (this.animatePopup) {
                    this.fadeIn = dojo.fadeIn({
                        node: this.domNode,
                        duration: this.duration,
                        onEnd: dojo.hitch(this, "_onShow")
                    });
                    this.fadeOut = dojo.fadeOut({
                        node: this.domNode,
                        duration: this.duration,
                        onEnd: dojo.hitch(this, "_onHide")
                    });
                }
			},
			
			initialize: function(parent, appendTo, backgroundIconURL)
			{
				this._parent = parent;
				this._node = appendTo;
				//this._menuEntries.className = name;
				this._menuEntries.style.background = "transparent url('" + backgroundIconURL + "') bottom right no-repeat";
				
                if (this.animatePopup) {
                    dojo.style(this.domNode, "opacity", 0);
                }
				this._menuPopup.onmouseover = dojo.hitch(this, "_onMouseOver");
				this._menuPopup.onmouseout = dojo.hitch(this, "_onMouseOut");
				this._menuPopup.onclick = dojo.hitch(this, "_onClick");
//                this._menuPopup.onblur = dojo.hitch(this, "hideMenu");
			},
			uninitialize: function()
			{
//				debugger;
				var menuPop = this._menuPopup;
				if (menuPop) {
					menuPop.onmouseover=null;
					menuPop.onmouseout=null;
					menuPop.onclick=null;
				}
				var allActions = this.allActions;
				var n = allActions.length;
				for (var i = 0; i < n; ++i){
					var li = allActions[i];
					li.onmouseover = null;
					li.onmouseout = null;
					li.onkeydown = null;
					li.onclick = null;
					li.onfocus = null;
					li.onblur = null;
					li.callbackMethod = null;
					li.callbackData = null;
					li.parent = null;
				}
			},		
	
			addEntry: function(name, callback, isBold, callbackData)
			{
				var doc=document;
				if (!this._currentList)
				{
					this._currentList = doc.createElement("ul");
                    this._currentList.className = "menu-popup-contents-ul";
					this._menuEntries.appendChild(this._currentList);
				}
				
                var classname = isBold ? "menu-popup-contents-li-bold" : "menu-popup-contents-li";
                var anchor = doc.createElement("span");
                var wrapName = wordWrapping.wrapText(
                    name, 
                    classname, 
                    165, // hard code the max width for now. The dom node has not been added to the tree. 
                    true,
                    "<br/>");
                anchor.innerHTML = wrapName;

				var li = doc.createElement("li");
                li.appendChild(anchor);
				li.tabIndex = 0;
                li.isBold = isBold;
                li.className = classname;
                this._currentList.appendChild(li);

                li.callbackMethod = callback;
                li.callbackData = callbackData;
                li.parent = this;
                li.onclick = function() {
                    this.callbackMethod(this.callbackData);
                    this.className = this.isBold ? "menu-popup-contents-li-bold" : "menu-popup-contents-li";
                };
                
                li.onmouseover = function(e) {
                    this.mouseOver = true;
                    var parent=this.parent;
                    if (!parent.currentAction) {
                        // No action is in focus yet so it must be coming from the main action entry in
                        // the action bar.  Set the main action entry movingToPopup flag so that it will
                        // not remove highlight when this action li grab focus.
                        parent._parent.movingToPopup = true;
                    }
                    parent.movingWithinWidget = true;
                    this.focus();
                };
                
                li.onmouseout = function(e) {
                    this.mouseOver = false;
                    if (!e) {
                        e = window.event;
                    }
                    var toNode = dojo.isIE ? e.toElement : e.relatedTarget;
                
                    if (toNode && dojo.isDescendant(toNode, this)) {
                        // move over the anchor, ignore
                        return;
                    } else if (toNode && dojo.isDescendant(toNode, this.parent._menuPopup)) {
                        // Moved out to the popup area but still within the popup. set the
                        // movingWithinWidget flag so that the popup won't go down when
                        // this action li lost focus to another action li within the popup.
                        this.parent.movingWithinWidget = true;
                        return;
                    } else {
                        this.blur();
                    }
                };
                
                li.onfocus = function(e) {
                    // Mouse events are sent to the browser even when the browser window
                    // is not in focus.  Focus invoked from the mouse event handler is
                    // honered but not blur that is implicitly invoked from focus here.
                    // Need to invoke the blur handler here or the last menu action will
                    // stay highlighted and won't get cleared.
                    var parent=this.parent;
                    var currentAction=parent.currentAction;
                    if (currentAction) {
                        // blur the last menu action
                        currentAction.onblur();
                    }
                    // keep track that this is the current action li
                    parent.currentAction = this;
                    this.className = this.isBold ? "menu-popup-contents-li-bold-hover" : "menu-popup-contents-li-hover";
                };
                
                li.onblur = function(e) {
                    // reset current action li
                    var parent=this.parent;
                    parent.currentAction = null;
                    this.className = this.isBold ? "menu-popup-contents-li-bold" : "menu-popup-contents-li";
                    
                    if (parent.movingWithinWidget) {
                        // ignore blur if moving among the widget
                        parent.movingWithinWidget = false;
                    } else {
                        // This covers the case that the action li is focused from keyboard and the mouse
                        // click on other part of the page.  However, if the browser window is not on top
                        // clicking on this action li will bring the browser window to focus (mouse down)
                        // which will trigger this blur to hide the pop up.  The mouse up to complete the
                        // click will no longer have the pop up to invoke the action.  IE has the support
                        // of activate event so it has no problem invoking the action.
                        if (dojo.isMozilla && this.mouseOver) {
                            // Mouse over and blur without mouse out, must be the case of browser not
                            // on top so do not hide the pop up menu.  Let the pop up menu do the hiding.
//                            console.debug("blur with mouse in without mouse out!!");
                        } else {
                            parent.hideMenu();
                        }
                    }
                };
                
				li.onkeydown = dojo.hitch(this, "_doKeyPress", callback, callbackData);
                
				if (this.entryNum == 0) {
					this.firstLi = li;
				}
                // keep reference of all actions in an array for arrow keys traversal
                this.allActions.push(li);
                li.entryIndex = this.entryNum++;
			},
           
			addSeparator: function()
			{
				var separator = document.createElement("div");
				separator.id = "menu-separator";
				this._menuEntries.appendChild(separator);
				
				this._currentList = null;
			},	
			
			_onClick: function(e)
			{
				this.hideMenu();
			},
				
			_onMouseOver: function(e)
			{
                if (dojo.isIE) {
                    // In IE, mouse out event is triggered from elements behind the popup menu
                    // on both sides of the popup (outside of the DIV with the UL but within the
                    // popup menu DIV).  Setting the movingToPopup flag on the main action entry
                    // icon to prevent bringing down the popup from focus changing while the mouse
                    // is still within the popup menu.
                    this._parent.movingToPopup = true;
                }
			},
			
			_onMouseOut: function(e)
			{
                if (!e)
                {
                    e = window.event;
                }
                var toNode = dojo.isIE ? e.toElement : e.relatedTarget;
				
                if (toNode) {
                                
				    if (dojo.isDescendant(toNode, this._menuPopup))
				    {
                        // false event; moved to the child node; ignore.
					   return;
				    }

                    if (dojo.isDescendant(toNode, this._parent._hoverArea))
                    {
                        // false event; moved to the parent (action bar entry); ignore.
                        if (this.currentAction) {
                            // There is a focused action that will blur when the main
                            // action bar entry grab focus.  Set flag to ignore the blur
                            // to not bringing down the popup.
                            this.movingWithinWidget = true;
                        }
                        return;
                    }

                    // The mouse is still within the pop up menu but mouse out even is
                    // triggerred to the element underneath (the action bar).  Check
                    // the mouse position to determine if the mouse is indeed out of
                    // the pop up menu.
                    if (dojo.isIE) {
                        var menuDim = dojo.marginBox(this._menuPopup);
                        if (((e.x > menuDim.l) && (e.x < menuDim.l + menuDim.w)) &&
                            ((e.y > menuDim.t) && (e.y < menuDim.t + menuDim.h))) {
                            // false event; mouse is still within the pop up menu; ignore.
                            return;
                        }
                    }
                }
                // Blur won't do anything if it is not in focus so to be safe hide the popup
                // menu in case the parent action entry in the action bar is not in focus.
                this.hideMenu();
                this._parent.movingToPopup = false;
                this._parent._hoverArea.blur();
			},
			
            _calculateTopOffset: function(node) {
                var topOffset = 0;
    
                if (node.offsetParent) {
                    while (node) {
                        topOffset += node.offsetTop;
                        node = node.offsetParent;
                    }
                } else if (node.y) {
                    topOffset += node.y;
                }
            
                return topOffset;
            },    

            _calculateLeftOffset: function(node) {
                var leftOffset = 0;
    
                if (node.offsetParent) {
                    while (node) {
                        leftOffset += node.offsetLeft;
                        node = node.offsetParent;
                    }
                } else if (node.x) {
                    leftOffset += node.x;
                }
            
                return leftOffset;
            },    

			showMenu: function(img, e)
			{
                if (this.isDisplayed) {
                    // the menu is already displayed, ignore
                    return;
                }
                this.isDisplayed = true;

                if (dojo.isIE) {
                    // IE tweak to hide last displayed pop up menu
                    if (this.currentDisplayed.menu) {
                        this.currentDisplayed.menu.hideMenu();
                    }
                    this.currentDisplayed.menu = this;
                }

                var buttonTop = this._calculateTopOffset(img.parentNode) + 3;
                var preferedTop = buttonTop;
                var preferedLeft = this._calculateLeftOffset(img) + img.offsetWidth + 15;
                
                var thisStyle=this._menuPopup.style;
				thisStyle.top = preferedTop + "px";
				thisStyle.left = preferedLeft + "px";

				var doc=document;
                var docElement=doc.documentElement;
                // get the document height
                var docHeight = docElement.scrollHeight;
//                console.debug("document height: " + docHeight);
                
                // get the visible height of the document window
                var docVisibleHeight = docElement.clientHeight;
//                console.debug("visible height: " + docVisibleHeight);

                // get the scroll height offset
                var scrollbarOffset = docElement.scrollTop;
//                console.debug("visible top: " + scrollbarOffset);

                // determine the visible document height
                var totalVisibleHeight = docVisibleHeight + scrollbarOffset;
//                console.debug("visible bottom: " + totalVisibleHeight);
        
                // get the popup menu height
                var menuHeight = this._actionMenu.clientHeight;

                // determine the popup absolute menu top (offset of the banner height) and bottom position
                var menuTop = preferedTop;
                var menuBottom = menuTop + menuHeight;
//                console.debug("menu top: " + menuTop + " menu bottom: " + menuBottom);
                
                if (menuBottom > totalVisibleHeight) {
                    // the popup extends beyond the visible bottom, move up
                    //
                    var diff = menuBottom - totalVisibleHeight;
//                    console.debug("move up " + diff);
                    
                    if ((docHeight - totalVisibleHeight) < PAGE_FOOTER_HEIGHT) {
                        // the footer is exposed, we have to move higher to give room for it
                        diff += PAGE_FOOTER_HEIGHT - (docHeight - totalVisibleHeight);
//                        console.debug("move up " + (PAGE_FOOTER_HEIGHT - (docHeight - totalVisibleHeight)) + " more for footer");
                    }
                    
                    menuTop -= diff;
                    thisStyle.top = menuTop + "px";
                }
                
                // check if the menu popup top is off the window top
                //
                if (menuTop < scrollbarOffset) {
//                    console.debug("menuTop: " + menuTop + " scrollbarOffset: " + scrollbarOffset);

                    menuTop = scrollbarOffset + 5;
                    thisStyle.top = menuTop + "px";
                }
                
                // default arrow position and style (lighter icon when further down)
                var thisArrowStyle=this._menuArrow.style;
                thisArrowStyle.top = "6px";
                this._menuArrow.className = "menu-arrow-top";

                // figure out the difference from the mebu top to determine the arrow's top
                var imgOffsetTop = buttonTop;
                
                if ((imgOffsetTop + 7) > menuTop) {
                    // higher than original
                    var arrowOffset = (imgOffsetTop + 7) - menuTop;
                    thisArrowStyle.top = Math.min(arrowOffset, menuHeight - POPUP_TOP_BOT_TRIM) + "px";
                    if (arrowOffset > 75) {
                        this._menuArrow.className = "menu-arrow-bot";
                    } else if (arrowOffset > 50) {
                        this._menuArrow.className = "menu-arrow-low";
                    } else if (arrowOffset > 25) {
                        this._menuArrow.className = "menu-arrow-med";
                    }
                }
                
                if (dojo.isIE) {
                    // setup a backing DIV behind the popup menu to prevent any widget behind the
                    // popup menu from generating a false mouse out event when the mouse is outside
                    // of the popup menu UL content DIV but still within the popup menu DIV
                    var thisBackingStyle=this._menuPopupBacking.style;
                    thisBackingStyle.top = thisStyle.top;
                    thisBackingStyle.left = preferedLeft + 5 + "px"; // 5 pixel inset to the right
                    // make the backing DIV 5 pixel narrower and offset to the right to prevent
                    // blocking the round corner at the top left and right
                    thisBackingStyle.width = (this._menuPopup.clientWidth - 10) + "px";
                    thisBackingStyle.height = this._menuPopup.clientHeight + "px";
                }
                if (this.animatePopup) {
                    dojo.style(this.domNode, "opacity", 0);
                    this.fadeIn.play();
                }
                else {
                    dojo.style(this.domNode, "display", "");
                    this._onShow();
                }
			},
			
			_onShow: function()
			{
                if (dojo.isIE) {
                    // for IE show the backing DIV after the fade in animation
                    this._menuPopupBacking.style.display = "block";
                }
			},
			
			hideMenu: function(e)
			{
                if (this.isDisplayed) {
                    // check if event is specified
                    if (dojo.isIE && e) {
                        // check if the mouse is within the menu boundary which
                        // we should ignore the event to hide the menu
                        var menuDim = dojo.marginBox(this._menuPopup);
                        if (((e.x > menuDim.l) && (e.x < menuDim.l + menuDim.w)) &&
                            ((e.y > menuDim.t) && (e.y < menuDim.t + menuDim.h))) {
                            // false event; mouse is still within the pop up menu; ignore.
                            return;
                        }
                    }
                    if (this.animatePopup) {
                        this.fadeIn.stop();
                        this.fadeOut.play();
                    }
                    else {
                        this._onHide();
                    }
                    
                    if (this.currentAction) {
                    	// clear the highlight on the current action
                        this.currentAction.className = this.currentAction.isBold ? "menu-popup-contents-li-bold" : "menu-popup-contents-li";
                    }
                    this.currentAction = null;
                    
                    this.isDisplayed = false;
                    this._parent._setHoverImage(false);
                    this._parent._isMenuShowing = false;
                    this._node.removeChild(this.domNode);
                
                    if (dojo.isIE) {
                        this.currentDisplayed.menu = null;
                    }
                }
			},
			
			showMenuWithFocus: function(img, e) {
				this.showMenu(img, e);
				if (this.firstLi)
                    this.firstLi.focus();
			},
            
            setActionFocus: function() {
                if (this.currentAction) {
                    this.currentAction.focus();
                } else if (this.firstLi) {
                    this.firstLi.focus();
                }
            },
			
			_doKeyPress: function(callback, callbackData , e) {
                if (!e) {
                    e = window.event;
                }
                if ((e.type == "keydown") && e.keyCode) {
                    if (e.keyCode == dojo.keys.ENTER) {
                        this.hideMenu();
                        callback(callbackData);
                    } else if (e.keyCode == dojo.keys.LEFT_ARROW) {
                        // move focus back to the action entry
                        this.movingWithinWidget = true;
                        this._parent.movingToPopup = false;
                        this._parent._hoverArea.focus();
                    } else if (e.keyCode == dojo.keys.UP_ARROW) {
                        // select the previous action
                        this.movingWithinWidget = true;
                        this.selectPrevAction();
                        dojo.stopEvent(e);
                    } else if (e.keyCode == dojo.keys.DOWN_ARROW) {
                        // select the next action
                        this.movingWithinWidget = true;
                        this.selectNextAction();
                        dojo.stopEvent(e);
                    } else if (e.keyCode == dojo.keys.TAB) {
                        // select next or previous action based on if the shift key is pressed
                        // or navigate back to the main action entry icon
                        this.movingWithinWidget = true;
                        this.handleTabKey(e.shiftKey);
                        dojo.stopEvent(e);
                    }
                }
			},
			
            selectPrevAction: function() {
                if (this.currentAction) {
                    // check previous entry index
                    var prevIndex = this.currentAction.entryIndex - 1;
                    if (prevIndex < 0) {
                        // loop around to the last index
                        prevIndex = this.entryNum - 1;
                    }
                    this.allActions[prevIndex].focus();
                }
            },

            selectNextAction: function() {
                if (this.currentAction) {
                    // check next entry index
                    var nextIndex = this.currentAction.entryIndex + 1;
                    if (nextIndex >= this.entryNum) {
                        // loop around to the first index
                        nextIndex = 0;
                    }
                    this.allActions[nextIndex].focus();
                }
            },
            
            handleTabKey: function(isShift) {
                if (this.currentAction) {
                    var currentIndex = this.currentAction.entryIndex;
                    if (isShift) {
                        // moving backward
                        if (--currentIndex < 0) {
                            // first item, tab back to the parent icon
                            this._parent._hoverArea.focus();
                        } else {
                            this.allActions[currentIndex].focus();
                        }
                    } else {
                        // moving forward
                        if (++currentIndex >= this.entryNum) {
                            // last action, tab back to the parent icon
                            this._parent._hoverArea.focus();
                        } else {
                            this.allActions[currentIndex].focus();
                        }
                    }
                }
            },
            
			_onHide: function()
			{
                if (dojo.isIE) {
                    // for IE hide the backing DIV after the fade out animation
                    this._menuPopupBacking.style.display = "none";
                }
			}
		}
	);
}
)();