/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.common.NewTestPlanPanel");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.NewTestPlanPanel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.template.TemplateClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function(){
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var TemplateClient = com.ibm.rqm.planning.web.client.internal.template.TemplateClient;
    var SelectUtility = dojo.getObject("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
    
    /**
     *
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.NewTestPlanPanel", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/NewTestPlanPanel.html"),
        
        labelname: Constants.getMessage("labelname"),
        labeldescription: Constants.getMessage("labeldescription"),
        labelTemplate: Constants.getMessage("labelTemplate"),
        labelowner: Constants.getMessage("labelowner"),
        labelSaveArtifact: Constants.getMessage("labelSaveArtifact"),
        
        constructor: function(args){
            this._templates = null;
            this.isDisabled = false;
            this.isInSection = false;
        },
        
        postCreate: function(){
            if (this.defaultSummary) {
                this._name.value = this.defaultSummary;
            }
            if (this.defaultDesc) {
                this._description.value = this.defaultDesc;
            }
            this._loadTemplates();
            
            EditorUtil.initContributors(this._owner, null);
            
        },
        
        _loadTemplates: function(){
            //load the templates
            SelectUtility.add(this._template, new Option(Constants.getMessage("labelLoading"), ""));
            var srh = new ServiceResponseHandler(this, "_setTemplates", "_error");
            TemplateClient.getTemplates(srh, {
                editorId: Constants.EDITOR_ID_PLAN
            });
        },
        
        _setTemplates: function(templates){
            this._templates = templates;
            dojox.data.dom.removeChildren(this._template);
            for (var i = 0; i < (this._templates ? this._templates.length : 0); i++) {
                    var option = new Option(Constants.getExtensionDisplayMessage(this._templates[i].name), this._templates[i].itemId);
                    option.selected = this._templates[i].isDefault;
                    SelectUtility.add(this._template, option);
            }
            if (this._name && !this._name.disabled) 
                dijit.focus(this._name);
        },
        
        _getTemplate: function(itemId){
            for (var i = 0; i < (this._templates ? this._templates.length : 0); i++) {
                if (this._templates[i].itemId == itemId) {
                    return this._templates[i];
                }
            }
            return null;
        },
        
        setDisabled: function(isDisabled){
            this.isDisabled = isDisabled;
            this._name.disabled = isDisabled;
            this._description.disabled = isDisabled;
            this._template.disabled = isDisabled;
            this._weight.disabled = isDisabled;
            this._owner.disabled = isDisabled;
            this._plan.disabled = isDisabled;
            if (this._categorySelect && this._categorySelect._selections && this._categorySelect._selections.length > 0) {
                var selections = this._categorySelect._selections
                for (var i = 0; i < selections.length; i++) {
                    selections[i].disabled = isDisabled;
                }
            }
        },
        
        getAttributes: function(){
            var item = {
                names: new Array(this._name.value),
                descriptions: new Array(this._description.value)
            }
            if (this._owner && this._owner.textContent != Constants.getMessage("labelUnassigned")) {
                //HACK - force owner into summary section's workitem until
                //we decide on an official artifact ownership strategy				
                var wiList = [{
                    state: "1",
                    section: "com.ibm.rqm.planning.editor.section.testCaseSummary",
                    owner: this._owner.value,
                    projectAreaName: EditorUtil.getCurrentProjectArea().name
                }];
                item.workItemsJsonStrings = new Array(dojo.toJson(wiList));
                item.ownerIds = new Array(this._owner.value);
            }
            else {
                item.workItemsJsonStrings = new Array("-1");
                item.ownerIds = new Array("-1");
            }
            if (this._template.value != null) {
                var template = this._getTemplate(this._template.value);
                item.templateItemId = template.itemId;
                item.templateStateId = template.stateId;
            }
            return item;
        },
        
        setAlign: function(){
            var width = dojo.marginBox(this._planTD).w + "px";
            this._categorySelect.setLabelWidth(width);
        },
        
        _error: function(e){
            alert(this.declaredClass + ": Error: " + e.message);
        }
        
    });
})();
