/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.view.common.RelatedInfo");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.ResizeNotifier");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.StationaryPane");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.SimpleCollapsibleArea");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

(function() 
{
	var ActionLinkCollapsibleArea=com.ibm.asq.common.web.ui.internal.view.common.ActionLinkCollapsibleArea;
	var CommonUtility=com.ibm.asq.common.web.ui.internal.util.Utility;
    // A stationary pane that hosts the related info.  The related info collapsible
    // area contains multiple panels and only one can be displayed at a time.
    //
    dojo.declare(
        "com.ibm.asq.common.web.ui.internal.view.common.RelatedInfoPanel",
        com.ibm.asq.common.web.ui.internal.widgets.StationaryPane,
        {
            templateString: '<div class="com-ibm-asq-common-web-ui-related-info-panel">' +
                            '<label><input type="checkbox" checked="true" dojoAttachPoint="stickyNode"' +
                            '   style="display:none;"></input></label>' +
                            '</div>',

            _infoGroup: [],      // the array of collapsible action links area in the relatedInfo area
            _actionInfo: [],
            
            postCreate: function() {
                this.inherited('postCreate', arguments);
                this._infoGroup = [];
                this._actionInfo = [];
            },
            uninitialize: function() {
            	var infoGroup = this._infoGroup;
            	if (infoGroup) {
            		var n = infoGroup.length; 
            		while( n--) {
            			infoGroup[n].group.destroy();
            		}
            		this._infoGroup=[];
            	}
            	if (this._uncacheMyPanel){	// RelatedInfo cached me.  Uncache it!
            		this._uncacheMyPanel();
            		delete this._uncacheMyPanel;
            	}
            },
            
            
            // Return if the the panel is empty i.e. no info group.
            //
            isEmpty: function() {
                return (this._infoGroup.length == 0);
            },
            
            // Get the info group with the specified title.  If the optional param "createOnNull"
            // is true, a new info group will be created if it does not exist yet.  This provides
            // complete control of the content DIV of the ActionLinkCollapsibleArea.
            //
            getInfoGroup: function(groupTitle, createOnNull) {
                for (var i = 0; i < this._infoGroup.length; i++) {
                    if (this._infoGroup[i].title == groupTitle) {
                        return this._infoGroup[i].group;
                    }
                }
                if (createOnNull) {
                    var group = new ActionLinkCollapsibleArea();
                    group.attr('title',groupTitle);
                    this.domNode.appendChild(group.domNode);
                    this.domNode.appendChild(document.createElement('p'));
                    this._infoGroup.push( { 'title': groupTitle, 'group': group } );
                    return group;
                } else {
                    return null;
                }
            },
			
			
			// Destroy the info group with the specified title.
			destroyInfoGroup: function(groupTitle) {
				var tempInfoGroup = [];
				var length = this._infoGroup.length;
				for (var i = 0; i < length; i++) {
					var infoGroup=this._infoGroup[i];
                    if (infoGroup.title != groupTitle) {
						tempInfoGroup.push(infoGroup);
                    } else {
						infoGroup.group.destroy();
					}
                }
				this._infoGroup = tempInfoGroup;
			},
            
            //This function gives the ability to get all of the actions for a related info panel.
            //The major use for this is that the actions can be checked before adding another set of the
            //same actions.  This is for the beta.  It can be removed or changed out after.  Just needed
            //a quick and dirty way to add constant data to all of the pages even when there is already
            //a group with the same title.
            getActionsForGroup: function( groupTitle )
            {
            	var length=this._actionInfo.length;
                for (var i = 0; i < length; i++) {
                	var actionInfo=this._actionInfo[i];
                    if (actionInfo.title == groupTitle) {
                        return actionInfo.actions;
                    }
                }
                
                this._actionInfo.push( { 'title': groupTitle, 'actions': [] } );
                
                return( this._actionInfo[ this._actionInfo.length - 1 ].actions );
            },
            
            // Add an action to the ActionLinkCollapsibleArea group with the specified groupTitle.
            // If the ActionLinkCollapsibleArea does not exist it will be created implicitly.
            //
            addAction: function(groupTitle, handler, label, callbackData, /* optional */ iconClass) {
                var group = this.getInfoGroup(groupTitle, true);
                var actions = this.getActionsForGroup(groupTitle);
                var action = group.addActionLink(handler, label, callbackData, iconClass);
                actions.push(action);
                return action;
            },

            // Add a link to the ActionLinkCollapsibleArea group with the specified groupTitle.
            // The link is just an action that is rendered like a link and take just the URL and label.
            // If the ActionLinkCollapsibleArea does not exist it will be created implicitly.
            //
            addLink: function(groupTitle, label, url, /* optional */ iconClass, /* optional */ anchorAsLink, /* optional */ ignoreHandler) {
                var group = this.getInfoGroup(groupTitle, true);
                var actions = this.getActionsForGroup(groupTitle);
                var link = group.addActionLink(dojo.hitch(this, "_launchUrl", url), label, null, iconClass, anchorAsLink, ignoreHandler);
                actions.push(link);
                return link;
            },

            // Internal method to launch the a browser window with the specified URL.
            //
            _launchUrl: function(url) {
//                window.open(url, "_blank");
				if(url && url.length > 0) {
					if(url.charAt(0)== '#') {
						//internal url, open it in current page
						window.location.href = url;
					}
					else{
						if(!url.match("://")){
							//external url, but not start with "http://".
							var prefix = "http://";
							url = prefix+url;
						}
						//external url, open in in a new window page
						window.open(url, "_blank");
					}
				}
				
            }
        }
    );
    
    // The related info collapsible area which contains multiple panels and only one
    // can be displayed at a time.  All panels are cached and reused based on a unique id.
    //
	dojo.declare(
		"com.ibm.asq.common.web.ui.internal.view.common.RelatedInfo",
		[dijit._Widget, dijit._Templated],
		{
            templateString: '<div class="com-ibm-asq-common-web-ui-related-info">' +
                            '  <div class="related-info-area" dojoAttachPoint="_relatedInfo"></div>' +
                            '  <span class="show-bar-open-inactive" dojoAttachPoint="_relatedInfoBar">' +
                            '		<div class="sash-bar" dojoAttachPoint="_sashBar">' +
                            '			<div class="arrow-right" dojoAttachPoint="_showHide"></div>' +
                            '		</div>' +
                            '  </span>' +
                            '</div>',
                            
            _panelCache: {},     // cache of static panel
            _currentPanel: null, // the panel that is currently displayed
			
            postCreate: function() {
                this.inherited('postCreate', arguments);
                this._isShowing = (this.open != null) ? !this.open : false;
				if(this._isShowing)
				{
					this._showHide.style.display = "none";
				}
				this.connect(this._showHide, "onclick", "_clickArrow");
				this.connect(this._relatedInfoBar, "onmouseover", "_mouseHandler");
				this.connect(this._relatedInfoBar, "onmouseout", "_mouseHandler");
                this._panelCache = {};
                this._onAnimationEnd();
            },
            
            uninitialize: function() {
//            	console.debug("======================UNINITIALIZE RELATEDINFO");
            	var cache = this._panelCache;
            	for (var id in cache) {
            		var panel = cache[id];
            		if (panel) {
            			panel.destroy();
            		}
            	}
            	this._panelCache = {};
            	
            },

			_mouseHandler: function(event) {
				var type;
				if( dojo.isIE && !event )
				{
					type = window.event.type;
				}
				else
					type = event.type;
				
				if(this._isShowing && type == "mouseout"){
					this._sashBar.style.display = "none";
					dojo.removeClass(this._relatedInfoBar, "show-bar");
					dojo.addClass(this._relatedInfoBar, "show-bar-open-inactive");
				}else if(this._isShowing && type == "mouseover"){
					this._sashBar.style.display = "block";
					dojo.removeClass(this._relatedInfoBar, "show-bar-open-inactive");
					dojo.addClass(this._relatedInfoBar, "show-bar");
				}
			},
            
            _clickArrow: function(e) {
                this._rightNavWidth = this._relatedInfo.clientWidth;
                var args = {
                    node: this._relatedInfo,
                    duration: 200,
                    properties: {
                        right: {
                            unit: "px"
                        },
                        marginLeft: {
                            unit: "px"
                        }
                    },
                    onEnd: dojo.hitch(this, "_onAnimationEnd")
                };
					
                if (this._isShowing) {
                    // hide
                    args.properties.right.start = 0;
                    args.properties.right.end = -this._rightNavWidth;
                    args.properties.marginLeft.end = -this._rightNavWidth;
                } else {
                    // show
                    args.properties.right.start = -this._rightNavWidth;
                    args.properties.right.end = 0;
                    args.properties.marginLeft.end = 0;
                    this._relatedInfo.style.display = "block";
                }
                
                var anim = dojo.animateProperty(args);
                anim.play();
                
                // Tweak to work around a dojo bug with the TabContainer resizing.
                // Resize the window to trigger the TabContainer to resize after
                // openning and closing the sash.
                //
//                window.resizeBy(0, 1);
//                window.resizeBy(0, -1);
            },
            
            _onAnimationEnd: function() {
                if (this._isShowing) {
                    dojo.removeClass(this._showHide, "arrow-right");
                    dojo.addClass(this._showHide, "arrow-left");
                    this._isShowing = false;
                    this._relatedInfo.style.display = "none";
					this._sashBar.style.display = "block";
					dojo.removeClass(this._relatedInfoBar, "show-bar-open-inactive");
					dojo.addClass(this._relatedInfoBar, "show-bar");
					
					//For defect 10092: deal with Chrome separately
					if(navigator.userAgent.indexOf("Chrome") >= 0)
						this._showHide.style.left = (this._relatedInfoBar.offsetLeft + 10) + "px";
                } else {
                    dojo.removeClass(this._showHide, "arrow-left");
                    dojo.addClass(this._showHide, "arrow-right");
                    this._isShowing = true;
					this._sashBar.style.display = "none";
					dojo.removeClass(this._relatedInfoBar, "show-bar");
					dojo.addClass(this._relatedInfoBar, "show-bar-open-inactive");
					
					//For defect 10092: deal with Chrome separately
					if(navigator.userAgent.indexOf("Chrome") >= 0)
						this._showHide.style.left = "";
                }
				if(dojo.isFF == 2)
					this.parent.switchBidiContentArea(this._isShowing);
                
                // The related info sash has moved. Generate a resize notification
                // to whomever interested in the content area size changed.
                setTimeout(jazz.util.ResizeNotifier.poke, 3);    
            },
            
            // Set the dynamic panel to display in the related info area.  If there
            // isn't one, a new dynamic panel will be created.
            //
            setCurrentDynamicPanel: function() {
                this.setCurrentPanel('_DYNAMIC', this.getDynamicPanel());
            },
			
			// Return the current panel. If there is no current panel, null is returned, 
			// so make sure to check the value before using it. 
			getCurrentPanel: function(){
				return this._currentPanel;
			},
            
            // Set the panel to display in the related info area. The specified panel
            // must be a RelatedInfoPanel widget. This method caches the panel into the
            // cache to reuse.
            //
            setCurrentPanel: function(id, panel) {
                if (this._getCachedPanel(id)) {
                    // the panel is already cached
                } else {
                    // cache panel based on id
                    this._cachePanel(id, panel);
                }

                if (this._currentPanel) {
                    // remove the current displayed panel if there is one
                    this._relatedInfo.removeChild(this._currentPanel.domNode);
                }
                // update the current display panel
                this._currentPanel = this._getCachedPanel(id);
                this._relatedInfo.appendChild(this._currentPanel.domNode);
            },

            // Returns the panel widget with dynamic contents.  If "clear" is
            // is specified, a new panel will be created.
            //
            getDynamicPanel: function(/* boolean */ clear) {
                if (clear) {
                	var cachedPanel = this._panelCache['_DYNAMIC'];
                    if (cachedPanel) {
                    	cachedPanel.destroyRecursive();
                        this._panelCache['_DYNAMIC'] = null;
                    }
                }
                if (!this._getCachedPanel('_DYNAMIC')) {
                    this._cachePanel('_DYNAMIC', new com.ibm.asq.common.web.ui.internal.view.common.RelatedInfoPanel());
                }
                return this._panelCache['_DYNAMIC'];
            },
            
            // Returns the panel widget if it is cached by the specified id.
            // 
            _getCachedPanel: function(id) {
                if (this._panelCache[id])
                    return this._panelCache[id];
                return null;
            },

            // Save the RelatedInfoPanel widget in the cache.
            //
            _cachePanel: function(id, panel) {
                this._panelCache[id] = panel;
                panel._refId = id;
                panel._uncacheMyPanel = dojo.hitch(this, function() {
                    var panel = this._panelCache[id];
                    if (panel) {
                        // if the panel we're being uncached is our 
                        // current panel, remove the ref too.
                        if(this._currentPanel && this._currentPanel._refId == panel._refId) {
                            this._currentPanel = null;
                        }
                        delete this._panelCache[id];
                    }
                });
            },

            // Get the info group with the specified title in the current panel.
            //
            getInfoGroup: function(groupTitle, createOnNull) {
                // use a cleared dynamic panel if there is no current displayed panel
                if (!this._currentPanel) {
                    this.setCurrentPanel('_DYNAMIC', this.getDynamicPanel(true));
                }
                return this._currentPanel.getInfoGroup(groupTitle, createOnNull);
            },
            
            // Add an action to the ActionLinkCollapsibleArea group with the
            // specified groupTitle in the current panel.
            //
            addAction: function(groupTitle, handler, label, callbackData, /* optional */ iconClass) {
                // use a cleared dynamic panel if there is no current displayed panel
                if (!this._currentPanel) {
                    this.setCurrentPanel('_DYNAMIC', this.getDynamicPanel(true));
                }
                this._currentPanel.addAction(groupTitle, handler, label, callbackData, iconClass);
            },

            // Add a link to the ActionLinkCollapsibleArea group with the
            // specified groupTitle in the current panel.
            //
            addLink: function(groupTitle, label, url, /* optional */ iconClass) {
                // use a cleared dynamic panel if there is no current displayed panel
                if (!this._currentPanel) {
                    this.setCurrentPanel('_DYNAMIC', this.getDynamicPanel(true));
                }
                this._currentPanel.addLink(groupTitle, label, url, iconClass);
            }

        });

    // A mixin class that defines the interfaces to specify the related info of a
    // widget.  A widget can contribute static or dynamic related info but not both.
    //
    dojo.declare(
        "com.ibm.asq.common.web.ui.internal.view.common.RelatedInfoMixin", null,
        {
            // Return the static RelatedInfoPanel of a widget.  Together with the
            // id, it will be cached by the RelatedInfo container widget.  The
            // dynamic setupRelatedInfo() method will be ignored.
            //
            getStaticRelatedInfo: function() {
                return null;
            },
            
            // Return the unique id for the RelatedInfoPanel.
            //
            getStaticRelatedInfoId: function() {
                return null;
            },
            
            // Handler to set up the related info in the specified RelatedInfoPanel.
            // The method will only be invoked if the getStaticRelatedInfo() returns
            // null for no static panel.
            //
            setupRelatedInfo: function(relatedInfoPanel){
            }
        });
    })
();
