/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.SectionWorkItem");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SimpleWorkItemPane");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.QMWorkItemPane");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.WorkItemUtils");

(function(){ // module container
	var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var ImageClient = com.ibm.team.workitem.web.client.internal.ImageClient;
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var TooltipBuilder = com.ibm.asq.common.web.ui.internal.util.TooltipBuilder;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var WorkItemUtils = com.ibm.rqm.planning.web.client.internal.WorkItemUtils;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.SectionWorkItem", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/SectionWorkItem.html"),
		
		labelWorkItem: Constants.getMessage("labelWorkItem"),
		labelType: Constants.getMessage("labelAuthoring"),
        isSaved: false,
        ///////////////////////////////////////////////////////////////////////////////////
        // Called on creation
        ///////////////////////////////////////////////////////////////////////////////////				
        constructor: function(){
            this.workItem = null;
            this.artifact = null;
            this.artifactSectionId = null;
            this.editable = false;
			this.wiLink = null;
        },
        
        postCreate: function(args){
            if (!this.sectionId) {
                throw new Error("Argument 'sectionId' is REQUIRED!");
            }
            this.initialize();
        },
        
        initialize: function(){
			dojox.data.dom.removeChildren(this._workItemLink);
			if(!this.editable && !this.workItem) {
				this._workItemLink.appendChild(document.createTextNode(Constants.getMessage("labelNoWorkItem")));
				return;
			}
			this.wiLink = document.createElement("a");
			this.wiLink.className = "wiLink";
            this.connect(this.wiLink, "onclick", this._displayWorkItemPopup);
			
			var sectionName = "";
			if (dojo.isFunction(this.parentSection.getDisplayTitle)) {
			 	sectionName = this.parentSection.getDisplayTitle();
			}
			var textData = " <span style='left:-3000px; position:absolute; display:inline;'>";
			if(this.workItem != null) {	
                this.setWorkItem(this.workItem);
                this._setStateIcon();			
				//for acc, defect 7806
				textData += Constants.getMessage("labelSectionWorkItemInfo", sectionName) + "</span>" + this.getWorkItemLabel();
				this.wiLink.innerHTML = textData;
				this.workItemTip = this._addTooltip(this.wiLink);
			} else {
				this.wiLink.innerHTML = Constants.getMessage("labelCreate") + textData + Constants.getMessage("labelSectionCreateWorkItemInfo", sectionName) + "</span>";
			}
			this._workItemLink.appendChild(this.wiLink);
        },
        
		_displayWorkItemPopup: function(defaultOwnerId){
            var self = this;
			if (this.workItem) {
				var args = {
					callback: {
						method: dojo.hitch(self, "_updateWorkItem"),
						returnAsDTO: true
					},
					workItem: self.workItem != null ? self.workItem : null,
					sectionId: self.sectionId,
					workItemTypeId: "authoring"
				}
				this._wiPopup = new com.ibm.rqm.planning.web.ui.internal.view.common.QMWorkItemPane(args);
			
			} else if (!this.workItem){
				var tempWIInfo = this.getWorkItemInfo();
				this._wiPopup = new com.ibm.rqm.planning.web.ui.internal.view.common.SimpleWorkItemPane({
					artifact: this.artifact,
					defaultSummary: this._getDefaultSummary(),
					sectionId: this.sectionId,
					editable: this.editable,
					workItemInfo: tempWIInfo,
					parent: self				
				});
			}
        },
		
        ///////////////////////////////////////////////////////////////////////////////////
        // API
        ///////////////////////////////////////////////////////////////////////////////////    		
        getWorkItem: function(){
            return this.workItem;
        },
        
        getOwnerId: function() {
			var wiInfo = this.getWorkItemInfo(); 
			return wiInfo ? wiInfo.owner : null;
        },
		
		getWorkItemLabel: function() {
			var workItem = this.workItem;
			var label = workItem.id + " (";
            if (workItem && WorkItemUtils.getType(workItem) == WorkItemUtils.TYPE_SUMMARY) {
                label += workItem.ownerName + ")";
            } else {
                label += workItem.owner.name + ")";
            }			
			return label;
		},
        
		
        getWorkItemInfo: function() {
			var workItem = this.workItem;
			if (workItem) {				
				var wiInfo = {};
				if (WorkItemUtils.getType(workItem) == WorkItemUtils.TYPE_SUMMARY) {
					wiInfo = this._fromSummaryDTO(workItem);
				} else {
					wiInfo = this._fromFullDTO(workItem);
				}
                return wiInfo;
				
			} else {
				return this._wiPopup ? this._wiPopup.getWorkItemInfo() : null;
			}
	    },
        
        getWorkflowAction: function(){
            return this._stateSelect.value;
        },
        
		setEditable: function(editable){
            this.editable = editable;
        },
		
		setWorkItem: function(workItemDTO) {
            if (WorkItemUtils.getType(workItemDTO) == WorkItemUtils.TYPE_FULL) {
                this._fullDTO = workItemDTO;
                this.workItem = WorkItemUtils.toSummaryDTO(workItemDTO);
            } else {
                this.workItem = workItemDTO;
            }			
		},
		
        ///////////////////////////////////////////////////////////////////////////////////
        // Internal
        ///////////////////////////////////////////////////////////////////////////////////  
        _fromSummaryDTO: function(/* WorkItemSummaryDTO */ dto) {
            var wiInfo = {};
            wiInfo.section = this.sectionId;
            wiInfo.state = dto.state;               
            wiInfo.owner = dto.ownerItemId;
            wiInfo.projectAreaName = EditorUtil.getCurrentProjectArea().name;
            wiInfo.workItemItemId = dto.workItemItemId;
            wiInfo.isSaved = this.isSaved;
            return wiInfo;          
        },
        
        _fromFullDTO: function(/* WorkItemDTO */ dto) {
            var wiInfo = {};
            wiInfo.section = this.sectionId;
            wiInfo.state = dto.attributes.internalState.id;               
            wiInfo.owner = (dto.owner != null) ? dto.owner.itemId : dto.attributes.owner.itemId;
            wiInfo.projectAreaName = EditorUtil.getCurrentProjectArea().name;
            wiInfo.workItemItemId = dto.itemId;
            wiInfo.isSaved = this.isSaved;
            return wiInfo;          
        },
				  		
		_updateWorkItem: function(/* WorkItemDTO */ workItem){
            if (workItem) {
				this.setWorkItem(workItem);				
				this._setStateIcon();
				dojox.data.dom.removeChildren(this.wiLink);
				this.wiLink.appendChild(document.createTextNode(this.getWorkItemLabel())); //attributes.id.id));
				//this.wiLink.appendChild(document.createTextNode(this.workItem.typeName+ " " + this.workItem.id));
				var workItemTip = this.workItemTip;
				if (workItemTip) {
					//update tooltip
					workItemTip.html = this._getTooltipHTML();
					workItemTip.close();
				}
				//this.parentSection.setDirty();
			} else {
				this._wiPopup = null;
			}
        },
		
        _updateWorkItemLink: function(time) {
            if (time) {
				var wiLink = this.wiLink;
				dojox.data.dom.removeChildren(wiLink);
				Utility.destroyWidgetsFromNode(wiLink);
				wiLink.appendChild(document.createTextNode(this.labelType + " <" + time + ">"));
				this.workItemTip = TooltipBuilder.addTooltip(this._getWorkItemInfoHTML(), wiLink);
				this.parentSection.setDirty();
			} else {
                dojox.data.dom.removeChildren(this._workItemLink);
				Utility.destroyWidgetsFromNode(this._workItemLink);
                var wiLink = document.createElement("a");
                wiLink.className = "wiLink";
                this.connect(wiLink, "onclick", this._displayWorkItemPopup);
                wiLink.appendChild(document.createTextNode(Constants.getMessage("labelCreate")));
                this._workItemLink.appendChild(wiLink);
				this.wiLink = wiLink;
				this._wiPopup = null;
			}
        },		
		
		_setStateIcon: function() {
            dojox.data.dom.removeChildren(this._stateIcon);
			var workItem = this.workItem;
			if (workItem) {
				var stateIcon = document.createElement("img");
				stateIcon.src = workItem.stateIconURL;
				stateIcon.title = workItem.stateName;
                this._stateIcon.appendChild(stateIcon);     
			} 			
		},
		
		_addTooltip: function(node) {
			var workItem = this.workItem;
            if (workItem) {
                if (WorkItemUtils.getType(workItem) == WorkItemUtils.TYPE_SUMMARY) {
                    return TooltipBuilder.addWorkItemSummaryTip(workItem, node);
                } else {
                    return TooltipBuilder.addWorkItemTip(workItem, node);
                }
            }
            return null;				      
		},
		
		_getTooltipHTML: function() {
			var workItem = this.workItem;
            if (workItem) {
				if (WorkItemUtils.getType(workItem) == WorkItemUtils.TYPE_SUMMARY) {
					return TooltipBuilder.getWorkItemSummaryTipHTML(workItem);
				} else {
					return TooltipBuilder.getWorkItemTipHTML(workItem);
				}
			}
			return null;
		},
		
        _getWorkItemInfoHTML: function() {
            var wiInfo = this.getWorkItemInfo();
            var html = "<table style=\"font-size: smaller; font-family:Verdana;\" border=0>";
            html += "<tr><td align=right><b>" + Constants.getMessage("labelType") +":</b></td><td>" + this.labelType + "</td></tr>";
            html += "<tr><td align=right><b>" + Constants.getMessage("labelOwner") + ":</b></td><td>" + wiInfo.ownerName + "</td></tr>";
            html += "<tr><td align=right><b>" + Constants.getMessage("labelState") + ":</b></td><td>" + Constants.getMessage("labelNew") + "</td></tr>";
            html += "</table>";
            return html;
        },
				
		_getDefaultSummary: function() {
			var parentSection = this.parentSection;
			if (parentSection && dojo.isFunction(parentSection.getParentEditor)) {
                var parentEditor = parentSection.getParentEditor();
				if (parentEditor) {
					var sectionName = Constants.getExtensionDisplayMessage(parentSection.getConfig().name);
					var artifactType = parentSection.getArtifactType();
					var artifactName = parentEditor.getHeader().getTitle();
					return Constants.getMessage("msgDefaultWorkItemSummary", sectionName, artifactType, artifactName);
				}
			}		
			return "";	
		},
        
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ SectionWorkItem#_error");
        }
    });
    
})();
