/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemDialog");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemContent");

(function(){
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemDialog", null, {
        
		width: "600px",   //default dialog width
		
		constructor: function(params) { 
            if (!dojo.isFunction(params.onOk)) {
                throw new Error("Missing function for 'onOk' parameter");
            }
            
            var contentArgs = {
                itemType: params.itemType,
                onOk: dojo.hitch(this, "_onOk"),
                onCancel: dojo.hitch(this, "_onCancel")
            }
            this.dialogContent = new com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemContent(contentArgs);
            
            var isMultiselect = params.isMultiselect == null || params.isMultiselect ? true : false; /* null = true */
            var isOKCancel = params.isOKCancel ? true : false;

			this.dialogContent.setAllowsMultiple(isMultiselect);
            this.dialogContent.setIsOKCancel(isOKCancel);            
            this.dialogContent.setExclusionList(params.exclusionList);
            
			this.show();
        },
		
		_createDialog: function() {
            var dialogArgs = {
				primaryTitle: this._getTitle(),
                isClosable: true,
                showActions: false
            };                
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.dialogContent.domNode);		
		},
		
		_getTitle: function() {
			return this.title ? this.title : Constants.getMessage("labelTitle")
		},
        
        setExclusionList: function(list){
            this.dialogContent.setExclusionList(list);
        },
        
        show: function() {
			if (!this._dialog) {
				this._createDialog();
			}
            this._dialog.open();
            this.dialogContent.setInitialFocus();
            this.dialogContent.doSearch();
        },
        
        destroyRendering: function(){
            this._dialog.destroyRendering();
            this.dialogContent.destroyRendering();
        },
		
		_onOk: function(items) {
			if (dojo.isFunction(this.onOk)) {
				this.onOk(items);
			}
		},
		
		_onCancel: function() {
			this._dialog.hide();
		}
    });
})();
