/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008 - 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function() {

var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Constants               = com.ibm.rqm.planning.web.client.internal.Constants;
var TestCaseClient		    = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
var TestScriptClient        = com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
var TestScriptTableView     = com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView;
var TestCaseTableView       = com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView;
var TestPlanTableView       = com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView;
var CSHelp                  = com.ibm.asq.common.web.ui.internal.util.CSHelp;

var RUNNING_SEARCH_MSG		= Constants.getMessage("labelRunningSearch");
var FETCH_ERR_MSG			= Constants.getMessage("msgErrorRunningQuery");

dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog",
	[dijit._Widget, dijit._Templated],
	{
		//templatePath: xojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/SelectItemTableDialog.html"),
        templateString: '<div class="com-ibm-rqm-test-common-select-item-table">' +
		                '    <div dojoAttachPoint="_msgArea" class="NormalStatus"></div>' +
						'    <div dojoAttachPoint="_itemListArea" class="ItemList"></div>' +
                        '</div>',
		
        width: "800px",   //default dialog width
        
		/*
		invokeTimes: 0,
		returnedNum: 0,
        */
        
        // If the tableContainer is specified, show the item table as the content
        // of the container instead of using a dialog.  
        tableContainer: null,
		
        constructor: function(params){
            this._items = [];
			//this._allSnapshots = [];  //all artifacts including snapshots
			dojo.mixin(this, params);			
            if (!this.itemType) {
                throw new Error("Missing parameter 'itemType' is REQUIRED!");
            }           
            if (!dojo.isFunction(this.onOk) && !this.tableContainer) {
                throw new Error("Missing function for 'onOk' parameter");
            }       
            this.setExclusionList(this.exclusionList ? this.exclusionList : []);
            this.allowSelectAll = true;
        },		
		
		postCreate: function() {            
            //only enable snapshots option for test case item type
            this._enableSnapshots = (this.itemType == Constants.TYPE_CASE || this.itemType == Constants.TYPE_SCRIPT);
			//this._showSnapshots = false;
			this.exclusionMap= {};
			this._initItemList(this.tableId);
            
            // reposition the dialog whenever the content of the table view 
            // is changed. This ensure the dialog is positioned in the center
            // in the vertical space.
            if (dojo.isFunction(this._list._searchCallback)) {
                this.connect(this._list, "_searchCallback", "repositionDialog");
            }
		},
		
        _showAsDialog: function() {
            var dialogArgs = {
                primaryTitle: this.title ? this.title : Constants.getMessage("labelTitle"),
                _yesAction: function(){
                    // override the button action to not close the
                    // dialog immediately. the base impl does check for
                    // return value to not popdown the dialog, but it
                    // will re-enable the yes button.
                    this._yesButton.disabled = true;
                    this.okCallback();
                },
                okCallback: dojo.hitch(this, "_onOK"),
                cancelCallback: dojo.hitch(this, "_onCancel"),
				_truncate: function() {}  //disable the Dialog's '_truncate'
            };          
                  
            if (this.showQueryFilters) {
                // Jazz uses super large z-index of 10000 which covers up the drop down menu
                // of the filters dojo widgets in the dialog which has z-index of 1000
                dialogArgs.zIndex = 998;
                dialogArgs.forceZIndex = true;
            }
            
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            if (this.itemType == Constants.TYPE_CASE) {
    	        var cshelp = new CSHelp({contextID: "rqm_add_test_cases.html"});
    	        cshelp.setButtonLabel(Constants.getMessage("labelAddExistingCSHelp"));
	            this._dialog.setSecondaryTitleNode(cshelp.domNode);
			} else if (this.itemType == Constants.TYPE_SCRIPT) {
	            var cshelp = new CSHelp({contextID: "rqm_add_test_scripts.html"});
	            cshelp.setButtonLabel(Constants.getMessage("labelAddExistingCSHelp"));
	            this._dialog.setSecondaryTitleNode(cshelp.domNode);
		    }	
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
			this._dialog.open(false);
			this._dialog.updatePosition();
			this._dialog.getOkButton().disabled
        },
        
        _showAsContent: function() {
            this.tableContainer.appendChild(this.domNode);
            this.okButtonDisabled(true);
        },
		
        // consumer needs to override this method to control the OK button enablement
        // when the table is used as the content in a container and not in a dialog
        okButtonDisabled: function(/* boolean */ disabled) {
            if (!this.tableContainer && this._dialog) {
                this._dialog.getOkButton().disabled = disabled;
            }
        },
        
        getSelection: function() {
            if (this._list) {
                return this._list.getSelectedItems();
            } else {
                return [];
            }
        },
        
        setSelection: function(items, force) {
            if (this._list && this._list.tableViewer) {
                // use timer to allow table rendering in a popup
                if(force) {
                    this._updateTableViewerSelection(items);
                    return;
                }
                setTimeout(dojo.hitch(this, this._updateTableViewerSelection, items), 100);
            } else {
                // the table viewer in the filter table view may not be finished with the filter query to able
                // to set selection cache the value to set on the filter table view onQueryComplete handler
                this._tableSelectionValue = items;
            }
        },
        
        _onFilterQueryComplete: function() {
            if (this._list && this._tableSelectionValue) {
                // set the selection in the table viewer of the filter table view if there is cached value
                setTimeout(dojo.hitch(this, this._updateTableViewerSelection, this._tableSelectionValue), 100);
                delete this._tableSelectionValue;
            }
        },
        
        _updateTableViewerSelection: function(items) {
            this._list.tableViewer.rowCheckedHandler.clearRowStates();
            var count = dojo.isArray(items) ? items.length : 0;
            if (count > 0) {
                for (var i = 0; i < count; i++) {
                    this._list.tableViewer.rowCheckedHandler.setRowCheckState(items[i], true);
                }
            }
            this._list.tableViewer.refresh();
        },
        
        repositionDialog: function() {
            if (this._dialog) {
                // set timeout to allow the browser to render dialog
                setTimeout(dojo.hitch(this._dialog, "updatePosition"), 100);
            }
        },
        
        _prepareCommonTableArgs: function(tableArgs) {
            tableArgs.allowSelectAll = this.allowSelectAll;
            tableArgs.isSingleSelect = this.isSingleSelect;
            tableArgs.showQueryFilters = this.showQueryFilters;
            tableArgs.minimalAction = true;
            tableArgs.displayInPopup = true;
            tableArgs.isInPicker = true;
            if(this.pageSize) {
                tableArgs.pageSize = this.pageSize;
            }
            tableArgs.onQueryComplete = dojo.hitch(this, "_onFilterQueryComplete");
            if (this.allowChangingTableSettings != undefined) {
                tableArgs.allowChangingTableSettings = this.allowChangingTableSettings;
            }
            return tableArgs;
        },
        
		_initItemList: function(tableId) {
            var tableArgs = {};
            
            // if a parent itemId is specified, pass that along to the table
            // view. The query will constraint the returned item to exclude
            // items that belongs to the parent. In that case, we don't need
            // to do post filtering on the returned items.
            // 
            if(this.parentItemIdArgName) {
                tableArgs.parentItemIdArgName = this.parentItemIdArgName;
			}
			if(this.parentVersionItemId) {
                tableArgs.parentVersionItemId = this.parentVersionItemId;
            }
			if (this.itemType == Constants.TYPE_SCRIPT) {
                tableArgs.scripts = [];
                tableArgs.tableId = tableId?tableId:"RQM-TS-SELECT-TABLEVIWER";
                tableArgs.profile = Constants.TABLE_PROFILE_MINIMAL;
				tableArgs.allowQuerySnapshots = this._enableSnapshots;
                tableArgs.queryFiltersCallback = dojo.hitch(this, "repositionDialog");
				tableArgs.postQueryFilterCallback = dojo.hitch(this, "_filterExcluded");
				tableArgs.rowCheckedListener = dojo.hitch(this, "_onTableRowsChecked");
                tableArgs = this._prepareCommonTableArgs(tableArgs);
				this._list = new TestScriptTableView(tableArgs);
			
			} else if (this.itemType == Constants.TYPE_CASE) {
                tableArgs.testCases = [];
                tableArgs.tableId = tableId?tableId:"RQM-TC-SELECT-TABLEVIWER";
                tableArgs.profile = Constants.TABLE_PROFILE_MINIMAL;
                tableArgs.allowQuerySnapshots = this._enableSnapshots;				
                tableArgs.queryFiltersCallback = dojo.hitch(this, "repositionDialog");
                tableArgs.postQueryFilterCallback = dojo.hitch(this, "_filterExcluded");		
				tableArgs.rowCheckedListener = dojo.hitch(this, "_onTableRowsChecked");
                tableArgs = this._prepareCommonTableArgs(tableArgs);
				this._list = new TestCaseTableView(tableArgs);
			} else if (this.itemType == Constants.TYPE_PLAN) {
                tableArgs.plans = [];
                tableArgs.tableId = tableId?tableId:"RQM-TP-SELECT-TABLEVIWER";
                tableArgs.staticProfile = this.staticProfile;
				tableArgs.isInSection = true;
                tableArgs.queryFiltersCallback = dojo.hitch(this, "repositionDialog");
                tableArgs.postQueryFilterCallback = dojo.hitch(this, "_filterExcluded");		
				tableArgs.rowCheckedListener = dojo.hitch(this, "_onTableRowsChecked");
                tableArgs = this._prepareCommonTableArgs(tableArgs);
				this._list = new TestPlanTableView(tableArgs);
			} else {
				//HACK - allow other types; default to script view due to KeywordList using this widget improperly
				console.warn("Warning: Unsupported ItemType: '" + this.itemType + "'; Defaults to TestScriptTableView, but Widget may not function correctly!");
                tableArgs.scripts = [];
				tableArgs.tableId = tableId?tableId:null;
                tableArgs.profile = Constants.TABLE_PROFILE_MINIMAL;
                tableArgs.queryFiltersCallback = dojo.hitch(this, "repositionDialog");
				tableArgs.rowCheckedListener = dojo.hitch(this, "_onTableRowsChecked");
                tableArgs = this._prepareCommonTableArgs(tableArgs);
                this._list = new TestScriptTableView(tableArgs);				
			}
		},
		
        show: function(items){
            if (this.tableContainer) {
                this._showAsContent();
            } else if (!this._dialog) {
                this._showAsDialog();
            }
            this.refresh(items);			            
        },
		
		refresh: function(items) {
            this.okButtonDisabled(true);
			if (this._list == null) {
				this._initItemList();
			}
            if (this.showQueryFilters) {
                this._setItems([]);
            } else if (items) {
				this._setItems(items);
			} else {
                this._setStatusMessage(RUNNING_SEARCH_MSG);
                var srh = new ServiceResponseHandler(this, "_setItems", "_queryError");
                if (this.itemType == Constants.TYPE_CASE) {
                    // we need to fetch the summary amount of data. When we add the 
                    // items to the section table, it expects the modified, weight,
                    // and suspect to be available.
                    this._runningRequest = TestCaseClient.getTestCasesSummary(srh);
                } else if (this.itemType == Constants.TYPE_SCRIPT) {
                    this._runningRequest = TestScriptClient.getTestScriptsMinimal(srh);
                }
            }
		},
        
        setAllowsMultiple: function(allowsMultiple) {
            this.allowsMultiple = allowsMultiple;
        },
        
        setExclusionList: function(exclusionList) {
            if(!exclusionList) {
                this.exclusionMap = null;
                return;
            }
            this.exclusionMap= {};
            for (var i=0; i<exclusionList.length; i++) {
                var id= exclusionList[i];
                this.exclusionMap[id]= true;
            }
        },		
		
        _onCancel: function() {
            //this.reset();
        },
        
        _onOK: function() {
			if (dojo.isFunction(this.onOk)) {
                // call the virtual get selected rows, this will handle select all
                // properly and return all query contents.
                this._list.getVirtualSelectedRows("itemId", dojo.hitch(this, function(items) {
                    this.onOk(items);
                    this._dialog.close();
                }));
			}
        },		
		
		_onTableRowsChecked: function() {
			//if any rows in the table are selected, make sure the Ok button is enabled
			var hasSelection = this._list.hasSelectedRows();
            this.okButtonDisabled( !hasSelection );
		},
		
		_setItems: function(items) {
			//filter the items 
			items = this._filterExcluded(items);
			
            //reset the content of the list with the non excluded items 
            this._list.refresh(items);          
            dojox.data.dom.removeChildren(this._itemListArea);
            this._itemListArea.appendChild(this._list.domNode);
            this._clearStatusMessage();
            this.okButtonDisabled(true);
            if (!this.tableContainer && this._dialog) {
                this._dialog.updatePosition();
            }	
		},
		
		_filterExcluded: function(items) {
            // no exclusion map, just return the items
            // no need to filter.
            if(!this.exclusionMap) {
                return items;
            }
            items = items || [];
			var filtered = [];
            for (var i = 0; i < items.length; i++) {
                // don't add an entry for the item if it is explicitly excluded
                if (!this.exclusionMap[items[i].itemId] && !this.exclusionMap[items[i].versionableItemId]) {
                    filtered.push(items[i]);
                }
            }
			return filtered;
		},
        
		_queryError: function(errorObj) {
			this._setStatusMessage(FETCH_ERR_MSG, true);
		},
		
        _clearStatusMessage: function() {
            this._msgArea.innerHTML= "";
        },
        
        _setStatusMessage: function(msg, isError) {
            this._msgArea.innerHTML= msg;
            this._msgArea.className= isError ? "ErrorStatus" : "NormalStatus";
        }		
	}
);

})();
