/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008,2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.common.SelectScriptTableDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView");


(function() {

var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Constants               = com.ibm.rqm.planning.web.client.internal.Constants;
var TestScriptClient        = com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
var TestScriptTableView     = com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView;

var RUNNING_SEARCH_MSG		= Constants.getMessage("labelRunningSearch");
var FETCH_ERR_MSG			= Constants.getMessage("msgErrorRunningQuery");

dojo.declare(
	"com.ibm.rqm.execution.web.ui.internal.view.common.SelectScriptTableDialog",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.execution.web", "ui/internal/view/templates/common/SelectScriptTableDialog.html"),		
		
        width: "800px",   //default dialog width
        
        constructor: function(params){
            this._items = [];
			dojo.mixin(this, params);			
            if (!this.itemType) {
                throw new Error("Missing parameter 'itemType' is REQUIRED!");
            }           
            if (!dojo.isFunction(this.onOk)) {
                throw new Error("Missing function for 'onOk' parameter");
            }       
            this.setExclusionList(this.exclusionList ? this.exclusionList : []);
			this.exclusionMap= {};
			if (params.ter) {
                var included = params.ter.testCase.scripts;
                this.includedScripts = included;
				this.onSelectedOk = params.onOk;
				this.ter = params.ter;
            }
        },		
		
		postCreate: function() {            
			this._initItemList();
		},
		
        _showAsDialog: function() {
            var dialogArgs = {
                primaryTitle: this.title ? this.title : Constants.getMessage("labelTitle"),
                okCallback: dojo.hitch(this, "_onOK"),
                cancelCallback: dojo.hitch(this, "_onCancel")
            };                
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
			this._dialog.open(false);
			this._dialog.updatePosition();
        },		
		
		_initItemList: function() {
			if (this.itemType == Constants.TYPE_SCRIPT) {
                var tableArgs = {
                    scripts: [], 
                    profile: Constants.TABLE_PROFILE_MINIMAL                
                }				
				this._list = new TestScriptTableView(tableArgs);	
				this._list.setTableRowsCheckable(true,true);	
			} 			
		},
		
        show: function(){
			if (!this._dialog) {
                this._showAsDialog();
            }
            this.refresh();			            
        },		
				
		reset: function() {
			this._clearStatusMessage();
			this._retArr = [];
			this._list = null;
			this.exclusionMap = {};
		},
		
		refresh: function() {
			if (this._list == null) {
				this._initItemList();
			}
			this._setStatusMessage(RUNNING_SEARCH_MSG);
			this._dialog.getOkButton().disabled = true;
			var	srh= new ServiceResponseHandler(this, "_loadArtifacts", "_queryError");	
		    if (this.itemType == Constants.TYPE_SCRIPT) {
                TestScriptClient.getTestScriptsMinimal(srh);
            }
		},
        
        setExclusionList: function(exclusionList) {
            this.exclusionMap= {};
			var len = exclusionList.length;
            for (var i=0; i<len; i++) {
                var id= exclusionList[i];
                this.exclusionMap[id]= true;
            }
        },		
		
        _onCancel: function() {
            this.reset();
        },
        
        _onOK: function() {
            this._retArr = [];
            var rows = this._list.getSelectedItems();
			var len = rows.length;
            for (var i=0; i<len; i++) {
                // last element is the stored artifact or snapshot info
                var row = rows[i]; 
                // an artifact
                this._retArr.push(row);
            }
			if (this.ter) {
				var returnArgs = {
					selected: rows[0],
					ewi: this.ter
				}
				this.onSelectedOk(returnArgs);
			}
        },		
		
		_loadArtifacts: function(items) {			
			items = items ? items : [];
			var artifacts = [];
			var len = items.length;
			var exclusionMap = this.exclusionMap;
			var includedScripts = this.includedScripts;
			for (var i = 0; i < len; i++) {
				var artifact = items[i]; //{name: items[i].name, artifactId: items[i].itemId}
				artifact.artifactId = artifact.itemId;			
				
				// don't add an entry for the item if its already in the item list
				var itemId = this._enableSnapshots ? items[i].versionableItemId : items[i].itemId;
				if (!exclusionMap[items[i].itemId] && !exclusionMap[items[i].versionableItemId]) {
					if(includedScripts){
						if(this._isIncluded(items[i].name)){
					   		artifacts.push(artifact);
						}
					}else{
						artifacts.push(artifact);
					}

				}	
			}
			//reset the content of the list with the non excluded items 
			this._list.refresh(artifacts);			
			dojox.data.dom.removeChildren(this._itemListArea);
			this._itemListArea.appendChild(this._list.domNode);
			this._clearStatusMessage();
			this._dialog.getOkButton().disabled = false;
			this._dialog.updatePosition();
	
		},
		_isIncluded: function(itemName){
            var included = false;
			var includedScripts = this.includedScripts
            if (includedScripts) {
				var len = includedScripts.length;
                for (var i = 0; i < len; i++) {
                    if (includedScripts[i].name.toString().indexOf(itemName) != -1) {
                    	included = true;
					}
                }
            }         
            return included;
        },
			
		_queryError: function(errorObj) {
			this._setStatusMessage(FETCH_ERR_MSG, /*isError*/ true);
		},
		
		_clearStatusMessage: function() {
			this._msgArea.innerHTML= "";
		},
		
		_setStatusMessage: function(msg, isError) {
			this._msgArea.innerHTML= msg;
			this._msgArea.className= isError ? "ErrorStatus" : "NormalStatus";
		}	
	}
);

})();
