/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectWeightPopup");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.parser");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

(function() {
var Constants           = com.ibm.rqm.planning.web.client.internal.Constants;
var CategoryClient      = com.ibm.rqm.planning.web.client.internal.CategoryClient;
var TestCaseClient		= com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
var EditorUtil 			= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;

var SelectUtility       = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
var FormNumericField = com.ibm.asq.common.web.ui.internal.widgets.layout.FormNumericField;
var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectWeightPopup",
             [dijit._Widget, dijit._Templated], 
{
    templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/properties/SelectWeightPopup.html"),
				    
	width: "450px",
    height: "450px", 
		
	labelWeight: Constants.getMessage("labelweight"),
	labelPoints: Constants.getMessage("labelPoints"),
		
    postCreate: function(){
    },
    
    initialize: function(args){
		this.okCallback = args.okCallback;
		this.actionId = args.actionId;
		
		dojox.data.dom.removeChildren(this._weight);
		this.weightWidget = new FormNumericField({
			fieldLabel: this.labelWeight, 
			required: false,
			disabled: false,
			labelWidthPercent: 38
		});
		this.weightWidget.setEdited(true);
		this.weightWidget.setErrorMessage(Constants.getMessage("msgNumericWeightRequired"));
		this.weightWidget.setPromptMessage(Constants.getMessage("msgWeightRequired"));
		this._weight.appendChild(this.weightWidget.domNode);
		this.weightWidget.onValueChanged = dojo.hitch(this, "_validate");
		
        this._showAsDialog();	
    },
    
	_showAsDialog: function()
    {
        if (!this._dialog) {
            var dialogArgs = {
                okCallback: dojo.hitch(this, "_okAction"),
                cancelCallback: dojo.hitch(this, "_cancelAction")
            };
            
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
			this._dialog.setTitle(Constants.getMessage("labelChangeProperties", this.actionId));
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
            this._dialog.getOkButton().disabled = false;                                
        }
        this._dialog.open(false);
    },  
	
	_okAction: function(){
		
		var args = {};
 		
		args.actionId = this.actionId;
		
		if (dojo.trim(this.weightWidget.getValue()).length != 0) {
			args.weight = Number(dojo.trim(this.weightWidget.getValue()));
		}
		
        if (this.okCallback) {
            this.okCallback(args);
        }
    },
 
	_cancelAction: function(){
        if (this.cancelCallback) {
            this.cancelCallback();
        }
    },
  
	_validate: function() {
		if (dojo.trim(this.weightWidget.getValue()).length != 0 && this._dialog) {
			var weight = Number(dojo.trim(this.weightWidget.getValue()));
			if (weight != NaN && 
				weight > 0 &&
				weight <= parseInt(Constants.MAX_INT_VALUE_IN_HEX, 10)) {
				this._dialog._yesButton.disabled = false;
			}
			else { 
				this._dialog._yesButton.disabled = true;
			}
		}
	}
    
});
})();