/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateCFGWizard");
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateCFGWizard");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.AbstractWizard");
dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateCFGConfigurationListPanel");
dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateCFGConfigurationPanel");
dojo.require("com.ibm.rqm.configuration.web.util.TestConfigurationUtil");
dojo.require("com.ibm.rqm.configuration.web.client.internal.configuration.WHITCHClient");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){

    var GenerateCFGConfigurationListPanel = com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateCFGConfigurationListPanel;
    var GenerateCFGConfigurationPanel = com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateCFGConfigurationPanel;
    var TestConfigurationUtil = com.ibm.rqm.configuration.web.util.TestConfigurationUtil;
	var WHITCHClient = com.ibm.rqm.configuration.web.client.internal.configuration.WHITCHClient;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var Constants  = com.ibm.rqm.planning.web.client.internal.Constants;
    
    var STEP_CONFIGURE = 1;
    var STEP_CONFIG_PREVIEW = 2;
    
    dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateCFGWizard", [dijit._Widget, dijit._Templated, com.ibm.rqm.configuration.web.ui.internal.view.configuration.AbstractWizard], {
        
        constructor: function(args, frag){
            this.currentStep = STEP_CONFIGURE;
            this.testPlan = args.testPlan;
        },
		
		
		//override for AbstractWizard
		uninitialize: function(){
//			console.debug("===============GenerateCFGWizard Uninitialize");
			this._generateEWIWidget.destroyRecursive();
			if(this._configurationListView){
				this._configurationListView.destroyRecursive();
			}
		},
        
        //override for AbstractWizard
        setTitle: function(title){
        	this.title = Constants.getMessage("labelGenerateConfigurations") + ":";
        },
        
        //override for AbstractWizard
        showCurrentStep: function(args){
            switch (this.currentStep) {
                case STEP_CONFIGURE:
                    this._showGenerateConfiguration(args);
                    break;
                case STEP_CONFIG_PREVIEW:
                    this._showTestConfigurationList(args);
                    break;
                default:
                    throw new Error(Constants.getMessage("labelInvalidStep") + " (" + this.currentStep + ")");
            }
        },
        
        _showGenerateConfiguration: function(){
            if (!this._generateEWIWidget) {
                var args = {
                    controller: this
                };
                args.testPlan = this.testPlan;
                this._generateEWIWidget = new GenerateCFGConfigurationPanel(args);
				this._setDialogContent(this._generateEWIWidget.domNode);
				this._generateEWIWidget.initialize();
            }else{
				this._setDialogContent(this._generateEWIWidget.domNode);
			}
        },
        
        _showTestConfigurationList: function(params){
            params.controller = this;
            this._configurationListView = new GenerateCFGConfigurationListPanel(params);
            this._setDialogContent(this._configurationListView.domNode);
        },
		
		commit: function(configs){
			this.createTestConfigurations(configs);
		},
		
		createTestConfigurations: function(configs){
			var modeledConfigs = [];
			var modeledConfigNames = [];
			var namePrefix = this._configurationListView.getNamePrefix();
            for (var i = 0; i < configs.length; i++) {
				var theCatalog = new com.ibm.asq.common.web.client.internal.Catalog();
                var config = theCatalog.createConfiguration("Machine");
                var attributeValues = new Array();
                
                for (var j = 0; j < configs[i].length; j++) {
                    var key = configs[i][j].key;
                    var value = configs[i][j].value;
                    attributeValues.push(value);
                    config.setValue(key, value);
                }
				modeledConfigs.push(config.toJson());
				modeledConfigNames.push(namePrefix + TestConfigurationUtil.getTestConfigurationName(attributeValues));
            }
			var params = {
                newConfigurations: modeledConfigs,
				newConfigurationNames: modeledConfigNames
            };
			
			var self = this;
			var handler = {
				success: function(configDTOs){
					self.doFinish(configDTOs);
				},
				
				error: function(e){
					console.error(e);
				}
			}
			var responseHandler = new ServiceResponseHandler(handler, "success", "error");
			
            WHITCHClient.postCreateConfigurations(responseHandler, params);
        }		
    });
    
})();
