/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWISaveContext");

dojo.require("com.ibm.rqm.configuration.web.util.TestConfigurationUtil");
dojo.require("com.ibm.rqm.configuration.web.client.internal.configuration.WHITCHClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function() {
	
var TestConfigurationUtil 	= com.ibm.rqm.configuration.web.util.TestConfigurationUtil;
var WHITCHClient 			= com.ibm.rqm.configuration.web.client.internal.configuration.WHITCHClient;
var ServiceResponseHandler  = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWISaveContext", null, {	
	
    constructor: function(args) {
		if (!args.ewis) {
			throw new Error("GenerateEWISaveContext#constructor: Missing required parameter 'args.ewis'");
		} 
		if (!args.parentTestPlanUUID) {
			throw new Error("GenerateEWISaveContext#constructor: Missing required parameter 'args.parentTestPlanUUID'");
		}
		if (!args.configurationAppliedTestPlanUUID) {
			throw new Error("GenerateEWISaveContext#constructor: Missing required parameter 'args.configurationAppliedTestPlanUUID'");
		}
		this.ewis = args.ewis;
		this.parentTestPlanUUID = args.parentTestPlanUUID;
		this.configurationAppliedTestPlanUUID = args.configurationAppliedTestPlanUUID;
		this.allConfigs = [];
    },
	
	commit: function(args) {
		if(args && args.callback) {
			this.callback = args.callback;
		}
		this.createTestConfigurations(this.ewis);
	},
	
	saveFinished: function() {
		console.debug("ewi save finished");
		if(this.callback) {
			this.callback();
		}
	},
	
	removeEwis: function(ewiArray) {
		var ewis = this.ewis;
		var length = ewis.length;
		var ewiArrayLength = ewiArray.length;
		var _ewis = [];
		for(var i = 0; i < length; i++){
			var ewi = ewis[i];
			var deleted = false;
			for(var j = 0; j < ewiArrayLength; j++){
				if (ewi.id == ewiArray[j].id){
					deleted = true;
					break;
				}
			}
			if(deleted == false){
				_ewis.push(ewi);
			}
		}
		this.ewis = _ewis;
	},
	
	updateEwis: function(ewiArray) {
		var ewis = this.ewis;
		var length = ewis.length;
		var ewiArrayLength = ewiArray.length;
		for(var i = 0; i < length; i++){
			var ewi = ewis[i];
			for(var j = 0; j < ewiArrayLength; j++){
				var updatedEwi = ewiArray[j];
				if(ewi.id == updatedEwi.id){
					ewi.weight = updatedEwi.weight;
					ewi.owner = updatedEwi.owner;
					ewi.testPhase = updatedEwi.phase;
					break;
				}
			}
		}
	},
	
	createTestConfigurations: function(ewis){
		//Configurations which need to be created
		var configs = {};
		var newConfigs = [];
		var newConfigNames = [];
		//some EWIs have the same configuration, so remove the duplication
		for(var i = 0; i < ewis.length; i++){
			if(ewis[i].config.isExist != true){
				configs[ewis[i].config.name] = ewis[i].config;
			}
			this.allConfigs[ewis[i].config.name] = ewis[i].config;
		}
		for(var prop in configs){
			newConfigs.push(configs[prop].toJson());
			newConfigNames.push(prop);
		}
		
		var params = {
            newConfigurations: newConfigs,
			newConfigurationNames: newConfigNames
        };
		
		var self = this;
		var handler = {
			success: function(configDTOs){
				if(dojo.isArray(configDTOs)){
					for(var i = 0; i < configDTOs.length; i++){
						self.allConfigs[configDTOs[i].name] = configDTOs[i];
					}
					if(self.parentTestPlanUUID != TestConfigurationUtil.NO_ASSOCIATED_TEST_PLAN){
						self.applyConfigToTestPlan(configDTOs);
					}else{
						self.createExecutionWrokItems();
					}
				}else{
					self.createExecutionWrokItems();
				}
			},
			
			error: function(e){
				console.error(e);
			}
		}
		var responseHandler = new ServiceResponseHandler(handler, "success", "error");
		
        WHITCHClient.postCreateConfigurations(responseHandler, params);
	},
	
	applyConfigToTestPlan: function(configDTOs){
		var self = this;
        var handler = {
            success: function(){
				self.createExecutionWrokItems();
            },
            error: function(e){
				console.error(e);
            }
        }
        var responseHandler = new ServiceResponseHandler(handler, "success", "error");
        var addedConfigurations = new Array();
		for (var i = 0; i < configDTOs.length; i++){
			addedConfigurations.push(configDTOs[i].id);
		}
        var params = {
            testsPlanUUID: this.configurationAppliedTestPlanUUID,
            addedConfigurations: addedConfigurations
        };
        WHITCHClient.postConfigurationsToTestPlan(responseHandler, params);
	},
	
	createExecutionWrokItems: function(){
		var params = {
			ownerIdArray: [],
			testConfigIdArray: [],
			testPhaseIdArray: [],
			executionWorkItemNameArray: [],
			testCaseIdArray: [],
			defaultScriptIdArray: [],
			testPlanItemIdArray: [],
			weightArray: []
		};
		var ewis = this.ewis;
		var length = ewis.length;
		var allConfigs = this.allConfigs;
		for(var i = 0; i < length; i++){
			var ewi = ewis[i];
			params.ownerIdArray.push(ewi.owner.userId);
			params.testConfigIdArray.push(allConfigs[ewi.testConfigName].id);
			params.testPhaseIdArray.push(ewi.testPhase == null ? -1 : ewi.testPhase.itemId);
			params.executionWorkItemNameArray.push(ewi.name);
			params.testCaseIdArray.push(ewi.testCaseId);
			params.defaultScriptIdArray.push(ewi.defaultScript);
			params.testPlanItemIdArray.push(ewi.testPlanItemId == null ? -1 : ewi.testPlanItemId);
			params.weightArray.push(ewi.weight);
		}
		var self = this;
		var handler = {
			success: function(){
				self.saveFinished();
			},
			error: function(e){
				console.error(e);
			}
		}
		var responseHandler = new ServiceResponseHandler(handler, "success", "error");
    	WHITCHClient.postCreateExecutionWorkitems(responseHandler, params);
	}
	
});

})();
