/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.configuration.web.ui.internal.view.configuration.OverviewSection");
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.configuration.OverviewSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

dojo.require("com.ibm.rqm.configuration.web.util.TestConfigurationUtil");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
dojo.require("com.ibm.asq.common.web.client.internal.ContributorClient");

(function(){ // module container
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
    var TestConfigurationUtil = com.ibm.rqm.configuration.web.util.TestConfigurationUtil;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
	var UserClient = com.ibm.asq.common.web.client.internal.ContributorClient;
   
    dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.configuration.OverviewSection", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.configuration.web", "ui/internal/view/templates/configuration/OverviewSection.html"),
        
        constructor: function(args){
            if (args.controller == null) {
                throw new Error("OverviewSection#constructor: Missing required parameter 'args.controller'");
            }
            this.controller = args.controller;
			this.contributors = [];
        },
        
        postCreate: function(){
			this._initTemplateLabel();
            this._initTestArtifact();
//			this._initContributors();
			this._initCurrentContributor();
			this._initOthers();
        },
		
		_initTemplateLabel: function(){
			this._overviewLabel.appendChild(document.createTextNode(Constants.getMessage("labelOverview")));
			this._originatorLabel.appendChild(document.createTextNode(Constants.getMessage("labelOriginator") + ":"));
			this._ownerLabel.appendChild(document.createTextNode(Constants.getMessage("labelOwner") + ":"));
			this._testPlanLabel.appendChild(document.createTextNode(Constants.getMessage("labelTestPlan") + ":"));
			this._scheduleTestPlanLabel.appendChild(document.createTextNode(Constants.getMessage("labelTestPlan") + ":"));
			this._scheduleLabel.appendChild(document.createTextNode(Constants.getMessage("labelIteration") + ":"));
		},
		
        //@Abstract
        _initTestArtifact: function(){
            return;
        },
		
		_initContributors: function(){
            var responseHandler = new ServiceResponseHandler(this, "_doInitContributors", "_error");
            UserClient.getContributors(responseHandler);
		},
        
		_initCurrentContributor: function(){
            var responseHandler1 = new ServiceResponseHandler(this, "_doInitCurrentContributor", "_error");
            RepositoryClient.getAuthenticatedContributor(responseHandler1);
		},
		
		//@Abstract
		_initOthers: function(){
			return;
		},
        
        _doInitContributors: function(contributors){
            dojox.data.dom.removeChildren(this._owner);
            for (var i = 0; i < contributors.length; i++) {
				this.contributors = contributors;
                if (contributors[i].name != "ADMIN" && contributors[i].name != "GUEST" && !contributors[i].archived) {
                    var contributorsName = contributors[i].name;
                    if (com.ibm.asq.common.web.ui.internal.util.Utility.isUnassignedUserId(contributors[i].itemId)) {
                        contributorsName = Constants.getMessage("labelUnassigned");
                    }
                    var option = new Option(contributorsName, contributors[i].userId);
                    if (contributors[i].name == this.contributor.name) {
                        option.selected = true;
                    }
					SelectUtility.add( this._owner, option );
                }
            }
        },
        
        _doInitCurrentContributor: function(contributor){
			this.contributor = contributor;
            dojox.data.dom.removeChildren(this._orig);
            this._orig.appendChild(document.createTextNode(contributor.name));
			this._initContributors();
        },
        
        getOwner: function(){
            var owner = null;
            var options = this._owner.options;
            for (var i = 0; i < options.length; i++) {
                if (options[i].selected) {
					for(var j = 0; j < this.contributors.length; j++){
						if(options[i].value == this.contributors[j].userId){
							owner = this.contributors[j];
							break;
						}
					}
                }
            }
            return owner;
        },
        
        _error: function(errorObj){
            alert("Error: (message: '" + errorObj.message + "') from OverviewSection#_error");
        }
    });
    
})();
