/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.appscan.web.ui.internal.view.construction.ASEScriptEditorSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox")
dojo.require("dijit.Dialog")  

dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.RemoteExecutionScriptClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
dojo.require("com.ibm.rqm.appscan.web.client.internal.ScanClient");
dojo.require("com.ibm.rqm.appscan.web.ui.internal.view.common.ASELink");

(function() { // module container

var Utility                         = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var AbstractArtifactEditorSection   = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
var Constants    					= null;
var ImageCache				       	= com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
var	EditorUtil					   	= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
var ScriptClient 					= com.ibm.rqm.execution.web.client.internal.execution.RemoteExecutionScriptClient;
var ServiceResponseHandler 			= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var SelectUtility 					= dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );

dojo.declare(
   	"com.ibm.rqm.appscan.web.ui.internal.view.construction.ASEScriptEditorSection", 
   	[dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], 
   	{
   	templatePath: dojo.moduleUrl("com.ibm.rqm.appscan.web", "ui/internal/view/templates/construction/ASEScriptEditorSection.html"),
   	   
   	constructor: function(args) {
      	this._editor = args.parentEditor;
      	this._script = args.script ? args.script: args.parentEditor._script;
      	this._jobId = null;
      	this._reportPackId = null;
      	this._templateName = null;
      	this._severityThreshold = null;
      	this._properties = this._script ? this._script.properties: null;
      	this._templates = null;
   	},
   
	postCreate: function() {
		// Require ASE code
		Constants  = com.ibm.rqm.appscan.web.client.internal.Constants;
		
		// Decode properties
		if (this._properties)
		{      		
			for (var i = 0; i < this._properties.length; i++) {
				if (this._properties[i].propertyName == Constants.JOBID) {
					this._jobId = this._properties[i].propertyValue;
				}
				else if (this._properties[i].propertyName == Constants.REPORTPACKID) {
					this._reportPackId = this._properties[i].propertyValue;
				}
				else if (this._properties[i].propertyName == Constants.TEMPLATENAME) {
					this._templateName = this._properties[i].propertyValue;
				}
				else if (this._properties[i].propertyName == Constants.SEVERITYTHRESHOLD) {
					this._severityThreshold = this._properties[i].propertyValue;
				}
			}
		}  	
		this._initLabels();
		this._initRadioButtonNames();
		this._initImages();
	},

	/**
	 * Initialize the static labels on the widget
	 */
	_initLabels: function() {
		this._introLabel.innerHTML = Constants.getMessage("testscript_intro");
		this._templateTitleLabel.innerHTML = Constants.getMessage("testscript_template_title");
		this._templateIntroLabel.innerHTML = Constants.getMessage("testscript_template_intro");
		this._typeTitleLabel.innerHTML = Constants.getMessage("testscript_type_title");
		this._verdictTitleLabel.innerHTML = Constants.getMessage("testscript_verdict_title");
		this._verdictIntroLabel.innerHTML = Constants.getMessage("testscript_verdict_intro");
		this._severityThresholdTitleLabel.innerHTML = Constants.getMessage("testscript_severity_threshold_title");
		this._400Label.innerHTML = Constants.getMessage("testscript_400");
		this._300Label.innerHTML = Constants.getMessage("testscript_300");
		this._200Label.innerHTML = Constants.getMessage("testscript_200");
		this._100Label.innerHTML = Constants.getMessage("testscript_100");
	},
	
	/**
	 * Initialize the radio button names for this instance of the widget. Necessary because otherwise all radio buttons, no matter the tab 
	 * (or open instance of a test script), belong to the same group.
	 */
	_initRadioButtonNames: function() {	
		if (this._script) {
			this._radioButtonGroupName = "_aseSeverityThresholdName" + this._script.id;
		}
		else {
			// Generate random id, in case multiple test scripts are being created at the same time
			this._radioButtonGroupName = "_aseSeverityThresholdName" + "_New" + Math.floor(Math.random()*Constants.MAX_NEW_SCRIPTS);
		}
		
		this._aseSeverityThreshold400.setAttribute("name", this._radioButtonGroupName);
		this._aseSeverityThreshold300.setAttribute("name", this._radioButtonGroupName);
		this._aseSeverityThreshold200.setAttribute("name", this._radioButtonGroupName);
		this._aseSeverityThreshold100.setAttribute("name", this._radioButtonGroupName);
	},
	
	/**
	 * Initialize the images that appear in the section.
	 */
	_initImages: function() {		
		this._aseHighImg.src = dojo.moduleUrl("com.ibm.rqm.adapter.appscan.web", "ui/internal/view/templates/common/images/issue_high.gif");
		this._aseMediumImg.src = dojo.moduleUrl("com.ibm.rqm.adapter.appscan.web", "ui/internal/view/templates/common/images/issue_medium.gif");
		this._aseLowImg.src = dojo.moduleUrl("com.ibm.rqm.adapter.appscan.web", "ui/internal/view/templates/common/images/issue_low.gif");
		this._aseInformationImg.src = dojo.moduleUrl("com.ibm.rqm.adapter.appscan.web", "ui/internal/view/templates/common/images/issue_information.gif");
	},

   	initialize: function() {   
		// Check to make sure ASE is properly configured before continuing
		this._checkVersion();
   	},
	
	_initialize: function() {
      	this._initCreateEditScanLink();    
      	this._initVerdictStrategy();
	},
   
   	//@Abstract: Override to contribute attributes to the parent's save
   	// We use the other method of creating properties and persisting them ourselves in onSave
   	getAttributes: function() {  
   	},

   	//@Overriden from AbstractArtifactEditorSection
   	getActions: function() {
      	return [];
   	},    
      
	onSave: function(responseHandler, scriptObj, lastSaved) {
		// If we're saving a new test script without having created an ASE scan, prompt to create it
		if (this._editor.getAttributes().newScript && !this._creatingScan && this._templates) { 
			if (!confirm(Constants.getMessage("confirm_create"))) {
				// Person hit Cancel to continue editing the test script, do not save
				// Set the focus to the template type
				dojo.byId("_aseSelectedTemplateId").focus();
				return;
			}
		}	
		
		// Ensure the name and description of the job and report match the test script
		if (!this._editor.getAttributes().newScript && this._editor.isDirty()) {				
			if (this._jobId && this._reportPackId) {
				var handler = new ServiceResponseHandler(this, "_success", "_error");	
				com.ibm.rqm.appscan.web.client.internal.ScanClient.updateScan(handler, {
						jobId: this._jobId,
						reportPackId: this._reportPackId,
						name: this._editor.getAttributes().id + ": " + this._editor.getAttributes().name,
						description: this._editor.getAttributes().description
					});
			}								
		}
		// Always persist the severity threshold property, only if it has changed or if we just created it
		var severityThresholdProperty = null;
		var radio = this._scriptInfoTable.getElementsByTagName("input");
		for (var i = 0; i < radio.length; i++) {
			if (radio[i].checked && this._aseSeverityThreshold != radio[i].value) {
				this._aseSeverityThreshold = radio[i].value;
				severityThresholdProperty = {
					propertyType: "String",
					propertyName: Constants.SEVERITYTHRESHOLD,
					propertyValue: this._aseSeverityThreshold
				}
				break;
			}
		}

		// Modify the script object to contain properties
		if (this._properties != null && this._properties.length > 0)
		{	
			for (var i = 0; i < this._properties.length; i++) {
				if (this._properties[i].propertyName == Constants.SEVERITYTHRESHOLD && severityThresholdProperty) {
					this._properties[i] = severityThresholdProperty;
					severityThresholdProperty = null;
				}			
			}

			// Just push it if it isn't already a property
			if (severityThresholdProperty) {
				this._properties.push(severityThresholdProperty);
			}
			scriptObj.properties = this._properties;
		}
		// Otherwise, save only the SeverityThreshold property
		else if (severityThresholdProperty) {	
			this._properties = new Array()
			this._properties.push(severityThresholdProperty);
			scriptObj.properties = this._properties;
		}
		
		// Save the script	
		//EditorUtil.setLastSaved(lastSaved, Constants.STATUS_SAVING);	
		ScriptClient.saveScript(responseHandler, scriptObj);
	},
	     
   	//@Overridden from AbstractArtifactEditorSection
   	saveFinished: function(savedItem) {
		this._script = savedItem;
		this._initRadioButtonNames();
   	},	
	
	_checkVersion: function(){
		var responseHandler = new ServiceResponseHandler(this, "_checkVersionSuccess", "_checkVersionError");
		com.ibm.rqm.appscan.web.client.internal.InfoClient.getVersion(responseHandler);
	},
	
	_checkVersionSuccess: function(result) {
			
		try{
			var version = eval("(" + result + ")"); 
			if (version.IsValidDatabaseVersion){
				this._initialize();
			}
			else {
				// Disable the save/cancel buttons
				this._editor.setReadOnly(true);
				this._editor.updateSaveable();
			
				// Show the error message
				dojox.data.dom.removeChildren(this._container);
				this._container.innerHTML = Constants.getMessage("ase_wrong_version", version.StoredProcedureVersion, version.DatabaseVersion, version.CheckDatabaseVersion);
			}		
		} catch(e) {
			// Disable the save/cancel buttons
			this._editor.setReadOnly(true);
			this._editor.updateSaveable();
				
			dojox.data.dom.removeChildren(this._container);
			this._container.innerHTML = result;
		}
	},
	
	_checkVersionError: function(e){
		// Disable the save/cancel buttons
		this._editor.setReadOnly(true);
		this._editor.updateSaveable();
		
		// Show the error message
        dojox.data.dom.removeChildren(this._container);
		this._container.innerHTML = Constants.getMessage("error_ase_not_configured");
    },
	
  	_eval: function(json) {
    	try {
			return eval( "(" + json + ")" );
    	}
		catch(e) {
			// Do nothing for now, return an object for convenience
			return {
				results: null
			};	
		}		
	},
		
	_success: function(e) {							
		// Do nothing
	},

   	_error: function(e) {
        console.error("An error occurred: [" + e.message + "] @ TestScriptEditorASE#_error");
    },  
	
	_initTemplates: function() {
		this._aseSelectedTemplate.appendChild(new Option(com.ibm.rqm.planning.web.client.internal.Constants.LOADING, ""));
		
		if (this._templates) {
			this._showTemplates();
		}
		else {
			var handler = new ServiceResponseHandler(this, "_doInitTemplates", "_doTemplatesError");
			com.ibm.rqm.appscan.web.client.internal.ScanClient.getTemplates(handler);
		}	
	},
	
	_doTemplatesError: function(e) {
		// Disable the save/cancel buttons
		this._editor.setReadOnly(true);
		this._editor.updateSaveable();
			
		// Show the error message	
        dojox.data.dom.removeChildren(this._container);
		this._container.innerHTML = Constants.getMessage("error_ase_retrieving_templates") + e.exception.errorMessage;
	},
	
	_doInitTemplates: function(templates) {
		this._templates = this._eval(templates).results;
		this._showTemplates();
	},
	
	_showTemplates: function() {
		dojox.data.dom.removeChildren(this._aseSelectedTemplate);
		if ( this._templates != null && this._templates.length > 0 ) {
			for ( var i = 0; i < this._templates.length; i++ ) {
				var ids = { jobId: this._templates[i].FolderItemId, reportPackId: this._templates[i].reportPackId };
				var opt = new Option(this._templates[i].Name, dojo.toJson(ids));
				SelectUtility.add(this._aseSelectedTemplate, opt);				
			}
			
			// A script does not exists, provide the opportunity to create it by enabling the Create Scan link
			if (this._aseCreateEditScanLink) {			
				this._aseCreateEditScanLink.innerHTML = Constants.getMessage("create_scan");
				this._extlinkImg.src = com.ibm.rqm.appscan.web.ui.internal.view.common.ImageCache.EXTERNALLINK;
				
				// disconnect handle if it exists
				if (this._aseCreateEditScanLinkHandle)
					this.disconnect(this._aseCreateEditScanLinkHandle);
					
				this._aseCreateEditScanLinkHandle = this.connect(this._aseCreateEditScanLink, "onclick", this._createScan);
			}
		}
		else {
			this._templates = null;
			
			// Disable the save/cancel buttons
			this._editor.setReadOnly(true);
			this._editor.updateSaveable();
			
			// Show the error message			
			dojox.data.dom.removeChildren(this._container);
			this._container.innerHTML = Constants.getMessage("ase_no_templates");
		}
	},
		
	_initVerdictStrategy: function() {			
		// Select the appropriate checkbox
		var valueToCheck = this._severityThreshold ? this._severityThreshold : Constants.DEFAULT_SEVERITY_THRESHOLD;
		var severityThresholdRadioToCheck = eval('this._aseSeverityThreshold' + valueToCheck);
		severityThresholdRadioToCheck.checked = true;
		
		this._aseSeverityThreshold400Handle = this.connect(this._aseSeverityThreshold400, "onclick", this._setDirty);
		this._aseSeverityThreshold300Handle = this.connect(this._aseSeverityThreshold300, "onclick", this._setDirty);
		this._aseSeverityThreshold200Handle = this.connect(this._aseSeverityThreshold200, "onclick", this._setDirty);
		this._aseSeverityThreshold100Handle = this.connect(this._aseSeverityThreshold100, "onclick", this._setDirty);
	},
	
	_setDirty: function() {
		this._editor.setDirty(true);	
	},
	
	_initCreateEditScanLink: function() {
		this._aseCreateEditScanLink.disabled = false;  				
   		this._aseCreateEditScanLink.jobId = null;
		if (this._jobId && this._templateName) {		
			this._aseCreateEditScanLink.jobId = this._jobId;
			
			// A script exists, provide the opportunity to edit it
			this._aseCreateEditScanLink.innerHTML = Constants.getMessage("edit_scan");
			this._extlinkImg.src = com.ibm.rqm.appscan.web.ui.internal.view.common.ImageCache.EXTERNALLINK;
			
			// disconnect handle if it exists
			if (this._aseCreateEditScanLinkHandle)
				this.disconnect(this._aseCreateEditScanLinkHandle);
				
			this._aseCreateEditScanLinkHandle = this.connect(this._aseCreateEditScanLink, "onclick", this._editScan);
			
			// Set the previously selected template, cannot select template again once scan has been created
			var opt = new Option(this._templateName, this._templateName);
			opt.selected = true;
			SelectUtility.add(this._aseSelectedTemplate, opt);	
			this._aseSelectedTemplate.disabled = true;
		}
		else {
			// Initialize template list, create scan link enabled based on success or failure of this operation
			this._initTemplates();
		}
	},

	_createScan: function() {			
		this._creatingScan = true;				
		// Make sure the test script has been saved before creating the ASE scan
		// This ensures a name has been set, the script type can no longer change and no data is lost
		if (this._editor.getAttributes().newScript /*|| this._editor.isDirty()*/) { 
			if (confirm(Constants.getMessage("confirm_save"))) {
				this._editor.onSave();
				var self = this;
	            setTimeout(function() {
					if (!self._editor.getAttributes().newScript) {
						self._executeCreateScan();
					}
            	}, 3000);
			}
		}		
		else {
			this._executeCreateScan();
		}
	},
	
	_executeCreateScan: function() {
		// Disconnect the Create Scan onclick
		this._aseCreateEditScanLink.innerHTML = Constants.getMessage("creating");
		this._extlinkImg.src = "";
		this._aseCreateEditScanLink.disabled = true;
		this.disconnect(this._aseCreateEditScanLinkHandle);
					
		// Show a dialog preventing user from interacting with the test script
		this._createScanPopup();
					
		// Create the ASE scan
		var handler = new ServiceResponseHandler(this, "_createScanSuccess", "_createScanFailure");	
		com.ibm.rqm.appscan.web.client.internal.ScanClient.createScan(handler, {
				folderId: Constants.DEFAULT_FOLDER_ID,
				templateJobId: this._eval(this._aseSelectedTemplate.value).jobId,
				templateReportPackId: this._eval(this._aseSelectedTemplate.value).reportPackId,
				name: this._editor.getAttributes().id+": "+this._editor.getAttributes().name,
				description: this._editor.getAttributes().description
			});
	},

    _createScanFailure: function(e) {
		this._creatingScan = false;	
				
    	// Reconnect the create scan link
    	this._aseCreateEditScanLink.innerHTML = Constants.getMessage("create_scan");
		this._extlinkImg.src = com.ibm.rqm.appscan.web.ui.internal.view.common.ImageCache.EXTERNALLINK;

    	this._aseCreateEditScanLink.disabled = false;
		
		// disconnect handle if it exists
		if (this._aseCreateEditScanLinkHandle)
			this.disconnect(this._aseCreateEditScanLinkHandle);
		
		this._aseCreateEditScanLinkHandle = this.connect(this._aseCreateEditScanLink, "onclick", this._createScan);		

		// Hide the dialog preventing user from interacting with the test script
		this.dialog.hide();
		
		// Rebind the template list in case one was deleted
		var handler = new ServiceResponseHandler(this, "_doInitTemplates", "_doTemplatesError");
		com.ibm.rqm.appscan.web.client.internal.ScanClient.getTemplates(handler);
		
		// Alert the user that an error has occurred
		this._createErrorDialog(Constants.getMessage("error_creating_scan"));
		
        this._error(e);
    },
	
	_createErrorDialog: function(message){
        var errorDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({
            'message': message
        });
        errorDialog.open();
	},
		
	_createScanSuccess: function(result) {
		this._creatingScan = false;	
		
		// Get the new job id from result and add to link
		var scanCreatedResult = this._eval(result);
		
		// Change create link to edit link		
		this._aseCreateEditScanLink.innerHTML = Constants.getMessage("edit_scan");
		this._extlinkImg.src = com.ibm.rqm.appscan.web.ui.internal.view.common.ImageCache.EXTERNALLINK;
		this._aseCreateEditScanLink.jobId = scanCreatedResult.jobId;
		this._aseCreateEditScanLink.disabled = false;
		
		// Properties to persist
		this._jobId = scanCreatedResult.jobId + "";		// Convert to string with + "", since Long properties currently cause runtime errors
		this._reportPackId = scanCreatedResult.reportPackId + "";
		this._templateName = this._aseSelectedTemplate.options[this._aseSelectedTemplate.selectedIndex].innerHTML;
		
		var jobIdProperty = {
			propertyType: "String",
			propertyName: Constants.JOBID,
			propertyValue: this._jobId
		};
		
		var reportPackIdProperty = {
			propertyType: "String", 
			propertyName: Constants.REPORTPACKID,
			propertyValue: this._reportPackId
		};
		
		var templateNameProperty = {
			propertyType: "String",
			propertyName: Constants.TEMPLATENAME,
			propertyValue: this._templateName
		};
		
		var tmpProperties = new Array();
		tmpProperties.push(jobIdProperty);
		tmpProperties.push(reportPackIdProperty);
		tmpProperties.push(templateNameProperty);

		// If the properties array only contains the severity, add it as well
		if (this._properties != null && this._properties.length == 1) {
			tmpProperties.push(this._properties[0]);
		}
		
		this._properties = tmpProperties;
		
		// Save the test script, so that we don't lose the link to the ASE script	
		this._editor.onSave();
		
		// Cannot select template again ocne scan has been created
		this._aseSelectedTemplate.disabled = true;	
		
		// Connect the Edit Scan onclick
		// disconnect handle if it exists
		if (this._aseCreateEditScanLinkHandle)
			this.disconnect(this._aseCreateEditScanLinkHandle);
			
		this._aseCreateEditScanLinkHandle = this.connect(this._aseCreateEditScanLink, "onclick", this._editScan);
		
		// Hide the dialog preventing user from interacting with the test script
		this.dialog.hide();
		
		// Edit the scan right away
		this._newScan();
	},	
	
	_newScan: function() {
		var path = "Jobs/PostCreateSimpleJob.aspx?fiid="+ this._aseCreateEditScanLink.jobId;
		var link = new com.ibm.rqm.appscan.web.ui.internal.view.common.ASELink();
		link.initialize({ path : path, text: "ASE" });
		var fulllink = link.getLink();
		var aseWindow = link.navigate();
	},
	
	_editScan: function() {
		var path = "Jobs/SimpleJobBasicConfig.aspx?fiid="+ this._aseCreateEditScanLink.jobId;
		var link = new com.ibm.rqm.appscan.web.ui.internal.view.common.ASELink();
		link.initialize({ path : path, text: "ASE" });
		var fulllink = link.getLink();
		var aseWindow = link.navigate();
	},
	
	_createScanPopup: function() {
		var props = {
			title: "<img src='" + ImageCache.PROGRESS + "'> &nbsp;&nbsp;" + Constants.getMessage("creating"),
          	style: "position: absolute; top: 20%; left: 40%; width: 450px; height: 125px;"
       	};
		var dialogContent = document.createElement("div");
		dialogContent.innerHTML = Constants.getMessage("creating_scan");		
        this.dialog = new dijit.Dialog(props, dialogContent);
		this.dialog.closeButtonNode.style.display = "none";
		this.dialog.show();
	}
});

})();