/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.planning.web.ui.internal.view.construction.ManageSectionsDialog");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.ManageSectionsDialog");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.SortableOptionChooser");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog");

(function(){ // module container

	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var SortableOptionChooser = com.ibm.asq.common.web.ui.internal.widgets.SortableOptionChooser;
    var NewSectionDialog = com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog;
	var YesNoDialog = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.ManageSectionsDialog", [dijit._Widget, dijit._Templated], {
        
		templateString: '<div class="com-ibm-rqm-planning-editor-manage-sections">' +
                        '    <div dojoAttachPoint="_showAll">' +
                        '        <input type="checkbox" dojoAttachPoint="_showAllCheckbox" dojoAttachEvent="onclick: _onShowAllChange"></input>' +
                        '        <span class="default-label" dojoAttachPoint="_showAllLabel">${labelShowAllSections}</span>' +
                        '    </div>' +
		                '    <div dojoAttachPoint="_chooserNode">' +
		                '</div>',		
		
		width: "600px",
		labelShowAllSections: Constants.getMessage("labelShowAllSections"),
		
        // object initializer
        constructor: function(args) {						
			this.allSections = [];
			this.templateSections = [];
			this.selectedSections = [];
			
			this.onOk = null;
			this.showAll = false;
			
			//by default, disallow creation of sections
			this.enableCreate = false;
			
			//internal
			this._newSections = [];
		},
		
		postCreate: function() {
            this.initialize();		
			this.show();	
		},
		
		initialize: function()
		{
            //append options to the 'chooser' widget
            var availableSections = this.showAll ? this.allSections : this.templateSections;		
			availableSections = availableSections.concat(this._newSections);
            var selectedSections = this._chooser ? this._chooser.getSelectedOptions() : this.selectedSections;
						
			this._chooser = new SortableOptionChooser({
				titleMaxWidth: 160,
				newLabel: Constants.getMessage("labelCreateSection"),
				upLabel: Constants.getMessage("labelMoveUp"),
				downLabel: Constants.getMessage("labelMoveDown"),
				leftTitle: Constants.getMessage("labelAvailableSections"), 
				rightTitle: Constants.getMessage("labelSelectedSections"),
				enableCreate: this.enableCreate
			});
			
			this._chooser.addNewListener(dojo.hitch(this, this.newAction));
			this._chooser.editButton.domNode.style.display = "none";
			this._chooser.deleteButton.domNode.style.display = "none";
			dojox.data.dom.removeChildren(this._chooserNode);
			this._chooserNode.appendChild(this._chooser.domNode);

            //add the unselected sections to the chooser widget
            for (var i = 0; i < availableSections.length; i++) {
                var section = availableSections[i];
                var selected = this._findIn(section, selectedSections) != null;
                //add the option to the chooser widget
                if (!selected) {
                    this._chooser.addOption(Constants.getExtensionDisplayMessage(section.name), section, false, false);
                }
            }			

            //add the unselected sections to the chooser widget
            for (var i = 0; i < selectedSections.length; i++) {
                var section = selectedSections[i];
                //add the option to the chooser widget
                this._chooser.addOption(Constants.getExtensionDisplayMessage(section.name), section, true, section.required);
            }			
		},
		
		show: function() {
			if (!this._dialog) {
	            var dialogArgs = {
	                primaryTitle: Constants.getMessage("labelManageSectionsTitle"),
					okCallback: dojo.hitch(this, "_onOk"),
	                cancelCallback: dojo.hitch(this, "_onCancel"),
					callbackResponse : false
	            };
	            
	            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
	            this._dialog.setWidth(this.width);
	            this._dialog.contentContainer.appendChild(this.domNode);
	            this._dialog.getOkButton().disabled = false;								
			}
			this._dialog.open(false);
		},
		
		_onOk: function()
		{
            var selected = this._chooser ? this._chooser.getSelectedOptions() : [];			
			if (dojo.isFunction(this.onOk)) {
				var message = this._checkChosenSections(selected);
				if (message != "") {
					// Provide a confirmation dialog when the user has removed sections on clicking on "OK"
					// notifying the user that content of removed sections will be lost.
					var self = this;
					var args = {
						title: Constants.getMessage("labelConfirmation"),
						message: message,
						yesCallback: function(){
							self._dialog.close();
							self.onOk(selected);
						},
						noCallback: function(){
							self._onCancel();
						}
					}
					var dialog = new YesNoDialog(args);
					dialog.setSecondaryTitle(args.title);
					dialog.open();
				}
				else {
					this._dialog.close();
					this.onOk(selected);
				}	 	       
			}
			else 		
				this._onCancel();		
			return false;	
		},
		
		_onCancel: function()
		{
			/* do nothing, SelectionDialog closes itself on cancel
            if (this._dialog) {
                this._dialog.close();    
            }
            */
		},
		
		_onShowAllChange: function() {
            this.showAll = this._showAllCheckbox.checked;
			this.initialize();		
		},
		
		newAction: function() {
			var dialogArgs = {		
			    onOk: dojo.hitch(this, "_newSectionAdded")
			}
			this._newSectionDialog = new NewSectionDialog(dialogArgs);			
		},
		
        _newSectionAdded: function(sectionId, sectionName, sectionDescription, args) {
            var newSection = {
                attributes: [{ attributeType: args.type, name: "content"}],
                id: sectionId,
                jsClass: args.type == Constants.DYNAMICSECTION_GRID ? "com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicGridSection": null,
                name: sectionName,
				description: sectionDescription
            };
			if(args.type == Constants.DYNAMICSECTION_GRID){
				newSection.configurationData = args.configurationData;
				newSection.attributes = [{type: "GRID", name: "GRID", id: "dynamicGrid"}];
			}
			if (this._chooser) {
                this._chooser.addOption(sectionName, newSection, true, true);	
				this._newSections.push(newSection);
			}            
        },
		
		_checkChosenSections: function(newSelectedArray) {
			var warningMessage = "";
			if(this._selectedOptions && this._selectedOptions.length) {
				for(var i = 0; i < this._selectedOptions.length; i++) {
					var section = this._selectedOptions[i].getSection();
					if(this._findIn(section, newSelectedArray) == null) {
						warningMessage = Constants.getMessage("msgLostRemovedSections");
						break;
					}
				}
			}
			return warningMessage;
		}, 
		
		_findIn: function(section, sectionArray) {
			if (sectionArray && sectionArray.length > 0) {
				for(var i= 0; i < sectionArray.length; i++) {
					if(sectionArray[i].id == section.id)
						return sectionArray[i];
				}
			}
			return null;
		},
		
		_error: function(e)
		{
			console.error(this.declaredClass + ": " + (e ? e.message : "Unknown"));
		}
    });
    
})();
