/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.grid.GridColumnEditor");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
	var SelectUtility = com.ibm.asq.common.web.ui.internal.util.SelectUtility;
	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

    /**
     *
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/construction/NewSectionDialog.html"),
        
		labelSectionName: Constants.getMessage("labelSectionName"),
		labelDescription: Constants.getMessage("labeldescription"),
		labelApplyToTemplate: Constants.getMessage("labelApplyToTemplate"),
		labelOk: Constants.getMessage("labelOk"),
        labelCancel: Constants.getMessage("labelCancel"),
        labelTypeOfSection: Constants.getMessage("labelTypeOfSection"),
		
		
        constructor: function(args) {
			this._typeIdMap = {};
			this._typeIdMap[Constants.DYNAMICSECTION_GRID] = Constants.DYNAMICSECTION_GRID_ID;
			this._typeIdMap[Constants.DYNAMICSECTION_RICHTEXT] = Constants.DYNAMICSECTION_RICHTEXT_ID;
			this.isEditMode = args.isEditMode ? true : false;
		},
        
        postCreate: function(){
            
			if (!dojo.isFunction(this.onOk)) {
				throw new Error("Parameter 'onOk' is REQUIRED and must be a Function.");
			}
			
			//hide not-yet-needed ui elements
			this._applyTemplateCheckbox.style.display = "none";
			this._applyTemplateLabel.style.display = "none";
			this._messages.style.display = "none";
			
			if (this.isEditMode && !dojo.isString(this.primaryTitle)) {
				throw new Error("Parameter 'primaryTitle' is REQUIRED and must be a String.");
			}
			
            var dialogArgs = {
				primaryTitle: this.isEditMode ? this.primaryTitle : Constants.getMessage("labelNewSection"),
                okCallback: dojo.hitch(this, "_onOk"),
                cancelCallback: dojo.hitch(this, "_onCancel"),
				_truncate: function() {}  //disable the Dialog's '_truncate'
            };      
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            this._dialog.setWidth("500px");
			var cshelp = new CSHelp({contextID: "rqm_new_custom.html"});
			cshelp.setButtonLabel(Constants.getMessage("labelNewSectionCSHelp"));
			this._dialog.setSecondaryTitleNode(cshelp.domNode);
			this._dialog.contentContainer.appendChild(this.domNode);
            this._dialog.getOkButton().disabled = false;
            this._dialog.open();		
			
			var opt = new Option(Constants.getMessage("labelRichTextEditor"), Constants.DYNAMICSECTION_RICHTEXT);
			var opt1 = new Option(Constants.getMessage("labelGrid"), Constants.DYNAMICSECTION_GRID);
			SelectUtility.add(this._typeCombo, opt);
			SelectUtility.add(this._typeCombo, opt1);
			this.currentColumnValue = "1";
			this.columnNames = [];
			
			if(this.isEditMode){
			    
                var sectionInfo = this.sectionInfo;
                //the server retrieve null when no description was type
                if( sectionInfo && sectionInfo.description == null){
                     sectionInfo.description = "";
                }
				if (!dojo.isString(sectionInfo.name)) {
					throw new Error("Parameter 'sectionInfo.name' is REQUIRED and must be a String.");
				}
				if (!dojo.isString(sectionInfo.description)) {
					throw new Error("Parameter 'sectionInfo.description' must be a String.");
				}
				this._name.value = Constants.getExtensionDisplayMessage(sectionInfo.name);
				this._description.value = Constants.getExtensionDisplayMessage(this.sectionInfo.description);
				
				       
                var colNames = dojo.fromJson(sectionInfo.configurationData);
                if (dojo.isArray(colNames)){
                    this.columnNames = colNames;
                    opt1.selected = true;
                    dojo.removeClass(this._gridConfigRow, "hidden");
                }
			}
			
			var columnArg = {
                parent: this,
                columnNames: this.columnNames
            };
            this.columnsWidget= new com.ibm.asq.common.web.ui.internal.widgets.grid.GridColumnEditor(columnArg);
        
            this._columnsWidget.appendChild(this.columnsWidget.domNode); 
            
			this.connect(this._typeCombo, "onchange", "_handleTypeChange");
			setTimeout(dojo.hitch(this, function() {this._name.focus()}), 250);		
        },
        
        _onCancel: function() {
			if (this._dialog) {
				this._dialog = null;
			}			
        },
        
        _onOk: function() {
			this._messages.style.display = "none";
			dojox.data.dom.removeChildren(this._messages);
            if (this._name.value == null || dojo.trim(this._name.value).length == 0) {
				dojox.data.dom.removeChildren(this._messages);
				this._messages.style.display = "inline";
				this._messages.appendChild(document.createTextNode(Constants.getMessage("labelSectionNameRequired")));
				return false;
            } else {
				var type = this._typeCombo.value;
				var isGrid = (type == Constants.DYNAMICSECTION_GRID);
				
				if (dojo.isFunction(this.onOk)) {
					var params = {
						type: type
					};
					if (isGrid){
					   if(this.columnsWidget.checkColumnNames()){
					       var msg = this.columnsWidget.getErrorMessage();
					       this._messages.style.display = "inline";
					       this._messages.appendChild(document.createTextNode(msg));
					       return false;
					   }
					   var columnNames = [];
					   var columnsInfo = this.columnsWidget.getColumnNames();
					   for(var i = 0; i < columnsInfo.length; i++){
					       columnNames.push(columnsInfo[i].value);
                       }
					   params.configurationData = dojo.toJson(columnNames);
					}
					
					var uid = this._typeIdMap[type] + new Date().getTime();
	                this.onOk(uid, this._name.value, this._description.value, params);
	            }		
            }
        },
        
        
         
        _onKeyup: function(e){
    /*
    enter key on any one of the text fields should not close the dialog 
                if (e.keyCode == dojo.keys.ENTER) {
                    if (this._dialog) {
    					this._dialog.getOkButton().onclick();
    				}
                }
    */
            if(e){
                dojo.stopEvent(e);
            }
        },

		_handleTypeChange: function() {
			var isGrid = (this._typeCombo.value == Constants.DYNAMICSECTION_GRID);
			if (isGrid) {
				dojo.removeClass(this._gridConfigRow, "hidden");
			} else {
				dojo.addClass(this._gridConfigRow, "hidden");
			}
		}
    
      
        
		
    });
})();
