//debug projectareaselection
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation  2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.construction.ProjectAreaSelector");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.ProjectAreaSelector");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("jazz.app.events");

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.QueryableProjectArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.ProjectAreaSelectorDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
// Summary
//
// Encapsulate Project Selection functionality
//
//

(function()
{
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
	var QueryableProjectArea = com.ibm.rqm.planning.web.client.internal.QueryableProjectArea;
	var ProjectAreaSelectorDialog = com.ibm.rqm.planning.web.ui.internal.view.construction.ProjectAreaSelectorDialog;		
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
	var MAX_LENGTH_GLOBAL_MODE = 10;
	var MAX_LENGTH_DIALOG_MODE = 50;

	// Setup handlers for project area add, and delete via admin UI	
	var EVENTS_ROOT = "jazz.app.events.contextSwitcher";
	dojo.setObject(EVENTS_ROOT, {deleteContext: EVENTS_ROOT + ".deleteContext", addContext: EVENTS_ROOT + ".addContext"});
		
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.ProjectAreaSelector", [dijit._Widget, dijit._Templated], {
	    templateString: '<div dojoAttachPoint="_projectAreaCombo">' +
		                '</div>',
		_projectAreaCombo: null,
		_fselect: null,
		_maxResults: MAX_LENGTH_DIALOG_MODE,
		_onValueChanged: null,
					
		 constructor: function(args){
		 	if(args)
		 		this.selectorArgs = args;
			else
				this.selectorArgs = {};
		 },
		 
		 
		// Request the project areas, create a placeholder project area to prevent redraw issues. Replace it
		// after the request is fulfilled.
				
       	postCreate: function() {			
			// Refresh the combobox if the user adds a project area
            this.subscriptions = [];
			this.subscriptions.push(dojo.subscribe(jazz.app.events.contextSwitcher.deleteContext, this, "deleteProject"));
			this.subscriptions.push(dojo.subscribe(jazz.app.events.contextSwitcher.addContext, this, "addProject"));
			
			
			if(this.globalMode) {
				this._fselect =  document.createElement("select");
				this._projectAreaCombo.appendChild(this._fselect);
			}				
			else {
                // show a loading message while loading the project areas contents. 
                dojo.style(this._projectAreaCombo, "width", "18em");
                this._projectAreaCombo.innerHTML = Constants.getMessage("labelLoading");
			}
            this._requestProjectAreas();
		},
		
		
		_onValueChanged: function()
		{
			if(!this.globalMode){
				var value = this._fselect.attr('displayedValue');			
				var pas = this._userProjectAreas;
				var len = pas ? pas.length : 0;
				for(var i = 0; i < len; i++){
					if(pas[i].name == value){
						this.onValueChanged(value, true);
						return;
					}
				}
				this.onValueChanged(value, false);
			}			
		},
			
		// First time in request project areas from the server, get either a small number of large depending on the context
		// In global mode the selector is always selectable, it's not a dialog context. Use a different control and fewer items
		// This is consistent with the current presentation in the UI.
		
		_requestProjectAreas: function()
		{
			this._maxResults =  this.selectorArgs.globalMode ? MAX_LENGTH_GLOBAL_MODE: MAX_LENGTH_DIALOG_MODE ;
			var args = {};				
			args.maxResults = this._maxResults;
			if(this.selectorArgs.itemTypeNSUri)
				args.itemTypeNSUri = this.selectorArgs.itemTypeNSUri;
			if(this.selectorArgs.itemTypeName)
				args.itemTypeName = this.selectorArgs.itemTypeName;
			var srh = new ServiceResponseHandler(this, "_buildProjectAreaList", "_buildProjectAreaListFailed");
			QueryableProjectArea.getInitialProjectAreas(srh, args);
		},
		
		// Callback from query. Build the items based on the return of the query
		_buildProjectAreaList: function(userProjectAreas)
		{
			
			this._userProjectAreas = userProjectAreas;
			this._useMore = false;
			if(userProjectAreas && userProjectAreas.length == this._maxResults)
				this._useMore = true;
			var currentProjectArea = null;
			// When the system is first starting up, there are cases where there is no current project area, and an excpetion is thrown
			try{
				var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			}
			catch(e){
			}
			this._userProjectAreas.sort(dojo.hitch(this, 'compare'));
			if(currentProjectArea == null && userProjectAreas && userProjectAreas.length > 0){
				currentProjectArea = userProjectAreas[0];				
			}
			this.generateItems(currentProjectArea, true);			
			this._buildUI();
			if(this.selectorArgs.onSelectorBuilt)
				this.selectorArgs.onSelectorBuilt();
		},
		
		compare: function(a, b){
			if(a.name > b.name)
				return 1;
			if(a.name == b.name)
				return 0;
			return -1;
		},
		
		// Service response failed		
		_buildProjectAreaListFailed: function()
		{
		},
		
		// Buid the user interface for the selector, either a combobox or a fancy combo box depending on the mode
		_buildUI: function(){
			if(this.selectorArgs.globalMode){
				this._createSelectionList();
				dojox.data.dom.removeChildren(this._projectAreaCombo);							
				this._projectAreaCombo.appendChild(this._fselect);
				this.connect(this._fselect, 'onchange', '_globalSelectionChanged');			
			}
			else {
				
				// Build the data structure necessary for Filtering Select
				var dstorejson = {
  				identifier:"name",
  				label: "label",
  				items: this.items};
				this.dstore = new dojo.data.ItemFileReadStore({data: dstorejson});
			
				// Assign a default value based on the currently selected value from item generation
				var args = {};
				if(this.selectedValue)
					args.value = this.selectedValue;
				args.store = this.dstore;
				args.autoComplete = "true";
                
                this._projectAreaCombo.innerHTML = "";
				this._fselect =  new dijit.form.FilteringSelect(args);
				this._projectAreaCombo.appendChild(this._fselect.domNode);
				this.connect(this._fselect, 'onChange', '_selectionChanged');
				this.connect(this._fselect, 'onKeyPress', 'onKeyPress');

				this._endTimer();											
				this.timer = setInterval(dojo.hitch(this, "_onValueChanged"), 1000);
				this._fselect.focus();
				if(this._fselect.textbox)
					dijit.selectInputText(this._fselect.textbox);				
			}
		},
		
		onKeyPress: function(evt){
			if(evt.keyCode == 13 && this.onEnterSelected) // CR
				this.onEnterSelected();
		},
		
		// Stop the polling timer, called when the action completes or fails				
		_endTimer:function(){
			if(this.timer){
				clearInterval(this.timer);
				delete this.timer;
		     }	
		},	
		
		// Build a selection list if in global mode, use classnames from Jazz ContextSelector to 
		// Give it the right look and feel.
		_createSelectionList: function()
		{
			this._fselect = document.createElement("select");
			dojo.addClass(this._fselect, this.selectorArgs.className);

			if(this.items.length > 0){
				for(var i = 0; i < this.items.length; i++){
					var item = this.items[i];
					var option = new Option(item.text, item);
					if(item.text == this.selectedValue)
					{
						option.className = "selected";						
						option.selected = true;
					}
					this._fselect.options[i] = option;
					
				}
			}
		},

		// Create an item, item is a project area item. It contains the project area name and item id
		// In some contexts just the name but always as a field
		// TODO index is no longer used
		createItem: function(pa, index)
		{
			var item = {};
			item.name = pa.name;			
			item.itemId = pa.itemId;								
			item.label = pa.name;
			item.text = pa.name;
			return item;	
		},
		
		// TODO autoAddDefault is no longer used in a meaningful way. Code could be simplified by building
		// user list and then generating items
		// Build Items from the list of project areas returned by the database call, or update with new items
		generateItems: function(defaultItem, autoAddDefault)
		{
			var userProjectAreas = this._userProjectAreas;
			var items = [];
			var j = 0;
			
			this.items = items;
			var selectedValue ="";	
			var len = userProjectAreas ? userProjectAreas.length: 0;
			// Only happens when there are no more items in the current list
			if(defaultItem != null){
				// Add the project areas to the list
				for (var i=0; i< len; i++) {
					if(userProjectAreas[i].name == defaultItem.name)
						selectedValue = defaultItem.name;
					items[j++] = this.createItem(userProjectAreas[i], j)																
				}
			
				// If the default isn't in the list add it if it should be, otherwise, make the first item
				// the default.
				if(selectedValue == "" && userProjectAreas){
					if(autoAddDefault){
						userProjectAreas[userProjectAreas.length] = defaultItem;
						items[j++] = this.createItem(defaultItem, j);
						selectedValue = defaultItem.name;
					}
					else {
						selectedValue = userProjectAreas[0].name;
					}
				}
			}
			
			// If more is required add it also	
			if(this._useMore){
				items[j]= {};
				items[j].name = Constants.getMessage("labelMoredot");
				items[j].itemId = "";
				items[j].label = Constants.getMessage("labelMoredot");
				items[j].text =  Constants.getMessage("labelMoredot");
			}
			this.selectedValue = selectedValue;
		},
		

		// A new item was created update the items and rebuild the UI. 
		_refreshListAndSetNewDefault: function(newItem)
		{
			this.items = [];
			this.generateItems(newItem, true);
			this._buildUI();						
		},

		// Ok from global context means take the action also, callback onChange
		_onOkAndChangeProjectArea: function(newItem)
		{
			this._refreshListAndSetNewDefault(newItem);
			try {
				this.selectorArgs.onChange(newItem);
				this.selectedValue = newItem.name;
			}catch(error){
				var items = this.items;
				var len = items.length;
				var _fselect = this._fselect;
				for(var i=0;i<len;i++){
					if(items[i].name == this.selectedValue){
						_fselect.options[_fselect.selectedIndex].className = "";
						_fselect.options[i].selected = true;
						_fselect.options[i].className = "selected";
						break;
					}
				}
			}				
		},
		
		
		
		// On Ok in the dialog context just updates the item		
		_onOk: function(newItem)
		{
			this._refreshListAndSetNewDefault(newItem);
		},
		
		// On cancel return back to the currently selected item, it will be more otherwise.
		_onCancel: function()
		{
			this._refreshListAndSetNewDefault({ name: this.selectedValue});
		},

		// A new selection in the global case, either show the advanced dialog or callback with the new project area 
		// and change
		_globalSelectionChanged: function(evt){
			var pos = evt.target.selectedIndex;
			var originalPos = evt.target.selectedIndex;
			var fselect = this._fselect;
			var text = fselect.options[pos].text;
			fselect.options[pos].className = "selected";									
			fselect.options[originalPos].className = "";												
						
			if (text == Constants.getMessage("labelMoredot")) {
				var dialogArgs = {
					onAdvancedCancel: dojo.hitch(this, "_onCancel"),
					onAdvancedOk: dojo.hitch(this, "_onOkAndChangeProjectArea"),
					title: Constants.getMessage("labelSelectProjectArea")
				};
				this.dialog = new ProjectAreaSelectorDialog(dialogArgs);
			}
			else { // Handle more, but context switch otherwise.
				try {
					this.selectorArgs.onChange({name: text});
					this.selectedValue = text;
				}catch(error){
					dojo.stopEvent(evt);
					var _fselect = this._fselect;
					var items = this.items;
					var len = items.length;
					for(var i=0;i<len;i++){
						if(items[i].name == this.selectedValue){
							_fselect.options[i].selected = true;
							_fselect.options[i].className = "selected";
							_fselect.options[pos].className = "";
							break;
						}
					}
				}
			}		
		},
		
		// A selection changed in the dialog case			
		_selectionChanged: function(item)
		{
			if(item == Constants.getMessage("labelMoredot")){
				var dialogArgs = {onAdvancedCancel: dojo.hitch(this, "_onCancel"),
							  onAdvancedOk: dojo.hitch(this, "_onOk"),
							  title: Constants.getMessage("labelSelectProjectArea")
							 };
				if(this.selectorArgs.itemTypeNSUri)
					dialogArgs.itemTypeNSUri = this.selectorArgs.itemTypeNSUri;
				if(this.selectorArgs.itemTypeName)
					dialogArgs.itemTypeName = this.selectorArgs.itemTypeName;			 
				this.dialog = new ProjectAreaSelectorDialog(dialogArgs);
			}
			else {
				this.onValueChanged(item, true);
			}			
		},		
		
		// Called to get the current selection only supported by Dialog mode 
		getSelection: function(selectionCallback)
		{
			this.selectionCallback = selectionCallback;
			if(!this.globalMode){
				var value = this._fselect.attr('displayedValue');
				this.dstore.fetch({
					query: { name: value },
					onItem: dojo.hitch(this, "_getSelection2")
				});
			}	
		},
			
		_getSelection2: function(item, request){
			if(!this.globalMode){
				var itemId = this.dstore.getValue(item, "itemId");			
				this.selectionCallback({
					itemId: itemId,
					name: item.name});
			}
		},
		
		// Context changed event from Project Area Admin
		addProject: function(newName) {
			var userProjectAreas = this._userProjectAreas;
			userProjectAreas.push({name : newName});
			userProjectAreas.sort(dojo.hitch(this, 'compare'));						
			var defaultItem = { name: this.selectedValue};
			this._refreshListAndSetNewDefault(defaultItem);			
		},
	
		deleteProject: function(archivedName) {
			var userProjectAreas = this._userProjectAreas;
			var success = false;
			
			// Archived name might be in the current project area list, or not if the list is really large
			// In that case no need to update combo
			for(var i = 0; i < this._userProjectAreas.length; i++){
				if (userProjectAreas[i].name == archivedName) {
					userProjectAreas.splice(i,1);
					success = true;
					break;
				}
		}
		// If the project area that was archived was the currently active one, then change the current selection
		if(success){
			var defaultItem = { name: this.selectedValue};
			if(this.selectedValue == archivedName){
				if(userProjectAreas.length > 0)
					defaultItem = userProjectAreas[0];
				else 
					defaultItem = null;
			}
			this._refreshListAndSetNewDefault(defaultItem);			
		}
	},
	
	uninitialize: function(){
		this._endTimer();	
        
        if(this.subscriptions) {
            dojo.forEach(this.subscriptions, dojo.hitch(dojo, "unsubscribe"));
            this.subscriptions = null;
        }
	}
	
	});

	
})();
