/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.planning.web.ui.internal.view.construction.TestGroupEditor");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.TestGroupEditor");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.core.loader");

// Require the common widgets needed
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

// Require local widgets
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.WorkItemUtils");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.EditableItemList");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.workitem.BasicWorkItemEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.AbstractArtifactEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotEnabledEditor");

(function(){ // module container
    /* Jazz objects */
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    
    /* QM objects */
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var WorkItemUtils = com.ibm.rqm.planning.web.client.internal.WorkItemUtils;
    var TestGroupClient = com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var SnapshotEnabledEditor = com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotEnabledEditor;
    var AbstractArtifactEditor = com.ibm.rqm.planning.web.ui.internal.view.construction.AbstractArtifactEditor;
    var ImageClient = com.ibm.team.workitem.web.client.internal.ImageClient;
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.TestGroupEditor", [dijit._Widget, dijit._Templated, SnapshotEnabledEditor, AbstractArtifactEditor], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/construction/TestGroupEditor.html"),
        //		templateCssPath: dojo.uri.dojoUri("../com.ibm.rqm.planning.web/ui/internal/view/templates/construction/TestGroupEditor.css"),
        
        // object initializer
        constructor: function(args){
            this._isInitialized = false;
            this._isMinimal = false;
            this._showGroups = true;
            this._showCases = true;
            this._workItemEditor = null;
            this._listener = null;
            this._nameListenerNode = null;
        },
        
        // populate the template with data, connect events, and show initial
        // sub-tab
        postCreate: function(){
        },
        
        initialize: function(args){
			//check for re-initialization hack; needs to be cleaned up
            if (this.getArtifact() && this.parentController) {
                args.testGroup = this.getArtifact();
                args.parentController = this.parentController;
            }
            this.setSnapshotClient(TestGroupClient);
            
            if (args != null) {
                //initialize widget attributes	
                if (args.parentController === null) {
                    throw new Error("NewTestGroup#initialize: Missing required parameter 'args.parentController'");
                }
                
                // specify the elements in the overview tab
                this.addOverviewElement(this._nameRow);
                this.addOverviewElement(this._detailsRow);
                this.addOverviewElement(this._detailsSection);
                this.addOverviewElement(this._editorSections);
                /*
                 this.addOverviewElement(this._testGroupsTree);
                 this.addOverviewElement(this._testCaseList);
                 this.addOverviewElement(this._testGroupList);
                 this.addOverviewElement(this._approval);
                 */
                if (args.testGroup != null) {
                    if (args.isCopy != null && args.isCopy == true) {
                        this.setCopy(true);
                        // clear out the snapshot info
                        args.testGroup.snapshotInfo = null;
                        // change the title to "Copy of ..."
                        args.testGroup.name = Constants.getMessage("labelCopyOf", args.testGroup.name);
                    }
                    else {
                        this.setNew(false);
                    }
                }
                
                this.parentController = args.parentController;
                this._showGroups = args.showGroups != null && args.showGroups == false ? false : true;
                this._showCases = args.showCases != null && args.showCases == false ? false : true;
                this._listener = args.listener != null ? args.listener : null;
                this._nameListenerNode = args.nameListenerNode != null ? args.nameListenerNode : null;
                
                // if this artifact is a snapshot then make it read only and display revision info
                if (args.testGroup != null && args.testGroup.snapshotInfo != null) {
                    this.setReadOnly(true);
                    // this editor is showing a snapshot revision.  remove the save buttons and add "read only" to title
                    this._cancelButton.style.display = "none";
                    this._saveButton.style.display = "none";
                    this.setDirty(false);
                    
                    var button = document.createElement("button");
                    button.appendChild(document.createTextNode("Go to current version"));
                    var currentPerspective = Utility.getCurrentPerspective(this.domNode);
                    var href = Utility.getTestGroupURI(args.testGroup.id, currentPerspective);
                    this.connect(button, "onclick", function(){
                        window.location.href = Utility.reuniquifyUrl(href)
                    });
                    this._currentRevision.appendChild(button);
                }
                else {
                    this._cancelButton.style.display = "inline";
                    this._saveButton.style.display = "inline";
                }
                
                this.setArtifact(args.testGroup);
                
                //hide info for minimal mode
                if (args.minimal && args.minimal == true) {
                    this._isMinimal = true;
                    this._titleRow.className += " hidden";
                    if (!this.isNew()) {
                        //this._nameRow.className += " hidden";
                    }
                    else {
                        this._status.className += " hidden";
                    }
                    this._detailsRow.className += " hidden";
                    //this._bottomButtonRow.className = "title-row";
                }
                
                this._isInitialized = true;
                
            }
            else 
                if (!this._isInitialized) {
                    throw new Error("TestGroupEditor#initialize: Missing required parameter 'args'");
                    return;
                }
            
            var cshelpDetails = new CSHelp();
            cshelpDetails.contextID = "rqm_details.html";
			dojox.data.dom.removeChildren(this._cshelpDetails);
            this._cshelpDetails.appendChild(cshelpDetails.domNode);
            
            //basic form information				
            if (args.testGroup != null) {
                this._name.value = args.testGroup.name;
                this._originator.innerHTML = args.testGroup.originator.name;
                if (args.testGroup.description != null) {
                    this._description.value = args.testGroup.description;
                }
                EditorUtil.setPageTitle(this._title, Constants.TYPE_GROUP, args.testGroup);
                EditorUtil.initContributors(this._owner, this._getWorkItem() != null ? this._getWorkItem().owner.id : Constants.getMessage("labelUnassigned"));
                EditorUtil.initArtifactStates(this._status, args.testGroup.state);
                
            }
            else {
                this._name.value = "";
                this._description.value = "";
                this._originator.innerHTML = EditorUtil.getCurrentContributor().name;
                EditorUtil.setPageTitle(this._title, Constants.TYPE_GROUP);
                EditorUtil.initContributors(this._owner);
                EditorUtil.setLastSaved(this._lastSaved, Constants.STATUS_NEW);
            }
            
            //display the sections
            this.loadSections("com.ibm.rqm.planning.editor.testgroup", this._editorSections);
            
			
            this._showEWILink();
			this.updateWorkItemInfo();
            
            //set initial focus
            this._name.focus();
        },
        
        _generateEWI: function(event){
            //TODO complete hack until we get pluggable UI actions...
            var EWI_WIZARD_TYPENAME = "com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIWizard";
            jazz.core.loader.load_sync(EWI_WIZARD_TYPENAME);
            var ewiWizardConstructor = dojo.getObject(EWI_WIZARD_TYPENAME);
            var group = this.getArtifact();
            if (group && group.id) {
                var self = this;
                var args = {
                    title: Constants.getMessage("labelGenEWIs"),
                    parentController: this.parentController,
                    callback: {
                        context: self,
                        method: self.update
                    },
                    artifact: group
                }
                this._ewiWizard = new ewiWizardConstructor(args);
                
            }
            else {
                alert("Please Save this Test Case Group before generating any Test Execution Records.")
            }
            if(event){
                dojo.stopEvent(event);
            }
            
        },
        
        _showEWILink: function(){
            var group = this.getArtifact();
            if (group && group.testCases && group.testCases.length > 0) {
                if (dojo.hasClass(this._ewiLink, "hidden")) {
                    dojo.removeClass(this._ewiLink, "hidden");
                }
            }
            else {
                if (!dojo.hasClass(this._ewiLink, "hidden")) {
                    dojo.addClass(this._ewiLink, "hidden");
                }
            }
        },


        
        ////////////////////////////////////////////////////////////////////////////////////////////////////
        //WORKITEM POPUP FUNCTIONALITY
        ////////////////////////////////////////////////////////////////////////////////////////////////////        
        updateWorkItemInfo: function(){
            //uses this._workItem
            var workItem = this._getWorkItem();
            var stateName = workItem != null ? workItem.state.label : EditorUtil.getWIStateForAction(this.getWorkflowAction()).name;
            var stateIconURL = workItem != null ? workItem.state.iconUrl : EditorUtil.getWIStateForAction(this.getWorkflowAction()).icon;
            
            EditorUtil.initContributors(this._owner, workItem != null ? workItem.owner.id : null);
            EditorUtil.initWorkItemStates(this._state, stateName);
            this._displayWorkItemNumber();
            this._stateIcon.src = ImageClient.getImageUrl(stateIconURL);
            
        },
        
        _displayWorkItemNumber: function(){
            if (this._getWorkItem() != null) {
                this._state.disabled = false;
                dojox.data.dom.removeChildren(this._workitemnumber);
                var anchor = EditorUtil.setWorkItemNumberAsURL(this._workitemnumber, this._getWorkItem());
                anchor.href = "javascript:{}";
                this.connect(anchor, "onclick", this._displayWorkItemPopup);
            }
        },
        
        _displayWorkItemPopup: function(){
            var self = this;
            var args = {
                callback: {
                    contextObj: self,
                    method: self.updateWorkItem,
                    returnAsDTO: true
                },
                workItem: self._getWorkItem() != null ? self._getWorkItem() : null
            }
            this._wiPopup = new com.ibm.rqm.planning.web.ui.internal.view.common.QMWorkItemPane(args);
            this._workitemnumber.appendChild(this._wiPopup.domNode);
        },
        
        updateWorkItem: function(workItem){
            if (this.getArtifact() != null && this.getArtifact().workItems != null && this.getArtifact().workItems.length > 0) {
                this.getArtifact().workItems[0].workItem = workItem;
            }
            this.updateWorkItemInfo();
        },
        ////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////		
        
        _getWorkItem: function(){
            if (this.getArtifact() != null && this.getArtifact().workItems != null && this.getArtifact().workItems.length > 0) {
                return this.getArtifact().workItems[0].workItem;
            }
            return null;
        },
        
        getWorkflowAction: function(){
            return this._state.value;
        },
        
        setNameListenerNode: function(node){
            this._nameListenerNode = node;
        },
        
        //		_showAssignmentLink: function() {
        //			dojox.data.dom.removeChildren(this._assignWorkItem);
        //			var link = document.createElement("a");
        //			link.title = "Edit Assignment Details";
        //			link.innerHTML = "View/Edit WorkItem";
        //			this.connect(link, "onclick", this._showWorkItemDialog);	
        //			this._assignWorkItem.appendChild(link);
        //		},
        
        _showWorkItemDialog: function(){
            var dialog = null;
            if (!this._workItemEditor) {
                dialog = new dijit.Dialog({
                    title: "View/Edit Assignment",
                    bgColor: "white",
                    bgOpacity: "0.5",
                    toggle: "plain",
                    toggleDuration: "250"
                });
                this._workItemEditor = new com.ibm.rqm.planning.web.ui.internal.view.workitem.BasicWorkItemEditor();
                this._workItemEditor.initialize({
                    workItem: this._getWorkItem(),
                    dialog: dialog,
                    defaultSummary: Constants.TEST_GROUP_WORKITEM_SUMMARY,
                    initOwner: this._owner.value
                });
                dialog.domNode.appendChild(this._workItemEditor.domNode);
                this.connect(dialog, "hide", this._onWorkItemUpdate);
                document.body.appendChild(dialog.domNode);
                this.dialog = dialog;
            }
            else {
                this._workItemEditor.synchronizeOwner(this._owner.value);
            }
            this.dialog.show();
        },
        
        _onWorkItemUpdate: function(){
            EditorUtil.initContributors(this._owner, this._workItemEditor.getWorkItemData().owner);
        },
        
        _onNameKeyUp: function(){
            if (dojo.trim(this._name.value).length > 0) {
                EditorUtil.setPageTitle(this._title, Constants.TYPE_GROUP, {
                    name: this._name.value,
                    snapshotInfo: this.getSnapshotInfo()
                });
                //if any node is listening for the name, update that node
                if (this._nameListenerNode != null) {
                    this._nameListenerNode.innerHTML = this._name.value;
                }
            }
            else {
                EditorUtil.setPageTitle(this._title, Constants.TYPE_GROUP, this.getArtifact() ? this.getArtifact() : null);
                if (this._nameListenerNode != null) {
                    this._nameListenerNode.innerHTML = this.getArtifact() ? this.getArtifact().name : "New " + Constants.TYPE_GROUP;
                }
            }
        },
        
        _testGroupSaved: function(testGroup){
            this.setArtifact(testGroup);
            this.setNew(false);
            this.setCopy(false);
            EditorUtil.setPageTitle(this._title, Constants.TYPE_GROUP, testGroup);
            EditorUtil.setLastSaved(this._lastSaved, Constants.STATUS_SUCCESS);
            this._workItemEditor = null; //force reload of BasicWorkItemEditor
            //this._showAssignmentLink();
            if (this._listener && this._listener.obj && this._listener.method) {
                if (dojo.isFunction(this._listener.method)) {
                    this._listener.method.call(this._listener.obj, testGroup, this._listener.index);
                }
            }
        },
        
        _saveError: function(status){
            status.caughtAt = "TestGroupEditor#_saveError";
            EditorUtil.setLastSaved(this._lastSaved, Constants.STATUS_FAILURE, status);
            //alert("An error occurred: [" + e.message + "] @ TestGroupEditor#_saveError");
        },
        
        // @override for SaveParticipant
        validate: function(){
            //validate fields
            if (dojo.trim(this._name.value).length == 0) {
                if (this.isNew()) {
                    alert("A Test Group Name is required.  Please provide a Test Group Name in the Overview section before saving.");
                    return false;
                }
                else {
                    this._name.value = this.getArtifact().name;
                    this._onNameKeyUp();
                }
            }
            EditorUtil.setLastSaved(this._lastSaved, Constants.STATUS_SAVING);
            return true;
        },
        
        // @override for SaveParticipant
        getAttributes: function(){
            var wiEditor = this._workItemEditor;
            
            var wi = this._getWorkItem();
            var workItemData = {
                projectAreaName: EditorUtil.getCurrentProjectArea().name,
                owner: this._owner.value,
                state: EditorUtil.getWIStateForAction(this.getWorkflowAction()).stateId,
                workItemItemId: wi != null ? wi.itemId : null
            };
            
            var groupObj = {
                newTestGroup: this.isNew(),
                id: this.isNew() ? null : this.getArtifact().id,
                name: this._name.value,
                ownerId: this._owner.value,
                description: this._description.value,
                workItemJsonString: dojo.toJson(workItemData)
            };
            
            groupObj.state = this._status.value;
            
            // snapshot properties
            if (this._createSnapshot == true) {
                groupObj.createSnapshot = true;
                groupObj.snapshotName = this._snapshotName;
                groupObj.snapshotDescription = this._snapshotDescription;
            }
            return groupObj;
        },
        
        // @override for SaveParticipant
        getItemType: function(){
            return Constants.MODEL_TYPE_GROUP;
        },
        
        // @override for SaveParticipant
        saveFinished: function(savedItem){
            savedItem = WorkItemUtils.fixupArtifactWorkItems(savedItem);
            this._testGroupSaved(savedItem);
            this.updateWorkItemInfo();
            
            if (this._createSnapshot) {
                // refresh the snapshot list
                this._snapshotPane.refreshSnapshots();
                
                // unset the snapshot properties on the editor so next save doesn't create a snapshot
                this._createSnapshot = false;
                this._snapshotName = null;
                this._snapshotDescription = null;
            }
            this._showEWILink();
        },
        
        _onChange: function(e){
            var iconUrl = EditorUtil.getWIStateForAction(this.getWorkflowAction()).icon;
            this._stateIcon.src = ImageClient.getImageUrl(iconUrl);
        },
        
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ TestGroupEditor#_error");
        }
    });
    
})();
