/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.AbstractTableSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.NewItemPopup");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function(){ // module container   
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
	var PlanningPageClient = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
	var TestPlanMapper = com.ibm.rqm.planning.web.client.internal.mapper.TestPlanMapper;
    var TestPlanTableView = com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView;
    var SelectItemTableDialog = com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog;
    var NewItemPopup = com.ibm.rqm.planning.web.ui.internal.view.construction.NewItemPopup;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
    var YesNoDialog = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
	
    // global save context
    var SaveParticipant = com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.AbstractTableSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        // @Abstract 
		// Template will likely need to be overridden and MUST include a '_tableNode'
		templateString: '<div class="com-ibm-rqm-planning-web-ui-internal-view-construction-extension-AbstractTableSection">' +                
						'  <div dojoAttachPoint="_tableNode"></div>' +
						'</div>',
        
        constructor: function(args){
            this._items = [];
            this._itemsAdded = [];
            this._itemsRemoved = [];
            this._initialized = false;
			this.useSaveContext = true;
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this.addWorkItemSupport();
			
			//init type specific variables - to be done in subclass postCreate
            this._itemType = null;
			this._modelType = null;	
			this._referencesAddedType = null;
            this._referencesRemovedType = null;
        },
        
        initialize: function(){
            this._itemsAdded = [];
            this._itemsRemoved = [];

			this.showTable();
			var ret = this.refreshTable("_setItems", "_error");
			if(!ret)
				this._initialized = true;		
			this.setSectionDirty(false);
        },
		
		isUseSaveContext: function() {
			return this.useSaveContext;
		},		
		
		refreshTable: function() {
            if (this.getParentEditor().isNew() && !this.getParentEditor().isCopy()) {
                return false;			 
			}
			this.refreshContent();
			return true;
		},
		
		// @Abstract
		// extenders MUST override this method to provide for the table content 
		refreshContent: function() {
			//to be implemented by subclasses
		},
		
		_refreshContentComplete: function(items) {
            
            this.setItems(items ? items : []);
			if(this.getParentEditor().isCopy())
				this._itemsAdded = items;			
            this.showTable();
            this._initialized = true;			
			//if new item was clicked before save, open it
			this._openNewlySavedItem();
		},		
		
		// @Abstract
		// extenders MUST override this method to create the table widget 
		createTable: function(artifact) {
			return null;
		},	
		
        //@Overridden from AbstractArtifactEditorSection
		// SHOULD be re-overridden if any attributes necessary
        getAttributes: function(){
            return null;
        },
        
        showTable: function(){
			//create the table if necessary; refresh it otherwise
            if (!this._tableViewer) {
                dojox.data.dom.removeChildren(this._tableNode);
                var tableViewer = this.createTable();
				if (tableViewer != null) {
					this._tableViewer = tableViewer;
					this._tableNode.appendChild(this._tableViewer.domNode);
				} else {
					//no table widget returned; throw exception
					var msg = this.delcaredClass + ": createTable returned NULL table widget!";
					console.error(msg);
					throw new Error(msg);
				}
            
			} else {
                this._tableViewer.refresh(this.getItems());
            }
        },
        
        getItems: function(){
            return this._items;
        },
        
        setItems: function(items){
            this._items = items;
        },
		
		containsItem: function(candidate, propertyName) {
			var items = this.getItems();
			var itemLen = (items || []).length;
			for (var i = 0; i < itemLen; i++) {
				var item = items[i];
				if (item[propertyName] == candidate) {
					return true;
				}
			}
			return false;
		},		
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(){
            this.inherited('saveFinished', arguments);
            
            // if we have not been shown, don't need to
            // create the content
            if (this._initialized) {
                this.initialize();
            }
            this.addWorkItemSupport();			
			this.updateRefreshable();
        },
		
		//@Overridden from AbstractArtifactEditorSection
		setSectionDirty: function(dirty) {
			this.inherited('setSectionDirty', arguments);
			this.updateRefreshable();
		},
		
		updateRefreshable: function() {
            if (this._tableViewer && this._tableViewer.refreshAction) {
                this._tableViewer.refreshAction.setDisabled(this.isSectionDirty());
            }			
		},	
		       
        _handleAddClick: function(){
            this._showItemsDialog();
        },
        
        _handleAddNewClick: function(){
            var args = {
                itemType: this._modelType,
				useSaveContext: this.isUseSaveContext(),
                onOkCallback: dojo.hitch(this, "_handleAddNewItem")
            }
            this._newItemDialog = new NewItemPopup(args);
        },
        
        _handleRemoveClick: function(){
            var items = this._tableViewer.getSelectedItems();
            this._removeItems(items);
            if (items && items.length != 0) {
                this.setDirty();
            }
        },
        
        _handleNewItemLink: function(participantId){
			var self = this;
			var args = {
				title: Constants.getMessage("labelConfirmation"),
				message: Constants.getMessage("msgSaveRequiredToView", this._itemType, this._itemType),
				yesCallback: function(){
					self._newItemToOpen = self._getItemIndexByParticipantId(participantId);
            		self.getParentEditor().onSave();
					return true;          
				},
				noCallback: function() {
					return;
				}
			};
			var dialog = new YesNoDialog(args);
	        dialog.setSecondaryTitle(args.title);
	        dialog.open();
        },
		
		_openNewlySavedItem: function() {
            //if a new item was clicked, display it
            if (this._newItemToOpen) {
                var savedItem = this._items[this._newItemToOpen];
                this._newItemToOpen = null;
                if (savedItem != null) {
                    window.location = Utility.getArtifactURI(savedItem, null, false);
                }
            }			
		},
        
        _showItemsDialog: function(){
            var parentType = this.getArtifactType();
            var artifact = this.getArtifact();
            var parentItemId = (artifact) ? artifact.versionableItemId : null;
            var parentIdName = null;
            
            // seed the parent item id in our selection table.
            // This would filter out any items that are already 
            // belong to the parent.
            if(parentType == Constants.TYPE_CASE) {
                parentIdName = 'parentVTestCase';
            }
            else if(parentType == Constants.TYPE_PLAN) {
                parentIdName = 'parentVTestPlan';                
            }
            var params = {
                title: Constants.getMessage("labelAddItems", this._itemType),
                showQueryFilters: true,
                itemType: this._itemType,
                onOk: dojo.hitch(this, "_handleAddedItems"),
                onAddNew: function(){
                    alert(Constants.getMessage("labelNotSupported")); //self._appendNewItem();
                }
            };
            if(parentIdName && parentItemId) {
                params.parentItemIdArgName = parentIdName;
                params.parentVersionItemId = parentItemId;
            }
            else {
				params.parentItemIdArgName = parentIdName;
                params.parentVersionItemId = null;
                params.exclusionList = this._getAddExclusionList();
            }
            var itemListWidget = new SelectItemTableDialog(params);
            itemListWidget.setExclusionList(this._getAddExclusionList());
            itemListWidget.show();
        },
        
        _getAddExclusionList: function(){
            var excludedItems = this.getItems();
            var excludedItemIds = [];
			
			//exclude any plans that are already children of this plan 			
            for (var i = 0; i < excludedItems.length; i++) {
                var excludedItem = excludedItems[i];
                excludedItemIds.push(excludedItem.versionableItemId);
                excludedItemIds.push(excludedItem.itemId);
            }
			
			var plan = this.getArtifact();
			if (plan && plan.itemId) {
				excludedItemIds.push(plan.versionableItemId);
            	excludedItemIds.push(plan.itemId);
			}			
            return excludedItemIds;
        },
        
        _handleAddNewItem: function() {
			var newItem = this._newItemDialog.getItemForDisplay();
			if (this.isUseSaveContext()) {
				var newItemParticipant = this._newItemDialog.getItemSaveParticipant();
				newItem.participantId = newItemParticipant.getParticipantId();
				this.getSaveContext().addParticipant(newItemParticipant);
				this.getSaveContext().createReference(this.getParentEditor(), this._referencesAddedType, newItemParticipant);
			} else {
				newItem.uid = new Date().getTime();
				this._itemsAdded.push(newItem);
			}
            
            this._items.push(newItem);
            this.showTable();
            this.setDirty();
        },
        
        _handleAddedItems: function(addedItems){
            for (var i = 0; i < addedItems.length; i++) {
                var item = addedItems[i];
                var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
                if (this._containsItemId(this._itemsRemoved, itemId)) {
                    this._itemsRemoved = this._removeItemId(this._itemsRemoved, itemId);
                    this.getSaveContext().unremoveReference(this.getParentEditor(), this._referencesRemovedType, itemId);
                    this._items.push(item);
                }
                else 
                    if (!this._contains(item)) {
                        this._items.push(item);
                        this._itemsAdded.push(itemId);
                    }
                    else {
                        alert(item.name + " is already part of the list!");
                    }
                //append the item to the list
				if (this.useSaveContext) {
					this._addItem(item);
				}
            }
            this.showTable();
            this.setDirty();
        },
        
        _addItem: function(item){
            //create a SaveParticipant for the item
            var versionableItemId = item.versionableItemId ? item.versionableItemId : null;
            var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
            var saveParticipant = this.getSaveContext().findParticipant(itemId);
            if (saveParticipant == null) {
                saveParticipant = new SaveParticipant();
                var itemType = Utility.getModelType(item);
                saveParticipant.setDirty(false);
                saveParticipant.getItemId = function(){
                    return itemId;
                };
                saveParticipant.getVersionableItemId = function(){
                    versionableItemId
                };
                saveParticipant.getItemType = function(){
                    return itemType;
                };
                this.getSaveContext().addParticipant(saveParticipant);
            }
            var parent = this.getParentEditor();
            // treat it as a new reference if the parent is a copy
            if (parent != null && parent.isCopy != null && parent.isCopy.call(parent)) {
                this.getSaveContext().createReference(parent, this._referencesAddedType, saveParticipant);
            }
            else 
                if (this._containsItemId(this._itemsAdded, itemId) && parent != null) {
                    this.getSaveContext().createReference(parent, this._referencesAddedType, saveParticipant);
                }
        },
        
        _removeItems: function(items){
            if (!items || items.length == 0) {
                var msg = Constants.getMessage("msgDeleteTypeRequired", this._itemType);
                this.getParentEditor().setHeaderStatus(Constants.OPERATION_WARNING, Constants.HEADER_STATUS_WARNING, msg);
                return;
            }
            var confirmMsg = "";
            if (items.length > 1) {
                confirmMsg = Constants.getMessage("msgRemoveItem2", items.length, this._itemType);
            }
            else {
                confirmMsg = Constants.getMessage("msgRemoveItem", this._itemType, items[0].name);
            }
			var self = this;
			var	args = {
				message: confirmMsg,
				yesCallback: function(){
					for (var i = 0; i < (items ? items.length : 0); i++) {
                		self._removeItem(items[i]);
            		}
            		self.showTable();
					self.setDirty();      
					return true;          
				},
				noCallback: function() {
					return;
				}
			}
			var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
        	dialog.setSecondaryTitle(Constants.getMessage("labelConfirmation"));
			dialog.open();
        },
        
        _removeItem: function(item){
            var updatedItems = [];
            var items = this.getItems();
            for (var i = 0; i < items.length; i++) {
                //if this item is not new && its itemId doesn't match the item to be removed's itemId
                if (items[i].itemId && items[i].itemId != item.itemId) {
                    updatedItems.push(items[i]);
                    //if this item is new and its participantId doesn't match the item to be removed's participantId
                }
                else 
                    if (items[i].newItem && items[i].participantId != item.participantId) {
                        updatedItems.push(items[i]);
                    }
                    else {
                        //handle item removal from save context   
                        var saveContext = this.getSaveContext();
                        var itemId = items[i].versionableItemId ? items[i].versionableItemId : items[i].itemId;
                        if (items[i].newItem) {
                            //handle new item CREATED during this saveContext session
							saveContext.uncreateReference(this.getParentEditor(), this._referencesAddedType, items[i].participantId);
							saveContext.removeParticipant(saveContext.findParticipant(items[i].participantId));
                        }
                        else 
                            if (this._containsItemId(this._itemsAdded, itemId)) {
                                //handle existing items ADDED during this saveContext session
                                this._itemsAdded = this._removeItemId(this._itemsAdded, itemId);
								saveContext.uncreateReference(this.getParentEditor(), this._referencesAddedType, itemId);
								saveContext.removeParticipant(saveContext.findParticipant(itemId));
                            }
                            else {
                                //handle existing references, add them 'removed'
                                this._itemsRemoved.push(itemId);
								saveContext.removeReference(this.getParentEditor(), this._referencesRemovedType, itemId);
								//handle batch updated participant
								saveContext.removeParticipant(saveContext.findParticipant(itemId));
                            }
                    }
            }
            this.setItems(updatedItems);
        },
				
        _containsItemId: function(array, itemId){
            for (var i = 0; i < array.length; i++) {
                if (array[i] == itemId) {
                    return true;
                }
            }
            return false;
        },
        
        _removeItemId: function(array, itemId){
            var updatedArray = [];
            for (var i = 0; i < array.length; i++) {
                if (array[i] != itemId) {
                    updatedArray.push(array[i]);
                }
            }
            return updatedArray;
        },
        
        _contains: function(item){
            var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
            return this._getItemById(itemId) != null;
        },
        
        _getItemById: function(id){
            for (var i = 0; i < this._items.length; i++) {
                var item = this._items[i];
                var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
                if (itemId == id) {
                    return this._items[i];
                }
            }
            return null;
        },
        
        _getItemIndexByParticipantId: function(participantId){
            for (var i = 0; i < this._items.length; i++) {
                if (this._items[i].participantId == participantId) {
					return i;
                }
            }
            return null;
        },
        
        _getItemByName: function(name){
            for (var i = 0; i < this._items.length; i++) {
                if (this._items[i].name == name) {
                    return this._items[i];
                }
            }
            return null;
        },
        
		_displayAlertDialog: function(message, handler)
		{
			var args = {
                'message': message
            };
			if(handler){
				args.okCallback = handler;
			}
			var alertDialog = new AlertDialog(args);
			
            alertDialog.setSecondaryTitle("");
            alertDialog.open();
		},
                		
        
        _error: function(error) {
            console.error(this.declaredClass + ": An error occurred: " + error.message);
        }		
        
    })
    
})();
