/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestCasesSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.core.loader");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.mapper.TestCaseMapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.AbstractTableSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.properties.TestPlanTestCaseConfigSelector");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

//xojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIWizard");

(function(){ // module container   
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
	var TestCaseMapper = com.ibm.rqm.planning.web.client.internal.mapper.TestCaseMapper;
    var AbstractTableSection = com.ibm.rqm.planning.web.ui.internal.view.construction.extension.AbstractTableSection;
	var TestCaseClient = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
    var TestCaseTableView = com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
    var YesNoDialog = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
	
    // global save context
    var SaveParticipant = com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestCasesSection", [dijit._Widget, dijit._Templated, AbstractTableSection], {
        //templateString: "<div dojoAttachPoint=\"_testCasesSection\" class=\"com-ibm-rqm-planning-web-ui-internal-view-construction-extension-TestCasesSection\"></div>",
        
        constructor: function(args) {},
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
			
			//type specific setup
            this._itemType = Constants.TYPE_CASE;
			this._modelType = Constants.MODEL_TYPE_CASE;
            this._referencesAddedType = "testCasesAdded";
            this._referencesRemovedType = "testCasesRemoved";
			this.ewiSaveContextGroup = [];
        },
        
        initialize: function(){
            this.inherited('initialize', arguments);
        },
		
		//@Overridden from AbstractTableSection
		refreshContent: function() {
			var srh = new TestCaseMapper(this, "_refreshContentComplete", "_error");
			TestCaseClient.getTestPlanTestCases(srh, {
				versionedTestPlanItemId: this.getArtifact().versionableItemId
			});
		},
        
		//@Overridden from AbstractTableSection
        createTable: function() {
			//create and return the test cases table                
            var tableWidget = new TestCaseTableView({
				parent: this,
                editable: this.isEditable(),
				printable: this.isPrintView(),
                enableRowSelection: true,
				enableRowEditable: this.isRolePermitsEdit(),
                testCases: this.getItems(),
                actions: this._getTableActions(),
                contentIsVirtual: false,
				refreshCallback: dojo.hitch(this, "refreshContent"),
                newItemLinkCallback: dojo.hitch(this, "_handleNewItemLink"),
                headerActionInvoked: dojo.hitch(this, "_handleBatchUpdateClick")
            });
			
			if (tableWidget.deleteActionNode){
				tableWidget.deleteActionNode.setDisabled(true);
			}
			if (tableWidget.changeSuspectActionNode){
				tableWidget.changeSuspectActionNode.setDisabled(true);
			}
			if (tableWidget.genEWIActionNode){
				tableWidget.genEWIActionNode.setDisabled(true);
			}
			if (tableWidget.executeActionNode){
				tableWidget.executeActionNode.setDisabled(true);
			}
			if (tableWidget.moveUpNode){
				tableWidget.moveUpNode.setDisabled(true);
			}
			if (tableWidget.moveDownNode){
				tableWidget.moveDownNode.setDisabled(true);
			}			
			return tableWidget;
        },
        
        //actions to be displayed in the table header
        _getTableActions: function(){
            var actions = [];
            
            if (this.isRolePermitsEdit()) {
				 actions.push({
                    actionId: "moveup",
                    label: Constants.getMessage("labelMoveUp"),
                    actionHandler: dojo.hitch(this, "_handleMoveUp"),
                    iconClass: "move-up-image",
                    disabledOnReadOnly: true,
                    disabled: true
                });
                actions.push({
                    actionId: "movedown",
                    label: Constants.getMessage("labelMoveDown"),
                    actionHandler: dojo.hitch(this, "_handleMoveDown"),
                    iconClass: "move-down-image",
                    disabledOnReadOnly: true,
                    disabled: true
                });				
                actions.push({
                    label: Constants.getMessage("labelAddExistingTestCases"),
                    actionHandler: dojo.hitch(this, "_handleAddClick"),
                    iconClass: "add-icon-image",
                    disabledOnReadOnly: true
                });
            }
            var tc_type = com.ibm.asq.common.web.client.internal.PermissionsClient.OP_TEST_CASE;
            var tc_perm = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(tc_type);
            if (!tc_perm || tc_perm.create) {
                actions.push({
                    label: Constants.getMessage("labelAddNewTestCase"),
                    actionHandler: dojo.hitch(this, "_handleAddNewClick"),
                    iconClass: "add-new-testcase-image",
                    disabledOnReadOnly: true
                });
            }
            if (this.isRolePermitsEdit()) {
                actions.push({
                    label: Constants.getMessage("labelRemoveTestCases"),
                    actionHandler: dojo.hitch(this, "_handleRemoveClick"),
                    iconClass: "remove-enabled-icon-image",
                    disabledOnReadOnly: true
                });
            }
			var tc_exec_permission = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(
											com.ibm.asq.common.web.client.internal.PermissionsClient.OP_TEST_CASE_EXE);
			var hasExePermission = !tc_exec_permission ? true : tc_exec_permission 
														   		&& tc_exec_permission[com.ibm.asq.common.web.client.internal.PermissionsClient.ACTION_EXECUTE];
			if (hasExePermission) {
                actions.push({
                    label: Constants.getMessage("labelExeTest"),
                    actionHandler: dojo.hitch(this, "_onExecuteActionClick"),
                    iconClass: "execute-icon-image",
                    disabledOnReadOnly: true
                });
            }
            var ewi_type = com.ibm.asq.common.web.client.internal.PermissionsClient.OP_EXECUTION_EWI;
            var ewi_perm = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(ewi_type);
            if (!ewi_perm || ewi_perm.create) {
                actions.push({
                    label: Constants.getMessage("labelGenEWIs"),
                    actionHandler: dojo.hitch(this, "_handleGenerateEWIClick"),
                    iconClass: "generate-ewis-icon",
                    disabledOnReadOnly: true
                });
            }
            return actions;
        },
		
        //@Overridden from AbstractArtifactEditorSection
        getAttributes: function(){
			//pass in the order of the associated test cases
			var cases = this.getItems();
			if (cases != null && cases.length > 0) {
				var orderedIds = [];
				var caseLen = cases.length;
				for (var i = 0; i < caseLen; i++) {
					var vItemId = cases[i].versionableItemId;
					if (vItemId) orderedIds.push(vItemId);
				}
				return {
					testCasesOrder: orderedIds.join(",")
				}
			} // END DEBUG HACK			
            return null;
        },		
        
        //@Overridden from AbstractTableSection
        saveFinished: function(){
            this.inherited('saveFinished', arguments);
			this._createGeneratedEwis();
        },
		
		_handleMoveUp:function(){
			var selectedItems = this._tableViewer.getSelectedItems();
			selectedItems = selectedItems.sort(function(case1,case2){
				return case1.rowId-case2.rowId;  //if case1 rowId is greater than case2 rowId return positive 
			});
			var content = this._tableViewer.getTableContent();
			var selectedItemsLength = selectedItems.length;
			for(var i=0;i<selectedItemsLength;i++){
				var rowId = selectedItems[i].rowId;
				if (rowId != 0) {
					var rowToSwap = dojo.clone(content[rowId - 1]);
					content[rowId - 1] = content[rowId];
					content[rowId - 1].rowId = rowId -1;
					content[rowId] = rowToSwap;
					content[rowId].rowId = rowId;
					this._tableViewer.setRowChecked(content[rowId-1],true);
					this._tableViewer.setRowChecked(content[rowId],false);
				} 
			}
			this.setDirty(true);
			this.setItems(content);
			this._tableViewer.tableViewer.content = content;
			this._tableViewer.tableViewer.refresh();
		},
		
		_handleMoveDown:function(){
			var selectedItems = this._tableViewer.getSelectedItems();
			selectedItems = selectedItems.sort(function(case1,case2){
				return case1.rowId-case2.rowId;
			});
			var content = this._tableViewer.getTableContent();
			var selectedItemsLength = selectedItems.length;
			for(var i=selectedItemsLength;i>0;i--){
				var rowId = selectedItems[i-1].rowId; 
				if (rowId != content.length-1) {
					var rowToSwap = dojo.clone(content[rowId + 1]);
					content[rowId + 1] = content[rowId];
					content[rowId + 1].rowId = rowId + 1;
					content[rowId] = rowToSwap;
					content[rowId].rowId = rowId;
					this._tableViewer.setRowChecked(content[rowId+1],true);
					this._tableViewer.setRowChecked(content[rowId],false);
				}
			}
			this.setDirty(true);
			this.setItems(content);
			this._tableViewer.tableViewer.content = content;
			this._tableViewer.tableViewer.refresh();
		},		
		
		_handleBatchUpdateClick: function(columnIndex, actionId) {
			var args = {
				testcases: this._tableViewer.getSelectedItems(),
				categoryTypes: this._tableViewer.categoryTypes, 
				actionId: actionId,
                onOkCallback: dojo.hitch(this, "_handleBatchUpdateItem")
            }
			this._updatedDialog = new com.ibm.rqm.planning.web.ui.internal.view.common.properties.TestPlanTestCaseConfigSelector();
			this._updatedDialog.initialize(args);
		},
        
		_onExecuteActionClick: function(){
			var selectedItems = this._tableViewer.getSelectedItems();
			var selectedCase = selectedItems[0];
			if(this.isSectionDirty() || selectedCase.id == ""){
				var self = this;
				var args = {
					title: Constants.getMessage("labelConfirmation"),
					message: Constants.getMessage("msgSaveRequiredToRunTestCase", Constants.TYPE_PLAN),
					yesCallback: function(){
	            		self.getParentEditor().onSave();
						return;
					},
					noCallback: function() {
						return;
					}
				};
				var dialog = new YesNoDialog(args);
		        dialog.setSecondaryTitle(args.title);
		        dialog.open();
			}else
				this._tableViewer._onExecuteActionClick();
		},
		
        _handleGenerateEWIClick: function(){
            var selected = this._tableViewer.getSelectedItems();
            if (!selected || selected.length == 0) {
 				this._displayAlertDialog(Constants.getMessage("msgGenEWITestCaseWarning"))
                //this.getParentEditor().setHeaderStatus(Constants.OPERATION_WARNING, Constants.HEADER_STATUS_WARNING, Constants.getMessage("msgGenEWITestCaseWarning"));
                return;
            }
            //HACK until we get contributed actions enabled  
            var wizardClassName = "com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIWizard";
            jazz.core.loader.load_async(wizardClassName, dojo.hitch(this, "_showEWIWizard", wizardClassName, selected));
            //            this._showEWIWizard(selected);     
        },
        
        _showEWIWizard: function(wizardClassName, testCases) {
			if (dojo.isFunction(this.getArtifact) && this.getArtifact() && this.getArtifact().id && this.getParentEditor() && !this.getParentEditor().isDirty()) {
	            var self = this;
	            var args = {
	                title: Constants.getMessage("labelGenEWIs"),
	                parentController: self,
	                callback: {context: self, method: function(arguments){
						var jsClassName = "com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWISaveContext";
						jazz.core.loader.load_async(jsClassName, dojo.hitch(this, function(jsClassName, args){
							var constructor = dojo.getObject(jsClassName);
							var saveContext = new constructor(args);
							this.ewiSaveContextGroup.push(saveContext);
							this.setDirty();
						}, jsClassName, arguments));
					}},
	                testCases: testCases,
	                parentTestPlan: this.getArtifact()
	            }
	            var constructor = dojo.getObject(wizardClassName);
	            this._ewiWizard = new constructor(args);
			} else {
				var msg = Constants.getMessage("msgSaveRequiredToGenEWI", this.getArtifactType());
				this._displayAlertDialog(msg);
            }
        },
		
		_createGeneratedEwis: function(){
			if(this.ewiSaveContextGroup) {
				var ewiSaveContextGroup = this.ewiSaveContextGroup;
				var length = ewiSaveContextGroup.length;
				var counter = 0;
				if (length){
					for(var i = 0; i < length; i++){
						ewiSaveContextGroup[i].commit({
							callback: dojo.hitch(this, function(){
								counter++;
								if(counter == length) {
									this.ewiSaveContextGroup = [];
								}
							})
						});
					}
				} 
			}
		},
		
		_handleBatchUpdateItem: function(updatesData) {
			var items = updatesData ? updatesData.testCases : [];
			for(var i = 0; i < (items ? items.length : 0); i++) {
				var item = items[i];
				this._updateItem(item, updatesData.weight, updatesData.state);
			}
            if (items && items.length != 0) {
				this.setDirty();
            }
//			this.showTestCaseTable();
			//only change table content, won't change checkboxs' state.
			this._tableViewer.tableViewer.content = this.getItems();
			this._tableViewer.tableViewer.refresh();
		},
        
		_updateItem: function(item, weight,state) {
			var categories = item.categories;
			if(item.newItem) {
				//if this item is new
				var participant = this.getSaveContext().findParticipant(item.participantId);
				var oldAttribute = participant.getAttributes();
				participant.getAttributes = function(){
            		var attributes = {
                		newItem: true,
                		name: oldAttribute.name,
                		description: oldAttribute.description,                               
                		state: state?state:oldAttribute.state,
						newTestCase: true,
						templateItemId: oldAttribute.templateItemId,
						templateStateId: oldAttribute.templateStateId,
						//changed
						categories: categories.join(),
						weight: weight ? weight : oldAttribute.weight
            		};          
					if (oldAttribute.workItemsJsonString) {
						attributes.workItemsJsonString = oldAttribute.workItemsJsonString;
					}
					return attributes;
            	}; 
			}
			else {
				var versionableItemId = item.versionableItemId ? item.versionableItemId : null;
            	var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
				var stateId = item.stateId;
				var itemType = Utility.getModelType(item);
				//existing item, then add saveParticipant
				var saveParticipant = this.getSaveContext().findParticipant(itemId);
				if(!saveParticipant) {
					//handle existing test case
					saveParticipant = new SaveParticipant();
					this.getSaveContext().addParticipant(saveParticipant);
				}
				saveParticipant.setDirty(true);
				saveParticipant.getItemId = function(){
                 	return itemId;
               	};
               	saveParticipant.getVersionableItemId = function(){
                	return versionableItemId;
               	};
               	saveParticipant.getItemType = function(){
                	return itemType;
               	};
				saveParticipant.getStateId = function(){
					return stateId;
				};
				saveParticipant.getAttributes = function(){
					var attributes = {
						itemId: itemId,
						//changed
						weight: weight ? weight : item.weight,
						state : state? state : item.state,
						categories: categories ? categories.join() : ""
					};
					return attributes;
				};
				saveParticipant.saveFinished = function() {
					saveParticipant.setDirty(false);
				};
			}
				
			//refresh items
			var oldItems = this.getItems();
			var updatedItems = [];
			for (var j = 0; j < oldItems.length; j++) {
				var tmpItem = oldItems[j];
				if ((tmpItem.newItem && tmpItem.participantId == item.participantId) ||
					(tmpItem.itemId && item.itemId && tmpItem.itemId == item.itemId)) {
						if(weight)
							tmpItem.weight = weight;
						if(state)
							tmpItem.state = state;
						tmpItem.categories = categories;
                   }
				updatedItems.push(tmpItem);
			}
			this.setItems(updatedItems);
		},
		
		addTestCases: function(cases){
			this._handleAddedItems(cases);
		},
		
		containsTestCase: function(testCase) {
			//look up by both versionableItemId and itemId 
			if (testCase == null) return false;
			if (this.containsItem(testCase.versionableItemId, "versionableItemId")) { //$NON-NLS-1$
				return true;
			} else if (this.containsItem(testCase.itemId, "itemId")) { //$NON-NLS-1$
				return true;
			}			
			return false;
		},		
	
        validateStateTransition: function(initialState, action, endState, editable, callback){
            //The only transiton we are interested in is to approved
            if (action == Constants.STATETRANSITION_ACTION_APPROVE && EditorUtil.applyTransactionConstraint(Constants.STATETRANSITION_PREFERENCE_ALLCHILDCASESAPPROVED)) {
                // We have not been initialized yet. Need to fetch the test cases
                // to do the validtion.
                if (!this._initialized && callback) {
                    this.validateCB = callback;
                    this.validateEndState = endState;
                    var fetched = this.refreshTestCases("_validateFetchSuccess", "_validateFetchError");
                    
                    // not a valid artifact, just return null
                    if(fetched == false) {
                        return null;
                    }
                    // return a delay flag.
                    return Constants.EDITORSECTION_STATE_VALIDATION_FETCHING;
                }
                // we have loaded the data, just do the validation.    
                return this._validateTestCasesApproval(endState);
            }
            //If we got here than nothing offensive happened so return null which indicates that it is ok to transition
            return null;
            
        },

        _validateFetchSuccess: function(testcases){
            this._items = testcases;
            var isValid = this._validateTestCasesApproval(this.validateEndState);
            
            // return the validation result and cleanup
            this.validateCB(isValid);
            delete this._items;
            delete this.validateCB;
            delete this.validateEndState;
        },
        
        _validateFetchError: function(errObj){
            var msg = (errObj && errObj.message) ? errObj.essage : null;
            this.validateCB(msg);
            delete this.validateCB;
            delete this.validateEndState;
        },
                		
        _validateTestCasesApproval: function(endState) {
            //Now loop over all the items (test cases) if any to ensure that they are all approved
            if (this._items && this._items.length > 0) {
                for (var i = 0; i < this._items.length; i++) {
                    if (this._items[i].state != Constants.STATETRANSITION_STATE_APPROVED) //Not approved
                    {
                        return Constants.getMessage("msgCannotTransitionNotAllTestCasesApproved", EditorUtil.getStateNameFromID(endState));
                    }
                }
            }
            return null;
        },
        
        _error: function(error) {
            console.error(this.declaredClass + ": Error occurred: " + error.message);
        }		
        
    })
    
})();
