/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading.. com.ibm.rqm.planning.web.ui.internal.view.construction.properties._workItem.QMWorkItemEditor");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.properties._workItem.QMWorkItemEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){ // module container
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var WorkItemUtil = com.ibm.team.workitem.web.client.util;
    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    var WorkItemClient = com.ibm.team.workitem.web.client.internal.WorkItemClient;
    var DEFAULT_TAB = "overview";
    
    var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
    var isGuest = com.ibm.team.repository.web.client.session.isGuest;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.properties._workItem.QMWorkItemEditor", [com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor], {
    
        // internal method to attach all editor parts to the tab container.
        // This allows the parent dialog destruction to find all widgets
        // created by the editor.
        preDestroyWidget: function() {
            //console.debug("QMWorkItemEditor - preDestroyWidget");
            var editorParts = this._editorParts;
            var contentNode = this._currentTabContainer;
            
            // make the container invisible.
            dojo.style(contentNode, "visibility", "hidden");
            dojox.data.dom.removeChildren(contentNode);
            
            for (var i in editorParts) {
                var part = editorParts[i];
                if(part.domNode)
                    contentNode.appendChild(part.domNode);
            }
        },

        setInitialValues: function(values) {
            this.initValues = values;    
        },
        
        setCallback: function(/* Function */callbackObj){
            this.callback = callbackObj;
        },
        
        _error: function(errorObj){
            alert("Error: (message: '" + errorObj.message + "') from DefectView#_error");
        },
        
        ////////////////////////////////////////////
        //OVERRIDDEN METHODS FROM WORKITEM EDITOR
        ////////////////////////////////////////////
        
        //@override from WorkItemEditor
        viewWorkItem: function(id, initialTab){
            checkNotNull(id);
            
            var self = this;
            var serviceObject = {
                viewWorkItemHandler: function(workItem){
                    if (workItem) {
                        self.setWorkItem(workItem, "overview");
                        //self.addDefect();
                        if (self.callback) {
                            self.callback(self._wiEditor._workItem);
                        }
                    } else {
                        alert(Constants.getMessage("msgWorkItemNotFound", id));
                    }
                },
                error: function(error){
                    self._error(error, "Error viewing work item");
                }
            };
            
            var responseHandler = new ServiceResponseHandler(serviceObject, "viewWorkItemHandler", "error");
            WorkItemClient.getWorkItemDTO(responseHandler, {
                id: id
            });
        },

        // Loads values specified in the parameter into the work item's form.
        // 
        _mixinWorkItemValues: function(values) {
            var workitem = this.workingCopy.object;
            
            if(values && dojo.isObject(values)) {
                for(var attrId in values) {
                    var value = values[attrId];
                    
                    // make sure that the workitem object has our custom
                    // attributes before initializing the pre-fill value.
                    if(attrId && (attrId == 'linkTypes' || workitem.attributes[attrId]) && value) {
                        var path = "attributes";
                        var parms;
                        if (attrId == 'linkTypes') {
                            parms = {
                                'caller': this,
                                'path': ['linkTypes'],
                                'savedAttributePath': ['linkTypes'],
                                'value': value
                            };
                        }
                        // need to set the value into the working copy
                        // object. This would mark the attribute as changed
                        // and would properly trigger the saving of the
                        // attribute.
                        else {
                            // if the value has no text, don't set the value
                            // in the work item.
                            if(value.length == 0)
                                continue;
                                
                            parms = {
                                'caller': this,
                                'path': ["attributes", attrId, "content"],
                                'savedAttributePath': ["attributes", attrId, "content"],
                                'attributeId': attrId,
                                'value': value
                            };
                        }
                        this.workingCopy.setValue(parms);
                    }
                }
            }
        },
        
        // @override from WorkItemEditor - to initialize a new workitem's values.
        //
        // The workitem editor now takes only an id for a workitem. It does the
        // fetching/caching of the workitem's content internally. This internal method
        // is called when the work item's data copy has been fetched/initialized.
        //
        _workingCopyInitialized: function(parms) {
            // if we have initial values set and this is a new defect,
            // load the values into the working copy of the workitem.
            if(this.initValues && this.workingCopy.isNewWorkItem()) {
                this._mixinWorkItemValues(this.initValues);
            }
            this.inherited("_workingCopyInitialized", arguments);
        },
        
        //@override from WorkItemEditor
        _getThisWorkingCopy: function(id, createIfNeeded) {
            if (!this.workingCopy) {
                // new item. don't want to reuse the old 'new' cached object.
                if(id == "-1") {
                    com.ibm.team.workitem.web.cache.internal.Cache.removeItem(id);
                }
                // First see if there is already a work item in the cache
                var workingCopy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id);
                if (workingCopy) {
                    this.workingCopy = workingCopy;
                } else {
                    if(createIfNeeded) {
                        this.workingCopy = this._createThisWorkingCopy(id);
                    } else {
                        return null;
                    }
                }
                this.workingCopy.incrementUseCount();

            }
            return this.workingCopy;
        },
        
        //@override from WorkItemEditor
        _saveCallback: function(savedWorkItemId, op){
            if (op) {
                this.parentController.finishOperation(op);
            }
            if (this.callback) {
                this.callback(savedWorkItemId);
            }
            else {
                this.parentController.setIsDirty(this, false);
                if (this.isNewWorkItem()) {
                    this.parentController.removeEditorTab(-1);
                    this.viewWorkItem(savedWorkItemId, this._getCurrentTab());
                } else {
                    this._editorParts.overview.clearNewCommentText();
                    this.refresh(/*skipConfirmation*/true);
                }
                this._isDirty = false;
            }
        },

        //@override from WorkItemEditor
        setWorkItem: function(workItem, tab){
            this.inherited("setWorkItem", arguments, [workItem, tab]);
        },
        
		//@override from WorkItemEditor. Update dialog's position when loading completely. 
		_setEditableProperties: function(workItem, editProps, presentationProps, initialTab,  retainedState, isTypeChange, newTypeId) {
			this.inherited("_setEditableProperties", arguments, [workItem, editProps, presentationProps, initialTab,  retainedState, isTypeChange, newTypeId]);
			if(this.parentController && this.parentController._workItemPageView && this.parentController._workItemPageView.dialog) {
				this.parentController._workItemPageView.dialog.updatePosition();
			}
		},
		
        //@override from WorkItemEditor
        _createTabPart: function(logicalName, widget, tabLinkLabel){
            console.debug("workitemEditor _createTabPart: " + tabLinkLabel)
            this.inherited("_createTabPart", arguments, [logicalName, widget, tabLinkLabel]);
            if (this._tabLinkContainer) {
                for (var i = 0; i < this._tabLinkContainer.childNodes.length; i++) {
                    if (this._tabLinkContainer.childNodes[i] && this._tabLinkContainer.childNodes[i].__jazzTabName == logicalName) {
                        this.connect(this._tabLinkContainer.childNodes[i], "onclick", "_handleTabClick");
                    }
                }
            }
        },
		
        _handleTabClick: function(e)
        {
			if (e.stopPropagation) {
				e.stopPropagation();
            	e.preventDefault();
			}
			e.cancelBubble = true;
			e.returnValue = false;
            this.showTab(e.currentTarget.__jazzTabName);
			return false;
        },
		
		//In pop-up Work Item Editor, there is no "recent viewed vork item", so needn't it
		_sendChangedEvent: function(reason) {
//			dojo.publish("workitem/editorChanged", [this._workItem]);
		}
        		
        /**
         * Save the changes to work item currently displayed in the editor.
         
        save: function() {
            // this gets run in the context of work item application controller
            var op= this.parentController.createOperation("Saving Work Item", "saveWorkitem");
            this.parentController.startOperation(op);
            this._editorParts.header.markBusy("Saving");
            this._editorParts.header.disableSave(true);

            
            var self= this;
            var serviceObj= {
                success: function(workItem) {
                    self._clearRetainedState();
                    self._saveCallback(workItem.attributes.id.id, op);
                },
                failure: function(errorObj) {
                    self._saveErrorCallback(errorObj, op);
                    self._editorParts.header.markFullyLoaded();
                }
            };
            var responseHandler= new ServiceResponseHandler(
                serviceObj,
                "success",
                "failure");

            var appArgs = this._getSaveAppArgs();
            
            WorkItemClient.postWorkItem(responseHandler, appArgs);
        }
		*/
    });
    
})();
