/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.execution.ExecutionPoints");

dojo.require("dojo.number");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.parser");
dojo.require("dijit.form.Slider");
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){ // module container
    
	var Constants 		= com.ibm.rqm.planning.web.client.internal.Constants;
	var HoverPopup 		= jazz.ui.HoverPopup;
	var TooltipShowTime = 3000;
    dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.execution.ExecutionPoints", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.execution.web", "ui/internal/view/templates/execution/ExecutionPoints.html"),
        
		labelPass: Constants.getMessage("labelPass"),
		labelFail: Constants.getMessage("labelFail"),
		labelBlocked: Constants.getMessage("labelBlocked"),
		labelInconclusive: Constants.getMessage("labelInconclusive"),
		labelAttempted: Constants.getMessage("labelAttempted"),
		
        constructor: function(){
            this.maxPoints = 0;
            this.pointsPassed = 0;
            this.pointsFailed = 0;
			this.pointsBlocked = 0;
			this.pointsInconclusive = 0;
            this.pointsAttempted = 0;
            this.isDisabled = false;
        },
        
        postCreate: function(){
            if (!this.maxPoints) {
				console.warn("maxPoints is required! the EWI under execution may not have a weight associated.");
                this.maxPoints = 0;
			}
            this.initialize();
            if (!this.validate())
                throw new Error("the total points exceeds the specified maximum!");
        },
        
        initialize: function(){
            this._title.innerHTML = Constants.getMessage("labelWeightDistribution") + " (" + Constants.getMessage("labelPossible") + this.maxPoints + ")";
            this._passed.innerHTML = this.pointsPassed;
			this._passedPointsSlider = this._getSlider(this.pointsPassed);
            if(!this.isDisabled)
			    this._passedPointsSlider.onChange = dojo.hitch(this,this.onChange,this._passed,this._passedPointsSlider,true);
            else
                this._passedPointsSlider.disabled = true;
			this._passedSlider.appendChild(this._passedPointsSlider.domNode);
            
			this._failed.innerHTML = this.pointsFailed;
			this._failedPointsSlider = this._getSlider(this.pointsFailed);
            if(!this.isDisabled)
			    this._failedPointsSlider.onChange = dojo.hitch(this,this.onChange,this._failed,this._failedPointsSlider,false);
            else
                this._failedPointsSlider.disabled = true;
			this._failedSlider.appendChild(this._failedPointsSlider.domNode);
			
			this._blocked.innerHTML = this.pointsBlocked;
			this._blockedPointsSlider = this._getSlider(this.pointsBlocked);
            if(!this.isDisabled)
			    this._blockedPointsSlider.onChange = dojo.hitch(this,this.onBlockedChange);
            else
                this._blockedPointsSlider.disabled = true;
			this._blockedSlider.appendChild(this._blockedPointsSlider.domNode);
			
			this._inconclusive.innerHTML = this.pointsInconclusive;
			this._inconclusivePointsSlider = this._getSlider(this.pointsInconclusive);
            if(!this.isDisabled)
			    this._inconclusivePointsSlider.onChange = dojo.hitch(this,this.onChange,this._inconclusive,this._inconclusivePointsSlider,false);
            else
                this._inconclusivePointsSlider.disabled = true;
			this._inconclusiveSlider.appendChild(this._inconclusivePointsSlider.domNode);
            
			var total = this.getTotal();
			this.pointsAttempted = total > this.pointsAttempted ? total:this.pointsAttempted;
			this._attempted.innerHTML = this.pointsAttempted;
			this._attemptedPointsSlider = this._getSlider(this.pointsAttempted);
            if(!this.isDisabled)
			    this._attemptedPointsSlider.onChange = dojo.hitch(this,this.onAttemptedChange);
            else
                this._attemptedPointsSlider.disabled = true;
			this._attemptedSlider.appendChild(this._attemptedPointsSlider.domNode);
        },
		
        _getSlider:function(value){
			var slider = new ExecutionPoints.Slider({
				minimum:0,
				maximum:this.maxPoints,
				discreteValues:this.maxPoints+1,
				intermediateChanges:false,
				showButtons:true,
				value:value,
                pageIncrement: 10,
                slideDuration: dojo.isIE ? 0 : 500 // no animation for IE
				});
			slider.startup();
			return slider;
		},
        
		validate: function(){
            if (this.getTotal() > this.maxPoints)
                return false;
            return true;
        },
        
        getTotal: function(){
            var passed = dojo.number.parse(this._passed.innerHTML);
            var failed = dojo.number.parse(this._failed.innerHTML);
			var blocked = dojo.number.parse(this._blocked.innerHTML);
			var inconclusive = dojo.number.parse(this._inconclusive.innerHTML);

            this.pointsPassed = passed != NaN ? passed : this.pointsPassed;
            this.pointsFailed = failed != NaN ? failed : this.pointsFailed;
			this.pointsBlocked = blocked != NaN ? blocked : this.pointsBlocked;
            this.pointsInconclusive = inconclusive != NaN ? inconclusive : this.pointsInconclusive;
			var total = this.pointsPassed + this.pointsFailed + this.pointsInconclusive;
            return total;
        },
        
        onChange: function(){
            /*
             * arguments
             * 0 -- points label
             * 1 -- slider
             * 2 -- compare Blocked points
             * 3 -- changed slider value
             */
			arguments[1]._setDisabledAttr(true);
			var value = arguments[3];
			arguments[0].innerHTML = value ;
			var total = this.getTotal();
			if(this._hideTimer){
				clearTimeout(this._hideTimer);
				delete this._hideTimer;
				this.tooltip.close();
			}
			if (!this.validate()){
				this._showTooltip(arguments[0]);
				value = this.maxPoints - (total - value);
				value = (value > 0)? value : 0;
				setTimeout(dojo.hitch(this,function(slider,value){
					slider._setValueAttr(value,false);
					},arguments[1],value),100
				);
				arguments[0].innerHTML = value;
				total = this.maxPoints;
			}
			if(this.callback)
				this.callback();
			if(arguments[2]){
				var blockedtotal = this.pointsBlocked + value;
				if(blockedtotal > this.maxPoints){
					this._showBlockedTooltip(arguments[0]);
					value = this.maxPoints - this.pointsBlocked;
					setTimeout(dojo.hitch(this,function(slider,value){
							slider._setValueAttr(value,false);
						},arguments[1],value),100);
					arguments[0].innerHTML = value;
					arguments[1]._setDisabledAttr(false);
					return;
				}
			}
			var passFailTotal = this.getTotal();
//			this.pointsAttempted = passFailTotal > this.pointsAttempted ? passFailTotal : this.pointsAttempted;
            this.pointsAttempted = passFailTotal; // adjust attempted point, not just increase
			setTimeout(dojo.hitch(this,function(slider,value){
				slider._setValueAttr(value,false);
			},this._attemptedPointsSlider,this.pointsAttempted),100);
			this._attempted.innerHTML = this.pointsAttempted;
			arguments[1]._setDisabledAttr(false);
        },

		_showTooltip:function(targetNode){
			if(!this.tooltip){	
				var _tooltipSpan = document.createElement("span");
				this.tooltip = new HoverPopup({
						isProgrammatic:true,
						html:Constants.getMessage("msgExceedExePoints")
					},
				 	_tooltipSpan);
			}
			this.tooltip.open(this._errorNode);
			this._hideTimer = setTimeout(dojo.hitch(this,function(){
					this.tooltip.close();
					delete this._hideTimer;
				}), TooltipShowTime); 			
		},
		_showBlockedTooltip:function(targetNode){
			if(!this.blockedTooltip){
				var _blockededTooltipSpan = document.createElement("span");
				this.blockedTooltip = new HoverPopup (
					{isProgrammatic:true,html:Constants.getMessage("msgExceedBlockedPoints")},
				 	_blockededTooltipSpan);
			}
			this.blockedTooltip.open(this._errorNode);
			this._hideBlockedTimer = setTimeout(dojo.hitch(this,function(){
					this.blockedTooltip.close();
					delete this._hideBlockedTimer;
			}), TooltipShowTime);
		},

		onBlockedChange:function(){
			this._blockedPointsSlider._setDisabledAttr(true);
			var value = arguments[0];
			var total = this.maxPoints - this.pointsPassed;
			if(this._hideBlockedTimer){
				clearTimeout(this._hideBlockedTimer);
				delete this._hideBlockedTimer;
				this.blockedTooltip.close();
			}
			if(value >total){
				this._showBlockedTooltip(this._blocked); 				
				value = total;
				setTimeout(dojo.hitch(this,function(slider,value){
						slider._setValueAttr(value,false);
					},this._blockedPointsSlider,value),100);
			}
			if(this.callback)
				this.callback();
			this.pointsBlocked = value;
			this._blocked.innerHTML = this.pointsBlocked;
			this._blockedPointsSlider._setDisabledAttr(false);
		},
		onAttemptedChange:function(){
			this._attemptedPointsSlider._setDisabledAttr(true);
			var value = arguments[0];
			var total = this.getTotal();
			if(this._hideAttemptedTimer){
				clearTimeout(this._hideAttemptedTimer);
				delete this._hideAttemptedTimer;
				this.attemptedTooltip.close();
			}
			if(value <total){
				if(!this.attemptedTooltip){
					var _attemptedTooltipSpan = document.createElement("span");
					this.attemptedTooltip = new HoverPopup (
						{isProgrammatic:true,html:Constants.getMessage("msgLessThanTotalPoints")},
					 	_attemptedTooltipSpan);
				}
				this.attemptedTooltip.open(this._errorNode);
				this._hideAttemptedTimer = setTimeout(dojo.hitch(this,function(){
						this.attemptedTooltip.close();
						delete this._hideAttemptedTimer;
					}), TooltipShowTime); 				
				value = total;
				setTimeout(dojo.hitch(this,function(slider,value){
						slider._setValueAttr(value,false);
					},this._attemptedPointsSlider,value),100);
			}
			if(this.callback)
				this.callback();
			this.pointsAttempted = value;
			this._attempted.innerHTML = this.pointsAttempted;
			this._attemptedPointsSlider._setDisabledAttr(false);
		},
        getAttachPointName: function(node){
            if (node != null && node.attributes != null && node.attributes.length > 0) {
				var len = node.attributes.length;
                for (var i = 0; i < len; i++) {
                    if (node.attributes[i].nodeName == "dojoAttachPoint") {
                        return node.attributes[i].nodeValue
                    }
                }
            }
            return null;
        },
        
        getPoints: function(){
            if (!this.validate())
                return false;
            return {
                pointsPassed: this.pointsPassed,
                pointsFailed: this.pointsFailed,
				pointsBlocked: this.pointsBlocked,
				pointsInconclusive: this.pointsInconclusive,
                pointsAttempted: this.pointsAttempted
            };
        },
		uninitialize:function(){
			if(this.tooltip)
				this.tooltip.destroyRecursive();
			this.tooltip = null;
			if(this._hideAttemptedTimer){
				clearTimeout(this._hideAttemptedTimer);
				delete this._hideAttemptedTimer;
			}
			if(this._hideBlockedTimer){
				clearTimeout(this._hideBlockedTimer);
				delete this._hideBlockedTimer;
			}
			if (this._hideTimer) {
				clearTimeout(this._hideTimer);
				delete this._hideTimer;
			}
			if(this.blockedTooltip){
				this.blockedTooltip.destroyRecursive();
			}
			this.blockedTooltip = null;
			if(this.attemptedTooltip){
				this.attemptedTooltip.destroyRecursive();
			}
			this.attemptedTooltip = null;
		}
    });
    dojo.declare("ExecutionPoints.Slider", 
             dijit.form.HorizontalSlider,
	{
		templateString:"<div class=\"com-ibm-rqm-test-execution-points\"><table class=\"dijit dijitReset dijitSlider\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\" dojoAttachEvent=\"onkeypress:_onKeyPress\"\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td dojoAttachPoint=\"containerNode,topDecoration\" class=\"dijitReset\" style=\"text-align:center;width:100%;\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderDecrementIconH\" tabIndex=\"-1\" style=\"display:none\" dojoAttachPoint=\"decrementButton\"><span class=\"dijitSliderButtonInner\">-</span></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderLeftBumper dijitSliderLeftBumper\" dojoAttachEvent=\"onclick:_onClkDecBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><input dojoAttachPoint=\"valueNode\" type=\"hidden\" name=\"${name}\"\n\t\t\t/><div class=\"dijitReset dijitSliderBarContainerH\" waiRole=\"presentation\" dojoAttachPoint=\"sliderBarContainer\"\n\t\t\t\t><div waiRole=\"presentation\" dojoAttachPoint=\"progressBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderProgressBar dijitSliderProgressBarH\" dojoAttachEvent=\"onclick:_onBarClick\"\n\t\t\t\t\t><div class=\"dijitSliderMoveable dijitSliderMoveableH\" \n\t\t\t\t\t\t><div dojoAttachPoint=\"sliderHandle,focusNode\" class=\"dijitSliderImageHandle dijitSliderImageHandleH\" dojoAttachEvent=\"onmousedown:_onHandleClick\" waiRole=\"slider\" valuemin=\"${minimum}\" valuemax=\"${maximum}\"></div\n\t\t\t\t\t></div\n\t\t\t\t></div\n\t\t\t\t><div waiRole=\"presentation\" dojoAttachPoint=\"remainingBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderRemainingBar dijitSliderRemainingBarH\" dojoAttachEvent=\"onclick:_onBarClick\"></div\n\t\t\t></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderRightBumper dijitSliderRightBumper\" dojoAttachEvent=\"onclick:_onClkIncBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\" style=\"right:0px;\"\n\t\t\t><div class=\"dijitSliderIncrementIconH\" tabIndex=\"-1\" style=\"display:none\" dojoAttachPoint=\"incrementButton\"><span class=\"dijitSliderButtonInner\">+</span></div\n\t\t></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td dojoAttachPoint=\"containerNode,bottomDecoration\" class=\"dijitReset\" style=\"text-align:center;\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n></table>\n"
	});
})();
