/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.datapool.web.ui.internal.view.extension.DatapoolDataSection");
dojo.provide("com.ibm.rqm.datapool.web.ui.internal.view.extension.DatapoolDataSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rqm.datapool.web.ui.internal.view.common.CsvStore");
dojo.require("dojox.uuid.generateRandomUuid");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableView");
dojo.require("com.ibm.rqm.datapool.web.client.internal.datapool.DatapoolClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() { // module container 	

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
var DataTableView = com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableView;
var DatapoolClient = com.ibm.rqm.datapool.web.client.internal.datapool.DatapoolClient;
var Constants  = com.ibm.rqm.planning.web.client.internal.Constants;
dojo.declare("com.ibm.rqm.datapool.web.ui.internal.view.extension.DatapoolDataSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
    templateString: '<div class="com-ibm-rqm-test-requirement-plan-section">' +
    '   <div dojoAttachPoint="_tableContainer"></div>' +
    '</div>',
    //@Overriden from AbstractArtifactEditorSection
    initialize: function(isEditable){
    	if (!this._dataViewer) {
            this._dataViewer = new DataTableView({
                actions: this.getTableActions()
            });
        }
        this._showDataTable();
        this.initialized = true;
    },
	
	//actions to be displayed in the table header
    getTableActions: function(){
        var actions = [];
 
        actions.push({
            actionId: "editColumnType",
            label: Constants.getMessage("labelEdit"),
            actionHandler: dojo.hitch(this, "_handleEditColumnType"),
            iconClass: "edit-icon-image"
        });
		
		actions.push({
            actionId: "discard",
            label: Constants.getMessage("labelDiscardChanges"),
            actionHandler: dojo.hitch(this, "_handleDiscard"),
            iconClass: "discard-edit-icon-image"
        });
		
		actions.push({
            actionId: "save",
            label: Constants.getMessage("labelSave"),
            actionHandler: dojo.hitch(this, "_handleSave"),
            iconClass: "save-icon-image"
        });
				
        return actions;
    },
    
	_handleEditColumnType: function(){
        this._dataViewer.toggleTableStateForEditing(false);
    },
    
	_handleSave: function(){
        this._dataViewer.toggleTableStateForEditing(true); 
		
        if (this.getArtifact()) {
			var bModified = false;
			var variables = this.datapool.variables;
			var editors = this._dataViewer.tableViewer.headerRenderer.columnTypeEditors;
			var index;
			for(index = 0; index < variables.length; index++){
				if (this.getSuggestType(editors[index].value) != variables[index].suggestedType){
					bModified = true;
					this.parentEditor.setDirty(true);
					break;
				}
			}
		}
    },
    
	_handleDiscard: function(){
        this._dataViewer.toggleTableStateForEditing(true);
		
        var variables = this.datapool.variables;
        var editors = this._dataViewer.tableViewer.headerRenderer.columnTypeEditors;
        var index;
        for(index = 0; index < variables.length; index++){
            if (this.getSuggestType(editors[index].value) != variables[index].suggestedType){
                editors[index].selectedIndex =  parseInt(variables[index].suggestedType);
            }
        }        
    },
	
    _showDataTable: function(){
        if (this.getArtifact() && this.getArtifact().dataAttachment) {
			dojox.data.dom.removeChildren(this._tableContainer);
			this._tableContainer.appendChild(this._dataViewer.domNode);
            var responseHandler = new ServiceResponseHandler(this, "_doShowDataTable", "_error");
            DatapoolClient.getDatapoolDTOs(responseHandler, {
                itemId: this.getArtifact().itemId,
                isDataRequired: "true"
            });
        }
    },
    
    _doShowDataTable: function(datapools){
    	if (datapools) {
			this.datapool = datapools[0];
        	this._dataViewer.refresh(datapools[0]);
        }
    },
    
    //@Overridden from AbstractArtifactEditorSection
    saveFinished: function(savedItem){
        this.inherited('saveFinished', arguments);
        if(this.initialized)
            this._showDataTable();
    },
    
    //@Abstract: Override if desired; Default: no header information			
    getHeaderContent: /* String || DOM Node */ function(isEditable){
    	return null; // override abstract header to disable section remove action. 
    },
	
    _error: function(e){
        alert("An error occurred: [" + e.message + "] @ DatapoolDataSection#_error");
    },
	
    //Override to contribute attributes to the parent's save
    getAttributes: function(){
		if (this.datapool) {
			var attributes = {
				variablesJsonString: ""
			};
			var variables = this.datapool.variables;
			var res = [];
			
			var editors = this._dataViewer.tableViewer.headerRenderer.columnTypeEditors;
			//attributes.variables
			for (var index = 0; index < variables.length; index++) {
				var id = variables[index].id? variables[index].id: dojox.uuid.generateRandomUuid();
				var name = variables[index].name;
				var type = editors[index].value;
				var description = variables[index].description ? variables[index].description : null;
				var role = null;
				res.push("" + id + ":" + name + ":" + type + ":" + description + ":" + role);
			}
			attributes.variablesJsonString = dojo.toJson(res);
			return attributes;
		}
		return null;
    },
	
	//7583
	updateDataTable: function(documentURI,sameFileName) {
		var store = new com.ibm.rqm.datapool.web.ui.internal.view.common.CsvStore({url: documentURI});
		var content = {
			numOfColumns: 0,
			variables: [],  //{name}
			records:[/*{
				cells: [{
					value: ""	
				}] 
			}*/]
		};
		
		// do it
		var self = this;
		this.sameFileName = sameFileName;
		var onFetch = function(items) {
			try {
				var iLength = items.length;
				for(var i = 0; i < iLength; i++) {
					var item = items[i];
					var attributes = store.getAttributes(item);
					var cells = new Array;
					var jLength = attributes.length;
					for (var j = 0; j < jLength; j++) {
						if(content.numOfColumns == 0) {
							var column = attributes[j].split(":");
							var attribute;
							if (column.length == 2) {
								attribute = {
									name: column[0],
									suggestedType: self.getSuggestType(column[1])
								};
							}else if (column.length ==1){
								attribute = {
									name: column[0],
									suggestedType: "0"
								};
							}
							content.variables.push(attribute);
						}
						var cell = {value: store.getValue(item, attributes[j])};
						cells.push(cell);
					}
					content.records.push({cells: cells});
					if(content.numOfColumns == 0) {
						content.numOfColumns = attributes.length;
					}
				}
				//test if its the same datapool with same number,name and type of variables and same file name
				//defect 23146
				if(self.datapool && self.sameFileName){
					var oldDatapoolVars = self.datapool.variables;
					var newDatapoolVars = content.variables;
					var len1 = oldDatapoolVars.length;
					var len2 = newDatapoolVars.length;
					var updatedVars = [];
					if(len1 == len2){
						for(var i=0;i<len1;i++){
							if(oldDatapoolVars[i].name ==newDatapoolVars[i].name && 
								oldDatapoolVars[i].suggestedType ==newDatapoolVars[i].suggestedType ){
									updatedVars[i] = {name:oldDatapoolVars[i].name,
													suggestedType:oldDatapoolVars[i].suggestedType,
													id:oldDatapoolVars[i].id,
													role:null,
													description:oldDatapoolVars[i].description}
							}else{
								updatedVars = newDatapoolVars;
								break;
							}	
						}
						content.variables = updatedVars;
					}
				}
				self.datapool = content;
				dojox.data.dom.removeChildren(self._tableContainer);
				self._tableContainer.appendChild(self._dataViewer.domNode);
				self._dataViewer.refresh(content);
			} catch(e) {
				console.error(e);
			}
		}
		var request = store.fetch({onComplete: onFetch});
	},
	
	getSuggestType: function(type){
		type = type.toUpperCase();
        if(type == "STRING") return "0";
		else if(type == "NUMBER") return "1";
		else if(type == "BOOLEAN") return "2";
		else if(type == "ENUMERATION") return "3";
		return "0";//default to string for unknown types
	}
});
})();