/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.extension.ReviewSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");

dojo.require("com.ibm.rqm.process.web.ui.internal.view.approval.MainReview");

(function(){ // module container
	var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;

    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    
    // global save context
    var SaveParticipant = com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.ReviewSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: "<div dojoAttachPoint=\"_reviewSection\" class=\"com-ibm-rqm-approval-web-ui-internal-ReviewSection\"></div>",
        
        constructor: function(args) {
			this._initialized = true;
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this.addWorkItemSupport(null, this.getParentEditor().isEditable());
        },
        
		//@Override
        isEditable: function() {
			if (this.isSnapshot()) {
				return false;
			}
			
			//locked artifact can't be editable
			var artifact = this.getParentEditor().getArtifact();
			var options = this.getParentEditor().getArtifactOptions();
			if (artifact && artifact.locked == true && options && options.lockingEnabled) {
                return false;
            }
			
			//in case of ready-only license, this section must be editable to viewers
            var op = PermissionsClient.getStoredPermission(PermissionsClient.OP_PROCESS_APPROVAL);
            if (op && op.create && op.edit) {
				return true;
			}
			return this.inherited('isEditable', arguments);
        },
				
        //@Overriden from AbstractArtifactEditorSection - invoked on first expand
        initialize: function(isEditable, initCallback){
            var artifactType = null;
            var parentEditor = this.getParentEditor();
              
            var self = this;
            var args = {
				itemType: self._getItemType(),
                itemId: self.getParentEditor().getVersionableItemId(),
				stateId: self.getParentEditor().getStateId(),
                parentController: self.parentController,
				parentSection: this,
                originator: (this.getArtifact() == null) ? null : this.getArtifact().originator,
                isEditable: this.isEditable(),
                sectionContainer: this.container,
                section: this,
                initCompleteCallback: initCallback
                //sectionOwner: (this.getWorkItemInfo() == null) ? null : this.getWorkItemInfo().owner
            }
            
            this._widget = new com.ibm.rqm.process.web.ui.internal.view.approval.MainReview();
            this._widget.initialize(args);
            
            dojo.addClass(this._widget.minimizedForm, "hidden");
            dojo.removeClass(this._widget.maximizedForm, "hidden");
            this.unloadContent(this._reviewSection);
            this._reviewSection.appendChild(this._widget.domNode);
            
			if (this.isEditable() && !this.saveParticipant) {
				this._addSaveParticipant();
			}
            this.initialized = true;
        },
		
		_getItemType: function() {
            switch (this.getParentEditor().getArtifactType()) {
                case Constants.TYPE_CASE:
                    return "VersionedTestCase";
                case Constants.TYPE_PLAN:
                    return "VersionedTestPlan";
                default:
                    return "";
            }			
		},
        
        _addSaveParticipant: function(){
            //create a SaveParticipant for the section
            this.saveParticipant = new SaveParticipant();
            this.saveParticipant.setDirty(false);
            this.saveParticipant.getItemType = function(){
                return "com.ibm.rqm.process.ApprovalRecord";
            };
			
			var self = this;
			this.saveParticipant.parent = this.getParentEditor();
			this.saveParticipant.saveContext = this.getSaveContext();
			
			this.saveParticipant.getItemId = function(){
                return self.getParentEditor().getVersionableItemId();
            };
			this.saveParticipant.getStateId = function(){
                return self.getParentEditor().getStateId();
            };
			this.saveParticipant.isDirty = function(){
				var commands = self._getUpdateCommands();
				return (commands && commands.length > 0);
            };
			this.saveParticipant.getAttributes = function(){
				var commands = self._getUpdateCommands();				 
				var editor = self.getParentEditor();                
				var attributes = {
					artifactItemTypeName: self._getItemType(),
					artifactPackageUri: Constants.PACKAGE_URI,
                    //Set the summary.  This is used if/when a work item is created for this new approval.
                    //Get the artifact type (Test Plan, Test Case) and artifact name from the parent editor					
					//summary: Constants.getMessage("msgApprovalSummary", editor.getArtifactType(), editor.getHeader().getTitle()),
					operationJson: dojo.toJson(commands)
				};
				//create new referece if it is dirty
				if (editor.isNew() && commands && commands.length > 0) { //(param.remove || param.add || param.update) {
					this.saveContext.createReference(this, "artifactItemId", this.parent);
				}
				if (!editor.isNew()) {
					attributes.artifactItemId = editor.getVersionableItemId();
				}
				return attributes;
            };
            this.getSaveContext().addParticipant(this.saveParticipant);
        },
		
		_getUpdateCommands: function() {
			if(this._widget && this._widget.approveWidget)
				return this._widget.approveWidget._getUpdateCommands();
			return "";
		},
        
        //@Overriden from AbstractArtifactEditorSection
        //Summary: provide section level validation; this is a hack since any section
        //requiring validation should be using the global save's validation mechanism 
        //@Deprecated: WILL BE REMOVED IN M7B - See RQM WI 2562
        //@Abstract: Override to provide section level validation
        validate: function(){
            var validate = {};
            if (this._widget && this._widget.getApprovalWidget()) {
				var tableView = this._widget.getApprovalWidget();
				validate = tableView.validate();
				if (validate.value) {
					validate = this._validateDraftState();
				}
				if (!validate.value) {
					this.getParentEditor().setHeaderStatus(validate.msg, Constants.HEADER_STATUS_ERROR, validate.msg);
					return false;
				}
			}
            return true;
        },
        
        //A workaround of WI 18841. If adding new approvers or reviewers and the test plan 
        //or test case is in Draft state, you need to set the Action to "Ready for Review".
        _validateDraftState: function() {
        	//Get the last row data of approval table
        	var content = this._widget.getApprovalWidget().tableViewer.getContent();
        	if (content && content.length > 0) {
        	    var approvalStatus = content[content.length-1].status;
        	    //Get the status of test plan/test case
        		var state = this.getParentEditor().getStateValue();
        		//Get the Action selection value
        		var selection = this.getParentEditor().getActionValue();
        	    //If you add new approvers or reviewers and in Draft state, 
        	    //the Action should be "Ready for Review".
        	    if ((state == Constants.STATETRANSITION_STATE_DRAFT) 
        	    	&& (selection == "NULL")
        	    	&& (!approvalStatus)) {
        	    	return {
						msg: Constants.APPROVAL_ADD_APPROVER_IN_DRAFT_WARNING_APPROVAL,
						value: false
					};
        	    }
        	}
        	return {value: true};
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.addWorkItemSupport(null, this.getParentEditor().isEditable());
            if(this.initialized)
			    this.initialize(this.isEditable());
        },
		
		//@Overridden from AbstractArtifactEditorSection
        saveRequiresSignature: function() {
			if (!this.initialized) {
				console.debug(this.declaredClass + ": esig NOT required. section wasn't used.");
				return false;
			}
			var options = this.getParentEditor().getArtifactOptions();
			if (!options || options[Constants.ARTIFACT_OPTIONS_PREFERENCE_SIGN_REVIEWS_KEY] != true) {
				//review signing feature disabled
				console.debug(this.declaredClass + ": esig NOT required. feature disabled.");
				return false;
			}
            //var param = this._getSaveParam();
			var originals = this._widget._approvalDTO.approvalResults || [];
			var updates = this._widget.getApprovalWidget().content || [];
			
			//determine whether any changes have been made that would necessitate esig
            for (var i = 0; i < originals.length; i++) {
                var original = originals[i];
                var update = this._findIn(updates, original);
                //has approval been removed
                if (update == null) {
                    console.debug(this.declaredClass + ": esig IS required. approval removed.");
                    return true;                                        
                }
				//look for changes requiring esig
                if (update.status != original.status) {
					//status has changed
                    console.debug(this.declaredClass + ": esig IS required. at least one status change detected.");
                    return true;                                        
                } else if (update.approvalType != original.approvalType) {
                    //type has changed
                    console.debug(this.declaredClass + ": esig IS required. at least one approval type has changed.");
                    return true;                                        					
				}  else if (update.ownerItem != original.owner.itemId) {
                    //owner has changed
                    console.debug(this.declaredClass + ": esig IS required. at least one owner has changed.");
                    return true;                                                            
                }	
            }
			
			//changes made do not require esig
			console.debug(this.declaredClass + ": esig NOT required. changes made do not require it.");
			return false;
        },		
		
		_findIn: function(approvals, approval) {
			approvals = approvals || [];
			for (var i = 0; i < approvals.length; i++) {
				if (approvals[i].id != null && approvals[i].id == approval.id) {
					return approvals[i];
				}
			}
			return null;
		},
        
        //only the artifact's owner or originator is the manager
        _isManager: function(){
            var artifact = this.getArtifact();
            var workItem = this.getWorkItem();
            var ownerItemId = (workItem == null) ? null : workItem.ownerItemId;
            var currentUser = EditorUtil.getCurrentContributor();
            var originator = (artifact == null) ? null : artifact.originator;
            if (this.isEditable() && (currentUser.name != "GUEST") &&
            (originator == null || ownerItemId == currentUser.itemId || originator.itemId == currentUser.itemId)) {
                return true;
            }
            return false;
        },
		
		validateStateTransition:function(initialState, action, endState, editable, callback)
		{
			if (!this._widget) {
                if (callback) {
                    // initialize the widget and provide a completion callback
                    // for it. 
                    this.initialize(editable, dojo.hitch(this, function(){
                        // get the state validation and return it to the callback.
                        var val = this._widget.validateStateTransition(initialState, action, endState);
                        callback(val);
                    }));
                    // return a delay flag.
                    return Constants.EDITORSECTION_STATE_VALIDATION_FETCHING;
                }
                else {
                    // no callback specified and we don't know the verdict at this point
                    // just return null. This is an unlikely scenario as the editor would
                    // always provide a callback.
                    return null;
                }
            }

			return this._widget.validateStateTransition(initialState, action, endState);
		},
		uninitialize: function(){
			this.initialized = false;
		}	
    });
    
})();
