/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.extension.RichTextSection");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.extension.RichTextSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var RichTextControl = com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	
    /**
     * Widget to display list of link elements in the LeftNavigation.
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.extension.RichTextSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: '<div dojoAttachPoint"_richTextNode"></div>',
        
        constructor: function() {	
            this.name = null;
			this.summary = null;
			this.content = null;						
			this._richText = null;	
        },
        
        postCreate: function(args){	
            if (!this.parentEditor) {
                throw new Error("RichTextSection: argument 'parentEditor' is REQUIRED!")
            } else if (!this.getConfig()) {
				throw new Error("RichTextSection: argument 'config' is REQUIRED!")
			}	
			this.name = this.name ? this.name : this.getConfig().name;
			this.content = this.content ? this.content : "";
            //this.summary
        },
        
		//will be lazily loaded by getMainContent
		//@Overridden from AbstractArtifactEditorSection
        initialize: function(isEditable){
            //create the rich text control
			dojox.data.dom.removeChildren(this._richTextNode);
			if (!this._richText) {
				this._richText = new RichTextControl({
					inEditMode: true,
					contentHTML: this.content,
					save: dojo.hitch(this, function(){
						var editor = this.getParentEditor();
						if(dojo.isFunction(editor.onSave)){
							editor.onSave();
						}
					})
				});
			} else {
				this._richText.setContentHTML(this.getSection().content);
			}
			this._richTextNode.appendChild(this._richText.domNode); 
            this.initialized = true;		                 
        },
               
        //@Overridden from AbstractArtifactEditorSection
        getAttributes: function() {  
            if (this._richText) {
				this.content = this._richText.getContentHTML();
			}
			var attributes = {};
			attributes[this.name] = this.content;
			return attributes;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.content = savedItem[this.name];
            if(this.initialized)
			    this.initialize();
        },
		
		//@Overriden from AbstractArtifactEditorSection
        getSummaryContent: function(isEditable) {
            return this.summary ? this.summary : null;
        },    
		
		//@Overriden from AbstractArtifactEditorSection
        getSummaryChangeHandler: /* Function */ function(isEditable) {
            return dojo.hitch(this, "_summaryChanged");
        },	
		
		//@Overriden from AbstractArtifactEditorSection
        getHeaderContent: function(isEditable) {
            return null; //"Last Modified ...";
        },	
        
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ RichTextSection#_error");
        }
    });
})();
