/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.execution.web.ui.internal.view.extension.TestSuiteSummarySection");
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.extension.TestSuiteSummarySection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");

(function(){
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var ImageClient = com.ibm.team.workitem.web.client.internal.ImageClient;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
    dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.extension.TestSuiteSummarySection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.execution.web", "ui/internal/view/templates/extension/TestSuiteSummarySection.html"),
        
        constructor: function(){
        },
        
        postCreate: function(args){
            this.inherited('postCreate', arguments);
			this.addWorkItemSupport(EditorUtil.getCurrentContributor().itemId);
			this.UNASSIGNED = Constants.getMessage("labelUnassigned");
			this.SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
			
			//create form widgets (but do not append to the dom until initialize)
			var testSuite = this.getArtifact();			
			
			this._categorySelect = new com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection({
				parent: this,
				artifactType: Constants.MODEL_TYPE_SUITE,
				artifact: testSuite,
				selectDefaults: this.getParentEditor().isNew(),
                
                // Adjust the non-category fields' label width to match that of the 
                // category labels. This ensures that all of the fields displayed in
                // the section would align correctly. Add a callback to do this after
                // all of the categories have been loaded.
                //
                afterInitialize: dojo.hitch(this, function() {
                    var labelWidth = this._categorySelect.getLabelWidth() - 2;
                    
                    // no categories, align our internal fields to a fixed width. 
                    if(labelWidth <= 0) {
                        labelWidth = 95;
                    }
                    // find all field label nodes and apply the width to them.
                    var labelTDs = dojo.query(".tc-category-section-label-td", this._categoriesDisplayNode);
                    for(var i = 0; i < labelTDs.length; i++) {
                        labelTDs[i].width = labelWidth;
                    }
                })			
			});						           
        },
		
        uninitialize: function(){
            // orphan widgets would be leaked if they have not attached
            if(!this.initialized) {
                if(this._categorySelect && this._categorySelect.destroy)
                    this._categorySelect.destroy();
            }
            this._categoriesEditor = null;
        },
        
        //@Overriden from AbstractArtifactEditorSection
        initialize: function(isEditable){
            var testSuite = this.getArtifact();
			
            //category labels
            this._categoriesTitle.innerHTML = Constants.getMessage("labelCategories");          
            var args = {
                label: Constants.getMessage("labelManageCategories", Constants.TYPE_SUITE),
                onClick: dojo.hitch(this, "_handleCategoriesClick"),
                iconClass: "categories-icon-image",
                showLabel: false
            };
            this._manageCategoriesActionButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(args);           
            this._manageCategoriesActionNode.appendChild(this._manageCategoriesActionButton.domNode);
            this._manageCategoriesActionButton.setDisabled(!this.isEditable()); 			
			
			//setup category selection
			dojox.data.dom.removeChildren(this._categoriesNode);
			this._categoriesNode.appendChild(this._categorySelect.domNode);
			this._categorySelect.setDisabled(!this.isEditable());
			this.initialized = true;            
        },
        
        //@Overriden from AbstractArtifactEditorSection
        getActions: function(){
            return null;
        },
        
        //@Overridden from AbstractArtifactEditorSection --> disable 'remove' capability for Summary section          
        getHeaderContent: /* String || DOM Node */ function(isEditable) {
            return null;
        },        
		
		validate: function() {
			//categories
			if (this._categorySelect && !this._categorySelect.validate()) {
				return false;
			}
			return true;
		},
		
        //@Overriden from AbstractArtifactEditorSection
        getAttributes: function(){
            var attributes = {};

			if (this._categorySelect) {
                //TODO send up an array instead of this comma delimited string
				attributes.categories = this._categorySelect.getSelectedItemIds().join();
			}
            return attributes;
        },
   
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.addWorkItemSupport();
        },
		
		        
        _handleCategoriesClick: function() {
            this._categoriesEditor = new com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor({
                artifactType: Constants.MODEL_TYPE_SUITE,
                onClose: dojo.hitch(this, "_updateCategories")
            });
        },      
        
        _updateCategories: function() {
            //refresh the category selection widget; refreshes the list
            if (this._categorySelect) {
                this._categorySelect.refresh();
            }
        },
		
		_onChange: function(){
			//Do nothing
		},
        
        _error: function(e){
            throw new Error("An error occurred: [" + e.message + "] @ TestSuiteSummarySection#_error");
        }
    });
})();


