/*HomePageView***************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplatesView");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.template.TemplateClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TemplatesTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplateEditor");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

(function()
{   
	var ActionEntityType = com.ibm.rqm.planning.web.ui.internal.model.action.ActionEntityType;	
    var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;	    
	var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
	
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var TemplateClient = com.ibm.rqm.planning.web.client.internal.template.TemplateClient;
	var TemplatesTableView = com.ibm.rqm.planning.web.ui.internal.view.table.TemplatesTableView;    
	var TemplateEditor = com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplateEditor;	
	var ViewHeaderLayout= com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplatesView",  [dijit._Widget, dijit._Templated], 
	{        		           
		templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/itemtemplates/TemplatesView.html"),        
		
		labelCreateTestPlanTemplate: Constants.getMessage("labelCreateTemplate", Constants.TYPE_PLAN),
		labelCreateTestCaseTemplate: Constants.getMessage("labelCreateTemplate", Constants.TYPE_CASE),
		labelCopyTemplate: Constants.getMessage("labelCopyTemplate"),
		
		constructor: function() {
			this._view = null;
			this._templates = null;
		},
		
		// initialization      
		postCreate: function() {
			//load the reports
			this.initialize();		
		},
		
        initialize: function(){
			this._templates = null;			
            var templatesHandler = new ServiceResponseHandler(this, "_showTemplateTable", "_error");
            TemplateClient.getTemplates(templatesHandler, {});   
		},	
				
		/*
		 * Create or Refresh the TemplatesTableView widget 
		 */
		_showTemplateTable: function(templates) {
			console.debug(this.declaredClass + ":_showTemplateTable --> entry.");
			this._templates = templates;
            if (!this._templatesTable) {
				dojox.data.dom.removeChildren(this._templatesTableNode);
				var tArgs = {
					queryName: Constants.getMessage("labelManageTemplates"),
					templates: this._templates,
					actions: this._getTemplateTableActions(),
					refreshCallback: dojo.hitch(this, "initialize")					
				}
				this._templatesTable = new TemplatesTableView(tArgs);				
                this._templatesTableNode.appendChild(this._templatesTable.domNode);			     
			} else {
				this._templatesTable.refresh(this._templates);
			}			
		},
		
		_getTemplateTableActions: function() {
			var permissions = PermissionsClient.getStoredPermission(PermissionsClient.OP_TEMPLATE);
            var actions = [];            
            actions.push({
				id: "tp_create", //$NON-NLS$
                label: this.labelCreateTestPlanTemplate,
                actionHandler: dojo.hitch(this, "_handleNewTemplate", Constants.EDITOR_ID_PLAN, this.labelCreateTestPlanTemplate),
                iconClass: "newplantemplate-action-icon-16",
				disabled: !(permissions != null && permissions[PermissionsClient.ACTION_CREATE])
            });
            actions.push({
				id: "tc_create", //$NON-NLS$
                label: this.labelCreateTestCaseTemplate,
                actionHandler: dojo.hitch(this, "_handleNewTemplate", Constants.EDITOR_ID_CASE, this.labelCreateTestCaseTemplate),
                iconClass: "newcasetemplate-action-icon-16",
				disabled: !(permissions != null && permissions[PermissionsClient.ACTION_CREATE])
            });  
			actions.push({
				id: "copy", //$NON-NLS$
                label: this.labelCopyTemplate,
                actionHandler: dojo.hitch(this, "_handleCopyTemplate", this.labelCopyTemplate),
                iconClass: "template-icon-16",
				requiresSelection: true,
				disabled: !(permissions != null && permissions[PermissionsClient.ACTION_CREATE])
            });  	
            actions.push({
				id: "setDefault", //$NON-NLS$
                label: Constants.getMessage("labelSetDefaultTemplate"),
                actionHandler: dojo.hitch(this, "_handleSetDefault"),
                iconClass: "template-default-icon-16",
				requiresSelection: true,
				disabled: !(permissions != null && permissions[PermissionsClient.ACTION_SETDEFAULT])
            });
            actions.push({
				id: "archive", //$NON-NLS$
                label: Constants.getMessage("labelArchive"),
                actionHandler: dojo.hitch(this, "_handleArchive"),
                iconClass: "delete-icon-image",
                requiresSelection: true,
				disabled: !(permissions != null && permissions[PermissionsClient.ACTION_ARCHIVE])
            });
            							
			return actions;
		},
		
		_handleNewTemplate: function(editorId, title) {
			if (this._editor && dojo.isFunction(this._editor.destroy)) {
				this._editor.destroy();
			}
	        this._editor = new TemplateEditor({
				dialogTitle: title,
	            editorId: editorId,
				onOK: dojo.hitch(this, "initialize")
	        });			
		},
		
		
		
        _handleCopyTemplate: function(title) {
			var selected = this._templatesTable.getSelected();
            if (!dojo.isArray(selected) || selected.length == 0) {
                alert(Constants.getMessage("msgTemplateMissing", Constants.getMessage("labelCopy")));
				return;
            } else if (selected.length > 1) {
                alert(Constants.getMessage("msgTemplateSingle", Constants.getMessage("labelCopy")));
				return;
            }
			
			var copy = selected[0];
			delete copy.itemId;
			delete copy.id;
			copy.modifiable = true;
            if (this._editor && dojo.isFunction(this._editor.destroy)) {
                this._editor.destroy();
            }			
            this._editor = new TemplateEditor({
				dialogTitle: title,           
				template: copy,
				isCopy: true,
                onOK: dojo.hitch(this, "initialize")
            });         
        },	
		
        _handleSetDefault: function(editorId) {
            var selected = this._templatesTable.getSelected();
            if (!dojo.isArray(selected) || selected.length == 0) {
                alert(Constants.getMessage("msgTemplateMissing", Constants.getMessage("labelSetDefaultTemplate")));
                return;
			} else if (selected.length > 1) {
                alert(Constants.getMessage("msgTemplateSingle", Constants.getMessage("labelSetDefaultTemplate")));
				return;
            }
            
            var template = selected[0];
			var srh = new ServiceResponseHandler(this, "initialize", "_error");
            TemplateClient.postSetDefaultTemplate(srh, template.editorId, template.itemId);
        },		
		
        _handleArchive: function(editorId) {
            var selected = this._templatesTable.getSelected();
            if (!dojo.isArray(selected) || selected.length == 0) {
                alert(Constants.getMessage("msgTemplateMissing", Constants.getMessage("labelArchive")));
                return;
			} else if (selected.length > 1) {
                alert(Constants.getMessage("msgTemplateSingle", Constants.getMessage("labelArchive")));
                return;
			}

            var self = this;
            var template = selected[0];
            var args = {
                title: Constants.getMessage("labelConfirmation"),
                message: Constants.getMessage("msgConfirmArchiveTemplate", template.name),
                yesCallback: function() {   
                    //archive the template		            
		            template.archive = true;
		            var args = {
		                itemId: template.itemId,
		                id: template.id,
		                editorId: template.editorId,
		                name: template.name,
		                description: template.description ? template.description : "",
		                archive: true
		            }
		            var srh = new ServiceResponseHandler(self, "initialize", "_error");
		            TemplateClient.postTemplate(srh, args);
                    return true;          
                },
                noCallback: function() {
                    return false;
                }
            }
            var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
            dialog.setSecondaryTitle(args.title);
            dialog.open();			        
        },			
		
		_error: function(e) {
			console.error("TemplatesView: Error: " + e.message);
		}	

	});
})();
