/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.keyword.web.ui.internal.view.keyword.KeywordsContentFilterProvider");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {

var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
var TableContentFilterProvider = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;

dojo.declare("com.ibm.rqm.keyword.web.ui.internal.view.keyword.KeywordsContentFilterProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider,
{
    constructor: function(tableView, openSection) {
        this.tableView = tableView;
        this.initOpenFilter = openSection;

        // define the static filters
        this.filters = [];
        this.filters.push({
            name: "name",  // must match key in the arg list of the API
            title: Constants.getMessage("labelKeyword") + Constants.getMessage("labelColen"),
            promptTitle: Constants.getMessage("labelKeyword"),
            type: TableContentFilterProvider.FIELD_TEXT,
            defaultValue: null, 
            params: null
        });
        this.filters.push({
            name: "tags",  // must match key in the arg list of the API
            title: Constants.getMessage("labelTags") + Constants.getMessage("labelColen"),
            promptTitle: Constants.getMessage("labelTags"),
            type: TableContentFilterProvider.FIELD_TEXT,
            defaultValue: null, 
            params: null
        });
        this.filters.push({
            name: "scriptName",  // must match key in the arg list of the API
            title: Constants.getMessage("labelAssociatedScript") + Constants.getMessage("labelColen"),
            promptTitle: Constants.getMessage("labelAssociatedScript"),
            type: TableContentFilterProvider.FIELD_TEXT,
            defaultValue: null, 
            params: null
        });
    },

    getTitle: function() {
        return Constants.getMessage("labelFilterTitle");    
    },
    
    getIsSectionOpenInitially: function() {
       return this.initOpenFilter; 
    },
    
    getSummaryText: function() {
        return Constants.getMessage("labelFilterDescriptionAllVisible", Constants.getMessage("labelKeywords"));
    },
    
    getInitialVisibleFilterCount: function() {
        return 4;    
    },
    
    getFilters: function() {
        return this.filters;
    },  
    
    getFilterCount: function() {
        return this.filters.length;
    },
    
    getFilterName: function(index) {
        return this.filters[index].name;
    },
    
    getFilterTitle: function(index) {
        return this.filters[index].title;
    },
    
    getFilterPromptTitle: function(index) {
        return this.filters[index].promptTitle;        
    },
    
    getFilterType: function(index) {
        return this.filters[index].type;
    },
    
    getFilterDefaultValue: function(index) {
        return this.filters[index].defaultValue;
    },
    
    getFilterParams: function(index) {        
        return this.filters[index].params;
    },
    
    getCustomField: function(index) {
        return null;
    },
    
    getActionsInfo: function() {
        var actions = [
            {
                id: "cancel",
                title: Constants.getMessage("labelReset"),
                callback: dojo.hitch(this, "filterResetActionInvoked"),
                callbackData: null,
                isDefault: false
            }, {
                id: "run",
                title: Constants.getMessage("labelRun"),
                callback: dojo.hitch(this, "filterRunActionInvoked"),
                callbackData: null,
                isDefault: true
            }
        ];
        return actions;
    },
    
    filterResetActionInvoked: function(actionName, filters) {
        // reset the filters.
        this.tableView.tableViewer.clearContentFilters();
    },

    filterRunActionInvoked: function(actionName, filters){
        var args = {};
        
        for (var nameKey in filters) {
            if (filters[nameKey] != null) {
                args[nameKey] = filters[nameKey] || "";
                if (args[nameKey] == 'Any@@') {
                    args[nameKey] = '';
                }
            }
        }
        this.tableView.runQuery(args);
    }
});

})();

