/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.planning.PlanSummarySection");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.planning.PlanSummarySection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor");


(function(){
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var PlanningClient = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    
    /**
     * Widget to display list of link elements in the LeftNavigation.
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.planning.PlanSummarySection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/planning/PlanSummarySection.html"),
        
		labelCategories: Constants.getMessage("labelCategories"),
		labelMasterTestPlan: Constants.getMessage("labelMasterTestPlan"),
		
        constructor: function(){
            this._workItem = null;
        },
        
        postCreate: function(args){
            this.inherited('postCreate', arguments);
            //set up workitem support for this section
            this.addWorkItemSupport(EditorUtil.getCurrentContributor().itemId);
        },
        
		//@Overriden from AbstractArtifactEditorSection - lazily loaded by getMainContent on first expansion
        initialize: function(isEditable) {
            var plan = this.getArtifact();			
			
			//category labels 			
	        var args = {
	            label: Constants.getMessage("labelManageCategories", Constants.TYPE_PLAN),
	            onClick: dojo.hitch(this, "_handleCategoriesClick"),
				iconClass: "categories-icon-image",
				showLabel: false
	        };
	        this._manageCategoriesActionButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(args);			
			this._manageCategoriesActionNode.appendChild(this._manageCategoriesActionButton.domNode);
			this._manageCategoriesActionButton.setDisabled(!this.isEditable());			
			
            //setup category selection
            this.unloadContent(this._categoriesNode);
            this._categorySelect = new com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection({
                artifactType: Constants.MODEL_TYPE_PLAN,
                artifact: plan,             
                parent: this,
				selectDefaults: this.getParentEditor().isNew() //select default categories for NEW plans only
            });
            this._categoriesNode.appendChild(this._categorySelect.domNode);
            this._categorySelect.setDisabled(!this.isEditable());	
			
			//fetch parent info
			var plan = this.getArtifact();
			if (plan != null && !plan.isMaster) {
				var responseHandler = new ServiceResponseHandler(this, "_showParentInfo", "_error");
				PlanningClient.getTestPlanParent(responseHandler, {uid: plan.versionableItemId});
			}		
        },
		
		_showParentInfo: function(parent) {
			//show parent link, if parent present	
			var CHILDREN_SECTION_ID = "com.ibm.rqm.planning.editor.section.planChildren";
			var editor = this.getParentEditor();

			//use 'child' icon for editor
			editor.getHeader().setIconClass(parent != null ? "testplan-child-icon-48" : "testplan-icon-48");
			editor.getArtifactModel().setTabClass(parent != null ? "testplan-child-icon-16" : "testplan-icon-16");
			
			if (parent != null) {
				var anchor = document.createElement("a");
				anchor.title = parent.name;
				anchor.href = Utility.getTestPlanURI({
					id: parent.id
				});
				anchor.innerHTML = parent.name;
				dojo.addClass(anchor, "section-link");
				dojo.removeClass(this._masterSection, "hidden");
				this._masterNode.appendChild(anchor);
				this._parent = parent;
				editor.blockSection(CHILDREN_SECTION_ID);
				
			} else if (editor.isBlocked(CHILDREN_SECTION_ID)) {
				editor.unblockSection(CHILDREN_SECTION_ID);
			}		
		},
               
        //@Overridden from AbstractArtifactEditorSection
        getAttributes: function(){
            var attributes = {};
            if (this.getWorkItemEditor()) {            
                attributes.ownerId = this.getWorkItemEditor().getOwnerId();                
            }
            if (this._categorySelect) {
                //TODO send up an array instead of this comma delimited string
                attributes.categories = this._categorySelect.getSelectedItemIds().join();
            }			
            return attributes;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
//            this.initialize();
			this.initialized = false;
			this.addWorkItemSupport(true);
        },
        
        //@Overridden from AbstractArtifactEditorSection --> disable 'remove' capability for Summary section          
        getHeaderContent: /* String || DOM Node */ function(isEditable) {
            return null;
        },
        
		//@Overridden from AbstractArtifactEditorSection: 
		validate: function () {
            //categories
            if (this._categorySelect && !this._categorySelect.validate()) {
                return false;
            }			
			return true;
		},
		
	    _handleCategoriesClick: function() {
	        this._categoriesEditor = new com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor({
	            artifactType: Constants.MODEL_TYPE_PLAN,
				onClose: dojo.hitch(this, "_updateCategories")
	        });
	    },		
		
		_updateCategories: function() {
			//refresh the category selection widget; refreshes the list
			if (this._categorySelect) {
				this._categorySelect.refresh();
			}
		},
		
        _onKeyUp: function(e){
            /*
             if (this._description.value != this._plan.description)
             {
             dojo.event.topic.publish("TPE-widget-event", {dirty: true});
             }
             */
        },
        
        _error: function(e){
            console.error(this.declaredClass + ": An error occurred: [" + e.message + "]");
        }
    });
})();
