/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener");
dojo.require("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableHeaderRenderer");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");

(function(){

    var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
	
    dojo.declare("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableListener", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener, {
        constructor: function(parentView){
            this.parentView = parentView;
        },
        
        rowCheckStateChanged: function(tableViewer, rowObject, state){
            var selectionCount = tableViewer.getCheckedRows().length;
            if (this.parentView.deleteActionNode) 
                this.parentView.deleteActionNode.setDisabled(selectionCount == 0);
        },
        
        groupRowsCheckStateChanged: function(tableViewer, rowObjects, state){
            this.rowCheckStateChanged(tableViewer, rowObjects, state);
        },
        
        tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback){
            if (callback != null && dojo.isFunction(callback)) {
                if (callbackData != null) {
                    callback(callbackData);
                }
                else {
                    callback();
                }
            }
        }
    });
    
    dojo.declare("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        getNumColumns: function(content){
            return content.numOfColumns > 0 ? content.numOfColumns : 1;
        },
        
        getColumnHeading: function(content, index){
            return content.numOfColumns > 0 ? content.variables[index].name : Constants.getMessage("labelNoneDatapool");
        },
        
        getColumnHeadingImage: function(content, atColumnIndex){
            return null;
        },
        
        getIsColumnGroupable: function(content, atColumnIndex){
            return false;
        },
        
        getIsColumnSortable: function(content, atColumnIndex){
            return true;
        },
        
        getColumnWidth: function(content, atColumnIndex){
            return content.numOfColumns > 0 ? 100 / content.numOfColumns > 0 : 20;
        },
        
        getContentObjects: function(content){
            return content.numOfColumns > 0 ? content.records : [];
        },
        
        getRowDataId: function(rowData){
            return null;
        },
        
        getCellDisplayType: function(rowData, atColumnIndex){
            var type = TableConstants.RENDER_TYPE_TEXT;
            return type;
        },
        
        getCellAlignment: function(rowData, atColumnIndex){
            return TableConstants.ALIGN_LEFT;
        },
        
        getCellDisplayData: function(rowData, atColumnIndex){
            var data = rowData.cells[atColumnIndex] ? rowData.cells[atColumnIndex].value : "";
			if (data != "" &&
				this.content &&
				this.content.variables && 
				this.content.variables[atColumnIndex].suggestedType == "1") {
				return parseFloat(data);
			}

			return data;
		},
        
        getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
            var val = null;
            return val;
        }
    });
    
    dojo.declare("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableView", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
        '</div>',
        
        constructor: function(){
            this.datapool = null;
        },
        
        uninitialize: function() 
        {
            this.datapool = null;
            this.tableViewer = null;
        },
        
        postCreate: function(){
            var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();
            this._tableViewerNode.appendChild(tableViewer.domNode);
            
            var content = this.datapool ? this.datapool : [];
			var provider = new com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableProvider();
			provider.content = content;
			tableViewer.headerRenderer = new com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableHeaderRenderer(tableViewer);
            tableViewer.setContentPageSize(20);
            tableViewer.setContent(content);
            tableViewer.setContentProvider(provider);
            tableViewer.setTableListener(new com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableListener(this));
			tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelDataRecords")));
            for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
                var action = this.actions[i];
                tableViewer.addAction(action.actionId, action.label, null, action.iconClass, action.actionHandler);
            }
            tableViewer.show();
            
            this.tableViewer = tableViewer;
			
			this.inEditMode = false;

            this.disableAction("save");
            this.disableAction("discard");
			this.hideAction("save");
            this.hideAction("discard");
						
            this.enableAction("editColumnType");            
            this.showAction("editColumnType");

        },
        
        refresh: function(content){
            //HACK - TableViewer should support a refresh
            this.tableViewer.setContent(content);
			this.tableViewer.getContentProvider().content = content;
            this.tableViewer.show();
			if(this.inEditMode)
				this.toggleTableStateForEditing(true);
			// update column type
            this._setColumnTypeValue(content.variables);
			var listener = this.tableViewer.getTableListener();
		 	if(listener)
				listener.rowCheckStateChanged(this.tableViewer);
        },
		
		toggleTableStateForEditing: function(disableEditing){
			if(disableEditing){
				this.inEditMode = false;
				this.disableAction("save");
				this.disableAction("discard");
				this.hideAction("save");
				this.hideAction("discard");
				
				this.enableAction("editColumnType");
				this.showAction("editColumnType");
				var button = this.tableViewer.getActionButton("editColumnType");
				if(button){
					if(!button.tagName){
						button.tagName = "button";
					}
					dijit.focus(button);
				}
            }else{
				this.inEditMode = true;
				this.disableAction("editColumnType");
                this.hideAction("editColumnType");

				this.enableAction("save");
                this.enableAction("discard");
				this.showAction("save");
				this.showAction("discard");
				var button = this.tableViewer.getActionButton("discard");
				if(button){
					if(!button.tagName){
						button.tagName = "button";
					}
					dijit.focus(button);
				}
			}	
            this._showColumnTypeEditor(this.inEditMode);
	    },
		  
		_setColumnTypeValue: function(variables){
			for(var i = 0; i < (variables ? variables.length : 0); i++) {
				var editor = this.tableViewer.headerRenderer.columnTypeEditors[i];
                var type = variables[i].suggestedType;
				editor.value = parseInt(type);  
				SelectUtility.select(editor, editor.value);
			}
		},
		
	    _showColumnTypeEditor: function(bShow){
	    	var columnTypeEditors=this.tableViewer.headerRenderer.columnTypeEditors;
	        for(var index in columnTypeEditors){
	            var editor = columnTypeEditors[index];
	            dojo.style(editor, "display", bShow?"block":"none");
				SelectUtility.select(editor, editor.value);
	        }
	    },
	
	    hideAction: function(actionId) {
	        var button = this.tableViewer.getActionButton(actionId);
	        if (button && !dojo.hasClass(button.domNode, "dijitDisplayNone")) {
	            dojo.addClass(button.domNode,"dijitDisplayNone");
	        }
	    },
	    
	    showAction: function(actionId) {
	        var button = this.tableViewer.getActionButton(actionId);
	        if (button && dojo.hasClass(button.domNode, "dijitDisplayNone")) {
	            dojo.removeClass(button.domNode,"dijitDisplayNone");
	        } 
	    },  
	
	    disableAction: function(actionId) {
	        var button = this.tableViewer.getActionButton(actionId);
	         if (button ) {
	            button.setDisabled(true);
	        } 
	    },
	    
	    enableAction: function(actionId) {
	        var button = this.tableViewer.getActionButton(actionId);
	        if (button ) {
	            button.setDisabled(false);
	        } 
	    }
		
		
    });
    
})();
