/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionResultTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.execution.web.ui.internal.util.Utility");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.rqm.execution.web.ui.internal.util.Utility;
var Constants      = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("ExecutionResultTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    getNumColumns: function(content){
        return 8;
    },

    _columnHeadings: [
        "",
		Constants.getMessage("labelEWINameLabel"),
        Constants.getMessage("labelTestCase"),
        Constants.getMessage("labelTestScript"),
		Constants.getMessage("labelStatus"),
		Constants.getMessage("labelExeBy"),
		Constants.getMessage("labelCompleted"),
		Constants.getMessage("labelExeResult")
    ],

    getColumnHeading: function(content, index){
        return this._columnHeadings[index];
    },

    getColumnHeadingImage: function(content, atColumnIndex){
        return null;
    },
	 
    getColumnHeadingAlignment: function(atColumnIndex){
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex){
        return (atColumnIndex >= 1 && atColumnIndex <= 5);     
    },

    getIsColumnSortable: function(content, atColumnIndex){
        return (atColumnIndex >= 1 && atColumnIndex <= 6);   
    },
    
    _columnWidths: [2,15,20,22,8,15,15,3],  

    getColumnWidth: function(content, atColumnIndex){
        return this._columnWidths[atColumnIndex];
    },
    
    getContentObjects: function(content){
        return content;
    },
    
    getRowDataId: function(rowData){
        return "_table_row_for_execution_result_" + rowData.id;
    },
    
    getCellDisplayType: function(rowData, atColumnIndex){
        var type = TableConstants.RENDER_TYPE_TEXT;

        switch(atColumnIndex) {
            case 0: 
                type = TableConstants.RENDER_TYPE_IMAGE;
                break;
            case 2: 
                type = TableConstants.RENDER_TYPE_LINK;
                break;
            case 3: 
                type = TableConstants.RENDER_TYPE_LINK;
                break;
				case 6: 
                type = TableConstants.RENDER_TYPE_DATE;
                break;					 			 
            case 7: 
                type = TableConstants.RENDER_TYPE_IMAGE;
                break;					 
        }
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex){
        if(atColumnIndex == 0 || atColumnIndex == 7)
            return TableConstants.ALIGN_CENTER;
        return TableConstants.ALIGN_LEFT;
    },
    
    getCellDisplayData: function(rowData, atColumnIndex){
        var val = "";
        switch(atColumnIndex) {
            case 0: 
                val = ""; //ImageCache.TEST_CASE;
                break;
            case 1: 
                val = Constants.getMessage("labelEWIName", rowData.ewi.id); //EWI Name
                break;
            case 2: 
                val = rowData.ewi.testCase.name; //TestCase Name
                break;
            case 3: 
                val = rowData.script? rowData.script.name:""; //TestScript Name
                break;          
            case 4: 
                val = rowData.state ? rowData.state.name : Constants.getMessage("labelUnknown"); // + "/" + rowData.verdict.name; //State
                break;  
            case 5: 
                val = rowData.ewi.owner.name; //Executed By
                break; 
				case 6: 
				    if (rowData.endTime > -1) {
                   val = new Date(); //Completed
					    val.setTime(rowData.endTime);
					 }
                break;
            case 7:
                val = "";
					 break;
        }
        return val;
    },
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex){  
        var val = null;
		if (atColumnIndex == 0) {
			val = {};
			val.image = ImageCache.EWI_LOG;
		}
		else if(atColumnIndex == 2) {
			val = {};
			val.href = Utility.getTestCaseURI(rowData.ewi.testCase.id);				
        }
        else if (atColumnIndex == 3 && rowData.script) {
           val = {};
			  val.href = Utility.getTestScriptURI(rowData.script.id, Utility.CONSTRUCTION_PERSPECTIVE);
        }		  
        else if (atColumnIndex == 7) {
           val = {};
			  val.image = ImageCache.EWI_LOG;
			  val.href = Utility.getResultURI({ewiId: rowData.ewi.id, resultItemId: rowData.itemId});
		  }		  
        return val;
    }
    
    
});

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionResultTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
	                 '   <div>' +
						  '      <span dojoAttachPoint="_queryTitle" class="query-title">${labelCurrentQuery}</span>' +
						  '      <span dojoAttachPoint="_queryName" class="query-name"></span>' +
						  '   </div>' +
						  '</div>',

    widgetsInTemplate: true,
	labelCurrentQuery: Constants.getMessage("labelCurrentQuery"),

    constructor: function() {
	 	this.results = null;
	 },

    postCreate: function(){
        this.inherited("postCreate", arguments);
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            allowMultiSort: true,
            tableId: "RQM-TE-RESULT-TABLEVIEWER"                    
        });        
        this._tableViewerNode.appendChild(tableViewer.domNode);
        var content = this.results ? this.results : [];
		this.queryName = this.queryName ? this.queryName : Constants.getMessage("labelExeResults");
        this._queryName.innerHTML = this.queryName;
        tableViewer.setContentPageSize(20);
        tableViewer.setContent(content);
        tableViewer.setContentProvider(new ExecutionResultTableView.TableProvider());
        tableViewer.setTableListener(new ExecutionResultTableView.TableListener());
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", this.queryName));
        tableViewer.show();
		this.tableViewer = tableViewer;
		  
    }
});

})();
