/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Load: com.ibm.rqm.risk.web.ui.internal.view.table.RiskProfilesTableView");
dojo.provide("com.ibm.rqm.risk.web.ui.internal.view.table.RiskProfilesTableView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.risk.web.client.internal.risk.RiskClient");
dojo.require("com.ibm.rqm.risk.web.client.internal.mapper.RiskProfileMapper");

(function(){
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var RiskClient = com.ibm.rqm.risk.web.client.internal.risk.RiskClient;
	var RiskProfileMapper = com.ibm.rqm.risk.web.client.internal.mapper.RiskProfileMapper;
	var NONE = "None";
	
    dojo.declare("RiskProfilesTableView.TableListener", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener, {
        parentView: null,
        
        constructor: function(testView){
            this.parentView = testView;
        },
        
        // Summary:
        //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
        //   called.
        //
        rowCheckStateChanged: function(tableViewer, rowObject, state){
			this.parentView.changeDetailFactors(rowObject, state);
			
            if (this.parentView.deleteActionNode) 
 	              this.parentView.deleteActionNode.setDisabled(this.parentView.getIsLastProfile(rowObject) || rowObject.defaultProfile);
        },
		
		 // Summary:
        //    If the client have registered table actions, this message will be fired when an action (button)
        //    is invoked by the user.
        //
        tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback){
            if (callback != null && dojo.isFunction(callback)) {
                if (callbackData != null) {
                    callback(callbackData);
                }
                else {
                    callback();
                }
            }
        }
    });
    
    dojo.declare("RiskProfilesTableView.CellModifier", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier, {
        tableViewer: null,
        
        constructor: function(viewer, parent){
            this.tableViewer = viewer;
            this.parent = parent;
        },
        
        cellValueChanged: function(oldValue, newValue, rowObject, columnIndex){
            switch (columnIndex) {
                case 0:
                    rowObject.targetArtifactType = newValue;
					this.parent.changeDetailFactors(rowObject, true);
                    break;
                case 1:
                    rowObject.profileName = newValue;
                    break;
				case 2: 
					rowObject.description = newValue;
					break;
				case 3: 
					rowObject.defaultProfile = newValue;	
					this.parent.resetToDefaultProfile(rowObject);
					break;
                default:
                    break;
            }
			if(this.parent.parentEditor) {
				this.parent.parentEditor.setDirty(true);
			}
        },
        
        cellActionInvoked: function(rowObject, columnIndex){
        }
    });
    
    dojo.declare("RiskProfilesTableView.TableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        constructor: function(args) {
			this.editable = args.editable;
			this.parent = args.parent;
    	},
	
		// Summary:
        //
        getNumColumns: function(content){
            return 4;
        },
        
        //If data in any of the columns is required indicate it here.
        //cols is an array of column indices.
        //The required column headings will have an asterisk appended to them ie "Name" becomes "Name *"
        //The asterisk will ONLY be applied if there are rows in the table.
        setRequiredColumns: function(cols){
            this.requiredCols = cols;
        },
        
        // Summary:
        //
        getColumnHeading: function(content, index){
            var headings = [Constants.getMessage("labelArtifactType"), Constants.getMessage("labelProfileName"), Constants.getMessage("labelDescription"), Constants.getMessage("labelDefault")];
            //If any of the columns are required then append an * to the column header to indicate the data is required
            if (this.requiredCols) {
                for (var i = 0; i < this.requiredCols.length; i++) 
                    headings[this.requiredCols[i]] += " *";
            }
            return headings[index];
        },
        
        getIsColumnSortable: function(content, atColumnIndex){
            return true;
        },
        
        getColumnWidth: function(content, atColumnIndex){
            var width = 2;
            switch (atColumnIndex) {
                case 0:
                    width = 20;
                    break;
                case 1:
                    width = 30;
                    break;
                case 2:
                    width = 40;
                    break;
				case 3:
                    width = 10;
                    break;
            }
            return width;
        },
        
        getIsCellEditable: function(rowData, atColumnIndex){
			var selected = this.parent.tableViewer.rowCheckedHandler.getRowCheckState(rowData);
			if(selected && (atColumnIndex == 3) && !this.parent.getIsLastProfile(rowData) && rowData.defaultProfile == false)
				return true;
			else if(selected && rowData.isNew)
				return true;
			else return false;
        },
        
        // Summary:
        //    The only assumptions the table have on the data that it displays
        //    is that it's an array contents. These are the data content/object that
        //    will be displayed in the table rows.
        //
        getContentObjects: function(content){
            // our content is already an array of objects
            return content;
        },
        
        getRowDataId: function(rowData){
            return rowData?rowData.id:"profileId";
        },
        
        // Summary:
        //    Returns the type of the cell to be displayed. By default, the TableViewer
        //    will support a set of cell types. The type of the cell will map to a specific
        //    cell renderer (to be discussed).
        getCellDisplayType: function(rowData, atColumnIndex){
            var type = TableConstants.RENDER_TYPE_EDITABLE_TEXT;
            switch (atColumnIndex) {
				case 0:
					type = TableConstants.RENDER_TYPE_ENUM;
                    break;
				case 3:
					type = TableConstants.RENDER_TYPE_CHECKBOX;
                    break;
            }
            return type;
        },
        
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "data";
            switch (atColumnIndex) {
                case 0:
                    val = rowData.targetArtifactType;
                    break;
                case 1:
                    val = rowData.profileName;
                    break;
                case 2:
                    val = rowData.description ? rowData.description : "";
                    break;
                case 3:
                    val = ""//rowData.defaultProfile;
                    break;
            }
            return val;
        },
        
        getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
            var val = {};
            switch (atColumnIndex) {
                case 0:
                    val['select'] = rowData.targetArtifactType;
                    val['choices'] = [{
                        'text': Constants.getMessage("labelTestPlan"),
                        'value': Constants.MODEL_TYPE_PLAN
                    }, {
                        'text': Constants.getMessage("labelTestCase"),
                        'value': Constants.MODEL_TYPE_CASE
                    }, {
                        'text': Constants.getMessage("labelRequirement"),
                        'value': Constants.MODEL_TYPE_REQUIREMENT
                    }];
                    break;
				 case 3:
                    val['state'] = rowData.defaultProfile;
                    break;
            }
            return val;
        }
    });
    
    dojo.declare("com.ibm.rqm.risk.web.ui.internal.view.table.RiskProfilesTableView", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="domNode" tabindex="-1">' +
        '</div>',
        
        editable: true,
        printable: false,
        widgetsInTemplate: true,
        parentEditor: null,
		detailTable: null,
		profiles: [],
		lastSelectedProfileId: null,
		
		//for wi 16029
		selectedProfileNameBeforeSave: null,
		
        postCreate: function(){
            this.inherited("postCreate", arguments);
        },
		
		init: function(args) {
			this.parentEditor = args.parentEditor;
			this.detailTable = args.detailTable;
			var responseHandler = new RiskProfileMapper(this, "_initialize", "_error");
			RiskClient.getRiskProfileDTOs(responseHandler, {});
		},
        
        _initialize: function(profiles){
			this.profiles = profiles ? profiles : [];
							
            this.deleted = new Array();
            this.updated = new Array();
			
            //create the table viewer
            var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();
            tableViewer.setTableRowsCheckable(true, /* isSingleSelect */ true);
			
			dojox.data.dom.removeChildren(this.domNode);
            this.domNode.appendChild(tableViewer.domNode);
            this.tableViewer = tableViewer;
            
            
            var contentProvider = new RiskProfilesTableView.TableProvider({
				editable: this.editable,
				parent: this
			});
            
            tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelRiskTable")));
            
			this.addActionNode = tableViewer.addAction("addProfile", // action id
             	Constants.getMessage("labelAddRiskProfile"), // label
             	null, // callback data
             	"add-icon-image", // icon class
             	dojo.hitch(this, "_onAddClick")); // callback
             	
            this.deleteActionNode = tableViewer.addAction("delete", // action id
            	Constants.getMessage("labelDeleteRiskProfile"), // label
             	null, // callback data
             	"delete-icon-image", // icon class
             	dojo.hitch(this, "_onDeleteActionClick")); // callback
			this.deleteActionNode.setDisabled(true);
			
            tableViewer.setContent(this.profiles);
            tableViewer.setContentProvider(contentProvider);
            tableViewer.setTableListener(new RiskProfilesTableView.TableListener(this));
            tableViewer.setCellModifier(new RiskProfilesTableView.CellModifier(tableViewer, this));
            tableViewer.show();
			
			if (this.profiles.length > 0) {
				var selectedProfile = this.profiles[0];
				if(this.selectedProfileNameBeforeSave) {
					for(var i = 0; i < this.profiles.length; i++) {
						if(this.profiles[i].profileName == this.selectedProfileNameBeforeSave) {
							selectedProfile = this.profiles[i];
							break;
						}
					}
				}
				this.tableViewer.rowCheckedHandler.setRowCheckState(selectedProfile,true);
				this.changeDetailFactors(selectedProfile, true);
			}
			else
				this.changeDetailFactors(null, true);
            //if parent section available and not editable, table should not be editable
            tableViewer.setEditable(true);
        },
        
		refresh: function(newContents) {
			this.profiles = newContents;
			this.tableViewer.setContent(newContents);
			this.tableViewer.refresh();
		},
		
        _onDeleteActionClick: function() {
			var dels = this.tableViewer.getCheckedRows();
			if (dels && dels.length > 0) {
                for (var i = 0; i < dels.length; i++) {
                    this.deleted.push(dels[i]);
                    this.tableViewer.deleteItem(dels[i]);
                }
				this.profiles = this.tableViewer.getContent();
				if (this.profiles.length > 0) {
					this.lastSelectedProfileId = null;
					this.tableViewer.rowCheckedHandler.setRowCheckState(this.profiles[this.profiles.length - 1], true);
					this.changeDetailFactors(this.profiles[this.profiles.length - 1], true);
				}
				else {
					this.changeDetailFactors(null, true);
					this.deleteActionNode.setDisabled(true);
				}
				this.tableViewer.refresh();
				if(this.parentEditor){
					this.parentEditor.setDirty(true);
				}
            }
		},
		
		_onAddClick: function() {
			var obj = {
                id: (new Date()).getTime(),
                targetArtifactType: Constants.MODEL_TYPE_PLAN,
                profileName: "",
                description: "",
                defaultProfile: false,
                isNew: true
            };
            this.profiles.push(obj);
            this.tableViewer.setContent(this.profiles);
			this.tableViewer.rowCheckedHandler.setRowCheckState(this.profiles[this.profiles.length-1],true);
            this.tableViewer.refresh();
			this.changeDetailFactors(obj, true);
			this.deleteActionNode.setDisabled(false);
			if(this.parentEditor){
				this.parentEditor.setDirty(true);
			}
		},
		
		getIsLastProfile: function(profile) {
			var profiles = this.profiles;
			var length = profiles.length;
			var isLastProfile = true;
			for(var i = 0; i < length; i++){
				if(profiles[i].targetArtifactType == profile.targetArtifactType && profiles[i].id != profile.id){
					isLastProfile = false;
					break;
				}
			}
			return isLastProfile;
		},

		resetToDefaultProfile: function(rowObj) {
			var targetArtifactType = rowObj.targetArtifactType;
			for(var i = 0; i < this.profiles.length; i++) {
				if((this.profiles[i].targetArtifactType == targetArtifactType) && (this.profiles[i].id != rowObj.id)) {
					this.profiles[i].defaultProfile = false;
					this.tableViewer.getContent()[i].defaultProfile = false;
				}
			}
			this.tableViewer.refresh();
		},
		
		changeDetailFactors: function(rowObject, state) {
			//restore last profile's factor info
			if (this.lastSelectedProfileId) {
				for (var i = 0; i < this.profiles.length; i++) {
					if (this.profiles[i].id == this.lastSelectedProfileId) {
						if (this.detailTable) {
							this.profiles[i].riskMeasurements = this.detailTable.getFactors();
						}
						break;
					}
				}
			}
			if(state && this.detailTable) {
				this.lastSelectedProfileId = rowObject ? rowObject.id : null;
				this.detailTable.refresh(rowObject ? rowObject.targetArtifactType : null, 
					rowObject? rowObject.riskMeasurements : [],
					rowObject? rowObject.profileName : "");
			}
			//reset editable row
			this.tableViewer.refresh();
		},
		
		getCurrentValues: function() {
			var results = [];
			//set current table factor
			this.changeDetailFactors();
			for(var i = 0; i < this.profiles.length; i++) {
				var riskFactorIds = [];
				var likelihoods = [];
				var costs = [];
				var weights = [];
				var dynamicRiskValues = [];
				for(var j = 0; j < (this.profiles[i].riskMeasurements ? this.profiles[i].riskMeasurements.length:0);j++) {
					riskFactorIds.push(this.profiles[i].riskMeasurements[j].assessFactor.id);
					weights.push(this.profiles[i].riskMeasurements[j].weight);
					if(this.profiles[i].riskMeasurements[j].cost) {
						likelihoods.push(this.profiles[i].riskMeasurements[j].likelihood);
						costs.push(this.profiles[i].riskMeasurements[j].cost);
						dynamicRiskValues.push(NONE);
					}
					else {
						likelihoods.push(NONE);
						costs.push(NONE);
						dynamicRiskValues.push(this.profiles[i].riskMeasurements[j].veryHigh+","+
							this.profiles[i].riskMeasurements[j].high+","+
							this.profiles[i].riskMeasurements[j].medium+","+
							this.profiles[i].riskMeasurements[j].low+","+
							this.profiles[i].riskMeasurements[j].veryLow);
					}
				}
				var value = {
					name: this.profiles[i].profileName,
					targetArtifactType: this.profiles[i].targetArtifactType,
					description: this.profiles[i].description,
					riskFactorIds: riskFactorIds,
					likelihoods: likelihoods,
					costs: costs,
					dynamicRiskValues: dynamicRiskValues,
					weights: weights,
					defaultProfile: this.profiles[i].defaultProfile
				}
				if(this.profiles[i].isNew) {
					value.isNew = true;
				}
				else {
					value.isNew = false;
					value.id = this.profiles[i].id;
				}
				results.push(value);
			}
			for(var i = 0; i < this.deleted.length; i++) {
				if (!this.deleted[i].isNew) {
					var value = {
						isNew: false,
						archived: true,
						name: this.deleted[i].profileName,
						targetArtifactType: this.deleted[i].targetArtifactType,
						id: this.deleted[i].id
					}
					results.push(value);
				}
			}
			return results;
		},
		
		onSave: function() {
			var self = this;
			var results = this.getCurrentValues();
			this.successStatus = true;
			this.savedProfilesAmount = 0;
			this.profilesAmount = results.length;
			this.parentEditor.setDirty(false);
			if(this.profilesAmount == 0){
				return;
			}
			
			//set the last selected profile
			var currentSelected = this.tableViewer.getCheckedRows();
			if (currentSelected && currentSelected.length == 1) {
				this.selectedProfileNameBeforeSave = currentSelected[0].profileName;
			}
			else	
				this.selectedProfileNameBeforeSave = null;
			
			this.parentEditor.parent.addSaveParticipantNum();
			for(var i = 0; i < results.length; i++) {
				var handler = {
					success: function(profile){
						if(profile){
							self.successStatus = self.successStatus && true;
							self.savedProfilesAmount++;
						}
						if(self.savedProfilesAmount == self.profilesAmount && self.successStatus == true){
							self.parentEditor.parent.saveFinished();
							self.loadProfiles();
						}else if (self.savedProfilesAmount == self.profilesAmount && self.successStatus == false){
							self.parentEditor.parent.saveFinished();
							self.setErrorMessage(Constants.getMessage("msgReqProFailSaveConn"));
						}
					},
					error: function(e){
						self.successStatus = false;
						self.savedProfilesAmount++;
						if(self.savedProfilesAmount == self.profilesAmount){
							self.setErrorMessage(Constants.getMessage("msgReqProFailSaveConn"));
							self.parentEditor.parent.saveFinished();
						}
					}
				}
				var responseHandler = new ServiceResponseHandler(handler, "success", "error");
				RiskClient.postRiskProfileDTOs(responseHandler, results[i]);
			}
		},
		
		loadProfiles: function(){
			var responseHandler = new RiskProfileMapper(this, "_initialize", "_error");
			RiskClient.getRiskProfileDTOs(responseHandler, {});
		},
		
        _error: function(errorObj){
            alert(errorObj.message);
        }
    });
})();
