/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Load: com.ibm.rqm.risk.web.ui.internal.view.table.RisksTableView");
dojo.provide("com.ibm.rqm.risk.web.ui.internal.view.table.RisksTableView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.risk.web.client.internal.risk.RiskClient");
dojo.require("com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskUtil");
dojo.require("com.ibm.rqm.risk.web.ui.internal.util.RiskMessageUtil");
dojo.require("com.ibm.rqm.risk.web.client.internal.mapper.RiskTypeMapper");

(function(){
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var util = com.ibm.team.workitem.web.client.util;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var RiskClient = com.ibm.rqm.risk.web.client.internal.risk.RiskClient;
	var RiskUtil = com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskUtil;
	var RiskMessageUtil = com.ibm.rqm.risk.web.ui.internal.util.RiskMessageUtil;
	var RiskTypeMapper = com.ibm.rqm.risk.web.client.internal.mapper.RiskTypeMapper;
	var NONE = "None";
	var TECHNICAL = "Technical";
	var BUSINESS = "Business";
	
    dojo.declare("RisksTableView.TableListener", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener, {
        parentView: null,
        
        constructor: function(testView){
            this.parentView = testView;
        },
        
        // Summary:
        //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
        //   called.
        //
        rowCheckStateChanged: function(tableViewer, rowObject, state){
            var selectionCount = tableViewer.getCheckedRows().length;
            if (this.parentView.deleteActionNode) 
                this.parentView.deleteActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable());
        },
		
		 // Summary:
        //    If the client have registered table actions, this message will be fired when an action (button)
        //    is invoked by the user.
        //
        tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback){
            if (callback != null && dojo.isFunction(callback)) {
                if (callbackData != null) {
                    callback(callbackData);
                }
                else {
                    callback();
                }
            }
        }
    });
    
    dojo.declare("RisksTableView.CellModifier", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier, {
        tableViewer: null,
        
        constructor: function(viewer, parent){
            this.tableViewer = viewer;
            this.parent = parent;
        },
        
        cellValueChanged: function(oldValue, newValue, rowObject, columnIndex){
            switch (columnIndex) {
                case 0:
                    rowObject.name = newValue;
                    break;
				case 1: 
					rowObject.riskType = newValue;
					break;
				case 2: 
					rowObject.description  = newValue;	
					break;
                default:
                    break;
            }
			if(this.parent.parentEditor) {
				this.parent.parentEditor.setDirty(true);
			}
        },
        
        cellActionInvoked: function(rowObject, columnIndex){
        }
    });
    
    dojo.declare("RisksTableView.TableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        constructor: function(args) {
			this.editable = args.editable;
			
			//riskType: {itemId, name}
			this.riskTypes = args.riskTypes || [];
    	},
	
		// Summary:
        //
        getNumColumns: function(content){
            return 3;
        },
        
        //If data in any of the columns is required indicate it here.
        //cols is an array of column indices.
        //The required column headings will have an asterisk appended to them ie "Name" becomes "Name *"
        //The asterisk will ONLY be applied if there are rows in the table.
        setRequiredColumns: function(cols){
            this.requiredCols = cols;
        },
        
        // Summary:
        //
        getColumnHeading: function(content, index){
            var headings = [Constants.getMessage("labelRiskName"),  Constants.getMessage("labelRiskTypeNoColon"), Constants.getMessage("labelDescription")];
            //If any of the columns are required then append an * to the column header to indicate the data is required
            if (this.requiredCols) {
                for (var i = 0; i < this.requiredCols.length; i++) 
                    headings[this.requiredCols[i]] += " *";
            }
            return headings[index];
        },
        
        getIsColumnSortable: function(content, atColumnIndex){
            return true;
        },
        
        getColumnWidth: function(content, atColumnIndex){
            var width = 2;
            switch (atColumnIndex) {
                case 0:
                    width = 35;
                    break;
                case 1:
                    width = 25;
                    break;
				case 2:
                    width = 40;
                    break;
            }
            return width;
        },
        
        getIsCellEditable: function(rowData, atColumnIndex){
			if(rowData.isNew)
				return this.editable;
			return false;
        },
        
        // Summary:
        //    The only assumptions the table have on the data that it displays
        //    is that it's an array contents. These are the data content/object that
        //    will be displayed in the table rows.
        //
        getContentObjects: function(content){
            // our content is already an array of objects
            return content;
        },
        
        getRowDataId: function(rowData){
            return rowData.itemId;
        },
        
        // Summary:
        //    Returns the type of the cell to be displayed. By default, the TableViewer
        //    will support a set of cell types. The type of the cell will map to a specific
        //    cell renderer (to be discussed).
        getCellDisplayType: function(rowData, atColumnIndex){
            var type = TableConstants.RENDER_TYPE_EDITABLE_TEXT;
            switch (atColumnIndex) {
				case 1:
					type = TableConstants.RENDER_TYPE_ENUM;
                    break;
            }
            return type;
        },
        
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "data";
            switch (atColumnIndex) {
                case 0:
                    val = rowData.name;
                    break;
				case 1:
					val = rowData.riskType ? rowData.riskType : (this.riskTypes[0] ?  this.riskTypes[0].name : "");
                    break;
                case 2:
                    val = rowData.description;
                    break;
            }
            return val;
        },
		
		getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
            var val = {};
            switch (atColumnIndex) {
                case 1:
                    val['select'] = rowData.riskType;
                    val['choices'] = [];
					for(var i = 0; i < this.riskTypes.length; i++) {
						val['choices'].push({
							'text': this.riskTypes[i].name,
                        	'value': this.riskTypes[i].name
						})
					}
                    break;
            }
            return val;
        }
        
    });
    
    dojo.declare("com.ibm.rqm.risk.web.ui.internal.view.table.RisksTableView", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="domNode" tabindex="-1">' +
		' <div><span dojoAttachPoint="viewTitle"></span></div>' +
		' <div dojoAttachPoint="tableNode"></div>' +
        '</div>',
        
        editable: true,
        printable: false,
        widgetsInTemplate: true,
		factors: [],
		deteletedFactors: [],
		riskTypes: [], //{itemId, name}
		
        postCreate: function(){
            this.inherited("postCreate", arguments);
        },
		
		init: function(args) {
			this.parentEditor = args.parentEditor;
			var responseHandler = new RiskTypeMapper(this, "_initialize", "_error");
			RiskClient.getAllRiskTypeDTOs(responseHandler, {});
		},
        
        _initialize: function(riskTypes){
			this.riskTypes = [];
			this.factors = this._getFactorsFromType(riskTypes);
			
            //create the table viewer
            var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
                //turn off all controls if in print mode 
                allowAdjustSettings: false,
                autoSaveSettings: false,
                tableId: "RQM-RISKS-TABLEVIEWER",
				showFilterControl: true,
				showTableControls: true,
				isCompactMode: true
            });
            tableViewer.setTableRowsCheckable(this.editable, /* isSingleSelect */ false);
			
            dojox.data.dom.removeChildren(this.tableNode);
			this.tableNode.appendChild(tableViewer.domNode);
            this.tableViewer = tableViewer;
            
            
            var contentProvider = new RisksTableView.TableProvider({
				editable: this.editable,
				riskTypes: this.riskTypes
			});
			//The first column (Name) is required			
			contentProvider.setRequiredColumns([0]);
            
            tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelRisksTable")));
            
            this.addActionNode = tableViewer.addAction("addRisk", // action id
             	Constants.getMessage("labelAddRisk"), // label
             	null, // callback data
             	"add-icon-image", // icon class
             	dojo.hitch(this, "_onAddClick")); // callback
			
			this.deleteActionNode = tableViewer.addAction("delete", // action id
            	Constants.getMessage("labelDeleteRisk"), // label
             	null, // callback data
             	"delete-icon-image", // icon class
             	dojo.hitch(this, "_onDeleteActionClick")); // callback
			this.deleteActionNode.setDisabled(true);
			
            tableViewer.setContent(this.factors);
            tableViewer.setContentProvider(contentProvider);
            tableViewer.setTableListener(new RisksTableView.TableListener(this));
            tableViewer.setCellModifier(new RisksTableView.CellModifier(tableViewer, this));
            tableViewer.show();
			
            //if parent section available and not editable, table should not be editable
            tableViewer.setEditable(this.editable);
        },
        
		refresh: function(factors) {
			this.factors = factors || [];
			this.tableViewer.setContent(this.factors);
			this.tableViewer.refresh();
			this._resetActionButtons();
		},
		
        _onDeleteActionClick: function() {
			var dels = this.tableViewer.getCheckedRows();
			if (dels && dels.length > 0) {
                for (var i = 0; i < dels.length; i++) {
					this.deteletedFactors.push(dels[i]);
                    this.tableViewer.deleteItem(dels[i]);
                }
				this.factors = this.tableViewer.getContent();
				this.tableViewer.refresh();
				if(this.parentEditor){
					this.parentEditor.setDirty(true);
				}
            }
			this._resetActionButtons();
		},
		
		_onAddClick: function() {
			var obj = {
				itemId: (new Date()).getTime(),
				isNew: true,
				name: "",
				riskType: this.riskTypes[0] ?  this.riskTypes[0].name : "",
				description: ""
			}
			this.factors.unshift(obj);
			this.refresh(this.factors);
		},
		
		getFactors: function() {
			return this.factors;
		},
		
		_resetActionButtons: function() {
			this.deleteActionNode.setDisabled(true);
		},
		
		_getFactorsFromType: function(types) {
			var factors = [];
			for(var i = 0; i < (types ? types.length : 0); i++) {
				var type = types[i];
				
				//can't add dynamic type
				if(type.name != RiskMessageUtil.getMessage("labelDynamic")) {
					this.riskTypes.push({
						itemId: type.itemId,
						name: type.name
					})
				}
				
				for(var j = 0; j < (type.riskFactors ? type.riskFactors.length : 0); j++) {
					factors.push(type.riskFactors[j]);
				}
			}
			return factors;
		},
		
		_getRiskTypeItemIdByName:function(typeName) {
			for(var i = 0; i < this.riskTypes.length; i++) {
				if(this.riskTypes[i].name == typeName) {
					return this.riskTypes[i].itemId
				}
			}
			return null;
		},
		
		getDeletedItemIds: function() {
			var results = [];
			for(var i = 0; i < this.deteletedFactors.length; i++) {
				if(!this.deteletedFactors[i].isNew) {
					results.push(this.deteletedFactors[i].itemId);
				}
			}
			if(results.length > 0)
				return {factorItemIds: results};
			return null;
		},
		
		getAddedParams: function() {
			var names = [];
			var descriptions = [];
			var riskTypeItemIds = [];
			for (var i = 0; i < (this.factors ? this.factors.length : 0); i++) {
				if(this.factors[i].isNew && dojo.trim(this.factors[i].name) != "") {
					names.push(this.factors[i].name);
					descriptions.push(this.factors[i].description)
					riskTypeItemIds.push(this._getRiskTypeItemIdByName(this.factors[i].riskType));
				}
			}
			if(names.length > 0) {
				return {names: names,descriptions: descriptions,riskTypeItemIds:riskTypeItemIds};
			}
			return null;
		},
		
		onSave: function() {
			var self = this;
			this.successStatus = true;
			this.parentEditor.setDirty(false);
			this.savedRisksAmount = 0;
			this.risksAmount = 0;
			
			var deletedParams = this.getDeletedItemIds();
			if(deletedParams)
				this.risksAmount++;
			var addedParams = this.getAddedParams();
			if(addedParams)
				this.risksAmount++;
				
			if(this.risksAmount == 0){
				return;
			}
			
			this.parentEditor.parent.addSaveParticipantNum();
			var handler = {
				success: function(){
					self.successStatus = self.successStatus && true;
					self.savedRisksAmount++;
					if(self.savedRisksAmount == self.risksAmount && self.successStatus == true){
						self.parentEditor.parent.saveFinished();
						self.loadFactors();
					}else if (self.savedRisksAmount == self.risksAmount && self.successStatus == false){
						self.parentEditor.parent.saveFinished();
						self.setErrorMessage(Constants.getMessage("msgReqProFailSaveConn"));
					}
				},
				error: function(e){
					self.successStatus = false;
					self.savedRisksAmount++;
					if(self.savedRisksAmount == self.risksAmount){
						self.setErrorMessage(Constants.getMessage("msgReqProFailSaveConn"));
						self.parentEditor.parent.saveFinished();
					}
				}
			}
			if(deletedParams) {
				var responseHandler = new ServiceResponseHandler(handler, "success", "error");
				RiskClient.postDeleteRiskFactors(responseHandler, deletedParams);
			}
			if(addedParams) {
				var responseHandler = new ServiceResponseHandler(handler, "success", "error");
				RiskClient.postRiskFactors(responseHandler, addedParams);
			}
		},
		
		loadFactors: function() {
			var responseHandler = new RiskTypeMapper(this, "_initialize", "_error");
			RiskClient.getAllRiskTypeDTOs(responseHandler, {});
		},
		
        _error: function(errorObj){
            alert(errorObj.message);
        }
    });
})();
