//SequenceStepsTableView
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.sequencer.web.ui.internal.view.table.SequenceStepsTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
/*
 * author: vasrivas
 */
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.sequencer.web.ui.internal.Constants");
dojo.require("com.ibm.rational.test.lm.sequencer.web.ui.internal.view.common.StepTypeSelector");

(function() {

var TableConstants 				= com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     				= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var ServiceResponseHandler		= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Constants 					= com.ibm.rational.test.lm.sequencer.web.ui.internal.Constants;
var StepTypeSelector			= com.ibm.rational.test.lm.sequencer.web.ui.internal.view.common.StepTypeSelector;

dojo.declare("SequenceStepsTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    constructor: function(args) {
		dojo.mixin(this, args);
    },
    
   
  rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        var selection = tableViewer.getCheckedRows();
        var selectionCount = selection.length;
		if(selectionCount > 0){
			this.parentView.removeActionNode.setDisabled(false);
			this.parentView.moveDownActionNode.setDisabled(false);
			this.parentView.moveUpActionNode.setDisabled(false);
		}else{
			this.parentView.removeActionNode.setDisabled(true);
			this.parentView.moveDownActionNode.setDisabled(true);
			this.parentView.moveUpActionNode.setDisabled(true);
		}
		//if table view has a rowCheckListener, invoke it
        if (this.rowCheckListener && dojo.isFunction(this.rowCheckListener)) {
			this.rowCheckListener(rowObject, state);
		}
    },
    
    // Summary:
    //    A group check box is shown when the tables rows are checkable and the in multi-select mode.
    //    This method is called when the group's check box is checked/unchecked. The affected rows and 
    //    their states are passed.
    //
    groupRowsCheckStateChanged: function(tableViewer, rowObjects, state) 
    {
        this.rowCheckStateChanged(tableViewer, rowObjects, state);
    },
    
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {                
        if (callback == null && callbackData != null && dojo.isFunction(callbackData)) {
            callbackData();
        }
		
		if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null) {
                callback(callbackData);
            } else {
                callback();
            }
        } 
    },

    tableContentFilterActionInvoked: function(tableViewer, actionId, filters, callback, callbackData)
    {
        if(callback) {
            callback(actionId, filters);
        }                     
    },
	
	// Summary:
    //    If the client have specified a header image and specified that it's an action (see TableContentProvider 
    //    for details), when the column's image is clicked, this message is invoked.
    //
    tableHeaderColumnActionInvoked: function(tableViewer, columnIndex, /*optional*/actionId)
    {
		this.parentView.tableHeaderColumnActionInvoked(columnIndex, actionId);  
    }  
});

dojo.declare("SequenceStepsTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    // Summary:
    //
    getNumColumns: function(content)
    {
        return 5;
    },
    
    // Summary:
    //
    _columnHeadings: [
            Constants.getMessage("labelNumber"),
			Constants.getMessage("labelStep"),
			Constants.getMessage("labelExecutable"),
            Constants.getMessage("labelTarget"),
			Constants.getMessage("labelStatus")
    ],
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        return this._columnHeadings[index];
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return false;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return false;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        switch(atColumnIndex) {
            case 0: 
                return 5;
			case 1: 
                return 20;
            case 2: 
                return 40;
            case 3: 
                return 25;
			case 4: 
				return 10;	
        }
        return 0;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_sequence_steps" + rowData.rowId;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        var type = TableConstants.RENDER_TYPE_TEXT;

		if(atColumnIndex == 4){
			type = TableConstants.RENDER_TYPE_IMAGE;
		}
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return (atColumnIndex != 3) ? TableConstants.ALIGN_LEFT : TableConstants.ALIGN_CENTER ;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "";
        switch(atColumnIndex) {
            case 0: 
                val = parseInt(rowData.rowId)+1;//Since rowId starts from 0 - Change only for display purpose.
                break;
			case 1: 
                val = rowData.stepName;
                break;
            case 2: 
                val = rowData.executable; 
                break;
            case 3: 
                val = rowData.machine;
                break;
			case 4: 
                val = "";
                break;	
        }
        return val;
    },
    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
		if(atColumnIndex == 4){
			var val = {};
			if(rowData.insideCell){
				val.image = ImageCache.ENGINE_AVAILABLE;
				val.tooltip = Constants.getMessage("msgValidMachineSelection");
			}else{
				val.image = ImageCache.ENGINE_NOTRESPONDED;
				val.tooltip = Constants.getMessage("msgMachineNotInCell");
			}
			return val;
		}
		return null;
    }
});

dojo.declare("com.ibm.rational.test.lm.sequencer.web.ui.internal.view.table.SequenceStepsTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
                    '    <div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>' +
                    '  </div>' +
				    '</div>',

    widgetsInTemplate: true,

    constructor: function() {
		this.actions = [];
		this.steps = [];
		this._actionButtons = [];
	 },

    postCreate: function() {
        this.inherited("postCreate", arguments);
    },
	
	initialize: function(){
		var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            showGroupFilterControls: false,
            tableId: "SEQUENCE-STEPS-TABLEVIEWER"                                
        });        
        this._tableViewerNode.appendChild(tableViewer.domNode);

		var steps = this.steps;
		var stepsLength = steps.length;
		for(var i=0;i<stepsLength;i++){
			steps[i].rowId = i;
		}
        var content = steps;

        tableViewer.setContentPageSize(20);
        tableViewer.setContent(content);
        tableViewer.setContentProvider(new SequenceStepsTableView.TableProvider());
        
		var listenerArgs = {
			parentView: this,
			rowCheckListener: dojo.hitch(this, function(rowObject, state) {
	                    this._rowChecked(rowObject, state);  
	                })
		}
		tableViewer.setTableListener(new SequenceStepsTableView.TableListener(listenerArgs));
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", this.queryName));
		tableViewer.setTableRowsCheckable(true, /* isSingleSelect */ true);
		
		this.actions = this.getTableActions();
		var actionsLength = (this.actions ? this.actions.length : 0);
		for (var i = 0; i < actionsLength; i++) {
            var action = this.actions[i];
            var actionButton = tableViewer.addAction(action.actionId, action.label, action.actionHandler, action.iconClass);
            if (action.disabled && action.disabled == true)
                actionButton.setDisabled(true);
            if (action.hidden && action.hidden == true)
                dojo.addClass(actionButton.domNode, "dijitDisplayNone");
			if(action.actionId == "moveup")
				this.moveUpActionNode = actionButton;
			if(action.actionId == "movedown")
				this.moveDownActionNode = actionButton;
			if(action.actionId == "remove")
				this.removeActionNode = actionButton;				
            this._actionButtons.push(actionButton);
        }
		
        tableViewer.show();
		this.tableViewer = tableViewer;
	},
	
	getTableActions: function(){
		var actions = [];
		actions.push({
                    actionId: "add",
                    label: Constants.getMessage("labelAddStep"),
                    actionHandler: dojo.hitch(this, "_handleAdd"),
                    iconClass: "add-icon-image",
                    disabledOnReadOnly: true
                });
		actions.push({
                    actionId: "moveup",
                    label: Constants.getMessage("labelMoveUp"),
                    actionHandler: dojo.hitch(this, "_handleMoveUp"),
                    iconClass: "move-up-image",
                    disabledOnReadOnly: true,
                    disabled: true
                });
		actions.push({
                    actionId: "movedown",
                    label: Constants.getMessage("labelMoveDown"),
                    actionHandler: dojo.hitch(this, "_handleMoveDown"),
                    iconClass: "move-down-image",
                    disabledOnReadOnly: true,
                    disabled: true
                });
		actions.push({
                    actionId: "remove",
                    label: Constants.getMessage("labelRemove"),
                    actionHandler: dojo.hitch(this, "_handleRemove"),
                    iconClass: "delete-icon-image",
                    disabledOnReadOnly: true,
					disabled: true
                });
		return actions;		
	},
	
	_handleRemove: function(){
		var selectedItems = this.tableViewer.getCheckedRows();
		var content = this.getContent();
		if(selectedItems.length == 0){
			return;
		}
		var contentLength = content.length;
		var selectedItemsLength = selectedItems.length;
		for(var i=0;i<selectedItemsLength;i++){
			var updatedContent = [];
			for(var j=0;j<contentLength;j++){
				if(content[j].rowId == selectedItems[i].rowId){
					this.mySection.modifySection(Constants.REMOVE_STEP, selectedItems[i]);
					continue;
				}
				updatedContent.push(content[j]);
			}
			var updatedContentLength = updatedContent.length;
			for(var k=0;k<updatedContentLength;k++){
				updatedContent[k].rowId = updatedContent[k].sectionIndex = k;
			}
			content = updatedContent;//swap updated content so that less traversal is needed.
		}
		this.mySection.setDirty(true);
		this.tableViewer.setContent(content);
		this.tableViewer.refresh();
		if(content.length ==0)
			this.disableAllActions();
	},
	
	_handleMoveUp:function(){
		var selectedItems = this.tableViewer.getCheckedRows();
		var content = this.getContent();
		var selectedItemsLength = selectedItems.length;
		for(var i=0;i<selectedItemsLength;i++){
			var rowId = parseInt(selectedItems[i].rowId,10);
			if (rowId != 0) {
				var rowToSwap = dojo.clone(content[rowId - 1]);
				content[rowId - 1] = content[rowId];
				content[rowId - 1].rowId = rowId -1;
				content[rowId] = rowToSwap;
				content[rowId].rowId = rowId;
				this.tableViewer.rowCheckedHandler.setRowCheckState(content[rowId],false);
				this.tableViewer.rowCheckedHandler.setRowCheckState(content[rowId-1],true);
			} 
		}
		this.mySection.setDirty(true);
		this.tableViewer.refresh();
	},
	
	_handleMoveDown:function(){
		var selectedItems = this.tableViewer.getCheckedRows();
		var content = this.getContent();
		var selectedItemsLength = selectedItems.length;
		for(var i=selectedItemsLength;i>0;i--){
			var rowId = parseInt(selectedItems[i-1].rowId,10); 
			if (rowId != content.length-1) {
				var rowToSwap = dojo.clone(content[rowId + 1]);
				content[rowId + 1] = content[rowId];
				content[rowId + 1].rowId = rowId + 1;
				content[rowId] = rowToSwap;
				content[rowId].rowId = rowId;
				this.tableViewer.rowCheckedHandler.setRowCheckState(content[rowId],false);
				this.tableViewer.rowCheckedHandler.setRowCheckState(content[rowId+1],true);
			}
		}
		this.mySection.setDirty(true);
		this.tableViewer.refresh();
	},

	_handleAdd: function(){
		var args = {
			okCallback: dojo.hitch(this, "_returnedFromDialog")
		}
		var stepSelector = new StepTypeSelector(args);
	},
	
	_returnedFromDialog: function(args){
		this.mySection.setDirty(true);
		if(args.type == Constants.TYPE_AUTOMATION){
			this._addRow(args);//Only single is allowed
		}else{
			//Can have more than one suites/ters selected
			var len = args.attributes.record.length;
			for(var i=0;i<len;i++){
				var arg = {type: args.type, attributes: {type: args.attributes.type,record: args.attributes.record[i]}};
				this._addRow(arg);
			}
		}
	},
	
	_addRow: function(args){
		var content = this.getContent();
		var template = {};
		
		if(args.type == Constants.TYPE_AUTOMATION){
			var attribute = args.attributes.automation;
			var job = attribute.jobs[0];
			var jobName = job ? job.jobName : Constants.getMessage("labelUnassigned");
			var machine =  attribute.machines[0];
			var insideCell = this._validateInsideCell(machine, Constants.TYPE_AUTOMATION);
			
			var machineName = machine ? machine["machine.name"] : Constants.getMessage("labelUnassigned");
			template = {
						id: Constants.AUTOMATION_ID,
						sectionIndex: -1,
						stepName: Constants.getMessage("labelAutomation"),
						executable: jobName, 
						machine: machineName,
						insideCell: insideCell,
						stepDetails: args
					};
		}else{
			var attribute = args.attributes.record;
			var type = args.attributes.type;
			var insideCell = true;//assume machines are there, at present machine info is not available and will be updated by STRD
			template = {
						id: Constants.TEST_RECORD_ID,
						sectionIndex: -1,
						stepName: (type == Constants.MODEL_TYPE_SUITE ? Constants.getMessage("labelTestSuite") : Constants.getMessage("labelTestExecutionRecord")),
						executable: attribute.name,
						machine:Constants.getMessage("labelMultiple"),
						insideCell: insideCell,
						stepDetails: args
					};
		}
		this.mySection.modifySection(Constants.ADD_STEP , template);
		template.rowId = content.length;
		content.push(template);
		this.refresh(content);
	},
	
	_rowChecked: function(rowObject, state){
		this.mySection.rowCheckChanged((state ? rowObject : null));
	},
	
	refresh: function(content){
		if(content){
			var len = content.length;
			for(var i=0;i<len;i++){
				content[i].rowId = i;
			}
			this.tableViewer.setContent(content);
		}
		this.tableViewer.refresh();
	},
	
	getContent: function(){
		return this.tableViewer.getContent();
	},
	
	disableAllActions: function(){
		var actions = this.tableViewer.getActionButtons();
		for(var action in actions){
			if(actions[action].actionId != "add")
				actions[action].setDisabled(true);
		}
	},
	
	_validateInsideCell: function(machine, type){
		var cell = this.mySection.getParentEditor().getCell();
		if(cell){
			if(cell.topologies){
				//if machine lays inside cell return true, else return false.
				var machinesInCell = cell.topologies;
				var machinesInCellLength = machinesInCell.length;
				for(var i=0;i<machinesInCellLength;i++){
					if( type== Constants.TYPE_TEST && machinesInCell[i].resourceId == machine.machineID){
						return true;
					}else if(type== Constants.TYPE_AUTOMATION && machinesInCell[i].resourceId == machine["machine.id"]){
						return true;
					}
				}
			}
			//If no topologies, then unsure of the situation return false		
			return false;
		}
		// if there is no cell associated, then anything is good.
		return true;
	},
	
	_error: function(error){
		console.error("Error in SequenceStepsTableView "+ error.message);
	}
	
});
})();
