/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.table.cellrenderers.ScriptComboCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {
var _textContent=dojo.isIE?"innerText":"textContent"; 

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.table.cellrenderers.ScriptComboCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var node;
		var selectLabel = document.createElement("label");
       
		var contentProvider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
        
        if (editable) {
            node = document.createElement("select");
            
            node.className = "combo-cell";
            
            if (typeof cellWidthHint != "undefined") {
                node.style.width = cellWidthHint + "px";
            }
            
            this.connect(node, "onchange", dojo.hitch(
            	{	'cellModifier': cellModifier,
	            	'rowObject': rowObject,
            		'columnIndex':columnIndex,
            		'lastSelected': null
            	},this._handleSelected));            
        }
        else {
            var node = document.createElement("div");
        }
		
        var tooltip = null;
        if (extendedDisplayInfo && extendedDisplayInfo['choices']) {
			var selected = (typeof extendedDisplayInfo['select'] != 'undefined') ? extendedDisplayInfo['select'] : cellData;
			tooltip = extendedDisplayInfo['tooltip'];
			
            
            if (editable) {
                // select combo box node whenever it is editable or not
                this.setEnumChoices(node, extendedDisplayInfo['choices'], selected);
            }
            else {
                var text = "";
                
                // div text node when not editable
                if (selected) {
                    var choices = extendedDisplayInfo['choices'];
                    // something is selected so get the corresponding text in the choices
                    for (var i = 0; i < choices.length; i++) {
                        if (choices[i].value == selected) {
                            text = choices[i].text;
                            break;
                        }
                    }
                    //Possible choice information is not received so check again and put data
                    if (text == "") 
                        text = (typeof extendedDisplayInfo['defaultText'] != 'undefined') ? extendedDisplayInfo['defaultText'] : "";
                }
                if (viewer.isCompactMode && typeof cellWidthHint != "undefined") {
                    text = this._handleCompactCellText(viewer, text, extendedDisplayInfo, node, cellWidthHint);
                }
                node[_textContent]=text;
            }
		}
        
        // the parent class's attach tooltip will not create a tooltip
        // if the string is null, or undefined. Don't have to check it here.
        this._attachTooltipToCell(node, tooltip);    
        
		selectLabel.appendChild(node);
        return selectLabel;
    },
    
    
    _handleSelected: function(e)
    {
        var node = e.target;
        if(node) {
            var val = null;
            var lastSelected = this.lastSelected;
            var options = node.options;
            var len = options.length;
            
            for(var i = 0; i < len; i++) {
                if(options[i].selected) {
                    val = options[i].value;
                    break;
                }
            }
            this.lastSelected = val;
            var cellModifier = this.cellModifier;
            if(cellModifier) {
                cellModifier.cellValueChanged(lastSelected, val, this.rowObject, this.columnIndex);
            }
        }            
    },

    setEnumChoices: function(select, choices, value)
    {
        var hasSelect = false;
        
        if(choices && choices.length > 0) {
            var length = choices.length;
            for (var i = 0; i < length; i++) {
                var choice = choices[i];
                var option = new Option(choice.text, choice.value);
                select.options[i] = option;
                option.title = choice.text;
                // make this option selected (shown) if a 
                // selectValue is specified and it matches this
                // option.
                if(value && choice.value == value) {
                    option.selected = true;
                    hasSelect = true;
                }    
            }
            // select the first item by default.
            if(!hasSelect)
                select.options[0].selected = true;
        }
    }        
});

})();
