/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.timebox.PlanDetailsDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("dijit.Menu");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient");

(function(){
    var ConfirmationDialog = com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog;
    var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
    var CommonUtility = com.ibm.asq.common.web.ui.internal.util.Utility;
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var TimeboxClient = com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var SelectUtility = dojo.getObject("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.timebox.PlanDetailsDialog", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.process.web", "ui/internal/view/templates/timebox/PlanDetailsDialog.html"),
        
        title: Constants.getMessage("labelExecutionProgressPlanning"),
        labelPlanDatils: Constants.getMessage("labelPlanDetails"),
        
        constructor: function(args){
            this.okCallback = null;
            this.isEditable = null;
            this.startDate = null;
            this.endDate = null;
            this.testPhaseItemId = null;
            this.testPlanId = null;
            this.planDetails = null;
			this.gridHeight = "280px";
			this.contentHeight = "325px";
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            
            if (!this._data) {
                this._data = this.planDetails;
            }
            this._isReadOnly = !this.isEditable;
            
            
            var dialogArgs = {
                primaryTitle: this.title,
                okCallback: dojo.hitch(this, "_onOK"),
                cancelCallback: dojo.hitch(this, "_onCancel"),
                isClosable: false,
                zIndex: 100,
				width: "580px"
            };
            this._dialog = new ConfirmationDialog(dialogArgs);
            this._dialog.contentContainer.appendChild(this.domNode);
            this._dialog.open();
            this._dialog._yesAction = function(){
                if (dojo.isFunction(this.yesCallback)) {
                    this.yesCallback(this.yesCallbackData);
                }
            };
            
            this.initialize();
            
        },
        
        initialize: function(){
            if (this._isReadOnly) {
                //this._initializeTableViewer();
            }
            else {
                this._initilizeGridEditor();
            }
        },
        
        _initilizeGridEditor: function(){
            var data = this._data;
            
            var model = new dojox.grid.data.Table(null, data);
            
            // Layout
            var dateCell = {
                name: Constants.getMessage("labelDate"),
                width: '100px',
                styles: 'text-align: left;'
            };
            var pointsAttemptedCell = {
                name: Constants.getMessage("labelPointsAttempted"),
                width: '100px',
                styles: 'text-align: center;'
            };
            var pointsCompletedCell = {
                name: Constants.getMessage("labelPointsCompleted"),
                width: '100px',
                styles: 'text-align: center;'
            };
            var commentCell = {
                name: Constants.getMessage("labelComment"),
                width: '190px'
            };
            
            
            if (!this._isReadOnly) {
                this.minDate = CommonUtility.dateFromISOString(this.startDate);
                this.maxDate = CommonUtility.dateFromISOString(this.endDate);
                
                var dateFormatLength = com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getDateFormatLength();
                var formatDate = function(inDatum){
                    return dojo.date.locale.format(new Date(inDatum), this.constraint);
                }
                dateCell.editor = dojox.grid.editors.DateTextBox;
                dateCell.formatter = formatDate;
                dateCell.constraint = {
                    formatLength: dateFormatLength,
                    selector: "date",
                    min: this.minDate,
                    max: this.maxDate
                };
                pointsAttemptedCell.editor = pointsCompletedCell.editor = commentCell.editor = dojox.grid.editors.Input;
            }
            
            var gridLayout = [{
                defaultCell: {
                    styles: 'text-align: left;'
                },
                rows: [[dateCell, pointsAttemptedCell, pointsCompletedCell, commentCell]]
            }];
            
            dojox.data.dom.removeChildren(this._gridEditorNode);
            this._grid = new com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer({
                rowsPerPage: 20,
                model: model,
                structure: gridLayout,
                rowCount: "10",
                elasticView: "2",
                defaultStaticRow: [this.minDate, "", "", ""],
                onchange: dojo.hitch(this, "_setDirty"),
                singleClickEdit: true,
				maxHeight: this.gridHeight
            });		
            this._grid.focus.doFocus = function(e){
            
                // trap focus only for grid dom node
                if (e && e.target != e.currentTarget) {
                    return;
                }
                // do not focus for scrolling if grid is about to blur
                if (!this.tabbingOut && this.isNoFocusCell()) {
                    // establish our virtual-focus, if necessary
                    //this.setFocusIndex(0, 0);
                }
                this.tabbingOut = false;
            };
            
            this._grid.addRow = function(inRowData, inIndex){
                this.edit.apply();
                var i = inIndex || -1;
                if (i < 0) {
                    i = this.selection.getFirstSelected() || 0;
                }
                if (i < 0) {
                    i = 0;
                }
                this.model.insert(inRowData, i);
                this.model.beginModifyRow(i);
                // begin editing row
                // FIXME: add to edit
                for (var j = 0, c; ((c = this.getCell(j)) && !c.editor); j++) {
                }
				
            };
            
            var dojoGrid = this._grid.domNode.lastChild;
            
            dojoGrid.childNodes[1].firstChild.style.maxWidth = "100%";
            this._gridEditorNode.appendChild(this._grid.domNode);
			this._gridEditorNode.style.height = this.contentHeight;
            this._grid.render();
            this._grid.initialize();
            
            var arg = {
                label: Constants.getMessage("labelAutoGen"),
                onClick: dojo.hitch(this, "_autoGenPopup")
            };
            arg["iconClass"] = "auto-gen-details-icon";
            arg["showLabel"] = false;
            this._autoGenerationButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(arg);
            
            this._grid.headerRow.appendChild(this._autoGenerationButton.domNode);
			this.connect(this._grid, "cellContentUpdated", this._validateContent);
            
        },
		
		_validateContent: function(e){
			if(this._grid.edit.isEditing()){
				if(this._grid.focus.cell.name == Constants.getMessage("labelPointsCompleted") || this._grid.focus.cell.name == Constants.getMessage("labelPointsAttempted")){
					var value = this._grid.focus.cell.editor.getValue(this._grid.focus.rowIndex);
					value = parseInt(value, 10);
					if (isNaN(value) || value < 0) {
						this._updateErrorMsg(Constants.getMessage("labelOnlyNonnegativeInteger"), false);
					}else{
						this._updateErrorMsg("", true);
					}
				}else{
					this._updateErrorMsg("", true);
				}	
			}
		},
		
		_updateErrorMsg: function(errorMsg, clear){
			if(!this.tableCreated){
				dojox.data.dom.removeChildren(this._errorMessageNode);
	            var table = document.createElement("table");
	            var tbody = document.createElement("tbody");
	            table.appendChild(tbody);
	            var tr = tbody.insertRow(0);
	            var errorImg = document.createElement("img");
	            errorImg.src = ImageCache.NOIMAGE;
	            if (!dojo.isIE) {
	                errorImg.style.marginBottom = "-3.5px";
	            }
	            var imgCell = tr.insertCell(0);
	            imgCell.appendChild(errorImg);
	            var errorMsgNode = document.createTextNode(""); 
				if(!clear){
					errorImg.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/error_obj.gif");
					errorMsgNode = document.createTextNode(errorMsg);
				}
	            this._errorMessageNode.appendChild(errorMsgNode);
	            var msgCell = tr.insertCell(1);
	            msgCell.appendChild(errorMsgNode);
	            this._errorMessageNode.appendChild(table);
				this.tableCreated = true;
				this._errorMsgNode = errorMsgNode;
				this._errorImg = errorImg;
			}else{
				if(clear){
					this._errorImg.src = ImageCache.NOIMAGE;
					this._errorMsgNode.textContent = "";
				}else{
					this._errorImg.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/error_obj.gif");
					this._errorMsgNode.textContent = errorMsg;
				}
			}
		},
		
        _autoGenPopup: function(){
            var dialogArgs = {
                okCallback: dojo.hitch(this, "_handleAutoGen")
            };
            
            this._autoGenDialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            this._autoGenDialog.setTitle(Constants.getMessage("labelAutoGen"));
            this._autoGenDialog.setWidth("330px");
            this._autoGenDialog.getOkButton().disabled = false;
            
            
            var tableContainer = document.createElement("table");
            tableContainer.width = "300px";
            var tbody = document.createElement("tbody");
            tableContainer.appendChild(tbody);
            var unitRow = tbody.insertRow(0);
            
            var unitCell = unitRow.insertCell(0);
            unitCell.width = "50%";
            var unitSelectCell = unitRow.insertCell(1);
            unitCell.appendChild(document.createTextNode(Constants.getMessage("labelUnit")));
            
            this._unitSelect = document.createElement("select");
            var dailyOption  = new Option(Constants.getMessage("labelDaily"), 1);
            var weeklyOption = new Option(Constants.getMessage("labelWeekly"), 7);
            var monthlyOption = new Option(Constants.getMessage("labelMonthly"), 31);
            dailyOption.selected = true;
            SelectUtility.add(this._unitSelect, dailyOption);
            SelectUtility.add(this._unitSelect, weeklyOption);
            SelectUtility.add(this._unitSelect, monthlyOption);
            unitSelectCell.appendChild(this._unitSelect);
            
            var totalPointsRow = tbody.insertRow(1);
            var totalPointsCell = totalPointsRow.insertCell(0);
            var totalPointsValueCell = totalPointsRow.insertCell(1);
            
            totalPointsCell.appendChild(document.createTextNode(Constants.getMessage("labelToatlPoints")));
            this._totalPointsValueInput = document.createElement("input");
            this._totalPointsValueInput.size = "6";
            totalPointsValueCell.appendChild(this._totalPointsValueInput);
            
            this._autoGenDialog.contentContainer.appendChild(tableContainer);
            this._autoGenDialog.open(false);
        },
        
        _handleAutoGen: function(){
            var totalPoints = this._totalPointsValueInput.value == "" ? 0 : this._totalPointsValueInput.value;
            var unit = this._unitSelect.value;
            
            var data = new Array();
            data.push([this.minDate, "0", "0", ""]);
            if ((this.minDate - this.maxDate) != 0) {
                var totalDays = (this.maxDate - this.minDate) / 1000 / 60 / 60 / 24;
                var units = Math.floor(((this.maxDate - this.minDate) / 1000 / 60 / 60 / 24) / unit);
                var detailDay = this.minDate;
                for (var i = 0; i < units; i++) {
                    var detailDay = new Date(detailDay.getTime() + (unit * 24 * 60 * 60 * 1000));
                    var attemptedPoints = this._calculateAutoGenAttemptedPoints(((detailDay - this.minDate) / 1000 / 60 / 60 / 24), totalDays, totalPoints);
                    var completePoints = this._calculateAutoGenCompletePoints(((detailDay - this.minDate) / 1000 / 60 / 60 / 24), totalDays, totalPoints);
                    
                    data.push([detailDay, attemptedPoints, completePoints, ""]);
                }
                if ((((this.maxDate - this.minDate) / 1000 / 60 / 60 / 24) % unit) == 0) {
					data.pop();
                }
				data.push([this.maxDate, totalPoints, totalPoints, ""]);
            }
            this._data = data;
            this._initilizeGridEditor();
        },
        
        _calculateAutoGenAttemptedPoints: function(dayNum, totalDays, totalPoints){
            var percent = (dayNum / totalDays) / ((dayNum / totalDays) + Math.exp(2 - 7.025 * dayNum / totalDays));
            return Math.floor(totalPoints * percent);
        },
        
        _calculateAutoGenCompletePoints: function(dayNum, totalDays, totalPoints){
            var percent = (dayNum / totalDays) / ((dayNum / totalDays) + Math.exp(3 - 8 * dayNum / totalDays));
            return Math.floor(totalPoints * percent);
        },
        
        _getDetailJsonString: function(){
            if (this._grid) {
                var gridContent = this._grid.getAllRows();
                var data = new Array();
                this._totalPoints = gridContent.length > 0 ? gridContent[gridContent.length - 1][1] : 0;
                for (var lineNum = 0; lineNum < gridContent.length; lineNum++) {
                    var dateData = dojo.date.locale.format(gridContent[lineNum][0], {
                        datePattern: "yyyy-MM-dd",
                        timePattern: "hh:mm:ss"
                    });
                    
                    var pointsAttemptedData;
					var pointsAttemptedDataOfLastLine;
                    var pointsCompletedData;
					var pointsCompletedDataOfLastLine;
                    var commentData;
                    for (var colNum = 0; colNum < gridContent[lineNum].length; colNum++) {
                        switch (colNum) {
                            case 1:
								pointsAttemptedDataOfLastLine = (lineNum > 0 && gridContent[lineNum - 1][colNum] != "") ? gridContent[lineNum - 1][colNum].toString() : "0";
                                pointsAttemptedData = gridContent[lineNum][colNum] != "" ? gridContent[lineNum][colNum].toString() : "0";
                                break;
                            case 2:
								pointsCompletedDataOfLastLine = (lineNum > 0 && gridContent[lineNum - 1][colNum] != "") ? gridContent[lineNum - 1][colNum].toString() : "0";
                                pointsCompletedData = gridContent[lineNum][colNum] != "" ? gridContent[lineNum][colNum].toString() : "0";
                                break;
                            case 3:
                                commentData = gridContent[lineNum][colNum];
                        }
                    }
                    data.push({
                        date: dateData,
                        pointsAttempted: pointsAttemptedData,
                        pointsCompleted: pointsCompletedData,
                        comment: commentData
                    });
                    
					var errorMsg = null;
					if (isNaN(pointsAttemptedData) || isNaN(pointsCompletedData)) {
						errorMsg = Constants.getMessage("labelNumberCheck");
					} else if (parseInt(pointsAttemptedDataOfLastLine) > parseInt(pointsAttemptedData) || parseInt(pointsCompletedDataOfLastLine) > parseInt(pointsCompletedData)){
						errorMsg = Constants.getMessage("msgAscendingPoints");
					} else if (parseInt(pointsAttemptedData) < parseInt(pointsCompletedData)){
						errorMsg = Constants.getMessage("msgCompletedPointIsBig");
					}
                    if (errorMsg != null) {
						this._updateErrorMsg(errorMsg, false);
                        return null;
                    }
                }
                return dojo.toJson(data);
            }
        },
        
        _setDirty: function(){
        },
        
        _onOK: function(){
            this._onSave();
        },
        
        _onSave: function(){
            var data = this._getDetailJsonString();
            if (data) {
                var saveArgs = {
                    itemId: this.testPhaseItemId,
                    expectedTotalPoints: this._totalPoints,
                    phaseDetailJsonString: data
                };
				if (dojo.isFunction(this.okCallback)) {
	                this.okCallback(saveArgs);
	            }
				this._dialog.close();
            }
            
        },
        
        _onCancel: function(){
        },
        
        _error: function(e){
            console.error(this.declaredClass + ": error: " + (e ? e.message : ""));
        }
        
    })
})();
