//debug testplantree
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTree");
dojo.require("dijit.Tree");


(function() {

var TestPlanTree = com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTree;
var TestPlanTreeNode = com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTreeNode;

// Summary
//  A subclass of the standard dojo tree for the test plan. Aligns the behavior of the explorer 
//  tree with the normal tree. Adds support for menus to the tree. Uses the ActionRegistry to 
//  support menus. 
	
dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTreeNode",
	dijit._TreeNode,
{
	// Prevent nodes from being higlighted when they are hovered over. Otherwise it's easy to confuse
	// hover with selection
	_onMouseEnter: function(evt){
	},

	_onMouseLeave: function(evt){
	},
	
	// Remove hover higlight when a new node is selected
	// 
	_onNodeFocus: function(evt){
		var oldNode = this.tree.lastFocused;
		if(oldNode)				
			dojo.removeClass(oldNode.contentNode, "dijitTreeNodeHover");
		this.inherited(arguments);						
	}
});

dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTree",
	dijit.Tree,
	// The list of actions that can be performed on given node
{
	actionRegistry: null, 	

	postCreate: function()
    {
        this.inherited('postCreate', arguments);
		this.connect(this.domNode, "oncontextmenu", "_onRightClick"); //conext menu
		this.connect(this.domNode, "ondblclick", "_onDblClick");	  // default action
	},
	
	addToTree: function(item){
		var node = this._itemNodeMap[this.model.getIdentity(item)];
		if(node){
			this._expandNode(node);
		} 		
	},
	
	_onDblClick: function(e)
	{
		var domElement = e.target;		
		// find node
		var nodeWidget = dijit.getEnclosingWidget(domElement);	
		if(!nodeWidget || !nodeWidget.isTreeNode){
			return;
		}
		this.focusNode(nodeWidget);
		
		// process dbl click
		
		this.modelItem = nodeWidget.item;
		var actionDesc = this.actionRegistry.getContextActions(this.model, this.modelItem);		
		var action = this.actionRegistry.getDefaultAction(actionDesc);
		if (action) action.callback(this.modelItem.parent, this.modelItem);		
	},
	
	_onRightClick: function(e)
	{
		var domElement = e.target;

		// find node
		var nodeWidget = dijit.getEnclosingWidget(domElement);	
		if(!nodeWidget || !nodeWidget.isTreeNode){
			return;
		}
		this.focusNode(nodeWidget);
		
		// process selected action, do nothing for non-artifact type nodes
		
		this.modelItem = nodeWidget.item;
		var type = this.modelItem.type;
		if(type == "testplanPagedFolder" || type == "project" || type == "pagedFolder")
			dojo.stopEvent(e);
		else {
			var nGroupLen = "group".length;
			var nLastPos = type.length;
			var group = "";
			if(nLastPos - nGroupLen >= 0)
				group = type.substring(nLastPos - nGroupLen , nLastPos);
			// Make sure it's not a group node type that you clicked on testplangroup, testcasegroup..., these aren't artifacts
			if(group == "Group")
				dojo.stopEvent(e);
			else {
				// Create menu and enable
				this.createMenu();
				// Open menu right click handler
				this.menu._openMyself(e);
				this.menu.onClose = function(){
					this.destroyRecursive();				
				}
				this.menu.onExecute = function(){
					this.destroyRecursive();
				}			
			}
		}
	},	
	
	// Change the behavior so nodes are loaded each time their parent is expanded whether they've 
	// been expanded or not before.
	onOpen: function(item, node)
	{
		node.state = "UNCHECKED";
	},
	
 
	// Remove hover highlight when a new node is selected
	focusNode: function(/* _tree.Node */ node){
		var oldNode = this.lastFocused;
		if(oldNode)				
			dojo.removeClass(oldNode.contentNode, "dijitTreeNodeHover");
		this.inherited(arguments);
	},

	// Execute an action if a right click action is selected
	onContextActionInvoke: function(action, modelitem)
	{
		if(action && action.callback && dojo.isFunction(action.callback)) {
            action.callback(modelitem.parent, modelitem);
        }      				
	},

	// Create the context menu based on the action registry, also setup enablements if they are defiend	
	createMenu: function()
	{
		if(this.actionRegistry) 
		{
			var actionDesc = this.actionRegistry.getContextActions(this.model, this.modelItem);
			var type = this.modelItem.type;
			if(actionDesc) 
			{
				this.actionDesc = this.actionDesc;
				this.menu = this.actionRegistry.getContextMenu(actionDesc, dojo.hitch(this, function(action) 
				{
					// delegate the action handling to our creator.
					this.onContextActionInvoke(action, this.modelItem);
				}));
				this.menu.getChildren().forEach(dojo.hitch(this, function(item){
            			var action = item.action;
            			var enabled = true;
            
            	// get the action's enablement state, if one is specified.
            			if(action && action.enablementCallback && dojo.isFunction(action.enablementCallback)) {
                			enabled = action.enablementCallback(this.model, this.modelItem);
                		item.attr("disabled", !enabled);
					}
				}));
            }
		}
	},
	
	// use hover style has the selection indicator
	onBlur: function(){
		this.inherited(arguments);
		var node = this.lastFocused;				
		if(node && node.contentNode)
			dojo.addClass(node.contentNode, "dijitTreeNodeHover");
	},
	
	_createTreeNode: function(/*Object*/ args){
		return new com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTreeNode(args);
	}
	
	
});


})();
