//debug TestPlanTreeModel
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTreeModel");
dojo.require("ibm_atom.io.atom");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function(){
var Constants      = com.ibm.rqm.planning.web.client.internal.Constants;

// Summary
// Change the behavior 
dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTreeModel",
	null,
{

	_idCount: 0,
	groups: null,
    root: null,
    projectAlias:null,
    atomIO:null,
	nPages: 0,
    pageSize: 0,
    totalSize: 0,
    fetchTestPlanPage: function(pageIndex, onComplete) {console.error("no fetch page function supplied!")},
    
	constructor: function(/* Object */ args){
	
		// summary: passed the arguments listed above (store, etc)
		dojo.mixin(this, args);
    
        if (this.groups==null || this.groups.length==0) {
            throw new Error("Tree model must be provided with group data");       
        }
        
        // atomIO widget to handle communication with RQM REST API
		this.atomIO = new ibm_atom.io.atom.AtomIO();
        
		var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
        this.projectAlias = com.ibm.asq.common.web.ui.internal.util.Utility.getCurrentProjectAreaAlias();
        
        // create the root node
    	this.root = this.newItem({
    		artifact:null,
    		name:projectArea.name,
    		type:"project"},null);
    	
        this._createTestPlans(this.groups, this.root);
        var nPages = this.nPages;
        
        // there are more then 1 page of test plan. Create
        // the folder node for each of the page. Show the item
        // index range as the name of the folder, i.e., '11..20', etc.
        if (nPages > 1) {
            var pageSize = this.pageSize;
            var totalSize = this.totalSize;
            for (var i = 1; i < nPages; i++) {
                var startPos = i * pageSize;
                var span = (i == (nPages - 1)) ? (totalSize - startPos) : pageSize;
                
                 var item = this.newItem({
                    artifact: null,
                    name: (startPos + 1) + ".." + (startPos + span),
                    type: "testplanPagedFolder",
                    mayHaveChildren: true
                }, this.root);
                item.pageIndex = i;
            }
        }  
	},
	
    getTestPlanPagedContent: function(pageItem, onComplete)
    {
        var item = pageItem;
        this.fetchTestPlanPage(item.pageIndex, dojo.hitch(this, function(contents) {
            item.children = [];
            this._createTestPlans(contents, item);
            onComplete(item.children);
        }));
    },
    
    _createTestPlans: function(groups, parent){
        // top level nodes are groups
        if (groups[0].name != null) {
            // groups
            for (var i = 0; i < groups.length; i++) {
                var group = groups[i];
                var testplans = group.objects;
                var item = this.newItem({
                    artifact: group,
                    name: group.name,
                    type: "group"
                }, parent);
            }
        }
        // no groups, top level nodes are test plans
        else {
            testplans = groups[0].objects;
            var plansLen = testplans.length;
            
            for (var i = 0; i < plansLen; i++) {
                var testplan = testplans[i];
                var item = this.newItem({
                    artifact: testplan,
                    name: testplan.name,
                    type: "testplan"
                }, parent);
            }
        }
    },
    
	// create a URL used to query the REST api
	_createQueryUrl: function(resourceType, xpath) {
		// encode spaces in the xpath
		xpath = xpath.replace(/\s+/g, "+");
		// get the project area alias
       	var feedPrefix = net.jazz.ajax._contextRoot + "/service/com.ibm.rqm.integration.service.IIntegrationService/resources/"
       	               + this.projectAlias + "/";
     	var feedSuffix = "?xpath=" + xpath;
       	return feedPrefix + resourceType + feedSuffix;
	},
	
	// create a URL used to get a feed page from the REST api
	_createFeedPageUrl: function(resourceType, token, page) {
		// get the project area alias
       	var feedPrefix = net.jazz.ajax._contextRoot + "/service/com.ibm.rqm.integration.service.IIntegrationService/resources/"
       	               + this.projectAlias + "/";
       	var feedSuffix = "?token=" + token
       	               + "&page=" + page
       	               + "&xpath=*";
       	return feedPrefix + resourceType + feedSuffix;
	},
	
	// get a feed page from the RQM REST API
	_getFeedPage: function(type, token, page, onComplete, parent) {
		var feedURL = this._createFeedPageUrl(type, token, page);
		var mayHaveChildren = this._resourceTypeMayHaveChildren(type);
		
		var callback = dojo.hitch(this, function(feed) {
			var results = [];
			if (feed!=null && feed.entries!=null && feed.entries.length>0) {
				for (var i=0; i<feed.entries.length; i++) {
					var entry = feed.entries[i];
					var artifact = this._resolveArtifact(entry, feedURL);
					if (artifact == null) {
						console.error("invalid feed entry: " + entry);
					} else {
						var item = this.newItem({
		        			artifact:artifact,
		        			mayHaveChildren: mayHaveChildren,
		        			name:entry.title.value,
		        			type:type}, parent);
		        		results.push(item);
					}
				}
			}
			onComplete(results);
		});
		
		var errorCallback = dojo.hitch(this, function(response) {
			var args = {};
			if (response.status == 410) {  // http "GONE" status code
				args.message = Constants.getMessage("msgErrorFeedExpired");
			} else {
				args.message = response.message;
			}
			var dialog = new com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog(args);
			if (response.status == 410) {  // http "GONE" status code
				dialog.setTitle(Constants.getMessage("msgErrorFeedExpiredTitle"));
			} else {
				dialog.setTitle(Constants.getMessage("msgErrorServerFeedTitle"));
			}
			dialog.open();
			onComplete([]);
		});
		
		this.atomIO.getFeed(feedURL, callback, errorCallback);
	},
	
	// send a query to RQM REST API
	_runQuery: function(/*result type*/resourceType, /*xpath query*/xpath, /*function*/onComplete, /*parent item*/parent) {
	
		var queryUrl = this._createQueryUrl(resourceType, xpath);
		var mayHaveChildren = this._resourceTypeMayHaveChildren(resourceType);
		var callback = dojo.hitch(this, function(feed) {
		
			var results = [];
			var totalFeedPages = 1;
			var feedToken;
			var entriesPerPage;
			
			// check for server side paging
			if (feed!=null && feed.links!=null && feed.links.length>0) {
				entriesPerPage = feed.entries == null ? 0 : feed.entries.length;				
				for (var i=0; i<feed.links.length; i++) {
					var link = feed.links[i];
					var href = link.href;
					if (link.rel == "last") {
						var matches = /[\?&]token=([^&]+).*&page=(\d+)/.exec(href);
						if (matches && matches.length>0) {
							feedToken = matches[1];
							totalFeedPages = new Number(matches[2]) + 1;
						}
					}
				}
			}
			
			// feed is not paged, go ahead and display results
			if (totalFeedPages==1 && feed!=null && feed.entries!=null && feed.entries.length>0) {
				for (var i=0; i<feed.entries.length; i++) {
					var entry = feed.entries[i];
					var artifact = this._resolveArtifact(entry, queryUrl);
					if (artifact == null) {
						console.error("invalid feed entry: " + entry);
					} else {
						var item = this.newItem({
		        			artifact:artifact,
		        			mayHaveChildren: mayHaveChildren,
		        			name:entry.title.value,
		        			type:resourceType}, parent);
		        		results.push(item);
					}
				}
                // sort the entries by the item's name.
                results.sort(function(item1, item2) {
                    var s1 = item1.name + "";
                    var s2 = item2.name + "";
                    
                    s1 = s1.toLowerCase();
                    s2 = s2.toLowerCase();
                    return (s1 == s2) ? 0 : ((s1 > s2) ? 1 : -1);
                })
			}
			// feed is paged, create a folder for each page of the results
			else if (totalFeedPages>1) {
				for (var i=0; i<totalFeedPages; i++) {
					var beginIndex = (i * entriesPerPage) + 1;  
					var endIndex = beginIndex + entriesPerPage - 1;
					var folderName = "[" + beginIndex + " .. " + endIndex + "]";
					var artifact = {
						 token:feedToken,
						 page: i,
						 type:resourceType
					};
					var item = this.newItem({
						 artifact: artifact,
						 mayHaveChildren: true,
						 name:folderName,
						 type:"pagedFolder"}, parent);
					results.push(item);
				}
			}
			onComplete(results);
		});
		
		var errorCallback = dojo.hitch(this, function(response) {
			var args = {};
			args.message = response.message;
			var dialog = new com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog(args);
			dialog.setTitle(Constants.getMessage("msgErrorServerFeedTitle"));
			dialog.open();
		});
		
		this.atomIO.getFeed(queryUrl, callback, errorCallback);
	},
	
	_resourceTypeMayHaveChildren: function(resourceType) {
		var mayHaveChildren = false;
        if (resourceType == "testplanPagedFolder") {
            mayHaveChildren = true;
        } else if (resourceType=="testplan") {
			mayHaveChildren=true;
		} else if (resourceType=="testcase") {
			mayHaveChildren=true;
		} else if (resourceType=="executionworkitem") {
			mayHaveChildren=true;
		} else if (resourceType=="executionresult") {
			mayHaveChildren=true;
		}
		return mayHaveChildren;
	},
	
	// create an artifact from a feed entry 
	_resolveArtifact: function(entry, queryUrl) {
		if (entry==null || entry.links==null || entry.links.length<1) {
			return null;
		}
	
		var artifact = {};
		var links = entry.links;
		for (var i=0; i<links.length; i++) {
			var link = links[i];
			if (link.rel == 'alternate') {
				var loc = queryUrl.indexOf("?");
				var prefix = queryUrl.substring(0,loc);
				loc = link.href.indexOf(prefix)
				if (loc > -1) {
					// it's a local link				
					var id = link.href.substring(loc + prefix.length + 1);
					artifact['id'] = id;
				} else {
					// it's a C/ALM link. id not needed.
				}
			}
			else if (link.rel == 'enclosure') {
				artifact['enclosure'] = link.href;
			}
		}
		return artifact;
	},
	
	// recursively destroy a tree item
	destroyRecursive: function(item) {
		if (item==null) return;
		if (item.children) {
			dojo.forEach(item.children,
				function(s){
					if (!s.destroyed) {
						s.destroyed = true;
						this.destroyRecursive(s);
					}
				}, this);
		}
		delete item.artifact;
		delete item.name;
		delete item.type;
		delete item.id;
		delete item.mayHaveChildren;
		delete item.parent;
		delete item.children;
		delete item;
	},
	
	// dijit.tree.Model  API
	destroy: function(){
		this.destroyRecursive(this.root);
	},
	
	// dijit.tree.Model  API
	getRoot: function(onItem){
		onItem(this.root);
	},
	
	// dijit.tree.Model  API
	mayHaveChildren: function(/*dojo.data.Item*/ item){
		return Boolean(item.mayHaveChildren);
	},
	
	// dijit.tree.Model  API
	getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ onComplete){
	
        // parent has no children
        if (!this.mayHaveChildren(parentItem)) {
        	onComplete([]);
        }
        // parent is the root node
		else if (parentItem.type == "project") {
			onComplete(parentItem.children);
		}
        // parent is the root node
        else if (parentItem.type == "testplanPagedFolder") {
            this.getTestPlanPagedContent(parentItem, onComplete);
        }
		// parent is high level grouping construct, usually
		// created by the testplan table view
		// children are testplans from the table view
		else if (parentItem.type == "group") {
			var childItems = [];
        	var group = parentItem.artifact;
        	var testplans = group.objects;
        	// testplans
	        for (var i=0; i<testplans.length; i++) {
	        	var testplan = testplans[i];
	        	var childItem = this.newItem({
	        		artifact:testplan,
	        		name:testplan.name,
	        		type:"testplan"}, parentItem);
	        	childItems.push(childItem);
	        }
	        onComplete(childItems);
		}
		// parent is a paged feed
		// children are the folders corresponding to each page of the feed
		else if (parentItem.type == "pagedFolder") {
			var type = parentItem.artifact.type;
			var token = parentItem.artifact.token;
			var page = parentItem.artifact.page;
			this._getFeedPage(type, token, page, onComplete, parentItem);
		}
        // parent is a testplan
		// children are testcase and requirement grouping nodes
		else if (parentItem.type == "testplan") {
			var childItems = [];
			var item = {};
			if (parentItem.artifact && parentItem.artifact.isMaster) {
				item = this.newItem({
					name:Constants.getMessage('RQM-KEY-TP-CHILDREN-TITLE'),
					type:"childrenGroup"}, parentItem);
				childItems.push(item);				
			}			
			item = this.newItem({
				name:Constants.getMessage('labelTestCases'),
				type:"testcaseGroup"}, parentItem);
			childItems.push(item);
			item = this.newItem({
				name:Constants.getMessage('labelRequirements'),
				type:"requirementGroup"}, parentItem);
			childItems.push(item);
			onComplete(childItems);
		}
        // parent is a testcase grouping node
        // children are the testcases
		else if (parentItem.type == "testcaseGroup") {
			var testplanId = parentItem.parent.artifact.id;
			if (typeof testplanId == 'number') {
				testplanId = "urn:com.ibm.rqm:testplan:" + testplanId;
			}
			var xpath = "/testplan[@rel='usedBy'] and /testplan[@href='"+testplanId+"']";
			this._runQuery("testcase", xpath, onComplete, parentItem);
		}
		// parent is a testplan child grouping node
        // children are the child test plans
		else if (parentItem.type == "childrenGroup") {
			var testplanId = "urn:com.ibm.rqm:testplan:" + parentItem.parent.artifact.id;
			var xpath = "/testplan[@rel='childrenOf'] and /testplan[@href='"+testplanId+"']";
			this._runQuery("testplan", xpath, onComplete, parentItem);
		}		
        // parent is a testcase
		// children are testscript, requirement, and TER grouping nodes
		else if (parentItem.type == "testcase") {
			var childItems = [];
			var item = this.newItem({
				name:Constants.getMessage('labelTestScripts'),
				type:"testscriptGroup"}, parentItem);
			childItems.push(item);
			item = this.newItem({
				name:Constants.getMessage('labelRequirements'),
				type:"requirementGroup"}, parentItem);
			childItems.push(item);
			item = this.newItem({
				name:Constants.getMessage('labelEWIs'),
				type:"executionworkitemGroup"}, parentItem);
			childItems.push(item);
	        onComplete(childItems);
		}
        // parent is a requirement grouping node
		// children are the requirements
		else if (parentItem.type == "requirementGroup") {
			var childItems = [];
			var targetType = parentItem.parent.type;
			var targetId;
			if (targetType=="testplan" && typeof parentItem.parent.artifact.id == 'number') {
				targetId = "urn:com.ibm.rqm:" + targetType + ":" + parentItem.parent.artifact.id;
			} else {
				targetId = parentItem.parent.artifact.id;
			}
			var xpath = "/"+targetType+"[@rel='testedBy'] and /"+targetType+"[@href='"+targetId+"']";
			this._runQuery("requirement", xpath, onComplete, parentItem);
		}
        // parent is a requirement
		// no children
		else if (parentItem.type == "requirement") {
        	onComplete([]);
		}
        // parent is a test script grouping node
		// children are the manual and automated (remote) testscript grouping nodes
		else if (parentItem.type == "testscriptGroup") {
			var childItems = [];
			var item = this.newItem({
				name:Constants.getMessage('labelAutomated'),
				type:"remotescriptGroup"}, parentItem);
			childItems.push(item);
			item = this.newItem({
				name:Constants.getMessage('labelManual'),
				type:"manualscriptGroup"}, parentItem);
			childItems.push(item);
	        onComplete(childItems);
		}
        // parent is a manual test script grouping node
		// children are the manual testscripts
		else if (parentItem.type == "manualscriptGroup") {
			var testcaseId = parentItem.parent.parent.artifact.id;
			var xpath = "/testcase[@rel='usedBy'] and /testcase[@href='"+testcaseId+"']";
			this._runQuery("testscript", xpath, onComplete, parentItem);
		}
        // parent is a remote test script grouping node
		// children are the remote testscripts
		else if (parentItem.type == "remotescriptGroup") {
			var testcaseId = parentItem.parent.parent.artifact.id;
			var xpath = "/testcase[@rel='usedBy'] and /testcase[@href='"+testcaseId+"']";
			this._runQuery("remotescript", xpath, onComplete, parentItem);
		}
		// no children
		else if (parentItem.type == "testscript") {
        	onComplete([]);
		}
		// no children
		else if (parentItem.type == "remotescript") {
        	onComplete([]);
		}
        // parent is an execution record grouping node
		// children are the execution records
		else if (parentItem.type == "executionworkitemGroup") {
			var testcaseId = parentItem.parent.artifact.id;
			var xpath = "/testcase[@rel='executedBy'] and /testcase[@href='"+testcaseId+"']";
			this._runQuery("executionworkitem", xpath, onComplete, parentItem);
		}
        // parent is an execution work item
		// children are executionresultGroup grouping nodes
		else if (parentItem.type == "executionworkitem") {
			var childItems = [];
			var item = this.newItem({
				name:Constants.getMessage('labelExeResults'),
				type:"executionresultGroup"}, parentItem);
			childItems.push(item);
	        onComplete(childItems);
		}
        // parent is an execution result grouping node
		// children are the execution results
		else if (parentItem.type == "executionresultGroup") {
			var ewiId = parentItem.parent.artifact.id;
			var xpath = "/executionworkitem[@rel='producedBy'] and /executionworkitem[@href='"+ewiId+"']";
			this._runQuery("executionresult", xpath, onComplete, parentItem);
		}
        // parent is an execution result
		// children are the work item group
		else if (parentItem.type == "executionresult") {
			var childItems = [];
			var item = this.newItem({
				name:Constants.getMessage('labelDefects'),
				type:"workitemGroup"}, parentItem);
			childItems.push(item);
	        onComplete(childItems);
		}
        // parent is a work item grouping node
		// children are the work items
		else if (parentItem.type == "workitemGroup") {
			var childItems = [];
			var targetType = parentItem.parent.type;
			if (targetType=="executionresult") {
				var targetId = parentItem.parent.artifact.id;
				var xpath = "/"+targetType+"[@rel='affectedBy'] and /"+targetType+"[@href='"+targetId+"']";
				this._runQuery("workitem", xpath, onComplete, parentItem);
			} else {
				// other target types not supported yet
			}
		}
        // parent is a workitem
		// no children
		else if (parentItem.type == "workitem") {
        	onComplete([]);
		}
	},
	
	// dijit.tree.Model  API
	getIdentity: function(/* item */ item){
		return item.id;
	},
	
	// dijit.tree.Model  API
	getLabel: function(/*dojo.data.Item*/ item){
		return item.name;
	},
	
	// dijit.tree.Model  API
	newItem: function(/* Object? */ args, /*Item?*/ parent){
		var mayHaveChildren = true; // default
        if (args.mayHaveChildren!=null) {
        	mayHaveChildren = Boolean(args.mayHaveChildren);
        }
    	var item = {
    		artifact: args.artifact,
    		name: args.name,
    		type: args.type,
    		id: this._idCount++,
    		mayHaveChildren: mayHaveChildren,
    		parent: parent,
    		destroyed: false,
    		children: []
    	};
    	if (parent!=null) {
	    	parent.children.push(item);
    	}
    	return item;
    },
	
	// dijit.tree.Model  API
	pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy){
		// not implemented
	},
	
	// dijit.tree.Model  API
	onChange: function(/*dojo.data.Item*/ item){
		//not implemented
	},
	
	// dijit.tree.Model  API
	onChildrenChange: function(/*dojo.data.Item*/ parent, /*dojo.data.Item[]*/ newChildrenList){
		// not implemented
	},
	
	onDelete: function(item)
	{
		
	}
	
});

})();
	