/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.configurations.AssetDescriptionSelectorDialog");

dojo.require("dijit.Dialog");
dojo.require( "dijit._Widget" );
dojo.require( "dijit._Templated" );
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.FilteredListChooser");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.ConfigSummary");
dojo.require("com.ibm.rational.test.lm.web.client.internal.config");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

( function() {
var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.configurations.AssetDescriptionSelectorDialog",
			 dijit.Dialog,
{	
	postCreate: function()
	{
		this.inherited( "postCreate", arguments );
		var args = {"okcallback": dojo.hitch( this, this.okAction ),
					"cancelcallback": dojo.hitch( this, this.cancelAction )};
		
		//Create the content node
		this.node = new AssetDescriptionSelectorDialog.Content( args );
		
		//Create convenience member variables for accessing content widgets
		this.configSelector = this.node.configSelectorNode;
		this.assetDescSelector = this.node.assetDescriptionList;
		this.summaryNode = this.node.summaryNode;
		
		//Hand selection in the config select control
		this.configSelector.setMultiSelect(false);
		this.configSelector.addChangeListener(dojo.hitch(this, this._onConfigChange));

		//Handle a change in the asset descriptor selector control
		this.connect(this.assetDescSelector, "onchange", this._onAssetDescriptorChange);

		//Show the summary in two columns.  More than that it gets pretty wide
		this.summaryNode.setMaxColumns(2);
		
		//Add the contents to the dialog
		this.containerNode.appendChild( this.node.domNode );
		
		//Initialize other member variables
		this.theCatalog = new com.ibm.asq.common.web.client.internal.Catalog();
		this.selectedPart = null;
	},
	
	_onConfigChange :function()
	{
		var selectedItems = this.configSelector.getSelectedItems();
		if (selectedItems != null)
		{
			var item = selectedItems[0];  //Single select so this should be the ONLY item selected
			//Save the name in case we are asked for it later
			this.selectedConfigName = item.name;
			var id = item.id; //items type should be a "FilteredListChooser.Option" where "id" is the UUID of the config
			if (id != null)
			{
				//Load the item and show its Asset Descriptions in the "AssetDescriptionList" select control.
				this._loadConfigurationById(id);
				this.selectedConfigID = id;
			}
		}	
	},
	
	_loadConfigurationById: function(id)
    {
		//Setup the response handlers and load the config
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadDataSuccess", "_loadDataFailure");
        var configService = new com.ibm.rational.test.lm.web.client.internal.config();
        
		configService.retrieveConfigurationWithDetails(responseHandler, id);
    },
	
	_loadDataSuccess: function(response)
    {
		//Create a config object from the response config string and load the individual asset descriptions into the select control
        this.config = this.theCatalog.createConfigurationFromJson(response.configuration);
		this._showAssetDescriptions(this.config);
    },
	
	_showAssetDescriptions :function(config)
	{
		//Clean out the asset descriptor select control
		this.assetDescSelector.options.length = 0;
		
		//Clean out the list of asset descriptors (AKA parts)
		this.parts = [];
		this.selectedPart = null;
		
		//Get the number of parts in the config
		var partCount = config.getRootParts().length;

		for (var i = 0; i < partCount; i++)
		{
			//Get the part
			var part = config.getRootPart(i);
			//Get the label of the part to show in the select object
			var label = config.getLabel(i);
			
			//If there is no label then initialize it to a default value
			if (label == null || label =="")
				label = RtlmUiUtil_getMessage("cfgNoLabel");
				
			//Create a new "option" to set in the select control
			//Setting the value to "i" will allow us to use that value as the index into the parts array when an asset descriptor is selected
			var option = new Option(label, i);
			this.assetDescSelector.options[i] = option;
			
			//store the part in the parts array for later retrieval
			this.parts[i] = part;
			
			//select the first asset description
			//This is especially useful when a config contains only one asset description.
			//In that case, the user will not have to do anything other than click OK.
			if (i == 0)
			{
				//select the first item in the selector
				this.assetDescSelector.selectedIndex = 0;
				//set the selected part to be the default selected part  
				this.selectedPart = part;
				this.selectedIndex = 0;
				//Initialize the summary display with this part
				this.node.summaryNode.setPart(this.selectedPart);
				this.node.summaryNode.refresh();
				this.selectedPartName = label;
			}
		}
	},
	
	_onAssetDescriptorChange: function()
	{
		//Find the selected asset descriptor
		var options = this.assetDescSelector.options;
        var options_length = options.length;
		for ( var i = 0; i < options_length; i++)
		{
			if (options[i].selected == true)  //the selected one
			{
				//Save the name to return if asked
				this.selectedPartName = options[i].text;
				//Get the index into the parts array.
				//the index will be the nuber we set into the "value" field when the option was created
				var index = options[i].value;
				//set the selected part from the appropriate part
				this.selectedPart = this.parts[index];
				this.selectedIndex = index;
				//update the summary node with the selected part
				this.node.summaryNode.setPart(this.selectedPart);
				this.node.summaryNode.refresh();
				break;  //Single select
			}
		}
	},
    
    _loadDataFailure: function(response)
    {
		//Should probably do something here
    },
	
	//allow the user to specify a callback when OK is clicked
	setOKAction :function(callback)
	{
		this.callback = callback;
	},
	
    //allow the user to specify a callback when Cancel is clicked
    setCancelAction :function(callback)
    {
        this.cancelCallback = callback;
    },
    
	//Add an item to the config select list
	addListItem :function(id, displayValue)
	{
		this.node.configSelectorNode.addOption(id, displayValue);
	},
	
	//Return the selected asset descriptor (Part)
	getSelectedAssetDescriptor: function()
	{
		return this.selectedPart;
	},
	
	//return the index of the selected part
	getSelectedIndex : function()
	{
		return this.selectedIndex;
	},
	
	//return the selected Config ID
	getSelectedConfigID : function()
	{
		return this.selectedConfigID;
	},
	
	getSelectedConfigName : function()
	{
		return this.selectedConfigName;
	},
			
	getSelectedPartName : function()
	{
		return this.selectedPartName;
	},

	//Hide the dialog and call the user supplied callback method if one was specified
	okAction: function( value )
	{
		this.hide();
		if(this.callback)
			this.callback( );
		
	},
	
	//Hide the dialog on cancel
	cancelAction: function( event )
	{
        if (this.cancelCallback)
            this.cancelCallback();
		this.hide();
	},
	
	selectItem : function(itemNumber)
	{
		this.configSelector.selectItem(itemNumber);
		this._onConfigChange();
	}
	
	
});

//This is the content to display in the dialog
dojo.declare("AssetDescriptionSelectorDialog.Content",
			 [dijit._Widget, dijit._Templated],
{
	templateString :	'<div id="${id}" dojoAttachPoint="domNode" tabIndex="-1">' +
						' <table>' +
						'  <tbody>' +
						'   <tr>' +
						'    <td>' +
						'     <div dojoAttachPoint="configSelectorNode" dojoType="com.ibm.rational.test.lm.web.ui.internal.widgets.FilteredListChooser"></div>' +
						'    </td>' +
						'   <tr>' +
						'  <tr> '+
						'    <td>' +
						'      <label><select dojoAttachPoint="assetDescriptionList" name="assetDescriptionList" class="optionSelectWide" size="6"  >' +
					    '      </select></label>' +
						'    </td>' +
						'  </tr>' +
						'  <tr>' +
                        '   <td>' +
                        '      <div dojoAttachPoint="summaryNode" dojoType="com.ibm.rational.test.lm.web.ui.internal.widgets.ConfigSummary"' +
                        '      </div>' +
                        '   </td>'+
						'  </tr>' +
					    '  </tbody>' +
						' </table>' +
						' <div class="assetdesctiption-dialog-button-area" align="center">' +
						' <table dojoAttachPoint="buttonTableNode" border="0">' +
						'  <tr>' +
						'   <td class="action-button-cell">' +
						'    <button dojoAttachPoint="okNode" class="option-action-button" dojoAttachEvent="onclick: _okAction"></button>' +
						'   </td>' +
						'   <td class="action-button-cell">' +
						'    <button dojoAttachPoint="cancelNode" class="option-action-button" dojoAttachEvent="onclick: _cancelAction"></button>' +
						'   </td>' +
						'  </tr>' +
						' </table>' +
						' </div>' +						
						'</div>',
					
	widgetsInTemplate: true,
	okcallback:null,
	cancelcallback:null,
	
	postCreate: function()
	{
		this.inherited( "postCreate", arguments );
		this.okNode.innerHTML = RtlmUiUtil_getMessage("buttonLabelOK");
		this.cancelNode.innerHTML = RtlmUiUtil_getMessage("labelCancelButton");
	},					
						
	_okAction: function( event )
	{
		if( this.okcallback )
			this.okcallback( );
	},
	
	_cancelAction: function( event )
	{
		if( this.cancelcallback )
			this.cancelcallback( );
	}				
});		 
})();
