/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.ConfigurationsTableProvider"); 

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer" );
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.ActionConfigSummary");
dojo.require("com.ibm.rational.test.lm.web.client.internal.config");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

(function() {

	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var ConfigurationUriUtil = com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil;
	var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var column_types = [
						TableConstants.RENDER_TYPE_LINK,
						TableConstants.RENDER_TYPE_TEXT ];
						

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.ConfigurationsTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider,
{
	constructor: function( column_headings, id_fields )
	{
		this.setParameters( { "column_headings" : this.getColumnHeadings(),
							  "id_fields" : this.getIdFields(),
							  "column_types" : column_types } );
	},
	
	getColumnHeadings : function()
	{
		return(  [
	      {title: RtlmUiUtil.getMessage("cfgName"), native_column: "name", isSortable: true, includeInFilter: true, width: 20},
	      {title: RtlmUiUtil.getMessage("cfgSummary"), native_column: "summary", isSortable: true, includeInFilter: true, width: 80}
        ] );
	},
	
	getIdFields: function()
	{
		return( "id" );
	},
    
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
        if(atColumnIndex == 0) {
            val = {};
            val['href'] = ConfigurationUriUtil.getEditConfigurationURI( this.getRowDataId( rowData ) );
        }

        return val;
    }  
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.ConfigurationsListRowRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{
	_config: new com.ibm.rational.test.lm.web.client.internal.config(),
	_configListWidgetMap: {},
	_configRequestMap: {},
	_configSearchActionMap: {},
	_configDetailsMap: {},
	
	_createListTitle: function(viewer, rowObject, listWidget) {
		this.inherited("_createListTitle", arguments);	
		
		this._configListWidgetMap[rowObject.id] = listWidget;	
		
		// New Request action
		listWidget.addAction(RtlmUiUtil.getMessage("createRequestAction"), dojo.hitch(this, "_createRequest"), rowObject.id, "create-request-icon-image");	
		// Search Asset action
		var searchAction = listWidget.addAction(RtlmUiUtil.getMessage("searchAssetAction"), dojo.hitch(this, "_searchAssets"), rowObject.id, "search-icon-image");	
		this._configSearchActionMap[rowObject.id] = searchAction;
	},
	
	_createListContents: function(viewer, rowObject, listWidget){
		this.inherited("_createListContents", arguments);		
		
		var rh = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_getConfigDetailsSucess", "_getConfigDetailsError");
		this._config.retrieveConfigurationWithDetails(rh, rowObject.id);
	},	
	
	_getConfigDetailsSucess: function(response) {
		
		this._configDetailsMap[response.id] = response;
		var listWidget = this._configListWidgetMap[response.id];
		
		// append description to the text content
		if(response.description) {
			var contentTable = listWidget.contentNode.firstChild;
			if (contentTable) {
				var tr = contentTable.insertRow(contentTable.rows.length);
				tr.insertCell(0).innerHTML = "<b>" + RtlmUiUtil.getMessage("cfgDescription") + ":</b>";
				tr.insertCell(1).innerHTML = response.description;
			}
		}
		
		var configSummary = new com.ibm.rational.test.lm.web.ui.internal.widgets.ActionConfigSummary({'configString': response.configuration, 'configID' : response.id});
		listWidget.contentNode.appendChild(configSummary.domNode);
		
		// Search Asset action
		var searchAction = this._configSearchActionMap[response.id];
		if (response.configuration && searchAction) {
		 	var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
			var config = catalog.createConfigurationFromJson(response.configuration);
			var parts = config.getRootParts();
			
			if (parts.length > 1) 
			{
				// If there are more than one parts, create Search action on each part. Remove the action from the list widget.
				configSummary.addPartAction(RtlmUiUtil.getMessage("searchAssetAction"), dojo.hitch(this, "_searchAssets"), "search-icon-image");
				listWidget.removeAction(searchAction);
			}
			else 
				if (parts.length == 1) 
				{
					// If there's only one part, search on this part and save the part with the action button.
					var callbackData = {configID : response.id,
										configPart : parts[0],
					                    partIndex : 0}
					searchAction.callbackData = callbackData;
				}
		}
	},
	
	_getConfigDetailsError: function(response) {
		console.debug(response);
	},
	
	_createRequest: function(configId) {
        var args = null;
        
        // pre-load the new request with the config id.
        if(configId)
            args = {'configId': configId};
		
        var newRequestUri = com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil.getNewRequestURI(args);
        window.location.href = newRequestUri;
	}, 
	
	_searchAssets: function(config) {
		
		//config is the id
		if(dojo.isString(config)) //This will be true if the config object has not been loaded yet (ie we only have the ID)
		{
			var configDetail = this._configDetailsMap[config];
			if(configDetail) 
				this._getConfigDetailsForSearchSuccess(configDetail);
			else 
			{
				var rh = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_getConfigDetailsForSearchSuccess", "_getConfigDetailsError");
				this._config.retrieveConfigurationWithDetails(rh, config);
			}
		} 
		else 
		{ //config is an object containing the config id and the part index to pass to the machine search page.
			var args = {'type' : "Machine",
						'initialConfigID' : config.configID,
						'initialPartIndex' : config.partIndex};
			
			var tempString = AssetUriUtil.getSearchAssetURIWithArgs(args);		
			window.location.href = AssetUriUtil.getSearchAssetURIWithArgs(args);
		}		
	},
	
	_getConfigDetailsForSearchSuccess: function(response) {
		
		var listWidget = this._configListWidgetMap[response.id];
		var searchAction = this._configSearchActionMap[response.id];
		
		if (response.configuration && searchAction) {
		 	var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
			var config = catalog.createConfigurationFromJson(response.configuration);
			var parts = config.getRootParts();
			
			if (parts.length > 1) {
				// If there are more than one parts, Search action should be on each part. Remove from the list widget.
				listWidget.removeAction(searchAction);
				
				// Prompt user which part to search
				this.selectDlg = new com.ibm.rational.test.lm.web.ui.internal.views.configurations.AssetDescriptionSelectorDialog({title: RtlmUiUtil.getMessage("cfgSelectAssetForSearch")});
				this.selectDlg.setOKAction( dojo.hitch( this, this._handleConfigPartSelectOK ) );
				this.selectDlg.addListItem(response.id, response.name);
				//Set the initial selection to the 0th item (actually there is only 1 item so select it)
				this.selectDlg.selectItem(0); 
				this.selectDlg.show();
			}
			else 
				if (parts.length == 1) {
					// If there's only one part, search on this part and save the part with the action button.
					var callbackData = {configID : response.id,
										configPart : parts[0],
					                    partIndex : 0}
					searchAction.callbackData = callbackData;
					this._searchAssets(callbackData);
				}
		}
	},
	
	_handleConfigPartSelectOK: function(){
		var selectedPart = this.selectDlg.getSelectedAssetDescriptor();
		if(selectedPart) 
		{
			var searchData = {configID : this.selectDlg.getSelectedConfigID(),
							  configPart : selectedPart,
							  partIndex : this.selectDlg.getSelectedIndex()}
			this._searchAssets(searchData);
		}
	}
});


})();
