/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.RequestTableProvider"); 

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

(function() {

	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var column_types = [
						TableConstants.RENDER_TYPE_IMAGE,
						TableConstants.RENDER_TYPE_LINK,
						TableConstants.RENDER_TYPE_LINK,
						TableConstants.RENDER_TYPE_TIMESTAMP,
						TableConstants.RENDER_TYPE_TIMESTAMP,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_IMAGE,
						TableConstants.RENDER_TYPE_IMAGE ];
						
    var iconMap = { 'High': ImageCache.PRIORITYHIGH_ICON, 'Normal': ImageCache.PRIORITYMED_ICON, 'Low': ImageCache.PRIORITYLOW_ICON,
					'Assigned': ImageCache.REQT_ASSIGNED, 'Fulfilled': ImageCache.REQT_COMPLETE, 'Draft': ImageCache.REQT_DRAFT, 'Returned': ImageCache.REQT_RETURNED, 
					'Submitted': ImageCache.REQT_SUBMITTED, 'Closed': ImageCache.REQT_CLOSED
				  };


dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.RequestTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider,
{
	constructor: function( column_headings, id_fields )
	{
		this.setParameters( { "column_headings" : this.getColumnHeadings(),
							  "id_fields" : this.getIdFields(),
							  "column_types" : column_types,
							  "iconmap" : iconMap } );
	},
	
	getColumnHeadings : function()
	{
		return( [{title: "", native_column: "none", isSortable: false, includeInFilter: false, width: 2},
					   {title: RtlmUiUtil.getMessage("reqId"), native_column: "visibleId", isSortable: true, includeInFilter: true, width: 10},              
                       {title: RtlmUiUtil.getMessage("reqSummary"), native_column: "summary", isSortable: true, includeInFilter: true, width: 40},              
                       {title: RtlmUiUtil.getMessage("reqCreated"), native_column: "createDate", isSortable: true, includeInFilter: true, width: 14},              
                       {title: RtlmUiUtil.getMessage("reqNeedBy"), native_column: "confirmationDate", isSortable: true, includeInFilter: true, width: 14},    
                       {title: RtlmUiUtil.getMessage("reqAssignedTo"), native_column: "fulfiller.name", isSortable: true, includeInFilter: true, width:10},
                       {title: RtlmUiUtil.getMessage("reqPriority"), native_column: "priority", isSortable: true, includeInFilter: true, width: 5},    
                       {title: RtlmUiUtil.getMessage("reqStatus"), native_column: "currentState", isSortable: true, includeInFilter: true, width: 5}] );
	},
	
	getIdFields: function()
	{
		return( "id" );
	},
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
    	var type = this.inherited("getCellDisplayType", arguments);
    	
    	if( this.column_headings[ atColumnIndex ][ "native_column" ] == "priority" )
    	{
    		var data = this.getCellDisplayData( rowData, atColumnIndex );
    		
    		var img = iconMap[ data ];
    		
    		if( !img )
    			return( TableConstants.RENDER_TYPE_TEXT );
    	}else
		if( this.column_headings[ atColumnIndex ][ "native_column" ] == "currentState" )
		{
			var data = this.getCellDisplayData( rowData, atColumnIndex );
    		
    		var img = iconMap[ data ];
    		
    		if( !img )
    			return( TableConstants.RENDER_TYPE_TEXT );
		}

		return( type );
    },
    
    // Summary:
    //    Returns the display data of the cell. To support type like 'dropdown' select,
    //    additional data would need to be passed as parameters.
    //
    getCellDisplayData: function(rowData, atColumnIndex)
    {
		var data = this.inherited( "getCellDisplayData", arguments );
    	
    	if( this.column_headings[ atColumnIndex ][ "native_column" ] == "priority" )
    	{
    		var type = iconMap[ data ];
    		
    		if( type == undefined )
    		{
    			data = "";
    		}
    	}
        else if( this.column_headings[ atColumnIndex ][ "native_column" ] == "fulfiller.name" && rowData.fulfiller)
        {
            // provide the translated display name for unassigned user.
            if(com.ibm.asq.common.web.ui.internal.util.Utility.isUnassignedUserId(rowData.fulfiller.id)) {
                data = RtlmUiUtil.getMessage("cfgUnassigned");
            }
        }
    	return( ( data ) ? data : "" );
    },
    
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
		var UriUtil = com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil;

		if (atColumnIndex == 0) 
		{
			val = {};
			val.image = ImageCache.REQUEST;
            val['href'] = UriUtil.getEditRequestURI(rowData[this.id_fields]);
		}

    	if( this.column_headings[ atColumnIndex ][ "native_column" ] == "priority" )
        {
            val = {};
            var data = rowData[ this.column_headings[ atColumnIndex ][ "native_column" ] ];
            val['image'] = iconMap[ data ];
            val['tooltip'] = RtlmUiUtil.getMessage(data);
        }
		else if( this.column_headings[ atColumnIndex ][ "native_column" ] == "currentState" )
		{
			val = {};
            var data = rowData[ this.column_headings[ atColumnIndex ][ "native_column" ] ];
			val['image'] = iconMap[ data ];
            val['tooltip'] = RtlmUiUtil.getMessage(data);
		}
        else if(atColumnIndex == 1 ||
        		atColumnIndex == 2 ) {
            val = {};
            val['href'] = UriUtil.getEditRequestURI(rowData[this.id_fields]);
			if(atColumnIndex == 1)
				val['linkType'] = RtlmUiUtil.getMessage("RequestID");	
        }

        return val;
    }       
});

})();
