/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.SimpleMachineSearchTableProvider"); 

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineTableProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

(function() {

	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	
	var column_types = {
		'Machine': [TableConstants.RENDER_TYPE_IMAGE, TableConstants.RENDER_TYPE_LINK, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT  ],
        'ArchiveMachine': [TableConstants.RENDER_TYPE_IMAGE, TableConstants.RENDER_TYPE_LINK, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT ],
		'VirtualImage': [TableConstants.RENDER_TYPE_IMAGE, TableConstants.RENDER_TYPE_LINK, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT  ]
	};
    var SEARCH_ATTRIBUTE_HEADINGS = {
        'Machine' : [
		  {title: "", native_column: "machine\\.name", isSortable: false, includeInFilter: false, width: 5},
		  {title: RtlmUiUtil.getCatalogMessage("pl_name"), native_column: "machine\\.name", isSortable: true, includeInFilter: true, width: 25}, 
          {title: RtlmUiUtil.getCatalogMessage("pl_group"), native_column: "machine\\.group", isSortable: true, includeInFilter: true, width: 15  },
		  {title: RtlmUiUtil.getCatalogMessage("pl_operatingsystem"), native_column: "machine\\.operatingsystem", isSortable: true, includeInFilter: true, width: 20  },
		  {title: RtlmUiUtil.getCatalogMessage("pl_ipAddress"), native_column: "machine\\.ipAddress", isSortable: true, includeInFilter: true, width: 15  },
		  {title: RtlmUiUtil.getMessage("Reserved"), native_column: "Reserved", isSortable: true, includeInFilter: true, width: 20  }
		],
        'ArchiveMachine' : [
		  {title: "", native_column: "machine\\.name", isSortable: false, includeInFilter: false, width: 5},
          {title: RtlmUiUtil.getCatalogMessage("pl_name"), native_column: "machine\\.name", isSortable: true, includeInFilter: true, width: 35}, 
          {title: RtlmUiUtil.getCatalogMessage("pl_operatingsystem"), native_column: "machine\\.operatingsystem", isSortable: true, includeInFilter: true, width: 35  },
          {title: RtlmUiUtil.getCatalogMessage("pl_ipAddress"), native_column: "machine\\.ipAddress", isSortable: true, includeInFilter: true, width: 25  }
        ],
		'VirtualImage' : [
		  {title: "", native_column: "machine\\.name", isSortable: false, includeInFilter: false, width: 5},
          {title: RtlmUiUtil.getMessage("VirtualImage.name"), native_column: "VirtualImage\\.name", isSortable: true, includeInFilter: true, width: 20}, 
          {title: RtlmUiUtil.getCatalogMessage("pl_group"), native_column: "VirtualImage\\.group", isSortable: true, includeInFilter: true, width: 15  },
          {title: RtlmUiUtil.getCatalogMessage("pl_ImageType"), native_column: "VirtualImage\\.ImageType", isSortable: true, includeInFilter: true, width: 15  },
          {title: RtlmUiUtil.getCatalogMessage("pl_operatingsystem"), native_column: "VirtualImage\\.operatingsystem", isSortable: true, includeInFilter: true, width: 20  },
          {title: RtlmUiUtil.getCatalogMessage("pl_VMFile"), native_column: "VirtualImage\\.VMFile", isSortable: true, includeInFilter: true, width: 25  }
		]
    };
    
    var ATTRIBUTE_ID_FIELDS = {
        'Machine' : "machine\\.id",
        'ArchiveMachine' : "machine\\.id",
        'VirtualImage' : "VirtualImage\\.id"
    };
    
    var SEARCH_ATTRIBUTE_CRITERIA = {
        'Machine' : "machine.id, machine.name, machine.group, machine.operatingsystem, machine.ipAddress",
        'ArchiveMachine' : "machine.id, machine.name, machine.operatingsystem, machine.ipAddress",
        'VirtualImage' : "VirtualImage.id, VirtualImage.name, VirtualImage.group, VirtualImage.ImageType, VirtualImage.operatingsystem, VirtualImage.VMFile"
    };
    

com.ibm.rational.test.lm.web.ui.internal.views.providers.SimpleMachineSearchInfo = {
	getSearchAttributeHeadings : function()
	{
		return( SEARCH_ATTRIBUTE_HEADINGS );
	},
	
	getIdFields: function()
	{
		return( ATTRIBUTE_ID_FIELDS );
	},
	
	getSearchAttributeCriteria: function()
	{
		return( SEARCH_ATTRIBUTE_CRITERIA );
	}
};

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.SimpleMachineSearchTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineTableProvider,
{
	constructor: function( assetType )
	{
		this.setParameters( { "column_headings" : SEARCH_ATTRIBUTE_HEADINGS,
					  "id_fields" : ATTRIBUTE_ID_FIELDS,
					  "column_types" : column_types,
					  "column_section" : assetType } );
	},
	
    // Summary:
    //    If the query results are for deleted lab resources, calling this method
    //    will pass in the 'deleted' flag to asset's detail page. The archive state
    //    of the resource is not retrieved as a part of the asset data. The deleted
    //    view will add call this method to pass along the deleted state.
    //
    setAssetStateIsDeleted: function()
    {
        this.assetStateIsDeleted = true;    
    },
    
	getCellDisplayData: function(rowData, atColumnIndex)
    {
		var data = this.inherited( "getCellDisplayData", arguments );
		var headings = this.column_headings[this.columns_section];
    	
    	if (headings[atColumnIndex]["native_column"] == "Reserved") 
		{
			if (this.receiver)
			{
				data = this.receiver._reservedBy(rowData["machine.id"]);
			}
		}
    	
    	return( ( data ) ? data : "" );
    },
	

	
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
		if (atColumnIndex == 0) 
		{
			val = {};
            var assetType = this.columns_section;
 			var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
            var flagArgs = null;
            
            if (this.columns_section == "ArchiveMachine") 
			{
				assetType = "Machine";
				// pass along the asset's state in the url flag.
				if (this.assetStateIsDeleted) 
				{
					flagArgs = {
						'deleted': true
					};
				}
			}
			val.image = assetType == "Machine" ? ImageCache.MACHINE : ImageCache.VIRTUALIMAGE;
            val['href'] = AssetUriUtil.getViewAssetDetailURI( assetType, this.getRowDataId( rowData ), flagArgs );
		}

        if(atColumnIndex == 1) {
            val = {};
            var assetType = this.columns_section;
 			var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
            var flagArgs = null;
            
            if(this.columns_section == "ArchiveMachine") {
                assetType = "Machine";
                
                // pass along the asset's state in the url flag.
                if (this.assetStateIsDeleted) {
                    flagArgs = {
                        'deleted': true
                    };
                }
            }
            val['href'] = AssetUriUtil.getViewAssetDetailURI( assetType, this.getRowDataId( rowData ), flagArgs );
        }

        return val;
    },
	
	getIsColumnGroupable: function(content, atColumnIndex)
    {
		//group by OS
        var assetType = (this.columns_section == "ArchiveMachine") ? "Machine" : this.columns_section;
		return( (assetType == "Machine" && (atColumnIndex == 2 || atColumnIndex == 3)) || 
                (assetType == "VirtualImage" && (atColumnIndex > 1 &&  atColumnIndex < 5)) );    }   
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineSearchListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
	constructor: function( params )
	{
		for( var param in params )
		{
			this[ param ] = params[ param ];
		}
	},
	
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
    	if( this.receiver )
    	{
    		this.receiver._checkButtonStatus();
    	}
    }
});


})();
