/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.viewlets.MyAssetGroupsViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.QueryResultWidget");
dojo.require("com.ibm.rational.test.lm.web.client.internal.user");
dojo.require("com.ibm.asq.common.web.client.internal.admin");
dojo.require("com.ibm.rational.test.lm.web.client.internal.group");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.GroupsTableProvider"); 
dojo.require("com.ibm.team.dashboard.web.ui.IMemento");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.GroupDetailView");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var AbstractViewlet = com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet;

var RESULT_ASSET_COLUMNS = {
    'Machine' : [
      {title: RtlmUiUtil.getMessage("Name"), isSortable: true, includeInFilter: true, width: "25%"}, 
      {title: RtlmUiUtil.getCatalogMessage("pl_Manufacturer"), isSortable: true, includeInFilter: true, width: "25%"  },
      {title: RtlmUiUtil.getMessage("Operating System"), isSortable: true, includeInFilter: true, width: "25%"  },
      {title: RtlmUiUtil.getMessage("Processor Type"), isSortable: true, includeInFilter: true, width: "25%"  }
    ],
    'VirtualImage' : [
      {title: "Image Name", isSortable: true, includeInFilter: true, width: "25%"}, 
      {title: "Image Type", isSortable: true, includeInFilter: true, width: "25%"  },
      {title: "Operating System", isSortable: true, includeInFilter: true, width: "25%"  },
      {title: "VM File", isSortable: true, includeInFilter: true, width: "25%"  }
    
    ]
};

var MACHINE_QUERY_COLUMNS = "machine.id, machine.name, machine.operatingsystem, machine.Manufacturer, machine.assetfamily," +
"VirtualImage.id, VirtualImage.name, VirtualImage.operatingsystem, VirtualImage.ImageType, VirtualImage.assetfamily";

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.viewlets.MyAssetGroupsViewlet",  AbstractViewlet,
{   
    templateString: "<div><div dojoAttachPoint=\"myAssetGroupsViewletContent\" \></div>",
    
    /*
     * Called by the framework after the viewlet is instantiated, but before the
     * first refresh. Viewlets that need to initialize themselves shoudl override.
     */
    init: function() {
        this.myQueryComboWidget = new MyQueryComboWidget( {loadQueryAtCreate: false, loadResultCallback: dojo.hitch(this, '_loadResults')}, this.myAssetGroupsViewletContent );
    },
    
    /*
     * Public Method
     * 
     * Refreshes the viewlet content.
     * 
     * @param <none>
     * @return <none>
     */ 
    refresh: function() {
        this.getSite().setLoading(true);
		this.myQueryComboWidget.loadQueryCombo();
    },
    
    /*
     * Repopulates the viewlet. New content should not be retrieved from the server.
     * Called when presentation of the viewlet is changed in some way, but new information
     * is not required.
     * 
     * Viewlets should override.
     */
    _update: function() {
		this.myQueryComboWidget.loadResults();
    }, 
	
	_loadResults: function() {		
        this._update();
		var site = this.getSite();
		site.setLoading(false);
        site.doneRefresh();
	}
	
    
});

dojo.declare("MyQueryComboWidget", 
    com.ibm.rational.test.lm.web.ui.internal.widgets.QueryResultWidget,
{	
	constructor: function() {
        this.label = RtlmUiUtil.getMessage("labelGroupType");  
        this.tableId = "RQM-LM-ASSET-GROUP-TABLEVIEWLET"; 
    },
	
	postCreate: function() {
		this.inherited("postCreate", arguments);
        
        // don't show the standard empty content message. This is intensional
        // to have an empty content in the table.
        this.machines = [];
		var table = this.resultTable;
        table.setContent(this.machines);
		
		// limits the table's controls in the viewlet. Don't
		// show too much controls.
        table.setContentPageSize(4);
        table.setShowingFilterControl(false);
        table.setShowingPagingRangeControl(false);
        table.show();
        table.showLoadingProgress();
	},
	   
    getTableContentProvider: function()
    {   
        return new MyAssetGroupsViewlet.TableProvider();
    },    
	
    loadQueryCombo: function()
    {
        // get the current user
        var user = new com.ibm.rational.test.lm.web.client.internal.user();
        var responseHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_getCurrentUserSuccess", "_getCurrentUserError");
        user.getCurrent(responseHandler);
    },
	
	_getCurrentUserSuccess: function(response) {
		// get the team areas the current user belongs to
		if(response) {
			var userId = response.userId;
			
	        var admin = new com.ibm.asq.common.web.client.internal.admin();
	        var responseHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_getTeamAreasSuccess", "_getTeamAreasError");
	        var projectArea = dojo.getObject("com.ibm.team.process.web.projectArea");
	        if (projectArea) {
				admin.getTeamAreasForUser(responseHandler, projectArea.itemId, userId);
			} 
			else if(this.loadResultCallback) {
                this.loadResultCallback();
			}
		}
	},
	
	_getCurrentUserError: function(response) {
        console.error(response);
        
		if(this.loadResultCallback)
            this.loadResultCallback();
	},
	
	_getTeamAreasSuccess: function(response) {		
        // get the groups these team areas contain
		if(response && response.length > 0) {
			this.NumOfTeamAreas = response.length;
			this.groups = [];
			var group = new com.ibm.rational.test.lm.web.client.internal.group();
			var responseHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_getGroupsSuccess", "_getGroupsError");
			
			for(var i in response) {
				group.getGroupsInTeamArea(responseHandler, response[i].itemId);
			}
		}
		else if(this.loadResultCallback) //need exit point when there's no team area found for current user.
            this.loadResultCallback();
	},
	
	_getTeamAreasError: function(response) {
		console.error(response);
		
		if(this.loadResultCallback)
            this.loadResultCallback();
	},
	
	_getGroupsSuccess: function(response) {
		this.NumOfTeamAreas --;
		
		// append the team's groups to our list.
		if(response) {
			for (var i = 0; i < response.length; i++) {
				this.groups.push(response[i]);
			}
		}
		if(this.NumOfTeamAreas == 0)
		    this._loadCombo();
	},
	
	_getGroupsError: function(response) {
        console.error(response);        
        this.NumOfTeamAreas --;	
		
		if(this.NumOfTeamAreas == 0 && this.loadResultCallback)
            this.loadResultCallback();	
	},
    
    _loadCombo: function()
    {
		var selectors = [];
		
		for(var i in this.groups) 
        {
			var selector = {name:RtlmUiUtil.getMessage(this.groups[i].name), value:this.groups[i].itemId, def:this.groups[i].itemId == this.defaultQuery}
			selectors.push(selector);
        }
		
		this.resultTable.setTableContentSelectors(selectors);
		this.resultTable.setTableListener(this);
        
		if(!this.defaultQuery && this.resultTable.contentSelect.options.length > 0)
		     this.resultTable.contentSelect.options[0].selected = true;
        
        //Now that the combo box is filled, update the table with the results of the default view.
        this.runSelectedQuery();
    },
    
	//Callback from the table when the view selector is changed
	tableContentSelectionChanged : function(table, selectedId)
	{
		this.runSelectedQuery(selectedId)
	},
	
    runSelectedQuery: function(id)
    {
		this.machines = [];
		
		if (id == null)
			id = this.getQuerySelectionId();
			
		if(id) {			
			var group = new com.ibm.rational.test.lm.web.client.internal.group();          		
	        var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
	        
	        //Search for all assets in the selected Group            
	        var machineResponseHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_getMachinesInGroupSuccess", "_getMachinesError");
		    group.searchForAllAssetsInGroup(machineResponseHandler, id, MACHINE_QUERY_COLUMNS);
		}
		else {
			if(this.loadResultCallback)
	            this.loadResultCallback();
	        else
	        {
	            this.loadResults();
	        }
		}
    },  
	
	_getMachinesInGroupSuccess: function(response) {		
        this.machines = response.Data;
		if(this.loadResultCallback)
		    this.loadResultCallback();
	},
	
    _getMachinesError: function(response) {
		this.machines = [];
		console.error(response);
		if(this.loadResultCallback)
            this.loadResultCallback();
	},
	
	loadResults: function() {
        this.resultTable.noItemMessage = "";
        this.resultTable.endLoadingProgress();
		this.resultTable.setContent(this.machines);
		this.resultTable.refresh();
	}		   
});

// Summary:
//    Override the group machine table provider to disable the 'group by' feature.
//    It's just too crowd in the viewlet.
//
dojo.declare("MyAssetGroupsViewlet.TableProvider", 
             GroupAssetsTable.TableProvider,
{
	getIsColumnGroupable: function(content, atColumnIndex) 
	{
        return false;     
    }
});


})();
