/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.ContributorSelect");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog");

dojo.require("com.ibm.asq.common.web.client.internal.SimpleCache");
dojo.require("com.ibm.asq.common.web.client.internal.ContributorClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function(){
	var _textContent=dojo.isIE?"innerText":"textContent"; 

	var TeamContributorSelectionDialog = com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog;
	var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
		
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var SimpleCache = com.ibm.asq.common.web.client.internal.SimpleCache;
	var ContributorClient = com.ibm.asq.common.web.client.internal.ContributorClient;
	
	var NULL_CONTRIB_ITEMID = "_YNh4MOlsEdq4xpiOKg5hvA"; //sentinel UUID - see DefaultModel.NULL_CONTRIBUTOR_ITEM_ID
	var MORE_ID = "-1";
	
	var RECENT_USERS_CACHE_ID = "com.ibm.asq.common.contributors.recent";
	
	var getMessage = function(msgId) {
		return MessageUtil.getMessage(MessageBundle.COMMON, msgId);
	}
	
	dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.ContributorSelectionDialog", TeamContributorSelectionDialog, {
		runContribSearch: function(params, srh) {
			params.hideGuest = (this.hideGuest === true);
			AdminClient.getContributors(params, srh);
		}
	});
		
    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.ContributorSelect", [dijit._Widget, dijit._Templated], {
        templateString: '<span class="com-ibm-asq-common-web-contributorSelect">' +
        				'   <select style="display: none" dojoAttachPoint="_select" dojoAttachEvent="onchange: _handleSelectChange" class="select"></select>' +
						'   <span style="display:none" dojoAttachPoint="_labelSpan" class="label"></span>' +
        				'</span>',
        
		selectOwnerLabel: getMessage("labelSelectOwner"),
		unassignedLabel: getMessage("labelUnassigned"),
		moreLabel: getMessage("labelMore"),
		
		
        constructor: function(args){
            //accepted params
			this.readOnly = false;
			this.selectSize = 1;
			this.defaultValue = null; //itemId of default user (optional)
			this.allowUnassigned = true; //if false, getSelectedValue will return null if Unassigned
			this.initialSize = 10;
			this.maxSize = 20;
			this.hideGuest = true;
			
			//values are expected to have: itemId, label, 
			this.initialValues = [];
			
			this._UNASSIGNED = {
				itemId: NULL_CONTRIB_ITEMID,
				label: this.unassignedLabel
			};		
			
			this._contributors = [
				this._UNASSIGNED
			];
        },
		
        postCreate: function() {
			this._valueCache = dojo.getObject(RECENT_USERS_CACHE_ID);
			if (this._valueCache == null) {
				this._valueCache = new SimpleCache({
					maxSize: this.maxSize
				});
				dojo.setObject(RECENT_USERS_CACHE_ID, this._valueCache);
				
				//get list of initial values (first X contributors from PA) to populate selection
				var responseHandler = new ServiceResponseHandler(this, "_initContributors", "_error");
				ContributorClient.getMembers(responseHandler, this.initialSize); 
			} else {
				this._initialize();	
			}
        },
		
		_initContributors: function(contributors) {
			contributors = contributors || [];
			var cLen = contributors.length;
			for (var i = 0; i < cLen; i++) {
				var contrib = contributors[i];
				var newContributor = this._fromContribDTO(contrib);
				this._cacheValue(newContributor);			
			}
			this._initialize();
		},
		
		_initialize: function() {
			this._contributors = this._contributors.concat(this._getCachedValues());
			
			if (this.initialValues.length > 0) {
				this._contributors = this._contributors.concat(this.initialValues);
			}
            if (this.readOnly) {
                this._createLabel();
            } else {
                this._createSelect();
            }       

			var itemId = this.defaultValue;
			if (itemId != null) {
				var contrib = this._findIn(this._contributors, itemId, "itemId");
				if (contrib == null) {
					//for initial 'default' selection, may need to fetch the contributor
					var srh = new ServiceResponseHandler(this, "_setInitialValue", "_error");
					ContributorClient.getContributors(srh, {itemId: itemId});
				} else {
					this._handleSelection([contrib]);
					this._initialized = true;	
				}
			}
		},
		
		_setInitialValue: function(contributors) {
			this._handleSelection(contributors);
			this._initialized = true;	
		},
		
        getSelectedValue: function(){
			if (!this.allowUnassigned && this.isUnassigned()) {
				return null;
			}
			var select = this._select;
			var selected = null;
			if (select != null && select.options.length > 0) {
				var itemId = select.options[select.selectedIndex].value;
				selected = this._findIn(this._contributors, itemId, "itemId"); 
			}
			return selected != null ? selected : this._UNASSIGNED;
        },

        setSelectedValue: function(itemId) {
			var selection = this._findIn(this._contributors, itemId, "itemId");
			if (selection != null) {				
				this._updateSelections(selection.itemId);
	            if (this._initialized && this.onValueChanged) {	                
	                this.onValueChanged(selection);
	            }				
			}
        },
				
        getSelectedItemId: function() {
			var selected = this.getSelectedValue();
			return (selected != null ? selected.itemId : null);
        },	
		
		getSelectNode: function(){
			return this._select;
        },
		
		hasSelection: function() {
			var selectedId = this.getSelectedItemId();
			var isUnassigned = selectedId != NULL_CONTRIB_ITEMID;
			return ((!isUnassigned || isUnassigned && this.allowUnassigned) && selectedId != MORE_ID);
		},
		
		isUnassigned: function() {
			var selected = this.getSelectedValue();
			return (!selected || selected.itemId == NULL_CONTRIB_ITEMID);
		},
        
        _createLabel: function(){
            var value = this.getSelectedValue();
            if (value.itemId == NULL_CONTRIB_ITEMID) {
                value.isNullValue = true;
            }
            
            if (!value || value.length == 0 || value.isNullValue) {
                value = this.unassignedLabel;
                return;
            }

            this._labelSpan.style.display = "inline";
            this._labelSpan[_textContent]=value;
        },
		
        _createSelect: function(){
			var selected = this.getSelectedValue();
            var value = this._bindValuesToSelect(this._contributors, selected.itemId, this._select, null, this.moreLabel);

			this._select.style.display = "inline";
            if (this.onSelectCreate) {
                this.onSelectCreate(value);
            }
        },
		
        _updateSelections: function(selectedItemId) {      
			this._sortByLabel(this._contributors);     
			var selected = this.getSelectedValue(); 
            var value = this._bindValuesToSelect(this._contributors, selectedItemId, this._select, null, this.moreLabel);
        },

        _handleSelectChange: function(e){
            this._isChanged = true;
            if (this._select.value != MORE_ID) {
                var selectedValue = this._findIn(this._contributors, this._select.value, "itemId");
				if (this.onValueChanged) {
                    this.onValueChanged(selectedValue);
                }
            } else { // More...
                this.handleMoreSelected(e);
            }
            if(e){
                dojo.stopEvent(e);
            }
        },
		
        handleMoreSelected: function(e){
            this._openOwnerSelectionDialog();
        },
		
        _openOwnerSelectionDialog: function(){
            var self = this;
            var params = {
                title: self.selectOwnerLabel,
                allowMultiple: (self.selectSize > 1),
				hideGuest: self.hideGuest,
                useOkCancel: (self.selectSize == 1) /* Not Add/Close */
            };
			var selectedValue = this.getSelectedValue();
			if (selectedValue.id != MORE_ID) {
				params.disableUsers = [
					{itemId: selectedValue.itemId}
				];
			}
			
            var ownerListWidget = new com.ibm.asq.common.web.ui.internal.widgets.ContributorSelectionDialog(params);
			ownerListWidget.dialog.domNode.style.zIndex = "10003";
            this.connect(ownerListWidget, "onClose", dojo.hitch(this, function(){
                self._select.focus();
            }));
            this.connect(ownerListWidget, "onOK", dojo.hitch(this, function(selectedOwnerList){
                self._handleSelection(selectedOwnerList);
            }));
            ownerListWidget.searchText.focus();
			
            /* for now, reset the selection back, in case the user cancels */
            this._select.selectedIndex = this._select.currentDefaultIndex;
        },
		
        _handleSelection: function(selectedOwnerList){
            if (!selectedOwnerList || selectedOwnerList.length == 0) {
                return;
            }
            
            var found = false;
			var selected = selectedOwnerList[0]; //not supporting multiple select yet
			var options = this._select.options;
			var optLen = options.length;
            for (var i = 0; i < optLen; i++) {
                if (options[i].value == selected.itemId) {
                    options[i].selected = true;
					found = true;
                    break;
                }
            }
            
            if (!found) {
				//cache new value
				var newContributor = this._fromContribDTO(selected);
				this._cacheValue(newContributor);
				this._contributors.push(newContributor); 
			}
			this.setSelectedValue(selected.itemId);                    
        },
		
		_bindValuesToSelect: function(values, selectedValueId, select, image, moreLabel) {
			dojox.data.dom.removeChildren(select);
			var resultValue = null;
			for (var i = 0; i < values.length; i++) {
				var option = new Option(values[i].name || values[i].label, values[i].itemId);
				option.valueObject = values[i];
				if (values[i].title) {
					option.title = values[i].title;
				}
				
				var isDefault = false;
				if (selectedValueId != "") {
					isDefault = selectedValueId && selectedValueId === values[i].itemId;
				} else {
					isDefault = i == 0;
				}
				
				if (isDefault) {
					resultValue = values[i];
					select.currentDefaultIndex = i;
				}
				
				option.selected = isDefault;
				select.options[select.options.length] = option;
			}
			
			if (moreLabel) {
				var option =  new Option(moreLabel, MORE_ID);
				select.options[select.options.length] = option;
				if (values.length == 0) {
					select.currentDefaultIndex = 0;
				}
			}
			return resultValue;
		},			
		
		_cacheValue: function(value) {
			this._valueCache.put(value.itemId, value);
		},
		
		_getCachedValues: function() {
			return this._valueCache.getAll();
		},
		
		_fromContribDTO: function(contribDTO) {
			return {
				//only keep necessary attributes
				itemId: contribDTO.itemId,
                name: contribDTO.name
			}			
		},
		
		_findIn: function(values, value, attributeName) {
			values = values || [];
			for (var i = 0; i < values.length; i++) {
				if (attributeName && values[i][attributeName] == value) {
					return values[i];
				} else if (values[i] == value) {
					return values[i];
				}
			}
			return null;
		},
		
		_sortByLabel: function(items) {
			items = items || [];
	        items.sort(function(item1, item2){
				//Unassigned should always be the first entry in the list
				if (item1.itemId == NULL_CONTRIB_ITEMID) return -1;
				if (item2.itemId == NULL_CONTRIB_ITEMID) return 1;
				//otherwise, sort by the label
	            return (item1.name > item2.name) ? 1 : -1;
	        });			
		},
		
		_error: function(e) {
            console.error(this.declaredClass + ": Error: [" + e.message + "]");
        }
    });
    
})();
