/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.InlineEditor");

dojo.require("dijit.InlineEditBox");
dojo.require("dijit.form.Textarea");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

(function() {

var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

// Customized TextBox widget that support clearing of its value on the first
// event after the flag "clearOnFirstEvent" is set.  This widget is used by the
// InlineEditBox widget.  With the string value being empty in a InlineEditBox,
// once the widget is in focus the prompt string is cleared.  In the case that
// the InlineEditBox widget is programmatically set to focus for convenient work
// flow, it will only show an empty TextBox without any prompt.  This widget
// will display the prompt string as its string value but will be clear on any
// event - mouse click, key press, and blur.
//
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.TextBox",
dijit.form.TextBox,
{
	_scroll: false,
    untouched: true,
    clearOnFirstEvent: false,
    
    postCreate: function(){
        this.inherited('postCreate', arguments);
        this.connect(this.textbox, "onclick", "_clickHandler");
        this.connect(this.textbox, "onkeyup", "_onKeyUp");
    },
    
    _clickHandler: function(e) {
        if (this.clearOnFirstEvent && this.untouched) {
            this.attr("value", "");
        }
        this.untouched = false;
    },

    _onKeyPress: function(e) {
        // restrict text area length if specified
        if (this.clearOnFirstEvent && this.untouched) {
            this.attr("value", "");
        }
        this.untouched = false;
        this.inherited('_onKeyPress', arguments);
    },
	
	_onKeyUp: function(e){
        
         var k = e.keyCode;   
         //ignore Ctr+C : WI27500
        //In some cases, 67+Ctrl is not triggered,  and  17(CTRL) is triggered instead when doing ctrl+C,
        //Ctrl is not triggered on other cases)       
        if(k == 67 && e.ctrlKey == true || k == dojo.keys.CTRL){   //67 = C   
            dojo.stopEvent(e); 
            return;
        } 
        // Note: the _formWidget's _lastValue is not being updated by the
        // dijit text box widget during editing. It's only being
        // updated when setValue is called. Don't use it
        // for active check. 
        //if(this.getValue() != this._lastValue){
            this._valueUpdated(this.attr('value'));
        //}
	},

    _onBlur: function(){
        // our host needs to know that from the onChange callback triggered by
        // our superclass onBlur, whether we're losing focusing. Set a flag 
        // to indicate this.
        this.isOnBlur = true;
        
        if (this.clearOnFirstEvent && this.untouched) {
            this.attr("value", "");
        }
        this.untouched = false;
        this.inherited('_onBlur', arguments);
        this.isOnBlur = false;
    },
	
	//fired when the value is updated after a onkeyup event
	_valueUpdated: function(newValue){
		//
	}
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.InlineEditBox",
dijit.InlineEditBox,
{
    isTextArea: false,

    noValueIcon: ImageCache.EDIT_FIELD_ICON, // src path for no value icon, specify null for no icon
    noValueLabel: "",  // empty string for default
    
    // Change default noValueIndicator to blank so we can check if the consumer
    // specify one or not.  Specifying this will override the no value icon and
    // label attributes above.
    noValueIndicator: "",
    width: "100%",
    editor: "com.ibm.asq.common.web.ui.internal.widgets.TextBox",
    
    minWidthHint: null,
    autoEditWidth: true,    
    onEditCallback: null,             // callback method to invoke on begin of the edit mode
    onEditCallbackData: null,         // callback data pass to the onEditCallback
    onEditCompleteCallback: null,     // callback method to invoke on finish of the edit mode
    onEditCompleteCallbackData: null, // callback data pass to the onEditCompleteCallback
            
    postCreate: function() {
        this.inherited('postCreate', arguments);
        this.setNoValuePrompt(this.noValueIndicator, this.noValueIcon, this.noValueLabel);
        this.setFocusWithPrompt = false;
    },

    // Set the default no value indicator with default to show the edit icon
    // with text prompt instead of just a wingdings character.
    //
    setNoValuePrompt: function(promptHTML, iconSrc, label) {
        if (!promptHTML) {
            // no overall indicator, use icon and label
            this.noValueIcon = iconSrc;
            this.noValueLabel = label;
            if (!label) {
                label = MessageUtil.getMessage(MessageBundle.COMMON, "promptLabel_clickToEdit");
            }
            if (iconSrc) {
                this.noValueIndicator = "<img style='vertical-align:middle' src='" + iconSrc + "'/>&nbsp;<i>" + label + "</i>";
            } else {
                this.noValueIndicator = "<i>" + label + "</i>";
            }
        } else {
            this.noValueIndicator = promptHTML;
        }
        if (this.displayNode) {
            // update the display if it is already displayed wiht old indicator
            this._setDisplayValue(this.value);
        }
    },
    
    // This method is invoked from the _onClick() handler through setTimeout()
    // in order to avoid FF gets upset if you move a node while in an event
    // handler for that node.  However, fast click will trigger the _onClick()
    // handler more than once and there more than one editer text box will
    // be created.  It should check the editing mode and skip if it is already
    // in the editing mode.
    //
    
    //Overwrite the code of creating _InlineEditor, replace Dijit._InlineEditor with com.ibm.asq.common.web.ui.internal.widgets._InlineEditor
    edit: function() {
        if (!this.editing) {
            
            // invoke callback on edit mode start
            if (this.onEditCallback) {
                this.onEditCallback(this.onEditCallbackData);
            }

            // summary: display the editor widget in place of the original (read only) markup
	
			this.editing = true;
	
			var editValue = 
					(this.renderAsHtml ?
					this.value :
					this.value.replace(/\s*\r?\n\s*/g,"").replace(/<br\/?>/gi, "\n").replace(/&gt;/g,">").replace(/&lt;/g,"<").replace(/&amp;/g,"&"));
	        
            // Auto edit width. Make the editor's width to 
            // match at least this dom node's width, or the 
            // the minWidthHint's width. So, when the editor
            // is popup, it will be at least the display text
            // width.
            if (this.autoEditWidth) {
                // maximize the width of the editor to at least match
                // our display node's width.
                var curWidth = this.domNode.offsetWidth - (dojo.isIE ? 10 : 0);
                
                // honor the min width hint if it is provided
                var width = this.minWidthHint || curWidth;
                this.width = width + "px";
            }
        
			// Placeholder for edit widget
			// Put place holder (and eventually editWidget) before the display node so that it's positioned correctly
			// when Calendar dropdown appears, which happens automatically on focus.
			var placeholder = document.createElement("span");
			dojo.place(placeholder, this.domNode, "before");
	
			var ew = this.editWidget = new com.ibm.asq.common.web.ui.internal.widgets._InlineEditor({
				value: dojo.trim(editValue),
				autoSave: this.autoSave,
				buttonSave: this.buttonSave,
				buttonCancel: this.buttonCancel,
				renderAsHtml: this.renderAsHtml,
				editor: this.editor,
				editorParams: this.editorParams,
				style: dojo.getComputedStyle(this.displayNode),
				save: dojo.hitch(this, "save"),
				cancel: dojo.hitch(this, "cancel"),
				width: this.width,
                forcedFocus: this.setFocusWithPrompt,
                fireChangeEvent: dojo.hitch(this, "fireChangeEvent"), // Added
                isTextArea: this.isTextArea                           // Added
			}, placeholder);
	
			// to avoid screen jitter, we first create the editor with position:absolute, visibility:hidden,
			// and then when it's finished rendering, we switch from display mode to editor
			var ews = ew.domNode.style;
			this.displayNode.style.display="none";
			ews.position = "static";
			ews.visibility = "visible";
            
    
            // Replace the display widget with edit widget, leaving them both displayed for a brief time so that
            // focus can be shifted without incident.  (browser may needs some time to render the editor.)
            this.domNode = ew.domNode;
            this.editWidget = ew;
            
            setTimeout(dojo.hitch( this, this.focusTimeout ), 100);

            if (this.setFocusWithPrompt) {
                // consumer is showoing the prompt string in edit mode so set the
                // TextBox widget to clear its content on first event
                this.setFocusWithPrompt = false;
                // set TextBox widget is within the Editor widget of this widget
                if (this.editWidget && this.editWidget.editWidget) {
                    this.editWidget.editWidget.clearOnFirstEvent = true;
                }
            }

            // add icon to the Save and Cancel buttons of the edit widget if it is not auto save
            if (!this.autoSave) {
                if (!dojo.hasClass(this.editWidget.saveButton.iconNode, "save-icon-image")) 
                    dojo.addClass(this.editWidget.saveButton.iconNode, "save-icon-image");
                if (!dojo.hasClass(this.editWidget.cancelButton.iconNode, "delete-icon-image")) 
                    dojo.addClass(this.editWidget.cancelButton.iconNode, "delete-icon-image");
            }
        }
    },

	focusTimeout: function()
	{
		//Sometimes the element is not visible yet, and asking to focus the edit widget causes an error.
		//Thus, catch the error and then try again.
		try
		{
			this.editWidget.focus();
		}
		catch( e )
		{
			//If there is any error ignore it.  It seems for IE the first time a page is created that this
			//widget is used on and focus is set to it as done here, the focus cannot be set even with multiple
			//retries possibly to some contention in IE.  Thus just ignore the error that IE produces when it
			//can't set focus to an item that is not visible yet.
             console.debug("problem setting focus!")
		}
	},
	
    save: function(focus) {
        // invoke callback on edit mode start
        this.inherited('save', arguments);
        if (this.onEditCompleteCallback) {
            this.onEditCompleteCallback(this.onEditCompleteCallbackData);
        }
    },
    
    cancel: function(focus) {
        // invoke callback on edit mode start
        this.inherited('cancel', arguments);
        if (this.onEditCompleteCallback) {
            this.onEditCompleteCallback(this.onEditCompleteCallbackData);
        }
    },
    
    // The display node render text string as HTML so we have to escape "<" and ">" for display.
    //
    _setDisplayValue: function(val) {
        var value = val.replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/\n/g, "<br>");
        com.ibm.asq.common.web.ui.internal.widgets.InlineEditBox.superclass._setValueAttr.apply(this, [value]);
    },
    
    // method to set the widget in focus with the prompt string
    focusWithPrompt: function() {
        // only if the current value is empty and there is a no value label set
        if (this.value == "" && this.noValueLabel && (this.noValueLabel != "")) {
            if (!this.editing) {
                // only if it is not already in edit mode
                this.value = this.noValueLabel;
                // edit is invoked through setTimeout so simply set the flag here
                this.setFocusWithPrompt = true;
                // click to grab the focus and activate the edit mode
                this._onClick();
            }
        }
    },
	
	//This method is fired when the input area value is updated after a keyup event. This method is different from onChange method. 
	//onChange method is fired by onblur event. onChange method will replace the editable area by uneditable label,
	//this is not as expected that the text area should still be editable.
	fireChangeEvent: function(newValue){
		//
	}
});

dojo.declare(
	"com.ibm.asq.common.web.ui.internal.widgets._InlineEditor",
	 [dijit._InlineEditor],
{
    templateString: '<span dojoAttachPoint="editNode" waiRole="presentation" style="position: absolute; ' +
                          (this.isTextArea ? 'dojoAttachEvent="onkeypress: _onKeyPress" ' : '') +
                          'visibility:hidden" class="dijitReset dijitInline"> ' +
                       '<span dojoAttachPoint="editorPlaceholder"></span> ' +
                       '<span dojoAttachPoint="buttonContainer"> ' +
                          '<button class="saveButton" dojoAttachPoint="saveButton" dojoType="dijit.form.Button" ' +
                                  'dojoAttachEvent="onClick:save" disabled="true" label="${buttonSave}"> ' +
                          '</button> ' +
                          '<button class="cancelButton" dojoAttachPoint="cancelButton" dojoType="dijit.form.Button" ' +
                                  'dojoAttachEvent="onClick:cancel" label="${buttonCancel}"> ' +
                          '</button> ' +
                       '</span> ' +
                    '</span>',

    // For text box, there is key press event handler to handle key such as enter and escape to exit the editing.
    // For text area, we can not use the same event handler such as enter would start a new line and not exiting
    // the editing.  The text area has its own key press event handler and this cause a memory leak when another
    // handler is connect to the same event and leave the old handler dangling.
    //
    isTextArea: false,
    
	postCreate: function(){
        // Create edit widget in place in the template
        var cls = dojo.getObject(this.editor);

        // Copy the style from the source
        // Don't copy ALL properties though, just the necessary/applicable ones
        var srcStyle = this.style;
        var editStyle = "line-height:" + srcStyle.lineHeight + ";";
        dojo.forEach(["Weight","Family","Size","Style"], function(prop){
            editStyle += "font-"+prop+":"+srcStyle["font"+prop]+";";
        }, this);
        dojo.forEach(["marginTop","marginBottom","marginLeft", "marginRight"], function(prop){
            this.domNode.style[prop] = srcStyle[prop];
        }, this);
        if(this.width=="100%"){
            // block mode
            editStyle += "width:100%;";
            this.domNode.style.display = "block";
        }else{
            // inline-block mode
            editStyle += "width:" + (this.width + (Number(this.width)==this.width ? "px" : "")) + ";";
        }
        this.editorParams.style = editStyle;
        this.editorParams[ "displayedValue" in cls.prototype ? "displayedValue" : "value"]= this.value;
        var ew = this.editWidget = new cls(this.editorParams, this.editorPlaceholder);

        this.connect(ew, "onChange", "_onChange");

        if (!this.isTextArea) {
            // Monitor keypress on the edit widget.   Note that edit widgets do a stopEvent() on ESC key (to
            // prevent Dialog from closing when the user just wants to revert the value in the edit widget),
            // so this is the only way we can see the key press event.
            this.connect(ew, "onKeyPress", "_onKeyPress");
        }
        
        if(this.autoSave){
            this.buttonContainer.style.display="none";
        }

        // if we're been programatically set to focus, ignore the next change.
        // the edit box would clear its value for the editor, that would trigger
        // an onChange. Need to ignore it.
        if(this.forcedFocus) {
            this._ignoreNextOnChange = true;
            delete this.forcedFocus;
        }
        // get a more fine-grind event of the edit box been changed. This is hooked
        // to the keyup event.
		this.connect(this.editWidget, "_valueUpdated", "_fireChangeEventOfParent");

        this.domNode.style.display = "inline";
	},

    _onChange: function(){
        // summary:
        //  Override our superclass implementation to ensure that we don't
        //  unneccessarily set focus back to ourself.
        //
        //  Called when the underlying widget fires an onChange event,
        //  which means that the user has finished entering the value
        if(this._ignoreNextOnChange){
            delete this._ignoreNextOnChange;
            return;
        }
        this.inherited("_onChange", arguments);
    },
    
	_fireChangeEventOfParent: function(newValue){
		this.fireChangeEvent(newValue);
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.Textarea",
dijit.form.Textarea,
{
     // maximum number of characters
     //
    maxTextAreaLength: 0,
        
    postCreate: function() {
        this.inherited('postCreate', arguments);
        if (this.eventNode) {
            this.connect(this.eventNode, "onkeyup", "_onKeyUp");
        };
    },
        
    // Must also remove newline, tab, and leading and trailing white space for the inline
    // editor to put up no value indicator properly.
    getValue: function() {
        var value = this.inherited('getValue', arguments);
        value = value.replace(/\t/g,"   ");
        return dojo.trim(value);
    },

    _onKeyPress: function(e) {
        // restrict text area length if specified
        if (this.maxTextAreaLength && (this.attr("value").length >= this.maxTextAreaLength)
            // delete keys and navigation keys are always allowed
            && (e.keyCode != dojo.keys.DELETE)
            && (e.keyCode != dojo.keys.BACKSPACE)
            && (e.keyCode != dojo.keys.LEFT_ARROW)
            && (e.keyCode != dojo.keys.RIGHT_ARROW)
            && (e.keyCode != dojo.keys.UP_ARROW)
            && (e.keyCode != dojo.keys.DOWN_ARROW)
            && (e.keyCode != dojo.keys.PAGE_UP)
            && (e.keyCode != dojo.keys.PAGE_DOWN)
            && (e.keyCode != dojo.keys.HOME)
            && (e.keyCode != dojo.keys.END)) {
            dojo.stopEvent(e);
            return;
        }

        // Esc key messed up the InlineEditBox, skip it.
        if (e.keyCode != dojo.keys.ESCAPE) {
            this.inherited('_onKeyPress', arguments);
        }
    },

    _onKeyUp: function(e) {
        // Note: the _formWidget's _lastValue is not being updated by the
        // widget during editing. It's only being updated when setValue is
        // called. Don't use it for active check. 
        this._valueUpdated(this.getValue());
    },
    
    //fired when the value is updated after a onkeyup event
    _valueUpdated: function(newValue){
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.InlineEditArea",
com.ibm.asq.common.web.ui.internal.widgets.InlineEditBox,
{
    // Can't just use textarea as the editor widget for InlineEditBox because the
    // textarea widget changed event trigger premature exiting of the edit node.
    //
    editor: "com.ibm.asq.common.web.ui.internal.widgets.Textarea",
    isTextArea: true,

    // track when to exit edit mode which is on blur
    _exitEdit: false,
    
    // track if a blur is triggered from canceling
    _blurFromCancel: false,
    
    _overrideBlurHandler: null,
         
    // On entering the edit mode, we need to connect the on blur event handler to
    // the edit widget.  On blur is the time to exit the edit mode.
    //
    edit: function() {
        this.inherited('edit', arguments);
        
        // connect the handler to onBlur of the edit widget only if auto save is used
        if (this.editWidget && this.autoSave) {
            this.editWidget.editWidget.attr("value", this.value);
            this._overrideBlurHandler = this.connect(this.editWidget, "_onBlur", "_onBlurHandler");
        }
    },

    // Save should not be invoked unless it is exiting the edit mode or auto save is off.
    //
    save: function(focus) {
        if (!this.autoSave || this._exitEdit) {
            this.inherited('save', arguments);
            this._exitEdit = false;
        }
    },
	
	//explicitly perform a save, regardless of the autoSave option
	forceSave: function(focus) {
		this.inherited('save', arguments);
	},

    cancel: function(focus){
        this.inherited('cancel', arguments);
        this._blurFromCancel = true;
    },

    _showText: function(/*Boolean*/ focus) {
        this.inherited(arguments);
        if (this._overrideBlurHandler) {
            // Safety net for in case any blur event is optimized
            // away by IE, disconnect the blur handler here.
            this.disconnect(this._overrideBlurHandler);
            delete this._overrideBlurHandler;
//            console.debug("_showText disconnect _overrideBlurHandler");
        }
    },
    

    // Our handler for on blur event which is the only case to exit
    // the edit mode.  Seems like IE sometimes optimizes and skips
    // the blur event if fast clicking in and out of the edit box.
    _onBlurHandler: function(e) {
        if (this.editWidget && this.autoSave) {
            if (this._overrideBlurHandler) {
                // disconnect the handler, edit will connect
                this.disconnect(this._overrideBlurHandler);
                delete this._overrideBlurHandler;
//                console.debug("_onBlurHandler disconnect _overrideBlurHandler");
            }

            if (this._blurFromCancel) {
                // blur is from cancel - no more work
                this._blurFromCancel = false;
            } else {
                // the underlying edit widget would need to save the changes too
                // when it receives a blur event. Since we added this blur handler after,
                // this method would called before the edit widget's. Need to yield to let 
                // the edit widget to process its blur event before we get the content 
                // from it. Otherwise, we would lost the last entered character in 
                // multi-byte languages.
                setTimeout(dojo.hitch(this, function(){
                    // set flag to allow save and exiting edit mode
                    this._exitEdit = true;
                    this.save(false);
                }), 0);
            }
        }
    }

});

// Summary: 
//    A simple text area that acts like an inline edit area. 
//
/**
 * Note: This widget is a raw copy of the Text area implemented in ViewHeaderLayout, so that widget should be removed and use this instead
 */ 
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets._InlineEditArea",
              com.ibm.asq.common.web.ui.internal.widgets.Textarea,
{
    noValueLabel: null,
    _scroll: false, // disable onfocus to enable scrolling, _formWidget's attribute.
    _width: "70%",
    
    postCreate: function() {
        this.inherited('postCreate', arguments);
        this.inEdit = false;
        
        if(this.noValueLabel) {
            this.setNoValuePrompt(this.noValueLabel);
        }
        var node = this.domNode;        
        this.domNode.style.width = this._width;
        this.setStyleBorder(false);      
        this.connect(node, "onmouseover", "onMouseOver")
        this.connect(node, "onmouseout", "onMouseOut")
        this.attr("multiLine",false);
        this.rows = 1;
        this.cols=1;
    },
    
    setStyleBorder: function(border /* boolean*/)
    {
      dojo.style(this.domNode,
      {
          "borderColor": border?"#B3B3B3":"transparent"
      });  
    },
   
     setStylePrompt: function(sprompt /* boolean*/)
    {
      dojo.style(this.domNode,
      {
          "fontStyle": sprompt?"italic":"normal"
      });  
    },
    
    setStyleHighlight: function(highlight /*Boolean*/)
    {
           dojo.style(this.domNode,{
            "backgroundColor": highlight?"#E2EBF2":"transparent",
            "cursor": highlight?"pointer":"auto"}); 
    },
    
    setNoValuePrompt: function(label) 
    {
        var oldPrompt = this.noValueLabel;
        var value = this.attr('value');
        value = dojo.trim(value);
        
        this.noValueLabel = null;
        
        if(label && label.length > 0) {
            label = dojo.trim(label);
            this.noValueLabel = label.replace(/\s*\r?\n\s*/g,"").replace(/<br\/?>/gi, "\n").replace(/&gt;/g,">").replace(/&lt;/g,"<").replace(/&amp;/g,"&");
        } 
        
        // insert the prompt into the field to display, if we don't have a value
        // or it was the old prompt.
        if((!value || value.length == 0) || (oldPrompt && value == oldPrompt)) {
            this.setStylePrompt(true);         
            this.attr('value', this.noValueLabel || "");
        }
    },
    
    focusHighlightPrompt: function()
    {
        setTimeout(dojo.hitch(this, "focus"), 0);
    },
    
    setValue: function(value)
    {
        var val = value ? dojo.trim(value) : null;
        
        // only load the value into the field if it's a real value.
        // otherwise, keep the prompt string as the current value
        // (if one is set). 
        if (val) {
            this.attr('value', val);
            this.setStylePrompt(false); 
        }
        else {
            this.clearValue();
        }
    },
    
    clearValue: function()
    {
        this.attr('value', "");
        this.setNoValuePrompt(this.noValueLabel);
    },
    
    getValue: function()
    {
        var val = this.inherited(arguments);
        
        if(this.noValueLabel && val == this.noValueLabel) {
            return "";
        }
        return val;
    },
    
    onFocus: function()
    {
        var val = dojo.trim(this.attr('value'));        
       this.setStyleBorder(true);
       this.setStyleHighlight(false);
       this.setStylePrompt(false);
        // clear the prompt if we're showing it.
        if(this.noValueLabel && val == this.noValueLabel) {
            this.attr('value', "");
        } 
        this.inEdit = true; 
        this.focus();          
    },

    onBlur: function()
    {
        var val = dojo.trim(this.attr('value'));        
       this.setStyleBorder(false); 
        
        // no editing, or it's the prompt string. set the prompt style
        if(this.noValueLabel && (!val || val.length == 0) || (val == this.noValueLabel)) {
            this.attr('value', this.noValueLabel);
            this.setStylePrompt(true);
        }        
        this.inEdit = false;  
        this.onMouseOut();   
    },

    
    onMouseOut: function(event)
    {   
        this.setStyleHighlight(false);
    },
    
    onMouseOver: function(event)
    {
        if(!this.inEdit) {
           this.setStyleHighlight(true);  
        }
    },
    
    //@overridden
    _valueUpdated: function(value)
    {        
        this.valueChanged(value);
    },
    
    valueChanged: function(newValue)
    {
        // Summary: notify external client of the value changed.
    }              
});



})();

