/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// author: Paul Beaulieu
//
// Summary: This is a widget to choose options.
//

dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.OptionChooser");

dojo.require( "dijit._Widget" );
dojo.require( "dijit._Templated" );

dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function() { // module container

var Constants = com.ibm.asq.common.web.client.internal.Constants;
var wrapText  = com.ibm.asq.common.web.ui.internal.util.wordWrapping.wrapText;

dojo.declare(
    "com.ibm.asq.common.web.ui.internal.widgets.OptionChooser",
    [dijit._Widget, dijit._Templated],
{
	    templateString: '<div id="${id}" dojoAttachPoint="domNode" tabindex="-1">' +
					 	' <table border="0">' +
						'  <tr>' +
						'   <td dojoAttachPoint="leftSelectTitleNode"/>'+
						'   <td/>' +
						'   <td dojoAttachPoint="rightSelectTitleNode"/>' +
						'  <tr>' +
						'   <td>' +
			            '    <label><select dojoAttachPoint="choosableOptionsNode" name="choosableOptions" class="optionSelect" multiple="true" size="${selectionSize}" dojoAttachEvent="onchange: _onChooseSelect">' +
						'    </select></label>' +
						'   </td>' +
						'   <td>' +
						'    <table border="0" style="width:35px">' +
						'     <tr>' +
                        '      <td><button dojoType="com.ibm.asq.common.web.ui.internal.widgets.Button" dojoAttachPoint="addItemNode" iconClass="com-ibm-asq-common-web-ui-option-chooser-move-right-button" dojoAttachEvent="onClick: _onAddItems,onKeyPress: _onAddItems" tabindex="-1"></button></td>' +
                        '     </tr>' +
                        '     <tr>' +
                        '      <td><button dojoType="com.ibm.asq.common.web.ui.internal.widgets.Button" dojoAttachPoint="addAllItemsNode" iconClass="com-ibm-asq-common-web-ui-option-chooser-move-all-right-button" dojoAttachEvent="onClick: _onAddAllItems,onKeyPress: _onAddAllItems" tabindex="-1"></button></td>' +
                        '     </tr>' +
                        '     <tr>' +
                        '      <td><button dojoType="com.ibm.asq.common.web.ui.internal.widgets.Button" dojoAttachPoint="removeItemNode" iconClass="com-ibm-asq-common-web-ui-option-chooser-move-left-button" dojoAttachEvent="onClick: _onRemoveItems,onKeyPress: _onRemoveItems" tabindex="-1"></button></td>' +
                        '     </tr>' +
                        '     <tr>' +
                        '      <td><button dojoType="com.ibm.asq.common.web.ui.internal.widgets.Button" dojoAttachPoint="removeAllItemsNode" iconClass="com-ibm-asq-common-web-ui-option-chooser-move-all-left-button" dojoAttachEvent="onClick: _onRemoveAllItems,onKeyPress: _onRemoveAllItems" tabindex="-1"></button></td>' +
                        '     </tr>' +
						'    </table>' +
						'   </td>' +
						'   <td>' +
		                '    <label><select dojoAttachPoint="chosenOptionsNode" class="optionSelect" multiple="true" size="${selectionSize}" dojoAttachEvent="onchange:_onChosenSelect">' +
						'    </select></label>' +
						'   </td>' +
						'  </tr>' +
						' </table>' +
						' <div align="center">' +
						' <table class="chooser-button-area" dojoAttachPoint="buttonTableNode" border="0">' +
						'  <tr>' +
						'   <td class="action-button-cell">' +
						'    <button dojoAttachPoint="okNode" class="option-action-button" dojoAttachEvent="onclick: _okAction"></button>' +
						'   </td>' +
						'   <td class="action-button-cell">' +
						'    <button dojoAttachPoint="cancelNode" class="option-action-button" dojoAttachEvent="onclick: _cancelAction"></button>' +
						'   </td>' +
						'  </tr>' +
						' </table>' +
						' </div>' +
	                    '</div>',

        widgetsInTemplate: true,
        selectionSize: 20,
		titleMaxWidth: 85, // hard code the max width since the dom node has not been added to the tree.
		options: null,
		numOptions: 0,
		oklistener: null,
		cancellistener:null,
		leftTitle: "",
		rightTitle: "",
		
		_attachEvent: function( /*Node*/ node, /*String*/ eventname, /*callback*/ method )
		{
			if( node.attachEvent )
			{
				node.attachEvent( eventname, dojo.hitch( this, method ) );
			}
			else if( node.addEventListener )
			{
				var name = eventname.substr( 2 );
				node.addEventListener( "change", dojo.hitch( this, method ), false );
			}
		},
								
		postCreate: function()
		{
			this.addItemNode.setDisabled(true);
			this.addAllItemsNode.setDisabled(true);
			this.removeItemNode.setDisabled(true);
			this.removeAllItemsNode.setDisabled(true);
			this.buttonTableNode.style.display = "none";
			this.okNode.innerHTML = Constants.getMessage("actionLabel_ok");
			this.cancelNode.innerHTML = Constants.getMessage("actionLabel_cancel");
			this.leftSelectTitleNode.innerHTML = wrapText(this.leftTitle, "", this.titleMaxWidth, true, "<BR/>");
			this.rightSelectTitleNode.innerHTML = wrapText(this.rightTitle, "", this.titleMaxWidth, true, "<BR/>");
			this.isDisabled = false; 
			//this._attachEvent( this.choosableOptionsNode, "onchange", this._onselect );
		},
		
		enableOkCancel: function()
		{
			this.buttonTableNode.style.display = "inline";
		},
		
		
		addOption: function( optionName, optionData, selected, required )
		{
			if( !this.options )
			{
				this.options = {};
			}
			
			var optString = this.numOptions.toString();
			
			this.options[ optString ] = new OptionChooser.Option( optString, optionName, optionData, required );
			
			if (selected  || required)
			{
				var option = new Option( optionName, optString );	
				option.title = optionName;			
	            if (required === true) {
					option.required = true;
	                dojo.addClass(option, "optionRequired");
	            }							
				this.chosenOptionsNode.options[ this.chosenOptionsNode.options.length ] = option;
				if(!this.isDisabled)
					this.removeAllItemsNode.setDisabled(false);
			}
			else
			{
				var option = new Option( optionName, optString );
				option.title = optionName;
				this.choosableOptionsNode.options[ this.choosableOptionsNode.options.length ] = option;				
				if(!this.isDisabled)
					this.addAllItemsNode.setDisabled(false);				
			}
			
			this.numOptions++;
		},		
		
		updateOption: function( optionName, optionData, selected, required, optionIndex, optionValue )
		{
			if( !this.options )
			{
				this.options = {};
			}
			
			this.options[ optionValue ] = new OptionChooser.Option( optionValue, optionName, optionData, required );
			
			if (selected  || required)
			{
				var option = new Option( optionName, optionValue );	
				option.title = optionName;			
	            if (required === true) {
					option.required = true;
	                dojo.addClass(option, "optionRequired");
	            }							
				this.chosenOptionsNode.options[ optionIndex ] = option;
				if(!this.isDisabled)
					this.removeAllItemsNode.setDisabled(false);
			}
			else
			{
				var option = new Option( optionName, optionValue );
				option.title = optionName;
				this.choosableOptionsNode.options[ optionIndex ] = option;				
				if(!this.isDisabled)
					this.addAllItemsNode.setDisabled(false);				
			}
		},	
				
		_enableButtons: function()
		{
			if (this.choosableOptionsNode.selectedIndex != -1 && !this.isDisabled) {
				this.addItemNode.setDisabled(false);
			}
			else
				this.addItemNode.setDisabled(true);
							
			if (this.chosenOptionsNode.selectedIndex != -1 && !this._requiredSelected(this.chosenOptionsNode) && !this.isDisabled) {
				this.removeItemNode.setDisabled(false);
			} else
				this.removeItemNode.setDisabled(true);
				
			if( this.choosableOptionsNode.length == 0 || this.isDisabled)
			{
				this.addAllItemsNode.setDisabled(true);
			}
			else
				this.addAllItemsNode.setDisabled(false);
			
			if( this.chosenOptionsNode.length == 0 || this.isDisabled)
			{
				this.removeAllItemsNode.setDisabled(true);
			}
			else
				this.removeAllItemsNode.setDisabled(false);
		},
		
		_requiredSelected: function(/*node*/ select) {
			for (var i = 0; i < (select ? select.options.length : 0); i++) {
				if (select.options[i].selected && select.options[i].required === true) {
					return true;
				}
			}
			return false;
		},
		
		setDisabled: function(/*boolean*/ disabled) {
			if (disabled) {
				this._disableButtons();
			} else {
				this._enableButtons();
			}
			this.isDisabled = disabled;
		},
		
		_disableButtons: function() {
			this.addItemNode.setDisabled(true);
			this.removeItemNode.setDisabled(true);
			this.addAllItemsNode.setDisabled(true);
			this.removeAllItemsNode.setDisabled(true);
		},
		
		_onChooseSelect: function( event )
		{
			this._enableButtons();
			if(event){
                dojo.stopEvent(event);
            }
		},
		
		_onChosenSelect: function( event )
		{
			this._enableButtons();
			if(event){
                dojo.stopEvent(event);
            }
		},
		
		_onAddItems: function( event )
		{
			for( var i = 0; i < this.choosableOptionsNode.options.length; i++ )
			{
				if( this.choosableOptionsNode.options[ i ].selected == true )
				{
					this.chosenOptionsNode.options[ this.chosenOptionsNode.options.length ] = 
						new Option( this.choosableOptionsNode.options[ i ].text, this.choosableOptionsNode.options[ i ].value );
					this.choosableOptionsNode.remove( i );
					i--;
				}
			}

			this._enableButtons();	
			if(event){
                dojo.stopEvent(event);
            }		
		},
		
		_onAddAllItems: function( event )
		{
			for( var i = 0; i < this.choosableOptionsNode.options.length; i++ )
			{
				this.chosenOptionsNode.options[ this.chosenOptionsNode.options.length ] = 
					new Option( this.choosableOptionsNode.options[ i ].text, this.choosableOptionsNode.options[ i ].value );
				this.choosableOptionsNode.remove( i );
				i--;
			}
						
			this._enableButtons();	
			if(event){
                dojo.stopEvent(event);
            }		
		},
		
		_onRemoveItems: function( event )
		{	     	
			for( var i = 0; i < this.chosenOptionsNode.options.length; i++ )
			{				
				if( this.chosenOptionsNode.options[ i ].selected == true )
				{
	                //skip required options
	                if (!this.chosenOptionsNode.options[i].required) {
						//remove the item
						var option = new Option(this.chosenOptionsNode.options[i].text, this.chosenOptionsNode.options[i].value);
						option.title = option.text;
						this.choosableOptionsNode.options[this.choosableOptionsNode.options.length] = option;
						this.chosenOptionsNode.remove(i);
						i--;
					}
				}
			}
			this._enableButtons();	
			if(event){
                dojo.stopEvent(event);
            }		
		},
		
		_onRemoveAllItems: function( event )
		{
			for( var i = 0; i < this.chosenOptionsNode.options.length; i++ )
			{
				//skip required options
				if (!this.chosenOptionsNode.options[i].required) {
					var option = new Option(this.chosenOptionsNode.options[i].text, this.chosenOptionsNode.options[i].value);
					option.title = option.text;
					this.choosableOptionsNode.options[this.choosableOptionsNode.options.length] = option;
					this.chosenOptionsNode.remove(i);
					i--;
				}
			}
									
			this._enableButtons();	
			if(event){
                dojo.stopEvent(event);
            }		
		},
		
		getOptions: function()
		{
			//var selected = new Array();
			var selected = {};
			
			for( var i = 0; i < this.chosenOptionsNode.options.length; i++ )
			{
				//selected[ i ] = options[ this.chosenOptionsNode.options[ i ].value ];
				selected[ this.options[ this.chosenOptionsNode.options[ i ].value ].data.id ] = true;
			}
			
			for( var i = 0; i < this.choosableOptionsNode.options.length; i++ )
			{
				selected[ this.options[ this.choosableOptionsNode.options[ i ].value ].data.id ] = false;
			}
			
			return( selected );
		},
		
        getSelectedOptions: function() {
            var selected = [];            
            for( var i = 0; i < this.chosenOptionsNode.options.length; i++ ) {
                selected.push(this.options[this.chosenOptionsNode.options[i].value].data);
            }            
            return selected;
        },		
		
		_okAction: function( event )
		{
			if( this.oklistener )
				this.oklistener( event );
			if(event){
                dojo.stopEvent(event);
            }
		},
		
		_cancelAction: function( event )
		{
			if( this.cancellistener )
				this.cancellistener( event );
			if(event){
                dojo.stopEvent(event);
            }
		},
		
		addOkListener: function( listener )
		{
			if( listener )
			{
				this.oklistener = listener;
			}
		},
		
		addCancelListener: function( listener )
		{
			if( listener )
			{
				this.cancellistener = listener;
			}			
		}
}
);

dojo.declare( "OptionChooser.Option", null,
{
	key: "",
	displayString: "",
	data: null,
	required: false,
	
	constructor: function( /*Position in list*/key, /*String*/displayString, /*Object*/data, /*boolean*/ required )
	{
		this.key = key;
		this.displayString = displayString;
		this.data = data;
		if (required === true) {
			this.required = required;
		}
	} 	
}
);
})();

