/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //   A simple pane widget that is capable of holding certain contents in its content area (center).
 //   The main feature of this pane is that it maintains a constant distance from the top of the
 //   page even when the page is being scrolled down or up. This maintains the pane
 //   to be always in view. It has a built in toggle control that allows the user to turn on/off
 //   the pane's scroll tracking.
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.StationaryPane");

dojo.require("dijit.Tooltip");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function() {

var AnimateDuration = 400;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.StationaryPane",
             [dijit._Widget, dijit._Templated, com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-web-ui-stationary-pane">' +
                    '  <div class="header" dojoAttachPoint="focusNode">' +
                    '    <div class="header-left">' +
                    '      <div class="header-right">' +
                    '        <div class="header-mid">' +
                    '          <div dojoAttachPoint="actionsNode" class="action-area">' +
                    '             <label><img class="sticky-toggle" dojoAttachPoint="stickyNode" dojoAttachEvent="onclick:_changeTracking"></img></label>' +
                    '          </div>' +
                    '          <div class="title-container"><span dojoAttachPoint="titleNode" class="title">' +
                    '          </span></div>' +
                    '        </div>' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
                    '  <div class="content-area" dojoAttachPoint="containerNode" waiRole="region" tabindex="-1"></div>' +
                    '  <div class="content-bottom-container">' +
                    '    <div class="content-bottom-left">' +
                    '      <div class="content-bottom-right">' +
                    '        <div class="content-bottom-mid">' +
                    '        </div>' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
					//The span below is used to programatically find the pixel length of a string.  Since it is off screen it is not seen nor does it interfere
					//with the page that it is on.  By having a visible but unseen span, the text can be assigned to the innerHTML and then the offsetWidth of
					//the text can be retrieved from the span.  This allows text to be "hyphenated" so that it will wrap appropriately within its confines
					//without having to have some predefined length for wrapping set up.  
					'  <span class="content-area" dojoAttachPoint="rulerNode" style="position: absolute; left:-3000px"></span>' +
                    '</div>',
    
    title: null,
    trackMessage: null,
    untrackMessage: null,
    
    postCreate: function() 
    {
        this.init = false;
        if(this.title)
            this.setTitle(this.title);
            
        // create a tooltip for our tracker toggle. This is to
        // turn on/off the auto tracking of the stationary pane.
        //
        if(this.trackMessage && this.untrackMessage) {
            this.toggleTooltip = new dijit.Tooltip({label: this.untrackMessage, connectId:[this.stickyNode]});
        }
                
        // give some slack to the top of the stationary pane and the 
        // top edge of the page. The stationary pane will be 10 px below
        // to edge of the scroll top.
        this.slackY = 10;
        
        // initialize the tracking.
        this.track = false;
        this._changeTracking();
    },
    
    uninitialize: function(){
    	this.stickyNode.src = null;
    },
    
    // Summary:
    //    Set the title of the pane.
    //
    setTitle: function(/*String*/ title)
    {
        this.titleNode.innerHTML = title;
    },
    
    _changeTracking: function(event)
    {
        this.track = !this.track;
        var imgName = this.track ? "unpinnedview.gif" : "pinnedview.gif";
        this.stickyNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/images/icons/elcl/" + imgName);
        
        if(this.toggleTooltip) {
            this.toggleTooltip.close();
            this.toggleTooltip.label = (this.track) ? this.trackMessage : this.untrackMessage;
        }
        this.lastScrollPos = 0;

        // if the tracking is turned on, add the handler to track
        // scrolling.
        //
        if(this.track) {
            this.resizeHandler = this.connect(window, "onresize", "_trackTopPosition");
            this.scrollHandler = this.connect(window, "onscroll", "_trackTopPosition");
        }
        else {
            // tracking is turned off, diable our handlers and
            // retrack back to the top.
            if(this.resizeHandler) {
                this.disconnect(this.resizeHandler);
                this.resizeHandler = null;
            }
            if(this.scrollHandler) {
                this.disconnect(this.scrollHandler);
                this.scrollHandler = null;
            }
			if (this._anim){
				this._anim.stop();
			}

            this._anim = dojo.fx.slideTo({
                node: this.domNode,
                duration: AnimateDuration,
                left: 0,
                top: 0,
				onEnd: dojo.hitch(this, "_endAnimation")
            }).play();
        }
        if(event){
            dojo.stopEvent(event);
        }
    },
    
    // Summary:
    //    Internal function to calculate the absolute distance from the top of the
    //    pane to the top of the page. This is returned as the offset from the top.
    //
    _calculateTopOffset: function()
    {
        var node = this.domNode;
        var topOffset = 0;
    
        if(node.offsetParent) {
            while(node) {
                topOffset += node.offsetTop;
                node = node.offsetParent;
            }            
        }
        else if(node.y)
            topOffset += node.y;
            
    return topOffset;
    },
    
    _trackTopPosition: function()
    {
		if( dojo.isIE && this.domNode.parentElement == null )
		{
			//Do not proceed if the domNode is not part of the document yet.
			//This circumvents problems with dojo.marginBox.
			return;
		}
		
        var viewport = dijit.getViewport();
        var curBound = dojo.marginBox(this.domNode);
        var scrollDir = viewport.t - this.lastScrollPos;

        if(!this.init) {
            this.topOffset = this._calculateTopOffset() - this.slackY;
            this.init = true;
        }
                
        this.lastScrollPos = viewport.t;
        
        // we have not scrolled to top of this pane
        // yet. It's still visible, don't have to move the the pane.
        // 
        if(viewport.t < this.topOffset && scrollDir > 0) {
            return;            
        }
            
        var t = Math.max(0, (viewport.t - this.topOffset));

        // our position is already here, don't have 
        // to move.
        if(curBound.t == t)
            return;
		if (this._anim){
			this._anim.stop();
		}
        this._anim = dojo.fx.slideTo({
            node: this.domNode,
            duration: AnimateDuration,
            left: 0,
            top: t,
			onEnd: dojo.hitch(this, "_endAnimation")
        }).play();
    },
	
	_endAnimation: function()
	{ 
		this._anim = undefined;
	}
});

})();
