/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    AttributePicker is a simple widget that displays the sub-attribute names of a 
 //    Catalog PART. This widget uses the common generic attribute picker to display
 //    catalog specific attribute lists.
 //
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributePicker");

dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Attribute");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Configuration");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributePicker");

(function() {
var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributePicker",
             [com.ibm.asq.common.web.ui.internal.widgets.layout.AttributePicker],
{    
    // specify an array of attribute names to be excluded from the pick list.
    excludeItems: null,   
    
    // the catalog part object of which the list of attribute names is obtained. 
    catalogPart: null,
    
    // Summary:
    //    Override the base class's implementation to read the attribute list from
    //    the catalog part object.
    //
    //    Regenerate the selection list of the picker. If an excludeItems list
    //    is set in the widget, those attributes are excluded from the list. 
    //
    refreshList: function()
    {
        this.attrSelectNode.length = 0;
        
        if(this.catalogPart) {
            var attrNames = this.catalogPart.getAttributeNames();
            var nItems = 0;
			var optArray = []            
            
            for(var i = 0; i < attrNames.length; i++) 
            {
                var attr = this.catalogPart.getAttribute(attrNames[i]);
                
                //Add the attributes to an array so they can be sorted before they are displayed
                if (attr.isArray() || !this._inExcludeList(attrNames[i])) {
                    var dispName = RtlmUiUtil.getCatalogMessage("pl_" + attrNames[i]);
                    
                    // this is a custom attribute, just use the name from catalog.
                    if(dispName.indexOf("pl_") > -1) {
                        dispName = attrNames[i];
                    }
                    var option = new Option(dispName, attrNames[i]);
                    option.title = dispName;
                    optArray.push(option);
                }
            }
        }

		//Sort the list of artifacts in the array
        optArray.sort(this._sortList);
        
        //Add the artifacts to the select control
		for (var i = 0; i < optArray.length; i++)
			this.attrSelectNode.options[i] = optArray[i];
			
        this.addButtonNode.disabled = true;
        this.addCloseButtonNode.disabled = true;
    },

	_sortList: function(option1, option2)
	{
		return option1.text < option2.text ? -1 : option1.text > option2.text ? 1 : 0;
	},
	
    _inExcludeList: function(value)
    {
        if(!this.excludeItems)
            return false;
            
        for(var i = 0; i < this.excludeItems.length; i++) {
            if(this.excludeItems[i] == value)
                return true;
        }
        return false;
    }
    
});

})();
