/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    A widget that's capable of displaying a single root attribute of an Catalog configuration.
 //    It displays the root attribute and its sub-attributes in a panel with an editable title.
 //    If the attribute can be remove/deleted, it also display a "Remove" action on the panel's 
 //    top-right corner.
 //
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.CatalogAttrBuilder");

dojo.require("dijit.Tooltip");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributeDisplay");

(function() {

var RtlmUiUtil 		= com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var AttributeWidget = com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributeDisplay;
var ImageCache      = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var CommonUtility   = com.ibm.asq.common.web.ui.internal.util.Utility;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.CatalogAttrBuilder",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-rational-test-lm-web-ui-catalogAttrBuilder">' +
                    '</div>',
    
    // the style of the attribute builder, "panel" to include a border/background,
    // title and action, or "plain", which displays only the attribute's content.
    style: "panel",
    
    title: null,
    
    catalogAttribute: null,
    
    arrayAttrTag: null,
    
    canRemove: true,
    
    setTitleFocus: false,
    
	mode: "EDITABLE",
    
    bgShading: true,
    
    assetType: "Machine",
    
    // Set this member to true if this is the root attribute and
    // it should show all asset root types, i.e,, Machine, virtualImage, etc.
    //
    showAllRootTypes: false,
    
    width: null,
    
    postCreate: function() 
    {
        if(this.width) {
            this.domNode.style.width = this.width + "px";
        }
        if (this.style == "panel") {
            var holder = document.createElement("div");
            this.domNode.appendChild(holder);
            this.sectionNode = new com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection({
                titleSize: "11pt",
                bgShading: this.bgShading,
                open: true,
                collapsible: false
            }, holder);
            
            this.containerNode = document.createElement("div");
            this.sectionNode.containerNode.appendChild(this.containerNode);
            
            // FF2 has rendering problem of shifting one pixel on floating DIV on top of
            // this CollapsibleSection.  Reset the overflow-x to inherit instead of auto
            // fixed the problem.  The panel is not collapsible in this case anyway.
            //
            if (dojo.isFF && (dojo.isFF == 2)) {
                this.sectionNode.containerNode.style.overflowX = "inherit";
            }
            
            if (this.title) 
                this.setTitle(this.title);
            
            this.sectionNode.setTitleMode((this.mode == "EDITABLE"));
            this.sectionNode.onTitleEdited = dojo.hitch(this, "_handleTitleEdited");
            this.sectionNode.setAttributeAreaHidden(true);
            
            if (this.setTitleFocus && this.sectionNode.viewEditableTitle) {
                this.sectionNode.viewEditableTitle._onClick();
            }
            
            if (this.canRemove && this.mode == "EDITABLE") {
                var remove = document.createElement("span");
                var img = document.createElement("img");
                img.src = ImageCache.REMOVE_ENABLED;
                img.title = RtlmUiUtil.getMessage("labelRemoveButton");
                remove.appendChild(img);
                dojo.addClass(img, "remove-action");
                this.sectionNode.headerRightNode.appendChild(remove);
                this.connect(remove, "onclick", "_handleRemoveAction");
				img.setAttribute("tabIndex", 0);
				this.connect(img, "onkeypress", function(e){
					var charCode = (e.which) ? e.which : e.keyCode
					if (charCode == 13 && this._handleRemoveAction) {
						this._handleRemoveAction();
					}
				});	
            }
        }
        else {
            this.containerNode = document.createElement("div");
            this.domNode.appendChild(this.containerNode);            
        }
        this.setCatalogAttribute(this.catalogAttribute, this.arrayAttrTag);
    },
    
    setCatalogAttribute: function(attribute, tag) 
    {
        this.catalogAttribute = attribute;
        this.arrayAttrTag = tag;

        // unload any widgets we've first, before rebuild                
        CommonUtility.destroyWidgetsFromNode(this.containerNode);
        this.containerNode.innerHTML = "";
        
        if(this.catalogAttribute) {
            this.attrWidget = new com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributeDisplay({
                'canDelete':false,
                'catalogAttribute':this.catalogAttribute,
                'arrayAttrTag': this.arrayAttrTag,
                'mode': this.mode,
                'onValueChanged': dojo.hitch(this, "_handleAttributeValueChanged"),
                'onRootTypeChanged':  dojo.hitch(this, "_handleRootAssetTypeChanged"),
                'reduceActionsVisibility': false,
                'showSubAttrWithValueOnly': true,
                'showAllRootTypes': this.showAllRootTypes
            });
            this.containerNode.appendChild(this.attrWidget.domNode); 
        }
        
    },
    
    // Summary:
    //    Set the title of the pane.
    //
    setTitle: function(/*String*/ title)
    {
        if(this.sectionNode)
            this.sectionNode.setTitle(title);
    },

    // Summary:
    //    Returns whether there is error in the attribute input fields. This includes the
    //    any children attributes that has user input values.
    //
    hasError: function()
    {
        return this.attrWidget.hasError();    
    },
    
    // Summary:
    //    This extension/notification method is called when an attribute value has been changed
    //    by user. This includes any children attributes.
    //
    onValueChanged: function(partBuilderWidget)
    {
    },
    
    // Summary:
    //   This extension/notification method is called when the panel's remove action is pressed.
    //   The builder itself, doesn't handle this event, i.e., change the configuration (model) object.
    //   The client of this builder would need to handle the removal of this panel display, and the 
    //   model change by overriding this method.
    //
    onRemove: function(partBuilderWidget)
    {
    },
    
    // Summary:
    //    Specific notification for root type changed events. This is only called when the
    //    builder's member 'showAllRootTypes' is set to true and that the user has changed
    //    the root asset type to the one other then the one that was set. 
    //
    onRootTypeChanged: function(assetType, asset)
    {
    },

    // Summary:
    //    This extension/notification method is called when the title of the panel has been changed.
    //
    onTitleChanged: function(partBuilderWidget, newTitle)
    {
    },

    _handleRemoveAction: function()
    {
        this.onRemove(this);  
    },

    _handleTitleEdited: function(newLabel)
    {
        this.onTitleChanged(this, newLabel);  
    },
    
    _handleAttributeValueChanged: function()
    {
        this.onValueChanged(this);
    },
    
    _handleRootAssetTypeChanged: function(type, asset)
    {
        this.assetType = type;
        this.onRootTypeChanged(type, asset);
    }
});

})();
