/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// author: Paul Beaulieu
//
// Summary: This is a widget that brings both the DateBox and TimePicker together
//

dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.datetimepicker.DateTimePicker");

dojo.require( "dijit._Widget" );
dojo.require( "dijit._Templated" );
dojo.require( "dijit.form.Button" );
dojo.require("dojo.date.locale");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.datetextbox.DateTextBox");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.TimePicker");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.DatePicker");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.SimpleTimeSelector");

(function() {


dojo.declare(
    "com.ibm.asq.common.web.ui.internal.widgets.datetimepicker.DateTimePicker",
    [dijit._Widget, dijit._Templated],
{
    templateString: '<div class="datetimepicker-widget" dojoAttachPoint="domNode" tabindex="-1">' +
					' <table>' +
					'  <tr>' +
					'   <td dojoAttachPoint="labelNode,fieldLabelNode"  class="label-class"></td>' +
					'   <td dojoAttachPoint="rowPickersNode" class="date-picker-class">' +
					'   </td>' +
					'   <td dojoAttachPoint="timePickersNode" class="time-picker-class">' +
					'   </td>' +
					'  </tr>' +
					' </table>' +
                    '</div>',

	dateName: "",
	dateClass: "",
	minimumDate: "",
	widgetsInTemplate: true,
	calendarButtonLabel: MessageUtil.getMessage(MessageBundle.COMMON, "scheduler_buttoncalendar"),

	postCreate: function()
	{
		if( this.label )
		{
			this.labelNode.innerHTML = this.label;
		}
		
        this.dateNode = new com.ibm.asq.common.web.ui.internal.widgets.DatePicker({
                    name: this.dateName,
                    fieldLabel: this.label,
                    value: "",
                    minDate:this.minDate,
                    maxDate:this.maxDate,
                    invalidMessage: MessageUtil.getMessage(MessageBundle.COMMON, "dtpInvalidDate"),
                    onChange: dojo.hitch(this, "onDateChange"),
                    onkeyup: ( this.onKeyUp ) ? this.onKeyUp : dojo.hitch(this, "_onkeyup"),
                    "class": this.dateClass
                });

		this.rowPickersNode.appendChild( this.dateNode.domNode );
		this.dateNode.init();
		if( this.date )
			this.dateNode.setValue( this.date );
			
        this.timeField = new com.ibm.asq.common.web.ui.internal.widgets.SimpleTimeSelector({time:this.time});
        this.timeField.onChange = dojo.hitch( this, this.onTimeChange );
        dojo.style(this.timeField.domNode, "display", "inline");
        this.timePickersNode.appendChild(this.timeField.domNode);
	},

    uninitialize: function()
    {
        if(this.dateNode) {
            this.dateNode.destroy();
            this.dateNode = null;
        }
        if(this.timeField) {
            this.timeField.destroy();
            this.timeField = null;
        }
    },
	
	_onCalendarClick: function()
	{
		if (!this._picker) 
		{
			var popupProto = dojo.getObject("dijit._Calendar", true /*false*/);
			this._picker = new popupProto({
				_currentDateBox: this.dateNode,
				_dateBoxId: null,
				_container: this,
				_parent: this.calendarButtonNode,
				closable:true,
				onValueSelected: function(date)
				{
					dijit.popup.close(this._container._picker);
					this._currentDateBox.setValue( date );
				}
			});
                
			// constraint the size of the calendar. Otherwise, it'll take
			// up the rest of the space of the page.
			dojo.style(this._picker.domNode, "width", "180px");
			dojo.style(this._picker.domNode, "height", "200px");
		}
				
		this._picker.setValue(new Date());
            
		// put the focus to the date box. This will allow
		// the popup to pop down when the user click elsewhere
		// in the form.
		this.calendarButtonNode.picker = this._picker;
		var body = document.getElementsByTagName( "body" );
		var self = this;
		
		if( body.length == 1 )
		{			
			var onclick = body[ 0 ].onclick;
			this.connect(body[0], "onclick", function( e )
								{ 
									var parent = e.target;
									var id = self.calendarButtonNode.picker.domNode.id;
									var lost_focus = true;
									
									while( parent != null )
									{
										if( parent.id && ( parent.id == id ) )
										{
											lost_focus = false;
											break;
										}
										
										parent = parent.parentNode;
									}
									
									if( lost_focus )
									{
										dijit.popup.close( self.calendarButtonNode.picker );
										if( onclick == undefined )
										{
											delete this[ "onclick" ];
										}
										else
										{
											this.onclick = onclick;
										}
									}
									
									if( onclick )
									{
										onclick( e );
									}
								});
		}											  
        
		dijit.popup.open({
			parent: this.calendarButtonNode,
			popup: this._picker,
			around: this.calendarButtonNode.domNode,
			onClose: function()
			{//do nothing
			}
		});				
	},
	
	onDateChange: function()
	{
	},
	
	onTimeChange: function()
	{
	},
	
	getValue: function()
	{
		var theDate = this.dateNode.getValue();
		if(theDate){
			var theTime = this.timeField.getTime();
			
			theDate.setHours( theTime.getHours(), theTime.getMinutes(), 0, 0 );
		}
		
		return( theDate );
	},
	
	getDateValue: function()
	{
		return( this.dateNode.getValue() );
	},
	
	getTimeValue: function()
	{
		return( this.timeField.getTime() );
	},
	
	clear: function()
	{
		this.dateNode.setDisplayedValue( "" );
		//this.timeNode.setDisplayedValue( "" );
		this.dateNode.setValue( null );

        if(this.dateNode.constraints['min'])
            delete this.dateNode.constraints['min'];
        
        if(this.dateNode.constraints['max'])
            delete this.dateNode.constraints['max'];

        if (this.dateNode._picker) {
            var picker = this.dateNode._picker;
            delete picker;
            this.dateNode._picker = null;
        }
        
		var defaultTime = new Date();
		defaultTime.setHours(0);
		defaultTime.setMinutes(0);
		this.timeField.setTime( defaultTime );
	},
	
	_onkeyup: function( evt )
	{
		//Override so that date is not validated as the user types it.  Wait till later
	},
	
    isValid: function(/* Boolean*/isFocused){
		return( this.dateNode && this.dateNode.isValid( isFocused ) );
	}
}	
);

})();