/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

/*
 * Example for usage
 data = [ 
		[ "normal", false, "new", 'But are not followed by two hexadecimal', 29.91, 10, false ],
		[ "important", false, "new", 'Because a % sign always indicates', 9.33, -5, false ],
		[ "important", false, "read", 'Signs can be selectively', 19.34, 0, true ],
		[ "note", false, "read", 'However the reserved characters', 15.63, 0, true ],
		[ "normal", false, "replied", 'It is therefore necessary', 24.22, 5.50, true ],
		[ "important", false, "replied", 'To problems of corruption by', 9.12, -3, true ],
		[ "note", false, "replied", 'Which would simply be awkward in', 12.15, -4, false ]
	];
	
	var rows = 2;
	for(var i=0, l=data.length; i<rows-l; i++){
		data.push(data[i%l].slice(0));
	}
	
	//Sample comparator to be used for sorting
	compare2ndLetter = function(inA, inB) {
			// Sample Comparator for second alphabet
			// return <0, 0, >0
			return inA.charCodeAt(1) - inB.charCodeAt(1);
		}
	
	model = new dojox.grid.data.Table([{}, {}, {}, {compare: compare2ndLetter}], data);
	
	//model = new dojox.grid.data.Table(null, data);//Use if sorting is not required
	
	//Formatting data
	formatMoney = function(inDatum) {
		return isNaN(inDatum) ? '...' : '$' + parseFloat(inDatum).toFixed(2);
	}

	// Layout
	statusCell = { field: 2, name: 'Status', width: '83px', styles: 'text-align: center;', editor: dojox.grid.editors.Select, options: [ "new", "read", "replied" ] };
	gridLayout = [{
		type: 'dojox.GridRowView', width: '20px'
	},{
		defaultCell: { editor: dojox.grid.editors.Input, styles: 'text-align: right;'  },
		rows: [[
			{ name: 'Id',width: '90px', get: function(inRowIndex) { return inRowIndex+1;} },
			{ name: 'Priority',width: '106px', styles: 'text-align: center;', editor: dojox.grid.editors.Select, options: ["normal", "note", "important"]},
			{ name: 'Mark', width: '112px', styles: 'text-align: center;', editor: dojox.grid.editors.Bool },
			statusCell,
			{ name: 'Message', width: '249px',styles: ''},
			{ name: 'Amount', width: '127px', formatter: formatMoney },
			{ name: 'Amount', field: 4, width: '200px', formatter: formatMoney }
		]]
	}];
	
	//
	//	Note the data[] and the rows[] differ in size. 
	//  ie. If there is data to be hidden from the UI but is needed during service call, append the values in data. 
	//	
		
		var grid = new com.ibm.common.web.ui.internal.widgets.grid.GridContainer({
			model: model,
			structure: gridLayout,
			rowCount:"10", elasticView:"2",
			defaultStaticRow: [ "normal", false, "new", 'Now is the time for all good men to come to the aid of their party.', 99.99, 9.99, false ] //Used for adding new row.
		});
		this._testGrid.appendChild(grid.domNode);
		grid.render();
		grid.initialize();
 
 <div>
 	<div dojoAttachPoint="_testGrid" style="height: 500px; width: 500px;"></div>
 </div>	
 */


dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer");

dojo.require("dojox.grid.Grid");
dojo.require("dojox.grid.VirtualGrid");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

(function() { // module container

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer",
dojox.Grid,
{
	templateString: '<div class="com-ibm-asq-common-gridcontainer">' +
	                '  <div style="text-align: right" dojoAttachPoint="headerRow"></div>' +
					'  <div class="dojoxGrid" hidefocus="hidefocus" role="wairole:grid" dojoAttachPoint="gridNode">' +
					'      <div class="dojoxGrid-master-header" dojoAttachPoint="viewsHeaderNode"></div>' +
					'      <div class="dojoxGrid-master-view" dojoAttachPoint="viewsNode"></div>' + 
					'      <span dojoAttachPoint="lastFocusNode" tabindex="0"></span>' +
					'  </div>' +
					'</div>',
					
	classTag: 'dojoxGrid',
	autoHeight: true,
    
	//client specified location for actions to be appended.  if provided, actions 
	//will beplaced in this node; otherwise they'll be placed in this.headerRow
	actionsNode: null,	
	
	/**
	 * Specifies the maximum height of the content of the grid.  If content exceeds this height,
	 * the content section of the grid will scroll veritcally.
	 */
	maxHeight: null,
	
	ACTION_ADD: "add",
	ACTION_REMOVE: "remove",
	
	/**
	 * Set the scrollbar height. If the grid has x-scrollbar, then expand grid's height to display 
	 * the whole row. 
	 */
	scrollbarHeight: 20,
	
	constructor: function() {
		this.onAdd = null;
		this.onRemove = null;
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this.domNode.style.overflow = 'auto';
		this._actionButtons = {};
		if (this.maxHeight != null) {
			this.viewsNode.style.maxHeight = this.maxHeight;
		}
	},
	
	/*
	 * Should be called after grid.render is called. Call only once for a given grid.
	 * 
	 * Buttons are for initial implementation only. They will be replaced by Icons
	 */
	
	initialize: function() {
		this._addDefaultActions();
	},
	
	/*
	 * returns the control selected rows
	 */
	
	getSelectedRows: function()
	{
		var rowIndicies = this.selection.getSelected();
		var selectedRows = [];
		//var sampleString = "Get Selected APi called (# is delimiter)\n";
		for(var i=0;i<rowIndicies.length;i++)
		{
			var row = this.model.getRow(rowIndicies[i]);
			//sampleString +=row+" # ";
			selectedRows.push(row);
		}
		//alert(sampleString);
		return selectedRows;
	},
	
	/*
	 * removes the CTRL+selected rows
	 */
	
	deleteSelectedRows: function()
	{
		var selectedRows = this.getSelectedRows();
		this.removeSelectedRows();
		this._updateActionStates();
		if (this.onRemoveRows && dojo.isFunction(this.onRemoveRows)) {
			this.onRemoveRows(selectedRows);
		}
		
		//this.updateContentScrolling();
	},
	
	/*
	 * Appends a new row below the current row with the defaultStaticRow passed during creation.
	 * Add Row button will not appear if this.defaultStaticRow is not passed during creation.
	 */
	
	appendRows: function(rows) {
		for (var i = 0; i < (rows || []).length; i++) {
			this.appendRow(rows[i]);
		}
		this.resize();
	},
	
	appendRow: function(rowData)
	{
		rowData = rowData || this.defaultStaticRow.slice(0);
		var i = this.selection.getFirstSelected() + 1 || this.model.getRowCount();
		this.addRow(rowData, i);
		//this.updateContentScrolling();
	},
	
	/*
	 * Returns all data in the Grid
	 */
	
	getAllRows: function()
	{
		this.edit.editorApply();
		var rowCount = this.model.getRowCount();
		var rowData = []
		for(var i=0; i<rowCount; i++)
			rowData.push(this.model.getRow(i));
		
		return rowData;	
	},
	
	updateContentScrolling: function() {
		var shouldScroll = this.model.getRowCount() > this.rowsPerPage; 
		this.viewsNode.style.overflowY = shouldScroll ? "scroll" : "";
	},
	
	/*
	 * Override from VirtualGrid.js. Expand grid height if it has x-scrollbar
	 */
	postresize: function() {
		if(this.autoHeight){
			var gridTableHeight =  this.views.measureContent();
			if (gridTableHeight > 0 && this.viewsNode.parentNode.clientWidth <= this.viewsNode.firstChild.clientWidth) {
				gridTableHeight += this.scrollbarHeight;
			}
			this.viewsNode.style.height = gridTableHeight + 'px';
            
            // need to extend the grid node to avoid the vertical scrollbar
            // On IE, the content is taller then the parent node and a vertical
            // scroll bar would show up due to the header, etc.
            if(dojo.isIE && !this.maxHeight) {
                dojo.marginBox(this.gridNode, {h: gridTableHeight + 60});
            }
		}
	},
	
	/*
	 * Overriden from Grid, fired when focus is released from editing control
	 */
//	doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
//		this.model.setDatum(inValue, inRowIndex, inFieldIndex);
//		this.onApplyCellEdit(inValue, inRowIndex, inFieldIndex);
//		this.onchange();
//	},
	
	dokeyup: function(e){
		if (this.cellContentUpdated(e)){
			this.onchange();
//			this.model.setDatum(value, this.focus.rowIndex, this.focus.cell.fieldIndex);
		}
		this._selectByKeys(e);
	},
	
	cellContentUpdated: function(e){
		if(this.edit.isEditing()){
			var value = this.focus.cell.editor.getValue(this.focus.rowIndex);
			var oldValue = this.model.getDatum(this.focus.rowIndex, this.focus.cell.fieldIndex);
			if(value != oldValue){
				return true;
			}else{
				return false;
			}
		}
	},
	
	_selectByKeys: function(e) {
		//if not in edit mode & space bar pressed, do same select as when clicked
		if (!this.edit.isEditing() && e.keyCode == dojo.keys.SPACE) {
			this.selection.clickSelect(this.focus.rowIndex, e.ctrlKey == true, e.shiftKey == true);
		}		
	},
	
	/*
	 * Overriden from dojox.grid._grid.publicEvents
	 */
	onCellClick: function(e){
		this.inherited("onCellClick", arguments);
		if (this.cellContentUpdated(e)){
			if (this.onchange != null && dojo.isFunction(this.onchange)) {
				this.onchange();
			}
		}		
        this._updateActionStates();
	},
	
	/*
	 * Overriden from dojox.grid._grid.publicEvents
	 */
	onRowClick: function(e) {
		this.inherited("onRowClick", arguments);
		this._updateActionStates();
	},
	
	
	_error: function(e)
	{
		console.error(this.declaredClass + ": Error occurred: " + e ? e.message : "Unknown Error");
	},
	
	_addDefaultActions: function() {	
		if (this._actionButtons[this.ACTION_ADD] != null || this._actionButtons[this.ACTION_REMOVE] != null) {
			return;
		}
        //define default actions 	
		var addAction = {
			id: this.ACTION_ADD,
			label: MessageUtil.getMessage(MessageBundle.COMMON, "gridContainer_addRow"),
			callback: dojo.hitch(this, function(e) {
                this.appendRow();
                this.onchange();    
            }),
			iconClass: "add-table-row-icon-image",
			disabled: false
		}
		if (this.onAdd != null && dojo.isFunction(this.onAdd)) {
			addAction.callback = this.onAdd;
		} 

        var deleteAction = {
            id: this.ACTION_REMOVE,
            label: MessageUtil.getMessage(MessageBundle.COMMON, "gridContainer_deleteRow"),
            callback: dojo.hitch(this, function(e) {
                this.deleteSelectedRows();
                this.onchange();    
            }),
            iconClass: "delete-table-row-icon-image",
            disabled: true
        }
        if (this.onRemove != null && dojo.isFunction(this.onRemove)) {
            deleteAction.callback = this.onRemove;
        }		
		
		//append actions
		this.addAction(addAction);
		this.addAction(deleteAction);
	},
	
	/*
	 * Appends an action to the header row
	 * action object should have the following attributes:
	 * --> id, label, callback, iconClass, disabled
	 */
	addAction: function(action) {
        var arg = {
            label: action.label,
            onClick: action.callback
        };
        arg["iconClass"] = action.iconClass;
        arg["showLabel"] = false;
		var button = new com.ibm.asq.common.web.ui.internal.widgets.Button(arg);
        button.setDisabled(action.disabled);
		dojo.addClass(button, "clickable");
		var actionsNode = this.actionsNode != null ? this.actionsNode : this.headerRow;
        actionsNode.appendChild(button.domNode);
		this._actionButtons[action.id] = button;		
		return button;
	},
	
	_updateActionStates: function() {
		var selected = this.selection.getSelected();
		this.setButtonDisabled(this.ACTION_REMOVE, !(selected != null && selected.length > 0));
	},
	
	setButtonDisabled: function(id, isDisabled) {
        var button = this._actionButtons[id];
        if (button) {
            button.setDisabled(isDisabled);
        }		
	},
	
	resize: function(){
		if(dojo.isIE && !this.domNode.parentElement) {
			return;
		} else if(!dojo.isIE && !this.domNode.parentNode) {
			return;
		}
		this.inherited( "resize", arguments );		
	}

});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer.CustomSelectEditor",
dojox.grid.editors.Select,
{
    constructor: function(){     
    },
    /**
     * Changed the format method in order to add title to the options, 
     * Before:  <option value="value" > option </option>
     * With this new method: <option value="value" title="option"> option </option>
     * To avoid text truncation and let the user read the complete text using the title.
     */    
    format: function(inDatum, inRowIndex){
        this.needFormatNode(inDatum, inRowIndex);
        var h = [ '<select class="dojoxGrid-select">' ];
        for (var i=0, o, v; ((o=this.options[i]) !== undefined)&&((v=this.values[i]) !== undefined); i++){
            h.push("<option", (inDatum==v ? ' selected' : ''), ' value="' + v + '" title="'+ o +'"', ">", o, "</option>");
        }
        h.push('</select>');
        return h.join('');
    }

});
})();