/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelHeader");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function() {

var ToolWidth = 22;
var MarginWidth = 16;
var DisabledActionCss = "com-ibm-asq-common-widget-htree-panel-action-disabled";

dojo.declare("PanelHeader.SimpleAction",
             [dijit._Widget, dijit._Templated],
{
    templateString: '<a href="javascript:{}" class="dijit dijitButtonNode com-ibm-asq-common-widget-htree-panel-action" waiRole="button" dojoAttachPoint="domNode" dojoAttachEvent="onclick:_handleClicked">' +
                    '  <img src="${_blankGif}" class="com-ibm-asq-common-widget-htree-panel-icon" dojoAttachPoint="iconLabelNode">' +
                    '  </img>' +
                    '</a>',

    label: null,
    iconClass: null,                    
    postCreate: function()
    {
        if(this.iconClass)
            dojo.addClass(this.iconLabelNode, this.iconClass);
        if(this.label)
            this.domNode.title = this.label;
    },
    setDisabled: function(disabled)
    {
        this.disabled = disabled;
		dojo.toggleClass(this.domNode, DisabledActionCss, this.disabled);        
    },
    onClick: function()
    {
        
    },
    _handleClicked: function(e)
    {        
        // don't let the event propagate up to chain. This may
        // trigger a 'link' clicked detection in the Jazz body
        // and would caused a dialog to popup saying 'are you sure
        // you want to leave' message. 
        dojo.stopEvent(e);

        if(this.disabled) {
            return;
        }
        this.onClick();
    }    
});


dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelHeader",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<div tabindex="-1" id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-widget-htree-panel-header">' +
                    '  <span class="com-ibm-asq-common-widget-htree-panel-header-actions" dojoAttachPoint="actions">' +
                    '  </span>' +
                    '  <img src="${_blankGif}" class="com-ibm-asq-common-widget-htree-panel-icon" dojoAttachPoint="iconLabelNode" style="display:none">' +
                    '  </img>' +
                    '  <span class="com-ibm-asq-common-widget-htree-panel-header-label" dojoAttachPoint="titleNode">${title}' +
                    '  </span>' +
                    '</div>',
    title: "",
    labelClass: null,
    ruler: null,
    
    postCreate: function()
    {
        this.inherited('postCreate', arguments);
        this.tools = [];
    },
    
    setTitle: function(title, labelClass)
    {
        this.title = title;
        this.titleNode.innerHTML = "";
        
        var text = title;
        if(this.widthHint) {
            var className = "com-ibm-asq-common-widget-htree-panel-header-label";
            if(labelClass)
                className += (" " + labelClass);
            var toolsWidth = (this.tools.length > 0) ? (this.tools.length * ToolWidth) + MarginWidth  : MarginWidth;
            var val = com.ibm.asq.common.web.ui.internal.util.wordWrapping.clipText(title, className, this.widthHint - toolsWidth, this.ruler, true);
            text = val.text;
            this.titleNode.title = title;
        }
        if(this.labelClass)
            dojo.removeClass(this.titleNode, this.labelClass);
            
        this.labelClass = labelClass;
        if(this.labelClass)
            dojo.addClass(this.titleNode, this.labelClass);
            
        this.titleNode.appendChild(document.createTextNode(text));    
    },

    buildActionTools: function(actionDesc, dispatcher)
    {
        if(!actionDesc)
            return null;
        
		//destroy any existing action buttons first
		var tools = this.tools;
		if (tools && tools.length)
		{
			var len = tools.length;
			for (var i = 0; i < len; i++)
			{
				var widget = tools[i];
				if (widget && widget.destroy)
					widget.destroy();
			}
		}
		
        this.tools = [];
        this.actions.innerHTML = "";
        
        for(var i = 0; i < actionDesc.actions.length; i++) {
            var actionItem = actionDesc.actions[i];
            var arg = {
                label: actionItem.label,
                iconClass: actionItem.iconClass,
                action: actionItem,
                dispatcher: dispatcher,
                onClick: function(e) {
                    this.dispatcher(this.action);
                }
            };
            var button = new PanelHeader.SimpleAction(arg);
            this.actions.appendChild(button.domNode);
            this.tools.push(button);
        }
    },
    
    dummy:null
});

})();
