/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 * 
 ******************************************************************************/
 // Summary:
 //    AttributePicker is a simple widget that displays a list of names in a popup
 //    panel that allows the user to make selection. The user can single/multi select 
 //    the list items. The user also can add the selected items without closing down
 //    popup. The picker presents those options in two action buttons.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributePicker");

dojo.require("jazz.ui.StyledBox");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.DomConstructor");

(function() {

var TrimStyles = jazz.ui.styledBox.TrimStyles;


dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributePicker",
             [jazz.ui.StyledBox],
{   
    // our dialog's content template.
    contentTemplate:'<div dojoAttachPoint="domNode" class="com-ibm-asq-common-widget-attribute-picker">' +
                    '  <div class="body" dojoAttachPoint="bodyNode">' +
                    '    <div dojoAttachPoint="promptNode" class="prompt">' +
                    '    </div>' +
                    '    <label>' +
                    '     <select dojoAttachPoint="attrSelectNode" class="optionSelect" multiple="true" size="14">' +
                    '     </select>' +
                    '    </label>' +
                    '  </div>' +
                    '  <div class="footer" dojoAttachPoint="footerNode">' +
                    '    <button dojoAttachPoint="addCloseButtonNode" name="add_and_close" disabled="true">' +
                    '    </button>' +
                    '    <button dojoAttachPoint="addButtonNode" name="add" disabled="true">' +
                    '    </button>' +
                    '  </div>' +
                    '</div>',
  
    primaryTitle: MessageUtil.getMessage(MessageBundle.COMMON, "select_attribute"),
    addAndCloseLabel: MessageUtil.getMessage(MessageBundle.COMMON, "add_close_button"),
    addLabel: MessageUtil.getMessage(MessageBundle.COMMON, "add_button"),
    closeLabel: MessageUtil.getMessage(MessageBundle.COMMON, "close_button"),
    secondaryTitle: "",
    promptText: "",
	selected : 0,	
    
    trim: TrimStyles.BLUE_ALPHA_SHADOW,
    closable: true,

    // specifies the list of names to be shown in the option list. This widget supports
    // both a single array of names, or an array of object with name/value pair attributes.
    attributeItems: null,
    
    postCreate: function() 
    {
        // attach the widgets created from the dom constructor to our object
        var dom = new com.ibm.asq.common.web.ui.internal.util.DomConstructor(this.contentTemplate);
        this.promptNode = dom.promptNode;
        this.attrSelectNode = dom.attrSelectNode;
        this.footerNode = dom.footerNode;
        this.addCloseButtonNode = dom.addCloseButtonNode;
        this.addButtonNode = dom.addButtonNode;
        
        this.addCloseButtonNode.innerHTML = this.addAndCloseLabel;
        this.addButtonNode.innerHTML = this.addLabel;
        this.promptNode.innerHTML = this.promptText;
        
        this.refreshList();
    
	    this.connect(this.addCloseButtonNode, "onkeypress", "_onKeyPress");
		this.connect(this.addButtonNode, "onkeypress", "_onKeyPress");
		this.connect(this.attrSelectNode, "onkeypress", "_onKeyPress");
        this.connect(this.addCloseButtonNode, "onclick", "_addCloseClicked");
	    this.connect(this.addButtonNode, "onclick", "_addClicked");
        this.connect(this.attrSelectNode, "onchange", "_onSelection");
        
        // supply the contentNode to the styled box, it
        // will inserted into its content area.
        this.contentNode = dom.domNode;
        this.inherited("postCreate", arguments);
        
        // need to actively size the styled box to the 
        // proper size of our content. Otherwise on IE
        // the box extends to edge of the window.
        if(dojo.isIE) {
            this.onOpen = dojo.hitch(this, function() {
                var dim = dojo.marginBox(this.contentNode);
                dojo.marginBox(this.domNode, {w:dim.w + 5});
            });
        }
    },
    
	
    // Summary:
    //    Notification of the add button being clicked. Client can override this method
    //    to receive add button clicked events. The list of selected items are passed in
    //    the argument.
    //
    onAdd: function(selectedItems)
    {    
    },
    
    // Summary:
    //    Notification of the close dialog event. Client can override this method
    //    to receive close events. 
    //
    onClose: function()
    {
    },
        
    getSelectedItems: function()
    {
        var selected = [];
        for(var i = 0; i < this.attrSelectNode.options.length; i++) {
            if(this.attrSelectNode.options[i].selected)
                selected.push(this.attrSelectNode.options[i].value);
        }
        return selected;
    },

    // Summary:
    //    Regenerate the selection list of the picker. 
    //
    refreshList: function()
    {
        this.attrSelectNode.length = 0;

        for(var i = 0; i < this.attributeItems.length; i++) {
            var attr = this.attributeItems[i];
            var name, value;
            
            if(dojo.isObject(attr)) {
                name = attr.name;
                value = attr.value;
            }
            else {
                name = value = attr;
            }
            this.attrSelectNode.options[i] = new Option(name, value);
        }
        this.addButtonNode.disabled = true;
        this.addCloseButtonNode.disabled = true;
    },
    
    // Summary:
    //    Internal method to handle the add button clicked.
    //
    _addClicked: function(e)
    {
        var selectedItems = this.getSelectedItems();
        this.onAdd(selectedItems);
        if (this.attrSelectNode.options.length == 0)
            this.onClose();
		this.selectAndSetFocus();
    },

    // Summary:
    //    Internal method to handle the add and close button clicked.
    //
    _addCloseClicked: function(e)
    {
        this._addClicked(e);
        this.onClose();
    },
	
	selectAndSetFocus: function(){
		var anySelected = false;
		dijit.focus(this.attrSelectNode);
		
  		for(var i = 0; i < this.attrSelectNode.length; i++){
			if(this.attrSelectNode.options[i].selected){
				anySelected =  true;
			break;
			}
		}
		
		if(!anySelected)
			this.attrSelectNode.options[0].selected = true;
		this.addButtonNode.disabled = false;
	    this.addCloseButtonNode.disabled = false;			
		selected = 1;			
	},
	
	_onKeyPress: function(evt)
	{
			if(evt.charOrCode == dojo.keys.ESCAPE)
				this.onClose();
			else if(evt.charOrCode === dojo.keys.TAB){
				if(selected == 0){
					this.selectAndSetFocus();
				}
				else if(selected == 1){
					dijit.focus(this.addCloseButtonNode);
					selected++;
				}
				else if(selected == 2){
					dijit.focus(this.addButtonNode);					
					selected = 0;
				}
			}
	},
    // Summary:
    //    Internal method to list selection events.
    //
    _onSelection: function()
    {
        this.addButtonNode.disabled = false;
        this.addCloseButtonNode.disabled = false;
    }
});

})();
