/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField");

dojo.require("dijit.Menu");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.Dialog");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField");

(function() {

var WordWrappingUtil = com.ibm.asq.common.web.ui.internal.util.wordWrapping;
var _textContent=dojo.isIE?"innerText":"textContent";

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
    // Summary:
    //   A form field with toggle list dropdown menu.  The array of options is in the form of
    //   { name: <display name>, value: <unique value>, checked: <boolean> }
    //
    options: [],
    value: null,
    displayValue: MessageUtil.getMessage(MessageBundle.COMMON, "labelAny"),
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.toggleField = document.createElement("div");
        this.fieldHolderNode.appendChild(this.toggleField);
        this.formField = this.toggleField;
        
        // we're going to use the options object to store our toggle
        // states. This is client data, make a copy of it first.
        if(this.options) {
            this.options = dojo.clone(this.options);
        }
        if (this.options.length > 0) {
            var menu = new dijit.Menu({
                'class': "asq-section-form-field-toggle-menu"
            });
            menu.domNode.style.display="none";
            
            var options = this.options;
            var optionsLength = options.length;
            for (var i = 0; i < optionsLength; i++) {
                // create toggle menu items
                var option = options[i];
                this._checkDefaultValue(option);
                var toggleMenuItem = new com.ibm.asq.common.web.ui.internal.widgets.layout.toggleMenuItem({
                    checked: option.checked,
                    label: option.name,
                    toggleClickCallback: dojo.hitch(this, "_menuItemToggle"),
                    toggleClickCallbackData: { index: i }
                });
                menu.addChild(toggleMenuItem);
            }

            this._updateDisplayValue();
            var params = {
                widthHint: this.widthHint,
                label: this.displayValue,
                dropDown: menu
            };
            this.toggleFilter = new com.ibm.asq.common.web.ui.internal.widgets.layout.DropDownButton(params);
            this.toggleField.appendChild(this.toggleFilter.domNode);
            this.toggleField.title = this.displayValue;
            this.menu = menu;
        } else {
            console.error("options array not specified");
        }
    },
    
    _checkDefaultValue: function(option)
    {
        if(this.defaultValue) {
            if(dojo.isArray(this.defaultValue)) {
                var checkedValues = this.defaultValue;
                var len = checkedValues.length
                for(var i = 0; i < len; i++) {
                    if(checkedValues[i] == option.value) {
                        option.checked = true;
                    }
                }
            }
            else {
                option.checked = (dojo.isString(this.defaultValue) ? this.defaultValue == option.value : false);    
            }
        }
    },
    
    uninitialize: function()
    {
        if (this.toggleFilter) {
            this.toggleFilter.destroy();
            this.toggleFilter = null;
        }
        if (this.menu) {
            this.menu.destroyRecursive();
            this.menu = null;
        }
    },

    _menuItemToggle: function(args) {
        this.options[args.index].checked = args.checked ? args.checked : false;
        this._updateDisplayValue();
        this.toggleFilter.setLabel(this.displayValue);
        this.toggleField.title = this.displayValue;
        this.changed = true;
    },
    
    setValue: function(value) {
        // The checked values are in the options and manipulated through
        // the dropdown menu, not set directly.
    },

    clearValue: function() {
        var d = dojo;

        d.forEach(this.options, function(option) {
            option.checked = false;
        });
        d.forEach(this.menu.getChildren(), function(item) {
            item.attr("checked", false);
        })
        this._updateDisplayValue();
        this.toggleFilter.setLabel(this.displayValue);
        this.toggleField.title = this.displayValue;
        this.changed = true;
    },

    getValue: function() {
        if (this.changed) {
            // convert the array of checked associative array to array of value only
            if (this.options && (this.options.length > 0)) {
                this.value = [];
                for (var i = 0; i < this.options.length; i++) {
                    if (this.options[i].checked) {
                        this.value.push(this.options[i].value);
                    }
                }
            } else {
                this.value = null;
            }
            this.changed = false;
        }
        return this.value;
    },

    _updateDisplayValue: function() {
        this.displayValue = "";
        if (this.options) {
            this.value = [];
            for (var i = 0; i < this.options.length; i++) {
                if (this.options[i].checked) {
                    this.value.push(this.options[i].value);
                    if (this.displayValue == "") {
                        this.displayValue = this.options[i].name;
                    } else {
                        this.displayValue = MessageUtil.getMessage(MessageBundle.COMMON, "labelOrValues", this.displayValue, this.options[i].name);
                    }
                }
            }
            this.changed = false;
        }
        if (this.displayValue == "") {
            this.value = null;
            this.displayValue = MessageUtil.getMessage(MessageBundle.COMMON, "labelAny");
        }
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.DropDownButton", dijit.form.DropDownButton, {
    // Summary:
    //   Simplified dropdown button that conform to the abstract form field style.
    //
    templateString: '<div class="asq-section-form-field toggle-field" style="width:100% !important"' +
                    '     dojoAttachEvent="onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse,onclick:_onDropDownClick,' +
                    '     onkeydown:_onDropDownKeydown,onblur:_onDropDownBlur,onkeypress:_onKey">' +
                    '  <div class="${titleClass}" dojoAttachPoint="focusNode,titleNode" waiRole="button" waiState="haspopup-true,labelledby-${id}_label">' +
                    '    <div class="${iconClosedClass}" dojoAttachPoint="iconNode">' +
                    '      <div class="dijitButtonText toggle-button-label" dojoAttachPoint="containerNode,popupStateNode"' +
                    '           id="${id}_label">${label}</div>' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',

    applyPopupStyle: true,
    widthHint: null,
    titleClass: "toggle-button",
    iconOpenedClass: "toggle-edit-opened",
    iconClosedClass: "toggle-edit-closed",
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.setLabel(this.label);
    },
    
	_setLabelAttr: function(val){
		this.containerNode[_textContent] = this.label = val;
	},
	
    _openDropDown: function() {
        this.inherited("_openDropDown", arguments);
        this.iconNode.className = this.iconOpenedClass;
        
        // the drop down of the button is a table which is a child of the popup DIV
        var domNode = this.dropDown.domNode;
        var popup = domNode.parentNode;
        var popupMB = dojo.marginBox(popup);

		var docElem = document.documentElement;
        // determine the top of the popup that is relative to the visible window
        var popupWindowTop = popupMB.t - docElem.scrollTop;

        // check if the popup DIV is too long for the visible height that needs to be trimmed
        // and put up a scrollbar
        if((popupMB.h + popupWindowTop) > docElem.clientHeight) {
        	var popStyle = popup.style;
            popStyle.height = (docElem.clientHeight - popupWindowTop - 10) + "px";
            popStyle.overflowX = "hidden";
            popStyle.overflowY = "scroll";
        }
    },
    
    onDropDownClose: function() {
        
    },
    
    _closeDropDown: function(focus) {
        this.inherited("_closeDropDown", arguments);
        this.iconNode.className = this.iconClosedClass;
        this.onDropDownClose();
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.toggleMenuItem", dijit.CheckedMenuItem, {
    // Summary:
    //   A check box menu item. Use dijit.CheckedMenuItem and overwrite _onClick function to make sure the selection list does not fold up when
	//   one item is clicked.
    //
	postCreate: function(){
		this.inherited("postCreate", arguments);
		dojo.addClass(this.iconNode,"dijitMenuItemIconCustom");
	},
	_setLabelAttr: function(val){
		this.containerNode[_textContent] = this.label = val;
	},
	
	_onClick: function(){
		// summary: Clicking this item just toggles its state
		if(!this.disabled){
			this.attr("checked", !this.checked);
			this.onChange(this.checked);
		}
		
		if (this.toggleClickCallbackData) {
            this.toggleClickCallbackData.checked = this.checked;
        } else {
            this.toggleClickCallbackData = { checked: this.checked };
        }
		
		if (dojo.isFunction(this.toggleClickCallback)) {
            this.toggleClickCallback(this.toggleClickCallbackData);
        }
	}
});

// Summary:
//    A field with a toggleList field display and capable of showing an
//    input field in a popup when the drop down is clicked.
//
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.PopupFormField", 
             com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField,
{
    // the popup input field's class string. It must be one of the layout form input field
    fieldClass: "com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField",
    fieldArgs: null,
    displayValue: "",
    fieldWidth: 200,
    
    postCreate: function() {
        com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField.superclass.postCreate.apply(this, arguments);
        
        this.toggleField = document.createElement("div");
        this.fieldHolderNode.appendChild(this.toggleField);
        this.formField = this.toggleField;
        
        var fieldCls = dojo.getObject(this.fieldClass, false);
        
        if(!this.fieldArgs) this.fieldArgs = {};
        this.fieldArgs['fieldLabel'] = this.fieldLabel;
        this.fieldArgs['defaultValue'] = this.defaultValue;
        this.fieldArgs['showFieldLabel'] = false;
        // include a method to allow filter field to change the pop-up width
        this.fieldArgs['setWidth'] = dojo.hitch(this, "_setWidth");
        // pass along if the filter is use in the dynamic filter control
        this.fieldArgs['inDynamicControl'] = this.inDynamicControl;
        
        this.field = new fieldCls(this.fieldArgs);
        dojo.style(this.field.domNode, "width", this.fieldWidth + "px");
        
        this.popupContainer = document.createElement("div");
        if (this.popupPrompt && this.popupPrompt != "") {
            // use a prompt label if it is defined
            var prompt = document.createElement("div");
            prompt.innerHTML = this.popupPrompt;
            dojo.style(prompt, "color", "gray");
            dojo.style(prompt, "marginBottom", "2px");
            this.popupContainer.appendChild(prompt);
        }
        this.popupContainer.appendChild(this.field.domNode);
        this.menu = new com.ibm.asq.common.web.ui.internal.widgets.layout.PopupFormField.TooltipDialog({
            content: this.popupContainer,
            formField: this.field
        });

        // get the initial display value from the real input field
        this.displayValue = this.field.getValueInString();
        this.toggleField.title = this.displayValue;
        var params = {
            widthHint: this.widthHint,
            label: this.displayValue,
            applyPopupStyle: false,
            dropDown: this.menu,
            onDropDownClose: dojo.hitch(this, "updateDisplayValue"),
            iconOpenedClass: "toggle-edit-popup",
            iconClosedClass: "toggle-edit-popup",
            titleClass: "toggle-button-popup"
        };
        this.toggleFilter = new com.ibm.asq.common.web.ui.internal.widgets.layout.DropDownButton(params);
        this.toggleField.appendChild(this.toggleFilter.domNode);
    },
    
    _setWidth: function(width) {
        dojo.style(this.popupContainer, "width", width + "px");
        if (this.menu) {
            // Resizing the widget might go beyond the edge of the screen.  Scrolling will not work
            // with the pop-up dismissed once it lost focus.  Closing and reopening will reposition
            // the pop-up but we want to skip the value change from closing the pop-up.
            this.repositionTweak = true;
            this.toggleFilter._closeDropDown();
            this.toggleFilter._openDropDown();
            this.repositionTweak = false;
        }
    },
    
    updateDisplayValue: function() {
        if (!this.repositionTweak) {
            // skip the update if we are just repositioning the pop-up
            this.displayValue = this.field.getValueInString();
            this.toggleField.title = this.displayValue;
            this.toggleFilter.setLabel(this.displayValue);
        }
    },
    
    setValue: function(value) {
        this.field.setValue(value);
        
        // get the value into our display
        this.updateDisplayValue();
    },

    clearValue: function() {
        this.field.clearValue();

        // get the value into our display
        this.updateDisplayValue();
    },

    getValue: function() {
        return this.field.getValue();
    },

    setFormFieldWidths: function(labelWidth, fieldWidth)
    {
        this.field.setFormFieldWidths(labelWidth, fieldWidth);
    },

    setLabel: function(label) 
    {
        this.field.setLabel(label);
    },
        
    setLabelIndentLevel: function(indentLevel, factor)
    {
    },
    
    setHasError: function(hasError)
    {
         this.field.setHasError(hasError);
    },
    
    setErrorMessage: function(message)
    {
        this.field.setErrorMessage(message);
    },
    
    getHasError: function()
    {
        return this.field.getHasError();
    },
    
    setEdited: function(isEdited)
    {
        this.field.setEdited(isEdited);
    },
    
    getIsEdited: function()
    {
        return this.field.getIsEdited();
    },

    setModified: function(isModified)
    {
        this.field.setModified(isModified);
    },
    
    getIsModified: function()
    {
        return this.field.getIsModified();
    },
    
    setIsRequired: function(isRequired)
    {
        this.field.setIsRequired(isRequired);
    },
    
    getIsRequired: function()
    {
        return this.field.getIsRequired();    
    }    
});

// This class is copied to com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DynamicFilterControl to work around
// defect 28106 with a Jazz Foundation problem that the dependency tree from dojo.require gets too big that cause
// exception on loading.  Sync up any changes here with the copy until the defect is resolved and the code is cleaned up.
//
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.PopupFormField.TooltipDialog", dijit.TooltipDialog, {
    templateString: '<div waiRole="presentation">' +
                       '<div class="dijitTooltipContainer" waiRole="presentation" style="text-align:right;">' +
                          '<div dojoAttachPoint="closeNode" style="text-align:right;height:18px;margin-right:5px">' +
                             '<img tabindex="0" dojoAttachPoint="closeImg" src=""' +
                                  'dojoAttachEvent="mouseenter:_onMouseOverClose, onmouseout:_onMouseOutClose, onclick:_selfClose"/>' +
                          '</div>' +
                          '<div style="text-align:left">' +
                             '<div class="dijitTooltipContents dijitTooltipFocusNode" dojoAttachPoint="containerNode" tabindex="-1" waiRole="dialog"></div>' +
                          '</div>' +
                       '</div>' +
                       '<div class="dijitTooltipConnector" dojoAttachPoint="connectorNode" waiRole="presentation"></div>' +
                    '</div>',

    closeIcon: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/images/icons/etool16/close.gif"),
    closeHoverIcon: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/images/icons/etool16/close-hov.gif"),
    useConnector: false,

    postCreate: function(){
        this.inherited("postCreate", arguments);
        if (this.useConnector) {
            dojo.style(this.closeNode, "marginTop", "5px");
        } else {
            dojo.style(this.connectorNode, "display", "none");
        }
        this.closeImg.src = this.closeIcon;
    },

    uninitialize: function()
    {
        if (this.formField) {
            this.formField.destroy();
            this.formField = null;
        }
        this.closeImg.src = null;
    },

    _onMouseOverClose: function(e){
        this.closeImg.src = this.closeHoverIcon;
        if(e){
            dojo.stopEvent(e);
        }
    },
            
    _onMouseOutClose: function(e) {
        this.closeImg.src = this.closeIcon;
        if(e){
            dojo.stopEvent(e);
        }
    },

    _selfClose: function(e) {
        this.closeImg.src = this.closeIcon;
        this.onCancel();
        if(e){
            dojo.stopEvent(e);
        }
    },
    
    orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ corner){
        // override to hide or show the connector  
        if (this.useConnector) {
            this.domNode.className = this["class"] + " dijitTooltipAB" + (corner.charAt(1) == 'L' ? "Left" : "Right") + " dijitTooltip" + (corner.charAt(0) == 'T' ? "Below" : "Above");
        } else {
            // we always go below and we need to remove the top padding when the connector is hidden
            this.domNode.className = this["class"] + " dijitTooltipAB" + (corner.charAt(1) == 'L' ? "Left" : "Right");
        }
    },

    onOpen: function(/*Object*/pos){
        this.inherited("onOpen", arguments);
        if (this.formField && dojo.isFunction(this.formField.onPopupOpen)) {
            // allow the form field in the popup to update the display on open
            this.formField.onPopupOpen();
        }
    }
});

})();
