/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    Simple utility class to animate progress status panel to slide to
 //    its docking area. Also, it is responsible for keeping the the status 
 //    panel in place when the user scrolls the page or resizing the window.
 //    The status panel's docking area is at the bottom left hand corner of 
 //    the browser's view port.
 // 
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.DockingAnimator");

dojo.require("dojo.fx");
dojo.require("dijit._Widget");

(function() {

com.ibm.asq.common.web.ui.internal.widgets.progressStatus.DockingAnimator = {
    
    alignLeft: true,
    
    // Summary:
    //    Main entry point to 'snap' a new progress panel to its
    //    docking position.    
    snapToDock: function(monitorPopup, delay)
    {
        this.monitor = monitorPopup;
        this._trackViewportSize(500, delay, true);
        
        var d = dojo;
        this.closeHandler = d.connect(monitorPopup, "_onClose", this, "_endTracking");
        this.resizeHandler = d.connect(window, "onresize", this, "_trackViewportSize");
        this.scrollHandler = d.connect(window, "onscroll", this, "_trackBottomPosition");
    },
    
    _trackViewportSize: function(duration, delay, initMove)
    {
        var node = this.monitor.domNode;
        var endPos = this._getMonitorDockPos(node);
        
        // record the initial top position of the panel.
        // we'll need it to add to the scrolling offset
        // when user scrolls the browser.
        this.initY = endPos.y;

        // If this is the first call to move the panel,
        // do it now. Otherwise, this is handling a resize.
        // record the initY and call track position to move 
        // the panel w/ the correct scroll offset top.
        if (initMove) {
            this._anim = dojo.fx.slideTo({
                node: node,
                duration: duration,
                left: endPos.x,
                top: endPos.y,
                onEnd: dojo.hitch(this, function(){
                    delete this._anim;
                })
            }).play(delay || 0);
        }
        else {
            // possible new view port width, reposition the x pos
            // for align to the right.
            this._trackBottomPosition(endPos.x);
        }
    },

    _getMonitorDockPos: function(node)
    {
        var top = 300;
        var height = (node) ? node.offsetHeight : 0;
        var vp = dijit.getViewport();
        var leftPos = this.alignLeft ? 2 : vp.w - node.offsetWidth - 1;
        
        // make the top position of the panel 2 pixels
        // short of the bottom. Otherwise, we could force
        // the browser's vertical scrollbar to come up.
        return {x: leftPos, y: vp.h - height - 2};        
    },
    
    _trackBottomPosition: function(newLeft)
    {
        var node = this.monitor.domNode;
        var viewport = dijit.getViewport();
        var leftPos = newLeft || node.offsetLeft;
        
        // the viewport's t (top position) is the scroll position.
        // Anything positive we'll need to add to our original Y position
        // when the panel was first shown. 
        this._anim = dojo.fx.slideTo({
            node: this.monitor.domNode,
            duration: 200,
            left: leftPos,
            top: this.initY + viewport.t,
            onEnd: dojo.hitch(this, function() {
                delete this._anim;
            })
        }).play(0);
    },

    _endTracking: function()
    {
        // the panel has been closed. Disconnect the handlers
        // and stop any ongoing animation task
        if(this._anim) {
            this._anim.stop();
            delete this._anim;
        }
        
        var d = dojo;
        d.disconnect(this.closeHandler);
        d.disconnect(this.resizeHandler);
        d.disconnect(this.scrollHandler);
    }    
};

})();
