/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
// Summary:
//    ComparePlugin extends dojo's dijit._editor._Plugin to provide cleanup action 
//    for rich text editing across the QM application.  

CleanupPlugin = dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.CleanupPlugin");
CleanupPlugin.PLUGIN_ID = 'cleanup';

dojo.require("dijit._editor._Plugin");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.CleanupPopup");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.RichTextClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
( function() {

	var Constants = com.ibm.asq.common.web.client.internal.Constants;
	var RichTextClient = com.ibm.rqm.planning.web.client.internal.RichTextClient;
	var ServiceResponseHandler 	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.CleanupPlugin", [dijit._editor._Plugin], {
        useDefaultCommand: false,
                
        _initButton: function(){
            var _this = this;
            this.command = 'inserthtml';            
            this.button = new dijit.form.Button({
                label: Constants.getMessage("labelCleanupContent"),
                showLabel: false,
                iconClass: "CleanupPluginIcon CleanupPluginIconPos",
				onClick: dojo.hitch(this, "_cleanContent"),
                tabIndex: "-1"
            });
        },
		_cleanContent:function(){
			var content = this.editor.getValue();
			if (content && content != "") {
				this._onOpenDialog();
				var handler = new ServiceResponseHandler(this, "_showCleanedContent", "_error");
				RichTextClient.contentToXHTML(handler, content);
			}
		},
        _onOpenDialog: function(){
            this.cleanupPopup = new com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.CleanupPopup();
			var args = {
                primaryTitle: Constants.getMessage("labelCleanedupContent"),
                isClosable: true,
				okCallback:dojo.hitch(this, "_onCloseDialog")
            }
            var dialog = new com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog(args);
			dialog.setWidth('600px');
			dialog.contentContainer.appendChild(this.cleanupPopup.domNode);
            dialog.open();
			this.dialog = dialog;
        },
        _showCleanedContent:function(content){
			this.cleanedContent = content;
			this.cleanupPopup.initialize({content:content});
		},
        _onCloseDialog: function(){
			if (this.cleanedContent) {
				setTimeout(dojo.hitch(this, function(){
					this.editor.setValue(this.cleanedContent);
					this.editor.onChange();
				}),0);
			}
        },

		_error:function(e){
			if(this.dialog){
				this.dialog.close();
			}
			var message = e?e.message?e.message:e:"error";
			var dialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({
				primaryTitle: Constants.getMessage("labelErr"),
				message:message
			});
			dialog.open();
		}
    });
    
})();

dojo.subscribe("dijit.Editor.getPlugin", null, function(o){
    if (o.plugin) {
        return;
    }
    switch (o.args.name) {
        case CleanupPlugin.PLUGIN_ID:
            o.plugin = new com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.CleanupPlugin({
                command: o.args.name
            });
    }
});
