/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The action cell renderer provides an clickable text, or image to invoke an
 //    action.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ActionCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ActionCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {   
        var node;
        
        // if the action has image create the image by calling our 
        // parent class method.
        //
        if(extendedDisplayInfo && 
           (extendedDisplayInfo['image'] ||
            extendedDisplayInfo['mouseOutImage'] ||
            extendedDisplayInfo['overImage'])) {
            node = this.inherited("renderCell", arguments);
            node.className = "image-action";
        }
        else {
            node = document.createElement("a");
            var divNode=document.createElement('div');
        	node.appendChild(divNode);
            
            node.setAttribute("href", "javascript:{}");
            this._renderCellText(viewer, cellData, extendedDisplayInfo, divNode, cellWidthHint);
        }
        
        this.connect(node, "onclick", dojo.hitch(
            	{	'cellModifier': cellModifier,
	            	'rowObject': rowObject,
            		'columnIndex':columnIndex,
            		'tooltip': this._tooltip
            	},this._actionClicked));            
        
        return node;
    },

    _actionClicked: function(e)
    {
        // local event don't propagate the event up.
        dojo.stopEvent(e);
        
        if(this.tooltip)
            this.tooltip.close();
       
        if(this.cellModifier)
            this.cellModifier.cellActionInvoked(this.rowObject, this.columnIndex);
    }
    
});

})();
