/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ComboBoxCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ComboBoxCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var node;
		var selectLabel = document.createElement("label");
       
		var contentProvider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
        
        if (editable) {
            node = document.createElement("select");
            node.className = "combo-cell";
            if(typeof cellWidthHint != "undefined") {
                node.style.width = cellWidthHint + "px";
            }
            
            this.connect(node, "onchange", dojo.hitch(
            	{	'cellModifier': cellModifier,
	            	'rowObject': rowObject,
            		'columnIndex':columnIndex,
            		'lastSelected': null
            	},this._handleSelected));            
            
        } 
        else {
            node = document.createElement("div");
        }
        
        var tooltip = null;
        if (extendedDisplayInfo && extendedDisplayInfo['choices']) {
            var selected = (typeof extendedDisplayInfo['select'] != 'undefined') ? extendedDisplayInfo['select'] : cellData;
            tooltip = extendedDisplayInfo['tooltip'];
            
            if (editable) {
                // select combo box node when editable
                this.setEnumChoices(node, extendedDisplayInfo['choices'], selected);
            } 
            else {
                // div node when not editable
                if (selected) {
                    var choices = extendedDisplayInfo['choices'];
                    var text = " ";
                    
                    // something is selected so get the corresponding text in the choices
                    var length=choices.length;
                    for (var i = 0; i < length; i++) {
                        if (choices[i].value == selected) {
                             text = (choices[i].text) ? choices[i].text : " ";
                            break;
                        }
                    }
                }
                this._renderCellText(viewer, text, extendedDisplayInfo, node, cellWidthHint);
            }
        }
        
        // the parent class's attach tooltip will not create a tooltip
        // if the string is null, or undefined. Don't have to check it here.
        if(editable)
            this._attachTooltipToCell(node, tooltip);    
        
		selectLabel.appendChild(node);
        return selectLabel;
    },
    
    _handleSelected: function(e)
    {
        var node = e.target;
        if(node) {
            var val = null;
            var lastSelected = this.lastSelected;
            var options = node.options;
            var len = options.length;
            
            for(var i = 0; i < len; i++) {
                if(options[i].selected) {
                    val = options[i].value;
                    break;
                }
            }
            this.lastSelected = val;
            if(this.cellModifier) {
                this.cellModifier.cellValueChanged(lastSelected, val, this.rowObject, this.columnIndex);
            }
        }            
    },
    
    setEnumChoices: function(select, choices, value)
    {
        var hasSelect = false;
        
        if(choices && choices.length > 0) {
            for (var i = 0; i < choices.length; i++) {
                select.options[i] = new Option(choices[i].text, choices[i].value);
                
                // make this option selected (shown) if a 
                // selectValue is specified and it matches this
                // option.
                if(value && choices[i].value == value) {
                    select.options[i].selected = true;
                    hasSelect = true;
                }
				if (choices[i].disabled)
					select.options[i].disabled = true;
            }
            // select the first item by default.
            if(!hasSelect)
                select.options[0].selected = true;
        }
    }    
});

})();
