/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DynamicFilterControl");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.Dialog");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributeBuilder");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");

(function() {

var TableContentFilterProvider = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;
var AttributeBuilder = com.ibm.asq.common.web.ui.internal.widgets.layout.AttributeBuilder;
var MoreFilterLabel = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_moreFilters");
var MoreFilterPrompt = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_moreFilterPrompt");
    
// Duplicated from com.ibm.asq.common.web.ui.internal.widgets.layout.PopupFormField.TooltipDialog to work around
// defect 28106 with a Jazz Foundation problem that the dependency tree from dojo.require gets too big that cause
// exception on loading.  Remove this and use dojo.require when the defect is resolved.
//
dojo.declare("temp.TooltipDialog", dijit.TooltipDialog, {
    templateString: '<div waiRole="presentation">' +
                       '<div class="dijitTooltipContainer" waiRole="presentation" style="text-align:right;">' +
                          '<div dojoAttachPoint="closeNode" style="text-align:right;height:18px;margin-right:5px">' +
                             '<img tabindex="0" dojoAttachPoint="closeImg" src=""' +
                                  'dojoAttachEvent="mouseenter:_onMouseOverClose, onmouseout:_onMouseOutClose, onclick:_selfClose"/>' +
                          '</div>' +
                          '<div style="text-align:left">' +
                             '<div class="dijitTooltipContents dijitTooltipFocusNode" dojoAttachPoint="containerNode" tabindex="-1" waiRole="dialog"></div>' +
                          '</div>' +
                       '</div>' +
                       '<div class="dijitTooltipConnector" dojoAttachPoint="connectorNode" waiRole="presentation"></div>' +
                    '</div>',

    closeIcon: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/images/icons/etool16/close.gif"),
    closeHoverIcon: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/images/icons/etool16/close-hov.gif"),
    useConnector: false,

    postCreate: function(){
        this.inherited("postCreate", arguments);
        if (this.useConnector) {
            dojo.style(this.closeNode, "marginTop", "5px");
        } else {
            dojo.style(this.connectorNode, "display", "none");
        }
        this.closeImg.src = this.closeIcon;
    },

    uninitialize: function()
    {
        if (this.formField) {
            this.formField.destroy();
            this.formField = null;
        }
        this.closeImg.src = null;
    },

    _onMouseOverClose: function(e){
        this.closeImg.src = this.closeHoverIcon;
        if(e){
            dojo.stopEvent(e);
        }
    },
            
    _onMouseOutClose: function(e) {
        this.closeImg.src = this.closeIcon;
        if(e){
            dojo.stopEvent(e);
        }
    },

    _selfClose: function(e) {
        this.closeImg.src = this.closeIcon;
        this.onCancel();
        if(e){
            dojo.stopEvent(e);
        }
    },
    
    orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ corner){
        // override to hide or show the connector  
        if (this.useConnector) {
            this.domNode.className = this["class"] + " dijitTooltipAB" + (corner.charAt(1) == 'L' ? "Left" : "Right") + " dijitTooltip" + (corner.charAt(0) == 'T' ? "Below" : "Above");
        } else {
            // we always go below and we need to remove the top padding when the connector is hidden
            this.domNode.className = this["class"] + " dijitTooltipAB" + (corner.charAt(1) == 'L' ? "Left" : "Right");
        }
    },

    onOpen: function(/*Object*/pos){
        this.inherited("onOpen", arguments);
        if (this.formField && dojo.isFunction(this.formField.onPopupOpen)) {
            // allow the form field in the popup to update the display on open
            this.formField.onPopupOpen();
        }
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DynamicFilterControl",     
             [dijit._Widget, dijit._Templated],
{
    templateString:'<div dojoAttachPoint="domNode" title=${filterTitle}>' +
                      '<div dojoAttachPoint="dropDownNode" dojoType="dijit.form.DropDownButton" showLabel="false" iconClass="add-filter-icon-image" style="border:none !important;">' +
                        '<div dojoAttachPoint="filterDialog" dojoType="temp.TooltipDialog" useConnector="true">' +
                        '  <div style="margin-bottom:2em !important; font-size:9pt; color:gray;">${promptTitle}' +
                        '  </div>' + 
                        '  <div dojoAttachPoint="filterContentNode" style="width:360px; margin: 1em 1em;"></div>' +
                        '</div>' +
                      '</div>' +                    
                   '</div>',
    
    filterTitle: MoreFilterLabel,
    promptTitle: MoreFilterPrompt,
    widgetsInTemplate: true,
    viewer: null,
    filters: null,
    
    postCreate: function()
    {
        // the maximum width specified in the filters
        var maxFieldWidth = 0;
        
        // hide the down arrow icon.
        dojo.query(".dijitArrowButtonInner", this.domNode).style("display", "none");

        if(this.filters && this.filters.length) {
            var length = this.filters.length;            
            var filterProvider = this.viewer.filterAttrProvider;
            
            this.fields = [];
            var d = document;
            for (var i = 0; i < length; i++) {
                var filter = this.filters[i];
                var params = filter.params;
                var field;
                
                // get the maximum width of the filters that have specified width
                var widthNum = 0;
                if (params && params.fieldWidth) {
                    widthNum = parseInt(params.fieldWidth);
                }
                if (!isNaN(widthNum) && widthNum > maxFieldWidth) {
                    maxFieldWidth = widthNum;
                }
                // create a 'std' field, or if the client specified a custom field,
                // get it from the provider.
                if (filter.type == TableContentFilterProvider.FIELD_CUSTOM) {
                    field = filterProvider.getCustomField(i);
                }
                else {
                    var fieldArgs = {
                        fieldLabel: filter.title,
                        fieldName: filter.name,
                        defaultValue: filter.defaultValue,
                        
                        // indicate the filter is used in the dynamic filter pop-up to give
                        // the individual filter option to render differently if necessary
                        // to avoid pou-up within pop-up
                        inDynamicControl: true,

                        // give the field a rough hint to clip text
                        widthHint: 200
                    };
                    for (var key in params) {
                        fieldArgs[key] = params[key];
                    }
                    var fieldCls = AttributeBuilder.FIELD_TYPE_CLASS_MAP[filter.type];
                    
                    // if the field type is a popup field, don't create
                    // a popup field, use the intended field type instead.
                    // Having a popup in this more popup doesn't make sense
                    //
                    if(filter.type == 'PopupTextField') {
                        // A popup field must have the intended fieldClass
                        // argument set. No field class is an incorrect 
                        // setting. Ignore this field.
                        if(!params.fieldClass) {
                            console.debug("No fieldClass specified for the PopupTextField field!");
                            continue;
                        }
                        fieldCls = dojo.getObject(params.fieldClass, false);
                        
                        // if we have args for the inner field, use them too. 
                        if (params.fieldArgs) {
                            var args = params.fieldArgs;
                            for (var key in args) {
                                fieldArgs[key] = args[key];
                            }
                        }
                    }
                    field = new fieldCls(fieldArgs);
                    if (filter.type == AttributeBuilder.FIELD_ENUM) {
                        field.setEnumChoices(params);
                    }
                }
                // wrap filter in div to apply style
                var container = d.createElement('div');
                dojo.style(container, "paddingTop", "5px");
                dojo.style(container, "paddingBottom", "5px");
                dojo.style(container, "borderTopWidth", "1px");
                dojo.style(container, "borderTopStyle", "solid");
                dojo.style(container, "borderTopColor", (i == 0) ? "transparent" : "#ededee");
                container.appendChild(field.domNode);
                this.filterContentNode.appendChild(container);
                this.fields.push(field);
            }
        }
        
        // Only pop-up filters specify a width and should have room to spare.
        // They do not show labels but they do here so make a bit extra room
        // for the label.
        maxFieldWidth += 130;
        if (maxFieldWidth > 360) {
            // change to larger width to hold the widest filter
            dojo.style(this.filterContentNode, "width", maxFieldWidth + "px");
        }
    },

    uninitialize: function()
    {
        dojo.forEach(this.fields, function(field) {
            if(field.destroyRecursive)
                field.destroyRecursive();
            else if(field.destroy())
                field.destroy();
        });
    },
    
    clearFilters: function()
    {
        dojo.forEach(this.fields, function(field) {
            field.clearValue();
        });
    },
    
    // Summary:
    //  
    getSelectedFilters: function()
    {
        var values = {};
        
        for(var i = 0; i < this.fields.length; i++) {
            var field = this.fields[i];
            values[field.fieldName] = field.getValue();
        }
        return values;
    }
    
});

})();
