/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListTableHeaderRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListTableHeaderRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer,
{
    sortSelect: null,
    sortDirSelect: null,
    sortDir: TableConstants.SORT_DIR_ASCEND,
    sortByCol: -1,
 
 
    destroyTableHeader: function() {
    	this.inherited(arguments);
 	},   
    createTableControls: function(pageRangeControlNode, pageNode, footerPageNode, filterControlNode, groupControlNode, viewModeControlNode)
    {
        var sep = document.createElement("span");
        sep.className = "controls-spacing";
                
        this._constructSortControl(groupControlNode);
        groupControlNode.appendChild(sep);
        
        this._constructSortDirControl(groupControlNode);
        sep = document.createElement("span");
        sep.className = "controls-spacing-wide";
        groupControlNode.appendChild(sep);

        this._constructGroupControl(groupControlNode);

		if(this.viewer.getIsShowingPagingRangeControl()) {
			this._createPageRangeControl(pageRangeControlNode);
		}
		
        if(this.viewer.getIsShowingPagingControl()) {
            this._createPageControls(pageNode);
            this._createPageControls(footerPageNode);
        }

        if(this.viewer.getIsShowingFilterControl()) {
            this._constructFilterControls(filterControlNode);            
        }   
        else {
            var emptyNode = document.createElement("span");
            emptyNode.innerHTML = "  ";
            filterControlNode.appendChild(emptyNode);
        }
    },


    _constructSortControl: function(controlNode)
    {
        //console.debug("entering: _constructSortControl");

        var provider = this.viewer.getContentProvider();
        var content = this.viewer.getContent();
        var nColumns = provider.getNumColumns(content);
        var hasSortables = false;
        
        for (var i = 0; i < nColumns; i++) {
            if (provider.getIsColumnSortable(content, i)) {
                hasSortables = true;
                break;
            }    
        }
        if(!hasSortables) {
            return;
        }
        
		var selectLabel = document.createElement("label");
		
        this.sortSelect = document.createElement("select");
        this.sortSelect.onchange = dojo.hitch(this, "_handleSortBy");
		
        // create an empty selection
        this.sortSelect.options[0] = new Option("");
        this.sortSelect.options[0].selected = true;
        this.sortSelect.options[0].value = -1;
        
        var pos = 1;
        for (var i = 0; i < nColumns; i++) {
            if (provider.getIsColumnSortable(this.content, i)) {
                var heading = provider.getColumnHeading(content, i);
                var option = new Option(heading);
                option.value = i;
                this.sortSelect.options[pos++] = option;
            }    
        }
        selectLabel.appendChild(this.sortSelect)
        controlNode.appendChild(document.createTextNode(MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_sortBy")));
		controlNode.appendChild(selectLabel);
    },

    _constructSortDirControl: function(controlNode)
    {
		var selectLabel = document.createElement("label");
		
        this.sortDirSelect = document.createElement("select");
        this.sortDirSelect.onchange = dojo.hitch(this, "_handleSortBy");
		
        this.sortDirSelect.options[0] = new Option("Ascend");
        this.sortDirSelect.options[0].selected = true;
        this.sortDirSelect.options[1] = new Option("Descend");
		
		selectLabel.appendChild(this.sortDirSelect)
        controlNode.appendChild(selectLabel);
        
    },
    
    _handleSortBy: function()
    {
        this.sortByCol = -1;
        
        for(var i = 0; i < this.sortSelect.options.length; i++) {
            if(this.sortSelect.options[i].selected) {
                this.sortByCol = parseInt(this.sortSelect.options[i].value);
                break;
            }
        }
        this.sortDir = (this.sortDirSelect.options[0].selected) ? TableConstants.SORT_DIR_ASCEND : TableConstants.SORT_DIR_DESCEND;
        this.viewer.performSortBy(this.sortByCol, this.sortDir);
    }
    
    
});


})();
