/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    Simple dialog to capture user changeable table settings for the common
//    table viewer. This dialog is shown when the table attribute 'allowAdjustSettings'
//    is set to true and that an default table action to change the settings is pressed.
// 
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSettingsDialog");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function() {
var _textContent=dojo.isIE?"innerText":"textContent"; 

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSettingsDialog", 
             com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog,
{
    displayModeSection: 
              '    <tr>' +
              '      <td dojoAttachPoint="displayPrefNode" colspan="4">' +
              '        <table>' +
              '          <tr>' +
              '            <th nowrap dojoAttachPoint="displayPrefLabel" class="settings-table-display-mode-header">' +
              '            </td>' +
              '            <td>' +
              '              <input type="radio" name="displayPref" id="compactPref" dojoAttachPoint="displayPrefCompactRadio" COMPACTCHECKED class="settings-table-display-mode-radio">' +
              '            </td>' +
              '            <td class="settings-table-display-mode-label"><label for="compactPref" dojoAttachPoint="displayPrefCompactLabel"></label></span>' +
              '            </td>' +
              '            <td>' +
              '              <input type="radio" name="displayPref" id="wordwrapPref" dojoAttachPoint="displayPrefWordwrapRadio" WORDWARPCHECKED class="settings-table-display-mode-radio">' +
              '            </td>' +
              '            <td class="settings-table-display-mode-label"><label for="wordwrapPref" dojoAttachPoint="displayPrefWordwrapLabel"></label></span>' +
              '            </td>' +
              '          </tr>' +
              '        </table>' +
              '      </td>' +
              '    </tr>' +
              '    <tr>' +
              '    <td colspan="4"><div class="settings-table-separator"></div></td>' +
              '    </tr>',

    columnSettingsSection: 
              '    <tr>' +
              '      <th nowrap dojoAttachPoint="columnHead" class="settings-table-header"></th>' +
              '      <th nowrap class="settings-table-header">' +
              '        <table>' +
              '          <tr>' +
              '            <th nowrap dojoAttachPoint="widthHead" class="settings-table-header-inner"></th>' +
              '            <th dojoAttachPoint="widthHeadIcon" class="settings-table-header-inner"></th>' +
              '          </tr>' +
              '        </table>' +
              '      </th>' +
              '      <th nowrap dojoAttachPoint="showHead" class="settings-table-header" style="align:center"></th>' +
              '      <th width="100%"></th>' +
              '    </tr>',

    columnSettingsTemplate:
              '  <table class="settings-table" dojoAttachPoint="domNode">',
              
    columnTitle: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_columnTitle"),
    widthTitle: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_widthTitle"),
    showTitle: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_showTitle"),
    dialogTitle: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_settingsTitle"),
    resetLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelReset"),
    csHelpID: "rqm_table_viewer.html", // default CS help id
    showDisplayMode: false,
    isCompactMode: true,
    
    labelDisplayPref: MessageUtil.getMessage(MessageBundle.COMMON, "labelTableDisplayPref"),
    labelCompact: MessageUtil.getMessage(MessageBundle.COMMON, "labelTableCompact"),
    labelWordwrap: MessageUtil.getMessage(MessageBundle.COMMON, "labelTableWordwrap"),
    width: "48%",
    
    postCreate: function() 
    {
        this.inherited('postCreate', arguments);
        this.cshelp.setButtonLabel(MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_settingsTitle")+" "+MessageUtil.getMessage(MessageBundle.COMMON, "labelHelp"));
        if (this.showDisplayMode) {
            if (dojo.isIE) {
                if (this.isCompactMode) {
                    this.displayModeSection = this.displayModeSection.replace(/COMPACTCHECKED/, "checked");
                    this.displayModeSection = this.displayModeSection.replace(/WORDWARPCHECKED/, "");
                } else {
                    this.displayModeSection = this.displayModeSection.replace(/COMPACTCHECKED/, "");
                    this.displayModeSection = this.displayModeSection.replace(/WORDWARPCHECKED/, "checked");
                }
            }
            this.columnSettingsTemplate += this.displayModeSection + this.columnSettingsSection + "</table>";
        } else {
            this.columnSettingsTemplate += this.columnSettingsSection + "</table>";
        }
        
        this.setTitle(this.dialogTitle);
        this.contents = new com.ibm.asq.common.web.ui.internal.util.DomConstructor(this.columnSettingsTemplate);  
        this.table = this.contents.domNode;

        if (this.showDisplayMode) {
            this.contents.displayPrefLabel[_textContent]=this.labelDisplayPref;
            this.contents.displayPrefCompactLabel.innerHTML = this.labelCompact;
            this.contents.displayPrefWordwrapLabel.innerHTML = this.labelWordwrap;
            this.contents.displayPrefCompactRadio.checked = this.isCompactMode;
            this.contents.displayPrefWordwrapRadio.checked = !this.isCompactMode;
        }
        
        this.contents.columnHead[_textContent]=this.columnTitle;
        this.contents.widthHead[_textContent]=this.widthTitle;
        this.contents.showHead[_textContent]=this.showTitle;

        var arg = {
            label: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_setEqualWidth"),
            showLabel: false,
            iconClass: "equal-column-width-icon-image",
            parent: this,
            onClick: dojo.hitch(this, "_handleSetEqualWidth")
        };
        this.setEqualWidthAction = new com.ibm.asq.common.web.ui.internal.widgets.Button(arg);
        this.contents.widthHeadIcon.appendChild(this.setEqualWidthAction.domNode);
        
        var provider = this.viewer.getContentProvider();
        var content = this.viewer.getContent();		
        var nColumns = provider.getNumColumns(content);

        
        this.inputs = [];
        for (var i = 0; i < nColumns; i++) {
            var tr = this.table.insertRow(-1);
            var td = tr.insertCell(0);
            var title = provider.getColumnHeading(content, i);
            var inputs = {'index': i};
            
            dojo.addClass(td, "settings-table-name-td");

            // if this is CJK locale add in our cjk locale specific CSS
            if (com.ibm.asq.common.web.ui.internal.util.Utility.isCJKLocale()) {
                dojo.addClass(td, "settings-table-name-td-cjk");
            }
            var label = document.createElement("span");
            
            // in case there's no client title provided, manufacture
            // a generic title for the column.
            if(!title || title.length == 0) {
                title = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_genericColumnTitle", (i+1));
            } 
            label[_textContent] = title;
            td.appendChild(label);
            inputs.label = label;
            
            td = tr.insertCell(1);
            var input = document.createElement("input");
            input.type = "text";
            input.value = this.viewer.getColumnWidth(i);
            input.size = 4;
            input.columnIndex = i;
            dojo.addClass(input,"settings-table-textcell");
            this.connect(input, "onkeyup", "_updateColumnWidthTally");
            dojo.addClass(td, "settings-table-td");
            td.appendChild(input);
            inputs.widthInput = input;
            
            td = tr.insertCell(2);
            input = document.createElement("input");
            input.type = "checkbox";
            input.checked = this.viewer.isColumnVisible(i);
            input.columnIndex = i;
            input.style.width = "95%";
            this.connect(input, "onclick", "_updateVisibilityChanged");
            dojo.addClass(td, "settings-table-td");
            td.align = "center";
            td.appendChild(input);
            inputs.showInput = input;
            
            this.inputs.push(inputs);
            
            if(!this.viewer.isColumnVisible(i)) {
                this._enableColumnInput(i, false)
            }
        }
        
        this.contentContainer.appendChild(this.contents.domNode);    
        this.messageNode = document.createElement("span");
        dojo.addClass(this.messageNode, "settings-table-message");
        this.contentContainer.appendChild(this.messageNode); 

        this._augmentDialogButtons();
        this._updateColumnWidthTally();   
    },

    _augmentDialogButtons: function()
    {
        this.actionsContainer[_textContent] = "";
        
        var resetDiv = document.createElement("span");
        this._resetButton = document.createElement("button");
        this._resetButton[_textContent] = this.resetLabel;
        this._resetButton.tabIndex = "0";
        this.connect(this._resetButton, "onclick", "_resetValues");
        resetDiv.appendChild(this._resetButton);
        this._resetButton.className = "button-div";
        
        var yesButtonDiv = document.createElement("span");
        this._yesButton = document.createElement("button");
        this._yesButton[_textContent] = this.yesLabel;
        this._yesButton.tabIndex = "0";
        this.connect(this._yesButton, "onclick", "_yesAction");
        yesButtonDiv.appendChild(this._yesButton);
        yesButtonDiv.className = "button-div";
        
        var noButtonDiv = document.createElement("span");
        this._noButton = document.createElement("button");
        this._noButton[_textContent] = this.noLabel;
        this._noButton.tabIndex = "0";
        this.connect(this._noButton, "onclick", "_noAction");
        noButtonDiv.appendChild(this._noButton);
        
        this.actionsContainer.appendChild(resetDiv);
        this.actionsContainer.appendChild(yesButtonDiv);
        this.actionsContainer.appendChild(noButtonDiv);
    },

    _handleSetEqualWidth: function()
    {
        var nVisible = 0;
        for (var i = 0; i < this.inputs.length; i++) {
            if(this.inputs[i].showInput.checked) {
                nVisible++;
            }
        }
        var width = Math.floor(100 / nVisible);
        var remainder = (100 % nVisible);
        
        for (var i = 0; i < this.inputs.length; i++) {
            var inputs = this.inputs[i];
            var checked = inputs.showInput.checked;
            
            if (checked) {
                var val = width;
                
                // apply the remainder until it runs out.
                if(remainder > 0) {
                    val++;
                    remainder--;
                }
                inputs.widthInput.value = val;
            }
        }
    },
    
    _resetValues: function()
    {
		this._resetColumnValues(true);
    },

    _refreshValues: function()
    {
        this._resetColumnValues();
    },

    _resetColumnValues: function(resetToDefault)
    {
        if (this.showDisplayMode) {
            this.contents.displayPrefCompactRadio.checked = this.isCompactMode;
            this.contents.displayPrefWordwrapRadio.checked = !this.isCompactMode;
        }
        for (var i = 0; i < this.inputs.length; i++) {
            var col = this.inputs[i];
            col.widthInput.value = this.viewer.getColumnWidth(i, resetToDefault);
            col.showInput.checked = this.viewer.isColumnVisible(i);
            this._enableColumnInput(i, col.showInput.checked);
        }        
        this._updateColumnWidthTally();   
   },
    
    _getWidthValueFromString: function(textbox)
    {
        var valStr = textbox.value;
        var val = 0;
        
        if (valStr && valStr.length > 0) {
            val = (!isNaN(valStr)) ? parseInt(valStr) : valStr;
        }
        return val;
    },
    
    _updateColumnWidthTally: function()
    {            
        var hasErr = false;
        var width = 0;
        
        for (i = 0; i < this.inputs.length; i++) {
            var inputs = this.inputs[i];
            var checked = inputs.showInput.checked;
            var val = 0;
            
            // update the column's error status.
            if (checked) {
                val = this._getWidthValueFromString(inputs.widthInput);
                if (isNaN(val)|| val ==0) { //IE doesn't like 0 width
                    val = 0;
                    hasErr = true;
                    dojo.addClass(inputs.label, "settings-column-label-attention");
                }
                else {
                    dojo.removeClass(inputs.label, "settings-column-label-attention");
                }
            }
            width += val;
        }
        
        var msg;
        if(!hasErr) {
            var remainder = (100 - width);
            if(remainder > 0)
                msg = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_settingsWidthUnder", width, (100 - width));
            else if(remainder == 0)
                msg = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_settingsWidthTally", width);            
            else
                msg = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_settingsWidthOver", width);   
        }
        else {
             msg = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_settingsWidthError");
        }
        this.messageNode.innerHTML = msg;             
        this._yesButton.disabled = hasErr;
    },
    
    _findAdjacentColumn: function(index, minWidth)
    {
        // find the adjacent column given the index parameter. Walk
        // forward, or back from the index to find one. If the index
        // is at already at the endpoints, we only have to walk the 
        // array once.
        var increment = 1;
        var pass = (index == 0) ? 1 : 0;
        
        // we're at the last column, start by walking back.
        if(index >= (this.inputs.length - 1)) {
            increment = -1;
            pass = 1;
        }
        
        // if we didn't find the right neighbor in one direction, 
        // go to the other direction and look again.
        while(pass < 2) {
            for (var i = index; (i >= 0 && i < this.inputs.length); (i += increment)) {
                if((increment < 0 && i == 0) || 
                   (increment > 0 && i == this.inputs.length - 1))
                    break;
                    
                var input = this.inputs[i + increment];
                if(input.showInput.checked) {
                    if(minWidth > 0) {
                        var width = this._getWidthValueFromString(input.widthInput);
                        if(isNaN(width) || width < minWidth)
                            continue;
                    }
                    return i + increment;
                }
            }
            // change the search direction
            increment = (increment == 1) ? -1 : 1;
            pass++;
        }
        return -1;
    },
    
    _updateVisibilityChanged: function(event)
    {
        event = event || window.event;
        if(!event || !event.currentTarget)
            return;
            
        var colIndex = event.currentTarget.columnIndex;
        var neighbor = -1;
        var isAdd = true;
        var widthToSpread = 0;

        this._enableColumnInput(colIndex, event.currentTarget.checked);
        
        var val = this._getWidthValueFromString(this.inputs[colIndex].widthInput);
        if (!isNaN(val)) {
            widthToSpread = val;
        }
        
        // set the 'operator' for subtracting, or adding 
        // the width of the column being hide/show. If it's 
        // hide, spread the width to other columns. If show,
        // the column's width is add back to the column and
        // substracts the spread from all other columns.
        isAdd = !event.currentTarget.checked;
        
        // spread the width to the next neighbor
        neighbor = this._findAdjacentColumn(colIndex, /*minWidth*/(!isAdd) ? 2 : 0);
        
        if(neighbor > -1) {
            var inputs = this.inputs[neighbor];
            var neighborWidth = this._getWidthValueFromString(inputs.widthInput);

            if (isNaN(neighborWidth)) {
                neighborWidth = 0;
            }
            
            // push the excess width to the neighbor column. 
            if(isAdd) {
                inputs.widthInput.value = neighborWidth + widthToSpread;
            }
            else {
                val = Math.floor(neighborWidth / 2);
                var remainder = (neighborWidth % 2);
            
                inputs.widthInput.value = val + remainder;
                this.inputs[colIndex].widthInput.value = val;
            }
        }
        else {
            
        }
        this._updateColumnWidthTally();
    },
    
    _enableColumnInput: function(colIndex, enable)
    {
        var input = this.inputs[colIndex];
        
        if(enable) {
            dojo.removeClass(input.label, "settings-column-disabled"); 
        }
        else {
            dojo.addClass(input.label, "settings-column-disabled");
        }
        input.widthInput.disabled = !enable;
    },
    
    // Override the "_yesAction" handler to save the settings.
    // 
    _yesAction: function() {
        this.settings = [];
        
        for (var i = 0; i < this.inputs.length; i++) {
            var inputs = this.inputs[i];
            var colSettings = {index: i};
            colSettings.width = parseInt(inputs.widthInput.value);
            colSettings.show = inputs.showInput.checked;
            this.settings.push(colSettings);
        }
        if (this.showDisplayMode) {
            this.isCompactMode = this.contents.displayPrefCompactRadio.checked;
        }
        this.inherited('_yesAction', arguments);
    },
    
    open: function(skipCompact) 
    {
        this.inherited('open', arguments);
        
        // initially setting the HTML checkbox input fields doesn't
        // show the correct checked settings. Need to reload the values
        // after the dialog is up.
        if(dojo.isIE) {
            this._refreshValues();
        }
    },

    // Returns the user modified settings.
    // 
    getSettings: function() 
    {
        return this.settings;
    },
    
    isCompactDisplayMode: function()
    {
        return this.isCompactMode;
    }
});

})();
