/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The default TableViewer's sorter object. Client can override the standard
 //    behavior of the sorter, which can only act on the content provider provided
 //    information to perform sorting.
 // 
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var RENDER_TYPE_TIMESTAMP=TableConstants.RENDER_TYPE_TIMESTAMP;
var SORT_DIR_ASCEND=TableConstants.SORT_DIR_ASCEND;
var TableViewerUtil=com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter", null,
{
    // Summary:
    //    Public API to sort the specified contents in the table viewer.
    sortBy: function(viewer, objects, column, sortDir)
    {
        var sortContainer = [];
        sortContainer = sortContainer.concat(objects);
        
        this.viewer = viewer;
        this.contentProvider=viewer.getContentProvider();
        this.groupCol = -1;
        this.sortCol = column;
        this.sortDir = sortDir;
        sortContainer.sort(dojo.hitch(this, "_sortComparator"));
        
        var collate = [];
        collate.push(sortContainer);
        return collate;
    },
    
    // Summary:
    //    Public API to group and sort the specified contents in the table viewer.
    groupSortBy: function(viewer, objects, /*primary key (group by)*/column1, /*secondary key*/column2, sortDir)
    {
        var sortContainer = [];
        sortContainer = sortContainer.concat(objects); 
        
        this.viewer = viewer;
        this.contentProvider=viewer.getContentProvider();
        this.groupCol = column1;
        this.sortCol = column2;
        this.sortDir = sortDir;
        sortContainer.sort(dojo.hitch(this, "_groupComparator"));
        return this._collateGroups(sortContainer);
    },

    // Summary:
    //    Public API to sort the specified contents by multiple columns in the table viewer. 
    //    columns is expected to be an array of column objects as follows:
    //     [ {'col': column position, 'dir': direction},
    //       ...
    //     ]
    sortByColumns: function(viewer, objects, columns)
    {
        var sortContainer = [];
        sortContainer = sortContainer.concat(objects);
        
        this.viewer = viewer;
        this.contentProvider=viewer.getContentProvider();
        this.groupCol = -1;
        this.sortColumns = columns;
        sortContainer.sort(dojo.hitch(this, "_multiSortComparator"));
        
        var collate = [];
        collate.push(sortContainer);
        return collate;
    },

    // Summary:
    //    Public API to group and sort the specified contents by multiple columns in the table viewer.
    //    columns is expected to be an array of column objects as follows:
    //     [ {'col': column position, 'dir': direction},
    //       ...
    //     ]
    groupSortByColumns: function(viewer, objects, /*primary key (group by)*/column1, /*sort columns*/columns)
    {
        var sortContainer = [];
        
        sortContainer = sortContainer.concat(objects); 
        
        this.viewer = viewer;
        this.contentProvider=viewer.getContentProvider();
        this.groupCol = column1;
        this.groupSortDir = SORT_DIR_ASCEND;
        this.sortColumns = null;
        
        if (columns) {
            this.sortColumns = dojo.clone(columns);
            
            // if the sort columns has the group column in it,
            // get the sort direction for the group column and
            // remove it from the sort column list. The group column
            // is the primary sort key.
            //
            var sortColumns=this.sortColumns;
            var length=sortColumns.length;
            var groupCol=this.groupCol;
            for (var i = 0; i < length; i++) {
                if (sortColumns[i].col == groupCol) {
                    this.groupSortDir = sortColumns[i].dir;
                    sortColumns.splice(i, 1);
                    break;
                }
            }
        }
        sortContainer.sort(dojo.hitch(this, "_groupMultiSortComparator"));
        return this._collateGroups(sortContainer);
    },

    _multiSortComparator: function(obj1, obj2)
    {
        var result = 0;
        var sortColumns=this.sortColumns;
        var length=sortColumns.length;
        for(var i = 0; i < length; i++) {
        	var theCol=sortColumns[i];
            result = this._compareByColumn(obj1, obj2, theCol.col, theCol.dir);
            
            // continue sorting on the next sort column if the items are the same. But
            // if this sort produced different order, we can stop.
            if(result != 0)
                break;
        }
        return result;
    },

    // Summary:
    //    Collate contents into groups.
    //
    collateGroups: function(viewer, sortContainer, groupBy)
    {
        this.viewer = viewer;
        this.contentProvider = viewer.getContentProvider();
        this.groupCol = groupBy;
        
        return this._collateGroups(sortContainer);
    },
    
    _groupMultiSortComparator: function(obj1, obj2)
    {
        var result = this._compareByColumn(obj1, obj2, this.groupCol, this.groupSortDir);    
            
        // same group, try secondary/third... sorting.
        if(result == 0) {
            // there's a secondary sort and it's not the same as the group
            // sort column, sort by it. 
            if(this.sortColumns && this.sortColumns.length > 0)
                return this._multiSortComparator(obj1, obj2);
        }
        return result;
        
    },
    
    _collateGroups: function(sortContainer)
    {
        var collate = [];
        var groupVal = null;
        var aGroup = null;
        
        // collate the objects into groups bins
        var length=sortContainer.length;
        var contentProvider=this.contentProvider;
        var groupCol=this.groupCol;
        for(var i = 0; i < length; i++) {
        	var container=sortContainer[i];
            var val = 	contentProvider.getCellRawData(container, groupCol) || 
            			contentProvider.getCellDisplayData(container, groupCol);
            if(val != groupVal) {
                if(aGroup != null) {
                    collate.push(aGroup);
                    aGroup = null;
                }
                if(aGroup == null)
                    aGroup = [];
                groupVal = val;
            }
            aGroup.push(container);
        }
        if(aGroup)
            collate.push(aGroup);
        
        return collate;
    },
     
    _groupComparator: function(obj1, obj2)
    {
        var result = this._compareByColumn(obj1, obj2, this.groupCol, SORT_DIR_ASCEND);    
            
        // same group, try secondary sorting.
        if(result == 0) {
            // there's a secondary sort and it's not the same as the group
            // sort column, sort by it. 
            if(this.sortCol > -1 && this.groupCol != this.sortCol)
                return this._sortComparator(obj1, obj2);
        }
        return result;
    },
    
    _sortComparator: function(obj1, obj2)
    {
        return this._compareByColumn(obj1, obj2, this.sortCol, this.sortDir);
    },
    
    _compareByColumn: function(obj1, obj2, col, dir)
    {
        // get the raw data out first, if the provider provides it.
        // this avoids any data conversions that happen in the get display
        // data that is unnecessary for sorting.
        var contentProvider=this.contentProvider;
        var val1 = contentProvider.getCellRawData(obj1, col);   
        var val2;
        if (val1){
        	val2 = contentProvider.getCellRawData(obj2, col);
        }

        if (!val1 || !val2) {
            val1 = contentProvider.getCellDisplayData(obj1, col);   
            val2 = contentProvider.getCellDisplayData(obj2, col);
            
            if (val1 && val2) {
                // this is a time stamp type. Convert the timestamp value to a Date
                // object for the comparison.
                //
                if (contentProvider.getCellDisplayType(obj1, col) == RENDER_TYPE_TIMESTAMP) {
                    var date1 = TableViewerUtil.convertTimeStampToDate(val1);
                    val1 = date1.getTime();
                }
                if (contentProvider.getCellDisplayType(obj2, col) == RENDER_TYPE_TIMESTAMP) {
                    var date2 = TableViewerUtil.convertTimeStampToDate(val2);
                    val2 = date2.getTime();
                }
            }
        }
        if (val1 && val1.toLowerCase) {
            val1 = val1.toLowerCase();
        }
        if (val2 && val2.toLowerCase) {
            val2 = val2.toLowerCase();
        }
        
        if(val1 == val2)
            return 0;
        
        if(dir == SORT_DIR_ASCEND) {
            return (val1 > val2) ? 1 : -1;
        }
        else {
            return (val1 > val2) ? -1 : 1;            
        }        
    }
});

})();
