/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.util.OSLCServicesDocUtil");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.rqm.defects.web.util.XMLUtil");

(function(){
    var XMLUtil = com.ibm.rqm.defects.web.util.XMLUtil;
    function getURI(rootUrl, serviceDescNode){
        var uriNode = XMLUtil.getNode(serviceDescNode.childNodes, "url", XMLUtil.NAMESPACE_OSLC_CM);
        if (!uriNode) {
            uriNode = XMLUtil.getNode(serviceDescNode.childNodes, "url", XMLUtil.NAMESPACE_CALM)
        }
        var relativeURI = dojox.data.dom.textContent(uriNode);
        return normalizeURI(rootUrl, relativeURI);
    }
    
    function normalizeURI(rootUrl, uri){
        // Strip leading "./" if this is a relative path.
        var re = /^.\//;
        if (re.test(uri)) {
            return rootUrl + "/" + uri.replace(re, "");
        }
        re = /^(..\/)+/;
        if (re.test(uri)) {
            return rootUrl + "/" + uri.replace(re, "");
        }
        // The URI is absolute.
        return uri;
    }
    
    function readServiceDescription(serviceDescDoc, projectLinkType){
        if (projectLinkType == "tests") {
            return readCmServiceDescription(serviceDescDoc);
        }
        else 
            if (projectLinkType == "validates") {
                return readRmServiceDescription(serviceDescDoc);
            }
    }
    
    function readRmServiceDescription(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var selectionDialogNode;
        
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "ServiceDescriptor", XMLUtil.NAMESPACE_OSLC_RM);
        var requirementsNode = XMLUtil.getNode(rootNode.childNodes, "requirements", XMLUtil.NAMESPACE_OSLC_RM);
        
        selectionDialogNode = XMLUtil.getNode(requirementsNode.childNodes, "SelectionDialog", XMLUtil.NAMESPACE_OSLC_RM);
        var reqSelectDialog = null;
        if (selectionDialogNode) {
            reqSelectDialog = _readRmDialog(selectionDialogNode);
        };
        
        var collectionsNode = XMLUtil.getNode(rootNode.childNodes, "collections", XMLUtil.NAMESPACE_OSLC_RM);
        selectionDialogNode = XMLUtil.getNode(collectionsNode.childNodes, "CollectionSelectionDialog", XMLUtil.NAMESPACE_OSLC_RM);
        var collectionSelectDialog = null;
        if (selectionDialogNode) {
            collectionSelectDialog = _readRmDialog(selectionDialogNode);
        }
        
        var linksNode = XMLUtil.getNode(rootNode.childNodes, "links", XMLUtil.NAMESPACE_OSLC_RM);
        var linkCreationServicesNode = XMLUtil.getNode(linksNode.childNodes, "LinkCreationServices", XMLUtil.NAMESPACE_OSLC_RM);
        var validatedByLinkNode = XMLUtil.getNode(linkCreationServicesNode.childNodes, "validatedByLinkFactory", XMLUtil.NAMESPACE_OSLC_RM);
        var linkCreationUri = XMLUtil.getAttributeNodeValue(XMLUtil.getAttributeNode(validatedByLinkNode, "resource", XMLUtil.NAMESPACE_RDF));
        
        return {
            reqSelectDialog: reqSelectDialog,
            collectionSelectDialog: collectionSelectDialog,
            linkCreationUri: linkCreationUri
        };
    }
    
    function _readRmDialog(dialogNode){
        var selectionDialogURI;
        var selectionPreferedWidth, selectionPreferedHeight;
        var widgetNode = XMLUtil.getNode(dialogNode.childNodes, "widget", XMLUtil.NAMESPACE_OSLC_RM);
        var widgetResourceAttr = XMLUtil.getAttributeNode(widgetNode, "resource", XMLUtil.NAMESPACE_RDF);
        if (widgetResourceAttr) {
            selectionDialogURI = widgetResourceAttr.value;
        }
        var widthNode = XMLUtil.getNode(dialogNode.childNodes, "widthHint", XMLUtil.NAMESPACE_OSLC_RM);
        if (widthNode) {
            selectionPreferedWidth = dojox.data.dom.textContent(widthNode);
        }
        var heightNode = XMLUtil.getNode(dialogNode.childNodes, "heightHint", XMLUtil.NAMESPACE_OSLC_RM);
        if (heightNode) {
            selectionPreferedHeight = dojox.data.dom.textContent(heightNode);
        }
        return {
            uri: selectionDialogURI,
            width: selectionPreferedWidth,
            height: selectionPreferedHeight
        }
    }
    
    
    /***
     * @return {Object} {defaultCreationDialogURI, selectionDialogURI}
     * @param {XMLDoc} serviceDescDoc
     */
    function readCmServiceDescription(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var defaultCreationDialogURI, defaultSelectionDialogURI, creationDialogURI, selectionDialogURI, openItemsUri, blockedItemsUri;
        var creationPreferedWidth, creationPreferedHeight, selectionPreferedWidth, selectionPreferedHeight;
        var creationDialogs = [], selectionDialogs = [];
        
        // get the rdf:about in case the uri is relative, it can be used as root
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "ServiceDescriptor", XMLUtil.NAMESPACE_OSLC_CM);
        var rootUrlNode = XMLUtil.getAttributeNode(rootNode, "about", XMLUtil.NAMESPACE_RDF);
        var rootUrl = XMLUtil.getAttributeNodeValue(rootUrlNode) ? XMLUtil.getAttributeNodeValue(rootUrlNode) : "./";
        var changeRequestsNode = XMLUtil.getNode(rootNode.childNodes, "changeRequests", XMLUtil.NAMESPACE_OSLC_CM);
        
        var creationDialogNodes = XMLUtil.getNodes(changeRequestsNode.childNodes, "creationDialog", XMLUtil.NAMESPACE_OSLC_CM);
        dojo.forEach(creationDialogNodes, function(creationDialog){
            var isDefault = false;
            var id;
            var defaultAttr = XMLUtil.getAttributeNode(creationDialog, "default", XMLUtil.NAMESPACE_OSLC_CM);
            if (defaultAttr && XMLUtil.getAttributeNodeValue(defaultAttr) == "true") {
                isDefault = true;
                defaultCreationDialogURI = getURI(rootUrl, creationDialog);
            }
            var idAttr = XMLUtil.getAttributeNode(creationDialog, "id", XMLUtil.NAMESPACE_CALM);
            if (idAttr) {
                id = XMLUtil.getAttributeNodeValue(idAttr);
            }
            creationDialogURI = getURI(rootUrl, creationDialog);
            var widthAttr = XMLUtil.getAttributeNode(creationDialog, "hintWidth", XMLUtil.NAMESPACE_OSLC_CM);
            if (widthAttr) {
                creationPreferedWidth = XMLUtil.getAttributeNodeValue(widthAttr);
            }
            var heightAttr = XMLUtil.getAttributeNode(creationDialog, "hintHeight", XMLUtil.NAMESPACE_OSLC_CM);
            if (heightAttr) {
                creationPreferedHeight = XMLUtil.getAttributeNodeValue(heightAttr);
            }
            creationDialogs.push({
                isDefault: isDefault,
                id: id,
                uri: creationDialogURI,
                width: creationPreferedWidth,
                height: creationPreferedHeight
            })
        });
        
        var selectionDialogNodes = XMLUtil.getNodes(changeRequestsNode.childNodes, "selectionDialog", XMLUtil.NAMESPACE_OSLC_CM);
        dojo.forEach(selectionDialogNodes, function(selectionDialog){
            var isDefault = false;
            var id;
            var defaultAttr = XMLUtil.getAttributeNode(selectionDialog, "default", XMLUtil.NAMESPACE_OSLC_CM);
            if (defaultAttr && (XMLUtil.getAttributeNodeValue(defaultAttr) == "true")) { // work for both IE & FF
                isDefault = true;
                defaultSelectionDialogURI = getURI(rootUrl, selectionDialog);
            }
            var idAttr = XMLUtil.getAttributeNode(selectionDialog, "id", XMLUtil.NAMESPACE_CALM);
            if (idAttr) {
                id = XMLUtil.getAttributeNodeValue(idAttr);
            }
            selectionDialogURI = getURI(rootUrl, selectionDialog);
            var widthAttr = XMLUtil.getAttributeNode(selectionDialog, "hintWidth", XMLUtil.NAMESPACE_OSLC_CM);
            if (widthAttr) {
                selectionPreferedWidth = XMLUtil.getAttributeNodeValue(widthAttr);
            }
            var heightAttr = XMLUtil.getAttributeNode(selectionDialog, "hintHeight", XMLUtil.NAMESPACE_OSLC_CM);
            if (heightAttr) {
                selectionPreferedHeight = XMLUtil.getAttributeNodeValue(heightAttr);
            }
            selectionDialogs.push({
                isDefault: isDefault,
                id: id,
                uri: selectionDialogURI,
                width: selectionPreferedWidth,
                height: selectionPreferedHeight
            });
        });
        
        var filterNodes = XMLUtil.getNodes(changeRequestsNode.childNodes, "filter", XMLUtil.NAMESPACE_CALM);
        dojo.forEach(filterNodes, function(filterNode){
            var idAttr = XMLUtil.getAttributeNode(filterNode, "id", XMLUtil.NAMESPACE_CALM);
            if (idAttr && idAttr.value == "openItems") {
                openItemsUri = getURI(rootUrl, filterNode);
            }
            else 
                if (idAttr.value == "blockedItems") {
                    blockedItemsUri = getURI(rootUrl, filterNode);
                }
        });
        // for back compatiable
        var defaultCreationDialog, defaultSelectionDialog;
        dojo.forEach(creationDialogs, function(creationDialog){
            if (creationDialog.isDefault || creationDialog.id == "defect") {
                defaultCreationDialog = creationDialog;
            }
        });
        dojo.forEach(selectionDialogs, function(selectionDialog){
            if (selectionDialog.isDefault || selectionDialog.id == "defect") {
                defaultSelectionDialog = selectionDialog;
            }
        });
        if (!defaultCreationDialogURI) {
            defaultCreationDialog = creationDialogs[0];
            defaultCreationDialogURI = creationDialogs[0].uri;
        }
        if (!defaultSelectionDialogURI) {
            defaultSelectionDialog = selectionDialogs[0];
            defaultSelectionDialogURI = selectionDialogs[0].uri;
        }
		
		// CM home that RQM can navigate to.
		var homeUrl = null;
		var cmHomeNode = XMLUtil.getNode(changeRequestsNode.childNodes, "home", XMLUtil.NAMESPACE_OSLC_CM);
		if (cmHomeNode) {
			var homeUrlNode = XMLUtil.getNode(cmHomeNode.childNodes, "url", XMLUtil.NAMESPACE_OSLC_CM);
			homeUrl = dojox.data.dom.textContent(homeUrlNode);
		}
        
        var obj = {
            defaultCreationDialogURI: defaultCreationDialogURI,
            creationDialog: defaultCreationDialog,
            creationDialogs: creationDialogs,
            selectionDialogURI: defaultSelectionDialogURI,
            selectionDialog: defaultSelectionDialog,
            selectionDialogs: selectionDialogs,
            openItemsUri: openItemsUri,
            blockedItemsUri: blockedItemsUri,
			homeUrl: homeUrl
        };
        return obj;
    }
    
    /***
     * @return {Object} links
     * @param {Object} projectLinkDescDoc
     */
    function readProjectLinksDescription(projectLinkDescDoc, projectLinkType){
        var links = [];
        var linksRootNode = XMLUtil.getNode(projectLinkDescDoc.childNodes, "links", XMLUtil.NAMESPACE_JP);
		if (linksRootNode) {
			var linkNodes = XMLUtil.getNodes(linksRootNode.childNodes, "link", XMLUtil.NAMESPACE_JP);
			dojo.forEach(linkNodes, function(linkNode){
				var url = null;
				var linkType = null;
				var servicesUrl = null;
				var name = null;
				dojo.forEach(linkNode.childNodes, function(childNode){
					// children of "link"
					if (childNode.tagName == "jp:url") {
						url = childNode.firstChild.data;
					}
					else 
						if (childNode.tagName == "jp:link-type") {
							linkType = childNode.firstChild.data;
						}
						else 
							if (childNode.tagName == "oslc:ServiceProvider") {
								// children of "ServiceProvider"
								dojo.forEach(childNode.childNodes, function(childNode){
									if (childNode.tagName == "dc:title") {
										name = childNode.firstChild.data;
									}
									else 
										if (childNode.tagName == "oslc:services") {
											servicesUrl = childNode.getAttribute("rdf:resource");
										}
								});
								
							}
				});
				if (name && linkType == projectLinkType) {
					links.push({
						url: url,
						linkType: linkType,
						servicesUrl: servicesUrl,
						name: name
					});
				}
			});
		}
        return links;
        
    }
    
    com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readServiceDescription = readServiceDescription;
    com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readProjectLinksDescription = readProjectLinksDescription;
})();
