<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" 
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:purl="http://purl.org/dc/elements/1.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:executionresult="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:testscript="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
    xmlns:time="xalan://com.ibm.rqm.xslfo.service.extension.TimeUtilsExtension">
    <xsl:output method="xml" indent="no"/>
    
    <!-- included stylesheets -->
    <xsl:include href="cache://stylesheets/attribute-sets.xsl"/>
    <xsl:include href="cache://stylesheets/work-item/table.xsl"/>
    <xsl:include href="cache://fopResources/xhtml-to-xslfo-ns.xsl"/>
    <!--<xsl:include href="../attribute-sets.xsl"/>
    <xsl:include href="../work-item/table.xsl"/>-->
    
    <!-- parameters -->
    <xsl:param name="locale"/>
	<xsl:param name="font"/>
    
    <xsl:param name="custom-header"/>
    <xsl:param name="custom-footer"/>
    
    <!-- variables -->
    <xsl:variable name="strings" select="document('cache://fopResources/fopmessages.xml')/fopmessages//*[name()=$locale]"/>
    
    <!-- attribute sets -->
    <xsl:attribute-set name="step-block" use-attribute-sets="simple-text">
        <xsl:attribute name="margin">0.05mm</xsl:attribute>
        <xsl:attribute name="padding">2mm</xsl:attribute>
        <xsl:attribute name="border">solid 0.5px black</xsl:attribute>
        <xsl:attribute name="space-before">8pt</xsl:attribute>
    </xsl:attribute-set>
    
    <xsl:attribute-set name="keyword-block" use-attribute-sets="step-block">
        
    </xsl:attribute-set>
    
    <xsl:attribute-set name="step-name">
        <xsl:attribute name="font-size">10pt</xsl:attribute>
        <xsl:attribute name="font-weight">bold</xsl:attribute>
        <xsl:attribute name="space-after">2pt</xsl:attribute>
    </xsl:attribute-set>
    
    <xsl:attribute-set name="step-type">
        <xsl:attribute name="font-weight">bold</xsl:attribute>
        <xsl:attribute name="color">grey</xsl:attribute>
        <xsl:attribute name="space-after">4pt</xsl:attribute>
    </xsl:attribute-set>
    
    <xsl:attribute-set name="step-header">
        <xsl:attribute name="space-before">8pt</xsl:attribute>
        <xsl:attribute name="text-decoration">underline</xsl:attribute>
        <xsl:attribute name="space-after">4pt</xsl:attribute>
        <xsl:attribute name="keep-with-next">always</xsl:attribute>
    </xsl:attribute-set>
    
    
    <!-- DOCUMENT LAYOUT -->
    <xsl:template match="/">
        <fo:root font-family="{$font}">
            <fo:layout-master-set>
                <fo:simple-page-master master-name="standard-letter-page" 
                    page-height="11in" page-width="8.5in"
                    margin="1in">
                    <fo:region-body/>
                    <fo:region-before extent="0.5in"/>
                    <fo:region-after extent="0.5in"/>
                </fo:simple-page-master>
            </fo:layout-master-set>
            
            <fo:page-sequence master-reference="standard-letter-page"
                initial-page-number="1" format="1">
                <!-- Header -->
                <fo:static-content flow-name="xsl-region-before">
                    <fo:block><xsl:value-of select="$custom-header"/></fo:block>
                </fo:static-content>
                
                <!-- Footer -->
                <fo:static-content flow-name="xsl-region-after">
                    <fo:block><xsl:value-of select="$custom-footer"/></fo:block>
                </fo:static-content>
                
                <!-- Body -->
                <fo:flow flow-name="xsl-region-body">
                    <xsl:apply-templates select="rqm:executionresult"/>
                </fo:flow>
            </fo:page-sequence>
        </fo:root>    
    </xsl:template>
    
    <!--
      If the Execution Result has not yet been expanded we can do so here.  Otherwise it will be caught by the
      next template.
      -->
    <xsl:template match="rqm:executionresult[@href]">
        <xsl:variable name="rest-uri" select="concat('rest://', @href)"/>
        <xsl:apply-templates select="document($rest-uri)/rqm:executionresult"/>
    </xsl:template>
    
    <!-- MAIN CONTENT -->
    <xsl:template match="rqm:executionresult">
        <!-- title, subtitle, description -->
        <fo:block xsl:use-attribute-sets="page-title"><xsl:value-of select="purl:title"/><xsl:text> </xsl:text><xsl:value-of select="$strings/textResults"/></fo:block>
        <fo:block xsl:use-attribute-sets="page-subtitle"><xsl:value-of select="$strings/titleArtifactSummary"/></fo:block>
        <fo:block xsl:use-attribute-sets="simple-text"><xsl:apply-templates select="rqm:executionresult/dc:description"/></fo:block>
        
        <!-- Summary -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/labelSummary"/></fo:block>
        
        <xsl:call-template name="exeResultSummary">
            <xsl:with-param name="execution-result" select="."/>
        </xsl:call-template>
        
        <!-- Work Items -->
        <xsl:variable name="workitems" select="rqm:workitem"/>
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/labelWorkItems"/></fo:block>
        
        <xsl:call-template name="buildWorkItemTable">
            <xsl:with-param name="strings" select="$strings"/>
            <xsl:with-param name="item-list" select="$workitems"/>
            
            <xsl:with-param name="render-id" select="true()"/>
            <xsl:with-param name="render-type" select="true()"/>
            <xsl:with-param name="render-headline" select="true()"/>
            <xsl:with-param name="render-state" select="true()"/>
        </xsl:call-template>
        
        <!-- CALM Work Items -->
        <xsl:variable name="calm-workitems" select="rqm:defect[@rel]"/>
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/RemoteWorkItems"/></fo:block>
        
        <xsl:call-template name="buildCalmWorkItemTable">
            <xsl:with-param name="strings" select="$strings"/>
            <xsl:with-param name="item-list" select="$calm-workitems"/>
            
            <xsl:with-param name="render-relationship" select="true()"/>
            <xsl:with-param name="render-summary" select="true()"/>
            <xsl:with-param name="link" select="true()"/>
        </xsl:call-template>
        
        <!-- Test Environment -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/labelTestEnvironment"/></fo:block>
        <fo:block xsl:use-attribute-sets="simple-text">
            <fo:block font-size="9pt" space-after="4pt"><xsl:value-of select="rqm:executionworkitem/rqm:configuration/dc:title"/></fo:block>
            <fo:block space-after="4pt"><xsl:apply-templates select="rqm:executionworkitem/rqm:configuration/rqm:description"/></fo:block>
<!--        <fo:block><xsl:value-of select="rqm:executionworkitem/rqm:configuration/rqm:configuration"/></fo:block>-->
        </fo:block>
        
        <!-- Notes -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Notes"/></fo:block>
        <fo:block xsl:use-attribute-sets="simple-text"><xsl:apply-templates select="executionresult:details"/></fo:block>
        
        <!-- Individual Step Details -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/individualStepDetail"/></fo:block>
        
        <xsl:call-template name="exeResultPointSummary">
            <xsl:with-param name="execution-result" select="."/>
        </xsl:call-template>
        
        <xsl:call-template name="exeResultStepDetails">
            <xsl:with-param name="result-list" select="executionresult:stepResults/executionresult:stepResult"/>
        </xsl:call-template>
    </xsl:template>
    
    <!-- Execution Result Summary -->
    <xsl:template name="exeResultSummary">
        <xsl:param name="execution-result"/>
        
        <fo:table xsl:use-attribute-sets="simple-text">
            <fo:table-body>
                <!-- ID -->
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/labelId"/>: <xsl:value-of select="rqm:alias"/></fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <!-- Owner -->                                           <!-- Last Modified -->
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/labelOwner"/>: <xsl:value-of select="alm:owner"/></fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/DateLastModified"/>: <xsl:value-of select="time:formatTimestamp(alm:updated)"/></fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <fo:table-row height="8pt">
                    <fo:table-cell>
                        <fo:block linefeed-treatment="preserve"/>
                    </fo:table-cell>
                </fo:table-row>
                <!-- Testplan -->                                        <!-- Test Phase (Milestone) -->
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/labelTestPlan"/>: <xsl:value-of select="$execution-result/rqm:testplan/dc:title"/></fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/labelTestMilestone"/>: <xsl:value-of select="$execution-result/rqm:testphase/dc:title"/></fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <!-- Testcase -->                                        <!-- Iterations -->
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/labelTestCase"/>: <xsl:value-of select="$execution-result/rqm:testcase/dc:title"/></fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/labelIterations"/>: <xsl:value-of select="$execution-result/executionresult:iterations"/></fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <!-- Testscript -->                                      <!-- Datapool (Test Data) -->
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/labelTestScript"/>: <xsl:value-of select="$execution-result/rqm:testscript/dc:title"/></fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/labelDatapool"/>: <xsl:value-of select="$execution-result/rqm:datapool/dc:title"/></fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <fo:table-row height="8pt">
                    <fo:table-cell><fo:block linefeed-treatment="preserve"/></fo:table-cell>
                </fo:table-row>
                <!-- Test Machine -->
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/Machine"/>: <xsl:value-of select="$execution-result/executionresult:machine"/>
                        </fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Start Time -->                                      <!-- PASSED / FAILED -->
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/startTime"/>: <xsl:value-of select="time:formatTimestamp($execution-result/executionresult:starttime)"/></fo:block>
                    </fo:table-cell>
                    <fo:table-cell number-rows-spanned="2">
                        <fo:block font-weight="bold" font-size="10"><xsl:apply-templates select="$execution-result/alm:state"/></fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <!-- End Time -->
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/endTime"/>: <xsl:value-of select="time:formatTimestamp($execution-result/executionresult:endtime)"/></fo:block>
                    </fo:table-cell>
                </fo:table-row>
            </fo:table-body>
        </fo:table>
        
    </xsl:template>
    
    <!-- Execution Result Point Summary -->
    <xsl:template name="exeResultPointSummary">
        <xsl:param name="execution-result"/>
    
        <fo:table xsl:use-attribute-sets="simple-text">
            <fo:table-body>
                <!-- Points Attempted -->                               <!-- Points Blocked -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/pointsAttempted"/>: <xsl:value-of select="$execution-result/executionresult:pointsattempted"/></fo:block></fo:table-cell>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/pointsBlocked"/>: <xsl:value-of select="$execution-result/executionresult:pointsblocked"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Points Passed -->                                  <!-- Points Inconclusive -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/pointsPassed"/>: <xsl:value-of select="$execution-result/executionresult:pointspassed"/></fo:block></fo:table-cell>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/pointsInconclusive"/>: <xsl:value-of select="$execution-result/executionresult:pointsinconclusive"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Points Failed -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/pointsFailed"/>: <xsl:value-of select="$execution-result/executionresult:pointsfailed"/></fo:block></fo:table-cell>
                </fo:table-row>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
    <!-- Execution Result Step Details -->
    <xsl:template name="exeResultStepDetails">
        <xsl:param name="result-list"/>
        
        <!-- print each result -->
        <xsl:for-each select="$result-list">
            <xsl:choose>
                <!-- keyword steps -->
                <xsl:when test="executionresult:stepType='com.ibm.rqm.execution.common.elementtype.keyword'">
                    <fo:block xsl:use-attribute-sets="keyword-block">
                        <!-- name -->
                        <fo:block xsl:use-attribute-sets="step-name">
                            <xsl:value-of select="executionresult:description"/>
                        </fo:block>
                        
                        <!-- type -->
                        <fo:block xsl:use-attribute-sets="step-type"><xsl:apply-templates select="executionresult:stepType"/></fo:block>
                        
                        <!-- sub-steps -->
                        <xsl:call-template name="exeResultStepDetails">
                            <xsl:with-param name="result-list" select="executionresult:stepResults/executionresult:stepResult"/>
                        </xsl:call-template>
                    </fo:block>
                </xsl:when>
                <!-- execution steps -->
                <xsl:otherwise>
                    <fo:block xsl:use-attribute-sets="step-block">                
                        <!-- name -->
                        <fo:block xsl:use-attribute-sets="step-name" text-align-last="justify">
                            <xsl:value-of select="$strings/step"/><xsl:text> </xsl:text><xsl:number/>
                            <fo:leader leader-pattern="space"/>
                            <xsl:apply-templates select="executionresult:result"/>
                        </fo:block>
                        
                        <!-- type -->
                        <fo:block xsl:use-attribute-sets="step-type"><xsl:apply-templates select="executionresult:stepType"/></fo:block>
                        
                        <!-- start time -->                                <!-- end time -->
                        <fo:block>
                            <xsl:value-of select="$strings/startTime"/>: <xsl:value-of select="time:formatTimestamp(executionresult:startTime)"/>
                            <fo:leader leader-pattern="space"/><fo:leader leader-pattern="space"/>
                            <xsl:value-of select="$strings/endTime"/>: <xsl:value-of select="time:formatTimestamp(executionresult:endTime)"/>
                        </fo:block>
                        
                        <!-- description -->
                        <fo:block xsl:use-attribute-sets="step-header"><xsl:value-of select="$strings/Description"/></fo:block>
                        <fo:block><xsl:apply-templates select="executionresult:description"/></fo:block>
                        
                        <!-- expected result -->
                        <fo:block xsl:use-attribute-sets="step-header"><xsl:value-of select="$strings/labelExpectedResult"/></fo:block>
                        <fo:block><xsl:apply-templates select="executionresult:expectedResult"/></fo:block>
                        
                        <!-- actual result -->
                        <fo:block xsl:use-attribute-sets="step-header"><xsl:value-of select="$strings/actualResults"/></fo:block>
                        <fo:block><xsl:apply-templates select="executionresult:actualResult"/></fo:block>
                        
                        <!-- comments -->
                        <fo:block xsl:use-attribute-sets="step-header"><xsl:value-of select="$strings/reqLabel_comments"/></fo:block>
                        <fo:block><xsl:apply-templates select="executionresult:comment"/></fo:block>
                        
                        <!-- work items -->
                        <fo:block space-before="8pt">
                            <xsl:value-of select="$strings/labelWorkItems"/>: 
                            <xsl:for-each select="rqm:properties/rqm:property">
                                <xsl:if test="propertyType='defectView'">
                                    <xsl:value-of select="propertyValue"/><xsl:text> </xsl:text>        <!-- TODO: Make this a link (to work item list) -->
                                </xsl:if>
                            </xsl:for-each>
                        </fo:block>
                        
                        <!-- attachments -->
                        <fo:block>
                            <!--<xsl:value-of select="$strings/labelAttachments"/>: 
                                <xsl:for-each select="rqm:stepAttachment">
                                    <xsl:variable name="attachment" select="document( concat('rest://', @href) )"/>
                                    <!-\- print the name of the attachment followed by a space -\->            <!-\- TODO: make this a link that downloads the attachment -\->
                                </xsl:for-each>-->
                        </fo:block>
                    </fo:block>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template match="executionresult:stepType">
        <xsl:choose>
            <!-- execution step -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.elementtype.execution'">
                <xsl:value-of select="$strings/labelExecutionStep"/></xsl:when>
            <!-- keyword step -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.elementtype.keyword'">
                <xsl:value-of select="$strings/labelKeywordStep"/></xsl:when>
            <!-- default -->
            <xsl:otherwise>
                <xsl:value-of select="current()"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="executionresult:result|alm:state">
        <xsl:choose>
            <!-- passed -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.passed'">
                <fo:inline color="green"><xsl:value-of select="$strings/labelPassed"/></fo:inline></xsl:when>
            <!-- failed -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.failed'">
                <fo:inline color="red"><xsl:value-of select="$strings/labelFailed"/></fo:inline></xsl:when>
            <!-- in progress -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.inprogress'">
                <fo:inline color="orange"><xsl:value-of select="$strings/InProgress"/></fo:inline></xsl:when>
            <!-- not run -->
            <!-- FIXME: don't have a translated string for "not run" so use "in progress" -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.notrun'">
                <fo:inline color="orange"><xsl:value-of select="$strings/InProgress"/></fo:inline></xsl:when>
            <!-- incomplete -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.incomplete'">
                <fo:inline color="orange"><xsl:value-of select="$strings/labelIncomplete"/></fo:inline></xsl:when>
            <!-- inconclusive -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.inconclusive'">
                <fo:inline color="orange"><xsl:value-of select="$strings/labelInconclusive"/></fo:inline></xsl:when>
            <!-- partially blocked -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.part_blocked'">
                <fo:inline color="orange"><xsl:value-of select="$strings/labelPartBlocked"/></fo:inline></xsl:when>
            <!-- error -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.error'">
                <fo:inline color="red"><xsl:value-of select="$strings/labelError"/></fo:inline></xsl:when>
            <!-- blocked -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.blocked'">
                <fo:inline color="orange"><xsl:value-of select="$strings/labelBlocked"/></fo:inline></xsl:when>
            <!-- perm fail -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.perm_failed'">
                <fo:inline color="grey"><xsl:value-of select="$strings/labelPermFailed"/></fo:inline></xsl:when>
            <!-- deferred -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.deferred'">
                <fo:inline color="rgb(100,100,175)"><xsl:value-of select="$strings/labelDeferred"/></fo:inline></xsl:when>
            <!-- default -->
            <xsl:otherwise>
                <fo:inline font-size="8pt"><xsl:value-of select="."/></fo:inline></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>