<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/">
    <xsl:output method="xml" indent="no"/>
    
    <!-- included stylesheets -->
    <xsl:include href="cache://stylesheets/execution-result/table.xsl"/>
    
    <!-- parameters -->
    <xsl:param name="locale"/>
	<xsl:param name="font"/>
    
    <xsl:param name="custom-header"/>
    <xsl:param name="custom-footer"/>
    
    <!-- variables -->
    <xsl:variable name="strings" select="document('cache://fopResources/fopmessages.xml')/fopmessages//*[name()=$locale]"/>
    
    <!-- DOCUMENT LAYOUT -->
    <xsl:template match="/">
        <fo:root font-family="{$font}">
            <fo:layout-master-set>
                <fo:simple-page-master master-name="standard-letter-page" 
                    page-width="8.5in" page-height="11in"
                    margin="1in">
                    <fo:region-body/>
                    <fo:region-before extent="0.5in"/>
                    <fo:region-after extent="0.5in"/>
                </fo:simple-page-master>
            </fo:layout-master-set>
            
            <fo:page-sequence master-reference="standard-letter-page"
                initial-page-number="1" format="1">
                <!-- Header -->
                <fo:static-content flow-name="xsl-region-before">
                    <fo:block><xsl:value-of select="$custom-header"/></fo:block>
                </fo:static-content>
                
                <!-- Footer -->
                <fo:static-content flow-name="xsl-region-after">
                    <fo:block><xsl:value-of select="$custom-footer"/></fo:block>
                </fo:static-content>
                
                <!-- Body -->
                <fo:flow flow-name="xsl-region-body">
                    <!-- page title -->
                    <fo:block xsl:use-attribute-sets="page-title"><xsl:value-of select="$strings/executionResults"/></fo:block>
                    <fo:block xsl:use-attribute-sets="page-subtitle"><xsl:value-of select="$strings/titleResultsTableView"/></fo:block>
                    
                    <!-- results table -->
                    <xsl:call-template name="buildExecutionResultTable">
                        <xsl:with-param name="strings" select="$strings"/>
                        <xsl:with-param name="results" select="list/rqm:executionresult"/>
                        
                        <xsl:with-param name="render-id" select="true()"/>
                        <xsl:with-param name="render-testcase-name" select="true()"/>
                        <xsl:with-param name="render-testscript-name" select="true()"/>
                        <xsl:with-param name="render-state" select="true()"/>
                    </xsl:call-template>
                </fo:flow>
            </fo:page-sequence>
        </fo:root>    
    </xsl:template>
    
</xsl:stylesheet>