<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/">
    
    <!-- included stylesheets -->
    <xsl:include href="cache://stylesheets/attribute-sets.xsl"/>
    <xsl:include href="cache://fopResources/wrapping-rules.xsl"/>
    <!--<xsl:include href="../attribute-sets.xsl"/>-->
    
    <xsl:template name="buildExecutionResultTable">
        <xsl:param name="strings"/>
        <xsl:param name="results"/>
        
        <xsl:param name="render-id"/>
        <xsl:param name="render-name"/>
        <xsl:param name="render-testcase-name"/>
        <xsl:param name="render-testscript-name"/>
        <xsl:param name="render-state"/>
        
        <fo:table xsl:use-attribute-sets="artifact-table">
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <!-- ID (no header) -->
                <xsl:if test="$render-id">
                    <fo:table-cell xsl:use-attribute-sets="border id-header">
                        <fo:block/>
                    </fo:table-cell>
                </xsl:if>
                <!-- Name -->
                <xsl:if test="$render-name">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labelName"/></fo:block>
                    </fo:table-cell>  
                </xsl:if>
                <!-- Testcase Name -->
                <xsl:if test="$render-testcase-name">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labelTestCase"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- Testscript Name -->
                <xsl:if test="$render-testscript-name">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labelTestScript"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- State -->
                <xsl:if test="$render-state">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labelState"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
            </fo:table-header>
            <fo:table-body>
                <xsl:for-each select="$results">
                    <fo:table-row>
                        <!-- ID -->
                        <xsl:if test="$render-id">
                            <fo:table-cell xsl:use-attribute-sets="border id-cell">
                                <fo:block><xsl:value-of select="substring-after(dc:identifier, 'executionresult:')"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- Name -->
                        <xsl:if test="$render-name">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="dc:title"/><xsl:text> </xsl:text><xsl:value-of select="$strings/textResults"/></fo:block>
                            </fo:table-cell>  
                        </xsl:if>
                        <!-- Testcase Name -->
                        <xsl:if test="$render-testcase-name">
                            <xsl:choose>
                                <xsl:when test="rqm:testcase/@href">
                                    <xsl:variable name="testcase" select="document(concat('rest://', rqm:testcase/@href))/rqm:testcase"/>
                                    
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block><xsl:apply-templates select="$testcase/dc:title"/></fo:block>
                                    </fo:table-cell>
                                </xsl:when>
                                <xsl:otherwise>
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block><xsl:apply-templates select="rqm:testcase/dc:title"/></fo:block>
                                    </fo:table-cell>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:if>
                        <!-- Testscript Name -->
                        <xsl:if test="$render-testscript-name">
                            <xsl:choose>
                                <xsl:when test="rqm:testscript">
                                    <xsl:choose>
                                        <xsl:when test="rqm:testscript/@href">
                                            <xsl:variable name="testscript" select="document(concat('rest://', rqm:testscript/@href))/rqm:testscript"/>
                                            
                                            <fo:table-cell xsl:use-attribute-sets="border">
                                                <fo:block><xsl:apply-templates select="$testscript/dc:title"/></fo:block>
                                            </fo:table-cell>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <fo:table-cell xsl:use-attribute-sets="border">
                                                <fo:block><xsl:apply-templates select="rqm:testscript/dc:title"/></fo:block>
                                            </fo:table-cell>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise>
                                    <fo:table-cell xsl:use-attribute-sets="border"><fo:block/></fo:table-cell>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:if>
                        <!-- State -->
                        <xsl:if test="$render-state">
                            <fo:table-cell xsl:use-attribute-sets="border" width="45">
                                <fo:block>
                                    <xsl:choose>
                                        <!-- passed -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.passed'">
                                            <xsl:value-of select="$strings/labelPassed"/></xsl:when>
                                        <!-- failed -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.failed'">
                                            <xsl:value-of select="$strings/labelFailed"/></xsl:when>
                                        <!-- in progress -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.inprogress'">
                                            <xsl:value-of select="$strings/InProgress"/></xsl:when>
                                        <!-- incomplete -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.incomplete'">
                                            <xsl:value-of select="$strings/labelIncomplete"/></xsl:when>
                                        <!-- inconclusive -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.inconclusive'">
                                            <xsl:value-of select="$strings/labelInconclusive"/></xsl:when>
                                        <!-- partially blocked -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.part_blocked'">
                                            <xsl:value-of select="$strings/labelPartBlocked"/></xsl:when>
                                        <!-- error -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.error'">
                                            <xsl:value-of select="$strings/labelError"/></xsl:when>
                                        <!-- blocked -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.blocked'">
                                            <xsl:value-of select="$strings/labelBlocked"/></xsl:when>
                                        <!-- perm fail -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.perm_failed'">
                                            <xsl:value-of select="$strings/labelPermFailed"/></xsl:when>
                                        <!-- deferred -->
                                        <xsl:when test="alm:state='com.ibm.rqm.execution.common.state.deferred'">
                                            <xsl:value-of select="$strings/labelDeferred"/></xsl:when>
                                        <!-- default -->
                                        <xsl:otherwise>
                                            <xsl:apply-templates select="alm:state"/></xsl:otherwise>
                                    </xsl:choose>
                                </fo:block>
                            </fo:table-cell>    
                        </xsl:if>
                    </fo:table-row>
                </xsl:for-each>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
</xsl:stylesheet>