<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:time="xalan://com.ibm.rqm.xslfo.service.extension.TimeUtilsExtension">
    
    <!-- included stylesheets -->
    <xsl:include href="cache://stylesheets/attribute-sets.xsl"/>
    <xsl:include href="cache://fopResources/wrapping-rules.xsl"/>
    <!--<xsl:include href="../attribute-sets.xsl"/>-->
    
    <!--  
      This template will generate a table consisting of the given execution sequence results.  The table will 
      only render the columns for which some value was set.
      -->
    <xsl:template name="buildExecutionSequenceResultTable">
        <xsl:param name="strings"/>
        <xsl:param name="sequence-results"/>
        
        <xsl:param name="render-sequence"/>
        <xsl:param name="render-build-record"/>
        <xsl:param name="render-state"/>
        <xsl:param name="render-start-time"/>
        <xsl:param name="render-end-time"/>
        <xsl:param name="render-summary-result"/>
        
        <fo:table xsl:use-attribute-sets="artifact-table">
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <!-- Sequence -->
                <xsl:if test="$render-sequence">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labelExecutionSequence"/></fo:block>
                    </fo:table-cell>
                </xsl:if>
                <!-- Build Record -->
                <xsl:if test="$render-build-record">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labelBuildRecord"/></fo:block>
                    </fo:table-cell>  
                </xsl:if>
                <!-- Start Time -->
                <xsl:if test="$render-start-time">
                    <fo:table-cell xsl:use-attribute-sets="border timestamp-header">
                        <fo:block><xsl:apply-templates select="$strings/startTime"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- End Time -->
                <xsl:if test="$render-end-time">
                    <fo:table-cell xsl:use-attribute-sets="border timestamp-header">
                        <fo:block><xsl:apply-templates select="$strings/endTime"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- State -->
                <xsl:if test="$render-state">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labelState"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- Summary Result -->
                <xsl:if test="$render-summary-result">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/summaryResult"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
            </fo:table-header>
            <fo:table-body>
                <xsl:for-each select="$sequence-results">
                    <fo:table-row>
                        <!-- Execution Sequence -->
                        <xsl:if test="$render-sequence">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="rqm:executionSequence"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- Build Record -->
                        <xsl:if test="$render-build-record">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="rqm:associatedBuildRecord"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- Start Time -->
                        <xsl:if test="$render-start-time">
                            <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                                <fo:block><xsl:value-of select="time:formatTimestamp(rqm:startTime)"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- End Time -->
                        <xsl:if test="$render-end-time">
                            <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                                <fo:block><xsl:value-of select="time:formatTimestamp(rqm:endTime)"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- State -->
                        <xsl:if test="$render-state">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <xsl:choose>
                                    <xsl:when test="rqm:isRunning='true'">
                                        <fo:block><xsl:apply-templates select="$strings/isRunning"/></fo:block>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <fo:block><xsl:apply-templates select="$strings/isFinished"/></fo:block>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- Summary Result -->
                        <xsl:if test="$render-summary-result">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="rqm:overallResult"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                    </fo:table-row>
                </xsl:for-each>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
</xsl:stylesheet>